package edu.ucla.cs.SMOsTranslator.AlterTableTranslator;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import net.sf.jsqlparser.schema.datatypes.DataType;
import net.sf.jsqlparser.schema.datatypes.DateDataType;
import net.sf.jsqlparser.schema.datatypes.NumericDataType;
import net.sf.jsqlparser.schema.datatypes.StringDataType;
import net.sf.jsqlparser.statement.Statement;
import net.sf.jsqlparser.statement.alter.table.AlterTableChangeColumnStatement;
import net.sf.jsqlparser.statement.alter.table.AlterTableModifyColumnStatement;
import net.sf.jsqlparser.statement.create.table.ColumnDefinition;
import edu.ucla.cs.SMOsTranslator.StatementTranslator;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.dbevolution.ApplicationParameters;
import edu.ucla.cs.dbevolution.DbBackend.DbColumn;
import edu.ucla.cs.velocity.template.RenameColumnTemplate;

/**
 * Translate a {@link AlterTableChangeColumnStatement} statement
 * @author fangar
 *
 */
public class AlterTableChangeColumnStatementTranslator implements StatementTranslator {

	
	private Schema schema;
	
	public AlterTableChangeColumnStatementTranslator(Schema schema){
		this.schema = schema;
	}
	
	@Override
	public List<String> translateStatement(Statement stmt) throws SQLException {

		String newColumnName = null;
		String columnName = null;
		String tableName = null;
		ArrayList<String> smos = null;
		
		AlterTableChangeColumnStatement alterChange = (AlterTableChangeColumnStatement)stmt;
		
		//extract field from statement
		newColumnName = alterChange.getNewColumnDefinition().getColumnName().getColumnName();
		columnName = alterChange.getColumnOld().getColumnName();
		tableName = alterChange.getTable().getName();

		//same name
		if(newColumnName.equalsIgnoreCase(columnName)){
		
			//same data type, same name  ---> NOTHING
			//check the data type against the database, the name is inside the statement
			if(!hasSameDatatype(alterChange)){
				//do not translate the statement if the data type is the same
				//same name, different datatype	----> CHANGE DATATYPE (MODIFY COLUMN)
			
				//create a fake AltertableModifyColumnStatement and translate it
				AlterTableModifyColumnStatement alterModify = new AlterTableModifyColumnStatement();
				//the column name is the same, the new_column_definition can be used to create the fake modify column statement
				//do not care about the column position, no reference in the SMO syntax
				alterModify.setColDef(alterChange.getNewColumnDefinition());
				alterModify.setTable(alterChange.getTable());
			
				AlterTableModifyColumnStatementTranslator alterModifyTranslator = new AlterTableModifyColumnStatementTranslator();
				
				//invoke the translate method of the AlterTableModifyColumnStatement class
				smos =  (ArrayList<String>)alterModifyTranslator.translateStatement(alterModify);
				
			}
		}
		else{
			if(hasSameDatatype(alterChange)){
				//same datatype, different name  ----> RENAME COLUMN
				//RENAME COLUMN SMO
				smos = new ArrayList<String>();
				smos.add(RenameColumnTemplate.getRenameColumnSMO(columnName, newColumnName, tableName));				
			}
			else{
				//different datatype, different name  ----> RENAME COLUMN + CHANGE DATATYPE

				//FROM THE ABOVE IF - CASE, GENERATE A MODIFY STATEMENT 
				//GENERATE A RENAME COLUMN STATEMENT
				//create a fake AltertableModifyColumnStatement and translate it
				AlterTableModifyColumnStatement alterModify = new AlterTableModifyColumnStatement();
				//the column name is the same, the new_column_definition can be used to create the fake modify column statement
				//do not care about the column position, no reference in the SMO syntax
				
				//define a new column with the old column name and the new data type
				ColumnDefinition colDef = new ColumnDefinition();
				colDef.setColumnName(alterChange.getColumnOld());
				colDef.setColDataType(alterChange.getNewColumnDefinition().getColDataType());
				
				//add the new column definition to the AlterTableModifyColumnStatement statement
				alterModify.setColDef(colDef);
				alterModify.setTable(alterChange.getTable());
				
				AlterTableModifyColumnStatementTranslator alterModifyTranslator = new AlterTableModifyColumnStatementTranslator();
				smos =  (ArrayList<String>)alterModifyTranslator.translateStatement(alterModify);
				
				smos.add(RenameColumnTemplate.getRenameColumnSMO(columnName, newColumnName, tableName));
			}
		}
		return smos;
	}
	
	
	/**
	 * Verify if the new column datatype is the same of the old one
	 * @param stmt
	 * @return
	 * @throws SQLException 
	 */
	private boolean hasSameDatatype(AlterTableChangeColumnStatement stmt) throws SQLException{
		
		String tableName = stmt.getTable().getName();
		String columnName = stmt.getColumnOld().getColumnName();
		
		DbColumn evColumn = DbColumn.getColumnDataType(ApplicationParameters.getBaseRevision(), columnName, tableName);
		//String dataType = schema.getTable(tableName).getColType(columnName);

		DataType columnDataType = stmt.getNewColumnDefinition().getColDataType();
		
		if(columnDataType instanceof NumericDataType){
			
			NumericDataType numeric = (NumericDataType)columnDataType;
			//perform numeric datatype comparison
			if(evColumn.getDATA_TYPE().equalsIgnoreCase(numeric.getDataTypeName())){
				//same datatype name, now check the length
				if(evColumn.getNUMERIC_PRECISION() == numeric.getPrecision() && evColumn.getNUMERIC_SCALE() == numeric.getScale()){
					return true;
				}
			}
			
		}
		else if(columnDataType instanceof StringDataType){
			//perform string datatype comparison
			
			StringDataType str = (StringDataType)columnDataType;
			if(evColumn.getDATA_TYPE().equalsIgnoreCase(str.getDataTypeName())){
				if(evColumn.getDATA_TYPE().equals("enum") || evColumn.getDATA_TYPE().equals("set")){
					//verify the list of possible values is the same
					if(evColumn.getCOLUMN_TYPE().equalsIgnoreCase(toStringEnumSetDataTYpe(str))){
						return true;
					}
					return false;
				}
				if(evColumn.getCHARACTER_MAXIMUM_LENGTH() == str.getLength()){
					return true;
				}
			}
		}
		else if(columnDataType instanceof DateDataType){
			//perform date datatype comparison	
			
			DateDataType date = (DateDataType)columnDataType;
			if(date.getDataTypeName().equalsIgnoreCase(evColumn.getDATA_TYPE())){
				return true;
			}
			return false;
		}
		return false;
	}
	
	
	private String toStringEnumSetDataTYpe(StringDataType str) {
		String toString = "";
		
		toString += str.getDataTypeName();
		toString += "(";
		ArrayList<String> values = (ArrayList<String>)str.getPossibleValues();
		
		toString += values.get(0);

		if(values.size()>1){
			
			for(int i=1; i<values.size(); i++){
				toString += ",";
				toString += values.get(i);
			}
		}
		toString += ")";
		
		return toString;
	}

}
