package edu.ucla.cs.SMOsTranslator.AlterTableTranslator;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import net.sf.jsqlparser.statement.Statement;
import net.sf.jsqlparser.statement.alter.table.AlterTableModifyColumnStatement;
import edu.ucla.cs.SMOsTranslator.StatementTranslator;
import edu.ucla.cs.dbevolution.ApplicationParameters;
import edu.ucla.cs.dbevolution.DbBackend.DbKeyColumnUsage;
import edu.ucla.cs.velocity.template.AddColumnTemplate;
import edu.ucla.cs.velocity.template.DropColumnTemplate;
import edu.ucla.cs.velocity.template.RenameColumnTemplate;


/**
 * Translate a {@link AlterTableModifyColumnStatement} statement 
 * @author fangar
 *
 */
public class AlterTableModifyColumnStatementTranslator implements StatementTranslator{

	
	@Override
	public List<String> translateStatement(Statement st){

		//MANAGE FOREIGN KEY, PRIMARY KEY CONSTRAINT!!
		//check if the column is referenced by a foreign or a primary key.
		//drop the primary key OR
		//drop all the foreign key (check all the table that refers to the current modified column)
		//modify the column
		//add all the foreign key
		// ======> method to read the evolution db table and check if the column is referenced
		// ======> method to drop all the keys
		// ======> add back the keys

		AlterTableModifyColumnStatement stmt = (AlterTableModifyColumnStatement)st;
		String tableName = stmt.getTable().getName();
		String columnName = stmt.getColDef().getColumnName().getColumnName();
		
		List<DbKeyColumnUsage> keyColumns=null;
		//ICMOs NOT AWARE
		//ArrayList<DbForeignKeyReference> refs = null; 
		
		try{
			//TODO
			keyColumns = DbKeyColumnUsage.getFKeyReferencesByColumn(columnName, tableName,ApplicationParameters.getBaseRevision());
			//NO ICMOs AWARE
			//refs = DbForeignKeyReference.getForeignKeyDefinition(keyColumns);
		}
		catch(SQLException sqlEx){
			sqlEx.printStackTrace();
			System.exit(-1);
		}
		List<String> smos = new ArrayList<String>();
		
		if(keyColumns == null){
			// the column has no foreign key constraints
			for(String smo: generateModifyColumnSmos(stmt)){
				smos.add(smo);
			}
		}
		else{
			//DISABLED
			//NO ICMO AWARE
			//smos.addAll(DropForeignKeyTemplate.getDropForeignKeySMOs(refs));
			smos.addAll(generateModifyColumnSmos(stmt));
			//DISABLED
			//NO ICMO AWARE
			//smos.addAll(AddForeignKeyTemplate.getAddForeignKeySMOs(refs,tableName));
		}
		return smos;
	}
	
	
	/**
	 * Generate the SMOs statement to modify the table's column
	 * @param stmt
	 * @return
	 */
	private List<String> generateModifyColumnSmos(AlterTableModifyColumnStatement stmt) {
		
		ArrayList<String> smos = new ArrayList<String>();
		String tableName = stmt.getTable().getName();
		
		String columnName = stmt.getColDef().getColumnName().getColumnName();
		
		String newColumnName = columnName +  "_t"; 
		String oldColumnName = columnName;
		String asFunction =  " AS id(" + oldColumnName + ")";

		//ADD COLUMN
		//smos.add(AddColumnTemplate.getAddColumnSMO(newColumnName, DataTypeTranslator.translateDataType(stmt.getColDef().getColDataType()), tableName, asFunction));
		smos.add(AddColumnTemplate.getAddColumnSMO(newColumnName, stmt.getColDef().getColDataType().toString(), tableName, asFunction));
	
		//DROP COLUMN
		smos.add(DropColumnTemplate.getDropColumnSMO(oldColumnName, tableName));

		//RENAME COLUMN
		smos.add(RenameColumnTemplate.getRenameColumnSMO(newColumnName,oldColumnName, tableName));

		return smos;
	}

}
	
	

