package edu.ucla.cs.SMOsTranslator;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.ArrayList;

import net.sf.jsqlparser.parser.CCJSqlParser;
import net.sf.jsqlparser.parser.ParseException;
import net.sf.jsqlparser.statement.Statement;

/**
 * Read an SQL evolution file.
 * An evolution file contains a list of SQL statement evolving the database between two version
 * @author fangar
 *
 */
public class EvolutionFileReader {
	
	/**
	 * Parse the statements from within the file and return a list of statements 
	 * @param filePath
	 * @return a list of statements
	 */
	public static ArrayList<Statement> parseStatements(String filePath){
		
		String line = null;
		BufferedReader rdr = null;	
		File evolutionFile = null;
		StringReader strReader = null;
		Statement stmt = null;
		ArrayList<Statement> statements = null;
		
		try{
			evolutionFile = new File(filePath);
			/*if(!evolutionFile.canRead())
			{
				System.out.println("\nUser not allowed to read the file");
				System.exit(-1);
			}*/
			if(!evolutionFile.exists())
			{
				System.out.println("\nThe file doesn't exist");
				System.exit(-1);
			}
			if(!evolutionFile.isFile())
			{
				System.out.println("\nThe path is not a file, please check the path configured");
				System.exit(-1);
			}
			rdr = new BufferedReader(new FileReader(evolutionFile));
		}
		catch(NullPointerException nEx){
			System.err.println("\nError in the configuration file, please set the patch correctly");
			System.exit(-1);
		}
		catch(FileNotFoundException f){
			//check other exception that can occurs during reading the file
			System.out.println(f.getMessage());
		}
		try{
			statements = new ArrayList<Statement>();
			while ((line = rdr.readLine()) != null) {
				
				if(!line.isEmpty()){
					strReader = new StringReader(line);
					CCJSqlParser simple = new CCJSqlParser(strReader);
				    try{
				    	stmt = simple.statement();
				    	statements.add(stmt);
				    }
				    //catch(JSQLParserException pex){
				    catch(ParseException pex){
				    	System.err.println("================================================================================================");
				    	System.err.println("An error occured during parsing the following statement, please check the syntax is correct");
				    	System.err.println(line);
				    	System.err.println("================================================================================================");
				    	System.err.println("The following is the error reported by the parser");			    	
				    	System.err.println(pex.getMessage());
				    	System.err.println("================================================================================================");
				    	System.exit(-1);
				    }
				    catch(NullPointerException nex){
				    	System.out.println(nex.getMessage());
				    }
				}
			}
			rdr.close();
			strReader.close();
		}
		catch(IOException IoEx){
			System.err.println("I/O error during reading the file.");
			System.exit(-1);
		}
		return statements;
	}		
	
}
