package edu.ucla.cs.SMOsTranslator.Pattern;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import net.sf.jsqlparser.expression.Expression;
import net.sf.jsqlparser.schema.Column;
import net.sf.jsqlparser.schema.Table;
import net.sf.jsqlparser.statement.Statement;
import net.sf.jsqlparser.statement.create.table.CreateTable;
import net.sf.jsqlparser.statement.drop.DropTable;
import net.sf.jsqlparser.statement.insert.Insert;
import edu.ucla.cs.SMOsTranslator.Pattern.Exceptions.UnrecognizedPatternException;
import edu.ucla.cs.SMOsTranslator.Pattern.visitor.CreateTableExtractColumnsVisitor;
import edu.ucla.cs.SMOsTranslator.Pattern.visitor.ExtractColumnsVisitor;
import edu.ucla.cs.SMOsTranslator.Pattern.visitor.PartitionConditionVisitor;
import edu.ucla.cs.SMOsTranslator.Pattern.visitor.ReplaceJoinConditionTableVisitor;
import edu.ucla.cs.SMOsTranslator.Pattern.visitor.ReversePartitionConditionVisitor;
import edu.ucla.cs.SMOsTranslator.Pattern.visitor.SourceTableVisitor;
import edu.ucla.cs.SMOsTranslator.Pattern.visitor.WhereConditionVisitor;
import edu.ucla.cs.SMOsTranslator.Pattern.visitor.WhereJoinVisitor;
import edu.ucla.cs.SMOsTranslator.Pattern.visitor.WherePartitionVisitor;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.dbevolution.DbBackend.DbColumn;
import edu.ucla.cs.dbevolution.DbBackend.DbTable;
import edu.ucla.cs.velocity.template.CopyTableTemplate;
import edu.ucla.cs.velocity.template.DropColumnTemplate;
import edu.ucla.cs.velocity.template.DropTableTemplate;
import edu.ucla.cs.velocity.template.JoinTableTemplate;
import edu.ucla.cs.velocity.template.JoinTemplateSource;
import edu.ucla.cs.velocity.template.PartitionTableTemplate;
import edu.ucla.cs.velocity.template.PartitionTemplateSource;
import edu.ucla.cs.velocity.template.RenameColumnTemplate;


public class JoinPattern extends Pattern{


	private final String[] patternLabels = {Pattern.CREATE_TAB, Pattern.INSERT, Pattern.DROP_TAB};
	private final String patternRecognitionMessage = "Patter Join (type 1) recognized";

	//Schema instance
	private Schema schema;


	//CREATE TABLE variables
	private CreateTable create;
	private Table tableCreated;


	//DROP TABLE variables
	private DropTable dropTable;
	private ArrayList<Table> tablesDropped;


	//INSERT variables
	private Insert insert;
	private ArrayList<Column> selectedColumns;
	private ArrayList<Column> insertColumns;
	private Expression whereCondition;	


	private ArrayList<Table> tablesFromInsert;

	private Table tableInserted;
	//contains the list of tables to be joined
	//Once a table has been copied, the join is performed between the copy and the others tables 
	//FIXME : initialization
	private ArrayList<String> tablesToJoin = new ArrayList<String>();


	private ArrayList<String> smos;



	public JoinPattern(Schema schema){
		this.schema = schema;
	}

	@Override
	public boolean isPattern(String[] label, ArrayList<Statement> stmts) throws UnrecognizedPatternException {

		//Check #1
		if(checkLabels(label)){

			//Get the CREATE variables
			create = (CreateTable)stmts.get(0);
			tableCreated = create.getTable();

			//Get the DROP variables
			try{
				dropTable = (DropTable)stmts.get(2);
				tablesDropped = (ArrayList<Table>) this.dropTable.getTable();
			}
			catch(IndexOutOfBoundsException iEx){
				//No DROP TABLE statement
				dropTable = null;
			}

			//Get the INSERT variables
			insert = (Insert)stmts.get(1);
			tableInserted = this.insert.getTable();
			tablesFromInsert = new SourceTableVisitor().getTablesFromInsert(this.insert);


			//Check #2
			//if in the INSERT statement data come from just one table, the SQL sequence is more similar to a RENAME pattern
			if(tablesFromInsert.size() == 1){
				throw new UnrecognizedPatternException("The INSERT statement must consider two table to be identified as a JOIN pattern\n");
			}

			//Check #3
			//The created table is the same of the INSERT INTO statement
			if(!this.tableCreated.getName().equalsIgnoreCase(this.tableInserted.getName())){
				throw new UnrecognizedPatternException("The CREATE TABLE and the INSERT statement tables must coincide");
			}

			//Check #4
			//The table being dropped is one of the table the SELECT FROM refers to
			//At least one table in the INSERT - SELECT FROM statement is being DROPPED
			if(dropTable != null){
				for(Table tblFromInsert: tablesFromInsert){
					for(Table tblDropped: tablesDropped){
						if(tblDropped.getName().equalsIgnoreCase(tblFromInsert.getName()))
							return true;
					}
				}
				throw new UnrecognizedPatternException("At least one of the DROP table must be in the INSERT statement");
			}

			//Initialize the tables to join to the tables from the insert
			//copy all the tableFromInsert elements to the tables to join
			for(Table table: tablesFromInsert){
				tablesToJoin.add(table.getName());
			}
			return true;
		}
		return false;
	}


	@Override
	public ArrayList<String> translatePattern(ArrayList<Statement> stmts) throws SQLException {

		boolean nameClash = true;
		boolean isDropped = false;
		boolean toCopy = true;

		String tableToBePartitioned = null;
		String tableToBeJoined = null;

		Expression whereJoinCondition = null;

		//VISITOR
		WhereConditionVisitor whereVisitor = new WhereConditionVisitor();
		ReplaceJoinConditionTableVisitor replaceConditionVisitor = new ReplaceJoinConditionTableVisitor();
		

		ArrayList<String> tablesToCopy = new ArrayList<String>();
		smos = new ArrayList<String>();

		whereCondition = whereVisitor.getWhereCondition(this.insert);
		selectedColumns = new ExtractColumnsVisitor().getSelectedColumns(this.insert);
		insertColumns = insert.getColumns()!=null?(ArrayList<Column>)insert.getColumns():null;

		//EXTRACT the JOIN CONDITION
		whereJoinCondition = whereVisitor.getWhereJoinCondition(whereCondition);
		if(whereJoinCondition != null){
			//check if the table in the where condition have an alias defined
			for(Table table: tablesFromInsert){
				if(table.getAlias() != null)
					//replace, if exist, the table alias with the table name in the join condition
					replaceConditionVisitor.replaceTableInJoinCondition(table.getAlias(),table.getName(), whereJoinCondition);
			}
		}

		//TABLEs TO COPY
		//the tables to copy are the tables in the INSERT statement, except the tables in the DROP statement
		//if a table is a source of the join, but the table is dropped after the join, it doesn't need to be copied

		//if no tables dropped 
		//copy all the tables involved in the join
		if(tablesDropped == null || tablesDropped.size() == 0){
			for(Table table: tablesFromInsert)
				tablesToCopy.add(table.getName());
		}
		else{
			for(Table tableFromInsert: this.tablesFromInsert){
				for(Table tableDropped: this.tablesDropped){
					if(tableFromInsert.getName().equalsIgnoreCase(tableDropped.getName())){
						toCopy = false;
					}
				}
				if(toCopy){
					tablesToCopy.add(tableFromInsert.getName());
				}
				toCopy = true;
			}
		}
		//COPY TABLE
		for(String table: tablesToCopy){
			smos.add(CopyTableTemplate.getCopyTableSMO(table));
			replaceJoinTable(table, CopyTableTemplate.getCopiedTableName(table), whereJoinCondition);
		}

		//COLUMN NAME CLASH
		//1) check if the tables in the INSERT statement have columns with the same name
		//2) if such a column exist check if the column is selected to be inserted in the destination table
		//2.b) exclude the column used in the join condition
		//3) if the column is a selected column, check which table the column belongs to
		//4) drop the column from the other table

		//get the column with the same name in both the tables
		HashMap<String, ArrayList<String>> columnNameClashes = DbColumn.getColumnNameClash(tablesFromInsert);

		//extract from the where condition the column in the join condition
		WhereJoinVisitor columnJoinVisitor = new WhereJoinVisitor();
		HashMap<Column,Column> columnInJoin = columnJoinVisitor.GetColumnFromJoin(whereCondition);

		Iterator itColumnClash = columnNameClashes.keySet().iterator();
		while(itColumnClash.hasNext()){

			String columnName = (String)itColumnClash.next();
			//compare the column name with the list of the columns involved in the join
			//if the column is in the join condition, skip it
			Iterator itColumn = columnInJoin.keySet().iterator();
			while(itColumn.hasNext()){
				Column c = (Column)itColumn.next();
				Column c2 = (Column)columnInJoin.get(c);
				if(c.getColumnName().equalsIgnoreCase(columnName) && c2.getColumnName().equalsIgnoreCase(columnName)){
					//the clashing column is the column of the join condition ---> NO NAME CLASH
					nameClash = false;
					break;
				}
			}
			if(nameClash){
				//POSSIBLE NAME CLASH, check whether the columns are in the select clause or not
				//if not selected, DROP the column with the same name
				if(isColumnSelected(columnName)){
					//DROP the column from the table not involved in the SELECT clause
					String tableInSelect = getTableByColumnInSelect(columnName);
					for(Table table: tablesFromInsert){
						if(!table.getName().equals(tableInSelect)){
							smos.add(DropColumnTemplate.getDropColumnSMO(columnName, table.getName()));
						}
					}
				}
				else{
					//drop the column from the BOTH the tables
					for(String table: columnNameClashes.get(columnName)){
						smos.add(DropColumnTemplate.getDropColumnSMO(columnName,table));
					}
				}
			}
		}


		//PARTITION
		Expression wherePartition = new WherePartitionVisitor().getPartitionCondition(whereCondition);
		if(wherePartition != null){
			//at least a partition condition exist

			Column columnPartitioned = new PartitionConditionVisitor().getColumnFromPartitionCondition(wherePartition);
			tableToBePartitioned = columnPartitioned.getTable().getName();
			if(tableToBePartitioned == null){
				//if no table is defined for the column, it means there's no name clash for it
				//only one table in the joined tables has this column
				//retrieve the name of the table from the database
				tableToBePartitioned = DbTable.getTableNameFromColumnAndRevision(tablesFromInsert, columnPartitioned);
			}
			//if the partitioned table has been copied, replace the table
			for(String t: tablesToCopy){
				if(tableToBePartitioned.equalsIgnoreCase(t))
					tableToBePartitioned = CopyTableTemplate.getCopiedTableName(tableToBePartitioned);
			}

			tableToBeJoined = PartitionTemplateSource.getKeptPartitionTable(tableToBePartitioned);

			//REPLACE THE JOIN TABLE to the partitioned table
			replaceJoinTable(tableToBePartitioned, tableToBeJoined,whereJoinCondition);

			//**********************************************************//			
			//Check if the table is being partitioned is dropped		//
			//**********************************************************//
			//		if the table is dropped do he following steps		//

			//1)	partition the table									//
			//2)	drop useless partition								//
			//3)	join the partition_1								//
			//**********************************************************//
			//		if the table is not dropped							//

			//1)	copy the table must be partitioned					//
			//2)	partition the copy of the table						//
			//3)	drop useless partition								//
			//4)	drop the copy										//
			//5)	join the partition_1 								//
			//**********************************************************//

			//1) check the table being partitioned will be dropped

			if (tablesDropped != null) {
				for (Table table : tablesDropped) {
					if (table.getName().equalsIgnoreCase(tableToBePartitioned)) {
						isDropped = true;
						break;
					}
				}
				if (!isDropped) {
					smos.add(CopyTableTemplate.getCopyTableSMO(new Table("",
							tableToBePartitioned)));

					// REMOVE THE TABLE FROM THE LIST OF TABLES TO BE COPIED
					tablesToCopy.remove(tableToBePartitioned);
					// removeTableFromCopy(tableToBePartitioned);

					tableToBePartitioned = CopyTableTemplate
							.getCopiedTableName(tableToBePartitioned);

					// REPLACE THE JOIN TABLE to the partitioned table
					replaceJoinTable(tableToBeJoined, PartitionTemplateSource
							.getKeptPartitionTable(tableToBePartitioned),
							whereJoinCondition);
				}
			}


			PartitionTemplateSource partitionSource = new PartitionTemplateSource();
			ReversePartitionConditionVisitor reverseVisitor = new ReversePartitionConditionVisitor();

			partitionSource.setTableName(tableToBePartitioned);
			partitionSource.setTablePartition1(tableToBePartitioned);
			partitionSource.setTablePartition2(tableToBePartitioned);
			partitionSource.setPartitionCondition1(replaceNonSmoChar(wherePartition.toString()));
			partitionSource.setPartitionCondition2(replaceNonSmoChar(reverseVisitor.getReversePartitionCondition(wherePartition).toString()));
			smos.add(PartitionTableTemplate.getPartitionTableSMO(partitionSource));


			//DROP the USELESS PARTITION
			//by convention the table with the useful partition condition is the partition one
			smos.add(DropTableTemplate.getDropTableSMO(PartitionTemplateSource.getDroppedPartitionTable(tableToBePartitioned)));

		}

		//SMO JOIN
		//the join operator join two tables and drop the second one.
		//If the statement drops only one table, a copy of the other table is created.
		//if the statement drops more than one table (every table involved in the SELECT) then it is correct for the pattern, no copy generated 

//		//COPY TABLE
//		for(String table: tablesToCopy){
//			smos.add(CopyTableTemplate.getCopyTableSMO(table));
//			replaceJoinTable(table, CopyTableTemplate.getCopiedTableName(table), whereJoinCondition);
//		}

		//JOIN TABLE
		JoinTemplateSource joinSource = new JoinTemplateSource(); 
		joinSource.setTable1(tablesToJoin.get(0));
		joinSource.setTable2(tablesToJoin.get(1));
		joinSource.setWhereCondition(whereJoinCondition);
		joinSource.setJoinedTable(this.tableCreated);
		smos.add(JoinTableTemplate.getJoinTableSMO(joinSource));


		//DROP COLUMN
		//given the tables in the JOIN , retrieve the columns of both
		ArrayList<DbColumn> columnsInTables = DbColumn.getColumnsFromTables(this.tablesFromInsert);

		for(String smo : DropColumnTemplate.getDropColumnSMOs(getColumnsToDrop(columnsInTables, this.selectedColumns))){
			smos.add(smo);
		}

		//RENAME COLUMNS
		//compare the columns in the INSERT INTO (if one) with the columns from the SELECT (a,b,c)
		//before comparing the column, if the insertColumn is null, retrieve the list of columns from the new table
		if( insertColumns == null || insertColumns.size() == 0){
			CreateTableExtractColumnsVisitor createVisitor = new CreateTableExtractColumnsVisitor();
			insertColumns = createVisitor.getColumnsFromCreateTable(this.create);
		}
		for(int i=0; i<selectedColumns.size(); i++){
			String oldName = selectedColumns.get(i).getColumnName();
			String newName = insertColumns.get(i).getColumnName();
			if(!oldName.equalsIgnoreCase(newName)){
				smos.add(RenameColumnTemplate.getRenameColumnSMO(oldName, newName,tableCreated.getName()));
			}
		}	
		return smos;
	}


	//	private void removeTableFromCopy(List<String> tablesToCopy, String tableToBePartitioned) {
	//		
	//		int tableToRemove = -1;
	//		for(String table: tablesToCopy){
	//			if(table.equalsIgnoreCase(tableToBePartitioned)){
	//				tableToRemove = tablesToCopy.indexOf(table);
	//			}
	//		}
	//		if(tableToRemove != -1){
	//			tablesToCopy.remove(tableToRemove);
	//		}
	//	}

	private void replaceJoinTable(String tableToReplace,String newTable, Expression whereJoinCondition) {
		//iterate over the table in the join list and replace the table with the given one
		if(tablesToJoin.contains(tableToReplace)){
			tablesToJoin.remove(tablesToJoin.indexOf(tableToReplace));
			tablesToJoin.add(newTable);
			replaceTableInJoinCondition(tableToReplace, newTable, whereJoinCondition);
		}
	}

	private void replaceTableInJoinCondition(String tableToReplace, String newTable, Expression whereJoinCondition) {
		ReplaceJoinConditionTableVisitor replace = new ReplaceJoinConditionTableVisitor();
		replace.replaceTableInJoinCondition(tableToReplace, newTable, whereJoinCondition);
	}

	private String getTableByColumnInSelect(String columnName) {
		for(Column column: selectedColumns){
			if(column.getColumnName().equals(columnName)){
				return column.getTable().getName();
			}
		}
		return null;
	}

	private boolean isColumnSelected(String columnName){
		for(Column column: this.selectedColumns){
			if(column.getColumnName().equalsIgnoreCase(columnName)){
				return true;
			}
		}
		return false;
	}


	/**
	 * Compare the column in the SELECT statement with the columns of both tables.
	 * <br>The columns which aren't in the SELECT stmt needs to be dropped. 
	 * @param columnsInTables
	 * @param columnsInSelect
	 * @return the list of {@link DbColumn} to drop
	 */
	private ArrayList<DbColumn> getColumnsToDrop(ArrayList<DbColumn> columnsInTables, ArrayList<Column> columnsInSelect){

		ArrayList<DbColumn> columnsToDrop = new ArrayList<DbColumn>();
		boolean toDrop = true;


		//compare the columns with the ones from the SELECT xx statement
		//check which columns must be dropped in order to get the same result as the SELECT xx FROM
		//drop the columns which are not listed in the CREATE TABLE statement

		for(DbColumn columnInTable: columnsInTables){
			toDrop = true;
			for(Column columnInSelect: columnsInSelect){
				if(columnInTable.getCOLUMN_NAME().equalsIgnoreCase(columnInSelect.getColumnName())){
					toDrop = false;
				}
			}
			if(toDrop){
				//change the table from the column to be dropped
				//the column will belong to the new created table and must be dropped from that table
				columnInTable.setTABLE_NAME(this.tableCreated.getName());
				columnsToDrop.add(columnInTable);
			}
		}

		return columnsToDrop;
	}

	public Expression getWhereJoinCondition() {
		return whereCondition;
	}

	public void setWhereJoinCondition(Expression whereJoinCondition) {
		this.whereCondition = whereJoinCondition;
	}

	@Override
	public ArrayList<String> getSMOs() {
		return smos;
	}

	@Override
	public String getPatternRecognitionMessage() {
		return patternRecognitionMessage;
	}

	@Override
	public boolean checkLabels(String[] labels) {
		
		if ((labels.length > patternLabels.length) || (labels.length < patternLabels.length-1))
			return false;
		
		for(int i=0; i<patternLabels.length; i++){
			//last label
			if(i==patternLabels.length-1){
				try{
				if(labels[i].equals(patternLabels[i]))
					return true;
				return false;
				}
				catch(IndexOutOfBoundsException iEx){
					//no label at index i 
					return true;
				}
			}
			if(!labels[i].equals(patternLabels[i]))
				return false;
		}
		return true;
	}

}
