package edu.ucla.cs.SMOsTranslator.Pattern;

import java.sql.SQLException;
import java.util.ArrayList;

import net.sf.jsqlparser.expression.Expression;
import net.sf.jsqlparser.schema.Column;
import net.sf.jsqlparser.schema.Table;
import net.sf.jsqlparser.statement.Statement;
import net.sf.jsqlparser.statement.alter.table.AlterTableAddColumnStatement;
import net.sf.jsqlparser.statement.drop.DropTable;
import net.sf.jsqlparser.statement.update.Update;
import edu.ucla.cs.SMOsTranslator.Pattern.Exceptions.UnrecognizedPatternException;
import edu.ucla.cs.SMOsTranslator.Pattern.visitor.ColumnJoinVisitor;
import edu.ucla.cs.SMOsTranslator.Pattern.visitor.ExtractColumnsVisitor;
import edu.ucla.cs.SMOsTranslator.Pattern.visitor.PartitionConditionVisitor;
import edu.ucla.cs.SMOsTranslator.Pattern.visitor.ReplaceAliasVisitor;
import edu.ucla.cs.SMOsTranslator.Pattern.visitor.ReplaceJoinConditionTableVisitor;
import edu.ucla.cs.SMOsTranslator.Pattern.visitor.ReversePartitionConditionVisitor;
import edu.ucla.cs.SMOsTranslator.Pattern.visitor.WhereConditionVisitor;
import edu.ucla.cs.SMOsTranslator.Pattern.visitor.WherePartitionVisitor;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.dbevolution.DbBackend.DbColumn;
import edu.ucla.cs.dbevolution.DbBackend.DbTable;
import edu.ucla.cs.velocity.template.AddColumnTemplate;
import edu.ucla.cs.velocity.template.CopyTableTemplate;
import edu.ucla.cs.velocity.template.DropColumnTemplate;
import edu.ucla.cs.velocity.template.DropTableTemplate;
import edu.ucla.cs.velocity.template.JoinTableTemplate;
import edu.ucla.cs.velocity.template.JoinTemplateSource;
import edu.ucla.cs.velocity.template.MergeTableTemplate;
import edu.ucla.cs.velocity.template.PartitionTableTemplate;
import edu.ucla.cs.velocity.template.PartitionTemplateSource;
import edu.ucla.cs.velocity.template.RenameColumnTemplate;


public class JoinPattern2 extends Pattern{


	private final String[] patternLabels = {Pattern.ALTER_ADD_COL, Pattern.UPDATE, Pattern.DROP_TAB};

	private final String patternRecognitionMessage = "Patter Join (type 2) recognized";

	//statement of the pattern
	private Update update;
	private AlterTableAddColumnStatement addColumn;
	private DropTable dropTable;

	//ADD COLUMN statement
	//Column added with the ALTER TABLE ADD COLUMN statement
	private Column columnAdded;
	private Table tableAltered;

	//UPDATE statement
	//WHERE conditions of the UPDATE statement
	private Expression whereCondition;
	private ArrayList<Table> tablesFromUpdate;
	private Column destinationColumn;
	private Table destinationTable;
	private Column sourceColumn;
	private Table sourceTable;

	//DROP statement
	//Table dropped
	//LIMITATION: at the moment the pattern support only one table as a source. 
	//This hypothesis brings to only ONE table in the DROP statement
	private ArrayList<Table> tablesDropped;



	private ArrayList<Table> tablesToCopy;
	//contains the list of tables to be joined
	//Once a table has been copied, the join is performed between the copy and the others tables 
	private ArrayList<Table> tablesToJoin;

	private ArrayList<String> smos;

	//the version of the schema the pattern is going to work with
	private Schema schema;

	public JoinPattern2(Schema schema){
		tablesToCopy = new ArrayList<Table>();
		tablesFromUpdate = new ArrayList<Table>();
		tablesDropped = new ArrayList<Table>();

		smos = new ArrayList<String>();

		this.schema = schema;
	}


	@Override
	public boolean isPattern(String[] labels, ArrayList<Statement> stmts) throws UnrecognizedPatternException {

		boolean isTable = false;

		//Check #1
		if(checkLabels(labels)){

			addColumn = (AlterTableAddColumnStatement)stmts.get(0);
			update = (Update)stmts.get(1);
			if(stmts.size()>2){
				//retrieve the Drop statement
				//the pattern has been already verified to contain a DROP statement
				//no check at this point
				this.dropTable = (DropTable)stmts.get(2);
			}

			//extract the object from the ADD COLUMN statement
			//get the table from the ADD COLUMN statement
			tableAltered = addColumn.getTable();
			//get the added column
			//LIMITATION: the pattern manage only one column at the moment
			columnAdded = addColumn.getColumnDefinitions().get(0).getColumnName();

			//extract the object from the UPDATE statement
			//read the tables involved in the UPDATE
			//force the cast to an ArrayList of Table (no cast exception from the grammar)
			tablesFromUpdate = (ArrayList<Table>)this.update.getTables();
			
			if(tablesFromUpdate.size() == 1){
				throw new UnrecognizedPatternException("The INSERT statement must consider two table to be identified as a JOIN pattern\n");
			}
			
			//get the column in the SET clause
			//LIMITATION: the pattern manage only one column in the SET clause, one column can be updated at time
			destinationColumn = (Column)update.getColumns().get(0);

			//Check #1
			//verify the table in the ADD COLUMN statement is one of the tables in the UPDATE
			for(Table table: tablesFromUpdate){
				if(table.getName().equalsIgnoreCase(tableAltered.getName())){
					isTable = true;
				}
			}
			if(!isTable){
				return false;
			}

			//Check #2
			//verify the column in the ADD COLUMN is the column in the SET clause
			//verify also the column belongs to the table from the ADD COLUMN 
			if(destinationColumn.getColumnName().equalsIgnoreCase(columnAdded.getColumnName())){
				//the column in the SET clause contains the table alias as the table name
				//get the alias and check the table associated to this alias is the same as the ADD COLUMN statament
				String tableAlias = destinationColumn.getTable().getName();
//				Table table = getTableByAlias(tableAlias);
//				if((table == null) || (!table.getName().equalsIgnoreCase(tableAltered.getName()))){
//					return false;
//				}
				//Check #3
				//The source of the SET clause must be a COLUMN
				//this means another table is being using to update the altered one
				//LIMITATION: force the statement to return the first object in the Expression list
				//if the object is a Column than the source of the UPDATE is a table
				if(!(update.getExpressions().get(0) instanceof Column)){
					return false;
				}
				sourceColumn = (Column)update.getExpressions().get(0);
				//sourceTable = getTableByAlias(sourceColumn.getTable().getName());
				for(Table t : (ArrayList<Table>)update.getTables()){
					if(!t.getName().equalsIgnoreCase(tableAltered.getName()))
						sourceTable = t;
				}


				//Check #4
				//The table in the DROP statement is the source table of the UPDATE
				//this means the table is become useless after the join and it can be dropped
				//LIMITATION: force the DROP TABLE to return the first table in the list
				if(dropTable != null){
					tablesDropped.add(dropTable.getTable().get(0));
					if(sourceTable.getName().equalsIgnoreCase(dropTable.getTable().get(0).getName())){
						return true;
					}
				}
				return true;
			}
		}
		return false;
	}


	@Override
	public ArrayList<String> translatePattern(ArrayList<Statement> stmts) throws SQLException {

		boolean isDropped = false;
		boolean isDestinationTablePartitioned = false;

		String tableToBePartitioned = null;
		String tableToBeJoined = tableAltered.getName();

		Expression whereJoinCondition = null;

		//VISITOR
		ExtractColumnsVisitor statementColumnVisitor = new ExtractColumnsVisitor();
		WhereConditionVisitor whereVisitor = new WhereConditionVisitor();
		ReplaceAliasVisitor aliasVisitor = new ReplaceAliasVisitor();
		WherePartitionVisitor partitionVisitor = new WherePartitionVisitor();
		PartitionConditionVisitor conditionVisitor = null;
		ReversePartitionConditionVisitor reverseVisitor = null;


		//initialize tables to join
		tablesToJoin = new ArrayList<Table>();
		for(Table table: tablesFromUpdate){
			tablesToJoin.add(new Table(table.getAlias(), table.getName()));
		}

		//replace the alias in the statement
		aliasVisitor.replaceTable(update);

		//initialize the destination table to the table from the ADD COLUMN statement
		//during the pattern translation the table will be substituted if some operation is performed on the original table 
		//make a copy of the original table
		destinationTable = new Table(null, tableAltered.getName());

		this.whereCondition = update.getWhere();

		//EXTRACT the JOIN CONDITION
		whereJoinCondition = whereVisitor.getWhereJoinCondition(whereCondition);

		//TABLEs TO COPY
		//The updated statement involves to tables
		//one is the destination table, the other is the source table.
		//the tables to be copied is the destination table is it is not DROPpes after the UPDATE has been performed
		//if a table is a source of the join, but the table is dropped after the join, it doesn't need to be copied

		//if no tables dropped (usually the source) 
		//copy the source table
		if(tablesDropped == null || tablesDropped.size() == 0){
			tablesToCopy.add(new Table(null, sourceTable.getName()));
		}
		//COPY TABLE
		for(Table table: tablesToCopy){
			smos.add(CopyTableTemplate.getCopyTableSMO(table));
			replaceJoinTable(table.getName(), CopyTableTemplate.getCopiedTableName(table.getName()), whereJoinCondition);
		}
		

		//COLUMN NAME CLASH
		//No column clash should occur: all the column except the source and the join column are kept in the source table
		//if the destination table already has the source column, DROP the column
		ArrayList<Table> tables = new ArrayList<Table>();
		tables.add(tableAltered);
		for(DbColumn column: DbColumn.getColumnsFromTables(tables)){
			if(column.getCOLUMN_NAME().equalsIgnoreCase(sourceColumn.getColumnName())){
				smos.add(DropColumnTemplate.getDropColumnSMO(column.getCOLUMN_NAME(),tableAltered.getName()));
			}
		}


		//PARTITION
		Expression wherePartition = partitionVisitor.getPartitionCondition(whereCondition);
		if(wherePartition != null){
			//at least a partition condition exist
			conditionVisitor = new PartitionConditionVisitor();
			Column columnPartitioned = conditionVisitor.getColumnFromPartitionCondition(wherePartition);
			tableToBePartitioned = columnPartitioned.getTable().getName();
			tableToBePartitioned = getTableByAlias(tableToBePartitioned).getName();
			if(tableToBePartitioned == null){
				//if no table is defined for the column, it means there's no name clash for it
				//only one table in the joined tables has this column
				//retrieve the name of the table from the database
				tableToBePartitioned = DbTable.getTableNameFromColumnAndRevision(tablesFromUpdate, columnPartitioned);
			}
			//if the partitioned table has been copied, replace the table
			for(Table t: tablesToCopy){
				if(tableToBePartitioned.equalsIgnoreCase(t.getName()))
					tableToBePartitioned = CopyTableTemplate.getCopiedTableName(tableToBePartitioned);
			}

			if(tableToBePartitioned.equalsIgnoreCase(tableAltered.getName())){
				isDestinationTablePartitioned = true;
				//update the destination of the JOIN
				destinationTable.setName(PartitionTemplateSource.getKeptPartitionTable(tableToBePartitioned));
			}

			tableToBeJoined = PartitionTemplateSource.getKeptPartitionTable(tableToBePartitioned);

			//REPLACE THE JOIN TABLE to the partitioned table
			replaceJoinTable(tableToBePartitioned, tableToBeJoined,whereJoinCondition);

			//**********************************************************//			
			//Check if the table is being partitioned is dropped		//
			//**********************************************************//
			//		if the table is dropped do he following steps		//

			//1)	partition the table									//
			//2)	drop useless partition								//
			//3)	join the partition_1								//
			//**********************************************************//
			//		if the table is not dropped							//

			//1)	copy the table must be partitioned					//
			//2)	partition the copy of the table						//
			//3)	drop useless partition								//
			//4)	join the partition_1 								//
			//**********************************************************//

			//1) check the table being partitioned will be dropped

			for(Table table: tablesDropped){
				if(table.getName().equalsIgnoreCase(tableToBePartitioned)){
					isDropped = true;
					break;
				}
			}
			if( (!isDropped) && (!isDestinationTablePartitioned)){
				smos.add(CopyTableTemplate.getCopyTableSMO(new Table("", tableToBePartitioned)));

				//REMOVE THE TABLE FROM THE LIST OF TABLES TO BE COPIED
				removeTableFromCopy(tableToBePartitioned);

				tableToBePartitioned = CopyTableTemplate.getCopiedTableName(tableToBePartitioned);

				//REPLACE THE JOIN TABLE to the partitioned table
				replaceJoinTable(tableToBeJoined, PartitionTemplateSource.getKeptPartitionTable(tableToBePartitioned),  whereJoinCondition);
			}


			PartitionTemplateSource partitionSource = new PartitionTemplateSource();
			reverseVisitor = new ReversePartitionConditionVisitor();

			partitionSource.setTableName(tableToBePartitioned);
			partitionSource.setTablePartition1(tableToBePartitioned);
			partitionSource.setTablePartition2(tableToBePartitioned);
			partitionSource.setPartitionCondition1(replaceNonSmoChar(wherePartition.toString()));
			partitionSource.setPartitionCondition2(replaceNonSmoChar(reverseVisitor.getReversePartitionCondition(wherePartition).toString()));
			smos.add(PartitionTableTemplate.getPartitionTableSMO(partitionSource));


			//DROP the USELESS PARTITION
			//by convention the table with the useful partition condition is the partition one
			//However in the JOIN - UPDATE pattern if the table partitioned is the destination table
			//after the partition and the join, the "useless" partition must be merged back to create the original table
			if(!isDestinationTablePartitioned){
				smos.add(DropTableTemplate.getDropTableSMO(PartitionTemplateSource.getDroppedPartitionTable(tableToBePartitioned)));
			}
		}

		//SMO JOIN
		//the join operator join two tables and drop the second one.
		//If the statement drops only one table, a copy of the other table is created.
		//if the statement drops more than one table (every table involved in the SELECT) then it is correct for the pattern, no copy generated 

//		//COPY TABLE
//		for(Table table: tablesToCopy){
//			smos.add(CopyTableTemplate.getCopyTableSMO(table));
//			replaceJoinTable(table.getName(), CopyTableTemplate.getCopiedTableName(table.getName()), whereJoinCondition);
//		}


		//DROP COLUMN
		//before performing the JOIN; DROP the COLUMNs not involved in it
		//the table must remain only with the following columns:
		//- columns in WHERE condition
		//- sourceColumn: the column used to update the data from
		//- column of the partition

		ArrayList<Column> patternColumns = new ArrayList<Column>();
		patternColumns = statementColumnVisitor.getColumnInUpdate(update);

		ArrayList<Table> sourceTables = new ArrayList<Table>();
		sourceTables.add(sourceTable);
		ArrayList<DbColumn> columnsInTables = DbColumn.getColumnsFromTables(sourceTables);
		for(DbColumn columnInSourceTable: columnsInTables){
			//recupero dal db elenco delle colonne associate alla tabelle sorgente
			//faccio un confronto tra le colonne originali e le colonne del db
			//droppo tutte le colonne tranne la colonna sorgente e quelle coninvolte nel join
			if(!isColumnForPattern(columnInSourceTable, patternColumns)){
				
				//if the table has been copied
				//adjust the column's table name to the copied table
				for(Table t: tablesToCopy){
					if(columnInSourceTable.getTABLE_NAME().equalsIgnoreCase(t.getName()))
						columnInSourceTable.setTABLE_NAME(CopyTableTemplate.getCopiedTableName(columnInSourceTable.getTABLE_NAME()));
				}
				
				smos.add(DropColumnTemplate.getDropColumnSMO(columnInSourceTable));
			}
		}


		//JOIN TABLE
		JoinTemplateSource joinSource = new JoinTemplateSource(); 
		joinSource.setTable1(tablesToJoin.get(0));
		joinSource.setTable2(tablesToJoin.get(1));
		joinSource.setWhereCondition(whereJoinCondition);
		joinSource.setJoinedTable(destinationTable);
		smos.add(JoinTableTemplate.getJoinTableSMO(joinSource));
		
		if(!sourceColumn.getColumnName().equalsIgnoreCase(destinationColumn.getColumnName()))
			smos.add(RenameColumnTemplate.getRenameColumnSMO(sourceColumn.getColumnName(), destinationColumn.getColumnName(), destinationTable.getName()));
		
		//check the JOIN condition
		for(String column : new ColumnJoinVisitor().getJoinDifferentColumns(whereJoinCondition))
			smos.add(DropColumnTemplate.getDropColumnSMO(column, destinationTable.getName()));


		//MERGE the partition of the table involved in the join with the partition not involved
		if(isDestinationTablePartitioned){

			//before merging the two tables, ADD the column to the second art
			//during the join the column is added to the destination table (partition1)
			//add the column to the table not involved in the join

			//Get the column DATA TYPE from the ADD COLUMN
			//String dataType = DataTypeTranslator.translateDataType(addColumn.getColumnDefinitions().get(0).getColDataType());
			String partition1 = PartitionTemplateSource.getDroppedPartitionTable(tableToBePartitioned);
			smos.add(AddColumnTemplate.getAddColumnSMO(columnAdded.getColumnName(), addColumn.getColumnDefinitions().get(0).getColDataType().toString(), partition1, ""));


			String partition2 = PartitionTemplateSource.getKeptPartitionTable(tableToBePartitioned);
			String table = tableToBePartitioned;
			smos.add(MergeTableTemplate.getMergeTableSMO(partition1, partition2, table));
		}
		return smos;
	}


	private Table getTableByAlias(String tableAlias) {
		for(Table table: tablesFromUpdate){
			if(table.getAlias().equalsIgnoreCase(tableAlias)){
				return table;
			}
		}
		return new Table(null,tableAlias);
	}

	private boolean isColumnForPattern(DbColumn columnInSourceTable,ArrayList<Column> patternColumns) {
		for(Column column: patternColumns){
			if(column.getColumnName().equalsIgnoreCase(columnInSourceTable.getCOLUMN_NAME())){
				return true;
			}
		}
		return false;
	}


	//	private void addColumnToPatternColumns(Column sourceColumn2,ArrayList<Column> patternColumns) {
	//		//The method before adding the column to the list of columns involved in the pattern check
	//		//if the column already exist in the list
	//		//the columnn to be added to the list are the source column and the ones involved in the WHERE condition
	//		//clash column could occurr in multiple column in the list
	//		boolean toAdd = true;
	//		if( (patternColumns == null) || (patternColumns.size() == 0)){
	//			//add the column without performing any other check
	//			patternColumns.add(sourceColumn2);
	//		}
	//		else{
	//			for(Column column: patternColumns){
	//				if(column.getColumnName().equalsIgnoreCase(sourceColumn2.getColumnName())){
	//					toAdd = false;
	//				}
	//			}
	//			if(toAdd){
	//				patternColumns.add(sourceColumn2);
	//			}
	//		}
	//	}


	private void removeTableFromCopy(String tableToBePartitioned) {

		int tableToRemove = -1;
		for(Table table: tablesToCopy){
			if(table.getName().equalsIgnoreCase(tableToBePartitioned)){
				tableToRemove = tablesToCopy.indexOf(table);
			}
		}
		if(tableToRemove != -1){
			tablesToCopy.remove(tableToRemove);
		}
	}

	private void replaceJoinTable(String tableToReplace,String newTable, Expression whereJoinCondition) {
		//iterate over the table in the join list and replace the table with the given one
		for(Table table: tablesToJoin){
			if(table.getName().equalsIgnoreCase(tableToReplace)){
				table.setName(newTable);
				replaceTableInJoinCondition(tableToReplace, newTable, whereJoinCondition);
				return;
			}
		}
	}


	private void replaceTableInJoinCondition(String tableToReplace, String newTable, Expression whereJoinCondition) {

		ReplaceJoinConditionTableVisitor replace = new ReplaceJoinConditionTableVisitor();
		replace.replaceTableInJoinCondition(tableToReplace, newTable, whereJoinCondition);
	}


	public String getPatternRecognitionMessage() {
		return patternRecognitionMessage;
	}


	@Override
	public boolean checkLabels(String[] labels) {

		if ((labels.length > patternLabels.length) || (labels.length < patternLabels.length-1))
			return false;
		
		for(int i=0; i<patternLabels.length; i++){
			//last label
			if(i==patternLabels.length-1){
				try{
				if(labels[i].equals(patternLabels[i]))
					return true;
				return false;
				}
				catch(IndexOutOfBoundsException iEx){
					//no label at index i 
					return true;
				}
			}
			try{
			if(!labels[i].equals(patternLabels[i]))
				return false;
			}
			catch(IndexOutOfBoundsException iEx){
				return false;
			}
		}
		return true;
	}

}
