package edu.ucla.cs.SMOsTranslator.Pattern;

import java.sql.SQLException;
import java.util.ArrayList;

import net.sf.jsqlparser.expression.Expression;
import net.sf.jsqlparser.schema.Column;
import net.sf.jsqlparser.schema.Table;
import net.sf.jsqlparser.statement.Statement;
import net.sf.jsqlparser.statement.drop.DropTable;
import net.sf.jsqlparser.statement.insert.Insert;
import edu.ucla.cs.SMOsTranslator.Pattern.Exceptions.TooManyTablesException;
import edu.ucla.cs.SMOsTranslator.Pattern.visitor.ExtractColumnsVisitor;
import edu.ucla.cs.SMOsTranslator.Pattern.visitor.ReversePartitionConditionVisitor;
import edu.ucla.cs.SMOsTranslator.Pattern.visitor.SourceTableVisitor;
import edu.ucla.cs.SMOsTranslator.Pattern.visitor.WhereConditionVisitor;
import edu.ucla.cs.SMOsTranslator.Pattern.visitor.WherePartitionVisitor;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.exception.IllegalSMOException;
import edu.ucla.cs.db.prism.backend.schema.exception.MergingDifferentTablesException;
import edu.ucla.cs.velocity.template.CopyTableTemplate;
import edu.ucla.cs.velocity.template.DropColumnTemplate;
import edu.ucla.cs.velocity.template.DropTableTemplate;
import edu.ucla.cs.velocity.template.JoinTableTemplate;
import edu.ucla.cs.velocity.template.MergeTableTemplate;
import edu.ucla.cs.velocity.template.PartitionTableTemplate;
import edu.ucla.cs.velocity.template.PartitionTemplateSource;


public class MergePattern extends Pattern {

	private final String patternRecognitionMessage = "Patter Merge recognized";
	private final String[] patternLabels = {Pattern.INSERT, Pattern.DROP_TAB};

	//Statement of the pattern
	private Insert insert;
	private DropTable dropTable;

	//INSERT statement
	private Table sourceTable;
	private Table destinationTable;
	private ArrayList<Column> selectedColumns;
	private ArrayList<Column> insertColumns;

	//DROP TABLE statement
	private Table tableDrop;

	private ArrayList<String> smos;

	//the version of the schema the pattern is going to work with
	private Schema schema;


	/**
	 * Public Constructor
	 * @param schema the {@link Schema} to modify
	 */
	public MergePattern(Schema schema){
		this.schema = schema;
		smos = new ArrayList<String>();
	}


	public boolean isPattern(String[] labels, ArrayList<Statement> stmts) throws TooManyTablesException, MergingDifferentTablesException {

		//VISITOR
		SourceTableVisitor sourceVisitor = null;

		//Check #1
		if(checkLabels(labels)){

			//Get the statements in the pattern
			insert = (Insert)stmts.get(0);
			try{
				dropTable = (DropTable)stmts.get(1);
			}
			catch(IndexOutOfBoundsException iEx){
				dropTable = null;
			}

			//INSERT statement
			destinationTable = insert.getTable();
			sourceVisitor = new SourceTableVisitor();
			ArrayList<Table> sourceTables = sourceVisitor.getTablesFromInsert(insert);
			if(sourceTables.size()>1){
				throw new TooManyTablesException("The Insert statement in the Merge pattern must contain only one source table");
			}
			sourceTable = sourceTables.get(0);


			//DROP TABLE statement
			try{
				if(dropTable.getTable().size()>1){
					throw new TooManyTablesException("The Drop Table statement in the Merge pattern must contain only one table");
				}
				tableDrop = dropTable.getTable().get(0);
			}
			catch(NullPointerException nEx){}

			//Check #2
			//the source table and the dropped table are the same
			try{
				if(!tableDrop.getName().equalsIgnoreCase(sourceTable.getName())){
					return false;
				}
			}
			catch(NullPointerException  nEx){}
			
			//Check #3
			//The destination table must be already in the schema, otherwise it can't be a MERGE pattern
			if(schema.getTable(destinationTable.getName()) == null)
				return false;

			return true;
		}
		return false;
	}

	@Override
	public ArrayList<String> translatePattern(ArrayList<Statement> stmts) throws SQLException, IllegalSMOException {

		edu.ucla.cs.db.prism.backend.schema.Table temporarySourceTable = null;
		edu.ucla.cs.db.prism.backend.schema.Table temporaryDestinationTable = null;
		
		
		try{
			temporarySourceTable = schema.getTable(this.sourceTable.getName()).clone();
			temporaryDestinationTable = schema.getTable(this.destinationTable.getName()).clone();
		}
		catch(NullPointerException nEx){
			if(temporarySourceTable == null)
				System.err.println("The source table "+ sourceTable.getName() +" does not exist on the original schema");
			else
				System.err.println("The destination table "+ destinationTable.getName() +" does not exist on the original schema");
			return null;
		}

		edu.ucla.cs.db.prism.backend.schema.Table sourceTable = schema.getTable(this.sourceTable.getName());
		edu.ucla.cs.db.prism.backend.schema.Table destinationTable = schema.getTable(this.destinationTable.getName());

		ExtractColumnsVisitor columnVisitor = new ExtractColumnsVisitor();
		boolean isDestinationToMerge = false;
		WherePartitionVisitor partitionVisitor = new WherePartitionVisitor();
		Expression wherePartition = null;

		//extract the columns from the two table
		//columns from the source table
		selectedColumns = columnVisitor.getSelectedColumns(insert);
		//columns from the destination table
		insertColumns = insert.getColumns()!=null?(ArrayList<Column>)insert.getColumns():null;

		if(tableDrop == null){
			smos.add(CopyTableTemplate.getCopyTableSMO(sourceTable.getTableName()));
			temporarySourceTable.setTableName(CopyTableTemplate.getCopiedTableName(tableDrop.getName()));
		}
		try{
			if(insertColumns.size() != destinationTable.getNumColumns()){
				smos.add(CopyTableTemplate.getCopyTableSMO(destinationTable.getTableName()));
				temporaryDestinationTable.setTableName(CopyTableTemplate.getCopiedTableName(temporaryDestinationTable.getTableName()));
				isDestinationToMerge = true;
			}
		}
		catch(NullPointerException nEx){

		}

		//extract, if exist the where partition condition
		wherePartition = partitionVisitor.getPartitionCondition(new WhereConditionVisitor().getWhereCondition(insert));
		if(wherePartition != null){

			PartitionTemplateSource partitionSource = new PartitionTemplateSource();
			ReversePartitionConditionVisitor reverseVisitor = new ReversePartitionConditionVisitor();

			partitionSource.setTableName(temporarySourceTable.getTableName());
			partitionSource.setTablePartition1(temporarySourceTable.getTableName());
			partitionSource.setTablePartition2(temporarySourceTable.getTableName());
			partitionSource.setPartitionCondition1(replaceNonSmoChar(wherePartition.toString()));
			partitionSource.setPartitionCondition2(replaceNonSmoChar(reverseVisitor.getReversePartitionCondition(wherePartition).toString()));
			smos.add(PartitionTableTemplate.getPartitionTableSMO(partitionSource));


			//DROP the USELESS PARTITION
			//by convention the table with the useful partition condition is the partition one
			smos.add(DropTableTemplate.getDropTableSMO(PartitionTemplateSource.getDroppedPartitionTable(sourceTable.getTableName())));

			//replace the temporary source table with the kept partition
			temporarySourceTable.setTableName(PartitionTemplateSource.getKeptPartitionTable(temporarySourceTable.getTableName()));
		}

		try{
			//remove all the columns not in the select clause
			ArrayList<String> columnsToDrop = null;
			columnsToDrop = new ArrayList<String>();
			boolean toDrop;
			for(String column: temporarySourceTable.getColumns()){
				toDrop = true;
				for(Column col :  selectedColumns){
					if(column.equalsIgnoreCase(col.getColumnName()))
						toDrop = false;
				}
				if(toDrop)
					columnsToDrop.add(column);
			}
			for(String column: columnsToDrop){
				temporarySourceTable.removeColumn(column);
				smos.add(DropColumnTemplate.getDropColumnSMO(column, temporarySourceTable.getTableName()));
			}
		}
		catch(NullPointerException nEx){}

		try{
			//remove all the columns not in the select clause
			ArrayList<String> columnsToDrop = null;
			columnsToDrop = new ArrayList<String>();
			boolean toDrop;
			for(String column: temporaryDestinationTable.getColumns()){
				toDrop = true;
				for(Column col :  insertColumns){
					if(column.equalsIgnoreCase(col.getColumnName()))
						toDrop = false;
				}
				if(toDrop)
					columnsToDrop.add(column);
			}
			for(String column: columnsToDrop){
				temporaryDestinationTable.removeColumn(column);
				smos.add(DropColumnTemplate.getDropColumnSMO(column, temporaryDestinationTable.getTableName()));
			}
		}
		catch(NullPointerException nEx){}

		if(!temporarySourceTable.hasSameSignature(temporaryDestinationTable))
			throw new MergingDifferentTablesException("\nTranslation aborted: \nThe columns in the SELECT clause don't match the COLUMNs of the INSERT");

		smos.add(MergeTableTemplate.getMergeTableSMO(temporaryDestinationTable.getTableName(), temporarySourceTable.getTableName(), temporaryDestinationTable.getTableName()));

		//join the original destination table and the "merged" one in order to get the original columns

		if(isDestinationToMerge){
			//Build the where condition
			//As the class works on a Schema not IC aware, the information for the primary keys are not loaded into the schema 
			//In order to complete the join, it must be performed over the columns left in the destination table
			String whereCondition = "";
			if(temporaryDestinationTable.getColumns().size()>=1){
				whereCondition += temporaryDestinationTable.getColumn(0) +  " = " + temporaryDestinationTable.getColumn(0);
			}
			for(int i=1; i< temporaryDestinationTable.getColumns().size(); i++){
				whereCondition += ",";
				whereCondition += temporaryDestinationTable.getColumn(i) + " = " + temporaryDestinationTable.getColumn(i); 
			}

			smos.add(JoinTableTemplate.getJoinTableSMO(temporaryDestinationTable.getTableName(), destinationTable.getTableName(), destinationTable.getTableName(), whereCondition));
		}
		return smos;
	}


	public String getPatternRecognitionMessage() {
		return this.patternRecognitionMessage;
	}


	@Override
	public boolean checkLabels(String[] labels) {
		
		if (labels.length > patternLabels.length)
			return false;
		
		for(int i=0; i<patternLabels.length; i++){
			//last label
			if(i==patternLabels.length-1){
				try{
					if(labels[i].equals(patternLabels[i]))
						return true;
					return false;
				}
				catch(IndexOutOfBoundsException iEx){
					//no label at index i 
					return true;
				}
			}
			if(!labels[i].equals(patternLabels[i]))
				return false;
		}
		return true;
	}

}
