package edu.ucla.cs.SMOsTranslator.Pattern;

import java.sql.SQLException;
import java.util.ArrayList;

import net.sf.jsqlparser.statement.Statement;
import edu.ucla.cs.SMOsTranslator.Pattern.Exceptions.UnrecognizedPatternException;
import edu.ucla.cs.db.prism.backend.schema.exception.IllegalSMOException;


public abstract class Pattern {

	//pattern labels
	public static final String DROP_TAB ="DROP TABLE";
	public static final String CREATE_TAB ="CREATE TABLE";
	public static final String INSERT ="INSERT";
	public static final String ALTER_ADD_COL ="ALTER_ADD_COL";
	public static final String ALTER_MODIFY ="ALTER_MODIFY";
	public static final String ALTER_CHANGE ="ALTER_MODIFY";
	public static final String ALTER_DROP ="ALTER_DROP";
	public static final String UPDATE ="UPDATE";
	public static final String DELETE="DELETE";
	
	private ArrayList<String> smos;
	
	/**
	 * Check if the given set of SQL statements match the pattern
	 * @param labels Set of label corresponding to the given SQL statements
	 * @param stmts The list of statement
	 * @return true if the set of statement match the pattern
	 * @throws {@linkUnrecognizedPatternException}, {@link IllegalSMOException}
	 */
	public abstract boolean isPattern(String[] labels,ArrayList<Statement> stmts) throws UnrecognizedPatternException, IllegalSMOException;

	/**
	 * Translate the given SQL statements into the corresponding SMOs sequence 
	 * @param stmts The list of SQL statement to translate into an SMO sequence
	 * @return list of SMO
	 * @throws SQLException
	 */
	public abstract ArrayList<String> translatePattern(ArrayList<Statement> stmts) throws SQLException, IllegalSMOException;
	
	/**
	 * Get the set of SMO obtained by the translation
	 * @return
	 */
	public ArrayList<String> getSMOs(){
		return smos;
	}
	
	/**
	 * Return the message associated to the pattern recognition
	 * @return
	 */
	public abstract String getPatternRecognitionMessage();
	
	
	protected String replaceNonSmoChar(String expression){
		expression = expression.replace("<>", "!=");
		expression = expression.replace("\'", "\"");
		return expression;
	}
	
	/**
	 * Check the labels given to the batch of statement coincide with the pattern labels
	 * @param labels
	 * @return
	 */
	public abstract boolean checkLabels(String[] labels);
	
}
