package edu.ucla.cs.SMOsTranslator.Pattern.visitor;

import java.util.ArrayList;
import java.util.List;

import net.sf.jsqlparser.expression.AllComparisonExpression;
import net.sf.jsqlparser.expression.AnyComparisonExpression;
import net.sf.jsqlparser.expression.CaseExpression;
import net.sf.jsqlparser.expression.DateValue;
import net.sf.jsqlparser.expression.DoubleValue;
import net.sf.jsqlparser.expression.Expression;
import net.sf.jsqlparser.expression.ExpressionVisitor;
import net.sf.jsqlparser.expression.Function;
import net.sf.jsqlparser.expression.InverseExpression;
import net.sf.jsqlparser.expression.JdbcParameter;
import net.sf.jsqlparser.expression.LongValue;
import net.sf.jsqlparser.expression.NullValue;
import net.sf.jsqlparser.expression.Parenthesis;
import net.sf.jsqlparser.expression.StringValue;
import net.sf.jsqlparser.expression.TimeValue;
import net.sf.jsqlparser.expression.TimestampValue;
import net.sf.jsqlparser.expression.WhenClause;
import net.sf.jsqlparser.expression.operators.arithmetic.Addition;
import net.sf.jsqlparser.expression.operators.arithmetic.Division;
import net.sf.jsqlparser.expression.operators.arithmetic.Multiplication;
import net.sf.jsqlparser.expression.operators.arithmetic.Subtraction;
import net.sf.jsqlparser.expression.operators.conditional.AndExpression;
import net.sf.jsqlparser.expression.operators.conditional.OrExpression;
import net.sf.jsqlparser.expression.operators.relational.Between;
import net.sf.jsqlparser.expression.operators.relational.EqualsTo;
import net.sf.jsqlparser.expression.operators.relational.ExistsExpression;
import net.sf.jsqlparser.expression.operators.relational.ExpressionList;
import net.sf.jsqlparser.expression.operators.relational.GreaterThan;
import net.sf.jsqlparser.expression.operators.relational.GreaterThanEquals;
import net.sf.jsqlparser.expression.operators.relational.InExpression;
import net.sf.jsqlparser.expression.operators.relational.IsNullExpression;
import net.sf.jsqlparser.expression.operators.relational.ItemsListVisitor;
import net.sf.jsqlparser.expression.operators.relational.LikeExpression;
import net.sf.jsqlparser.expression.operators.relational.MinorThan;
import net.sf.jsqlparser.expression.operators.relational.MinorThanEquals;
import net.sf.jsqlparser.expression.operators.relational.NotEqualsTo;
import net.sf.jsqlparser.schema.Column;
import net.sf.jsqlparser.statement.StatementVisitor;
import net.sf.jsqlparser.statement.alter.table.AlterTableStatement;
import net.sf.jsqlparser.statement.create.table.CreateTable;
import net.sf.jsqlparser.statement.delete.Delete;
import net.sf.jsqlparser.statement.drop.DropTable;
import net.sf.jsqlparser.statement.insert.Insert;
import net.sf.jsqlparser.statement.replace.Replace;
import net.sf.jsqlparser.statement.select.AllColumns;
import net.sf.jsqlparser.statement.select.AllTableColumns;
import net.sf.jsqlparser.statement.select.PlainSelect;
import net.sf.jsqlparser.statement.select.Select;
import net.sf.jsqlparser.statement.select.SelectExpressionItem;
import net.sf.jsqlparser.statement.select.SelectItem;
import net.sf.jsqlparser.statement.select.SelectItemVisitor;
import net.sf.jsqlparser.statement.select.SelectVisitor;
import net.sf.jsqlparser.statement.select.SubSelect;
import net.sf.jsqlparser.statement.select.Union;
import net.sf.jsqlparser.statement.truncate.Truncate;
import net.sf.jsqlparser.statement.update.Update;


/**
 * Extract the {@link Column} for the current statement
 * @author fangar
 *
 */
public class ExtractColumnsVisitor implements StatementVisitor, SelectVisitor, ItemsListVisitor, SelectItemVisitor, ExpressionVisitor{

	
	private ArrayList<Column> columnsInStatement;
	private Expression whereCondition;
	

	/**
	 * Get the list of columns in an INSERT statement
	 * INSERT INTO table SELECT (col1, col2, ...) return (col1, col2, ...)
	 * @param insert
	 * @return the list of columns in the statement; //FIXME null if all the columns are selected
	 */
	public ArrayList<Column> getSelectedColumns(Insert insert) {
		columnsInStatement = new ArrayList<Column>();
		insert.getItemsList().accept(this);
		return columnsInStatement;
	}
	
	public ArrayList<Column> getColumnInUpdate(Update update){
		columnsInStatement = new ArrayList<Column>();
		update.accept(this);
		return columnsInStatement;
	}
	
	public Expression getWhereCondition(Insert insert) {
		insert.getItemsList().accept(this);
		return whereCondition;
	}

	public void visit(SubSelect subSelect){
		subSelect.getSelectBody().accept(this);
	}
	
	@Override
	public void visit(PlainSelect plainSelect){
		for(SelectItem item: (List<SelectItem>)plainSelect.getSelectItems()){
			item.accept(this);
		}
		if(plainSelect.getWhere()!=null){
			whereCondition = plainSelect.getWhere();
		}
	}
	
	public void visit(Column column){
		columnsInStatement.add(column);
	}

	@Override
	public void visit(Insert insert) {
		insert.getItemsList().accept(this);
	}
	
	
	@Override
	public void visit(Delete arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(CreateTable arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(AlterTableStatement arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(Replace arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(Select arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(Update update) {
		for(Column column: (ArrayList<Column>)update.getColumns()){
			column.accept(this);
		}
		for(Expression expr: (List<Expression>)update.getExpressions()){
			expr.accept(this);
		}
		update.getWhere().accept(this);
	}


	@Override
	public void visit(Truncate arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(Union arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(ExpressionList arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void visit(AllColumns arg0) {
		//*
		columnsInStatement = null;
	}

	@Override
	public void visit(AllTableColumns arg0) {
		//tableName.*
		columnsInStatement = null;
	}

	@Override
	public void visit(SelectExpressionItem selectExpreItem) {
		selectExpreItem.getExpression().accept(this);
	}


	@Override
	public void visit(NullValue arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(Function arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(InverseExpression arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(JdbcParameter arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(DoubleValue arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(LongValue arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(DateValue arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(TimeValue arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(TimestampValue arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(Parenthesis arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(StringValue arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(Addition arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(Division arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(Multiplication arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(Subtraction arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(AndExpression andExpression) {
		andExpression.getLeftExpression().accept(this);
		andExpression.getRightExpression().accept(this);
	}


	@Override
	public void visit(OrExpression orExpression) {
		orExpression.getLeftExpression().accept(this);
		orExpression.getRightExpression().accept(this);
	}


	@Override
	public void visit(Between arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(EqualsTo equalsTo) {
		equalsTo.getLeftExpression().accept(this);
		equalsTo.getRightExpression().accept(this);
	}


	@Override
	public void visit(GreaterThan arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(GreaterThanEquals arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(InExpression arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(IsNullExpression arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(LikeExpression arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(MinorThan arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(MinorThanEquals arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(NotEqualsTo arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(CaseExpression arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(WhenClause arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(ExistsExpression arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(AllComparisonExpression arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(AnyComparisonExpression arg0) {
		// TODO Auto-generated method stub
		
	}


	@Override
	public void visit(DropTable drop) {
		// TODO Auto-generated method stub
		
	}


}
