package edu.ucla.cs.SMOsTranslator.Pattern.visitor;

import java.util.ArrayList;
import java.util.List;

import net.sf.jsqlparser.expression.operators.relational.ExpressionList;
import net.sf.jsqlparser.expression.operators.relational.ItemsListVisitor;
import net.sf.jsqlparser.schema.Table;
import net.sf.jsqlparser.statement.Statement;
import net.sf.jsqlparser.statement.StatementVisitor;
import net.sf.jsqlparser.statement.alter.table.AlterTableStatement;
import net.sf.jsqlparser.statement.create.table.CreateTable;
import net.sf.jsqlparser.statement.delete.Delete;
import net.sf.jsqlparser.statement.drop.DropTable;
import net.sf.jsqlparser.statement.insert.Insert;
import net.sf.jsqlparser.statement.replace.Replace;
import net.sf.jsqlparser.statement.select.FromItemVisitor;
import net.sf.jsqlparser.statement.select.Join;
import net.sf.jsqlparser.statement.select.PlainSelect;
import net.sf.jsqlparser.statement.select.Select;
import net.sf.jsqlparser.statement.select.SelectVisitor;
import net.sf.jsqlparser.statement.select.SubJoin;
import net.sf.jsqlparser.statement.select.SubSelect;
import net.sf.jsqlparser.statement.select.Union;
import net.sf.jsqlparser.statement.truncate.Truncate;
import net.sf.jsqlparser.statement.update.Update;

/**
 * Return the tables for the current statement.
 */
public class TablesForStatementVisitor implements StatementVisitor, ItemsListVisitor, SelectVisitor, FromItemVisitor {

	
	private ArrayList<String> tablesForStatement;
	
	public ArrayList<String> getAffectedTablesFromStatement(Statement stmt){
		tablesForStatement = new ArrayList<String>();		
		stmt.accept(this);
		return tablesForStatement;
	}
	
	@Override
	public void visit(Insert insert) {
		insert.getItemsList().accept(this);
		tablesForStatement.add(insert.getTable().getName());
	}
	
	public void visit(SubSelect subSelect){
		subSelect.getSelectBody().accept(this);
	}
	
	public void visit(PlainSelect plainSelect){
		plainSelect.getFromItem().accept(this);
		
		if(plainSelect.getJoins() != null){
			List<Join> joins = plainSelect.getJoins();
			for(Join j: joins){
				j.getRightItem().accept(this);
			}
		}
	}
	
	public void visit(Table table){
		tablesForStatement.add(table.getName());
	}

	
	public void visit(DropTable drop) {
		for(Table table: drop.getTable()){
			tablesForStatement.add(table.getName());
		}
	}

	
	@Override
	public void visit(Delete delete) {
		for(Table table: delete.getTables()){
			tablesForStatement.add(table.getName());
		}
	}

	@Override
	public void visit(CreateTable create) {
		tablesForStatement.add(create.getTable().getName());
	}

	@Override
	public void visit(AlterTableStatement alterTable) {
		tablesForStatement.add(alterTable.getTable().getName());
	}

	@Override
	public void visit(Replace arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void visit(Select arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void visit(Update update) {
		for(Table table : (List<Table>)update.getTables()){
			tablesForStatement.add(table.getName());
		}
	}

	@Override
	public void visit(Truncate arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void visit(SubJoin arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void visit(Union arg0) {
		// TODO Auto-generated method stub
		
	}

	@Override
	public void visit(ExpressionList arg0) {
		// TODO Auto-generated method stub
		
	}

	
}
