package edu.ucla.cs.SMOsTranslator;

import java.sql.SQLException;

import net.sf.jsqlparser.statement.Statement;
import net.sf.jsqlparser.statement.alter.table.AlterTableAddColumnStatement;
import net.sf.jsqlparser.statement.alter.table.AlterTableChangeColumnStatement;
import net.sf.jsqlparser.statement.alter.table.AlterTableDropColumnStatement;
import net.sf.jsqlparser.statement.alter.table.AlterTableModifyColumnStatement;
import net.sf.jsqlparser.statement.alter.table.AlterTableStatement;
import net.sf.jsqlparser.statement.create.table.CreateTable;
import net.sf.jsqlparser.statement.drop.DropTable;
import net.sf.jsqlparser.statement.rename.RenameTable;
import edu.ucla.cs.SMOsTranslator.AlterTableTranslator.AlterTableAddColumnStatementTranslator;
import edu.ucla.cs.SMOsTranslator.AlterTableTranslator.AlterTableChangeColumnStatementTranslator;
import edu.ucla.cs.SMOsTranslator.AlterTableTranslator.AlterTableDropColumnStatementTranslator;
import edu.ucla.cs.SMOsTranslator.AlterTableTranslator.AlterTableModifyColumnStatementTranslator;
import edu.ucla.cs.SMOsTranslator.CreateTranslator.CreateTableStatementTranslator;
import edu.ucla.cs.SMOsTranslator.DropTranslator.DropTableStatementTranslator;
import edu.ucla.cs.SMOsTranslator.Rename.RenameTableStatementTranslator;
import edu.ucla.cs.db.prism.backend.schema.Schema;

public class StatementTranslatorDispatcher {


	/**
	 * Identify the type of the statement and return the right {@link StatementTranslator}
	 * @param stmt
	 * @return
	 * @throws UnknowStatementTranslatorException 
	 * @throws SQLException 
	 */
	public static StatementTranslator getStatementTranslator(Statement stmt,Schema schema) throws UnknowStatementTranslatorException{

		if(stmt instanceof CreateTable){
			return new CreateTableStatementTranslator();
		}
		else if(stmt instanceof AlterTableStatement){
			if (stmt instanceof AlterTableModifyColumnStatement){
				return new AlterTableModifyColumnStatementTranslator();
			}
			else if(stmt instanceof AlterTableDropColumnStatement){
				return new AlterTableDropColumnStatementTranslator();
			}
			else if(stmt instanceof AlterTableChangeColumnStatement){
				return new AlterTableChangeColumnStatementTranslator(schema);
			}
			else if(stmt instanceof AlterTableAddColumnStatement){
				return new AlterTableAddColumnStatementTranslator();
			}
		}
		else if(stmt instanceof DropTable){
			return new DropTableStatementTranslator();
		}
		else if(stmt instanceof RenameTable){
			return new RenameTableStatementTranslator();
		}
		else {
			throw new UnknowStatementTranslatorException("Traduzione statement Sconosciuta");
		}

		return null;
	}


}