package edu.ucla.cs.SMOsTranslator;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;
import org.jgrapht.DirectedGraph;
import org.jgrapht.alg.ConnectivityInspector;
import org.jgrapht.graph.DefaultDirectedGraph;

/**
 * Create a graph with the dependencies between the statement's tables
 * @author fangar
 *
 */
public class TableDependenciesGraph {


	private DirectedGraph<String, DependenciesGraphEdge> dependenciesGraph;
	
	Logger log = Logger.getLogger(TableDependenciesGraph.class);	
	
	/**
	 * Public constructor.
	 * Initialize the dependencies graph
	 */
	public TableDependenciesGraph(){
		dependenciesGraph = new DefaultDirectedGraph<String, DependenciesGraphEdge>(DependenciesGraphEdge.class);
	}

	/**
	 * Add the table to the dependencies graph
	 * @param table
	 */
	public void addTableToDependenciesGraph(String table){
		dependenciesGraph.addVertex(table);
	}

	/**
	 * Get an iterator over the graph vertex
	 * @return
	 */
	public Iterator<String> iterator(){
		return this.dependenciesGraph.vertexSet().iterator();
	}
	
	/**
	 * Add an edge between the given tables
	 * @param table1
	 * @param table2
	 */
	public void addDependencies(String table1, String table2){
		if(!dependenciesGraph.containsEdge(table1,table2) && (!table1.equalsIgnoreCase(table2))){
			dependenciesGraph.addEdge(table1, table2);
		}
	}

	/**
	 * Add the given tables to the dependencies graph.of tables (tabl1, table2, table3), add the following dependencies:
	 * <br>table1 -> table2
	 * <br>table1 -> table3
	 * <br>table2 -> table3
	 * @param tables list of tables
	 */
	public void addTablesToDependenciesGraph(List<String> tables){

		ArrayList<String> dependentTable = (ArrayList<String>)tables;


		//add all the affected tables to the graph
		//if the table already exist, the framework manage the double table( no other table is inserted)
		for(int i=0;i<dependentTable.size();i++){
			dependenciesGraph.addVertex(dependentTable.get(i));
		}

		//create a vertex between each pair of table in the list
		//loop through the list and add a vertex between the table and all the next tables
		//loop until i < size -1; no edge needed for the last table!!
		for(int i=0;i<dependentTable.size()-1;i++){
			for(int j=i+1;j<dependentTable.size();j++){
				dependenciesGraph.addEdge(dependentTable.get(i),dependentTable.get(j));
			}
		}
	}

	/**
	 * Compute the connected components of a {@link TableDependenciesGraph} 
	 */
	public List<Set<String>> connectedComponents(){
		
		ConnectivityInspector sci = new ConnectivityInspector(dependenciesGraph);
		List<Set<String>> connectedSubgraphs = sci.connectedSets();

		log.info("Strongly connected components computed");
		
		return connectedSubgraphs;
	}

	/**
	 * Get the size of the table dependencies graph
	 * @return
	 */
	public int getSize() {
		return this.dependenciesGraph.vertexSet().size();
	}


}
