package edu.ucla.cs.db.prism.backend;

import java.sql.Connection;
import java.sql.Timestamp;
import java.util.Iterator;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.SQL.SQL;
import edu.ucla.cs.db.prism.backend.SQL.SQLUnion;
import edu.ucla.cs.db.prism.backend.ded.DEDSetHistory;
import edu.ucla.cs.db.prism.backend.exception.SchemaErrorException;
import edu.ucla.cs.db.prism.backend.icmo.ICMOSMOSequence;
import edu.ucla.cs.db.prism.backend.icmo.MOFactory;
import edu.ucla.cs.db.prism.backend.icmo.MOImpl;
import edu.ucla.cs.db.prism.backend.marsManage.CodiGen;
import edu.ucla.cs.db.prism.backend.rewrite.PrimRewriteEngine;
import edu.ucla.cs.db.prism.backend.rewrite.QueryRewriteResult;
import edu.ucla.cs.db.prism.backend.rewrite.QueryRewriteResultSet;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.SchemaHistory;
import edu.ucla.cs.db.prism.backend.schema.SchemaHistoryLoader;
import edu.ucla.cs.db.prism.backend.schema.Table;
import edu.ucla.cs.db.prism.backend.schema.exception.IllegalSMOException;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.exceptions.ExpressionParsignErrorException;
import edu.ucla.cs.db.prism.backend.smo.AddColumnSMO;
import edu.ucla.cs.db.prism.backend.smo.CreateTableSMO;
import edu.ucla.cs.db.prism.backend.smo.JoinTableSMO;
import edu.ucla.cs.db.prism.backend.smo.SMOImpl;
import edu.ucla.cs.db.prism.backend.smo.SMOInterface;
import edu.ucla.cs.db.prism.backend.smo.exception.NoTableInSchemaException;
import edu.ucla.cs.db.prism.backend.smo.exception.SMOParsingErrorException;
import edu.ucla.cs.db.prism.backend.smo.exception.noColumnsMatchInfPres;
import edu.ucla.cs.db.prism.backend.smo.exception.noCommonColumnInfPres;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequenceInterface;
import edu.ucla.cs.db.prism.backend.smo.util.SMOFactory;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequence;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequenceHistory;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequenceHistoryLoader;
import edu.ucla.cs.db.prism.backend.smo.util.SMOTypeUnknownException;
import edu.ucla.cs.db.prism.backend.util.db.DBUtil;
import edu.ucla.cs.db.prism.gui.client.SingleQueryRewriteResult;

public class BackendUtil {

	public String dburltb = null;
	public String usertb = null;
	public String passwordtb = null;
	public String schemaName = null;
	public String metaSchemaName = null;
	public Connection conn = null;
	public SchemaHistory sh = null;
	public SMOSequenceHistory smosh = null;
	
	public SMOSequence newsmos = null;
	protected SMOSequence inverses = null;
	
	protected boolean hideComputation = true;
	
	public SMOSequence cachedNewsmos = null;
	protected SMOSequence cachedInverses = null;
	
	protected String cachedFwdDeds = null;
	protected String cachedBkwdDeds = null;
	protected int userStartingVersion;
	
	public Schema currentSchema = null;	
	public Schema startingSchema = null;	
	public SchemaHistory currentFwdSchemaHistory = null;
	public SchemaHistory currentInverseSchemaHistory = null;
	
	
	protected String[] inputqueries = null;
	protected QueryRewriteResultSet queryres = null;
	protected QueryLoader queryLoader = null;
	protected Timestamp timestamp = null;
	protected String userip = null;
	
	public BackendUtil(){
		
	}
	
	public BackendUtil(String dburltb, String usertb, String passwordtb,
			String schemaName, String version, String metaSchemaName, String userIp) {
		
		//logging some information about the users
		userip = userIp;
		timestamp = new Timestamp(System.currentTimeMillis());
		
		// MySQL DRIVER LOADING
		try {
			Class.forName("com.mysql.jdbc.Driver").newInstance();
		} catch (Exception ex) {
			ex.printStackTrace();
		}

		this.dburltb = dburltb;
		this.usertb = usertb;
		this.passwordtb = passwordtb;
		this.schemaName = schemaName;
		this.metaSchemaName = metaSchemaName;
		
		
		conn = DBUtil.connect(dburltb, metaSchemaName, usertb, passwordtb);
		
		sh = (new SchemaHistoryLoader(conn, schemaName))
				.loadSchemaHistoryFromDB(true);
				
		if(version.equals("first"))
			this.userStartingVersion = sh.getOldestMajorVersion().versionID;
		else 
			if(version.equals("last"))
				this.userStartingVersion = sh.getMostRecent().versionID;
			else
				this.userStartingVersion = Integer.parseInt(version);
		
		startingSchema = sh.getSchemaByVersion(userStartingVersion, 0);
		currentSchema = startingSchema;
		
		currentFwdSchemaHistory = new SchemaHistory();
		currentFwdSchemaHistory.addSchema(startingSchema);
		currentInverseSchemaHistory = new SchemaHistory();
		
		//THE FOLLOWING CAN BE MOVED IN ANOTHER METHOD TO MAKE LOADING FASTER!!
		smosh = (new SMOSequenceHistoryLoader(conn, schemaName)).loadSMOSequenceHistoryFromDB(false);
		newsmos = new SMOSequence();
		queryLoader = new QueryLoader(conn, schemaName, metaSchemaName);
		inputqueries = queryLoader.loadQueryTemplatesFromDB(userStartingVersion);
		queryres = new QueryRewriteResultSet(inputqueries);
	}

	/**
	 * Add the given SMO to the list of new smos being designed. Update inverses consequently.
	 * 
	 * @param smotext
	 * @return
	 */
	public SMOSequence addNewSMO(String smotext) {
		MOImpl smo = null;
		try {
		
			//smo = SMOSequenceHistoryLoader.parseSMOText(smotext);
			smo = MOFactory.createMOFromString(smotext);
		
		} catch (SMOParsingErrorException e1) {
			e1.printStackTrace();
		}
		if (smo != null) {
			setVersion(smo);
			Schema newSch = currentSchema.clone();
			
			
			// CARLO: CHANGED TO USE EXCEPTION 
			try {
					newSch.applyOperator(smo);
					// maintain the schema history updated
										
					currentFwdSchemaHistory.addSchema(newSch);
					if(newsmos instanceof ICMOSMOSequence)
						((ICMOSMOSequence)newsmos).addMos(smo);
					else
						newsmos.add((SMOImpl)smo);
					
					queryres.addQueryResult(smo);
					// update the inverses by adding the first candidate inverse of this new smo
					if(inverses == null){
						//smos inverses have never been computed so i compute all of them
						getInverseSMOs();	
					}	
					else{
						//add the inverse of this smo
						// ATTENTION: currentSchema here is the one BEFORE the new SMO being applied, 
						//to allow the inverse generation to know about column lists e.g., DROP TABLE page -> CREATE TABLE page(a,b,c)
						//if(smo instanceof ICMOImpl)
							if(inverses instanceof ICMOSMOSequence)
								((ICMOSMOSequence)inverses).getMos().addAll(((ICMOSMOSequence) smo.getBestCandidateInverses(currentSchema)).getMos());
							else
								inverses.smos.addAll(( smo.getBestCandidateInverses(currentSchema)).smos);
							
								//else
						//	inverses.getMos().addAll(smo.getBestCandidateInverses(currentSchema).smos);

					}
					
					currentSchema = newSch;
					
					//sometimes the ordering get lost... let's play safe and reassign versions..
					reassignFwdVersions();
					reassignInverseVersions();
					recomputeForwardSchemaHistory();
					recomputeInverseSchemaHistory();
				
			} catch (IllegalSMOException e) {
				
				//FIXME  fix smo version in case smo is not accepted, pay attention to SMO / ICMO
				//newsmos.getMos().lastElement().setTargetMajor(newsmos.smos.lastElement().getTargetMajor()+1);
				//newsmos.getMos().lastElement().setTargetMinor(0);
				
					System.err.println("The smo: " + smo.toMetadataString() + " cannot be applied to the given schema.");
					e.printStackTrace();
			} catch (ExpressionParsignErrorException e) {
				e.printStackTrace();
			} catch (SchemaErrorException e) {
				e.printStackTrace();
			}	
		}
		return newsmos;
	}

	protected void reassignInverseVersions() {
		
		if(inverses instanceof ICMOSMOSequence){
			for(int i = 0; i<((ICMOSMOSequence)inverses).getMos().size();i++){
				MOImpl inv = ((ICMOSMOSequence)inverses).getMos().get(i);
				MOImpl fwd = null;
				if(newsmos instanceof ICMOSMOSequence)
					 fwd = ((ICMOSMOSequence)newsmos).getMos().get(i);
				else
					fwd = newsmos.smos.get(i);
				inv.setTargetMajor(fwd.getSourceMajor());
				inv.setTargetMinor(fwd.getSourceMinor());
				inv.setSourceMajor(fwd.getTargetMajor());
				inv.setSourceMinor(fwd.getTargetMinor());
			}
		}
		else{
		for(int i = 0; i<inverses.smos.size();i++){
			MOImpl inv = inverses.smos.get(i);
			MOImpl fwd = newsmos.smos.get(i);
			inv.setTargetMajor(fwd.getSourceMajor());
			inv.setTargetMinor(fwd.getSourceMinor());
			inv.setSourceMajor(fwd.getTargetMajor());
			inv.setSourceMinor(fwd.getTargetMinor());

		}
		}
	}

	protected void recomputeInverseSchemaHistory() {
		 currentInverseSchemaHistory = inverses.generateSchemaHistory(currentSchema.clone(),true);	
	}

	/**
	 * get Data Migration SQLs from the given SMO
	 * 
	 * @param 
	 * @return
	 */
	
	public String[] getDM() {
		System.out.println("-- COMPUTING DATA MIGRAION -- ");
		//System.out.println(currentSchema.getTable("archive").getColType("ar_user").toString());
		String returnDM[] = new String[newsmos.size()];
		for(int i=0;i<newsmos.size();i++) { // for each new SMOs
			MOImpl tempsmo = newsmos.get(i); // smo sequence currentFwdSchemaHistory 
			
			// CARLO FIX: use the proper schema at each step instead of the current one always!!!	
			// returnDM[i] = tempsmo.getSQLDDL(currentSchema.clone(),schemaName);//sh.getCurrentSchema()
			Schema tsch = currentFwdSchemaHistory.getSchemaByVersion(tempsmo.getSourceMajor(), tempsmo.getSourceMinor());
			if(tsch != null)
				returnDM[i] = "-- " + tempsmo.toMetadataString() +" "+tempsmo.getSQLDDL(tsch,schemaName);//sh.getCurrentSchema()
			else
				System.err.println("BackendUtil.getDM: schema mismatch error");
			
			System.out.println("--- "+returnDM[i]+" ---");
		}				
		return returnDM;
	}
	
	/**
	 * Generate the views for the old schema, given the inverse SMOs and the current schema
	 * @param 
	 * @return
	 */
	public String[] getViews() {
		if (inverses == null)
			getInverseSMOs();

			
		Vector<String> views=new Vector<String>();
		Schema cs = startingSchema.clone();
		Iterator<Table> tit = cs.getTables().iterator();
		while(tit.hasNext()){
			Table tab = tit.next();
			//QueryRewriteResult qrr = queryres.results.lastElement();
			//querySchema qrr = queryres.results.elementAt(j);
			//System.out.println(
			String view  = "\n-- " + cs.versionID  + "-"+ (cs.versionID+1) + ".0 " + tab.getTableName() + ".0\n";			
			view += "CREATE OR REPLACE VIEW " + schemaName + "_" + cs.versionID +"." + tab.getTableName();
			view+= "(" + tab.getColumnsString() + ")";
			view+= " AS \n";
			// COMPUTE THE BODY OF THE VIEW BY MEANS OF REWRITING
			System.out.println("SELECT " + tab.getColumnsStringMars() + " FROM " + tab.getTableName() + " t;");
			
			SQL inputSQL = new SQL("SELECT " + tab.getColumnsStringMars() + " FROM " + tab.getTableName() + " t;");
				
			//System.out.println(inputSQL.toViewString(schemaName));
			//System.out.println(view+" "+rewriteQuery(inputSQL.toString()));
			
			String body = rewriteQuery(inputSQL.toString(),schemaName).getQueryResult();
			
			if(body != null)
				views.add(view+" "+body + ";");
				
		}
		//System.out.println("views.size():"+views.size());
		//System.out.println("newsmos.size():"+newsmos.size());
		String[] returnview=new String[views.capacity()];
		for(int i=0;i<views.size();i++) {
			returnview[i] = views.get(i);
		}
		return returnview;		
	}

	/**
	 * Set the version of a new SMO, as the following in the newsmos if any or
	 * as the first after the history in smosh.
	 * 
	 * @param smo
	 */
	protected void setVersion(MOImpl smo) {
		
		boolean empty = false;
		if(newsmos instanceof ICMOSMOSequence){
			if (((ICMOSMOSequence)newsmos).getMos().isEmpty())
				empty = true;
		}else{
			if(newsmos.smos.isEmpty())
				empty = true;
			
		}	
			
		if (empty) {
			
			
			// If there are no prior version of the schema  (i.e., no ICMO evolving it) we start from 0
			if(smosh.smoSequences.isEmpty()){
				smo.setSourceMajor(0);
				smo.setSourceMinor(0);
				smo.setTargetMajor(1);
				smo.setTargetMinor(0);
			}
			else{	
				ICMOSMOSequence temp  = (ICMOSMOSequence) smosh.smoSequences.lastElement();

				smo.setSourceMajor(temp.getMos().lastElement().getTargetMajor());
				smo.setSourceMinor(temp.getMos().lastElement().getTargetMinor());
				smo.setTargetMajor(temp.getMos().lastElement().getTargetMajor() + 1);
				smo.setTargetMinor(0);
			}
			
			
			} else {
			
		
			if(newsmos instanceof ICMOSMOSequence){	
				// SET THE MOST RECENT SMO AS AN INTERMEDIATE ONE
				((ICMOSMOSequence)newsmos).getMos().lastElement().setTargetMajor(((ICMOSMOSequence)newsmos).getMos().lastElement().getTargetMajor()-1);
				((ICMOSMOSequence)newsmos).getMos().lastElement().setTargetMinor(((ICMOSMOSequence)newsmos).getHighestMinor(((ICMOSMOSequence)newsmos).getMos().lastElement().getTargetMajor())+1);
				
				//SET THE NEW SMO AS THE NEW "TERMINATION"
				smo.setSourceMajor(((ICMOSMOSequence)newsmos).getMos().lastElement().getTargetMajor());
				smo.setSourceMinor(((ICMOSMOSequence)newsmos).getMos().lastElement().getTargetMinor());
				smo.setTargetMajor(((ICMOSMOSequence)newsmos).getMos().lastElement().getTargetMajor()+1);
				smo.setTargetMinor(0);
			}else{
				// SET THE MOST RECENT SMO AS AN INTERMEDIATE ONE
				newsmos.smos.lastElement().setTargetMajor(newsmos.smos.lastElement().getTargetMajor()-1);
				newsmos.smos.lastElement().setTargetMinor(newsmos.getHighestMinor(newsmos.smos.lastElement().getTargetMajor())+1);
				
				//SET THE NEW SMO AS THE NEW "TERMINATION"
				smo.setSourceMajor(newsmos.smos.lastElement().getTargetMajor());
				smo.setSourceMinor(newsmos.smos.lastElement().getTargetMinor());
				smo.setTargetMajor(newsmos.smos.lastElement().getTargetMajor()+1);
				smo.setTargetMinor(0);
				
			}

		}
	}

	/**
	 * Return the list of newsmos.
	 * 
	 * @return
	 */
	public SMOSequence getCurrentSMOs() {

		return newsmos;
	}

	/**
	 * Remove an smo from the list of current smos and recompute the
	 * currentSchema. It also cascade and remove every non-valid following SMO.
	 * Update inverses consequently.
	 * 
	 * @return
	 */
	public SMOSequence removeSMO(int index) {
		
		// remove the SMO and its inverse
		newsmos.removeReassignVersions(index);
		inverses.removeReassignVersions(index);
		queryres.remove(index);
		// update the current schema accordingly
		currentSchema = startingSchema.clone();
		currentFwdSchemaHistory.setSchemas(new Vector<Schema>());
		currentFwdSchemaHistory.getSchemas().add(currentSchema);
		// the temp vector is needed because removing we shift back and the
		// index is no more correct
		Vector<Integer> toberemoved = new Vector<Integer>();
		for (int i = 0; i < newsmos.size(); i++) {
			Schema tempSchema = currentSchema.clone();
			
			// CARLO: CHANGED TO USE EXCEPTION 
			try{
				tempSchema.applyOperator(newsmos.get(i));
				// for the successful SMOs add the new intermediate schema to currentFwdSchemaHistory
				currentFwdSchemaHistory.getSchemas().add(tempSchema);
				currentSchema = tempSchema;				
			}
			catch (IllegalSMOException e) {
				toberemoved.add(new Integer(i));
				System.err.println("The smo: " + newsmos.get(i).toMetadataString() + " cannot be applied therefore is removed from the SMO set.");
			} catch (ExpressionParsignErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (SchemaErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}	
		}
		
		// check other affected SMOs and remove them all, update inverses too
		// we do it backward to have the indexes valid after deletion
		for (int i = toberemoved.size()-1; i>=0; i--) {
			newsmos.removeReassignVersions(toberemoved.get(i));
			inverses.removeReassignVersions(toberemoved.get(i));
			queryres.remove(toberemoved.get(i).intValue());
		}
		
		// Reassign Versions (the above might fail in some cases)
		reassignFwdVersions();
		reassignInverseVersions();
		
		// regenerate a valid inverse schema history
		recomputeForwardSchemaHistory();
		recomputeInverseSchemaHistory();

		return newsmos;
	}
	
	protected void reassignFwdVersions() {

		
		if(newsmos instanceof ICMOSMOSequence){
			
			if(((ICMOSMOSequence)newsmos).getMos().size()>0){
				((ICMOSMOSequence)newsmos).getMos().get(0).setSourceMajor(startingSchema.versionID);
				((ICMOSMOSequence)newsmos).getMos().get(0).setSourceMinor(startingSchema.minorVersionID);
				((ICMOSMOSequence)newsmos).getMos().get(0).setTargetMajor(startingSchema.versionID);
				((ICMOSMOSequence)newsmos).getMos().get(0).setTargetMinor(startingSchema.minorVersionID+1);
				
				
				for(int i = 1 ; i<((ICMOSMOSequence)newsmos).getMos().size() ; i++){
					((ICMOSMOSequence)newsmos).getMos().get(i).setSourceMajor(((ICMOSMOSequence)newsmos).getMos().get(i-1).getTargetMajor());
					((ICMOSMOSequence)newsmos).getMos().get(i).setSourceMinor(((ICMOSMOSequence)newsmos).getMos().get(i-1).getTargetMinor());
					((ICMOSMOSequence)newsmos).getMos().get(i).setTargetMajor(((ICMOSMOSequence)newsmos).getMos().get(i-1).getTargetMajor());
					((ICMOSMOSequence)newsmos).getMos().get(i).setTargetMinor(((ICMOSMOSequence)newsmos).getMos().get(i-1).getTargetMinor()+1);
				}
				((ICMOSMOSequence)newsmos).getMos().lastElement().setTargetMajor(startingSchema.versionID+1);
				((ICMOSMOSequence)newsmos).getMos().lastElement().setTargetMinor(0);
			}
		}
		else{
			if(newsmos.smos.size()>0){
				newsmos.smos.get(0).setSourceMajor(startingSchema.versionID);
				newsmos.smos.get(0).setSourceMinor(startingSchema.minorVersionID);
				newsmos.smos.get(0).setTargetMajor(startingSchema.versionID);
				newsmos.smos.get(0).setTargetMinor(startingSchema.minorVersionID+1);
				
				
				for(int i = 1 ; i<newsmos.smos.size() ; i++){
					newsmos.smos.get(i).setSourceMajor(newsmos.smos.get(i-1).getTargetMajor());
					newsmos.smos.get(i).setSourceMinor(newsmos.smos.get(i-1).getTargetMinor());
					newsmos.smos.get(i).setTargetMajor(newsmos.smos.get(i-1).getTargetMajor());
					newsmos.smos.get(i).setTargetMinor(newsmos.smos.get(i-1).getTargetMinor()+1);
				}
				newsmos.smos.lastElement().setTargetMajor(startingSchema.versionID+1);
				newsmos.smos.lastElement().setTargetMinor(0);
			}
			
			
		}
		
	}

	/**
	 * Edit an smo from the list of current smos and recompute the
	 * currentSchema. It also cascade and remove every non-valid following SMO.
	 * Update inverses consequently.
	 * 
	 * @return
	 */
	public SMOSequence editSMO(String smo, int index) {
		SMOImpl editedSMO = null;
		try {
			editedSMO = SMOSequenceHistoryLoader.parseSMOText(smo);
		} catch (SMOParsingErrorException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		
		if(newsmos instanceof ICMOSMOSequence){
			editedSMO.setSourceMajor(((ICMOSMOSequence)newsmos).getMos().get(index).getSourceMajor());
			editedSMO.setSourceMinor(((ICMOSMOSequence)newsmos).getMos().get(index).getSourceMinor());
			editedSMO.setTargetMajor(((ICMOSMOSequence)newsmos).getMos().get(index).getTargetMajor());
			editedSMO.setTargetMinor(((ICMOSMOSequence)newsmos).getMos().get(index).getTargetMinor());
		
			// remove the SMO and its inverse
			((ICMOSMOSequence)newsmos).getMos().set(index, editedSMO);
		}else{
			editedSMO.setSourceMajor(newsmos.smos.get(index).getSourceMajor());
			editedSMO.setSourceMinor(newsmos.smos.get(index).getSourceMinor());
			editedSMO.setTargetMajor(newsmos.smos.get(index).getTargetMajor());
			editedSMO.setTargetMinor(newsmos.smos.get(index).getTargetMinor());
		
			// remove the SMO and its inverse
			newsmos.smos.set(index, editedSMO);
		}
		
		// update the current schema and the forward schema history accordingly
		currentSchema = startingSchema.clone();
		currentFwdSchemaHistory.getSchemas().add(currentSchema);

		// the temp vector is needed because removing we shift back and the
		// index is no more correct
		Vector<Integer> toberemoved = new Vector<Integer>();
		for (int i = 0; i < newsmos.size(); i++) {
			Schema tempSchema = currentSchema.clone();
			if(i==index){
				//ADD the first candidate inverse in the inverses at the position of the edited smo
			
				if (inverses instanceof ICMOSMOSequence)
					((ICMOSMOSequence)inverses).getMos().set(index, ((ICMOSMOSequence) editedSMO.getCandidateInverses(tempSchema).firstElement()).getMos().firstElement());
				else	
					inverses.smos.set(index,  editedSMO.getCandidateInverses(tempSchema).firstElement().smos.firstElement());
			}
			

			// CARLO: CHANGED TO USE EXCEPTION (copy of above dangerzone)
			
			try{
				tempSchema.applyOperator(newsmos.get(i));
				// for the successful SMOs add the new intermediate schema to currentFwdSchemaHistory
				currentFwdSchemaHistory.getSchemas().add(tempSchema);
				currentSchema = tempSchema;				
			}
			catch (IllegalSMOException e) {
				toberemoved.add(new Integer(i));
				System.err.println("The smo: " + newsmos.get(i).toMetadataString() + " cannot be applied therefore is removed from the SMO set.");
			} catch (ExpressionParsignErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (SchemaErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}	
			
		}
		
		// check other affected SMOs and remove them all, update inverses too
		// we do it backward to have the indexes valid after deletion
		for (int i = toberemoved.size()-1; i>=0; i--) {
			newsmos.removeReassignVersions(toberemoved.get(i));
			inverses.removeReassignVersions(toberemoved.get(i));
			queryres.remove(toberemoved.get(i).intValue());
		}
		
		//Added by Swami
		//When edit and save is made on FwdSMO's the corres inverses get created - however those newly created inverses
		//have to be checked for dependencies with the existing inverses
		
		Schema tempSchema = currentSchema.clone();
		for(int i = inverses.size() - 1; i >= 0 ; i--)
		{
			try 
			{
				tempSchema.applyOperator(inverses.get(i));
				
			}
			catch (IllegalSMOException ise)
			{
				System.err.println("The smo: " + inverses.get(i).toMetadataString() + " cannot be applied to the given schema.");
				ise.printStackTrace();
			
				try {
					SMOImpl nop = SMOFactory.createSMO(SMOInterface.NOP);
			
					if(inverses instanceof ICMOSMOSequence){
					nop.setSourceMajor(((ICMOSMOSequence)inverses).getMos().get(i).getSourceMajor());
					nop.setSourceMinor(((ICMOSMOSequence)inverses).getMos().get(i).getSourceMinor());
					nop.setTargetMajor(((ICMOSMOSequence)inverses).getMos().get(i).getTargetMajor());
					nop.setTargetMinor(((ICMOSMOSequence)inverses).getMos().get(i).getTargetMinor());
					((ICMOSMOSequence)inverses).getMos().set(i, nop);
					}else{
						nop.setSourceMajor(inverses.smos.get(i).getSourceMajor());
						nop.setSourceMinor(inverses.smos.get(i).getSourceMinor());
						nop.setTargetMajor(inverses.smos.get(i).getTargetMajor());
						nop.setTargetMinor(inverses.smos.get(i).getTargetMinor());
						inverses.smos.set(i, nop);	
						
					}
					
					
				} catch (SMOTypeUnknownException e) {
					e.printStackTrace();
				}
			} catch (ExpressionParsignErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (SchemaErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		//End - Swami
		
		// reassign the versions... since some SMOs might have been dropped
		reassignFwdVersions();
		reassignInverseVersions();

		// recompute the schema histories
		recomputeInverseSchemaHistory();
		recomputeForwardSchemaHistory();
		
		// everything might have been changed by the editing thus the query success must be re-computed
		computQuerySuccess(schemaName);
		
		return newsmos;
	}
	
	public SMOSequence editInverseSMO(String text, int major, int minor) {
		
		SMOImpl editedSMO = null;
		try {
			editedSMO = SMOSequenceHistoryLoader.parseSMOText(text);
		} catch (SMOParsingErrorException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		if(editedSMO == null){
			try {
				editedSMO = SMOFactory.createSMO(SMOInterface.NOP);
			} catch (SMOTypeUnknownException e) {
				e.printStackTrace();
			}
		}
		SMOImpl sub = (SMOImpl) inverses.getSMOBySource(major,minor);
		int index = inverses.getSMOIndexBySource(major, minor);
		editedSMO.setSourceMajor(major);
		editedSMO.setSourceMinor(minor);
		editedSMO.setTargetMajor(sub.getTargetMajor());
		editedSMO.setTargetMinor(sub.getTargetMinor());
		
		// substitute the inverse
		
		if(inverses instanceof ICMOSMOSequence)
			((ICMOSMOSequence)inverses).getMos().set(index, editedSMO);
		else
			inverses.smos.set(index, editedSMO);
	
		//Added by Swami
		//Need to change the loop from i = 0 to inverses.size() --> inverses.size() to 0
		//Because we need to check applySMO in the reverse order of inverse smo's to satisfy dependency check
		//for (int i = 0; i < inverses.size(); i++) {
			//Schema tempSchema = currentSchema.clone();
		Schema tempSchema = currentSchema.clone();
		for(int i = inverses.size() - 1; i >= 0 ; i--)
		{	
			//CARLO: CHANGED TO USE EXCEPTION
			try
			{
				tempSchema.applyOperator(inverses.get(i));
			}
			catch (IllegalSMOException ise)
			{
				System.err.println("The smo: " + inverses.get(i).toMetadataString() + " cannot be applied to the given schema.");
				ise.printStackTrace();
				//Added by Swami - Incase if applysmo failed on the schema then the SMO must be made NOP
				//otherwise the smo is fine and so need not be changed
				try 
				{
					if(inverses instanceof ICMOSMOSequence){
						SMOImpl nop = SMOFactory.createSMO(SMOInterface.NOP);
						nop.setSourceMajor(((ICMOSMOSequence)inverses).getMos().get(i).getSourceMajor());
						nop.setSourceMinor(((ICMOSMOSequence)inverses).getMos().get(i).getSourceMinor());
						nop.setTargetMajor(((ICMOSMOSequence)inverses).getMos().get(i).getTargetMajor());
						nop.setTargetMinor(((ICMOSMOSequence)inverses).getMos().get(i).getTargetMinor());
						((ICMOSMOSequence)inverses).getMos().set(i, nop);
					}else{
						SMOImpl nop = SMOFactory.createSMO(SMOInterface.NOP);
						nop.setSourceMajor(inverses.smos.get(i).getSourceMajor());
						nop.setSourceMinor(inverses.smos.get(i).getSourceMinor());
						nop.setTargetMajor(inverses.smos.get(i).getTargetMajor());
						nop.setTargetMinor(inverses.smos.get(i).getTargetMinor());
						inverses.smos.set(i, nop);
						
					}
					
				} 
				catch (SMOTypeUnknownException e) 
				{
					e.printStackTrace();
				}
				
			} catch (ExpressionParsignErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (SchemaErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		//Ended - Swami
		
		recomputeInverseSchemaHistory();
		
		return inverses;
	}
	
	
	

	protected void recomputeForwardSchemaHistory() {
		 currentFwdSchemaHistory = newsmos.generateSchemaHistory(startingSchema,false);	
	}

	/**
	 * return the entire history of SMOs so far.
	 * 
	 * @return
	 */
	public SMOSequenceHistory getSMOHistory() {
		return smosh;
	}


	/**
	 * return the request schema, if the index is too big it returns the most recent schema available
	 * @param index
	 * @return
	 */
	public Schema getForwardSchema(int index) {
		if(index >= currentFwdSchemaHistory.schemas.size())
			return currentFwdSchemaHistory.getMostRecent();
		if(currentFwdSchemaHistory.schemas.elementAt(index)!=null)
			return currentFwdSchemaHistory.schemas.elementAt(index);
		else
			return null;
		//if (currentSchema == null)
		//	currentSchema = sh.schemas.lastElement().clone();
		//return currentSchema;
	}
	

	public Schema getInverseSchema(int index) {
		if(index >= currentInverseSchemaHistory.schemas.size())
			return null;
		
		if(currentFwdSchemaHistory.schemas.elementAt(index)!=null) {
			return currentInverseSchemaHistory.schemas.elementAt(index);
		}
		return null;
	}
	
	public Schema getInverseSchemaByVersion(int major,int minor) {
		return currentInverseSchemaHistory.getSchemaByVersion(major,minor);
	}	
	
	public Schema getForwardSchemaByVersion(int major,int minor) {
		return currentFwdSchemaHistory.getSchemaByVersion(major,minor);
	}	
	
	
	public Schema getCurrentSchema() {
		if (currentSchema == null)
			currentSchema = startingSchema.clone();
		return currentSchema;
	}	
	

	/**
	 * Get the schema version specified from the history.
	 * 
	 * @param major
	 * @param minor
	 * @return
	 */
	public Schema getSchemaByVersion(int major, int minor) {
		return sh.getSchemaByVersion(major, minor);
	}

	/*
	 * public static Schema getSchemaByVersion(String dburl, String
	 * username,String password, String metadbname, String schemaName,int
	 * major,int minor){ System.err.println(major + ":" +minor); // MySQL DRIVER
	 * LOADING try { Class.forName("com.mysql.jdbc.Driver").newInstance(); }
	 * catch (Exception ex) { ex.printStackTrace();}
	 * 
	 * 
	 * Connection conn = DBUtil.connect(dburl,metadbname,username,password);
	 * SchemaHistoryLoader shl = new SchemaHistoryLoader(conn,schemaName);
	 * SchemaHistory sh = shl.loadSchemaHistoryFromDB(false);
	 * 
	 * return sh.getSchemaByVersion(major, minor); }
	 */

	/**
	 * read from the information schema instead of the metadb... deprecated!!
	 * 
	 * @deprecated
	 * @param dburl
	 * @param user
	 * @param password
	 * @param schemaName
	 * @return
	 */
	/*
	 * public static Schema getSchemaByName(String dburl, String user,String
	 * password, String schemaName) { // MySQL DRIVER LOADING try {
	 * Class.forName("com.mysql.jdbc.Driver").newInstance(); } catch (Exception
	 * ex) { ex.printStackTrace();}
	 * 
	 * 
	 * System.out.println(dburl + " " + user); Connection conn =
	 * DBUtil.connect(dburl,"information_schema",user,password);
	 * 
	 * String schemaListSQL ="SELECT '1' AS schema_version," + "'0' AS
	 * schema_version_minor, " + "sc.DEFAULT_CHARACTER_SET_NAME, " +
	 * "sc.DEFAULT_COLLATION_NAME," + "sc.SQL_PATH " + "FROM
	 * information_schema.SCHEMATA sc " + "WHERE sc.schema_name = '" +
	 * schemaName +"';";
	 * 
	 * ResultSet schemaList = DBUtil.executeSQLQuery(schemaListSQL,conn);
	 * 
	 * String schemaVersion = null; String schemaVersionMinor = null; Schema
	 * schema = null;
	 * 
	 * try { while(schemaList.next()){ schemaVersion =
	 * schemaList.getString("schema_version"); schemaVersionMinor =
	 * schemaList.getString("schema_version_minor");
	 * 
	 * schema = new Schema(schemaList.getInt("schema_version"),
	 * schemaList.getInt("schema_version_minor"), schemaName,
	 * schemaList.getString("DEFAULT_CHARACTER_SET_NAME"),
	 * schemaList.getString("DEFAULT_COLLATION_NAME"),
	 * schemaList.getString("SQL_PATH"), "", "", new Timestamp(1970, 01, 01, 0,
	 * 0, 0, 0) ); schema.setSchemaName(schemaName); String tableListSQL =
	 * "SELECT table_name " + "FROM information_schema.TABLES t " + "WHERE
	 * t.table_schema = '" + schemaName + "';";
	 * 
	 * ResultSet tableList = DBUtil.executeSQLQuery(tableListSQL,conn);
	 * 
	 * String tableName = null; String columnQuery = null;
	 * 
	 * while(tableList.next()){
	 * 
	 * tableName = tableList.getString("table_name"); columnQuery = "SELECT
	 * c.column_name, c.COLUMN_TYPE " + "FROM information_schema.COLUMNS c " +
	 * "WHERE c.table_schema = '" + schemaName + "' AND "+ "c.table_name = '" +
	 * tableName + "';";
	 * 
	 * ResultSet columnList = DBUtil.executeSQLQuery(columnQuery,conn); Table tt =
	 * new Table(tableName); String columnName = null; String columnType = null;
	 * while(columnList.next()){ columnName =
	 * columnList.getString("column_name"); columnType =
	 * columnList.getString("COLUMN_TYPE"); tt.addColumn(columnName,columnType); }
	 * schema.addTable(tt); }
	 * 
	 *  }
	 *  } catch (SQLException e) { e.printStackTrace(); }
	 * 
	 * return schema; }
	 */

	/*
	 * public static SMOSequenceHistory getSMO(String dburl, String user, String
	 * password, String metaSchemaName, String schemaName) {
	 *  // MySQL DRIVER LOADING try {
	 * Class.forName("com.mysql.jdbc.Driver").newInstance(); } catch (Exception
	 * ex) { ex.printStackTrace();}
	 * 
	 * 
	 * System.out.println(dburl + " " + user); Connection conn =
	 * DBUtil.connect(dburl,metaSchemaName, user,password);
	 * SMOSequenceHistoryLoader sshl = new
	 * SMOSequenceHistoryLoader(conn,schemaName); return
	 * sshl.loadSMOSequenceHistoryFromDB(false); }
	 */
	/*
	 * public static String testSMOSyntax(String dburltb, String usertb, String
	 * passwordtb, String metaSchemaName, String schemaName, String smotext, int
	 * majorTargetVersion, int minorTargetVersion) { boolean applied = true;
	 * Schema s = getSchemaByVersion(dburltb,usertb,passwordtb,metaSchemaName,
	 * schemaName, majorTargetVersion, minorTargetVersion); if(s == null)
	 * System.err.println("SCHEMA IS NULL"); try{
	 * applied=s.applySMO(SMOSequenceHistoryLoader.parseSMOText(smotext));
	 * }catch(Exception i){ i.printStackTrace(); return "<b>syntax error:</b> "
	 * +smotext + " is not a valid SMO."; } if(applied) return "syntax ok"; else
	 * return "<b>syntax error:</b> schema doesn't match!"; }
	 * 
	 */
	/**
	 * Verifies SMO syntax and applicability to the schema. Return if ok or not.
	 */
	public String testSMOSyntax(String smotext, int schemaIndex,boolean isInverse) {
		
		if(schemaIndex >= currentFwdSchemaHistory.schemas.size()){
			return "SYS ERROR: accessing an unexisting schema: " + schemaIndex;
		}
		Schema s = null;
		if(isInverse)
			s = currentInverseSchemaHistory.schemas.elementAt(schemaIndex);
		else
			 s = currentFwdSchemaHistory.schemas.elementAt(schemaIndex);
		
		MOImpl smo = null;
		if (s == null)
			System.err.println("SCHEMA IS NULL");
		Schema s2 = s.clone();
		try {
			//smo = SMOSequenceHistoryLoader.parseSMOText(smotext);
			smo =  MOFactory.createMOFromString(smotext);
			//System.err.println("I parsed: " + smo.toString());
			
			s2.applyOperator(smo);
			return "syntax ok";
					
			
		} catch (IllegalSMOException i) {
			i.printStackTrace();
			 
			
			return "<b>Error:</b>" + i.getMessage();
			
			//FIXME improve this with subexception 
			//Added by Swami to handle same column names in create table
		/*	if (smo instanceof CreateTableSMO)
			{
				if(i instanceof TableExistsException)
					return "<b>Schema Mismatch Error:</b> the table you are trying to insert already exists!";
				if(i instanceof ColumnExistsException)
					return "<b>Schema Mismatch Error:</b> The Column names must be different!";
				if(i instanceof TableHasNoColumns)
					return "<b>Schema Mismatch Error:</b>" + i.toString();
			}
			
			if(i instanceof TableConstrainedException)
				return "<b>Schema is Constrained Error:</b>:" + i.toString();
			
			if (smo instanceof AddColumnSMO)
				return "<b>Schema Mismatch Error:</b> the column you are trying to insert already exists!";
			return "<b>Schema Mismatch Error:</b> you refer to a not existing table/column or you are trying to create an already existing table/column!";
		*/
			
		} catch (ExpressionParsignErrorException e) {
			return "<b>Error:</b>" + e.getMessage();
		} catch (SchemaErrorException e) {
			return "<b>Error:</b>" + e.getMessage();
		} catch (SMOParsingErrorException e) {
			return "<b>Error:</b>" + e.getMessage();
		} 
		
		}

	/**
	 * Verifies SMO syntax and applicability to the schema. Return if ok or not.
	 * @throws SchemaErrorException 
	 */
	public String testSMOSyntax(String smotext, int major, int minor,boolean isInverse) throws SchemaErrorException {
		
		Schema s = currentInverseSchemaHistory.getSchemaByVersion(major, minor);
		
		boolean applied = false;
		MOImpl smo = null;
		if (s == null){
			System.err.println("BackendUtil.testSMOSyntax(): SCHEMA IS NULL");
			throw new SchemaErrorException("The input schema is null");
		}	
		Schema s2 = s.clone();

		try {
		
			//	smo = SMOSequenceHistoryLoader.parseSMOText(smotext);
			smo = MOFactory.createMOFromString(smotext);
			
			applied = s2.applyOperator(smo);
		} catch (IllegalSMOException e) {
			e.printStackTrace();
			return "<b>Error:</b>" + e.getMessage();

		} catch (SMOParsingErrorException e) {
			return "<b>Syntax error:</b>" + e.getMessage();
		}
	
		if (applied)
			return "syntax ok";
		// the subsequent code should not be used...
		if (smo instanceof CreateTableSMO)
			return "<b>Schema Mismatch Error:</b> the table you are trying to insert already exists!";
		if (smo instanceof AddColumnSMO)
			return "<b>Schema Mismatch Error:</b> the column you are trying to insert already exists!";
		if (smo instanceof JoinTableSMO )
			return "<b>Schema Mismatch Error:</b> you might have two columns with same name, which are not part of the conditions," +
					" you might refer to not existing table/column or you try to create an already existing table/column";
				
			
		return "<b>Schema Mismatch Error:</b> you refer to a not existing table/column or you are trying to create an already existing table/column!";
	}
	
	
	/*
	 * public static Schema applySMO(String dburltb, String usertb, String
	 * passwordtb, String metaSchemaName, String schemaName, String smotext, int
	 * majorTargetVersion, int minorTargetVersion) { boolean applied = true;
	 * Schema s = getSchemaByVersion(dburltb,usertb,passwordtb,metaSchemaName,
	 * schemaName, majorTargetVersion, minorTargetVersion); Schema s2 = null;
	 * if(s == null) System.err.println("SCHEMA IS NULL"); try{ s2 = s.clone();
	 * applied=s2.applySMO(SMOSequenceHistoryLoader.parseSMOText(smotext));
	 * }catch(Exception i){ i.printStackTrace(); return null; } if(applied)
	 * return s2;
	 * 
	 * return null; }
	 */

	/**
	 * Verifies whether the given SMO is information preserving or not.
	 * 
	 * @param smotext
	 * @return
	 */
	public String informationPreservation(String smotext) {
		MOImpl smo = null;
		try {
			//smo = SMOSequenceHistoryLoader.parseSMOText(smotext);
			smo = MOFactory.createMOFromString(smotext);
		
		} catch (SMOParsingErrorException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		try
		{
			//added by swami to avoid pessimistic not ok always when no smo is written
			if(smo == null)
				return "information preservation check: checking...";
			//changed by swami - for Inf Pres
			//CARLO add exception to handle NoTableInSchema case
			try {
				if (smo.isInfPreserving(getCurrentSchema()))
					return "information preservation check: OK";
			} catch (NoTableInSchemaException e) {
				e.printStackTrace();
				return "ATTENTION: the input table is not in the schema";							
			}
	
			return "ATTENTION: <b>" + smo.getOperatorType()
					+ "</b> is not information preserving.<br> See <a href=\"http://yellowstone.cs.ucla.edu/schema-evolution/index.php/SMO\">details</a>.";
		}
		catch(noColumnsMatchInfPres nCMIF)
		{
			//return "ATTENTION: <b>" + smo.getOperatorType()
			//+ "</b> is not information preserving.<br> See <a href=\"http://yellowstone.cs.ucla.edu/schema-evolution/index.php/SMO\">SMO description</a> for details.";
			return "ATTENTION: <b>" + smo.getOperatorType()+ "</b> is not information preserving.<br>" + nCMIF.toString();
		}
		catch(noCommonColumnInfPres nCCIF)
		{
			return "ATTENTION: <b>" + smo.getOperatorType()+ "</b> is not information preserving.<br>" + nCCIF.toString();
		}

	}
	

	public Schema applyMO(String smotext) {
		
		
		Schema s = getCurrentSchema();
		boolean applied = false;

		if (s == null) {
			System.err.println("SCHEMA IS NULL");
			return null;
		}
		Schema s2 = s.clone();
		try {
			//SMOImpl mysmo = SMOSequenceHistoryLoader.parseSMOText(smotext);
			MOImpl mysmo = MOFactory.createMOFromString(smotext);
			applied = s2.applyOperator(mysmo);
		} catch (IllegalSMOException e) {
			e.printStackTrace();
		} catch (SMOParsingErrorException e) {
			e.printStackTrace();
		} catch (ExpressionParsignErrorException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SchemaErrorException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	
		if (applied)
			return s2;

		return null;
	}

	public String checkRedundancy(String smotext) {
		MOImpl smo = null;
		try {
			//smo = SMOSequenceHistoryLoader.parseSMOText(smotext);
			smo = MOFactory.createMOFromString(smotext);
		} catch (SMOParsingErrorException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
		if(smo == null)
			//return "redundancy check: smo syntax is not ok...";
			//added by swami to avoid pessimistic not ok always when no smo is written
			return "redundancy check: checking...";
		if (!smo.isRedundant())
			return "redundancy check: OK";

		return "ATTENTION: <b>" + smo.getOperatorType()
				+ "</b> might generate redundancy.<br> See <a href=\""
				+ smo.getOperatorType() + ".html\">details</a> .";

	}

	public SMOSequence getInverseSMOs() {
		if (inverses == null)
			if(newsmos instanceof ICMOSMOSequence)
				inverses = (ICMOSMOSequence) newsmos.getCandidateInverseSequence(startingSchema);
			else
				inverses = newsmos.getCandidateInverseSequence(startingSchema);
				
		// we past the schema before applying the sequence
		return inverses;
	}

	/**
	 * Compute the query success for a give target.
	 * @return
	 */
	public boolean computQuerySuccess(String schemaName) {
		
		int unchangedHistoryIndex = 0;
		
		if (inverses == null)
			getInverseSMOs();
		
		if(!inverses.isComplete()){
			return false;
		}
		
		if(hideComputation){
		// CARLO: check if the smos fwd and bacward did not change since last invocation, return immediately true avoiding computation
		if(cachedNewsmos != null && cachedInverses != null){	
			if(cachedNewsmos instanceof ICMOSMOSequence){
				
				if(((ICMOSMOSequence)cachedNewsmos).getMos().equals(((ICMOSMOSequence)newsmos).getMos()) && ((ICMOSMOSequence)cachedNewsmos).getMos().equals(((ICMOSMOSequence)inverses).getMos())){
					System.out.println("Caching previously computed queries results");
					return true;
				}
			}else{
				
				if(cachedNewsmos.smos.equals(newsmos.smos) && cachedNewsmos.smos.equals(inverses.smos)){
					System.out.println("Caching previously computed queries results");
					return true;
				}	
				
			}
			
			int unchangedIndex = cachedNewsmos.getCommonHistoryIndex(newsmos,false);			
			int unchangedIndex2 = cachedInverses.getCommonHistoryIndex(inverses,true);
			if(unchangedIndex < unchangedIndex2)
				unchangedHistoryIndex = unchangedIndex;
			else
				unchangedHistoryIndex = unchangedIndex2;
			}
		}
		
		// CREATE a clone of inverses and newsmos to later check if they changed
		if(newsmos instanceof ICMOSMOSequence){
			cachedNewsmos = new ICMOSMOSequence(newsmos);
			cachedInverses = new ICMOSMOSequence(inverses);			
		}else{
			cachedNewsmos = new SMOSequence(newsmos);
			cachedInverses = new SMOSequence(inverses);			
		}
		
		System.out.println("No caching is possible. Proceeding to compute query results");
		

		

		//Schema temp = sh.schemas.lastElement();

/*		
		SchemaHistory schemaHistory = inverses.generateSchemaHistory(getCurrentSchema().clone(),true);
		Schema querySchema = schemaHistory.getOldestMajorVersion();
		
		DEDSetHistory deds = inverses.getAllDED(getCurrentSchema().clone(),true);
		schemaHistory.reOrder();
		PrimRewriteEngine pre = new PrimRewriteEngine(schemaHistory, deds);
		
	*/	

		//for each intermediate schema in the changed part compute query rewriting 
		for(int j = unchangedHistoryIndex; j< queryres.results.size(); j++){
			
			// the following lines can be optimized avoiding all this recomputation at every query.
			// i leave the optimization for a later stage of the development
			Schema tempcurr = null;
			// generate the schema on which we start applying the inverse sequence
			if(startingSchema instanceof ICSchema)
				 tempcurr = applyPartialSequence((ICSchema) startingSchema, newsmos, 0,j);
			else
				 tempcurr = applyPartialSequence(startingSchema, newsmos, 0,j);
					
			//apply the inverses generating the inverse schema history
			SchemaHistory schemaHistory = inverses.generateSchemaHistory(tempcurr.clone(),true);
			
			//get the target schema (the one on which queries are issued)
			Schema querySchema = schemaHistory.getOldestMajorVersion();
			
			
			// checking 
			if(querySchema == null){
				System.err.println("BackendUtil.computQuerySuccess(): the function schemaHistory.getOldestMajorVersion(); returned a null schema");
			}
			
			//generate the DEDS
			DEDSetHistory deds = inverses.getAllDED(tempcurr.clone(),true);

			//to make sure reOrder schemaHistory
			schemaHistory.reOrder();
			
			//instantiate a rewriter
			PrimRewriteEngine pre = new PrimRewriteEngine(schemaHistory, deds);
			
			//for each query in queryres compute the rewriting
			QueryRewriteResult qrr = queryres.results.elementAt(j);
			
			for(int i = 0; i< qrr.size(); i++){
				
				System.out.println("REWRITING: " + qrr.inputqueries[i] + " FROM " + querySchema.versionID +"."+ querySchema.minorVersionID  + " TO " + tempcurr.versionID +"." +tempcurr.minorVersionID );
				
				SQL inputSQL = new SQL(qrr.inputqueries[i]);
				SQLUnion rew = null;
				try{
					// tempcurr is the temporary current schema to be considered to test the queries after each fwd SMO
					rew = pre.rewrite(inputSQL,tempcurr.versionID,tempcurr.minorVersionID,querySchema.versionID,querySchema.minorVersionID);
				}catch(Exception e){
					e.printStackTrace();
					qrr.rewrittenqueries[i] = null;
					qrr.results[i]=0;
				}
				// if there is rewriting we just list the failure 
				if(rew==null){
					qrr.rewrittenqueries[i] = null;
					qrr.results[i]=0;
				}
				else{
					//otherwise we store the rewritten query...
					//System.out.println("rewritten query: " + rew.toString());

					if(!rew.sqls.isEmpty()){
						//another check of not failure
						
						//qrr.rewrittenqueries[i] = rew.sqls.firstElement().toString();
						qrr.rewrittenqueries[i] = rew.toViewString("");
						qrr.results[i]=1;
					}	
					else{
						qrr.rewrittenqueries[i] = null;
						qrr.results[i]=0;
					}	
				}
			}
			qrr.computed = true;
		}	
		return true;
	}

	public QueryRewriteResultSet getQueryResults(String schemaName) {
		computQuerySuccess(schemaName);
		return queryres;
	}
	
	public Schema applyPartialSequence(Schema startingSchema2, SMOSequenceInterface tempsmos, int start, int end){
		Schema ret = null;
		if(startingSchema2 instanceof ICSchema)
			 ret = ((ICSchema)startingSchema2).clone();
		else	
			 ret = startingSchema2.clone();

			for (int i = start; i <= end; i++) {
				try {
					ret.applyOperator(tempsmos.get(i));
				} catch (IllegalSMOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (ExpressionParsignErrorException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (SchemaErrorException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			return ret;
		
		
	}

	/**
	 * given an input query rewrite from the oldest schema to the currentSchema.
	 * @param query
	 * @return
	 */
	public SingleQueryRewriteResult rewriteQuery(String query,String dbName) {
		
		System.out.println("REWRITE");
		SchemaHistory schemaHistory = inverses.generateSchemaHistory(getCurrentSchema().clone(),true);
		Schema querySchema = schemaHistory.getOldestMajorVersion();
		//generate the DEDS
		DEDSetHistory deds = inverses.getAllDED(getCurrentSchema().clone(),true);
		//to make sure reOrder schemaHistory
		schemaHistory.reOrder();
		
		//instantiate a rewriter
		PrimRewriteEngine pre = new PrimRewriteEngine(schemaHistory, deds);
		
		// FIXME plug here the query decomposition in positive and negative parts
		
		SQL negative = SQL.extractNegative(query);
		SQL positive = SQL.extractExternalPositive(query);
		SQL inputSQL = null;
	
		
//		SQL inputSQL = new SQL(query);
		
		 
		
		SQLUnion rew = null;
		String output = "";
		long ts,te;
		try{
			System.out.println("REWRITING SINGLE QUERY: " + query + " FROM " + querySchema.versionID +"."+ querySchema.minorVersionID  + " TO " + getCurrentSchema().versionID +"." +getCurrentSchema().minorVersionID );
			ts = System.currentTimeMillis();
			
			if(negative==null){
				rew= pre.rewrite(positive,getCurrentSchema().versionID,getCurrentSchema().minorVersionID,querySchema.versionID,querySchema.minorVersionID);
				output = rew.toViewString(dbName);
			}
			else{
				SQLUnion rew1 = pre.rewrite(positive,getCurrentSchema().versionID,getCurrentSchema().minorVersionID,querySchema.versionID,querySchema.minorVersionID);
				SQLUnion rew2 = pre.rewrite(negative,getCurrentSchema().versionID,getCurrentSchema().minorVersionID,querySchema.versionID,querySchema.minorVersionID);			
				output = rew1.toViewString(dbName) + " AND NOT EXISTS (\n\t " + rew2.toViewString(dbName) + "\n\t)";
			}
				
			//rew= pre.rewrite(inputSQL,getCurrentSchema().versionID,getCurrentSchema().minorVersionID,querySchema.versionID,querySchema.minorVersionID);
			
			te = System.currentTimeMillis();
		}catch(Exception e){
			return new SingleQueryRewriteResult("","ERROR:" + e.getMessage(),"");
		}
		if(output.equals(""))
			return new SingleQueryRewriteResult("","ERROR: no rewriting available",(te-ts) + "ms");
		return new SingleQueryRewriteResult(output,"",(te-ts) + "ms"); 	
	}
	
	public String getDEDs(boolean inverse) {
		
		if(hideComputation){
			// if the SMOS did not changed we reuse the precomputed DEDs (if any)
			if(cachedBkwdDeds!=null && cachedFwdDeds!=null){
				if(cachedNewsmos != null && cachedInverses != null){	
					
					if(cachedNewsmos instanceof ICMOSMOSequence){
						if(((ICMOSMOSequence)cachedNewsmos).getMos().equals(((ICMOSMOSequence)newsmos).getMos()) && ((ICMOSMOSequence)cachedNewsmos).getMos().equals(((ICMOSMOSequence)inverses).getMos())){
							//System.out.println("Caching previously computed Deds");
							if(inverse)
								return cachedBkwdDeds;
							else
								return cachedFwdDeds;
						}
					}else{
						if(cachedNewsmos.smos.equals(newsmos.smos) && cachedInverses.smos.equals(inverses.smos)){
							//System.out.println("Caching previously computed Deds");
							if(inverse)
								return cachedBkwdDeds;
							else
								return cachedFwdDeds;
						}
					}
				}
			}
		}
		
		if(inverse){
			DEDSetHistory d = inverses.getAllDED(currentSchema, true);
			cachedBkwdDeds = CodiGen.writeHTMLCodiPrelude(currentInverseSchemaHistory,d) + d.toHTMLString();
			return cachedBkwdDeds;
		}
		else{
			
			DEDSetHistory d = newsmos.getAllDED(startingSchema, false);
			cachedFwdDeds = CodiGen.writeHTMLCodiPrelude(currentFwdSchemaHistory,d) + d.toHTMLString();
			return cachedFwdDeds;
		}
				
	}

	public SMOSequence addMultipleNewSMO(String[] smotext) {
		
		for(int i =0; i<smotext.length;i++){
			addNewSMO(smotext[i]);
		}	
		
		return newsmos;
	}


	
}
