package edu.ucla.cs.db.prism.backend;

import java.sql.Timestamp;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.SQL.SQL;
import edu.ucla.cs.db.prism.backend.SQL.SQLUnion;
import edu.ucla.cs.db.prism.backend.ded.DEDSetHistory;
import edu.ucla.cs.db.prism.backend.exception.ErrorRewritingException;
import edu.ucla.cs.db.prism.backend.exception.SchemaErrorException;
import edu.ucla.cs.db.prism.backend.icmo.DropForeignKeyException;
import edu.ucla.cs.db.prism.backend.icmo.ICMOImpl;
import edu.ucla.cs.db.prism.backend.icmo.ICMOSMOSequence;
import edu.ucla.cs.db.prism.backend.icmo.MOFactory;
import edu.ucla.cs.db.prism.backend.icmo.MOImpl;
import edu.ucla.cs.db.prism.backend.icmo.exceptions.AddForeignKeyException;
import edu.ucla.cs.db.prism.backend.icmo.exceptions.AddPrimaryKeyException;
import edu.ucla.cs.db.prism.backend.marsManage.ICCodiGen;
import edu.ucla.cs.db.prism.backend.rewrite.ICPrimRewriteEngine;
import edu.ucla.cs.db.prism.backend.rewrite.QueryRewriteResult;
import edu.ucla.cs.db.prism.backend.rewrite.QueryRewriteResultSet;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.exception.IllegalSMOException;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchemaHistory;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchemaHistoryLoader;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.exceptions.ExpressionParsignErrorException;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.exceptions.SchemaMismatchException;
import edu.ucla.cs.db.prism.backend.smo.SMOImpl;
import edu.ucla.cs.db.prism.backend.smo.SMOInterface;
import edu.ucla.cs.db.prism.backend.smo.exception.SMOParsingErrorException;
import edu.ucla.cs.db.prism.backend.smo.util.SMOFactory;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequence;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequenceHistoryLoader;
import edu.ucla.cs.db.prism.backend.smo.util.SMOTypeUnknownException;
import edu.ucla.cs.db.prism.backend.util.db.DBUtil;
import edu.ucla.cs.db.prism.gui.client.SingleQueryRewriteResult;

public class ICBackendUtil extends BackendUtil {

	public ICBackendUtil(String dburltb, String usertb, String passwordtb,
			String schemaName, String version, String metaSchemaName, String userIp, boolean loadHistory) throws IllegalSMOException {
		//logging some information about the users
		userip = userIp;
		timestamp = new Timestamp(System.currentTimeMillis());
		
		// MySQL DRIVER LOADING
		try {
			Class.forName("com.mysql.jdbc.Driver").newInstance();
		} catch (Exception ex) {
			ex.printStackTrace();
		}

		this.dburltb = dburltb;
		this.usertb = usertb;
		this.passwordtb = passwordtb;
		this.schemaName = schemaName;
		this.metaSchemaName = metaSchemaName;
		
		
		conn = DBUtil.connect(dburltb, metaSchemaName, usertb, passwordtb);
		
		
		sh = (new ICSchemaHistoryLoader(conn, schemaName)).loadSchemaHistoryFromDB(true,loadHistory,version);
		
	
		if(version.equals("first"))
			this.userStartingVersion = ((ICSchemaHistory)sh).getOldestMajorVersion().versionID;
		else 
			if(version.equals("last"))
				this.userStartingVersion = ((ICSchemaHistory)sh).getMostRecent().versionID;
			else
				this.userStartingVersion = Integer.parseInt(version);
		
	
		startingSchema = ((ICSchemaHistory)sh).getSchemaByVersion(userStartingVersion, 0);
		currentSchema = startingSchema;
		currentFwdSchemaHistory = new ICSchemaHistory();
		((ICSchemaHistory)currentFwdSchemaHistory).addSchema((ICSchema)startingSchema);
		currentInverseSchemaHistory = new ICSchemaHistory();
		
		// THE FOLLOWING MIGHT BE MOVED IN ANOTHER METHOD TO MAKE LOADING FASTER!!
		smosh = (new SMOSequenceHistoryLoader(conn, schemaName))
				.loadSMOSequenceHistoryFromDB(false);
		newsmos = new ICMOSMOSequence();
		queryLoader = new QueryLoader(conn, schemaName, metaSchemaName);
		inputqueries = queryLoader.loadQueryTemplatesFromDB(userStartingVersion);
		queryres = new QueryRewriteResultSet(inputqueries);
	}

	
	@Override
	protected void recomputeInverseSchemaHistory() {
		 currentInverseSchemaHistory = ((ICMOSMOSequence)inverses).generateSchemaHistory(((ICSchema)currentSchema).clone(),true);	
	}
	
	@Override
	public SMOSequence getInverseSMOs() {
		if (inverses == null)
			inverses = ((ICMOSMOSequence)newsmos).getICCandidateInverseSequence((ICSchema) startingSchema);

		// we past the schema before applying the sequence
		return inverses;
	}
	
	
	/**
	 * return the request schema, if the index is too big it returns the most recent schema available
	 * @param index
	 * @return
	 */
	@Override
	public Schema getForwardSchema(int index) {
		if(index >= ((ICSchemaHistory)currentFwdSchemaHistory).schemas.size())
			return ((ICSchemaHistory)currentFwdSchemaHistory).getMostRecent();
		if(((ICSchemaHistory)currentFwdSchemaHistory).schemas.elementAt(index)!=null)
			return ((ICSchemaHistory)currentFwdSchemaHistory).schemas.elementAt(index);
		return null;
	}
	
	
	@Override
	public ICSchema applyMO(String smotext) {
			
			
			ICSchema s = getCurrentSchema();
			boolean applied = false;
	
			if (s == null) {
				System.err.println("SCHEMA IS NULL");
				return null;
			}
			ICSchema s2 = s.clone();
			try {
				//SMOImpl mysmo = SMOSequenceHistoryLoader.parseSMOText(smotext);
				MOImpl mysmo = MOFactory.createMOFromString(smotext);
				applied = s2.applyOperator(mysmo);
			} catch (IllegalSMOException e) {
				e.printStackTrace();
			} catch (SMOParsingErrorException e) {
				e.printStackTrace();
			} catch (ExpressionParsignErrorException e) {
				e.printStackTrace();
			} catch (SchemaErrorException e) {
				e.printStackTrace();
			}
		
			if (applied)
				return s2;
	
			return null;
		}

	
	/**
	 * given an input query rewrite from the oldest schema to the currentSchema.
	 * @param query
	 * @return
	 */
	@Override
	public SingleQueryRewriteResult rewriteQuery(String query,String dbName) {
		
		//System.out.println("TRYING TO REWRITE WITH THE FOLLOWING SMO:" +((ICMOSMOSequence)inverses).toString());
		
		
		ICSchemaHistory schemaHistory = ((ICMOSMOSequence)inverses).generateICSchemaHistory(getCurrentSchema().clone(),true);
		
		ICSchema querySchema = schemaHistory.getOldestMajorVersion();
		//generate the DEDS
		DEDSetHistory deds = ((ICMOSMOSequence)inverses).getAllDED(getCurrentSchema().clone(),true);
		//to make sure reOrder schemaHistory
		schemaHistory.reOrder();
		
		//instantiate a rewriter
		ICPrimRewriteEngine pre;
		try {
			pre = new ICPrimRewriteEngine(schemaHistory, deds);
		} catch (ErrorRewritingException e1) {
			e1.printStackTrace();
			return new SingleQueryRewriteResult("","ERROR: internal error generating rewriting engine.", "-1ms");
		}
		
		
		
		
		/*  using new method in primrewriteengine for query with negation
		SQL negative = SQL.extractNegative(query);
		SQL positive = SQL.extractExternalPositive(query);
		SQL inputSQL = null;


		SQLUnion rew = null;
		String output = "";
		long ts;
		long te;
		try{
			
			ts = System.currentTimeMillis();
			if(negative==null){
				rew= pre.rewrite(positive,getCurrentSchema().versionID,getCurrentSchema().minorVersionID,querySchema.versionID,querySchema.minorVersionID);
				output = rew.toViewString(dbName);
			}
			else{
				SQLUnion rew1 = pre.rewrite(positive,getCurrentSchema().versionID,getCurrentSchema().minorVersionID,querySchema.versionID,querySchema.minorVersionID);
				SQLUnion rew2 = pre.rewrite(negative,getCurrentSchema().versionID,getCurrentSchema().minorVersionID,querySchema.versionID,querySchema.minorVersionID);			
				output = rew1.toViewString(dbName) + " AND NOT EXISTS (\n\t " + rew2.toViewString(dbName) + "\n\t)";
			}
				
			
			//rew= pre.rewrite(inputSQL,getCurrentSchema().versionID,getCurrentSchema().minorVersionID,querySchema.versionID,querySchema.minorVersionID);
			te = System.currentTimeMillis();
		}catch(Exception e){
			return new SingleQueryRewriteResult("","ERROR:" + e.getMessage(),"");
		}
		if(output.equals(""))
			return new SingleQueryRewriteResult("","ERROR: no rewriting available",(te-ts) + "ms");

		
		return new SingleQueryRewriteResult(output,"",(te-ts) + "ms"); 	
		*/

		
		// REWRITING WITH THE NEGATION. ACCEPT QUERY WITH NOT EXISTS STATEMENTS
		String output = "";
		long ts;
		long te;
		try{
			
			ts = System.currentTimeMillis();
			output= pre.rewriteWithNegationAndICMO(query,getCurrentSchema().versionID,getCurrentSchema().minorVersionID,querySchema.versionID,querySchema.minorVersionID);
			te = System.currentTimeMillis();
		}catch(Exception e){
			e.printStackTrace();
			return new SingleQueryRewriteResult("","ERROR:" + e.getMessage(),"");
		}
		if(output.equals(""))
			return new SingleQueryRewriteResult("","ERROR: no rewriting available",(te-ts) + "ms");

		return new SingleQueryRewriteResult(output,"",(te-ts) + "ms"); 	
		
	}
	
	@Override
	public ICSchema getCurrentSchema() {
		if (currentSchema == null)
			currentSchema = ((ICSchema)startingSchema).clone();
		return (ICSchema) currentSchema;
	}	
	
	
	@Override
	public String getDEDs(boolean inverse) {
		
		/*if(hideComputation){
			// if the SMOS did not changed we reuse the precomputed DEDs (if any)
			if(cachedBkwdDeds!=null && cachedFwdDeds!=null){
				if(cachedNewsmos != null && cachedInverses != null){	
					if(cachedNewsmos.getMos().equals(newsmos.smos) && cachedInverses.getMos().equals(inverses.smos)){
						//System.out.println("Caching previously computed Deds");
						if(inverse)
							return cachedBkwdDeds;
						else
							return cachedFwdDeds;
					}
				}
			}
		}*/
		
		if(inverse){
			DEDSetHistory d = ((ICMOSMOSequence)inverses).getAllDED((ICSchema)currentSchema, true);
			cachedBkwdDeds = ICCodiGen.writeHTMLCodiPrelude((ICSchemaHistory)currentInverseSchemaHistory,d) + d.toHTMLString();
			return cachedBkwdDeds;
		}
		else{
			DEDSetHistory d = ((ICMOSMOSequence)newsmos).getAllDED((ICSchema)startingSchema, false);
			cachedFwdDeds = ICCodiGen.writeHTMLCodiPrelude((ICSchemaHistory)currentFwdSchemaHistory,d) + d.toHTMLString();
			return cachedFwdDeds;
		}
				
	}
	
	
	
	/**
	 * Compute the query success for a give target.
	 * @return
	 */
	@Override
	public boolean computQuerySuccess(String schemaName) {
		
		int unchangedHistoryIndex = 0;
		
		if (inverses == null)
			getInverseSMOs();
		
		if(!inverses.isComplete()){
			return false;
		}
		
		if(hideComputation){
		// CARLO: check if the smos fwd and bacward did not change since last invocation, return immediately true avoiding computation
		if(cachedNewsmos != null && cachedInverses != null){	
			if(((ICMOSMOSequence)cachedNewsmos).getMos().equals(((ICMOSMOSequence)newsmos).getMos()) && ((ICMOSMOSequence)cachedInverses).getMos().equals(((ICMOSMOSequence)inverses).getMos())){
				System.out.println("Caching previously computed queries results");
				return true;
			}
		
			
			int unchangedIndex = ((ICMOSMOSequence)cachedNewsmos).getCommonHistoryIndex(newsmos,false);			
			int unchangedIndex2 = ((ICMOSMOSequence)cachedInverses).getCommonHistoryIndex(inverses,true);
			if(unchangedIndex < unchangedIndex2)
				unchangedHistoryIndex = unchangedIndex;
			else
				unchangedHistoryIndex = unchangedIndex2;
			}
		}
		
		// CREATE a clone of inverses and newsmos to later check if they changed
		cachedNewsmos = new ICMOSMOSequence(newsmos);
		cachedInverses = new ICMOSMOSequence(inverses);			
	
		
		System.out.println("No caching is possible. Proceeding to compute query results");
		

		

		//Schema temp = sh.schemas.lastElement();

/*		
		SchemaHistory schemaHistory = inverses.generateSchemaHistory(getCurrentSchema().clone(),true);
		Schema querySchema = schemaHistory.getOldestMajorVersion();
		
		DEDSetHistory deds = inverses.getAllDED(getCurrentSchema().clone(),true);
		schemaHistory.reOrder();
		PrimRewriteEngine pre = new PrimRewriteEngine(schemaHistory, deds);
		
	*/	

		//for each intermediate schema in the changed part compute query rewriting 
		for(int j = unchangedHistoryIndex; j< queryres.results.size(); j++){
			
			// the following lines can be optimized avoiding all this recomputation at every query.
			// i leave the optimization for a later stage of the development
			
			// generate the schema on which we start applying the inverse sequence
			ICSchema tempcurr = (ICSchema) applyPartialSequence(startingSchema, newsmos, 0,j);
			
			//apply the inverses generating the inverse schema history
			ICSchemaHistory schemaHistory = ((ICMOSMOSequence)inverses).generateICSchemaHistory(tempcurr.clone(),true);
			
			//get the target schema (the one on which queries are issued)
			ICSchema querySchema = (schemaHistory).getOldestMajorVersion();
			
			
			// checking 
			if(querySchema == null){
				try {
					throw new ErrorRewritingException("BackendUtil.computQuerySuccess(): the function schemaHistory.getOldestMajorVersion(); returned a null schema");
				} catch (ErrorRewritingException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			DEDSetHistory deds = null;
			//generate the DEDS
			if(inverses instanceof ICMOSMOSequence)
				 deds = ((ICMOSMOSequence)inverses).getAllDED(tempcurr.clone(),true);
			else
				 deds = inverses.getAllDED(tempcurr.clone(),true);

			//to make sure reOrder schemaHistory
			schemaHistory.reOrder();
			
			//instantiate a rewriter
			ICPrimRewriteEngine pre = null;
			try {
				pre = new ICPrimRewriteEngine(schemaHistory, deds);
			} catch (ErrorRewritingException e1) {
				e1.printStackTrace();
				return false;
			}
			
			//for each query in queryres compute the rewriting
			QueryRewriteResult qrr = queryres.results.elementAt(j);
			
			for(int i = 0; i< qrr.size(); i++){
				
				System.out.println("REWRITING: " + qrr.inputqueries[i] + " FROM " + querySchema.versionID +"."+ querySchema.minorVersionID  + " TO " + tempcurr.versionID +"."+tempcurr.minorVersionID);
				
				SQL inputSQL = new SQL(qrr.inputqueries[i]);
				SQLUnion rew = null;
				try{
					// tempcurr is the temporary current schema to be considered to test the queries after each fwd SMO
					rew= pre.rewrite(inputSQL,tempcurr.versionID,tempcurr.minorVersionID,querySchema.versionID,querySchema.minorVersionID);
				}catch(Exception e){
					e.printStackTrace();
					qrr.rewrittenqueries[i] = null;
					qrr.results[i]=0;
				}
				// if there is rewriting we just list the failure 
				if(rew==null){
					qrr.rewrittenqueries[i] = null;
					qrr.results[i]=0;
				}
				else{
					//otherwise we store the rewritten query...
					//System.out.println("rewritten query: " + rew.toString());

					if(!rew.sqls.isEmpty()){
						//another check of not failure
						
						//qrr.rewrittenqueries[i] = rew.sqls.firstElement().toString();
						qrr.rewrittenqueries[i] = rew.toViewString("");
						qrr.results[i]=1;
					}	
					else{
						qrr.rewrittenqueries[i] = null;
						qrr.results[i]=0;
					}	
				}
			}
			qrr.computed = true;
		}	
		return true;
	}

	@Override
	public QueryRewriteResultSet getQueryResults(String schemaName) {
		computQuerySuccess(schemaName);
		return queryres;
	}
	
	
	/**
	 * Add the given SMO to the list of new smos being designed. Update inverses consequently.
	 * 
	 * @param smotext
	 * @return
	 */
	@Override
	public SMOSequence addNewSMO(String smotext) {
		MOImpl smo = null;
		try {
			smo = MOFactory.createMOFromString(smotext);
		} catch (SMOParsingErrorException e1) {
			e1.printStackTrace();
		}
		if (smo != null) {
			
			setVersion(smo);

			ICSchema newSch = ((ICSchema)currentSchema).clone();
			
			
			try {
					newSch.applyOperator(smo);
					
					// maintain the schema history updated			
					((ICSchemaHistory)currentFwdSchemaHistory).addSchema(newSch);
					((ICMOSMOSequence)newsmos).addMos(smo);

					
					queryres.addQueryResult(smo);
					// update the inverses by adding the first candidate inverse of this new smo
					if(inverses == null){
						//smos inverses have never been computed so i compute all of them
						getInverseSMOs();	
					}	
					else{
						//add the inverse of this smo
						// ATTENTION: currentSchema here is the one BEFORE the new SMO being applied, 
						//to allow the inverse generation to know about column lists e.g., DROP TABLE page -> CREATE TABLE page(a,b,c)
						if(smo instanceof ICMOImpl)
							((ICMOSMOSequence)inverses).getMos().addAll(((ICMOSMOSequence) smo.getBestCandidateInverses(currentSchema)).getMos());
						else
							((ICMOSMOSequence)inverses).getMos().addAll(smo.getBestCandidateInverses(currentSchema).smos);
						
							
					}
					
					currentSchema = newSch;
				
			} catch (IllegalSMOException e) {
					System.err.println("The smo: " + smo.toMetadataString() + " cannot be applied to the given schema.");
					e.printStackTrace();
			} catch (ExpressionParsignErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (SchemaErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
			//sometimes the ordering get lost... let's play safe and reassign versions..
			reassignFwdVersions();
			reassignInverseVersions();
			recomputeForwardSchemaHistory();
			recomputeInverseSchemaHistory();
		}
		return newsmos;
	}
	
	

	/**
	 * Compares the schema in input with a schema in the history loaded from DB, identified by major and minor version 
	 * 
	 * @param s1
	 * @param maj1
	 * @param min1
	 * @return
	 * @throws SchemaMismatchException 
	 */
	public boolean compareSchemaInHistory(ICSchema s1, int maj1, int min1,boolean checktype) throws SchemaMismatchException{
		
		return compareSchemas(s1,((ICSchemaHistory)sh).getSchemaByVersion(maj1, min1),checktype);
		
	}

	
	
	/**
	 * Compares the schema given in input with the schema in the history loaded from DB in position index1
 	 * @param s1
	 * @param index1
	 * @return
	 * @throws SchemaMismatchException 
	 */
	public boolean compareSchemaInHistory(ICSchema s1, int index1,boolean checktype) throws SchemaMismatchException{
		
		return compareSchemas(s1,((ICSchemaHistory)sh).getSchema(index1),checktype);
		
	}
	
	
	/**
	 * Compare two ICSchema s1,s2 and return true if they are equivalent (i.e., same tables, same columns, same types, same IC)
	 * @param s1
	 * @param s2
	 * @return
	 * @throws SchemaMismatchException 
	 */
	public boolean compareSchemas(ICSchema s1, ICSchema s2,boolean checktype) throws SchemaMismatchException{
		
		return s1.equivalentToSchema(s2,checktype);
		
	}
	
	/**
	 * Verifies SMO syntax and applicability to the schema. Return if ok or not.
	 */
	@Override
	public String testSMOSyntax(String smotext, int schemaIndex,boolean isInverse) {
		
		if(schemaIndex >= ((ICSchemaHistory)currentFwdSchemaHistory).schemas.size()){
			return "SYS ERROR: accessing an unexisting schema: " + schemaIndex;
		}
		ICSchema s = null;
		if(isInverse)
			s = ((ICSchemaHistory)currentFwdSchemaHistory).schemas.elementAt(schemaIndex);
		else
			 s = ((ICSchemaHistory)currentFwdSchemaHistory).schemas.elementAt(schemaIndex);
		
		MOImpl smo = null;
		if (s == null)
			System.err.println("SCHEMA IS NULL");
		ICSchema s2 = s.clone();
		try {
			//smo = SMOSequenceHistoryLoader.parseSMOText(smotext);
			smo =  MOFactory.createMOFromString(smotext);
			
			if(smo==null){
				return "Error: syntax not correct";
				
			}
		} catch (SMOParsingErrorException e) {
			return "<b>Error:</b>" + e.getMessage();
		} 
			
		try {	
			s2.applyOperator(smo);
			return "syntax ok";
		}  catch (ExpressionParsignErrorException e) {
			return "<b>Error:</b> parsing the expression";
		} catch (SchemaErrorException e) {
			return "<b>Error:</b>" + e.getMessage();
		} catch (IllegalSMOException e) {
			//e.printStackTrace();
			return "<b>Error:</b>" + e.getMessage().toString();
		}
		
		}
	
	
	

	@Override
	protected void recomputeForwardSchemaHistory() {
		 currentFwdSchemaHistory = ((ICMOSMOSequence)newsmos).generateICSchemaHistory((ICSchema) startingSchema,false);	
	}
	
	
	@Override
	public SMOSequence editInverseSMO(String text, int major, int minor) {
		
		MOImpl editedSMO = null;
		try {
			editedSMO = MOFactory.createMOFromString(text);
		} catch (SMOParsingErrorException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		if(editedSMO == null){
			try {
				editedSMO = SMOFactory.createSMO(SMOInterface.NOP);
			} catch (SMOTypeUnknownException e) {
				e.printStackTrace();
			}
		}
		MOImpl sub = ((ICMOSMOSequence)inverses).getSMOBySource(major,minor);
		int index = inverses.getSMOIndexBySource(major, minor);
		editedSMO.setSourceMajor(major);
		editedSMO.setSourceMinor(minor);
		editedSMO.setTargetMajor(sub.getTargetMajor());
		editedSMO.setTargetMinor(sub.getTargetMinor());
		
		// substitute the inverse
		((ICMOSMOSequence)inverses).getMos().set(index, editedSMO);
	
		//Added by Swami
		//Need to change the loop from i = 0 to inverses.size() --> inverses.size() to 0
		//Because we need to check applySMO in the reverse order of inverse smo's to satisfy dependency check
		//for (int i = 0; i < inverses.size(); i++) {
			//Schema tempSchema = currentSchema.clone();
		ICSchema tempSchema = (ICSchema) currentSchema.clone();
		for(int i = inverses.size() - 1; i >= 0 ; i--)
		{	
			//CARLO: CHANGED TO USE EXCEPTION
			try
			{
				tempSchema.applyOperator(inverses.get(i));
			}
			catch (IllegalSMOException ise)
			{
				System.err.println("The smo: " + inverses.get(i).toMetadataString() + " cannot be applied to the given schema.");
				ise.printStackTrace();
				//Added by Swami - Incase if applysmo failed on the schema then the SMO must be made NOP
				//otherwise the smo is fine and so need not be changed
				try 
				{
					MOImpl nop = SMOFactory.createSMO(SMOInterface.NOP);
					nop.setSourceMajor(((ICMOSMOSequence)inverses).getMos().get(i).getSourceMajor());
					nop.setSourceMinor(((ICMOSMOSequence)inverses).getMos().get(i).getSourceMinor());
					nop.setTargetMajor(((ICMOSMOSequence)inverses).getMos().get(i).getTargetMajor());
					nop.setTargetMinor(((ICMOSMOSequence)inverses).getMos().get(i).getTargetMinor());
					((ICMOSMOSequence)inverses).getMos().set(i, nop);
				} 
				catch (SMOTypeUnknownException e) 
				{
					e.printStackTrace();
				}
				
			} catch (ExpressionParsignErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (SchemaErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		//Ended - Swami
		
		recomputeInverseSchemaHistory();
		
		return inverses;
	}
	
	
	
	
	
	
	
	
	
	/**
	 * Edit an smo from the list of current smos and recompute the
	 * currentSchema. It also cascade and remove every non-valid following SMO.
	 * Update inverses consequently.
	 * 
	 * @return
	 */
	@Override
	public SMOSequence editSMO(String smo, int index) {
		MOImpl editedSMO = null;
		try {
			editedSMO = MOFactory.createMOFromString(smo);
		} catch (SMOParsingErrorException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		editedSMO.setSourceMajor(((ICMOSMOSequence)newsmos).getMos().get(index).getSourceMajor());
		editedSMO.setSourceMinor(((ICMOSMOSequence)newsmos).getMos().get(index).getSourceMinor());
		editedSMO.setTargetMajor(((ICMOSMOSequence)newsmos).getMos().get(index).getTargetMajor());
		editedSMO.setTargetMinor(((ICMOSMOSequence)newsmos).getMos().get(index).getTargetMinor());
	
		// remove the SMO and its inverse
		((ICMOSMOSequence)newsmos).getMos().set(index, editedSMO);
	
		// update the current schema and the forward schema history accordingly
		currentSchema = startingSchema.clone();
		currentFwdSchemaHistory.getSchemas().add(currentSchema);

		// the temp vector is needed because removing we shift back and the
		// index is no more correct
		Vector<Integer> toberemoved = new Vector<Integer>();
		for (int i = 0; i < newsmos.size(); i++) {
			Schema tempSchema = currentSchema.clone();
			if(i==index){
				//ADD the first candidate inverse in the inverses at the position of the edited smo
				try {
					((ICMOSMOSequence)inverses).getMos().set(index, ((ICMOSMOSequence) editedSMO.getBestCandidateInverses(tempSchema)).getMos().firstElement());
				} catch (AddPrimaryKeyException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (DropForeignKeyException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (AddForeignKeyException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (SchemaErrorException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			

			// CARLO: CHANGED TO USE EXCEPTION (copy of above dangerzone)
			
			try{
				tempSchema.applyOperator(newsmos.get(i));
				// for the successful SMOs add the new intermediate schema to currentFwdSchemaHistory
				((ICSchemaHistory)currentFwdSchemaHistory).schemas.add((ICSchema) tempSchema);
				currentSchema = tempSchema;				
			}
			catch (IllegalSMOException e) {
				toberemoved.add(new Integer(i));
				System.err.println("The smo: " + newsmos.get(i).toMetadataString() + " cannot be applied therefore is removed from the SMO set.");
			} catch (ExpressionParsignErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (SchemaErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}	
			
		}
		
		// check other affected SMOs and remove them all, update inverses too
		// we do it backward to have the indexes valid after deletion
		for (int i = toberemoved.size()-1; i>=0; i--) {
			((ICMOSMOSequence)newsmos).removeReassignVersions(toberemoved.get(i));
			((ICMOSMOSequence)inverses).removeReassignVersions(toberemoved.get(i));
			queryres.remove(toberemoved.get(i).intValue());
		}
		
		//Added by Swami
		//When edit and save is made on FwdSMO's the corres inverses get created - however those newly created inverses
		//have to be checked for dependencies with the existing inverses
		
		Schema tempSchema = currentSchema.clone();
		for(int i = inverses.size() - 1; i >= 0 ; i--)
		{
			try 
			{
				tempSchema.applyOperator(inverses.get(i));
				
			}
			catch (IllegalSMOException ise)
			{
				System.err.println("The smo: " + inverses.get(i).toMetadataString() + " cannot be applied to the given schema.");
				ise.printStackTrace();
			
				try {
					SMOImpl nop = SMOFactory.createSMO(SMOInterface.NOP);
					nop.setSourceMajor(((ICMOSMOSequence)inverses).getMos().get(i).getSourceMajor());
					nop.setSourceMinor(((ICMOSMOSequence)inverses).getMos().get(i).getSourceMinor());
					nop.setTargetMajor(((ICMOSMOSequence)inverses).getMos().get(i).getTargetMajor());
					nop.setTargetMinor(((ICMOSMOSequence)inverses).getMos().get(i).getTargetMinor());
					((ICMOSMOSequence)inverses).getMos().set(i, nop);
				} catch (SMOTypeUnknownException e) {
					e.printStackTrace();
				}
			} catch (ExpressionParsignErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (SchemaErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		//End - Swami
		
		// reassign the versions... since some SMOs might have been dropped
		reassignFwdVersions();
		reassignInverseVersions();

		// recompute the schema histories
		recomputeInverseSchemaHistory();
		recomputeForwardSchemaHistory();
		
		// everything might have been changed by the editing thus the query success must be re-computed
		computQuerySuccess(schemaName);
		
		return newsmos;
	}
	
	
	/**
	 * Remove an smo from the list of current smos and recompute the
	 * currentSchema. It also cascade and remove every non-valid following SMO.
	 * Update inverses consequently.
	 * 
	 * @return
	 */
	@Override
	public SMOSequence removeSMO(int index) {
		
		// remove the SMO and its inverse
		((ICMOSMOSequence)newsmos).removeReassignVersions(index);
		((ICMOSMOSequence)inverses).removeReassignVersions(index);
		queryres.remove(index);
		// update the current schema accordingly
		currentSchema = startingSchema.clone();
		((ICSchemaHistory)currentFwdSchemaHistory).schemas  = new Vector<ICSchema>();
		((ICSchemaHistory)currentFwdSchemaHistory).schemas.add((ICSchema)currentSchema);
		// the temp vector is needed because removing we shift back and the
		// index is no more correct
		Vector<Integer> toberemoved = new Vector<Integer>();
		for (int i = 0; i < newsmos.size(); i++) {
			Schema tempSchema = currentSchema.clone();
			
			// CARLO: CHANGED TO USE EXCEPTION 
			try{
				tempSchema.applyOperator(newsmos.get(i));
				// for the successful SMOs add the new intermediate schema to currentFwdSchemaHistory
				((ICSchemaHistory)currentFwdSchemaHistory).schemas.add((ICSchema)tempSchema);
				currentSchema = tempSchema;				
			}
			catch (IllegalSMOException e) {
				toberemoved.add(new Integer(i));
				System.err.println("The smo: " + newsmos.get(i).toMetadataString() + " cannot be applied therefore is removed from the SMO set.");
			} catch (ExpressionParsignErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (SchemaErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}	
		}
		
		// check other affected SMOs and remove them all, update inverses too
		// we do it backward to have the indexes valid after deletion
		for (int i = toberemoved.size()-1; i>=0; i--) {
			((ICMOSMOSequence)newsmos).removeReassignVersions(toberemoved.get(i));
			((ICMOSMOSequence)newsmos).removeReassignVersions(toberemoved.get(i));
			queryres.remove(toberemoved.get(i).intValue());
		}
		
		// Reassign Versions (the above might fail in some cases)
		reassignFwdVersions();
		reassignInverseVersions();
		
		// regenerate a valid inverse schema history
		recomputeForwardSchemaHistory();
		recomputeInverseSchemaHistory();

		return newsmos;
	}
	
	
	
	
	/**
	 * Set the version of a new SMO, as the following in the newsmos if any or
	 * as the first after the history in smosh.
	 * 
	 * @param smo
	 */
	@Override
	protected void setVersion(MOImpl smo) {
		if (((ICMOSMOSequence)newsmos).getMos().isEmpty()) {
			//If there are no prior SMO let's start from the current schema, and go to the next version.
			if(smosh.smoSequences.isEmpty()){
				smo.setSourceMajor(currentSchema.versionID);
				smo.setSourceMinor(currentSchema.minorVersionID);
				smo.setTargetMajor(currentSchema.versionID + 1);
				smo.setTargetMinor(0);
			}
			else{	
				ICMOSMOSequence temp  = (ICMOSMOSequence) smosh.smoSequences.lastElement();

				smo.setSourceMajor(temp.getMos().lastElement().getTargetMajor());
				smo.setSourceMinor(temp.getMos().lastElement().getTargetMinor());
				smo.setTargetMajor(temp.getMos().lastElement().getTargetMajor() + 1);
				smo.setTargetMinor(0);
			}
			
			
			} else {
			
			// SET THE MOST RECENT SMO AS AN INTERMEDIATE ONE
				((ICMOSMOSequence)newsmos).getMos().lastElement().setTargetMajor(((ICMOSMOSequence)newsmos).getMos().lastElement().getTargetMajor()-1);
				((ICMOSMOSequence)newsmos).getMos().lastElement().setTargetMinor(((ICMOSMOSequence)newsmos).getHighestMinor(((ICMOSMOSequence)newsmos).getMos().lastElement().getTargetMajor())+1);
			
			//SET THE NEW SMO AS THE NEW "TERMINATION"
			smo.setSourceMajor(((ICMOSMOSequence)newsmos).getMos().lastElement().getTargetMajor());
			smo.setSourceMinor(((ICMOSMOSequence)newsmos).getMos().lastElement().getTargetMinor());
			smo.setTargetMajor(((ICMOSMOSequence)newsmos).getMos().lastElement().getTargetMajor()+1);
			smo.setTargetMinor(0);

		}
	}
	
	
	 
		protected void setSMOVersion(MOImpl smo) {
			if (((ICMOSMOSequence)newsmos).getMos().isEmpty()) {
				//If there are no prior SMO let's start from the current schema, and go to the next version.
				if(smosh.smoSequences.isEmpty()){
					smo.setSourceMajor(currentSchema.versionID);
					smo.setSourceMinor(currentSchema.minorVersionID);
					smo.setTargetMajor(currentSchema.versionID + 1);
					smo.setTargetMinor(0);
				}
				else{	
					ICMOSMOSequence temp  = (ICMOSMOSequence) smosh.smoSequences.lastElement();

					smo.setSourceMajor(temp.getMos().lastElement().getTargetMajor());
					smo.setSourceMinor(temp.getMos().lastElement().getTargetMinor());
					smo.setTargetMajor(temp.getMos().lastElement().getTargetMajor() + 1);
					smo.setTargetMinor(0);
				}
				
				
				} else {
				
				//SET THE NEW SMO AS THE NEW "TERMINATION"
				smo.setSourceMajor(((ICMOSMOSequence)newsmos).getMos().lastElement().getTargetMajor());
				smo.setSourceMinor(((ICMOSMOSequence)newsmos).getMos().lastElement().getTargetMinor());
				smo.setTargetMajor(((ICMOSMOSequence)newsmos).getMos().lastElement().getTargetMajor()+1);
				smo.setTargetMinor(0);

			}
		}
	
	
}
