	package edu.ucla.cs.db.prism.backend;

import it.polimi.elet.cautils.configurator.CACmdLineParser;
import it.polimi.elet.cautils.configurator.CAConfiguration;

import java.sql.Connection;
import java.util.Iterator;

import org.apache.log4j.Logger;

import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.SchemaHistory;
import edu.ucla.cs.db.prism.backend.schema.SchemaHistoryLoader;
import edu.ucla.cs.db.prism.backend.schema.exception.IllegalSMOException;
import edu.ucla.cs.db.prism.backend.smo.SMOImpl;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequence;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequenceHistory;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequenceHistoryLoader;
import edu.ucla.cs.db.prism.backend.util.Log4JHelper;
import edu.ucla.cs.db.prism.backend.util.db.DBUtil;

public class MainDeltaSchemaGeneration {
	
	//STATIC LOGGER
	static Logger logger = Logger.getLogger(MainDeltaSchemaGeneration.class);
	
	public static void main(String[] args) {
		 
         //LOG4J SET UP
         Log4JHelper.log4JSetup(logger);
        
        
		//Use CAConfigurator to load configuration from shell or xml-configuration file (-c inputfile)
		 CACmdLineParser par = null;
	 	 try {
			par = CACmdLineParser.getInstance();
		 } catch (InstantiationException e) {
			e.printStackTrace();
		 }
		//List here all the interesting parameters
		 par.addStringParameter('h',"host");	
		 par.addStringParameter('u',"dburl");
		 par.addStringParameter('m',"metadbname");	
		 par.addStringParameter('u',"username");
		 par.addStringParameter('p',"password");
		 par.addStringParameter('l',"logfile");
		 
		//parse them all 
		 CAConfiguration conf = par.parse(args);
		 
		//read the parameter you are interested in 
		 String host = (String) conf.getParameterValue("host");
		 String dburl = (String) conf.getParameterValue("dburl");
		 String metadbname = (String) conf.getParameterValue("metadbname");
		 String dbname = (String) conf.getParameterValue("dbname");
		 String username = (String) conf.getParameterValue("username");
		 String password = (String) conf.getParameterValue("password");
		 
		//use them
		 System.out.println("PRIM is about to be executed with the follwing parameters:");
		 System.out.println("  - host: " + host);		
		 System.out.println("  - user: " + username);
		 System.out.println("  - pass: " + password);
		 System.out.println("  - metadb: " + metadbname);
		 
		// MySQL DRIVER LOADING 
        try {
            Class.forName("com.mysql.jdbc.Driver").newInstance();
        } catch (Exception ex) { ex.printStackTrace();}
	 

         Connection conn = DBUtil.connect(dburl,metadbname,username,password);
         Connection conn2 = DBUtil.connect(dburl,"prim_metadb2",username,password);
         
         // example of usage of the logger
         logger.info("Connected to the DB:" + dburl +" " +metadbname+" " +username);
		 SMOSequenceHistoryLoader smosshl = new SMOSequenceHistoryLoader(conn,dbname);
		 SMOSequenceHistory smossh = smosshl.loadSMOSequenceHistoryFromDB();
		 
		 SchemaHistoryLoader shl = new SchemaHistoryLoader(conn,dbname);
		 SchemaHistory sh = shl.loadSchemaHistoryFromDB();
		 SchemaHistory shoutput = new SchemaHistory();
		 Iterator<Schema> schemaIt = sh.schemas.iterator();
		 Iterator<SMOSequence> smoIt = smossh.smoSequences.iterator();
		 
		 
		 while(schemaIt.hasNext()){
			 Schema schema  = schemaIt.next();
			 shoutput.addSchema(schema);
			 if(smoIt.hasNext()){
			 SMOSequence sequence = smoIt.next();
				 Iterator<SMOImpl> smoit = sequence.smos.iterator();
				 int index = 0;
				 while(smoit.hasNext()){
					 index++;
					 SMOImpl smo = smoit.next();
					 try {
						schema = schema.cloneApplySMO(smo);
					} catch (IllegalSMOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					 System.out.println(schema.toStringWithMetadata());
					 schema.minorVersionID = index;
					 if(smoit.hasNext())
						 shoutput.addSchema(schema);	 
				 }			 
			 }
		 }
			
		/* Schema schema  = sh.schemas.firstElement();
			
		 while(smoIt.hasNext()){
			 shoutput.addSchema(schema);
			 if(smoIt.hasNext()){
			 SMOSequence sequence = smoIt.next();
			 	schema.versionID +=1; 
				 Iterator<SMOImpl> smoit = sequence.smos.iterator();
				 int index = 0;
				 while(smoit.hasNext()){
					 index++;
					 SMOImpl smo = smoit.next();
					 schema = schema.cloneApplySMO(smo);
					 schema.minorVersionID = index;
					 shoutput.addSchema(schema);	 
				 }			 
			 }
		 }*/
		 SchemaHistoryLoader shl2 = new SchemaHistoryLoader(conn2,dbname);
		 shl2.storeSchemaHistoryToDB(shoutput);
		 
		 logger.info("Schema Loaded");
		 
	//	 PrimRewriteEngine pre = new PrimRewriteEngine(sh, null);
	//	 SQLUnion sqlUnion = pre.rewrite(new SQLUnion(), 0, 0);
	}
	

	
}
