package edu.ucla.cs.db.prism.backend.SQL;

import java.io.Serializable;
import java.util.Vector;

public class RelationDeclaration implements Serializable {
	static final long serialVersionUID = 0;
	public int id; // used to identify relation declarations with same relation name
	public String relation;
	public String relationAlias;
	
	public int level = -1; // used in version tagging, 0: root, 1: db, 2: table, 3: row, 4: column
	public Vector<String> matchingTables = null; // used in version tagging
	public Vector<String> matchingColumns = null; // used in version tagging
	public int versionTag = -1;

	// used for xbind-to-xquery translation (xqueryxbind.XBindToXQuery.java)
	public int index = -1;
	public String xpath = null;
	public String var = null;
	public RelationDeclaration parent = null;
	
	public RelationDeclaration(String relation) {
		this.relation=relation;
	}
	public RelationDeclaration(String relation, int index) {
		this.relation=relation;
		relationAlias= "R" + (new Integer(index)).toString();
	}
	public RelationDeclaration(String relation, String relationAlias) {
		this.relation=relation;
		this.relationAlias= relationAlias;
	}
	
	@Override
	public RelationDeclaration clone() {
		RelationDeclaration newRelationDeclaration = 
			new RelationDeclaration(this.relation, this.relationAlias);
		newRelationDeclaration.id = this.id;
		return newRelationDeclaration;
	}

	public void setRelation(String relation) {
		this.relation = relation;
	}
	public void setRelationAlias(int index) {
		// index starts from zero
		relation = "R" + (new Integer(index)).toString();
	}
	public String getRelation() {
		return this.relation;
	}
	public String getRelationAlias() {
		return this.relationAlias;
	}
	
	// decode MARS output
	public void decodePredicateName() {
		relation = relation.replace('S', '/');
		relation = relation.replace('D', '.');
		relation = relation.replace('L', '[');
		relation = relation.replace('R', ']');
		relation = relation.replace('C', ':');
	}
	
	public void setId(int id) {
		this.id = id;
	}
	public int getId() {
		return id;
	}
	
	@Override
	public String toString() {
		return relation + " " + relationAlias;
	}
	
	
	public String toViewString(String dbstring) {
		if (dbstring==null || dbstring.equals(""))
			return relation + " " + relationAlias;
		else
			return dbstring + "." + relation + " " + relationAlias;
	}
	
	public String toStringDebugging() {
		//return relation + " " + relationAlias + " (" + level + ")";
		/*
		return relation + " " + relationAlias + " (" + level + "," + matchingTables + "." 
		+ matchingColumns + "," + versionTag + ")";
		*/
		return relation + " " + relationAlias +
				" (" + index + "," + xpath + "," + var + ");";
	}
	public String toStringConstraintStyle() {
		return relationAlias + " in " + relation;
	}
	
	public void versionTag() {
		if(versionTag > -1)
			relation = "v" + versionTag + relation;
	}

	public void versionTag(int version,int minor_version) {
		
		// prim version
		relation = "v" + version + "m" + minor_version+ "_" + relation;

		/* prima versin below
		if(version > -1 && relation.equals("root")) // version tag root only - for grex (10/3/2007)
			relation = "v" + version + relation;
		
		if(version > -1 && relation.equals("db")) // version tag root only - for grev (10/3/2007)
			relation = "v" + version + relation;
		*/
	}
	
	public void versionTag(String tag) {
		
		relation = tag + "_" + relation;
	}
	
	/*
	public void versionUntag() {
		if(!relation.matches("v(\\p{Digit})+(\\p{Alpha})(.)*")) // e.g. v2relation
			return;

		String untaggedRelation = relation.substring(1); // take out prefix 'v'
		String versionTagString = "";
		while(true) {
			String firstChar = untaggedRelation.substring(0,1);
			if(firstChar.matches("\\p{Digit}")) {
				untaggedRelation = untaggedRelation.substring(1);
				versionTagString += firstChar;
			}
			else
				break;
		}						
		this.versionTag = Integer.parseInt(versionTagString);
		relation = untaggedRelation;
	}
	*/
	
	// 9/6/2007 - simplified, now it works with root only (e.g. v2_2root -> root)
	public void versionUntag() {
		// prim version
		if(relation.charAt(0) != 'v') // don't touch REF (_ref prefix)
			return;
		int index = relation.indexOf("_");
		if(index>=0) 
			relation = relation.substring(index+1);
		
		// prima version
		/*
		if(!relation.startsWith("v") || !relation.endsWith("root"))
			return;
		relation = "root";
		*/
	}
	
	static public String versionUntag(String relationName) {
		if(relationName == null) return null;
		if(relationName.charAt(0) != 'v') // don't touch REF (_ref prefix)
			return null;
		int index = relationName.indexOf("_");
		if(index>=0) 
			relationName = relationName.substring(index+1);
		return relationName;
	}
	
	public boolean equals(RelationDeclaration rd) {
		return this.relation.equals(rd.relation) && this.relationAlias.equals(rd.relationAlias);
	}
	
	@Override
	public boolean equals(Object object) {
		return equals((RelationDeclaration) object);
	}

}
