package edu.ucla.cs.db.prism.backend.SQL;

import java.io.Serializable;
import java.io.StringReader;
import java.util.*;

import edu.ucla.cs.db.prism.backend.parser.parser.PrimaRParser;
import edu.ucla.cs.db.prism.backend.restructure.SQLRestructure;
import edu.ucla.cs.db.prism.backend.smo.util.*;
import edu.ucla.cs.db.prism.backend.util.*;

public class SQL implements Serializable {
	static final long serialVersionUID = 0;
	//public WithClause with;
	public SelectClause select;
	public SelectClause2 select2;
	public FromClause from;
	// updated by Hyun 5/8/07
//	public WhereClause where;
	public Disjunctive where;
	
	public SQL() {
		//this.with = new WithClause();
		this.select = new SelectClause();
		this.from = new FromClause();
		this.where = new Disjunctive();
	}
	
	public SQL(String sqlStatement){
		SQL  temp = SQL.parseSQLString(sqlStatement);
		//this.with = temp.with;
		this.select = temp.select;
		this.from = temp.from;
		this.where = temp.where;
	}
	
	public static SQL extractExternalPositive(String inputsql){
		String positive = "";
		if(inputsql.indexOf("AND NOT EXISTS")>0){
			positive = inputsql.substring(0, inputsql.indexOf("AND NOT EXISTS"));	
		}else{
			positive = inputsql;
		}
		//System.out.println(positive);
		return SQL.parseSQLString(positive);
	}
	
	public static SQL extractNegative(String inputsql){
		String negative = "";
		
		if(inputsql.indexOf("AND NOT EXISTS")>0){
			int ind = inputsql.indexOf("(",inputsql.indexOf("AND NOT EXISTS"));
			negative = inputsql.substring(ind+1,inputsql.lastIndexOf(")"));	
			
			return SQL.parseSQLString(negative);
		}
		return null;
	}
	
	
	public SQL(WithClause with, SelectClause select, FromClause from, Disjunctive where) {
		//this.with = with;
		this.select = select;
		this.from = from;
		this.where = where;
		setRelationDeclarationIds(); 
	}
	
	public SQL(SelectClause select, FromClause from, Disjunctive where) {
		//this.with = null;
		this.select = select;
		this.from = from;
		this.where = where;
		setRelationDeclarationIds(); 
	}

	@Override
	public SQL clone() {
		SQL newSQL = new SQL();
		//newSQL.with = this.with.clone();
		newSQL.select = this.select.clone();
		newSQL.from = this.from.clone();
		newSQL.where = this.where.clone();
		
		return newSQL;
	}
	
	public void setSelectClause(SelectClause select) {
		this.select = select;
	}
	public void setSelectClause2(SelectClause2 select2) {
		this.select2 = select2;
	}
	public void setFromClause(FromClause from) {
		this.from = from;
	}
	public void setWhereClause(Disjunctive where) {
		this.where = where;
	}
	public SelectClause getSelectClause() {
		return this.select;
	}
	public SelectClause2 getSelectClause2() {
		return this.select2;
	}
	public FromClause getFromClause() {
		return this.from;
	}
	public Disjunctive getWhereClause() {
		return this.where;
	}
	
	public void decodePredicateNames() {
		from.decodePredicateNames();
	}
	
	public void setRelationDeclarationIds() {
		from.setIds();
	}

	@Override
	public String toString() {
		String result = "";
		//result += with.toString();
		//if(with.withDeclarations.size() > 0)
		//	result += "\n";
		
		if (select != null && from != null)
			result += select.toString() + "\n" + from.toString();
		else
			return "Incomplete SQL!";
		
		if (where != null && where.size() > 0 && where.conjunctives.get(0).size() > 0)
			result += "\nWHERE " + where.toString();// + ";";
		//else
		//	result += ";";
		
		return result;
	}
	
	
	
	public String toStringWithSelect2() {
		String result = "";
		//result += with.toString();
		//if(with.withDeclarations.size() > 0)
		//	result += "\n";
		
		if (select != null && from != null)
			result += select2.toString() + "\n" + from.toString();
		else
			return "Incomplete SQL!";
		
		if (where != null && where.size() > 0 && where.conjunctives.get(0).size() > 0)
			result += "\nWHERE " + where.toString();// + ";";
		//else
		//	result += ";";
		
		return result;
	}
	public String toStringBenchmark() {
		String result = "";
		//result += with.toString();
		//if(with.withDeclarations.size() > 0)
		//	result += "\n";
		
		if (select != null && from != null)
			result += select.toStringBenchmark() + "\n" + from.toString();
		else
			return "Incomplete SQL!";
		
		if (where != null && where.size() > 0)
			result += "\nWHERE " + where.toString();// + ";";
		//else
		//	result += ";";
		
		return result;
	}
	public String toViewString(String dbString) {
		String result = "";
		//result += with.toString();
		//if(with.withDeclarations.size() > 0)
		//	result += "\n";
		
		if (select != null && from != null)
			result += select.toViewString() + "\n" + from.toViewString(dbString);
		else
			return "Incomplete SQL!";
		
		if (where != null && where.size() > 0)
			result += "\nWHERE " + where.toString();// + ";";
		//else
		//	result += ";";
		
		return result;
	}
	
	
	
	
	public String toStringDebugging() {
		String result = "";
		//result += with.toString();
		//if(with.withDeclarations.size() > 0)
		//	result += "\n";

		if (select != null && from != null)
			result += select.toString() + "\n" + from.toStringDebugging();
		else
			return "Incomplete SQL!";
		
		if (where != null && where.size() > 0)
			result += "\nWHERE " + where.toString();// + ";";
		//else
		//	result += ";";
		
		return result;
	}
	// if relationDeclaration already has its versionTag, assigned by optimalVersionTagging, use it
	// otherwise (if it's set default value -1), use the targetVersion
	public void versionTag(int version,int minor_version) {
		for(int i=0; i<this.getFromClause().relationDeclarationsSize(); i++) {
			RelationDeclaration rd = this.getFromClause().getRelationDeclaration(i);
			//if(rd.versionTag == -1)
			//	rd.versionTag = version;
			rd.versionTag(version,minor_version);
			//rd.versionTag = -1;
			//rd.versionUntag(); // tested and works well
		}
	}
	
	public void versionTag(String tag) {
		for(int i=0; i<this.getFromClause().relationDeclarationsSize(); i++) {
			RelationDeclaration rd = this.getFromClause().getRelationDeclaration(i);
			rd.versionTag(tag);
		}
	}
	
	public void versionUntag() {
		for(int i=0; i<this.getFromClause().relationDeclarationsSize(); i++) {
			RelationDeclaration rd = this.getFromClause().getRelationDeclaration(i);
			//System.out.print("before untag: " + rd.toString() + "\t");
			rd.versionUntag();
			//System.out.println("after untag: " + rd.toString());
		}
	}
	
	public void simplify() {
		//with.simplify();
		where.simplify();
	}
	
	public void removeTautologies() {
		if(where != null) 
			where.removeTautologies();
	}

	
	public void convertFunctionToREF(int targetVersion, int targetVersionMinor) {

		// find the functions in select-clause and where-clause and
		// turn them into REFs
		
		String refAliasPrefix = "F";
		int refAliasIndex = 0;
		// part 1: select clause
		for(int i=0; i<select.selectAttributesSize(); i++) {
			SelectAttribute sa = select.selectAttributes.get(i);
			if(sa.function == null) continue;
			String functionName = sa.function;
			int numFunctionArgs = sa.functionArgs.size();
			String refAlias = refAliasPrefix + refAliasIndex++;

			// step 1. add _ref2_func F0 to the FROM-clause
			String refRelationName = 
				REFConstant.getREF(numFunctionArgs, 
						targetVersion, targetVersionMinor, functionName);
			from.relationDeclarations.add(new RelationDeclaration(refRelationName, refAlias));
			
			// step 2. add F0.a0=arg0 and F0.a1=arg1, ... to where-clause
			if(where.conjunctives.size() == 0)
				where.conjunctives.add(new Conjunctive());
			for(int j=0; j<sa.functionArgs.size(); j++) {
				Predicate p = new Predicate();
				p.term1 = new Term(refAlias, "a" + j);
				p.op = "=";
				p.term2 = sa.functionArgs.get(j);
				where.conjunctives.get(0).add(p);
			}
			
			// step 3. replace select attribute with F0.aN
			select.selectAttributes.remove(i);
			select.selectAttributes.add(i, new SelectAttribute(
					sa.outputAttributeName, refAlias, "a" + sa.functionArgs.size()));
		}
		
		// part 2: where clause
		for(int i=0; where.conjunctives.size() > 0 &&
					 i<where.conjunctives.get(0).size(); i++) {
			//SelectAttribute sa = select.selectAttributes.get(i);
			Predicate p = where.conjunctives.get(0).predicates.get(i);;
			Term functionTerm = null;
			boolean term1HasFunction = false;
			if(p.term1.function == null && p.term2.function == null) 
				continue;
			else if(p.term1.function != null) {
				functionTerm = p.term1;
				term1HasFunction = true;
			}
			else {// p.term1.function == null && p.term2.function != null	
				functionTerm = p.term2;
			}

			String functionName = functionTerm.function;
			int numFunctionArgs = functionTerm.functionArgs.size();
			String refAlias = refAliasPrefix + refAliasIndex++;

			// step 1. add _ref2_func F0 to the FROM-clause
			String refRelationName = 
				REFConstant.getREF(numFunctionArgs, 
						targetVersion, targetVersionMinor, functionName);
			from.relationDeclarations.add(new RelationDeclaration(refRelationName, refAlias));
			
			// step 2. add F0.a0=arg0 and F0.a1=arg1, ... to where-clause
			if(where.conjunctives.size() == 0)
				where.conjunctives.add(new Conjunctive());
			for(int j=0; j<functionTerm.functionArgs.size(); j++) {
				Predicate p2 = new Predicate();
				p2.term1 = new Term(refAlias, "a" + j);
				p2.op = "=";
				p2.term2 = functionTerm.functionArgs.get(j);
				where.conjunctives.get(0).add(p2);
			}
			
			// step 3. replace select attribute with F0.aN
			
			if(term1HasFunction) {
				p.term1 = new Term(refAlias, "a" + functionTerm.functionArgs.size());				
				i--; // check fof term2 in the next iteration
			}
			else { // term2 has function
				p.term2 = new Term(refAlias, "a" + functionTerm.functionArgs.size());				
			}
		}		
	}
	
	/**
	 * convert inequalities in where clause into RelationEncodedFunction
	 * R0.a1 < R1.a1 -> _ref2_less(R0.a1, R1.a1)
	 * R0.a1 <= R1.a1 -> _ref2_leq(R0.a1, R1.a1)
	 */
	public void convertInequalityToREF(int targetVersion, int targetVersionMinor) {
		if(where == null) return; // nothing to do
		if(where.conjunctives.size() == 0) return; // nothing to do

		int index = 1; // 0 used by vidb
		Conjunctive conjunctive = where.conjunctives.get(0);
		int size = conjunctive.predicates.size();;
		for(int i=0; i<size; i++) {
			Predicate p = conjunctive.predicates.get(i);
			if(p.op.equals(">")) {
				p.op = "<";
				Term term1 = p.term1;
				p.term1 = p.term2;
				p.term2 = term1;
			}
			else if(p.op.equals(">=")) {
				p.op = "<=";
				Term term1 = p.term1;
				p.term1 = p.term2;
				p.term2 = term1;
			}
			
			if(!p.op.equals("<") && 
					!p.op.equals("<=") &&
					!p.op.equals("!=") &&
					!p.op.equals("<>")) // <> not supported in prim parser now - 2/1/2008
				continue;

			String relation = null;
			if(p.op.equals("<"))
				relation = REFConstant.getREF(2, targetVersion, targetVersionMinor, "less"); // _ref2_less
			else if(p.op.equals("<="))
				relation = REFConstant.getREF(2, targetVersion, targetVersionMinor, "leq"); // _ref2_leq
			else if(p.op.equals("!=") || p.op.equals("<>"))
				relation = REFConstant.getREF(2, targetVersion, targetVersionMinor, "neq"); // _ref2_neq
			
			String relationAlias = "y" + index;
			index++;
			
			from.relationDeclarations.add(
					new RelationDeclaration(relation, relationAlias));
			conjunctive.predicates.remove(i);
			conjunctive.predicates.add(i, 
					new Predicate(new Term(relationAlias, "a0"), "=", p.term1));
			conjunctive.predicates.add(i+1, 
					new Predicate(new Term(relationAlias, "a1"), "=", p.term2));
			i++;
			size++;
		}
	}
	

	public void convertREFToInequality(int targetVersion, int targetVersionMinor) {
		int relationDeclarationsSize = from.relationDeclarationsSize();
		for(int i=0; i<relationDeclarationsSize; i++) {
			// step 1: find a relation with a name "_ref2_less"
			RelationDeclaration rd = from.relationDeclarations.get(i);
			if(!rd.relation.startsWith(REFConstant.getREF(2, targetVersion, targetVersionMinor, "less")) &&
					!rd.relation.startsWith(REFConstant.getREF(2, targetVersion, targetVersionMinor, "leq")) &&
					!rd.relation.startsWith(REFConstant.getREF(2, targetVersion, targetVersionMinor, "neq")))
				continue;
			
			// step 2: the number of relation arguments 
			int numArguments = 2;
			
			// step 3: find R1.a1 and R2.a2 such that 
			//         a = R1.a1, b = R2.a2 to use in a < b
			Term[] argumentTerms = new Term[numArguments];
			for(int j=0; j<numArguments; j++) {
				Vector<Term> terms = 
					where.get(0).getTermEQualToRAClosure(rd.relationAlias, "a" + j);
				while(terms.size() > 0) {
					// don't use REF in the new term
					String alias = terms.get(0).tableName;
					String relationName = from.getRelation(alias);
					if(relationName.startsWith(REFConstant.getRelationEncodedFunctionPrefix()))
						terms.remove(0);
					else
						break;
				}
				if(terms.size() == 0) return; // error
				//for(int k=0; k<terms.size(); k++) {
				//	if(terms.get(k).equals(new Term(rd.relationAlias, "a" + j)))
				//		continue;
				//	argumentTerms[j] = terms.get(k); // use the first, except the self
				//}
				argumentTerms[j] = terms.get(0); // use the first - self was excluded above
			}
			
			// step 4: add a new predicate, argumentTerms[0] < argumentTerms[1]
			Predicate newComparisonPredicate = new Predicate();
			newComparisonPredicate.term1 = argumentTerms[0];
			newComparisonPredicate.term2 = argumentTerms[1];
			if(rd.relation.startsWith(REFConstant.getREF(2, targetVersion, targetVersionMinor, "less")))
				newComparisonPredicate.op = "<";
			else if(rd.relation.startsWith(REFConstant.getREF(2, targetVersion, targetVersionMinor, "leq")))
				newComparisonPredicate.op = "<=";
			else if(rd.relation.startsWith(REFConstant.getREF(2, targetVersion, targetVersionMinor, "neq")))
				newComparisonPredicate.op = "!=";
				
			where.conjunctives.get(0).add(newComparisonPredicate);
			
			// step 5: search each predicate and replace ref_func.a* with equiv values
			for(int j=0; j<where.conjunctives.get(0).size(); j++) {
				Predicate p = where.conjunctives.get(0).get(j);
				// TODO: p.term1.isRATerm() is a temp-fix - should do recursive function navigation & substitution
				if(p.term1.isRATerm() && p.term1.tableName.equals(rd.relationAlias)) {
					int attributeIndex =
						Integer.parseInt(p.term1.columnName.substring(1)); // "1" in "a1"
					p.term1 = argumentTerms[attributeIndex].clone();
				}
				if(p.term2.isRATerm() && p.term2.tableName.equals(rd.relationAlias)) {
					int attributeIndex =
						Integer.parseInt(p.term2.columnName.substring(1)); // "1" in "a1"
					p.term2 = argumentTerms[attributeIndex].clone();
				}
			}
			
			
			// step 6: do the same as in step 5, for Select clause
			for(int j=0; j<select.selectAttributes.size(); j++) {
				SelectAttribute sa = select.selectAttributes.get(j);
				if(sa.relationAlias != null && sa.relationAlias.equals(rd.relationAlias)) {
					int attributeIndex =
						Integer.parseInt(sa.attribute.substring(1)); // "1" in "a1"
					Term newTerm = argumentTerms[attributeIndex];
					sa.relationAlias = null;
					sa.attribute = null;
					if(newTerm.isConstantTerm()) {
						sa.constant = newTerm.constant;
					}
					else if(newTerm.isRATerm()) {
						sa.relationAlias = newTerm.tableName;
						sa.attribute = newTerm.columnName;
					}
					else if(newTerm.isFunctionTerm()) {
						sa.function = newTerm.function;
						sa.functionArgs = (Vector<Term>)newTerm.functionArgs.clone();
					}
				}
				else if(sa.function != null) {
					sa.replaceFunctionArg(rd.relationAlias, argumentTerms);
				}
			}
			
			
			
			
			from.relationDeclarations.remove(i);
			i--;
			relationDeclarationsSize--;
		}
		
		// 4/7/2008 Hyun
		// to minimize the query - otherwise the following MARS run complains
		this.removeTautologies();
		//this.simplify();
	}



	/**
	 * convert SQL's Relation-Encoded-Functions (REFs) into regular functions
	 * e.g. ref2_max(a,b,c) -> c=max(a,b)
	 * @param sql
	 * @return
	 */
	public void convertREFToFunction(int targetVersion, int targetVersionMinor) {
		int relationDeclarationsSize = from.relationDeclarationsSize();
		for(int i=0; i<relationDeclarationsSize; i++) {
			// step 1: find a relation with a prefix "ref" - util.Constants.relationEncodedFunctionPrefix
			RelationDeclaration rd = from.relationDeclarations.get(i);
			if(!rd.relation.startsWith(REFConstant.getRelationEncodedFunctionPrefix()))
				continue;
			// inequality handled specially by convertREFToInequality()
			else if(rd.relation.equals(REFConstant.getREF(2, targetVersion, targetVersionMinor, "less"))) 
				continue;
			else if(rd.relation.equals(REFConstant.getREF(2, targetVersion, targetVersionMinor, "leq"))) 
				continue;
			else if(rd.relation.equals(REFConstant.getREF(2, targetVersion, targetVersionMinor, "neq"))) 
				continue;
			
			// step 2: extract the number of function arguments 
			//         and function name (e.g.from rf2_max)
			int numArguments = REFConstant.getNumOfArgs(rd.relation);
			String functionName = REFConstant.getFunctionName(rd.relation);
			
			// step 3: find R1.a1 and R2.a2 such that 
			//         a = R1.a1, b = R2.a2 to use in max(a,b)
			Term[] argumentTerms = new Term[numArguments+1];
			for(int j=0; j<numArguments; j++) {
				Vector<Term> terms = 
					where.get(0).getTermEQualToRAClosure(rd.relationAlias, "a" + j);
				while(terms.size() > 0) {
					// don't use REF in the new term
					String alias = terms.get(0).tableName;
					String relationName = from.getRelation(alias);
					if(relationName.startsWith(REFConstant.getRelationEncodedFunctionPrefix()))
						terms.remove(0);
					else
						break;
				}
				if(terms.size() == 0) {
					System.out.println("terms.size() became 0");
					System.out.println(functionName + j);
					return; // error
				}
				argumentTerms[j] = terms.get(0);
			}
			
			// step 4: fill argumentTerms[numArugments] = func(rest_arguments)
			// TODO : decide whether clone() is needed below
			Term functionOutputTerm = new Term();
			functionOutputTerm.function = functionName;
			functionOutputTerm.functionArgs = new Vector<Term>();
			for(int j=0; j<numArguments; j++) {
				functionOutputTerm.functionArgs.add(argumentTerms[j].clone());
			}
			argumentTerms[numArguments] = functionOutputTerm;
			
			//System.out.print(functionName + "'s argumentTerms: ");
			//for(int j=0; j<argumentTerms.length; j++)
			//	System.out.print(argumentTerms[j] + ", ");
			//System.out.println("");
			
			// step 5: search each predicate and replace ref_func.a* with equiv values
			int predicatesSize = where.conjunctives.get(0).size();
			for(int j=0; j<predicatesSize; j++) {
				Predicate p = where.conjunctives.get(0).get(j);
				if(p.term1.isRATerm() && p.term1.tableName.equals(rd.relationAlias)) {
					int attributeIndex =
						Integer.parseInt(p.term1.columnName.substring(1)); // "1" in "a1"
					if(p.term2.equals(argumentTerms[attributeIndex]) && p.op.equals("=")) {
						// term = term (always true predicate)
						where.conjunctives.get(0).remove(j);
						j--;
						predicatesSize--;
					}
					else {
						p.term1 = argumentTerms[attributeIndex].clone();
					}
						
				}
				else if(p.term1.isFunctionTerm())
					p.term1.replaceFunctionArg(rd.relationAlias, argumentTerms);
				
				else if(p.term2.isRATerm() && p.term2.tableName.equals(rd.relationAlias)) {
					int attributeIndex =
						Integer.parseInt(p.term2.columnName.substring(1)); // "1" in "a1"
					if(p.term1.equals(argumentTerms[attributeIndex]) && p.op.equals("=")) { 
						// term = term (always true predicate)
						where.conjunctives.get(0).remove(j);
						j--;
						predicatesSize--;
					}
					else {
						p.term2 = argumentTerms[attributeIndex].clone();
					}
				}
				else if(p.term2.isFunctionTerm())
					p.term2.replaceFunctionArg(rd.relationAlias, argumentTerms);
			}
			
			// step 6: do the same as in step 5, for Select clause
			for(int j=0; j<select.selectAttributes.size(); j++) {
				SelectAttribute sa = select.selectAttributes.get(j);
				if(sa.relationAlias != null && sa.relationAlias.equals(rd.relationAlias)) {
					int attributeIndex =
						Integer.parseInt(sa.attribute.substring(1)); // "1" in "a1"
					Term newTerm = argumentTerms[attributeIndex];
					sa.relationAlias = null;
					sa.attribute = null;
					if(newTerm.isConstantTerm()) {
						sa.constant = newTerm.constant;
					}
					else if(newTerm.isRATerm()) {
						sa.relationAlias = newTerm.tableName;
						sa.attribute = newTerm.columnName;
					}
					else if(newTerm.isFunctionTerm()) {
						sa.function = newTerm.function;
						sa.functionArgs = (Vector<Term>)newTerm.functionArgs.clone();
					}
				}
				else if(sa.function != null) {
					sa.replaceFunctionArg(rd.relationAlias, argumentTerms);
				}
			}
			
			// delete the processed relation-encoded-function from the FROM clause
			from.relationDeclarations.remove(i);
			relationDeclarationsSize--;
			i--;
		}
		
		// 4/7/2008 Hyun
		// to minimize the query - otherwise the following MARS run complains
		this.removeTautologies();
		//this.simplify();
	}
	
	public static SQL parseSQLString(String inString) {
		StringReader input = null;
		input = new StringReader(inString);

		// step 1.2: parse it
		PrimaRParser parser = new PrimaRParser(input);
		//PrimaRScript script = null;
		edu.ucla.cs.db.prism.backend.parser.syntaxtree.sql sql = null;
		try {
			sql = parser.sql();
			//script = parser.PrimaRScript();
		} catch(Exception e) {
			e.printStackTrace();
		}
		return SQLRestructure.restructure(sql);
	}

	public String toStringNegated() {
		String result = "";
		//result += with.toString();
		//if(with.withDeclarations.size() > 0)
		//	result += "\n";
		
		if (select != null && from != null)
			result += select.toViewString() + "\n" + from.toViewString(null);
		else
			return "Incomplete SQL!";
		
		if (where != null && where.size() > 0)
			result += "\nWHERE NOT (" + where.toString() + ")";// + ";";
		//else
		//	result += ";";
		
		return result;	}

	
	
	

}
