package edu.ucla.cs.db.prism.backend.SQL;

import java.io.Serializable;
import java.util.*;
import edu.ucla.cs.db.prism.backend.smo.util.*;

public class SelectAttribute implements Serializable {
	static final long serialVersionUID = 0;
	public String outputAttributeName = null;
	public String relationAlias = null;
	public String attribute = null;
	public String aggregate = null;
	public String constant = null;
	public String function = null;
	public Vector<Term> functionArgs = null;
	
	public SelectAttribute() {
		//outputAttributeName = "";
		//relationAlias = "";
		//attribute = "";
	}
	public SelectAttribute(int outputAttributeIndex, 
			String relationAlias, int attributeIndex) throws SelectAttributeException {
		this.outputAttributeName = "S" + (new Integer(outputAttributeIndex)).toString();
		this.relationAlias = relationAlias;
		this.attribute = "a" + (new Integer(attributeIndex)).toString();
		// xbind predicates have only one or two attributes
		if(attributeIndex > 1) {
			System.out.println("Unexpected Error: attributeIndex > 1");
			throw new SelectAttributeException("Unexpected Error: attributeIndex > 1");
			//System.exit(-1);
		}
		
	}
	public SelectAttribute(String outputAttribute, 
			String relationAlias, String attribute) {
		this.outputAttributeName = outputAttribute;
		this.relationAlias = relationAlias;
		this.attribute = attribute;
	}

	@Override
	public SelectAttribute clone() {
		SelectAttribute newSelectAttribute = new SelectAttribute();
		newSelectAttribute.outputAttributeName = this.outputAttributeName;
		newSelectAttribute.relationAlias = this.relationAlias;
		newSelectAttribute.aggregate = this.aggregate;
		newSelectAttribute.attribute = this.attribute;
		return newSelectAttribute;
	}
	
	public void setOutputAttributeName(int index) {
		// index starts from zero
		outputAttributeName = "S" + (new Integer(index)).toString();
	}
	public void setRelationAlias(String relationAlias) {
		this.relationAlias = relationAlias;
	}
	public void setAttribute(String attribute) {
		this.attribute = attribute;
	}
	public String getOutputAttributeName() {
		return this.outputAttributeName;
	}
	public String getRelationAlias() {
		return this.relationAlias;
	}
	public String getAttribute() {
		return this.attribute;
	}
	

	/**
 	 * look for relationAlias.ak and replace it with the clone of newTerms[k]
 	 * used for relationEncodedFunction decoding
	 * @param relationAlis
	 * @param newTerms
	 */
	public void replaceFunctionArg(String relationAlias, Term[] newTerms) {
		if(this.function == null)
			return;
		for(int i=0; i<this.functionArgs.size(); i++) {
			if(this.functionArgs.get(i).isRATerm() && 
					this.functionArgs.get(i).tableName.equals(relationAlias)) {
				int attributeIndex =
					Integer.parseInt(this.functionArgs.get(i).columnName.substring(1)); // "1" in "a1"
				this.functionArgs.remove(i);
				this.functionArgs.add(i, newTerms[attributeIndex].clone());
			}
			else if(this.functionArgs.get(i).isFunctionTerm())
				this.functionArgs.get(i).replaceFunctionArg(relationAlias, newTerms);	
		}
	}
	
	@Override
	public String toString() {
		if(constant != null)
			return outputAttributeName + "=" + stripQuote(constant);
		else if(function != null && functionArgs != null) {
			String str = outputAttributeName + "=" + function + "(";
			for(int i=0; i<functionArgs.size(); i++) {
				str += functionArgs.get(i);
				if(i < functionArgs.size()-1)
					str += ",";
			}
			str += ")";
			return str;
		}
		else if(aggregate == null)
			return outputAttributeName + "=" + relationAlias + "." + attribute;
		else
			return outputAttributeName + "=" + aggregate + 
					"(" +relationAlias + "." + attribute + ")";
	}
	public String toStringBenchmark() {
		if(constant != null)
			return constant;
		else if(function != null && functionArgs != null) {
			String str = function + "(";
			for(int i=0; i<functionArgs.size(); i++) {
				str += functionArgs.get(i);
				if(i < functionArgs.size()-1)
					str += ",";
			}
			str += ")";
			return str;
		}
		else if(aggregate == null)
			return relationAlias + "." + attribute;
		else
			return aggregate + 
					"(" +relationAlias + "." + attribute + ")";
	}

	public String toViewString() {
		if(constant != null)
			return  getSingleQuoteConstant();
		else if(function != null && functionArgs != null) {
			String str = function + "(";
			for(int i=0; i<functionArgs.size(); i++) {
				str += functionArgs.get(i);
				if(i < functionArgs.size()-1)
					str += ",";
			}
			str += ")";
			return str;
		}
		else if(aggregate == null)
			return relationAlias + "." + attribute;
		else
			return  aggregate + "(" +relationAlias + "." + attribute + ")";
	}
	
	public String stripQuote(String str) {
		if(str == null) return null;
		if(str.equals("")) return str;
		if(str.charAt(0) == '"') {
			str = str.substring(1, str.length()-1);
		}
		return str;
	}
	
	public String getSingleQuoteConstant(){
		
		
		return "\"" + stripQuote(stripQuote(constant)) + "\"";
		
	}
	
}
