package edu.ucla.cs.db.prism.backend.SQL;

import java.io.Serializable;

import edu.ucla.cs.db.prism.backend.smo.util.Predicate;

public class WhereCondition implements Serializable {
	static final long serialVersionUID = 0;
	// atom op atom
	 // atom is either rel.attr or a constant
	boolean constantFlag1;
	String relationAlias1;
	String attribute1;
	String constant1;

	String op;
	
	boolean constantFlag2;
	String relationAlias2;
	String attribute2;
	String constant2;
	
	// used for SQL-to-XBind translation
	String xbindVariableName;

	public WhereCondition() {
		constantFlag1 = false;
		relationAlias1 = "";
		attribute1 = "";
		constant1 = null;
		
		op = "=";
		
		constantFlag2 = false;
		relationAlias2 = "";
		attribute2 = "";
		constant2 = null;
	}

	// join condition generation
	public WhereCondition(String relationAlias1, int attributeIndex1, String op,
		String relationAlias2, int attributeIndex2) {
		constantFlag1 = false;
		this.relationAlias1 = relationAlias1;
		attribute1 = "a" + (new Integer(attributeIndex1)).toString();
		constant1 = null;
		
		this.op = op;
		
		constantFlag2 = false;
		this.relationAlias2 = relationAlias2;
		attribute2 = "a" + (new Integer(attributeIndex2)).toString();
		constant2 = null;
	}
	
	// conditionstatement translation - attribute-index based constructor
	public WhereCondition(
			Boolean constantFlag1, String relationAlias1, int attributeIndex1, String constant1,
			String op,
			Boolean constantFlag2, String relationAlias2, int attributeIndex2, String constant2) {

			this.constantFlag1 = constantFlag1;
			this.relationAlias1 = relationAlias1;
			attribute1 = "a" + (new Integer(attributeIndex1)).toString();
			this.constant1 = constant1;
			
			this.op = op;
			
			this.constantFlag2 = constantFlag2;
			this.relationAlias2 = relationAlias2;
			attribute2 = "a" + (new Integer(attributeIndex2)).toString();
			this.constant2 = constant2;
	}
	// conditionstatement translation - general-purpose constructor
	public WhereCondition(
			Boolean constantFlag1, String relationAlias1, String attribute1, String constant1,
			String op,
			Boolean constantFlag2, String relationAlias2, String attribute2, String constant2) {

			this.constantFlag1 = constantFlag1;
			this.relationAlias1 = relationAlias1;
			this.attribute1 = attribute1;
			this.constant1 = constant1;
			
			this.op = op;
			
			this.constantFlag2 = constantFlag2;
			this.relationAlias2 = relationAlias2;
			this.attribute2 = attribute2;
			this.constant2 = constant2;
	}
	public boolean getConstantFlag1() {
		return constantFlag1;
	}
	public String getRelationAlias1() {
		return relationAlias1;
	}
	public String getAttribute1() {
		return attribute1;
	}
	public String getConstant1() {
		return constant1;
	}
	public String getOp() {
		return op;
	}
	public boolean getConstantFlag2() {
		return constantFlag2;
	}
	public String getRelationAlias2() {
		return relationAlias2;
	}
	public String getAttribute2() {
		return attribute2;
	}
	public String getConstant2() {
		return constant2;
	}
	public void setConstant2(String constant) {
		constant2 = constant;
	}	
	public void setXBindVariableName(String name) {
		this.xbindVariableName = name;
	}
	public String getXBindVariableName() {
		return this.xbindVariableName;
	}
	
	public boolean isJoin() {
		return constantFlag1 == false && constantFlag2 == false && op.contentEquals("=");
	}

	public boolean hasCommonVariable(WhereCondition wc) {
		boolean result=false;
		
		if(this.getConstantFlag1() == false && wc.getConstantFlag1() == false &&
				this.getRelationAlias1().contentEquals(wc.getRelationAlias1()) &&
				this.getAttribute1().contentEquals(wc.getAttribute1()))
			result = true;
		else if(this.getConstantFlag1() == false && wc.getConstantFlag2() == false &&
				this.getRelationAlias1().contentEquals(wc.getRelationAlias2()) &&
				this.getAttribute1().contentEquals(wc.getAttribute2()))
			result = true;
		else if(this.getConstantFlag2() == false && wc.getConstantFlag1() == false &&
				this.getRelationAlias2().contentEquals(wc.getRelationAlias1()) &&
				this.getAttribute2().contentEquals(wc.getAttribute1()))
			result = true;
		else if(this.getConstantFlag2() == false && wc.getConstantFlag2() == false &&
				this.getRelationAlias2().contentEquals(wc.getRelationAlias2()) &&
				this.getAttribute2().contentEquals(wc.getAttribute2()))
			result = true;

		return result;
	}
	
	public boolean hasVariable(String relationAlias3, String attribute3) {
		boolean result=false;

		if(this.getConstantFlag1() == false &&
				this.getRelationAlias1().contentEquals(relationAlias3) &&
				this.getAttribute1().contentEquals(attribute3))
			result = true;
		else if(this.getConstantFlag2() == false &&
				this.getRelationAlias2().contentEquals(relationAlias3) &&
				this.getAttribute2().contentEquals(attribute3))
			result = true;

		//System.out.print(this.toString());
		//System.out.println("hasVariable(" + relationAlias3 + "," + attribute3 + 
		//					"): " + result);
		
		return result;
	}
	
	// introduced to make it easy to convert whereCondition into a predicate with two terms
	public static Predicate newPredicate(
			Boolean constantFlag1, String relationAlias1, String attribute1, String constant1,
			String op,
			Boolean constantFlag2, String relationAlias2, String attribute2, String constant2) {
		
			if(constantFlag1 == false && constantFlag2 == false)
				return new Predicate(relationAlias1, attribute1, op,
						relationAlias2, attribute2);
			else if(constantFlag1 == false && constantFlag2 == true)
				return new Predicate(relationAlias1, attribute1, op,
						constant2);
			else
				return null;
	}
	
	@Override
	public String toString() {
		String atom1, atom2;
		if(constantFlag1 == true)
			atom1 = constant1;
		else
			atom1 = relationAlias1 + "." + attribute1;
		
		if(constantFlag2 == true)
			atom2 = constant2;
		else
			atom2 = relationAlias2 + "." + attribute2;
		
		return atom1 + " " + op + " " + atom2;
		//if(xbindVariableName == null)
		//	return atom1 + " " + op + " " + atom2 + "(null)";
		//else
		//	return atom1 + " " + op + " " + atom2 + "(" + xbindVariableName + ")";
	}
	
}
