package edu.ucla.cs.db.prism.backend.compose;

import edu.ucla.cs.db.prism.backend.SQL.*;
import edu.ucla.cs.db.prism.backend.ded.*;
import edu.ucla.cs.db.prism.backend.exception.ErrorRewritingException;
import edu.ucla.cs.db.prism.backend.exception.RewritingOptimizationException;
import edu.ucla.cs.db.prism.backend.marsManage.exception.MarsConstantsSuffixException;
import edu.ucla.cs.db.prism.backend.rewrite.*;
import edu.ucla.cs.db.prism.backend.schema.*;
import edu.ucla.cs.db.prism.backend.smo.util.*;

import java.util.*;

public class DEDComposition {
	private SchemaHistory schemaHistory;
	private DEDSetHistory dedSetHistory;
	final int numberOfDEDSetsToComposeInSingleRewrite = 2;
	
	public DEDComposition(SchemaHistory schemaHistory, DEDSetHistory dedSetHistory) {
		this.schemaHistory = schemaHistory;
		this.dedSetHistory = dedSetHistory;
	}
	
	
	// first testing scenario
	// one designated target version (last version)
	// many source versions (historical versions)
	// forward deds (from input SMOs): source version < target version
	public void composeAllMajorVersionsToTargetVersionForwardDEDs(int targetVersion) {
		// - construct forward ded composition from each major version to the
		// specified target version (into this.dedSetHistory)
		// - major versions used as source versions should be less than (or before)
		// the specified target version
		
		// assume contiguous version numbering (e.g. 1,2,3,4,...)
		int sourceVersion = targetVersion - 2;
		while(true) {
			if(!schemaHistory.hasVersion(sourceVersion, 0))
				break;
			try {
				composeMultipleRewrite(sourceVersion, 0, targetVersion, 0);
			} catch (RewritingOptimizationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			sourceVersion--;
		}
	}

	// PRIM scenario
	// many target versions (historical versions)
	// one designated source version (last version)
	// backward deds (from inverse SMOs): target version < source version
	
	public void composeSourceVersionToAllMajorVersionsBackwardDEDs(int targetVersion) {
		// TODO
	}
	
	public DEDSet compose(int sourceVersion, int sourceMinorVersion,
			int targetVersion, int targetMinorVersion) {
		try {
			return composeMultipleRewrite(sourceVersion, sourceMinorVersion,
					targetVersion, targetMinorVersion);
		} catch (RewritingOptimizationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}
	/*
	public DEDSet composeMultipleRewrite(int sourceVersion, int sourceMinorVersion,
			int targetVersion, int targetMinorVersion) {
		int compareResult =
			PrimRewriteEngine.compareVersions(sourceVersion, sourceMinorVersion,
				targetVersion, targetMinorVersion);
		if(compareResult < 0)
			return composeMultiRewriteForwardDEDs(sourceVersion, sourceMinorVersion,
					targetVersion, targetMinorVersion);
		else if(compareResult > 0)
			return composeMultiRewriteBackwardDEDs(sourceVersion, sourceMinorVersion,
					targetVersion, targetMinorVersion);
		else
			return null;
	}
	*/
	
	private DEDSet composeMultipleRewrite(int sourceVersion, int sourceMinorVersion,
			int targetVersion, int targetMinorVersion) throws RewritingOptimizationException {
		// determine how many dedSets (= number of version distances)
		// to compose at a time
		

		int[] tempSourceVersionFull;
		int[] sourceVersionFull = {sourceVersion, sourceMinorVersion};
		int[] targetVersionFull = {targetVersion, targetMinorVersion};
		DEDSet composedDEDSet = null;
		
		while(true) {
			// step 1: find the tempSourceVersion that is 
			// (1) farthest from target version,
			// (2) closest to source version
			// (3) reached within n hop (n: given number of dedSets used for one composition)
			// (4) equal to the source version or between the source and the target versions
			tempSourceVersionFull =
				getBestTempSourceVersion(dedSetHistory, 
						sourceVersion, sourceMinorVersion, 
						targetVersion, targetMinorVersion, 
						numberOfDEDSetsToComposeInSingleRewrite);
			
			System.out.println("tempSourceVersionFull: " + 
					"v" + tempSourceVersionFull[0] + 
					"m" + tempSourceVersionFull[1]);
			
			// source version can be reached without any composition
			if(tempSourceVersionFull == null)
				break;
				
			// step 2: create some partial composition
			composedDEDSet = composeSingleRewrite(
					tempSourceVersionFull[0], tempSourceVersionFull[1],
					targetVersionFull[0], targetVersionFull[1]);

			// step 3: add the composed dedSet to dedSetHistory
			this.dedSetHistory.add(composedDEDSet);
			
			// step 4: stop looping if reached the source version
			int compareResult =
				PrimRewriteEngine.compareVersions(
						sourceVersionFull, tempSourceVersionFull);
			if(compareResult == 0)
				break;
		}

		
		// source version can be reached without any composition
		if(tempSourceVersionFull == null) {
			return dedSetHistory.get(sourceVersionFull, targetVersionFull);
		}
		else
			return composedDEDSet;
	}
	

	/**
	 * composition of data-migration-direction deds
	 * this runs a single mars, which can be quite slow, if there are many deds
	 * @param sourceVersion
	 * @param minorSourceVersion
	 * @param targetVersion
	 * @param minorTargetVersion
	 * @return
	 */
	private DEDSet composeSingleRewrite(int sourceVersion, int sourceMinorVersion,
			int targetVersion, int targetMinorVersion) {
		
		// step 1: construct one query for each table in sourceVersion schema
		Schema sourceSchema = 
			schemaHistory.getSchemaByVersion(targetVersion, targetMinorVersion);
		SQLUnionSet inputSQLUnionSet = new SQLUnionSet();
		for(int i=0; i<sourceSchema.size(); i++) {
			SQL canonicalSQL = getCanonicalSQL(sourceSchema.getTables().get(i));
			SQLUnion sqlUnion = new SQLUnion();
			sqlUnion.name = sourceSchema.getTables().get(i).getTableName();
			sqlUnion.sqls.add(canonicalSQL);
			inputSQLUnionSet.sqlUnions.add(sqlUnion);
		}
		
		// step 2: rewrite inputSQLUnionSet
		PrimRewriteEngine pre = new PrimRewriteEngine(schemaHistory, dedSetHistory);
		SQLUnionSet outputSQLUnionSet = null;
		try {
			outputSQLUnionSet = pre.rewrite(inputSQLUnionSet, sourceVersion, sourceMinorVersion, targetVersion, targetMinorVersion);
		} catch (ErrorRewritingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		// version untagged inside pre.rewrite - re-tag it using source version
		outputSQLUnionSet.versionTag(sourceVersion, sourceMinorVersion);
		
		// step 4: using one-to-one attribute match between input query and
		//         output query, construct DED for each query
		DEDSet composedDEDSet = new DEDSet();
		for(int i=0; i<inputSQLUnionSet.sqlUnions.size(); i++) {
			SQL inputSQL = inputSQLUnionSet.sqlUnions.get(i).sqls.get(0); // singleton union
			SQLUnion outputSQLUnion =
				outputSQLUnionSet.sqlUnions.get(i);

			DED dedQR = getQRDirectionDEDs(inputSQL, outputSQLUnion, sourceVersion, sourceMinorVersion,
					targetVersion, targetMinorVersion);
			composedDEDSet.add(dedQR);
			
			// rewritten query can be empty - then no DM-direction DED
			if(outputSQLUnion != null) {
				DEDSet dedsDM = reverseQRDEDDirectionIntoDM(dedQR);
				composedDEDSet.addAll(dedsDM);
			}
		}
		
		composedDEDSet.sourceVersion = sourceVersion;
		composedDEDSet.minorSourceVersion = sourceMinorVersion;
		composedDEDSet.targetVersion = targetVersion;
		composedDEDSet.minorTargetVersion = targetMinorVersion;
		
		// naming deds
		for(int i=0; i<composedDEDSet.deds.size(); i++) {
			DED ded = composedDEDSet.deds.get(i);
			ded.dedName = 
				"v" + sourceVersion + "m" + sourceMinorVersion + "to" +
				"v" + targetVersion + "m" + targetMinorVersion + "_" + i;
			if(ded.chaseDEDFlag)
				ded.dedName += "DM";
			else
				ded.dedName += "QR";
		}
		
		// hyun 2/27/2008
		// to fix mars specific fix, we add suffix to all constants
		try {
			edu.ucla.cs.db.prism.backend.marsManage.MarsSpecificFix.addSuffixToConstants(composedDEDSet);
		} catch (MarsConstantsSuffixException e) {
			e.printStackTrace();
		}
		
		return composedDEDSet;
	}

	/**
	 * SELECT S0=R0.a0, S1=R0.a1, ..., Sn=R0.an FROM tableName R0;
	 * @param table
	 * @return
	 */
	
	private SQL getCanonicalSQL(Table table) {
		SQL sql = new SQL();

		// step 1: FROM clause
		RelationDeclaration rd = new RelationDeclaration(table.getTableName(), "R0");
		sql.from.relationDeclarations.add(rd);
		
		// step 2: SELECT clause
		for(int i=0; i<table.getNumColumns(); i++) {
			SelectAttribute sa = new SelectAttribute("S" + i, "R0", table.getColumns().get(i));
			sql.select.addSelectAttribute(sa);
		}
		
		return sql;
	}
	
	private DEDSet reverseQRDEDDirectionIntoDM(DED inputDED) {
		DEDSet dedSet = new DEDSet();
		for(int i=0; i<inputDED.thenClauses.size(); i++) {
			QuantifiedDisjunctive inputTC = inputDED.thenClauses.get(i);
			
			DED ded = new DED();
			ded.globalQuantifiers = inputTC.quantifiers.clone();
			
			QuantifiedDisjunctive newTC = new QuantifiedDisjunctive();
			ded.thenClauses.add(newTC);
			newTC.quantifiers = inputDED.globalQuantifiers.clone();
			newTC.disjunctive = inputTC.disjunctive.clone();
			
			// TODO: ded.globalQuantifiers, some to all
			for(int j=0; j<ded.globalQuantifiers.quantifiers.size(); j++)
				ded.globalQuantifiers.quantifiers.get(j).type = "all";
			
			// TODO: newTC.quantifiers, all to some
			for(int j=0; j<newTC.quantifiers.quantifiers.size(); j++)
				newTC.quantifiers.quantifiers.get(j).type = "some";
			
			ded.chaseDEDFlag = true; // DM-direction
			ded.targetRelations = inputDED.sourceRelations;
			ded.sourceRelations = new Vector<String>();
			for(int j=0; j<ded.globalQuantifiers.quantifiers.size(); j++) {
				String sourceRelation = ded.globalQuantifiers.quantifiers.get(j).relation;
				// sourceRelation may contain REF, which we don't want in ded.sourceRelations
				if(!ded.sourceRelations.contains(sourceRelation) && sourceRelation.startsWith("v"))
					ded.sourceRelations.add(RelationDeclaration.versionUntag(sourceRelation));
			}
			
			dedSet.add(ded);
		}
		
		return dedSet;
	}
	
	
	
	private DED getQRDirectionDEDs(SQL inputSQL, SQLUnion outputSQLUnion,
			int sourceVersion, int sourceMinorVersion,
			int targetVersion, int targetMinorVersion) {
		// inputSQL (table2 on target version)
		// SELECT S0=R0.a0, S1=R0.a1
		// FROM table2 R0
		
		// outputSQL (on source version)
		// SELECT S0=R1.a1, S1=R2.a5
		// FROM table5 R0, table3 R1, table4 R2
		// WHERE R0.a0=R1.a0 and R1.a2=R2.a2
		// UNION
		// ...

		// construct ded such that
		// all (R0 in table2)
		// some (S0 in table5) (S1 in table3) (S2 in table4)
		// R0.a0 = S1.a1 (for select S0) and R0.a1 = S2.a5 (for select S1) and  -- select clause
		// S0.a0 = S1.a0 and S1.a2 = S2.a2
		// OR
		// some ...
		
		
		// step 1: construct global quantifier (all) from inputSQL
		//         create sql-alias-to-ded-alias hashtable and use it
		DED ded = new DED();
		Hashtable<String, String> sqlAliasToDEDAlias = new Hashtable<String, String>();
		// there's only one relation declaration here
		String relation = inputSQL.from.relationDeclarations.get(0).relation;
		String sqlAlias = inputSQL.from.relationDeclarations.get(0).relationAlias;
		String dedAlias = "R0";
		sqlAliasToDEDAlias.put("in" + sqlAlias, dedAlias);
		ded.globalQuantifiers.add(new Quantifier("all", dedAlias, relation));
		
		// step 1.1: set target relations
		// TODO
		ded.sourceRelations = new Vector<String>();
		ded.sourceRelations.add(RelationDeclaration.versionUntag(relation));
		
		// step 2.1: if outputSQLUnion is empty (=null), create a false DED
		//           very similar to query a newly created table by CREATE TABLE
		if(outputSQLUnion == null) {
			QuantifiedDisjunctive tc = new QuantifiedDisjunctive();
			tc.disjunctive.conjunctives.add(new Conjunctive()); // one and only one conjunctive
			ded.thenClauses.add(tc);
			tc.disjunctive.conjunctives.get(0).add(
					new Predicate(new Term("\"0\""), "=", new Term("\"1\"")));
			return ded;
		}
		
		// step 2.2: construct one then-clause for each sql in outputSQLUnion
		int dedAliasIndex = 0;
		for(int i=0; i<outputSQLUnion.sqls.size(); i++) {
			QuantifiedDisjunctive tc = new QuantifiedDisjunctive();
			tc.disjunctive.conjunctives.add(new Conjunctive()); // one and only one conjunctive
			ded.thenClauses.add(tc);
			
			
			// step 2.1: construct then-clause quantifiers (some) form outputSQL
			SQL outputSQL = outputSQLUnion.sqls.get(i);
			for(int j=0; j<outputSQL.from.relationDeclarationsSize(); j++) {
				RelationDeclaration rd = outputSQL.from.relationDeclarations.get(j);
				relation = rd.relation;
				sqlAlias = rd.relationAlias;
				dedAlias = "S" + dedAliasIndex++;
				sqlAliasToDEDAlias.put("out" + sqlAlias, dedAlias);
				tc.quantifiers.add(new Quantifier("some", dedAlias, relation));
			}
			
			// step 2.2: construct where clause based on select clause correspondence (do alias renaming)
			for(int j=0; j<inputSQL.select.selectAttributesSize(); j++) {
				SelectAttribute inputSQLSA = inputSQL.select.selectAttributes.get(j);
				SelectAttribute outputSQLSA = outputSQL.select.get(inputSQLSA.outputAttributeName);
				
				Predicate p = new Predicate();
				if(inputSQLSA.relationAlias != null && inputSQLSA.attribute != null)
					p.term1 = new Term(inputSQLSA.relationAlias, inputSQLSA.attribute);
				else
					p.term1 = new Term(inputSQLSA.constant);
				translateRelationAlias(p.term1, sqlAliasToDEDAlias, "in");
				
				if(outputSQLSA.relationAlias != null && outputSQLSA.attribute != null)
					p.term2 = new Term(outputSQLSA.relationAlias, outputSQLSA.attribute);
				else
					p.term2 = new Term(outputSQLSA.constant);
				translateRelationAlias(p.term2, sqlAliasToDEDAlias, "out");

				p.op = "=";
				tc.disjunctive.conjunctives.get(0).add(p);
			}
			
			// step 2.3: construct where clause based on where clause (after alias renaming)
			for(int j=0; 
				outputSQL.where.conjunctives != null &&
				outputSQL.where.conjunctives.size() > 0 &&
				j < outputSQL.where.conjunctives.get(0).size(); j++) {
				
				Predicate sqlP = outputSQL.where.conjunctives.get(0).get(j);
				Predicate dedP = new Predicate(sqlP);
				translateRelationAlias(dedP.term1, sqlAliasToDEDAlias, "out");
				translateRelationAlias(dedP.term2, sqlAliasToDEDAlias, "out");
			}
			
			// step 2.4: set sourceRelations
			if(ded.targetRelations == null)
				ded.targetRelations = new Vector<String>();
			for(int j=0; j<outputSQL.from.relationDeclarationsSize(); j++) {
				String targetRelation = outputSQL.from.relationDeclarations.get(j).relation;
				// sourceRelation may contain REF, which we don't want in ded.sourceRelations
				if(!ded.targetRelations.contains(targetRelation) && targetRelation.startsWith("v"))
					ded.targetRelations.add(RelationDeclaration.versionUntag(targetRelation));
			}
		}		
		
		ded.chaseDEDFlag = false; // QR direction
		return ded;
	}
	
	/**
	 * translate the use of relation alias in term, using hashtable and inOrOut prefix
	 * @param term
	 * @param hashtable
	 * @param inOrOut
	 */
	private void translateRelationAlias(Term term, Hashtable<String, String> hashtable, String inOrOut) {
		if(term == null) return;
		else if(term.isRATerm()) {
			String newRelationAlias = hashtable.get(inOrOut + term.tableName);
			if(newRelationAlias != null)
				term.tableName = newRelationAlias;
		}
		else if(term.isFunctionTerm()) {
			for(int i=0; i<term.functionArgs.size(); i++)
				translateRelationAlias(term.functionArgs.get(i), hashtable, inOrOut);
		}
	}
	
	/**
	 * find the tempSourceVersion that is 
	 * (1) farthest from target version,
	 * (2) closest to source version
	 * (3) reached within n hop (n: given number of dedSets used for one composition)
	 * (4) equal to the source version or between the source and target versions
	 * 
	 * return null if sourceVersion can be reached without any composition (1 hop)
	 * 
	 * @param dedSetHistory
	 * @param targetVersion
	 * @param targetMinorVersion
	 * @param numHopsToCompose
	 * @return
	 * @throws RewritingOptimizationException 
	 */
	private int[] getBestTempSourceVersion(DEDSetHistory dedSetHistory, 
			int sourceVersion, int sourceMinorVersion,
			int targetVersion, int targetMinorVersion, 
			int numHopsToCompose) throws RewritingOptimizationException {
		
		int[] sourceVersionFull = {sourceVersion, sourceMinorVersion};
		int[] targetVersionFull = {targetVersion, targetMinorVersion};
		int hopCount = 0;

		int[] tempSourceVersionFull = targetVersionFull;
		while(true) {
			tempSourceVersionFull
				= PrimRewriteEngine.getBestTempSourceVersion(dedSetHistory,
					sourceVersionFull, tempSourceVersionFull);

			hopCount++;
			if(hopCount == numHopsToCompose)
				break;
			else if(PrimRewriteEngine.compareVersions(
					sourceVersionFull, tempSourceVersionFull) == 0)
				break;
		}
		
		
		if(hopCount <= 1)
			return null;
		else
			return tempSourceVersionFull;
	}

	
}
