package edu.ucla.cs.db.prism.backend.ded;
//import smo.Conjunctive;

import java.util.Vector;
import java.io.*;

public class DED implements Serializable {
	static final long serialVersionUID = 1;
	public String dedName;
	public String dedId;
	public QuantifierSet globalQuantifiers;
	public QuantifiedDisjunctive ifClause;
	public Vector<QuantifiedDisjunctive> thenClauses;
	
	// extra information for unmodified-schema-part optimization
	public boolean chaseDEDFlag; // true if this was generated as a chase rule (not backchase)
								// the following three are meaningful when this is true
	public boolean rootDBFlag = false; // is this DED for root and DB?
	public String tableName; // targetSchema table name
	public String columnName; // targetSchema column name, if this DED is for a column
							  // null, otherwise
	public boolean modifiedFlag; // whether this target schema part was modified between 
								// target schema and source schema
	public boolean islocal = false; //this has been added for the DEDs generated by "local" IC, 
							//such as foreign keys, primary keys, value constraints
	// Added by Hyun (5/3/07) used for PRIMA-R smo analysis and view composition
	public String targetTable = null;					// set when created from smo
	/**
	 * set when created from smo
	 */
	public Vector<String> renamedExpandedSourceTables = null; 
	public boolean finalView = false; 			// determined by analysis that follows
	// ends here
	
	// Added by Hyun (1/23/2008) used for metadata-based DED selection
	public Vector<String> sourceRelations = null;
	public Vector<String> targetRelations = null;
	//
	
	private String dedText = null;
	
	public DED() {
		dedName = "";
		globalQuantifiers = new QuantifierSet();
		ifClause = new QuantifiedDisjunctive();
		thenClauses = new Vector<QuantifiedDisjunctive>();
		chaseDEDFlag = false; // default, this should be changed if it's a chase rule
	}
	
	public DED(String dedName) {
		this.dedName = dedName;
		globalQuantifiers = new QuantifierSet();
		ifClause = new QuantifiedDisjunctive();
		thenClauses = new Vector<QuantifiedDisjunctive>();
		chaseDEDFlag = false; // default, this should be changed if it's a chase rule
	}
	
	public DED(String dedId, String dedName, boolean chaseFlag, Vector<String> sourceTables, Vector<String> targetTables, String dedText) {		
		this.dedId = dedId;
		this.dedName = dedName;
		this.dedText = dedText;
		chaseDEDFlag=chaseFlag;
		this.sourceRelations = sourceTables;
		this.targetRelations = targetTables;
		globalQuantifiers = new QuantifierSet();
		ifClause = new QuantifiedDisjunctive();
		thenClauses = new Vector<QuantifiedDisjunctive>();
	}

	@Override
	public DED clone() {
		DED ded = new DED();
		ded.dedName = new String(this.dedName);
		ded.globalQuantifiers = this.globalQuantifiers.clone();
		ded.ifClause = this.ifClause.clone();
		for(int i=0; i<this.thenClauses.size(); i++)
			ded.thenClauses.add(this.thenClauses.get(i).clone());
		
		ded.chaseDEDFlag = this.chaseDEDFlag;
		ded.tableName = this.tableName;
		ded.columnName = this.columnName;
		ded.modifiedFlag = this.modifiedFlag;
		
		//edu.ucla.db.prim.ded.sourceVariableIndex = this.sourceVariableIndex;
		return ded;
	}
	
	@Override
	public String toString() {
		String str = "";
		if(this.chaseDEDFlag) {
			str += "tableName: " + this.tableName + "\n";
			str += "columnName: " + this.columnName + "\n";
			str += "modifiedFlag: " + this.modifiedFlag + "\n";
		}
		
		// smo-to-view information passing debugging
		if(false) {
			str += "renamedExpandedSouceTables: ";
			if(renamedExpandedSourceTables != null)
				str += renamedExpandedSourceTables.toString() + "\n";
			else
				str += "\n";
			str += "targetTable:\t" + targetTable + "\n";
			str += "finalView:\t" + finalView + "\n";
		}
		
		str += toStringNoMetadata();
		return str;
	}
	
	
	public String toHTMLString() {
		String str = "";		
		
		str += "Constraint " + this.dedName + " :=<br>";

		// to avoid MARS error, replace '-' in the constraint name with '_'
		str = str.replace('-', '_');
		
		
		if(globalQuantifiers.size() > 0)
			str += globalQuantifiers.toString() + "<br>";
		/*
		if(ifClause.size() > 0) {
			str += "if\n";
			str += ifClause.toString();
			str += "then\n";
		}
		*/

		// 10/5/2007 modified to test the existence of the predicates in the only conjunctive under disjunctive
		if(ifClause.disjunctive.conjunctives != null &&
				ifClause.disjunctive.conjunctives.size() > 0 &&
				ifClause.disjunctive.conjunctives.get(0).size() > 0) {
			str += "if<br>";
			str += ifClause.toString();
			str += "then<br>";
		}

		
		// 2/7/2007 no then-clause case - deleted column
		if(thenClauses.size() == 0 && this.columnName != null) {
			str += "\"0\"=\"1\";<br>";
		}
		// do nothing for deleted table - 4/6/2007
		else if(thenClauses.size() == 0 && this.columnName == null) { 
			return "";
		}
		
		// then clauses
		// if there is only one then clause (no OR)
		else if(thenClauses.size() == 1) {
			str += thenClauses.get(0).toString();
			str += ";<br>";
		}
		else {
			for(int i=0; i<thenClauses.size(); i++) {
				str += "(";
				str += thenClauses.get(i).toString();
				str += ")";
				if(i<thenClauses.size()-1)
					str += " OR<br>";
			}
			str += ";<br>";		
		}
		
		return str;
	}
	
	public String toStringNoMetadata() {
		if(dedText !=null)
			return dedText;
		
		String str = "";

		str += "Constraint " + this.dedName + " :=\n";

		// to avoid MARS error, replace '-' in the constraint name with '_'
		str = str.replace('-', '_');
		
		
		if(globalQuantifiers.size() > 0)
			str += globalQuantifiers.toString() + "\n";
		/*
		if(ifClause.size() > 0) {
			str += "if\n";
			str += ifClause.toString();
			str += "then\n";
		}
		*/

		// 10/5/2007 modified to test the existence of the predicates in the only conjunctive under disjunctive
		if(ifClause.disjunctive.conjunctives != null &&
				ifClause.disjunctive.conjunctives.size() > 0 &&
				ifClause.disjunctive.conjunctives.get(0).size() > 0) {
			str += "if\n";
			str += ifClause.toString();
			str += "then\n";
		}

		
		// 2/7/2007 no then-clause case - deleted column
		if(thenClauses.size() == 0 && this.columnName != null) {
			str += "\"0\"=\"1\";\n";
		}
		// do nothing for deleted table - 4/6/2007
		else if(thenClauses.size() == 0 && this.columnName == null) { 
			return "";
		}
		
		// then clauses
		// if there is only one then clause (no OR)
		else if(thenClauses.size() == 1) {
			str += thenClauses.get(0).toString();
			str += ";\n";
		}
		else {
			for(int i=0; i<thenClauses.size(); i++) {
				str += "(";
				str += thenClauses.get(i).toString();
				str += ")";
				if(i<thenClauses.size()-1)
					str += " OR\n";
			}
			str += ";\n";		
		}
		
		return str;
	}
	
	/*
	static public int getNewDBQuantifier(QuantifierSet quantifiers, Conjunctive conjunctive,
			String type, String aliasPrefix, int aliasID, int versionID) {
		String var1 = aliasPrefix + new Integer(aliasID).toString();
		String var2 = aliasPrefix + new Integer(aliasID+1).toString(); // var2.a1 is db variable
		String var3 = aliasPrefix + new Integer(aliasID+2).toString(); 
		quantifiers.add(new Quantifier(type, var1, "v" + versionID + "root"));
		quantifiers.add(new Quantifier(type, var2, "v" + versionID + "child"));
		quantifiers.add(new Quantifier(type, var3, "v" + versionID + "tag"));
		conjunctive.add(new Predicate(var1, "a0", "=", var2, "a0"));
		conjunctive.add(new Predicate(var2, "a1", "=", var3, "a0"));
		conjunctive.add(new Predicate(var3, "a1", "=", "\"db\""));
		return aliasID+3;
	}
	
	//		edu.ucla.db.prim.ded.ifClause.conjunctive.add(new Predicate("R0", "a0", "=", "R1", "a0"));
	
	// dbVariable-table-row
	static public int getNewTableRowQuantifier(QuantifierSet quantifiers, Conjunctive conjunctive,
			String type, String aliasPrefix, int aliasID, int versionID, String dbVariable, String tablename) {
		String var1 = aliasPrefix + new Integer(aliasID).toString(); 
		String var2 = aliasPrefix + new Integer(aliasID+1).toString();
		String var3 = aliasPrefix + new Integer(aliasID+2).toString(); // var3.a1 is row variable
		String var4 = aliasPrefix + new Integer(aliasID+3).toString(); 
		quantifiers.add(new Quantifier(type, var1, "v" + versionID + "child"));
		quantifiers.add(new Quantifier(type, var2, "v" + versionID + "tag"));
		quantifiers.add(new Quantifier(type, var3, "v" + versionID + "child"));
		quantifiers.add(new Quantifier(type, var4, "v" + versionID + "tag"));
		conjunctive.add(new Predicate(dbVariable, "a1", "=", var1, "a0"));
		conjunctive.add(new Predicate(var1, "a1", "=", var2, "a0"));
		conjunctive.add(new Predicate(var2, "a1", "=", "\"" + tablename + "\""));
		conjunctive.add(new Predicate(var1, "a1", "=", var3, "a0"));
		conjunctive.add(new Predicate(var3, "a1", "=", var4, "a0"));
		conjunctive.add(new Predicate(var4, "a1", "=", "\"row\""));
		return aliasID+4;
	}

	// rowVariable-column
	static public int getNewColumnQuantifier(QuantifierSet quantifiers, Conjunctive conjunctive,
			String type, String aliasPrefix, int aliasID, int versionID, String rowVariable, String columnname) {
		String var1 = aliasPrefix + new Integer(aliasID).toString(); // var1.a1 is column variable
		String var2 = aliasPrefix + new Integer(aliasID+1).toString();
		quantifiers.add(new Quantifier(type, var1, "v" + versionID + "child"));
		quantifiers.add(new Quantifier(type, var2, "v" + versionID + "tag"));
		conjunctive.add(new Predicate(rowVariable, "a1", "=", var1, "a0"));
		conjunctive.add(new Predicate(var1, "a1", "=", var2, "a0"));
		conjunctive.add(new Predicate(var2, "a1", "=", "\"" + columnname + "\""));
		return aliasID+2;
	}
	
	// columnVariable-text
	static public int getNewTextQuantifier(QuantifierSet quantifiers, Conjunctive conjunctive,
			String type, String aliasPrefix, int aliasID, int versionID, String colVariable) {
		String var1 = aliasPrefix + new Integer(aliasID).toString(); // var1.a1 is text variable
		quantifiers.add(new Quantifier(type, var1, "v" + versionID + "text"));
		conjunctive.add(new Predicate(colVariable, "a1", "=", var1, "a0"));
		return aliasID+1;
	}
	*/
	
	// by Hyun (5/7/07) originally in PRIMA-R compose.ViewCompose class
	// moved to here to avoid compiler problem - not a very good desigin decision
	// quick and dirty fix
	
	//  for each thenClause of view1 - assume only one thenClause for now
	// assume that Sx.attr = term appears in disjunctive.conjunctives.get(0)

	/* deprecated and moved to Disjunctive
	static public Term getTermEqualToSxAttr(QuantifiedDisjunctive viewThenClause, String Sx, String attr) {
		Term term = null;

		// assume disjunctive.conjunctives[0] has all view-target-defining predicates
		for(int i=0; i<viewThenClause.disjunctive.conjunctives.get(0).size(); i++) {
			Predicate predicate = 
				viewThenClause.disjunctive.conjunctives.get(0).get(i);

			if(predicate.term1.constant == null &&
					predicate.term1.tableName.equals(Sx) &&
					predicate.term1.columnName.equals(attr) &&
					predicate.op.equals("=")) {
				term = predicate.term2;
				break;
			}
			else if(predicate.term2.constant == null &&
					predicate.term2.tableName.equals(Sx) &&
					predicate.term2.columnName.equals(attr) &&
					predicate.op.equals("=")) {
				term = predicate.term1;
				break;
			}					
		}
		
		return term;
	}
	*/
	
	public void versionTag(int version) {
		for(int i=0; i<this.thenClauses.size(); i++) {
			QuantifierSet thenClauseQuantifiers = this.thenClauses.get(i).quantifiers;
			for(int j=0; j<thenClauseQuantifiers.quantifiers.size(); j++)
				thenClauseQuantifiers.quantifiers.get(j).relation =
					"v" + version + thenClauseQuantifiers.quantifiers.get(j).relation;
		}
	}
	
	public void simplifyDisjunctive() {
		ifClause.disjunctive.simplify();
		for(int i=0; i<thenClauses.size(); i++)
			thenClauses.get(i).disjunctive.simplify();
	}

	public boolean isLocal() {
		return islocal;
	}



}

