package edu.ucla.cs.db.prism.backend.ded;

import java.util.List;
import java.util.Vector;
import java.io.*;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.SchemaModifiedInfo;

public class DEDSetHistory implements Serializable {
	static final long serialVersionUID = 1;
	public Vector<DEDSet> dedSets;
	
	public DEDSetHistory() {
		dedSets = new Vector<DEDSet>();
	}

	public DEDSetHistory(List<DEDSet> subList) {
		dedSets = new Vector<DEDSet>();
		for(DEDSet d:subList){
			dedSets.add(d);
		}
	}

	public int size() {
		return dedSets.size();
	}
	
	public boolean add(DEDSet dedSet) {
		return dedSets.add(dedSet);
	}
	
	public DEDSet get(int index) {
		return dedSets.get(index);
	}
	
	
	public DEDSet get(int sourceVersion, int targetVersion) {
		for(int i=0; i<size(); i++)
			if(dedSets.get(i).targetVersion == targetVersion &&
					dedSets.get(i).minorTargetVersion == 0 &&
				dedSets.get(i).sourceVersion == sourceVersion &&
				dedSets.get(i).minorSourceVersion == 0)
				return dedSets.get(i);
		
		return null;
	}

	public DEDSet get(int[] sourceVersion, int[] targetVersion) {
		
		if(sourceVersion == null){
			System.err.println("ERROR in DEDSetHisotry.get(): sourceVersion is null!");
			return null;
		}
		
		if(targetVersion == null){
			System.err.println("ERROR in DEDSetHisotry.get(): sourceVersion is null!");
			return null;
		}		
			
		for(int i=0; i<size(); i++)
			if(dedSets.get(i).targetVersion == targetVersion[0] &&
					dedSets.get(i).minorTargetVersion == targetVersion[1] &&
				dedSets.get(i).sourceVersion == sourceVersion[0] &&
				dedSets.get(i).minorSourceVersion == sourceVersion[1])
				return dedSets.get(i);
		
		return null;
	}
	
	public DEDSet remove(int index) {
		return dedSets.remove(index);
	}
	@Override
	public String toString() {
		String str = "";
		for(int i=0; i<this.size(); i++) {
			str += this.get(i).toString() + "\n";
		}
		return str;
	}
	
	public String toHTMLString() {
		String str = "";
		for(int i=0; i<this.size(); i++) {
			str += this.get(i).toHTMLString() + "\n";
		}
		return str;
	}

	
	public String toStringNoMetadata() {
		return toString();
	}
	// for the purpose of no-change-optimization (2/14/2007)
	/*public Schema getUnmodifiedTargetSchemaPart(int targetVersion, int sourceVersion) {
		DEDSet dedSet = this.get(targetVersion, sourceVersion);
		SchemaModifiedInfo schema = dedSet.getUnmodifiedTargetSchemaPart();
		return schema;
	}*/
	
	public Schema getModifiedTargetSchemaPart(int sourceVersion, int targetVersion) {
		DEDSet dedSet = this.get(sourceVersion, targetVersion);
		SchemaModifiedInfo schema = dedSet.getModifiedTargetSchemaPart();
		return schema;
	}
	
	
	public DEDSet getModifiedTargetSchemaPartDED(int sourceVersion, int targetVersion) {
		DEDSet dedSet = this.get(sourceVersion, targetVersion); 
		
		SchemaModifiedInfo modifiedSchemaPart = dedSet.getModifiedTargetSchemaPart();
		DEDSet newDEDSet = dedSet.getDEDsMacthincTargetSchemaPart(modifiedSchemaPart);
		return newDEDSet;
	}
	
	public Vector<Integer> getAllSourceVersions() {
		Vector<Integer> sourceVersions = new Vector<Integer>();
		for(int i=0; i<dedSets.size(); i++) {
			Integer sourceVersion = new Integer(dedSets.get(i).sourceVersion);
			if(!sourceVersions.contains(sourceVersion))
				sourceVersions.add(sourceVersion);
		}
		
		return sourceVersions;
			
	}
	
	public void simplifyDisjunctive() {
		for(int i=0; i<dedSets.size(); i++) {
			dedSets.get(i).simplifyDisjunctive();
		}
	}

	
}
