package edu.ucla.cs.db.prism.backend.icmo;

import java.util.Iterator;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.ded.DED;
import edu.ucla.cs.db.prism.backend.ded.DEDSet;
import edu.ucla.cs.db.prism.backend.ded.QuantifiedDisjunctive;
import edu.ucla.cs.db.prism.backend.ded.Quantifier;
import edu.ucla.cs.db.prism.backend.exception.SchemaErrorException;
import edu.ucla.cs.db.prism.backend.icmo.exceptions.AddForeignKeyException;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.Table;
import edu.ucla.cs.db.prism.backend.schema.exception.IllegalSMOException;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.exceptions.ExpressionParsignErrorException;
import edu.ucla.cs.db.prism.backend.smo.util.Conjunctive;
import edu.ucla.cs.db.prism.backend.smo.util.Predicate;
import edu.ucla.cs.db.prism.backend.smo.util.Term;

public class AddForeignKey extends ICMOImpl {
	public boolean check = false;
	public boolean enforce = false;	
	public boolean declare = false;
	
	public Vector<String> sourceColumns = null;
	public Vector<String> targetColumns = null;


	
	/**
	 * 
	 * @param id
	 * @param sourceTableName
	 * @param targetTableName
	 * @param sourceColumns
	 * @param targetColumns
	 * @param check
	 * @param enforce
	 * @param declare
	 * @throws AddForeignKeyException
	 */
	public AddForeignKey(String id,  int source_Major, int source_Minor, int target_Major,
			int target_Minor, String sourceTableName, String targetTableName, Vector<String> sourceColumns, Vector<String> targetColumns, boolean check, boolean enforce, boolean declare) throws AddForeignKeyException{
		super(id,source_Major,source_Minor,target_Major,target_Minor);
		this.tableName = sourceTableName;	
		this.tableName2 = targetTableName;
		this.sourceColumns = sourceColumns;
		this.targetColumns = targetColumns;
		
		if(check && !enforce &&! declare){
			this.check = true;
			return;
		}	
		if(!check && enforce &&! declare){
			this.enforce = true;
			return;
		}	
		if(!check && !enforce && declare){
			this.declare = true;
			return;
		}
		throw new AddForeignKeyException("The AddForeignKey exception cannot be check,enforce and declare at the same time");
	
	}
	
public AddForeignKey(String id,String sourceTableName, String targetTableName, Vector<String> sourceColumns, Vector<String> targetColumns,String type) throws  AddForeignKeyException{
		
		this.id = id;
		this.tableName = sourceTableName;	
		this.tableName2 = targetTableName;
		this.sourceColumns = sourceColumns;
		this.targetColumns = targetColumns;
		
		if(type.equals(ICMOInterface.CHECK)){
			this.check = true;
			return;
		}	
		if(type.equals(ICMOInterface.ENFORCE)){
			this.enforce = true;
			return;
		}	
		if(type.equals(ICMOInterface.DECLARE)){
			this.declare = true;
			return;
		}
		throw new AddForeignKeyException("The AddForeignKey exception type not recognized!");
	
	}
	
	
	
	
	public AddForeignKey(String ICMOType) {
	// TODO Auto-generated constructor stub
		super(ICMOType);
}

	public void setCheck(){
		check = true;
		enforce = false;	
		declare = false;
	}
	
	public void setEnforce(){
		check = false;
		enforce = true;	
		declare = false;
	}
	
	public void setDeclare(){
		check = false;
		enforce = true;	
		declare = false;
	}
	
	public boolean isCheck(){
		return check;
	}
	
	public boolean isEnforce(){
		return enforce;
	}
	
	public boolean isDeclare(){
		return declare;
	}

	public Vector<String> getTargetColumns() {
		return targetColumns;
	}


	public Vector<String> getSourceColumns() {
		return sourceColumns;
	}

	@Override
	public ICMOSMOSequence getBestCandidateInverses(Schema inputSchema)
			throws SchemaErrorException {

		if(!(inputSchema instanceof ICSchema))
			throw new SchemaErrorException("You are trying to add constraints to a non-ic-aware Schema object");
		
		
		DropForeignKey dvc = new DropForeignKey(this.id,this.getTargetMajor(),this.getTargetMinor(),this.getSourceMajor(),this.getSourceMinor(),this.tableName);
		dvc.setSourceMajor(this.getTargetMajor());
		dvc.setSourceMinor(this.getTargetMinor());
		dvc.setTargetMajor(this.getSourceMajor());
		dvc.setTargetMinor(this.getSourceMinor());

		ICMOSMOSequence iss = new ICMOSMOSequence();
		iss.sourceVersion = this.getTargetMajor();
		iss.targetVersion = this.getSourceMajor();
		
		iss.getMos().add(dvc);
		return iss;
	}
	
	
	@Override
	public String toString(){
		
		String t= "ALTER TABLE " +tableName + " ADD FOREIGN KEY \"" + this.id + "\" (";
		
		
		for(int i =0; i< sourceColumns.size(); i++){
			t+= sourceColumns.elementAt(i);
			if(i<sourceColumns.size()-1)
				t+= ",";
		}
		
		t += ") REFERENCES " + tableName2 + "(";
		
		for(int i =0; i< targetColumns.size(); i++){
			t+= targetColumns.elementAt(i);
			if(i<targetColumns.size()-1)
				t+= ",";
		}
		t += ")";
	
		if(check){
			t+=" " + ICMOInterface.CHECK +"\n";
		}
		if(enforce){
			t+=" " + ICMOInterface.ENFORCE+ "\n";
		}
		if(declare){
			t+=" " + ICMOInterface.DECLARE + "\n";
		}

		return t;
		
	}
	
	

	@Override
	public DEDSet getAllDEDs(Schema inputSchema) throws SchemaErrorException{
		
		if(!(inputSchema instanceof ICSchema))
			throw new SchemaErrorException("You are trying to apply an ICMO on a Schema.java object");
		DEDSet ds = new DEDSet(this.getSourceMajor(),this.getSourceMinor(),this.getTargetMajor(),this.getTargetMinor());
				
		if(declare){
			ds.addAll(getIdentityDEDs((ICSchema)inputSchema,null));			
			return ds;
		}
		if(check){
			ds.addAll(getIdentityDEDs((ICSchema)inputSchema,null));			
			return ds;
		}
		if(enforce){
			
			Vector<String> excludeList = new Vector<String>();
			excludeList.add(getTableName());
			excludeList.add(this.getTableName()+"__viol_"+this.id); 
			
			ds = getDEDs(((ICSchema)inputSchema).clone());
			ds.addAll(getIdentityDEDs((ICSchema)inputSchema,excludeList));
			
			// FIXME implement the enforce case
			return ds;
		}
		return ds;
	}
	

	public DEDSet getDEDs(ICSchema inputSchema) {
		DEDSet ds = new DEDSet(getSourceMajor(),getSourceMinor(),getTargetMajor(), getTargetMinor());
		
		String dedName = "ADD_FK_" + getTableName() + "_" + getTableName2();	
		Vector<String> sourceTables = new Vector<String>();
		sourceTables.add(getTableName());
		sourceTables.add(getTableName2());
		
		
		Vector<String> targetTables = new Vector<String>();
		targetTables.add(getTableName());
		targetTables.add(getTableName2());
		
		
		targetTables.add(this.getTableName()+"__viol_"+this.getTargetMajor()+"_"+this.getTargetMinor());
	
		genDed(inputSchema.clone(), ds, id, dedName, false, sourceTables,targetTables);
		genDed(inputSchema.clone(), ds, id, dedName, true,targetTables,sourceTables);
		
		return ds;
	}

	private void genDed(Schema inputSchema, DEDSet ds, String dedId,
			String dedName, boolean chaseFlag, Vector<String> sourceTables,
			Vector<String> targetTables) {
		
		if(chaseFlag)
			dedName += "_dm"; 
		else
			dedName += "_qr"; 
		
		DED d = new DED(dedId, dedName, chaseFlag, sourceTables, targetTables, null);
				
		Quantifier q1,q2 = null;
		
		if(chaseFlag){			 
			 q1 = new Quantifier("all", "R0", "v" + getSourceMajor() + "m" +getSourceMinor() + "_" + getTableName());
			 q2 = new Quantifier("all", "R1", "v" + getSourceMajor() + "m" +getSourceMinor() + "_" + getTableName2());
			 d.globalQuantifiers.quantifiers.add(q1);
			 d.globalQuantifiers.quantifiers.add(q2);

		}
		 else{
			 q1 = new Quantifier("all", "R0", "v" + getTargetMajor() + "m" +getTargetMinor() + "_" + getTableName());
  			 d.globalQuantifiers.quantifiers.add(q1);
		}		
		
		QuantifiedDisjunctive tc1 = new QuantifiedDisjunctive(); 
		d.thenClauses.add(tc1);

		if(chaseFlag){

			q1 = new Quantifier("some", "R2", "v" + getTargetMajor() + "m" + getTargetMinor() + "_" + getTableName());
			tc1.quantifiers.add(q1);
		
		}
		else{	
			q1 = new Quantifier("some", "R1", "v" + getSourceMajor() + "m" + getSourceMinor() + "_" + getTableName());
			q2 = new Quantifier("some", "R2", "v" + getSourceMajor() + "m" + getSourceMinor() + "_" + getTableName2());
			tc1.quantifiers.add(q1);
			tc1.quantifiers.add(q2);			
		}
		
		Conjunctive c1 = new Conjunctive();
		tc1.disjunctive.add(c1);
		c1.predicates = new Vector<Predicate>();
		
		Table in1 = inputSchema.getTable(getTableName());
		Table in2 = inputSchema.getTable(getTableName2());
		try {
			inputSchema.applyOperator(this);
		} catch (IllegalSMOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ExpressionParsignErrorException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SchemaErrorException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Table outTab = inputSchema.getTable(getTableName());
				
		Conjunctive c3 = new Conjunctive();
		
		if(chaseFlag){
			for(int i =0; i<sourceColumns.size();i++){
					c3.predicates.add(new Predicate(new Term("R0",sourceColumns.elementAt(i)),"=",new Term("R1",targetColumns.elementAt(i))));
			}
			d.ifClause.disjunctive.add(c3);			
		}
		else{
			for(int i =0; i<sourceColumns.size();i++){
				c1.predicates.add(new Predicate(new Term("R1",sourceColumns.elementAt(i)),"=",new Term("R2",targetColumns.elementAt(i))));
			}
		}
		
		
		
		Iterator<String> inColumns1 = in1.getColumns().iterator();
		
		while(inColumns1.hasNext()){
			String inCol = inColumns1.next();
			Iterator<String> outColumns = outTab.getColumns().iterator();
			while(outColumns.hasNext()){
				String outCol = outColumns.next();
				if(outCol.equalsIgnoreCase(inCol)){
					Predicate p1 = new Predicate();
					c1.predicates.add(p1);
					if(chaseFlag){
						p1.term1 = new Term("R0", inCol);
						p1.term2 = new Term("R2", outCol);
					}else{
						p1.term1 = new Term("R0", inCol);
						p1.term2 = new Term("R1", outCol);			
					}
					
					
					p1.op = "=";
				}
			}
		}	
	
		Iterator<String> inColumns2 = in2.getColumns().iterator();
		while(inColumns2.hasNext()){
			String inCol = inColumns2.next();
			Iterator<String> outColumns = outTab.getColumns().iterator();
			while(outColumns.hasNext()){
				String outCol = outColumns.next();
				if(outCol.equalsIgnoreCase(inCol)){
					Predicate p1 = new Predicate();
					c1.predicates.add(p1);
					if(chaseFlag){
						p1.term1 = new Term("R1", inCol);
						p1.term2 = new Term("R2", outCol);
					}else{
						p1.term1 = new Term("R0", inCol);
						p1.term2 = new Term("R2", outCol);						
					}
					
					p1.op = "=";
				}
			}
		}	
				
		ds.add(d);
	}


	
	
	
	
	
	
	
	@Override
	public boolean isInfPreserving(Schema currentSchema) {
		if(check)
			return true;
		if(enforce)
			return false;
		if(declare)
			return false;
		return false;
		
	}
	
	
	
	
	
	@Override
	public String getOperatorType() {
		if(check)
			return ICMOInterface.ADD_FOREIGN_KEY + " " + ICMOInterface.CHECK;
		if(enforce)
			return ICMOInterface.ADD_FOREIGN_KEY + " " + ICMOInterface.ENFORCE;
		if(declare)
			return ICMOInterface.ADD_FOREIGN_KEY + " " + ICMOInterface.DECLARE;
		return null;
	}
	
	
	
	
	
	
	
	
	
	
	
	
}
