package edu.ucla.cs.db.prism.backend.icmo;

import java.util.Vector;

import edu.ucla.cs.db.prism.backend.SQL.RelationDeclaration;
import edu.ucla.cs.db.prism.backend.SQL.SQL;
import edu.ucla.cs.db.prism.backend.ded.DEDSet;
import edu.ucla.cs.db.prism.backend.exception.SchemaErrorException;
import edu.ucla.cs.db.prism.backend.icmo.exceptions.AddPrimaryKeyException;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequence;

public class AddPrimaryKey extends ICMOImpl {

	public boolean check = false;
	public boolean enforce = false;	
	public boolean declare = false;
	
	
	public Vector<String> columns = null;

	public AddPrimaryKey(String id, int i, int j, int k, int l, String tableName, Vector<String> columns,boolean check, boolean enforce, boolean declare) throws AddPrimaryKeyException{
		
		super(id,i,j,k,l);
		this.tableName = tableName;		
		this.columns = columns;
		
		if(check && !enforce &&! declare){
			this.check = true;
			return;
		}	
		if(!check && enforce &&! declare){
			this.enforce = true;
			return;
		}	
		if(!check && !enforce && declare){
			this.declare = true;
			return;
		}
		throw new AddPrimaryKeyException("The AddPrimaryKey exception cannot be check,enforce and declare at the same time");
	
	}
	
public AddPrimaryKey(String id, String tableName, Vector<String> columns,String type) throws AddPrimaryKeyException{
		
		this.id = id;
		this.tableName = tableName;	
		this.columns = columns;
		if(type.equals(ICMOInterface.CHECK)){
			this.check = true;
			return;
		}	
		if(type.equals(ICMOInterface.ENFORCE)){
			this.enforce = true;
			return;
		}	
		if(type.equals(ICMOInterface.DECLARE)){
			this.declare = true;
			return;
		}
		throw new AddPrimaryKeyException("The AddPrimaryKey exception type not recognized!");
	
	}
	
	public AddPrimaryKey(String ICMOType) {
		super(ICMOType);
	// TODO Auto-generated constructor stub
}

	public void setCheck(){
		check = true;
		enforce = false;	
		declare = false;
	}
	
	public void setEnforce(){
		check = false;
		enforce = true;	
		declare = false;
	}
	
	public void setDeclare(){
		check = false;
		enforce = true;	
		declare = false;
	}
	
	public boolean isCheck(){
		return check;
	}
	
	public boolean isEnforce(){
		return enforce;
	}
	
	public boolean isDeclare(){
		return declare;
	}

	public Vector<String> getColumns() {
		return columns;
	}

	@Override
	public SMOSequence getBestCandidateInverses(Schema inputSchema)
			throws SchemaErrorException {

		if(!(inputSchema instanceof ICSchema))
			throw new SchemaErrorException("You are trying to add constraints to a non-ic-aware Schema object");
		
		
		DropPrimaryKey dvc = new DropPrimaryKey(this.id,this.getTargetMajor(),this.getTargetMinor(),this.getSourceMajor(),this.getSourceMinor(), this.tableName);
		dvc.setSourceMajor(this.getTargetMajor());
		dvc.setSourceMinor(this.getTargetMinor());
		dvc.setTargetMajor(this.getSourceMajor());
		dvc.setTargetMinor(this.getSourceMinor());

		ICMOSMOSequence iss = new ICMOSMOSequence();
		iss.sourceVersion = this.getTargetMajor();
		iss.targetVersion = this.getSourceMajor();
		
		iss.getMos().add(dvc);
		return iss;
	}
	
	@Override
	public String toString(){
		
		String t= "ALTER TABLE " +tableName + " ADD PRIMARY KEY \"" + this.id + "\" (";
		
		if(columns!=null){
			for(int i =0; i< columns.size(); i++){
				t+= columns.elementAt(i);
				if(i<columns.size()-1)
					t+= ",";
			}
		}else{
			t+="null";
		}
		
		t += ")";
		
		if(check){
			t+=" " + ICMOInterface.CHECK +"\n";
		}
		if(enforce){
			t+=" " + ICMOInterface.ENFORCE+ "\n";
		}
		if(declare){
			t+=" " + ICMOInterface.DECLARE + "\n";
		}

		return t;
		
	}
	
	
	
	@Override
	public DEDSet getAllDEDs(Schema inputSchema) throws SchemaErrorException{
		
		if(!(inputSchema instanceof ICSchema))
			throw new SchemaErrorException("You are trying to apply an ICMO on a Schema object, use an ICSchema object instead");
		
		DEDSet ds = new DEDSet(this.getSourceMajor(),this.getSourceMinor(),this.getTargetMajor(),this.getTargetMinor());
				
		if(declare){
			ds.addAll(getIdentityDEDs((ICSchema) inputSchema,null));			
			return ds;
		}
		if(check){
			ds.addAll(getIdentityDEDs((ICSchema) inputSchema,null));			
			return ds;
		}
		if(enforce){	
		
			// enforce is managed otherwise. The ds are empty
			return ds;	
			//throw new SchemaErrorException("Not implemented ENFORCE for PRIMARY KEY yet");
			
			// violationtablename = this.getTableName() +"__viol_"+this.id;
		}
		return ds;
	}
	
	
	@Override
	public boolean isInfPreserving(Schema currentSchema) {
		if(check)
			return true;
		if(enforce)
			return false;
		if(declare)
			return false;
		return false;
		
	}
	
	@Override
	public String getOperatorType() {
		if(check)
			return ICMOInterface.ADD_PRIMARY_KEY + " " + ICMOInterface.CHECK;
		if(enforce)
			return ICMOInterface.ADD_PRIMARY_KEY + " " + ICMOInterface.ENFORCE;
		if(declare)
			return ICMOInterface.ADD_PRIMARY_KEY + " " + ICMOInterface.DECLARE;
		return null;
	}
	
	public String rewriteQuery(String inputquery, Schema inputSchema){
		
		SQL outputquery = new SQL(inputquery);
		
		String alias = "";
		for(RelationDeclaration r : outputquery.from.getRelationDeclarationsByRelation(tableName))
			if(r.relation.equals(tableName))
				alias = r.relationAlias;
		
		String postfix ="";
		if(outputquery.where == null || outputquery.where.size()==0)
			postfix = " NOT EXISTS (\n\tSELECT " + alias +"_2." + this.columns.get(0) + " \n\tFROM " + this.tableName + " " + alias +"_2 \n\tWHERE ";
		else
			postfix = " AND NOT EXISTS (\n\tSELECT " + alias +"_2." + this.columns.get(0) + " \n\tFROM " + this.tableName + " " + alias +"_2 \n\tWHERE ";
		
		if(outputquery.getWhereClause() == null || outputquery.getWhereClause().countPredicates()<1)
			postfix = " WHERE " + postfix;
		
		String keycond ="";
		String nonkeycond="";
		for(String col:inputSchema.getTable(tableName).getColumns()){
			
			if(columns.contains(col))
				keycond += alias +"."+col+ " = " + alias +"_2."+col + " AND ";
			else 
				nonkeycond += alias +"."+col+ " != " + alias +"_2."+col + " OR ";
		}
		
		nonkeycond = nonkeycond.substring(0,nonkeycond.length()-3);
		
		return outputquery.toViewString(null) + postfix + keycond + "\n\t"+nonkeycond + "\n\t);";
		
	}
}
