package edu.ucla.cs.db.prism.backend.icmo;

import java.util.Iterator;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.ded.DED;
import edu.ucla.cs.db.prism.backend.ded.DEDSet;
import edu.ucla.cs.db.prism.backend.ded.QuantifiedDisjunctive;
import edu.ucla.cs.db.prism.backend.ded.Quantifier;
import edu.ucla.cs.db.prism.backend.exception.SchemaErrorException;
import edu.ucla.cs.db.prism.backend.icmo.exceptions.AddValueConstraintException;
import edu.ucla.cs.db.prism.backend.marsManage.exception.MarsConstantsSuffixException;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.Table;
import edu.ucla.cs.db.prism.backend.schema.exception.IllegalSMOException;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ComplexValueCosntraintNotImplementedException;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ValueConstraint;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.exceptions.ExpressionParsignErrorException;
import edu.ucla.cs.db.prism.backend.smo.util.Conjunctive;
import edu.ucla.cs.db.prism.backend.smo.util.Disjunctive;
import edu.ucla.cs.db.prism.backend.smo.util.Predicate;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequence;
import edu.ucla.cs.db.prism.backend.smo.util.Term;
import edu.ucla.cs.db.prism.backend.util.REFConstant;

public class AddValueConstraint extends ICMOImpl {
	public boolean check = false;
	public boolean enforce = false;	
	public boolean declare = false;
	
	public String condition = null;
	
	public AddValueConstraint(String id, int i, int j, int k, int l, String tableName, String condition,boolean check, boolean enforce, boolean declare) throws AddValueConstraintException{
		super(id,i,j,k,l);
		this.tableName = tableName;		
		this.condition = condition;
		
		if(check && !enforce &&! declare){
			this.check = true;
			return;
		}	
		if(!check && enforce &&! declare){
			this.enforce = true;
			return;
		}	
		if(!check && !enforce && declare){
			this.declare = true;
			return;
		}
		throw new AddValueConstraintException("The AddValueConstraint exception cannot be check,enforce and declare at the same time");
	
	}
	
public AddValueConstraint(String id, String tableName, String condition,String type) throws AddValueConstraintException{
		this.id = id;
		this.tableName = tableName;	
		this.condition = condition;

		if(type.equals(ICMOInterface.CHECK)){
			this.check = true;
			return;
		}	
		if(type.equals(ICMOInterface.ENFORCE)){
			this.enforce = true;
			return;
		}	
		if(type.equals(ICMOInterface.DECLARE)){
			this.declare = true;
			return;
		}
		throw new AddValueConstraintException("The AddValueConstraint exception type not recognized!");
	
	}
	
	
	public AddValueConstraint(String addValueConstraint) {
	// TODO Auto-generated constructor stub
		super(addValueConstraint);
}

	public void setCheck(){
		
		check = true;
		enforce = false;	
		declare = false;
	}
	
	public void setEnforce(){
		check = false;
		enforce = true;	
		declare = false;
	}
	
	public void setDeclare(){
		check = false;
		enforce = true;	
		declare = false;
	}
	
	public boolean isCheck(){
		return check;
	}
	
	public boolean isEnforce(){
		return enforce;
	}
	
	public boolean isDeclare(){
		return declare;
	}

	public String getCondition() {
		return condition;
	}

	public void setCondition(String condition) {
		this.condition = condition;
	}
	
	
	public DEDSet getAllDEDs(ICSchema inputSchema) throws ExpressionParsignErrorException, ComplexValueCosntraintNotImplementedException{
		
		DEDSet ds = new DEDSet(this.getSourceMajor(),this.getSourceMinor(),this.getTargetMajor(),this.getTargetMinor());
				
		if(declare){
			ds.addAll(getIdentityDEDs(inputSchema,null));			
			return ds;
		}
		if(check){
			ds.addAll(getIdentityDEDs(inputSchema,null));			
			return ds;
		}
		if(enforce){
			
			Vector<String> vc = new Vector<String>();
			vc.add(this.getTableName());
			ds.addAll(getIdentityDEDs(inputSchema,vc));			
			
			
			String dedId = null;
			
			Vector<String> sourceTables = new Vector<String>();
			sourceTables.add(this.getTableName());
			Vector<String> targetTables = new Vector<String>();
			targetTables.add(this.getTableName());
	
			
			genDed(inputSchema.clone(), ds, null, id, true, sourceTables,
					targetTables, getTableName(),new Disjunctive(ValueConstraint.createPredicateFromString(condition)));

			genDed(inputSchema.clone(), ds, null, id, false, targetTables, sourceTables,
					getTableName(),new Disjunctive(ValueConstraint.createPredicateFromString(condition)));
		
			
			targetTables = new Vector<String>();
			targetTables.add(this.getTableName()+"__viol_"+this.id);
		
			genDed(inputSchema.clone(), ds, null, id, true, sourceTables,
					targetTables, this.getTableName()+"__viol_"+this.id,
					new Disjunctive(ValueConstraint.negatedPredicate(ValueConstraint.createPredicateFromString(condition))));


			genDed(inputSchema.clone(), ds, null, id, false, sourceTables,
					targetTables, this.getTableName()+"__viol_"+this.id,
					new Disjunctive(ValueConstraint.negatedPredicate(ValueConstraint.createPredicateFromString(condition))));

			// hyun 2/27/2008
			// to fix mars specific fix, we add suffix to all constants
			try {
				edu.ucla.cs.db.prism.backend.marsManage.MarsSpecificFix.addSuffixToConstants(ds);
			} catch (MarsConstantsSuffixException e) {
				e.printStackTrace();
			}

			return ds;
			
		}
		
		
		return ds;
		
	}
	
	
	

	private void genDed(Schema inputSchema, DEDSet ds, String dedId,
			String dedName, boolean chaseFlag, Vector<String> sourceTables,
			Vector<String> targetTables,  String tt, Disjunctive cond) {
	
		dedName += "_" + tt;
		
		if(chaseFlag)
			dedName += "_dm"; 
		else
			dedName += "_qr"; 
		
		DED d = new DED(dedId, dedName, chaseFlag, sourceTables, targetTables, null);
		
		Quantifier q1 = null;
		
		if(chaseFlag)
			 q1 = new Quantifier("all", "R0", "v" + getSourceMajor() + "m" +getSourceMinor() + "_" + getTableName());
		else
			 q1 = new Quantifier("all", "R0", "v" + getTargetMajor() + "m" +getTargetMinor() + "_" + tt);
				
		d.globalQuantifiers.quantifiers.add(q1);
		
		QuantifiedDisjunctive tc1 = new QuantifiedDisjunctive(); 
		d.thenClauses.add(tc1);

		if(chaseFlag)
			q1 = new Quantifier("some", "R1", "v" + getTargetMajor() + "m" + getTargetMinor() + "_" + tt);
		else
			q1 = new Quantifier("some", "R1", "v" + getSourceMajor() + "m" + getSourceMinor() + "_" + getTableName());
		
		tc1.quantifiers.add(q1);
		
		Conjunctive c1 = new Conjunctive();
		tc1.disjunctive.add(c1);
		c1.predicates = new Vector<Predicate>();
		
		Table inTable = inputSchema.getTable(getTableName());
		try {
			inputSchema.applyOperator(this);
		} catch (IllegalSMOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ExpressionParsignErrorException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (SchemaErrorException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Table outputTable = inputSchema.getTable(tt);
		
		Iterator<String> inColumns = inTable.getColumns().iterator();
		Iterator<String> outColumns = outputTable.getColumns().iterator();
		
		while(inColumns.hasNext()){
			String inCol = inColumns.next();
			String outCol = outColumns.next();
			
			Predicate p1 = new Predicate();
			c1.predicates.add(p1);
			p1.term1 = new Term("R0", inCol);
			p1.term2 = new Term("R1", outCol);
			p1.op = "=";
		}
		if(chaseFlag){
		Iterator<Conjunctive> condIt = cond.conjunctives.iterator();
			while(condIt.hasNext()){
				Conjunctive conCond = condIt.next();
				Iterator<Predicate> cjIt = conCond.predicates.iterator();
				int index = 0;
				while(cjIt.hasNext()){
					Predicate pr = cjIt.next();
					
					boolean switched = false;
					if(pr.op.equals(">")) { // reverse disequalities for uniformity
						switched = true;
						pr.op = "<";
						Term term1 = pr.term1;
						pr.term1 = pr.term2;
						pr.term2 = term1;
					}
					
					if(pr.op.equalsIgnoreCase("<")){ // inequalities
						String relation = REFConstant.getREF(2, getTargetMajor(), getTargetMinor(), "less");
						String relationAlias = "y" + index;
						index++;
						q1 = new Quantifier("some",relationAlias ,  relation);
						tc1.quantifiers.add(q1);
						
						
						if(pr.term2.constant!=null || pr.term1.constant!=null){
							
							if(!switched){
								
								Term temp = pr.term2.clone();
								if(!temp.constant.startsWith("\""))	
									temp.constant = "\"" + temp.constant  + "\"";
	
								c1.predicates.add(new Predicate(new Term(relationAlias, "a0"), "=", new Term("R0",pr.term1.columnName)));
								c1.predicates.add(new Predicate(new Term(relationAlias, "a1"), "=",temp));
							}
							else{
								Term temp = pr.term1.clone();
								if(!temp.constant.startsWith("\""))	
									temp.constant = "\"" + temp.constant  + "\"";
	
								c1.predicates.add(new Predicate(new Term(relationAlias, "a0"), "=", new Term("R0",pr.term2.columnName)));
								c1.predicates.add(new Predicate(new Term(relationAlias, "a1"), "=",temp));			
							}
						}
						
						else{
							c1.predicates.add(new Predicate(new Term(relationAlias, "a0"), "=", new Term("R0",pr.term1.columnName)));
							c1.predicates.add(new Predicate(new Term(relationAlias, "a1"), "=", new Term("R0",pr.term2.columnName)));
						}
					
					
					}else{ // for normal equalities
						if(pr.term2.constant!=null){
							Term temp = pr.term2.clone();
							if(!temp.constant.startsWith("\""))	
								temp.constant = "\"" + temp.constant  + "\"";
							c1.predicates.add(new Predicate(new Term("R0",pr.term1.columnName),pr.op, temp));
						}
						else
							c1.predicates.add(new Predicate(new Term("R0",pr.term1.columnName),pr.op,pr.term2));
						}
					}
			}
		}
		ds.add(d);
	}
	
	
	@Override
	public SMOSequence getBestCandidateInverses(Schema inputSchema) throws SchemaErrorException {


		if(!(inputSchema instanceof ICSchema))
			throw new SchemaErrorException("You are trying to add constraints to a non-ic-aware Schema object");
		
		
		DropValueConstraint dvc = new DropValueConstraint(this.id,this.getTargetMajor(),this.getTargetMinor(),this.getSourceMajor(),this.getSourceMinor(), this.tableName);
		dvc.setSourceMajor(this.getTargetMajor());
		dvc.setSourceMinor(this.getTargetMinor());
		dvc.setTargetMajor(this.getSourceMajor());
		dvc.setTargetMinor(this.getSourceMinor());

		ICMOSMOSequence iss = new ICMOSMOSequence();
		iss.sourceVersion = this.getTargetMajor();
		iss.targetVersion = this.getSourceMajor();
		
		iss.getMos().add(dvc);
		return iss;
	}

	
	@Override
	public String toString(){
		
		String t= "ALTER TABLE " +tableName + " ADD VALUE CONSTRAINT \"" + this.id + "\" " + this.condition;
		
		if(check){
			t+=" " + ICMOInterface.CHECK +"\n";
		}
		if(enforce){
			t+=" " + ICMOInterface.ENFORCE+ "\n";
		}
		if(declare){
			t+=" " + ICMOInterface.DECLARE + "\n";
		}

		return t;
		
	}
	
	@Override
	public boolean isInfPreserving(Schema currentSchema) {
		if(check)
			return true;
		if(enforce)
			return false;
		if(declare)
			return false;
		return false;
		
	}
	
	@Override
	public String getOperatorType() {
		if(check)
			return ICMOInterface.ADD_VALUE_CONSTRAINT + " " + ICMOInterface.CHECK;
		if(enforce)
			return ICMOInterface.ADD_VALUE_CONSTRAINT + " " + ICMOInterface.ENFORCE;
		if(declare)
			return ICMOInterface.ADD_VALUE_CONSTRAINT + " " + ICMOInterface.DECLARE;
		return null;
	}
	
	
	
	
}
