package edu.ucla.cs.db.prism.backend.icmo;

import java.util.Vector;

import edu.ucla.cs.db.prism.backend.exception.SchemaErrorException;
import edu.ucla.cs.db.prism.backend.icmo.exceptions.AddForeignKeyException;
import edu.ucla.cs.db.prism.backend.icmo.exceptions.AddPrimaryKeyException;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ForeignKey;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequence;

public class DropForeignKey extends ICMOImpl {

	
	public DropForeignKey(String id, int i, int j, int k, int l, String tableName) {
		super(id,i,j,k,l);
		this.tableName = tableName;
	}
	
	
	public DropForeignKey(String dropForeignKey) {
		// TODO Auto-generated constructor stub
		super(dropForeignKey);
	}


	@Override
	public SMOSequence getBestCandidateInverses(Schema inputSchema)
			throws SchemaErrorException, AddPrimaryKeyException, DropForeignKeyException, AddForeignKeyException {
		if(!(inputSchema instanceof ICSchema))
			throw new SchemaErrorException("You are trying to add constraints to a non-ic-aware Schema object");
		
		Vector<ForeignKey> vvc = ((ICSchema)inputSchema).getTable(this.getTableName()).getForeignKeys();
		ForeignKey target = null;
		for(ForeignKey fk :vvc){
			
			if(fk.getId().equals(this.id))
				target = fk;
			
		}
		
		if(target == null)
			throw new DropForeignKeyException("You are trying to drop an unexisitng ForeignKey");
		
		AddForeignKey dvc = new AddForeignKey(this.id,this.tableName,target.getTargetTable(),target.getSourceColumns(),target.getTargetColumns(),ICMOInterface.CHECK);
		dvc.setSourceMajor(this.getTargetMajor());
		dvc.setSourceMinor(this.getTargetMinor());
		dvc.setTargetMajor(this.getSourceMajor());
		dvc.setTargetMinor(this.getSourceMinor());

		ICMOSMOSequence iss = new ICMOSMOSequence();
		iss.sourceVersion = this.getTargetMajor();
		iss.targetVersion = this.getSourceMajor();
		
		iss.getMos().add(dvc);
		return iss;
	}

	
	@Override
	public String toString(){
		
		return "ALTER TABLE " +tableName + " DROP FOREIGN KEY \"" + id + "\"\n";
		
	}
	
	@Override
	public boolean isInfPreserving(Schema currentSchema) {
		return true;
	}
	
	@Override
	public String getOperatorType() {
		return ICMOInterface.DROP_FOREIGN_KEY;
	}
	
	
}
