package edu.ucla.cs.db.prism.backend.icmo;

import java.util.Vector;

import edu.ucla.cs.db.prism.backend.exception.SchemaErrorException;
import edu.ucla.cs.db.prism.backend.icmo.exceptions.AddValueConstraintException;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ValueConstraint;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequence;

public class DropValueConstraint extends ICMOImpl {
	
	public DropValueConstraint(String id,int i, int j, int k, int l, String tableName) {
		super(id,i,j,k,l);
		this.tableName = tableName;
	}

	public DropValueConstraint(String dropValueConstraint) {
		// TODO Auto-generated constructor stub
		super(dropValueConstraint);
	}

	@Override
	public SMOSequence getBestCandidateInverses(Schema inputSchema) throws SchemaErrorException {
		
		if(!(inputSchema instanceof ICSchema))
			throw new SchemaErrorException("You are trying to add constraints to a non-ic-aware Schema object");
		Vector<ValueConstraint> vvc = ((ICSchema)inputSchema).getTable(this.getTableName()).getValueConstraint();
		ValueConstraint target = null;
		for(ValueConstraint vc : vvc){
			if(vc.getId().equals(this.id)){
				target = vc;
			}
			
		}
		
		//FIXME pay attention, here we determine the default inverse, including the ENFORCE/CHECK/DECLARE semantics
		AddValueConstraint dvc = null;
		try {
			dvc = new AddValueConstraint(target.getId(), this.getTableName(), target.getAsPredicate().toString(),ICMOInterface.CHECK);
		} catch (AddValueConstraintException e) {
			e.printStackTrace();
		}
		dvc.setSourceMajor(this.getTargetMajor());
		dvc.setSourceMinor(this.getTargetMinor());
		dvc.setTargetMajor(this.getSourceMajor());
		dvc.setTargetMinor(this.getSourceMinor());

		ICMOSMOSequence iss = new ICMOSMOSequence();
		iss.sourceVersion = this.getTargetMajor();
		iss.targetVersion = this.getSourceMajor();
		
		iss.getMos().add(dvc);
		return iss;
	}
	
	
	
	@Override
	public String toString(){
		
		return "ALTER TABLE " +tableName + " DROP VALUE CONSTRAINT \"" + this.id + "\"\n";
		
	}
	
	@Override
	public boolean isInfPreserving(Schema currentSchema) {
		return true;
	}
	
	@Override
	public String getOperatorType() {
		return ICMOInterface.DROP_VALUE_CONSTRAINT;
	}
	
	
}
