package edu.ucla.cs.db.prism.backend.icmo;

import java.util.Iterator;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.ded.DED;
import edu.ucla.cs.db.prism.backend.ded.DEDSet;
import edu.ucla.cs.db.prism.backend.ded.QuantifiedDisjunctive;
import edu.ucla.cs.db.prism.backend.ded.Quantifier;
import edu.ucla.cs.db.prism.backend.exception.SchemaErrorException;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.Table;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICTable;
import edu.ucla.cs.db.prism.backend.smo.util.Conjunctive;
import edu.ucla.cs.db.prism.backend.smo.util.Predicate;
import edu.ucla.cs.db.prism.backend.smo.util.Term;

/**
 * General class implementing an ICMO
 * @author Carlo A. Curino (carlo@curino.us)
 *
 */
public abstract class ICMOImpl extends MOImpl implements ICMOInterface {

	//This is shud be used for identifying teh IC id such as PK1 or FK1 or VC1 that is used to
	//identify the IC
	public String id = null;
	//public String key_id = null; 
	private int source_Major = 0;
	private int source_Minor = 0;
	private int target_Major = 0;
	private int target_Minor = 0;
	protected String tableName = null;
	protected String tableName2 = null;
	//ADD Primary Key
	//public Vector<String> columns = null;
	//ADD Foreign Key
	//public Vector<String> sourceColumns = null;
	//public Vector<String> targetColumns = null;
	//ADD Value Constraint
	//public String condition = null;
	
	//Basic for all icmo types

	//public boolean check = false;
	//public boolean enforce = false;	
	//public boolean declare = false;

	
	
	
	private String comment = "";
	
	public ICMOImpl(){
		
	}
	
	public ICMOImpl(String id, int source_Major, int source_Minor, int target_Major,
			int target_Minor) {
		
		if(id.startsWith("\"") && id.endsWith("\""))
			this.id = id.substring(1,id.length()-1);
		else
			this.id = id;
		this.source_Major = source_Major;
		this.source_Minor = source_Minor;
		this.target_Major = target_Major;
		this.target_Minor = target_Minor;
	}
	

	public ICMOImpl(String ICMOtype) {
		// TODO Auto-generated constructor stub
		this.setComment(ICMOtype);
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#getSource_Major()
	 */
	public int getSourceMajor() {
		return source_Major;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#setSource_Major(int)
	 */
	public void setSourceMajor(int source_Major) {
		this.source_Major = source_Major;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#getSource_Minor()
	 */
	public int getSourceMinor() {
		return source_Minor;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#setSource_Minor(int)
	 */
	public void setSourceMinor(int source_Minor) {
		this.source_Minor = source_Minor;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#getTarget_Major()
	 */
	public int getTargetMajor() {
		return target_Major;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#setTarget_Major(int)
	 */
	public void setTargetMajor(int target_Major) {
		this.target_Major = target_Major;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#getTarget_Minor()
	 */
	public int getTargetMinor() {
		return target_Minor;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#setTarget_Minor(int)
	 */
	public void setTargetMinor(int target_Minor) {
		this.target_Minor = target_Minor;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#getComment()
	 */
	public String getComment() {
		return comment;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#setComment(java.lang.String)
	 */
	public void setComment(String comment) {
		this.comment = comment;
	}
	
	public DEDSet getIdentityDEDs(ICSchema inputSchema,Vector<String> excludeList){
		
		DEDSet ds = new DEDSet(getSourceMajor(),getSourceMinor(),getTargetMajor(), getTargetMinor());
				
		String dedId = null;
		Iterator<ICTable> tabIt = inputSchema.getICTables().iterator();
		while(tabIt.hasNext()){
			ICTable tab = tabIt.next();
			if(!isExcluded(tab.getTableName(),excludeList)){	
				creatIDDED(inputSchema, ds, tab.getTableName(), dedId, false);
				creatIDDED(inputSchema, ds, tab.getTableName(), dedId, true);
			}
		}	
		return ds;
	}
	

	
	public static boolean isExcluded(String tableName,Vector<String> excludeList) {
		if(excludeList == null)
			return false;
		Iterator<String> it = excludeList.iterator();
		while(it.hasNext())
			if(it.next().equalsIgnoreCase(tableName))	
				return true;
		return false;
	}
	
	
	public DEDSet getAllDEDs(ICSchema inputSchema, Vector<String> excludelist){
		DEDSet ds = new DEDSet(getSourceMajor(),getSourceMinor(),getTargetMajor(), getTargetMinor());
		ds.addAll(getIdentityDEDs(inputSchema,excludelist));
		return ds;
	}
	
	
	private void creatIDDED(ICSchema inputSchema, DEDSet ds, String tabName,
			String dedId, boolean chaseFlag) {
		
		String dedName = "ID_" + tabName;
		Vector<String> sourceTables = new Vector<String>();
		sourceTables.add(tabName);
		Vector<String> targetTables = new Vector<String>();
		targetTables.add(tabName);
		DED d = null;
		if(chaseFlag)
			d = new DED(dedId, dedName + "_dm", chaseFlag, sourceTables, targetTables, null);
		else
			d = new DED(dedId, dedName + "_qr", chaseFlag, sourceTables, targetTables, null);
		
		Quantifier q1 = null;
		if(chaseFlag)
			q1= new Quantifier("all", "R0", "v" + getSourceMajor() + "m" +getSourceMinor() + "_" + tabName);
		else
			q1= new Quantifier("all", "R0", "v" + getTargetMajor() + "m" +getTargetMinor() + "_" + tabName);
				
		d.globalQuantifiers.quantifiers.add(q1);
		
		QuantifiedDisjunctive tc1 = new QuantifiedDisjunctive(); 
		d.thenClauses.add(tc1);
		
		if(chaseFlag)
			q1 = new Quantifier("some", "R1", "v" + getTargetMajor() + "m" + getTargetMinor() + "_" + tabName);
		else
			q1 = new Quantifier("some", "R1", "v" + getSourceMajor() + "m" + getSourceMinor() + "_" + tabName);
		
		tc1.quantifiers.add(q1);

		Conjunctive c1 = new Conjunctive();
		tc1.disjunctive.add(c1);
		c1.predicates = new Vector<Predicate>();
		Table inTable = inputSchema.getTable(tabName);
		Iterator<String> inColumns = inTable.getColumns().iterator();
		while(inColumns.hasNext()){
			Predicate p1 = new Predicate();
			String col = inColumns.next();
			c1.predicates.add(p1);
			p1.term1 = new Term("R0", col);
			p1.term2 = new Term("R1", col);
			p1.op = "=";

		}
		
		ds.add(d);
	}

	public String getTableName() {
		return tableName;
	}

	public void setTableName(String tableName) {
		this.tableName = tableName;
	}

	public String getTableName2() {
		return tableName2;
	}

	public void setTableName2(String tableName2) {
		this.tableName2 = tableName2;
	}

	@Override
	public DEDSet getAllDEDs(Schema inputSchema) throws SchemaErrorException{
		
		if(!(inputSchema instanceof ICSchema))
			throw new SchemaErrorException("You are trying to apply an ICMO on a Schema object, use an ICSchema object instead");
		
		DEDSet ds = new DEDSet(this.getSourceMajor(),this.getSourceMinor(),this.getTargetMajor(),this.getTargetMinor());
		ds.addAll(getIdentityDEDs((ICSchema) inputSchema,null));			
		return ds;
	}
	
	@Override
	public boolean isRedundant() {
		return false;
	}
	
	@Override
	public String toMetadataString() {
		return this.source_Major + "." + this.source_Minor + "-" + this.target_Major + "." +this.target_Minor + " " + toString() ;
	}

	public String getId() {
		return id;
	}

	public void setId(String id) {
		
		if(id.startsWith("\"") && id.endsWith("\""))
			this.id = id.substring(1,id.length()-1);
		else
			this.id = id;
	}
	
	@Override
	public void setId(int i) {
		
		this.id = Integer.toString(i);
		
	}
	
	@Override
	public boolean isComplete(){
		
		return true;
	}
	
	@Override
	public String getSQLDDL(Schema tsch, String schemaName){
		
		return "--NOT IMPLEMENTED FOR ICMO YET";
		
	}

}
