/**
 * 
 */
package edu.ucla.cs.db.prism.backend.icmo;

import java.util.Vector;

import edu.ucla.cs.db.prism.backend.ded.DEDSet;
import edu.ucla.cs.db.prism.backend.ded.DEDSetHistory;
import edu.ucla.cs.db.prism.backend.exception.SchemaErrorException;
import edu.ucla.cs.db.prism.backend.icmo.exceptions.AddForeignKeyException;
import edu.ucla.cs.db.prism.backend.icmo.exceptions.AddPrimaryKeyException;
import edu.ucla.cs.db.prism.backend.schema.exception.IllegalSMOException;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchemaHistory;
import edu.ucla.cs.db.prism.backend.smo.SMOImpl;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequence;

/**
 * @author Carlo A. Curino (carlo@curino.us)
 *
 */
public class ICMOSMOSequence extends SMOSequence{

	private Vector<MOImpl> mos = new Vector<MOImpl>();	
	
	public ICMOSMOSequence(SMOSequence in) {
		
		if(in instanceof ICMOSMOSequence)
			for(MOImpl temp : ((ICMOSMOSequence)in).getMos()){
				getMos().add(temp);
			}
		else{
			for(SMOImpl temp : in.smos){
				getMos().add(temp);
			}
			
		}
		sourceVersion = in.sourceVersion;
		targetVersion = in.targetVersion;
	}

	
	public ICMOSMOSequence subSequence(int i, int j){
		
		ICMOSMOSequence seq =  new ICMOSMOSequence();
		seq.mos = (Vector<MOImpl>) ((Vector<MOImpl>) mos.clone()).subList(i, j);
		seq.sourceVersion = mos.firstElement().getSourceMajor();
		seq.targetVersion = mos.lastElement().getTargetMajor();
		return seq;
	}
	
	public ICMOSMOSequence() {
		// TODO Auto-generated constructor stub
	}

	public ICMOSMOSequence(MOImpl mo) {

		getMos().add(mo);
		
		sourceVersion = mo.getSourceMajor();
		targetVersion = mo.getTargetMajor();
	}

	@Override
	public String toString() {
		String str = "";
		str += "v" + this.sourceVersion;
		str += " - v" + this.targetVersion;
		str += getMos().toString();
		return str;
	}

	
	
	/* (non-Javadoc)
	 * @see edu.ucla.cs.db.prism.backend.smo.util.MOSequenceInterface#add(edu.ucla.cs.db.prism.backend.smo.SMOImpl)
	 */
	public boolean add(MOImpl smo) {
		return mos.add(smo);
	}
	
	public ICMOSMOSequence getICCandidateInverseSequence(ICSchema lastElement) {
		ICSchema inputSchema = lastElement.clone();
		ICMOSMOSequence inverseSMOSequence = new ICMOSMOSequence();
		
		// FIXME this returns only the first candidate
		for(int i=0; i<getMos().size(); i++){
			ICMOSMOSequence temp = null;
			try {
				temp = new ICMOSMOSequence(getMos().get(i).getBestCandidateInverses(inputSchema));
			} catch (SchemaErrorException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			} catch (AddPrimaryKeyException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (DropForeignKeyException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (AddForeignKeyException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			try {
				inputSchema.applyOperator(getMos().get(i));
			} catch (IllegalSMOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (SchemaErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}			
			inverseSMOSequence.getMos().addAll(temp.getMos());
		}
		inverseSMOSequence.sourceVersion = this.targetVersion;
		inverseSMOSequence.targetVersion = this.sourceVersion;
		return inverseSMOSequence;
	}

	public ICSchemaHistory generateICSchemaHistory(ICSchema ics, boolean b) {
		
		ICSchema temp = ics.clone();
		ICSchemaHistory sh = new ICSchemaHistory();
		
		int schmaj = ics.versionID;
		int schmin = ics.minorVersionID;
		int startingsmo = -1;
		// we find the SMOs from which is appropriate to start applying the sequence, i.e., the one with source version equal to the schema version
		for(int i = 0; i< getMos().size(); i++){
			if(schmaj == getMos().elementAt(i).getSourceMajor() && schmin == getMos().elementAt(i).getSourceMinor()){
				startingsmo = i;
				break;
			}
		}
				
		if(b){
			//we process the SMOSequence in inverse direction (from the last to the first element)
			for(int i = startingsmo; i>=0; i--){
				sh.addSchema(temp.clone()); 

				MOImpl smo = getMos().elementAt(i);
				try {
					temp.applyOperator(smo);
				} catch (IllegalSMOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (SchemaErrorException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			sh.addSchema(temp);
			
		}else{
			// otherwise we process the sequence forward
			//Added by Swami - Incase of deleting the only smo
			//startingsmo value will be -1 and so throws error
			//Also changed some code in SMODesign to accomodate unexisting schema error
			if(startingsmo >=0)
			{
				for(int i = startingsmo; i < getMos().size();i++){
					sh.addSchema(temp.clone());
					MOImpl smo = getMos().elementAt(i);
					try {
						temp.applyOperator(smo);
					} catch (IllegalSMOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					} catch (SchemaErrorException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
				sh.addSchema(temp);
			}
			else
			{
				sh.addSchema(temp.clone());
				
			}
			//Ended - Swami
		} 
		
		//we return the new schemaHistory
		return sh;
	}


public DEDSetHistory getAllDED(ICSchema schema, boolean isInverse) {
		
		ICSchema curSchema = schema.clone();
		DEDSetHistory dedSetHistory = new DEDSetHistory();

		int schmaj = schema.versionID;
		int schmin = schema.minorVersionID;
		int startingsmo = -1;
		// we find the SMOs from which is appropriate to start applying the sequence, i.e., the one with source version equal to the schema version
		for(int i = 0; i< getMos().size(); i++){
			if(schmaj == getMos().elementAt(i).getSourceMajor() && schmin == getMos().elementAt(i).getSourceMinor()){
				startingsmo = i;
				break;
			}
		}
		
		
		
		if(isInverse){
			for(int i = startingsmo; i>= 0; i--){
				MOImpl smo = getMos().elementAt(i);
				//System.out.println("Generating DEDS for " + smo.toMetadataString() + "SCHEMA: 48_" + curSchema.minorVersionID );
				DEDSet ds = null;
				try {
					ds =smo.getAllDEDs(curSchema);
					curSchema.applyOperator(smo);
				} catch (IllegalSMOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (SchemaErrorException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				dedSetHistory.add(ds);		
			}
		}else{
			for(int i = startingsmo; i<getMos().size(); i++){
				MOImpl smo = getMos().elementAt(i);
				DEDSet ds = null;
				try {
					ds = smo.getAllDEDs(curSchema);
		
					curSchema.applyOperator(smo);
				} catch (IllegalSMOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (SchemaErrorException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				dedSetHistory.add(ds);		
			}
			
			
		}
			
		return dedSetHistory;
	}
	

public ICSchemaHistory generateSchemaHistory(ICSchema input, boolean isInverse) {
	
	ICSchema temp = input.clone();
	ICSchemaHistory sh = new ICSchemaHistory();
	
	int schmaj = input.versionID;
	int schmin = input.minorVersionID;
	int startingsmo = -1;
	// we find the SMOs from which is appropriate to start applying the sequence, i.e., the one with source version equal to the schema version
	for(int i = 0; i< getMos().size(); i++){
		if(schmaj == getMos().elementAt(i).getSourceMajor() && schmin == getMos().elementAt(i).getSourceMinor()){
			startingsmo = i;
			break;
		}
	}
	
	//System.out.println("APPLYING INVERSE.generateSchemaHistory FROM: " + startingsmo + " TO 0");
	
	if(isInverse){
		//we process the SMOSequence in inverse direction (from the last to the first element)
		for(int i = startingsmo; i>=0; i--){
			//System.out.println("PROCESSING SMO: " + smos.elementAt(i).toMetadataString());
			sh.addSchema(temp.clone()); 
			MOImpl smo = getMos().elementAt(i);
			try {
				(temp).applyOperator(smo);
			} catch (IllegalSMOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (SchemaErrorException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		sh.addSchema(temp);
	}else{
		// otherwise we process the sequence forward
		//Added by Swami - Incase of deleting the only smo
		//startingsmo value will be -1 and so throws error
		//Also changed some code in SMODesign to accomodate unexisting schema error
		if(startingsmo >=0)
		{
			for(int i = startingsmo; i < getMos().size();i++){
				sh.addSchema(temp.clone());
				MOImpl smo = getMos().elementAt(i);
				try {
					(temp).applyOperator(smo);
				} catch (IllegalSMOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (SchemaErrorException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			sh.addSchema(temp);
		}
		else
		{
			sh.addSchema(temp);
			
		}
		//Ended - Swami
	} 
	
	//we return the new schemaHistory
	return sh;
}

public void addMos(MOImpl mo) {
	this.targetVersion = mo.getTargetMajor();
	getMos().add(mo);
	return;
}

public void add(int index, MOImpl mo) {
	this.targetVersion = mo.getTargetMajor();
	getMos().add(index, mo);
	return;
}

/**
 * @param mos the mos to set
 */
public void setMos(Vector<MOImpl> mos) {
	this.mos = mos;
}

/**
 * @return the mos
 */
public Vector<MOImpl> getMos() {
	return mos;
}

@Override
public MOImpl get(int index) {
	return mos.get(index);
}

@Override
public MOImpl getSMOBySource(int major, int minor) {
	
	for(int i=0;i<mos.size();i++){
		if(mos.get(i).getSourceMajor() == major && mos.get(i).getSourceMinor() == minor)
			return mos.get(i);
		
	}
	return null;
}


/* (non-Javadoc)
 * @see edu.ucla.cs.db.prism.backend.smo.util.MOSequenceInterface#getSMOIndexBySource(int, int)
 */
@Override
public int getSMOIndexBySource(int major, int minor) {
	
	for(int i=0;i<mos.size();i++){
		if(mos.get(i).getSourceMajor() == major && mos.get(i).getSourceMinor() == minor)
			return i;
		
	}
	return -1;
}


/* (non-Javadoc)
 * @see edu.ucla.cs.db.prism.backend.smo.util.MOSequenceInterface#removeReassignVersions(int)
 */
@Override
public MOImpl removeReassignVersions(int index) {
	for(int i = index; i< mos.size();i++){
		mos.get(i).setSourceMinor(mos.get(i).getSourceMinor()-1);
		
		// LAST SMOS SHOULD NOT HAVE INDEX CHANGED
		if(i!= mos.size()-1){
			mos.get(i).setTargetMinor(mos.get(i).getTargetMinor()-1);
		}
	}
	
	return mos.remove(index);
}


public int getCommonHistoryIndex(ICMOSMOSequence newsmos, boolean isInverse) {
	int index=0;
	int limit = 0;
	if(getMos().size()<newsmos.getMos().size())
		limit = getMos().size();
	else
		limit = newsmos.getMos().size();
	
	if(isInverse){
		for(int i=0; i<limit; i++){
			if(getMos().elementAt(i).equals(newsmos.getMos().elementAt(i)))
				index++;
			else
				break;
		}
	}else{
		for(int i=limit-1; i>=0; i--){
			if(getMos().elementAt(i).equals(newsmos.getMos().elementAt(i)))
				index++;
			else
				break;
		}	
	}
	
	return index;
}


	
}



