package edu.ucla.cs.db.prism.backend.marsManage;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.*;

import edu.ucla.cs.db.prism.backend.SQL.RelationDeclaration;
import edu.ucla.cs.db.prism.backend.SQL.SQL;
import edu.ucla.cs.db.prism.backend.SQL.SQLUnion;
import edu.ucla.cs.db.prism.backend.SQL.SQLUnionSet;
import edu.ucla.cs.db.prism.backend.ded.*;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.SchemaHistory;
import edu.ucla.cs.db.prism.backend.schema.Table;
import edu.ucla.cs.db.prism.backend.smo.util.Conjunctive;
import edu.ucla.cs.db.prism.backend.smo.util.Predicate;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequenceHistory;
import edu.ucla.cs.db.prism.backend.smo.util.Term;
import edu.ucla.cs.db.prism.backend.util.REFConstant;


public class CodiGen {
	
	static public void generateOneCodiScriptMultipleQueries(SQLUnionSet sqlUnions, 
			SchemaHistory schemaHistory, SMOSequenceHistory smoSequenceHistory, DEDSet deds,
			int numSteps, String codiScriptFileName) {
		// file writing preparation
        BufferedWriter out = null;
        try {
        	// chase & backchase in a single codie script
        	out = new BufferedWriter(new FileWriter(codiScriptFileName));
        	

        	// step 1: write schema declaration
        	//writeSchemaDeclaration(numSteps, out);
        	//writeCodiPrelude(out);
        	writeCodiPrelude(sqlUnions, deds, out);
        	
        	// step 2: write deds
        	out.write(deds.toStringNoMetadata());
        	
        	// step 3: write input query
        	writeInputQueries(sqlUnions, out);
        	
        	// step 4: write rewriting commands
			//writeRewritingCommands(sqlUnions, numSteps, out);
        	writeChaseCommandsOld(sqlUnions, numSteps, out);
        	
        	out.close();
        } catch (IOException e) {}
	}
	
	/*
	static public void generateOneCodiScript(SQL sql, DEDSet deds, String resultFileName) {
		// file writing preparation
        BufferedWriter out = null;
        try {
        	// chase & backchase in a single codie script
        	out = new BufferedWriter(new FileWriter(resultFileName));
        	writeCodiPrelude(out);
        	//out.write(deds.getDEDsforChase(true).toStringNoMetadata());
        	//out.write(deds.getDEDsforChase(false).toStringNoMetadata());
        	out.write(deds.toStringNoMetadata());
			out.write("val Q2 =\n" + sql.toString() + "\n");
			writeCodiEpilogueCompile(out);
			writeCodiEpilogueTranslate(out);
			out.close();
        } catch (IOException e) {}
	}
    */
	
	/*
	// deprecated
	static public void generateTwoCodiScripts(SQL sql, DEDSet deds) {
		// file writing preparation
        BufferedWriter out = null;
        try {
        	// Part 1: chase
        	out = new BufferedWriter(new FileWriter("prima-c.codi"));
        	writeCodiPrelude(out);
        	out.write(deds.getDEDsforChase(true).toStringNoMetadata());
			out.write("val Q2 =\n" + sql.toString() + "\n");
			writeCodiEpilogueCompile(out);
			writeCodiEpilogueChase(out);
			out.close();

        	// Part 2: backchase
        	out = new BufferedWriter(new FileWriter("prima-bc.codi"));
        	writeCodiPrelude(out);
        	out.write(deds.getDEDsforChase(false).toStringNoMetadata());
        	// next two lines need be filled after running mars with chase codi
			//out.write("val Q2 =\n" + sql.toString() + "\n"); 
			//writeCodiEpilogueTranslate(out);
			out.close();
        } catch (IOException e) {}
	}
	
	*/
	
	static protected void writeSchemaDeclaration(int numSteps, BufferedWriter out) {
        try {
			out.write("Type OneArgTy = Record [ a0 : String ];\n");
			out.write("Type TwoArgTy = Record [ a0 : String, a1 : String ];\n");
			out.write("Type ThreeArgTy = Record [ a0 : String, a1 : String, a2 : String ];\n\n");

			// root is not "stepped"
			out.write("Schema root: Coll { SET of OneArgTy };\n");

			// step 0
			out.write("Schema child: Coll { SET of TwoArgTy };\n");
			out.write("Schema tag: Coll { SET of TwoArgTy };\n");
			out.write("Schema text: Coll { SET of TwoArgTy };\n");
			out.write("Schema attr: Coll { SET of ThreeArgTy };\n");
			
			for(int i=1; i<=numSteps; i++) {
				out.write("Schema s" + i + "child: Coll { SET of TwoArgTy };\n");
				out.write("Schema s" + i + "tag: Coll { SET of TwoArgTy };\n");
				out.write("Schema s" + i + "text: Coll { SET of TwoArgTy };\n");
				out.write("Schema s" + i + "attr: Coll { SET of ThreeArgTy };\n");
			}
			
        } catch (IOException e) {}
	}
	
	static public void writeInputQueries(SQLUnionSet sqlUnions, BufferedWriter out) {
		
    	// step 0: encode less-than ("<") in query into REF
    	convertLessIntoRelationEncodedFunction(sqlUnions);
		//System.out.println(sqlUnions);
    	
        try {
        	for(int i=0; i<sqlUnions.sqlUnions.size(); i++) {
        		out.write("val Q" + sqlUnions.sqlUnions.get(i).name + " = \n" + 
        				sqlUnions.sqlUnions.get(i).toString() + ";\n");
        	}
        } catch (IOException e) {}
	}
	
	static private void convertLessIntoRelationEncodedFunction(SQLUnionSet sqlUnions) {
		for(int i=0; i<sqlUnions.sqlUnions.size(); i++)
			convertLessIntoRelationEncodedFunction(sqlUnions.sqlUnions.get(i));
	}
	static private void convertLessIntoRelationEncodedFunction(SQLUnion sqlUnion) {
		for(int i=0; i<sqlUnion.sqls.size(); i++)
			convertLessIntoRelationEncodedFunction(sqlUnion.sqls.get(i));
	}
	static private void convertLessIntoRelationEncodedFunction(SQL sql) {
		if(sql.where == null) return; // nothing to do
		if(sql.where.conjunctives.size() == 0) return; // nothing to do

		int index = 1; // 0 used by vidb
		Conjunctive conjunctive = sql.where.conjunctives.get(0);
		int size = conjunctive.predicates.size();;
		for(int i=0; i<size; i++) {
			Predicate p = conjunctive.predicates.get(i);
			if(p.op.equals(">")) {
				p.op = "<";
				Term term = p.term1;
				p.term1 = p.term2;
				p.term2 = p.term1;
			}
			
			if(!p.op.equals("<")) continue;
			String relation = "ref2_less";
			String relationAlias = "y" + index;
			index++;
			sql.from.relationDeclarations.add(
					new RelationDeclaration(relation, relationAlias));
			conjunctive.predicates.remove(i);
			conjunctive.predicates.add(i, 
					new Predicate(new Term(relationAlias, "a0"), "=", p.term1));
			conjunctive.predicates.add(i+1, 
					new Predicate(new Term(relationAlias, "a1"), "=", p.term2));
			i++;
			size++;
		}
	}
	
	static private void writeRewritingCommands(SQLUnionSet sqlUnions, int finalStep, BufferedWriter out) {
		int maxArg = 10;
		try {
        	/*
        	String translateTarget = "[root, ";
        	translateTarget += "s" + finalStep + "child, "; 
        	translateTarget += "s" + finalStep + "tag, "; 
        	translateTarget += "s" + finalStep + "attr, "; 
        	translateTarget += "s" + finalStep + "text]"; 
        	*/
			String translateTarget = "";
			translateTarget += "v1root, v1child, v1tag, v1text, v1attr, ";
			for(int i=1; i<=maxArg; i++) {
				translateTarget += "v2xb" + i;
				if(i<maxArg)
					translateTarget += ", ";
			}
			translateTarget = "[" + translateTarget + "]";
        	
        	for(int i=0; i<sqlUnions.sqlUnions.size(); i++) {
        		String queryName = "Q" + sqlUnions.sqlUnions.get(i).name;
    			out.write("\nQuery " + queryName + "_compiled" + 
    					"= Compile " + queryName + ";\n");
    			out.write("Query " + queryName + "_translated" + 
    					"= Translate " + queryName + "_compiled against " + 
    					translateTarget + ";\n");
        	}
        	out.write("Print[");
        	for(int i=0; i<sqlUnions.sqlUnions.size(); i++) {
        		String queryName = "Q" + sqlUnions.sqlUnions.get(i).name;
    			out.write(queryName + "_translated");
    			if(i<sqlUnions.sqlUnions.size()-1)
    				out.write(",");
        	}
        	out.write("];\n\n");
        } catch (IOException e) {}	
	}
	
	// new in PRIMAv0.6
	// added by Hyun Moon (9/6/2007)
	// translate is done by postprocessing algorithm
	protected static void writeChaseCommandsOld(SQLUnionSet sqlUnions, int finalStep, BufferedWriter out) {
        try {

        	for(int i=0; i<sqlUnions.sqlUnions.size(); i++) {
        		String queryName = "Q" + sqlUnions.sqlUnions.get(i).name;
    			out.write("\nQuery " + queryName + "_compiled" + 
    					"= Compile " + queryName + ";\n");
    			out.write("Query " + queryName + "_chased" + 
    					"= Chase " + queryName + "_compiled;\n");
        	}
        	out.write("Print[");
        	for(int i=0; i<sqlUnions.sqlUnions.size(); i++) {
        		String queryName = "Q" + sqlUnions.sqlUnions.get(i).name;
    			out.write(queryName + "_chased");
    			if(i<sqlUnions.sqlUnions.size()-1)
    				out.write(",");
        	}
        	out.write("];\n\n");
        } catch (IOException e) {}	
	}
	
	static public void writeCompileCommands(SQLUnionSet sqlUnions, BufferedWriter out) {
        try {

        	for(int i=0; i<sqlUnions.sqlUnions.size(); i++) {
        		String queryName = "Q" + sqlUnions.sqlUnions.get(i).name;
    			out.write("\nQuery " + queryName + "_compiled" + 
    					" = Compile " + queryName + ";\n");
        	}

        } catch (IOException e) {}	
	}
	static public void writeChaseCommands(SQLUnionSet sqlUnions, BufferedWriter out, int chaseIndex) {
        try {

        	for(int i=0; i<sqlUnions.sqlUnions.size(); i++) {
        		String inputQueryName = "Q" + sqlUnions.sqlUnions.get(i).name;
        		if(chaseIndex == 0)
        			inputQueryName += "_compiled";
        		else
        			inputQueryName += "_chased" + (chaseIndex-1);
        		String outputQueryName = "Q" + sqlUnions.sqlUnions.get(i).name +
        			"_chased" + chaseIndex;
    			out.write("Query " + outputQueryName+ 
    					" = Chase " + inputQueryName +";\n");
        	}
        } catch (IOException e) {}	
	}
	static public void writeTranslateCommands(SQLUnionSet sqlUnions, BufferedWriter out, int chaseIndex,
			SchemaHistory schemaHistory, DEDSetHistory dedSetHistory, 
			int sourceVersion, int sourceMinorVersion, Vector<String> sourceTablesForTranslate) {
		
		// construct a source relation list
		String srcTableList = "";
		// use passed sourceTablesForTranslate, if not null
		// extract it from schemaHistory, otherwise
		if(sourceTablesForTranslate != null) {
			for(int i=0; i<sourceTablesForTranslate.size(); i++) {
				srcTableList += sourceTablesForTranslate.get(i);
				if(i<sourceTablesForTranslate.size()-1)
					srcTableList += ",";
			}
		}
		else {
			Schema srcSchema = schemaHistory.getSchemaByVersion(sourceVersion,sourceMinorVersion);
			for(int i=0; i<srcSchema.getTables().size(); i++) {
				srcTableList += "v" + sourceVersion + "m" +sourceMinorVersion + "_"  + srcSchema.getTables().get(i).getTableName();
				if(i<srcSchema.getTables().size()-1)
					srcTableList += ",";
			}
		}
		
		// for Relational-Encoded-Functions (REFs)
		Vector<String> refRelations = new Vector<String>();
		for(int i=0; i<dedSetHistory.size(); i++) {
			DEDSet dedSet = dedSetHistory.dedSets.get(i);
			for(int j=0; j<dedSet.size(); j++) {
				DED ded = dedSet.deds.get(j);
				// global quantifiers
				for(int k=0; k<ded.globalQuantifiers.size(); k++) {
					String relationName = ded.globalQuantifiers.get(k).relation;
					if(REFConstant.hasREFPrefix(relationName) &&
							//!REFConstant.isInequalityREF(relationName) &&
							!refRelations.contains(relationName))
						refRelations.add(relationName);
				}
				// then-clause quantifiers
				for(int k2=0; k2<ded.thenClauses.size(); k2++) {
					QuantifiedDisjunctive qd = ded.thenClauses.get(k2);
					for(int k3=0; k3<qd.quantifiers.size(); k3++) {
						String relationName = qd.quantifiers.get(k3).relation;
						if(REFConstant.hasREFPrefix(relationName) &&
								//!REFConstant.isInequalityREF(relationName) &&
								!refRelations.contains(relationName))
							refRelations.add(relationName);
					}
				}
			}
		}
		
		// REFs coming from queries
		Vector<String> refRelationsInQuery = 
			REFConstant.getREFRelationNames(sqlUnions);
		for(int i=0; i<refRelationsInQuery.size(); i++) {
			if(!refRelations.contains(refRelationsInQuery.get(i)))
				refRelations.add(refRelationsInQuery.get(i));
		}
		
		String refList = "";
		for(int i=0; i<refRelations.size(); i++) {
			refList += refRelations.get(i);
			if(i<refRelations.size()-1)
				refList += ", ";
		}
		if(refList.length() > 0)
			refList = ", " + refList;
		srcTableList = "[" + srcTableList + refList + "]";
		
		try {
        	for(int i=0; i<sqlUnions.sqlUnions.size(); i++) {
        		String inputQueryName = "Q" + sqlUnions.sqlUnions.get(i).name;
        		if(chaseIndex == 0)
        			inputQueryName += "_compiled";
        		else
        			inputQueryName += "_chased" + (chaseIndex-1);
        		String outputQueryName = "Q" + sqlUnions.sqlUnions.get(i).name +
        			"_chased" + chaseIndex;
        		
    			out.write("Query " + outputQueryName+ 
    					" = Translate " + inputQueryName + 
    					" against " + srcTableList + ";\n");
        	}
        } catch (IOException e) {}	
	}

	static public void writeBackchaseCommands(SQLUnionSet sqlUnions, BufferedWriter out, int chaseIndex) {
		try {
        	for(int i=0; i<sqlUnions.sqlUnions.size(); i++) {
        		String inputQueryName = "Q" + sqlUnions.sqlUnions.get(i).name;
        		if(chaseIndex == 0)
        			inputQueryName += "_compiled";
        		else
        			inputQueryName += "_chased" + (chaseIndex-1);
        		String outputQueryName = "Q" + sqlUnions.sqlUnions.get(i).name +
        			"_chased" + chaseIndex;
        		
    			out.write("Query " + outputQueryName+ 
    					" = First_Sip_Backchase " + inputQueryName + ";\n");
        	}
        } catch (IOException e) {}	
	}
	static public void writePrintCommands(SQLUnionSet sqlUnions, BufferedWriter out, int chaseIndex) {
        try {


        	for(int i=0; i<sqlUnions.sqlUnions.size(); i++) {
            	String queryName = "Q" + sqlUnions.sqlUnions.get(i).name;
    			out.write("Print[" + queryName + "_chased" + (chaseIndex-1) + "];\n");
        	}
        	out.write("\n");
        } catch (IOException e) {}	
	}
	
	static public void writeCodiPrelude(SQLUnionSet sqlUnions, DEDSet dedSet,	BufferedWriter out) {
		String relationEncodedFunctionPrefix = REFConstant.getRelationEncodedFunctionPrefix();
		int maxNumArgs = 0;
        try {
        	String typeDeclaration = "";
        	String schemaDeclaration = "";


        	// part 1: loop through deds and for each unique relation declaration
        	//       generate schema declaration (use hashtable for duplicate check)
        	Hashtable<String, String> relationsFound = new Hashtable<String, String>();
        	
        	// for the query
        	for(int i=0; i<sqlUnions.sqlUnions.size(); i++) {
        		SQLUnion sqlUnion = sqlUnions.sqlUnions.get(i);
        		for(int j=0; j<sqlUnion.sqls.size(); j++) {
        			SQL sql = sqlUnion.sqls.get(j);
        			for(int k=0; k<sql.from.relationDeclarationsSize(); k++) {
        				RelationDeclaration rd = sql.from.relationDeclarations.get(k);
        				if(rd.relation != null)
        					relationsFound.put(rd.relation, ""); // value is set to "" for all keys
        			}
        		}
        	}
        	
        	// for all deds
        	for(int i=0; i<dedSet.size(); i++) {
        		DED ded = dedSet.get(i);
        		// if-clause
        		for(int j=0; j<ded.globalQuantifiers.size(); j++) {
        			Quantifier q = ded.globalQuantifiers.quantifiers.get(j);
       				relationsFound.put(q.relation, ""); // value is set to "" for all keys
        		}
        		// then-clause
        		for(int k=0; k<ded.thenClauses.size(); k++) {
        			QuantifiedDisjunctive tc = ded.thenClauses.get(k);
            		for(int j=0; j<tc.quantifiers.size(); j++) {
            			Quantifier q = tc.quantifiers.get(j);
           				relationsFound.put(q.relation, ""); // value is set to "" for all keys
            		}
        		}
        	}
        	
        	
        	
        	
        	 for (Enumeration<String> e = relationsFound.keys(); e.hasMoreElements();) {
        		 String relation = e.nextElement();
        		 String oneSchemaDeclaration = null;
        		 
        		 if(relation.startsWith("v")) { // vidb

        			 oneSchemaDeclaration = 
        				 "Schema " + relation + ": Coll { SET of typedb };\n";
        		 }
        		 else if(relation.startsWith("tab_")) {
        			 oneSchemaDeclaration = 
        				 "Schema " + relation + ": Coll { SET of typetablerow };\n";
        		 }
        		 else if(relation.startsWith("row")) {
        			 oneSchemaDeclaration = 
        				 "Schema " + relation + ": Coll { SET of typetablerow };\n";        			 
        		 }
        		 else if(relation.startsWith("col_")) {
        			 oneSchemaDeclaration = 
        				 "Schema " + relation + ": Coll { SET of typecol };\n";        			 
        		 }
        		 else if(relation.startsWith("ref")) {
        			 int index = relation.indexOf('_');
        			 int numArgs = Integer.parseInt(relation.substring(3, index));
        			 if(numArgs > maxNumArgs)
        				 maxNumArgs = numArgs;
        			 oneSchemaDeclaration = 
        				 "Schema " + relation + ": Coll { SET of type" + (numArgs+1) + "};\n";  
        		 }
        		 else if(relation.startsWith("const")) {
        			 int index = relation.indexOf('_');
        			 int numConst = Integer.parseInt(relation.substring(5, index));
        			 if(numConst > maxNumArgs)
        				 maxNumArgs = numConst;
        			 oneSchemaDeclaration = 
        				 "Schema " + relation + ": Coll { SET of type" + numConst + "};\n";  
        		 }
        		 else
        			 continue; //error
        		 
        		 schemaDeclaration += oneSchemaDeclaration;
        	 }
        	 schemaDeclaration += "\n";
        	 
        	// part 2: type declaration
        	for(int i=1; i<=maxNumArgs+1; i++) {
        		String str = "Type type" + i + " = Record [ ";
        		for(int j=1; j<=maxNumArgs+1; j++) {
        			str += "a" + (j-1) + " : String"; // begins from a0
        			if(j<maxNumArgs+1)
        				str += ", ";
        		}
        		str += " ];\n";
        		typeDeclaration += str;
        	}
        	
        	typeDeclaration +=
        		"Type typedb = Record [ el : String, ts : String, te : String ];\n" +
        		"Type typetablerow = Record [ par : String, el : String, ts : String, te : String ];\n" +
        		"Type typecol = Record [ par : String, el : String, ts : String, te : String, text : String ];\n";
        	typeDeclaration += "\n";
        	
 
           	out.write(typeDeclaration);
        	out.write(schemaDeclaration);
 
        } catch (IOException e) {}
	}

	
	static public void writeCodiPrelude(SchemaHistory schemaHistory, 
			DEDSetHistory dedSetHistory, SQLUnionSet inputSQLUnionSet, 
			BufferedWriter out) {
		

		
		String integrityConstraintsDec = "";
	
	

		// step 1: to make types a minimal set, 
		//         don't repeat the same type if already defined
		Schema schemaForTypeDef = new Schema();
		for(int i=0; i<schemaHistory.size(); i++) {
			Schema schema = schemaHistory.getSchema(i);
			for(int j=0; j<schema.size(); j++) {
				Table table = schema.getTable(j);
				if(!schemaForTypeDef.containsTable(table))
					schemaForTypeDef.addTable(table);
			}
		}

		String typeDec = "";
		for(int i=0; i<schemaForTypeDef.size(); i++) {
			typeDec += "Type type" + i + " = Record [";
			Table table = schemaForTypeDef.getTable(i);
			for(int j=0; j<table.getNumColumns(); j++) {
				typeDec += " " + table.getColumn(j) + " : String";
				if(j<table.getNumColumns()-1)
					typeDec += ",";
			}
			typeDec += " ];\n";
		}
		
		
		// MANAGEMENT OF > < >= <=
		typeDec += "Type relational = Record [ a0 : String, a1 : String ];\n";

		int typeIndex=-1;
		
		
		// step 2: 
		String schemaDec = "";
		for(int i=0; i<schemaHistory.size(); i++) {
			Schema schema = schemaHistory.getSchema(i);
			for(int j=0; j<schema.getTables().size(); j++) {
			
				Table table	= schema.getTables().get(j);
				
				typeIndex = schemaForTypeDef.indexOf(table);
				if(schema.isHTableSchema)
					schemaDec += "Schema vh" + schema.versionID;
				else
					schemaDec += "Schema v" + schema.versionID;
				schemaDec += "m" + schema.minorVersionID;
				schemaDec += "_" + table.getTableName() + " : Coll { SET of type" + 
							typeIndex + " };\n";
			}
		}

		// MANAGEMENT OF > < >= <=
		typeIndex++;
		//schemaDec += "Schema _ref2_less : Coll { SET of relational };\n";
		//schemaDec += "Schema _ref2_leq : Coll { SET of relational };\n";
		//schemaDec += "Schema _ref2_neq : Coll { SET of relational };\n";

		
		// step 3: for Relational-Encoded-Functions (REFs), coming from DEDs
		Vector<String> refRelations = new Vector<String>();
		for(int i=0; i<dedSetHistory.size(); i++) {
			DEDSet dedSet = dedSetHistory.dedSets.get(i);
			for(int j=0; j<dedSet.size(); j++) {
				DED ded = dedSet.deds.get(j);
				// global quantifiers
				for(int k=0; k<ded.globalQuantifiers.size(); k++) {
					String relationName = ded.globalQuantifiers.get(k).relation;
					if(REFConstant.hasREFPrefix(relationName) &&
							//!REFConstant.isInequalityREF(relationName) &&
							!refRelations.contains(relationName))
						refRelations.add(relationName);
				}
				// then-clause quantifiers
				for(int k2=0; k2<ded.thenClauses.size(); k2++) {
					QuantifiedDisjunctive qd = ded.thenClauses.get(k2);
					for(int k3=0; k3<qd.quantifiers.size(); k3++) {
						String relationName = qd.quantifiers.get(k3).relation;
						if(REFConstant.hasREFPrefix(relationName) &&
								//!REFConstant.isInequalityREF(relationName) &&
								!refRelations.contains(relationName))
							refRelations.add(relationName);
					}
				}
			}
		}
		// REFs coming from queries
		Vector<String> refRelationsInQuery = 
			REFConstant.getREFRelationNames(inputSQLUnionSet);
		for(int i=0; i<refRelationsInQuery.size(); i++) {
			if(!refRelations.contains(refRelationsInQuery.get(i)))
				refRelations.add(refRelationsInQuery.get(i));
		}
		
		Vector<Integer> numAttrs = new Vector<Integer>();
		for(int i=0; i<refRelations.size(); i++) {
			int numArg = REFConstant.getNumOfArgs(refRelations.get(i)); 
			int numAttr = numArg + 1;
			if(!numAttrs.contains(new Integer(numAttr))) {
				numAttrs.add(new Integer(numAttr));
			}
			
			schemaDec += "Schema " + refRelations.get(i) + " : Coll { SET of type_attr_" +  (numArg+1) + " };\n";
			
	
			
		}
		
		for(int i=0; i<numAttrs.size(); i++) {
			int numAttr = numAttrs.get(i);
			String oneTypeDec = "Type type_attr_" + numAttr + " = Record [ ";
			for(int j=0; j<numAttr; j++) {
				oneTypeDec += "a" + j + " : String";
				if(j<numAttr-1)
					oneTypeDec += ", ";
			}
			oneTypeDec += " ];\n";
			typeDec += oneTypeDec;
		}
		
		
		

	
	
		
		
		
		try {
			out.write(typeDec + "\n" + schemaDec + "\n" + integrityConstraintsDec);
		}
		catch(Exception e) {
			//FIXME is BAD having muted Exception around the code!!
			
		}
	
	
	
	
	
	}
	

	static public String writeHTMLCodiPrelude(SchemaHistory schemaHistory, 
			DEDSetHistory dedSetHistory) {
		String output = "";
		// step 1: to make types a minimal set, 
		//         don't repeat the same type if already defined
		Schema schemaForTypeDef = new Schema();
		for(int i=0; i<schemaHistory.size(); i++) {
			Schema schema = schemaHistory.getSchema(i);
			for(int j=0; j<schema.size(); j++) {
				Table table = schema.getTable(j);
				if(!schemaForTypeDef.containsTable(table))
					schemaForTypeDef.addTable(table);
			}
		}

		String typeDec = "";
		for(int i=0; i<schemaForTypeDef.size(); i++) {
			typeDec += "Type type" + i + " = Record [";
			Table table = schemaForTypeDef.getTable(i);
			for(int j=0; j<table.getNumColumns(); j++) {
				typeDec += " " + table.getColumn(j) + " : String";
				if(j<table.getNumColumns()-1)
					typeDec += ",";
			}
			typeDec += " ];<br>";
		}
		
		
		// MANAGEMENT OF > < >= <=
		typeDec += "Type relational = Record [ a0 : String, a1 : String ];<br>";

		int typeIndex=-1;
		
		
		// step 2: 
		String schemaDec = "";
		for(int i=0; i<schemaHistory.size(); i++) {
			Schema schema = schemaHistory.getSchema(i);
			for(int j=0; j<schema.getTables().size(); j++) {
				Table table = schema.getTables().get(j);
				typeIndex = schemaForTypeDef.indexOf(table);
				if(schema.isHTableSchema)
					schemaDec += "Schema vh" + schema.versionID;
				else
					schemaDec += "Schema v" + schema.versionID;
				schemaDec += "m" + schema.minorVersionID;
				schemaDec += "_" + table.getTableName() + " : Coll { SET of type" + 
							typeIndex + " };<br>";
			}
		}

		// MANAGEMENT OF > < >= <=
		typeIndex++;
		//schemaDec += "Schema _ref2_less : Coll { SET of relational };\n";
		//schemaDec += "Schema _ref2_leq : Coll { SET of relational };\n";
		//schemaDec += "Schema _ref2_neq : Coll { SET of relational };\n";

		
		// step 3: for Relational-Encoded-Functions (REFs), coming from DEDs
		Vector<String> refRelations = new Vector<String>();
		for(int i=0; i<dedSetHistory.size(); i++) {
			DEDSet dedSet = dedSetHistory.dedSets.get(i);
			for(int j=0; j<dedSet.size(); j++) {
				DED ded = dedSet.deds.get(j);
				// global quantifiers
				for(int k=0; k<ded.globalQuantifiers.size(); k++) {
					String relationName = ded.globalQuantifiers.get(k).relation;
					if(REFConstant.hasREFPrefix(relationName) &&
							//!REFConstant.isInequalityREF(relationName) &&
							!refRelations.contains(relationName))
						refRelations.add(relationName);
				}
				// then-clause quantifiers
				for(int k2=0; k2<ded.thenClauses.size(); k2++) {
					QuantifiedDisjunctive qd = ded.thenClauses.get(k2);
					for(int k3=0; k3<qd.quantifiers.size(); k3++) {
						String relationName = qd.quantifiers.get(k3).relation;
						if(REFConstant.hasREFPrefix(relationName) &&
								//!REFConstant.isInequalityREF(relationName) &&
								!refRelations.contains(relationName))
							refRelations.add(relationName);
					}
				}
			}
		}
		
		Vector<Integer> numAttrs = new Vector<Integer>();
		for(int i=0; i<refRelations.size(); i++) {
			int numArg = REFConstant.getNumOfArgs(refRelations.get(i)); 
			int numAttr = numArg + 1;
			if(!numAttrs.contains(new Integer(numAttr))) {
				numAttrs.add(new Integer(numAttr));
			}
			schemaDec += "Schema " + refRelations.get(i) + " : Coll { SET of type_attr_" +  (numArg+1) + " };<br>";
		}
		
		for(int i=0; i<numAttrs.size(); i++) {
			int numAttr = numAttrs.get(i);
			String oneTypeDec = "Type type_attr_" + numAttr + " = Record [ ";
			for(int j=0; j<numAttr; j++) {
				oneTypeDec += "a" + j + " : String";
				if(j<numAttr-1)
					oneTypeDec += ", ";
			}
			oneTypeDec += " ];<br>";
			typeDec += oneTypeDec;
		}
		
	
	
			output += typeDec + "<br>" + schemaDec + "<br>";
			return output;
	}
	
	
	
	/*
	static public void writeCodiPrelude(SQLUnionSet sqlUnions, 
			DEDSetHistory dedSetHistory, BufferedWriter out) {
		
		// convert dedSetHistory into dedSet
		DEDSet dedSet = null;
		if(dedSetHistory != null) {
			for(int i=0; i<dedSetHistory.dedSets.size(); i++) {
				dedSet.deds.addAll(dedSetHistory.get(i).deds);
			}
		}
		
		// call the original method
		writeCodiPrelude(sqlUnions, dedSet, out);
	}
	*/

	// not used currently, 10-3-2007
	static private void writeCodiPrelude(BufferedWriter out) {
		int maxArg = 10;
        try {
        	for(int i=1; i<=maxArg; i++) {
        		String str = "Type type" + i + " = Record [ ";
        		for(int j=1; j<=maxArg; j++) {
        			str += "a" + (j-1) + " : String"; // begins from a0
        			if(j<maxArg)
        				str += ",";
        		}
        		str += " ];\n";
        		out.write(str);
        	}
        	/*
       		out.write("Type OneArgTy = Record [ a0 : String ];\n");
			out.write("Type TwoArgTy = Record [ a0 : String, a1 : String ];\n");
			out.write("Type ThreeArgTy = Record [ a0 : String, a1 : String, a2 : String ];\n\n");
        	 */
			
        	// added by Hyun (8/2/2007)
        	for(int i=1; i<=maxArg; i++) {
            	out.write("Schema v2xb" + i + ": Coll { SET of type" + i + "};\n");
        	}
        	// end

			out.write("Schema v1root: Coll { SET of type1 };\n");
			out.write("Schema v1child: Coll { SET of type2 };\n");
			out.write("Schema v1tag: Coll { SET of type2 };\n");
			out.write("Schema v1text: Coll { SET of type2 };\n");
			out.write("Schema v1attr: Coll { SET of type3 };\n");
			out.write("Schema v2root: Coll { SET of type1 };\n");
			out.write("Schema v2child: Coll { SET of type2 };\n");
			out.write("Schema v2tag: Coll { SET of type2 };\n");
			out.write("Schema v2text: Coll { SET of type2 };\n");
			out.write("Schema v2attr: Coll { SET of type3 };\n\n");

        	/*
			out.write("Schema v1root: Coll { SET of OneArgTy };\n");
			out.write("Schema v1child: Coll { SET of TwoArgTy };\n");
			out.write("Schema v1tag: Coll { SET of TwoArgTy };\n");
			out.write("Schema v1text: Coll { SET of TwoArgTy };\n");
			out.write("Schema v1attr: Coll { SET of ThreeArgTy };\n");
			out.write("Schema v2root: Coll { SET of OneArgTy };\n");
			out.write("Schema v2child: Coll { SET of TwoArgTy };\n");
			out.write("Schema v2tag: Coll { SET of TwoArgTy };\n");
			out.write("Schema v2text: Coll { SET of TwoArgTy };\n");
			out.write("Schema v2attr: Coll { SET of ThreeArgTy };\n\n");
			*/
        } catch (IOException e) {}
	}
	
	static private void writeCodiEpilogueCompile(BufferedWriter out) {
		try {
			out.write("\nQuery Qi = Compile Q2;\n");
		} catch (IOException e) {}
	}
	
	static private void writeCodiEpilogueChase(BufferedWriter out) {
		try {
			//out.write("\nQuery Qi = Compile Q2;\n");
			out.write("Query Qc = Chase Qi;\n");
			out.write("Print[Qc];\n");
			//out.write("Query Qo = Translate Qi against [v1root, v1child, v1tag, v1text, v1attr];\n");
			//out.write("Print[Qo];\n");
		} catch (IOException e) {}
	}

	// not used currently, 10-3-2007
	static private void writeCodiEpilogueTranslate(BufferedWriter out) {
		int maxArg = 10;
		try {
			String translateTarget = "";
			translateTarget += "v1root, v1child, v1tag, v1text, v1attr, ";
			for(int i=1; i<=maxArg; i++) {
				translateTarget += "xb" + i;
				if(i<maxArg)
					translateTarget += ", ";
			}
			translateTarget = "[" + translateTarget + "]";
			out.write("Query Qo = Translate Qi against " + translateTarget + ";\n");
			out.write("Print[Qo];\n");
		} catch (IOException e) {}
	}
}
