package edu.ucla.cs.db.prism.backend.marsManage;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.SQL.SQLUnionSet;
import edu.ucla.cs.db.prism.backend.ded.DED;
import edu.ucla.cs.db.prism.backend.ded.DEDSet;
import edu.ucla.cs.db.prism.backend.ded.DEDSetHistory;
import edu.ucla.cs.db.prism.backend.ded.QuantifiedDisjunctive;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchemaHistory;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICTable;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequenceHistory;
import edu.ucla.cs.db.prism.backend.util.REFConstant;

public class ICCodiGen extends CodiGen {

	public ICCodiGen() {
		// TODO Auto-generated constructor stub
	}
	
	static public void generateOneCodiScriptMultipleQueries(SQLUnionSet sqlUnions, 
			ICSchemaHistory schemaHistory, SMOSequenceHistory smoSequenceHistory, DEDSet deds,
			int numSteps, String codiScriptFileName) {
		// file writing preparation
        BufferedWriter out = null;
        try {
        	// chase & backchase in a single codie script
        	out = new BufferedWriter(new FileWriter(codiScriptFileName));
        	

        	// step 1: write schema declaration
        	writeCodiPrelude(sqlUnions, deds, out);
        	
        	// step 2: write deds
        	out.write(deds.toStringNoMetadata());
        	
        	// step 3: write input query
        	writeInputQueries(sqlUnions, out);
        	
        	// step 4: write rewriting commands
        	writeChaseCommandsOld(sqlUnions, numSteps, out);
        	
        	out.close();
        } catch (IOException e) {}
	}
	

	static public void writeCodiPrelude(ICSchemaHistory schemaHistory, 
			DEDSetHistory dedSetHistory, SQLUnionSet inputSQLUnionSet, 
			BufferedWriter out) {
		

		String integrityConstraintsDec = "";

		// step 1: to make types a minimal set, 
		//         don't repeat the same type if already defined
		ICSchema schemaForTypeDef = new ICSchema();
		for(int i=0; i<schemaHistory.size(); i++) {
			ICSchema schema = schemaHistory.getSchema(i);
			for(int j=0; j<schema.size(); j++) {
				ICTable table = schema.getTable(j);
				if(!schemaForTypeDef.containsTable(table))
					schemaForTypeDef.addTable(table);
			}
		}

		String typeDec = "";
		for(int i=0; i<schemaForTypeDef.size(); i++) {
			typeDec += "Type type" + i + " = Record [";
			ICTable table = schemaForTypeDef.getTable(i);
			for(int j=0; j<table.getNumColumns(); j++) {
				
				// CARLO TRYING TO ADD TYPES TO SCHEMA DEFINITION
				//typeDec += " " + table.getColumn(j) + " : String";
				typeDec += " " + table.getColumn(j) + " : " + table.getMarsColType(j);
				if(j<table.getNumColumns()-1)
					typeDec += ",";
			}
			typeDec += " ];\n";
		}
		
		
		// MANAGEMENT OF > < >= <=
		typeDec += "Type relational = Record [ a0 : String, a1 : String ];\n";

		int typeIndex=-1;
		

		// step 2: 
		String schemaDec = "";
		for(int i=0; i<schemaHistory.size(); i++) {
			ICSchema schema = schemaHistory.getSchema(i);
			for(int j=0; j<schema.ictables.size(); j++) {
				
				
				ICTable table	= schema.ictables.get(j);

				typeIndex = schemaForTypeDef.indexOf(table);
				
				String tableprefix = "";
				
				if(schema.isHTableSchema){
					tableprefix +=" vh" + schema.versionID;
					//schemaDec += "Schema vh" + schema.versionID;
				}
				else{
					tableprefix +=" v" + schema.versionID;
					//schemaDec += "Schema v" + schema.versionID;
				}
				
				tableprefix +="m" + schema.minorVersionID +"_";
				schemaDec += "Schema " + tableprefix + table.getTableName() + " : Coll { SET of type" + 
							typeIndex + " };\n";
				

				integrityConstraintsDec += table.getConstraintsAsDEDString(tableprefix);
			}
		}
		
		
		

		// MANAGEMENT OF > < >= <=
		typeIndex++;
		//schemaDec += "Schema _ref2_less : Coll { SET of relational };\n";
		//schemaDec += "Schema _ref2_leq : Coll { SET of relational };\n";
		//schemaDec += "Schema _ref2_neq : Coll { SET of relational };\n";

		
		// step 3: for Relational-Encoded-Functions (REFs), coming from DEDs
		Vector<String> refRelations = new Vector<String>();
		if(dedSetHistory!=null)
		for(int i=0; i<dedSetHistory.size(); i++) {
			DEDSet dedSet = dedSetHistory.dedSets.get(i);
			for(int j=0; j<dedSet.size(); j++) {
				DED ded = dedSet.deds.get(j);
				// global quantifiers
				for(int k=0; k<ded.globalQuantifiers.size(); k++) {
					String relationName = ded.globalQuantifiers.get(k).relation;
					if(REFConstant.hasREFPrefix(relationName) &&
							//!REFConstant.isInequalityREF(relationName) &&
							!refRelations.contains(relationName))
						refRelations.add(relationName);
				}
				// then-clause quantifiers
				for(int k2=0; k2<ded.thenClauses.size(); k2++) {
					QuantifiedDisjunctive qd = ded.thenClauses.get(k2);
					for(int k3=0; k3<qd.quantifiers.size(); k3++) {
						String relationName = qd.quantifiers.get(k3).relation;
						if(REFConstant.hasREFPrefix(relationName) &&
								//!REFConstant.isInequalityREF(relationName) &&
								!refRelations.contains(relationName))
							refRelations.add(relationName);
					}
				}
			}
		}
		// REFs coming from queries
		Vector<String> refRelationsInQuery = 
			REFConstant.getREFRelationNames(inputSQLUnionSet);
				
		for(int i=0; i<refRelationsInQuery.size(); i++) {
			if(!refRelations.contains(refRelationsInQuery.get(i)))
				refRelations.add(refRelationsInQuery.get(i));
		}
		
		Vector<Integer> numAttrs = new Vector<Integer>();
		for(int i=0; i<refRelations.size(); i++) {
			int numArg = REFConstant.getNumOfArgs(refRelations.get(i)); 
			int numAttr = numArg + 1;
			if(!numAttrs.contains(new Integer(numAttr))) {
				numAttrs.add(new Integer(numAttr));
			}
			
			schemaDec += "Schema " + refRelations.get(i) + " : Coll { SET of type_attr_" +  (numArg+1) + " };\n";
			
		}
		
		for(int i=0; i<numAttrs.size(); i++) {
			int numAttr = numAttrs.get(i);
			String oneTypeDec = "Type type_attr_" + numAttr + " = Record [ ";
			for(int j=0; j<numAttr; j++) {
				oneTypeDec += "a" + j + " : String";
				if(j<numAttr-1)
					oneTypeDec += ", ";
			}
			oneTypeDec += " ];\n";
			typeDec += oneTypeDec;
		}
		
		
		
		try {
			out.write(typeDec + "\n" + schemaDec + "\n" + integrityConstraintsDec);
		}
		catch(Exception e) {
			e.printStackTrace();
		}
	
	}

	
	static public void writeTranslateCommands(SQLUnionSet sqlUnions, BufferedWriter out, int chaseIndex,
			ICSchemaHistory schemaHistory, DEDSetHistory dedSetHistory, 
			int sourceVersion, int sourceMinorVersion, Vector<String> sourceTablesForTranslate) {

		// construct a source relation list
		String srcTableList = "";
		// use passed sourceTablesForTranslate, if not null
		// extract it from schemaHistory, otherwise
		if(sourceTablesForTranslate != null) {
			for(int i=0; i<sourceTablesForTranslate.size(); i++) {
				srcTableList += sourceTablesForTranslate.get(i);
				if(i<sourceTablesForTranslate.size()-1)
					srcTableList += ",";
			}
		}
		else {
			ICSchema srcSchema = schemaHistory.getSchemaByVersion(sourceVersion,sourceMinorVersion);
			for(int i=0; i<srcSchema.ictables.size(); i++) {
				srcTableList += "v" + sourceVersion + "m" +sourceMinorVersion + "_"  + srcSchema.ictables.get(i).getTableName();
				if(i<srcSchema.ictables.size()-1)
					srcTableList += ",";
			}
		}
		
		// for Relational-Encoded-Functions (REFs)
		Vector<String> refRelations = new Vector<String>();
		for(int i=0; i<dedSetHistory.size(); i++) {
			DEDSet dedSet = dedSetHistory.dedSets.get(i);
			for(int j=0; j<dedSet.size(); j++) {
				DED ded = dedSet.deds.get(j);
				// global quantifiers
				for(int k=0; k<ded.globalQuantifiers.size(); k++) {
					String relationName = ded.globalQuantifiers.get(k).relation;
					if(REFConstant.hasREFPrefix(relationName) &&
							//!REFConstant.isInequalityREF(relationName) &&
							!refRelations.contains(relationName))
						refRelations.add(relationName);
				}
				// then-clause quantifiers
				for(int k2=0; k2<ded.thenClauses.size(); k2++) {
					QuantifiedDisjunctive qd = ded.thenClauses.get(k2);
					for(int k3=0; k3<qd.quantifiers.size(); k3++) {
						String relationName = qd.quantifiers.get(k3).relation;
						if(REFConstant.hasREFPrefix(relationName) &&
								//!REFConstant.isInequalityREF(relationName) &&
								!refRelations.contains(relationName))
							refRelations.add(relationName);
					}
				}
			}
		}
		
		// REFs coming from queries
		Vector<String> refRelationsInQuery = 
			REFConstant.getREFRelationNames(sqlUnions);
		for(int i=0; i<refRelationsInQuery.size(); i++) {
			if(!refRelations.contains(refRelationsInQuery.get(i)))
				refRelations.add(refRelationsInQuery.get(i));
		}
		
		String refList = "";
		for(int i=0; i<refRelations.size(); i++) {
			refList += refRelations.get(i);
			if(i<refRelations.size()-1)
				refList += ", ";
		}
		if(refList.length() > 0)
			refList = ", " + refList;
		srcTableList = "[" + srcTableList + refList + "]";
		
		try {
        	for(int i=0; i<sqlUnions.sqlUnions.size(); i++) {
        		String inputQueryName = "Q" + sqlUnions.sqlUnions.get(i).name;
        		if(chaseIndex == 0)
        			inputQueryName += "_compiled";
        		else
        			inputQueryName += "_chased" + (chaseIndex-1);
        		String outputQueryName = "Q" + sqlUnions.sqlUnions.get(i).name +
        			"_chased" + chaseIndex;
        		
    			out.write("Query " + outputQueryName+ 
    					" = Translate " + inputQueryName + 
    					" against " + srcTableList + ";\n");
        	}
        } catch (IOException e) {}	
	}
	
	
	
	
	
	
	
	
	
	static public String writeHTMLCodiPrelude(ICSchemaHistory schemaHistory, 
			DEDSetHistory dedSetHistory) {
		
		//System.err.println("ICCodigen.writeHTMLCodiPrelude invoked");
		
		String output = "";
		// step 1: to make types a minimal set, 
		//         don't repeat the same type if already defined
		ICSchema schemaForTypeDef = new ICSchema();
		for(int i=0; i<schemaHistory.size(); i++) {
			ICSchema schema = schemaHistory.getSchema(i);
			for(int j=0; j<schema.size(); j++) {
				ICTable table = schema.getTable(j);
				if(!schemaForTypeDef.containsTable(table))
					schemaForTypeDef.addTable(table);
			}
		}

		String typeDec = "";
		for(int i=0; i<schemaForTypeDef.size(); i++) {
			typeDec += "Type type" + i + " = Record [";
			ICTable table = schemaForTypeDef.getTable(i);
			for(int j=0; j<table.getNumColumns(); j++) {
				typeDec += " " + table.getColumn(j) + " : String";
				if(j<table.getNumColumns()-1)
					typeDec += ",";
			}
			typeDec += " ];<br>";
		}
		
		
		// MANAGEMENT OF > < >= <=
		typeDec += "Type relational = Record [ a0 : String, a1 : String ];<br>";

		int typeIndex=-1;
		
		
		// step 2: 
		String schemaDec = "";
		for(int i=0; i<schemaHistory.size(); i++) {
			ICSchema schema = schemaHistory.getSchema(i);
			for(int j=0; j<schema.ictables.size(); j++) {
				ICTable table = schema.ictables.get(j);
				typeIndex = schemaForTypeDef.indexOf(table);
				if(schema.isHTableSchema)
					schemaDec += "Schema vh" + schema.versionID;
				else
					schemaDec += "Schema v" + schema.versionID;
				schemaDec += "m" + schema.minorVersionID;
				schemaDec += "_" + table.getTableName() + " : Coll { SET of type" + 
							typeIndex + " };<br>";
			}
		}

		// MANAGEMENT OF > < >= <=
		typeIndex++;
		//schemaDec += "Schema _ref2_less : Coll { SET of relational };\n";
		//schemaDec += "Schema _ref2_leq : Coll { SET of relational };\n";
		//schemaDec += "Schema _ref2_neq : Coll { SET of relational };\n";

		
		// step 3: for Relational-Encoded-Functions (REFs), coming from DEDs
		Vector<String> refRelations = new Vector<String>();
		for(int i=0; i<dedSetHistory.size(); i++) {
			DEDSet dedSet = dedSetHistory.dedSets.get(i);
			for(int j=0; j<dedSet.size(); j++) {
				DED ded = dedSet.deds.get(j);
				// global quantifiers
				for(int k=0; k<ded.globalQuantifiers.size(); k++) {
					String relationName = ded.globalQuantifiers.get(k).relation;
					if(REFConstant.hasREFPrefix(relationName) &&
							//!REFConstant.isInequalityREF(relationName) &&
							!refRelations.contains(relationName))
						refRelations.add(relationName);
				}
				// then-clause quantifiers
				for(int k2=0; k2<ded.thenClauses.size(); k2++) {
					QuantifiedDisjunctive qd = ded.thenClauses.get(k2);
					for(int k3=0; k3<qd.quantifiers.size(); k3++) {
						String relationName = qd.quantifiers.get(k3).relation;
						if(REFConstant.hasREFPrefix(relationName) &&
								//!REFConstant.isInequalityREF(relationName) &&
								!refRelations.contains(relationName))
							refRelations.add(relationName);
					}
				}
			}
		}
		
		Vector<Integer> numAttrs = new Vector<Integer>();
		for(int i=0; i<refRelations.size(); i++) {
			int numArg = REFConstant.getNumOfArgs(refRelations.get(i)); 
			int numAttr = numArg + 1;
			if(!numAttrs.contains(new Integer(numAttr))) {
				numAttrs.add(new Integer(numAttr));
			}
			schemaDec += "Schema " + refRelations.get(i) + " : Coll { SET of type_attr_" +  (numArg+1) + " };<br>";
		}
		
		for(int i=0; i<numAttrs.size(); i++) {
			int numAttr = numAttrs.get(i);
			String oneTypeDec = "Type type_attr_" + numAttr + " = Record [ ";
			for(int j=0; j<numAttr; j++) {
				oneTypeDec += "a" + j + " : String";
				if(j<numAttr-1)
					oneTypeDec += ", ";
			}
			oneTypeDec += " ];<br>";
			typeDec += oneTypeDec;
		}
		
	
	
			output += typeDec + "<br>" + schemaDec + "<br>";
			return output;
	}
	
	


}
