package edu.ucla.cs.db.prism.backend.marsManage;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.SQL.*;
import edu.ucla.cs.db.prism.backend.parser.parser.*;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.*;
import edu.ucla.cs.db.prism.backend.parser.visitor.SmoTranslateVisitor;
import edu.ucla.cs.db.prism.backend.util.PrimaExperimentResult;
import errors.MarsException;

public class MarsResultParser {

	// retrieve as many as queryNames.size()
	static public SQLUnionSet getRewrittenXBind(String marsResultFileName, 
			Vector<String> queryNames, Vector<Integer> sourceVersions) {
		//String rewrittenXBindQueries = "";
		BufferedReader bufRead = null;
		SQLUnionSet oneChasePointSQLUnionSet = null;
		SQLUnionSet allSQLUnionSet = null;

		// step 1: file open
		try {
			bufRead = new BufferedReader(new FileReader(marsResultFileName));
		} catch (FileNotFoundException e) {
			System.out.println("Error: mars result file " + marsResultFileName 
					+ " is not found.");
			throw new MarsException("Error: mars result file " + marsResultFileName 
					+ " is not found.");
			
			//System.exit(-1);
		}
		
		allSQLUnionSet = new SQLUnionSet();
		boolean eofReached = false;
		
		if(sourceVersions == null) { // only one source version
			sourceVersions = new Vector<Integer>();
			sourceVersions.add(new Integer(-1));
		}
		
		SQLUnionSet sqlUnionSet = new SQLUnionSet();
		for(int i=0; i<sourceVersions.size(); i++) {
			//System.out.println("starting a loop");
			for(int j=0; j<queryNames.size(); j++) {
				try{
					// step 2: search for the rewritten query part preamble
					if(!locateFirstRewrittenXBind(bufRead)) {
						eofReached = true;
						break;
					}
					// step 3: parse the rewritten query
					String oneRewrittenXBindQuery = readFirstRewrittenXBind(bufRead);
					if(oneRewrittenXBindQuery == null) {// couldn't read an many queries as queryNames.size()
						sqlUnionSet.sqlUnions.add(null); // empty rewritten query
					}
					else {
						oneRewrittenXBindQuery = "QUERY " + queryNames.get(j) + " := " + oneRewrittenXBindQuery + ";\n";

						// parse it and create SQLUnion
						try {
							ByteArrayInputStream bais = new ByteArrayInputStream(oneRewrittenXBindQuery.getBytes());
							
							// step 5: parse it using our general PRIMA parser
							PrimaRParser parser = new PrimaRParser(bais);
							SQLQueryDeclaration script = null;
							try {
								script = parser.SQLQueryDeclaration();
							} catch(Exception e) {
								e.printStackTrace();
							}
				
							SmoTranslateVisitor smotv = new SmoTranslateVisitor();
							script.accept(smotv);
							
							sqlUnionSet.sqlUnions.add(smotv.sqlUnion);
						} catch(Exception e) {
							e.printStackTrace();
						}
					}
				} catch(Exception e) {}	
			}
			if(eofReached)
				break;
		}
		try {
			bufRead.close();
		} catch(Exception e) {}
		
		return sqlUnionSet;
	}
	static public SQLUnionSet getRewrittenXBindTogether(String marsResultFileName, 
			Vector<String> queryNames, Vector<Integer> sourceVersions) {
		String rewrittenXBindQueries = "";
		BufferedReader bufRead = null;
		SQLUnionSet oneChasePointSQLUnionSet = null;
		SQLUnionSet allSQLUnionSet = null;

		// step 1: file open
		try {
			bufRead = new BufferedReader(new FileReader(marsResultFileName));
		} catch (FileNotFoundException e) {
			System.out.println("Error: mars result file " + marsResultFileName 
					+ " is not found.");
			throw new MarsException("Error: mars result file " + marsResultFileName 
					+ " is not found.");
			//System.exit(-1);
		}
		
		allSQLUnionSet = new SQLUnionSet();
		boolean eofReached = false;
		
		if(sourceVersions == null) { // only one source version
			sourceVersions = new Vector<Integer>();
			sourceVersions.add(new Integer(-1));
		}
		for(int i=0; i<sourceVersions.size(); i++) {
			//System.out.println("starting a loop");
			rewrittenXBindQueries = "";
			for(int j=0; j<queryNames.size(); j++) {
				try{
					// step 2: search for the rewritten query part preamble
					if(!locateFirstRewrittenXBind(bufRead)) {
						eofReached = true;
						break;
					}
					// step 3: store into rewrittenXBind
					String oneRewrittenXBindQuery = readFirstRewrittenXBind(bufRead);
					if(oneRewrittenXBindQuery != null) {// couldn't read an many queries as queryNames.size()
						oneRewrittenXBindQuery = "QUERY " + queryNames.get(j) + " := " + oneRewrittenXBindQuery + ";\n";
						rewrittenXBindQueries += oneRewrittenXBindQuery;
					}
					/*
					for(int i=0; i<queryNames.size(); i++) {
						String oneRewrittenXBindQuery = readFirstRewrittenXBind(bufRead);
						if(oneRewrittenXBindQuery == null) // couldn't read an many queries as queryNames.size()
							return null;
							
						oneRewrittenXBindQuery = "QUERY " + queryNames.get(i) + " := " + oneRewrittenXBindQuery + ";\n";
						
						//System.out.println(oneRewrittenXBindQuery);
						rewrittenXBindQueries += oneRewrittenXBindQuery;
					}
					*/
				} catch(Exception e) {}	
			}
			if(eofReached)
				break;
			
			// empty result
			if(rewrittenXBindQueries.length() == 0)
				return null;
			
			// step 4: create an input stream out of xbind String
			try {
				ByteArrayInputStream bais = new ByteArrayInputStream(rewrittenXBindQueries.getBytes());
				
				// step 5: parse it using our general PRIMA parser
				PrimaRParser parser = new PrimaRParser(bais);
				PrimaRScript script = null;
				try {
					script = parser.PrimaRScript();
				} catch(Exception e) {
					e.printStackTrace();
				}
	
				SmoTranslateVisitor smotv = new SmoTranslateVisitor();
				script.accept(smotv);
				
				//bufRead.close();
				
				//System.out.println(smotv.sqlUnionSet);
				oneChasePointSQLUnionSet =  smotv.sqlUnionSet;
				for(int k=0; k<oneChasePointSQLUnionSet.sqlUnions.size(); k++) {
					SQLUnion sqlUnion = oneChasePointSQLUnionSet.sqlUnions.get(k);
					if(sqlUnion == null) continue;
					// source version information will be used by postprocessing
					//sqlUnion.sourceVersion = sourceVersions.get(i).intValue();
				}
			} catch(Exception e) {
				e.printStackTrace();
			}
			
			//mergeSQLUnionsFromMultipleChasePoints(allSQLUnionSet, oneChasePointSQLUnionSet);
			allSQLUnionSet.sqlUnions.addAll(oneChasePointSQLUnionSet.sqlUnions);
		}
		try {
			bufRead.close();
		} catch(Exception e) {}
		return allSQLUnionSet;
	}

	/*
	static public SQLUnion getFirstRewrittenXBind(String marsResultFileName, 
			String queryName) {
		String rewrittenXBind = "";
		BufferedReader bufRead = null;
		
		// step 1: file open
		try {
			bufRead = new BufferedReader(new FileReader(marsResultFileName));
		} catch (FileNotFoundException e) {
			System.out.println("Error: mars result file " + marsResultFileName 
					+ " is not found.");
			System.exit(-1);
		}
		
		try{
			// step 2: search for the rewritten query part preamble
			if(!locateFirstRewrittenXBind(bufRead))
				return null;
		
			// step 3: store into rewrittenXBind
			rewrittenXBind = readFirstRewrittenXBind(bufRead);
			rewrittenXBind = "QUERY " + queryName + " := " + rewrittenXBind + ";";
			
			
			// step 4: create a input stream out of xbind String
			ByteArrayInputStream bais = new ByteArrayInputStream(rewrittenXBind.getBytes());
			
			// step 5: parse it using our general PRIMA parser
			PrimaRParser parser = new PrimaRParser(bais);
			PrimaRScript script = null;
			try {
				script = parser.PrimaRScript();
			} catch(Exception e) {
				e.printStackTrace();
			}

			SmoTranslateVisitor smotv = new SmoTranslateVisitor();
			script.accept(smotv);
			return smotv.sqlUnionSet.sqlUnions.get(0);
			
		} catch(Exception e) {
			e.printStackTrace();
		}
		return null;
	}
	*/
	
	// merge by query name
	static private void mergeSQLUnionsFromMultipleChasePoints(SQLUnionSet allSQLUnionSet, SQLUnionSet partialSQLUnionSet) {
		for(int i=0; i<partialSQLUnionSet.sqlUnions.size(); i++) {
			SQLUnion newSQLUnion = partialSQLUnionSet.sqlUnions.get(i);
			boolean sameNamedSQLUnionFound = false;
			for(int j=0; j<allSQLUnionSet.sqlUnions.size(); j++) {
				SQLUnion existingSQLUnion = allSQLUnionSet.sqlUnions.get(j);
				if(newSQLUnion.name.equals(existingSQLUnion.name)) {
					existingSQLUnion.sqls.addAll(newSQLUnion.sqls);
					sameNamedSQLUnionFound = true;
					break;
				}
			}
			if(!sameNamedSQLUnionFound)
				allSQLUnionSet.sqlUnions.add(newSQLUnion);
		}
	}
	
	
	
	// search for the rewritten query part preamble (i.e. "[")
	static private boolean locateFirstRewrittenXBind(BufferedReader bufRead) 
	throws IOException {
		while(true) {
			String line = bufRead.readLine();
			if(line == null) return false;
			if(line.startsWith("[") && line.length() == 1) break;
		}
		return true;
	}
	
	// store into str until
	//         1) "]" is met or
	//         2) SELECT with no preceding UNION is met
	static private String readFirstRewrittenXBind(BufferedReader bufRead) 
	throws IOException {
		String str = "";
		boolean unionMet = true; // to pass the first select
		while(true) {
			bufRead.mark(10000); // to rewind the already-read next query line
			String line = bufRead.readLine();
			if(line == null) return null;

			if(line.startsWith("    UNION")) {
				unionMet = true;
				str += line + "\n";
			}
			else if(line.startsWith("    SELECT")) {
				if(unionMet) {
					unionMet = false;
					str += line + "\n";
				}
				else
					break;
			}
			else if(line.startsWith("]")) 
				break;
			else
				str += line + "\n";
		}
		bufRead.reset(); // rewind the already-read next query line
		
		if(str.equals("") || str.equals("\n"))
			return null; // no query read
		return str;
	}
	
	// return the chase time sum in milliseconds
	static public void getChaseTime(String marsResultFileName, PrimaExperimentResult expResult) {
		BufferedReader bufRead = null;
		long chaseTime = 0;
		// step 1: file open
		try {
			bufRead = new BufferedReader(new FileReader(marsResultFileName));
		} catch (FileNotFoundException e) {
			System.out.println("Error: mars result file " + marsResultFileName 
					+ " is not found.");
			
			throw new MarsException("Error: mars result file " + marsResultFileName 
					+ " is not found.");
			//System.exit(-1);
		}

		
		try{
			// step 2: search for the rewritten query part preamble
			while(true) {
				String line = bufRead.readLine();
				if(line == null) break; // end of file
				else if(line.startsWith("Time spent in chase = ")) {
					int endIndex = line.indexOf(" ms");
					String oneChaseTimeStr = line.substring(22, endIndex);
					chaseTime += Long.parseLong(oneChaseTimeStr);
				}
				else if(line.startsWith("Time for parsing: ")) {
					int endIndex = line.indexOf(" ms");
					String str = line.substring(18, endIndex);
					expResult.mars_input_parse_time += Long.parseLong(str);
				}
				else if(line.startsWith("Time for semantics + execution: ")) {
					int endIndex = line.indexOf(" ms");
					String str = line.substring(32, endIndex);
					expResult.mars_semex_time += Long.parseLong(str);
				}
			}
		}
		catch(Exception e) {
			e.printStackTrace();
		}
		
		expResult.mars_chase_time = chaseTime;
	}

	
}
