package edu.ucla.cs.db.prism.backend.marsManage;

import edu.ucla.cs.db.prism.backend.SQL.*;
import edu.ucla.cs.db.prism.backend.ded.*;
import edu.ucla.cs.db.prism.backend.marsManage.exception.MarsConstantsSuffixException;
import edu.ucla.cs.db.prism.backend.smo.util.*;

/**
 * This class has several methods for quick fixes for mars specific bugs/bad behaviors
 * @author hjmoon
 *
 */
public class MarsSpecificFix {
	
	// fix 1: mars says ("a" = "a") is true, but ("" = "") is false
	//        add "_" as a sufix to all constants, before calling mars
	public static final String suffix = "0";
	public static void addSuffixToConstants(SQLUnionSet sqlUnionSet) throws MarsConstantsSuffixException {
		if(sqlUnionSet == null || sqlUnionSet.sqlUnions == null) return;
		for(int i=0; i<sqlUnionSet.sqlUnions.size(); i++) {
			addSuffixToConstants(sqlUnionSet.sqlUnions.get(i));
		}
	}
	public static void addSuffixToConstants(SQLUnion sqlUnion) throws MarsConstantsSuffixException {
		if(sqlUnion == null || sqlUnion.sqls == null) return;
		for(int i=0; i<sqlUnion.sqls.size(); i++) {
			addSuffixToConstants(sqlUnion.sqls.get(i));
		}
	}
	public static void addSuffixToConstants(SQL sql) throws MarsConstantsSuffixException {
		if(sql == null) return;

		// select clause
		for(int i=0; i<sql.select.selectAttributesSize(); i++) {
			SelectAttribute sa = sql.select.selectAttributes.get(i);
			if(sa.constant != null)
				sa.constant = addSuffixToConstant(sa.constant);
		}
			
		// where clause
		if(sql.where == null || sql.where.conjunctives == null ||
				sql.where.conjunctives.size() == 0 ||
				sql.where.conjunctives.get(0).predicates == null)
			return;
		for(int i=0; i<sql.where.conjunctives.get(0).predicates.size(); i++) {
			Predicate p = sql.where.conjunctives.get(0).predicates.get(i);
			if(p.term1.constant != null)
				p.term1.constant = addSuffixToConstant(p.term1.constant);
			if(p.term2.constant != null)
				p.term2.constant = addSuffixToConstant(p.term2.constant);		
		}
	}
	
	public static void stripSuffixFromConstants(SQLUnionSet sqlUnionSet) throws MarsConstantsSuffixException {
		if(sqlUnionSet == null || sqlUnionSet.sqlUnions == null) return;
		for(int i=0; i<sqlUnionSet.sqlUnions.size(); i++) {
			stripSuffixFromConstants(sqlUnionSet.sqlUnions.get(i));
		}
	}
	public static void stripSuffixFromConstants(SQLUnion sqlUnion) throws MarsConstantsSuffixException {
		if(sqlUnion == null || sqlUnion.sqls == null) return;
		for(int i=0; i<sqlUnion.sqls.size(); i++) {
			stripSuffixFromConstants(sqlUnion.sqls.get(i));
		}
	}
	public static void stripSuffixFromConstants(SQL sql) throws MarsConstantsSuffixException {
		if(sql == null) return;

		// select clause
		for(int i=0; i<sql.select.selectAttributesSize(); i++) {
			SelectAttribute sa = sql.select.selectAttributes.get(i);
			sa.constant = stripSuffixFromConstant(sa.constant);
		}
			
		// where clause
		if(sql.where == null || sql.where.conjunctives == null ||
				sql.where.conjunctives.size() == 0 ||
				sql.where.conjunctives.get(0).predicates == null)
			return;
		for(int i=0; i<sql.where.conjunctives.get(0).predicates.size(); i++) {
			Predicate p = sql.where.conjunctives.get(0).predicates.get(i);
			p.term1.constant = stripSuffixFromConstant(p.term1.constant);
			p.term2.constant = stripSuffixFromConstant(p.term2.constant);		
		}
	}

	public static void addSuffixToConstants(DEDSetHistory dedSetHistory) throws MarsConstantsSuffixException {
		if(dedSetHistory == null || dedSetHistory.dedSets == null) return;
		for(int i=0; i<dedSetHistory.dedSets.size(); i++)
			addSuffixToConstants(dedSetHistory.dedSets.get(i));
	}
	public static void addSuffixToConstants(DEDSet dedSet) throws MarsConstantsSuffixException {
		if(dedSet == null || dedSet.deds == null) return;
		for(int i=0; i<dedSet.deds.size(); i++)
			addSuffixToConstants(dedSet.deds.get(i));
	}
	public static void addSuffixToConstants(DED ded) throws MarsConstantsSuffixException {
		if(ded == null) return;

		// if-clause
		boolean skipIfClause = false;
		if(ded.ifClause.disjunctive == null ||
				ded.ifClause.disjunctive.conjunctives == null ||
				ded.ifClause.disjunctive.conjunctives.size() == 0 ||
				ded.ifClause.disjunctive.conjunctives.get(0).predicates == null)
			skipIfClause = true;
		for(int i=0; !skipIfClause && i<ded.ifClause.disjunctive.conjunctives.get(0).predicates.size(); i++) {
			Predicate p = ded.ifClause.disjunctive.conjunctives.get(0).predicates.get(i);
			p.term1.constant = addSuffixToConstant(p.term1.constant);
			p.term2.constant = addSuffixToConstant(p.term2.constant);
		}
		
		// then-clauses
		if(ded.thenClauses == null)
			return;
		for(int i=0; i<ded.thenClauses.size(); i++) {
			QuantifiedDisjunctive tc = ded.thenClauses.get(i);
			if(tc.disjunctive == null || tc.disjunctive.conjunctives == null ||
					tc.disjunctive.conjunctives.size() == 0 ||
					tc.disjunctive.conjunctives.get(0).predicates == null)
				continue;
			for(int j=0; j<tc.disjunctive.conjunctives.get(0).predicates.size(); j++) {
				Predicate p = tc.disjunctive.conjunctives.get(0).predicates.get(j);
				p.term1.constant = addSuffixToConstant(p.term1.constant);
				p.term2.constant = addSuffixToConstant(p.term2.constant);
			}
		}
	}
	
	public static void stripSuffixFromConstants(DEDSetHistory dedSetHistory) throws MarsConstantsSuffixException {
		if(dedSetHistory == null || dedSetHistory.dedSets == null) return;
		for(int i=0; i<dedSetHistory.dedSets.size(); i++)
			stripSuffixFromConstants(dedSetHistory.dedSets.get(i));
	}
	public static void stripSuffixFromConstants(DEDSet dedSet) throws MarsConstantsSuffixException {
		if(dedSet == null || dedSet.deds == null) return;
		for(int i=0; i<dedSet.deds.size(); i++)
			stripSuffixFromConstants(dedSet.deds.get(i));
	}
	public static void stripSuffixFromConstants(DED ded) throws MarsConstantsSuffixException {
		if(ded == null) return;

		// if-clause
		boolean skipIfClause = false;
		if(ded.ifClause.disjunctive == null ||
				ded.ifClause.disjunctive.conjunctives == null ||
				ded.ifClause.disjunctive.conjunctives.size() == 0 ||
				ded.ifClause.disjunctive.conjunctives.get(0).predicates == null)
			skipIfClause = true;
		for(int i=0; !skipIfClause && i<ded.ifClause.disjunctive.conjunctives.get(0).predicates.size(); i++) {
			Predicate p = ded.ifClause.disjunctive.conjunctives.get(0).predicates.get(i);
			p.term1.constant = stripSuffixFromConstant(p.term1.constant);
			p.term2.constant = stripSuffixFromConstant(p.term2.constant);
		}
		
		// then-clauses
		if(ded.thenClauses == null)
			return;
		for(int i=0; i<ded.thenClauses.size(); i++) {
			QuantifiedDisjunctive tc = ded.thenClauses.get(i);
			if(tc.disjunctive == null || tc.disjunctive.conjunctives == null ||
					tc.disjunctive.conjunctives.size() == 0 ||
					tc.disjunctive.conjunctives.get(0).predicates == null)
				continue;
			for(int j=0; j<tc.disjunctive.conjunctives.get(0).predicates.size(); j++) {
				Predicate p = tc.disjunctive.conjunctives.get(0).predicates.get(j);
				p.term1.constant = stripSuffixFromConstant(p.term1.constant);
				p.term2.constant = stripSuffixFromConstant(p.term2.constant);
			}
		}
	}
	
	private static String addSuffixToConstant(String constant) throws MarsConstantsSuffixException {
		
		if(constant == null) 
			throw new MarsConstantsSuffixException("An error occurred while processing the addSuffixToConstant(): bad constant: " + constant);

		constant = singleQuote(constant);
		
		int length = constant.length();
		if(constant.charAt(0) == '"') {
			if(constant.charAt(length-1) == '"') { // good	
				//System.out.println(" ADD SUFFIX:" + constant.substring(0, length-2) + suffix + constant.substring(length-2, length));
				//FIXME this is tricky.. check if it should be -1 or -2
						
				return constant.substring(0, length-1) + suffix + constant.substring(length-1, length);
				
			}
			else { // bad - open quote, no close quote
				throw new MarsConstantsSuffixException("An error occurred while processing the addSuffixToConstant(): bad constant: " + constant);
			}
		}
		else { // no open quote - ok
			return constant + suffix;
		}
	}
	
	private static String singleQuote(String input){
		String output = "";
		for(int i=0; i<input.length(); i++){
			if(input.charAt(i)!='"')
				output += input.charAt(i);
		}
		
		return "\"" + output + "\"";
	}
	
	private static String stripSuffixFromConstant(String constant) throws MarsConstantsSuffixException {
		if(constant == null) return null;
		
		constant = singleQuote(constant);
		
		int length = constant.length();
		if(constant.charAt(0) == '"') {
			if(constant.charAt(length-1) == '"') { // good
				if(constant.substring(length-2, length-1).equals(suffix)){
					return constant.substring(0, length-2) + constant.substring(length-1, length);
				}else{
				//FIXME this was treated as an error, but it might be ok. removing error.
				//throw new MarsConstantsSuffixException("An error occurred while processing the stripSuffixFromConstant(): bad constant: " + constant);
					return constant;
				}
			}else
				throw new MarsConstantsSuffixException("An error occurred while processing the stripSuffixFromConstant(): bad constant: " + constant);

		}else
			if(constant.substring(length-1, length).equals(suffix))
				return constant.substring(0, length-1);
			else
				throw new MarsConstantsSuffixException("An error occurred while processing the stripSuffixFromConstant(): bad constant: " + constant);
	}
}
