package edu.ucla.cs.db.prism.backend.oldtestmain;

import it.polimi.elet.cautils.configurator.CACmdLineParser;
import it.polimi.elet.cautils.configurator.CAConfiguration;

import java.sql.Connection;
import java.util.Iterator;
import org.apache.log4j.Logger;

import edu.ucla.cs.db.prism.backend.exception.ErrorRewritingException;
import edu.ucla.cs.db.prism.backend.SQL.SQL;
import edu.ucla.cs.db.prism.backend.SQL.SQLUnion;
import edu.ucla.cs.db.prism.backend.compose.DEDComposition;
import edu.ucla.cs.db.prism.backend.ded.*;
import edu.ucla.cs.db.prism.backend.rewrite.PrimRewriteEngine;
import edu.ucla.cs.db.prism.backend.schema.*;
import edu.ucla.cs.db.prism.backend.smo.*;
import edu.ucla.cs.db.prism.backend.smo.util.*;
import edu.ucla.cs.db.prism.backend.util.Log4JHelper;
import edu.ucla.cs.db.prism.backend.util.Stopwatch;
import edu.ucla.cs.db.prism.backend.util.db.DBUtil;
import edu.ucla.cs.db.prism.gui.server.DBLoader;

public class MainComposeTest {

	//STATIC LOGGER
	static Logger logger = Logger.getLogger(DBLoader.class);
	
	public static void main(String[] args) {
		
		  //LOG4J SET UP
        Log4JHelper.log4JSetup(logger);
       
       
		//Use CAConfigurator to load configuration from shell or xml-configuration file (-c inputfile)
		 CACmdLineParser par = null;
	 	 try {
			par = CACmdLineParser.getInstance();
		 } catch (InstantiationException e) {
			e.printStackTrace();
		 }
		//List here all the interesting parameters
		 par.addStringParameter('h',"host");	
		 par.addStringParameter('u',"dburl");
		 par.addStringParameter('m',"metadbname");	
		 par.addStringParameter('u',"username");
		 par.addStringParameter('p',"password");
		 par.addStringParameter('l',"logfile");
		 
		//parse them all 
		 CAConfiguration conf = par.parse(args);
		 
		//read the parameter you are interested in 
		 String host = (String) conf.getParameterValue("host");
		 String dburl = (String) conf.getParameterValue("dburl");
		 String metadbname = (String) conf.getParameterValue("metadbname");
		 String dbname = (String) conf.getParameterValue("dbname");
		 String username = (String) conf.getParameterValue("username");
		 String password = (String) conf.getParameterValue("password");
		 
		//use them
		 System.out.println("PRIM rewriting query: ");
	//	 System.out.println("  - host: " + host);		
	//	 System.out.println("  - user: " + username);
	//	 System.out.println("  - metadb: " + metadbname);
		 
		// MySQL DRIVER LOADING 
       try {
           Class.forName("com.mysql.jdbc.Driver").newInstance();
       } catch (Exception ex) { ex.printStackTrace();}
	 

        Connection conn = DBUtil.connect(dburl,metadbname,username,password);
		    
       System.out.println("Connected to the DB:" + dburl +" " +metadbname+" " +username);
		 
		 SchemaHistoryLoader shl = new SchemaHistoryLoader(conn,dbname);
		 SchemaHistory schemaHistory = shl.loadSchemaHistoryFromDB();

		 logger.info("Schema Loaded");
		
		 SMOSequenceHistoryLoader sshl = new SMOSequenceHistoryLoader(conn,dbname);
	     SMOSequenceHistory ssh = sshl.loadSMOSequenceHistoryFromDB();
		 
		DEDSetHistory dedSetHistory = new DEDSetHistory();
		
		Iterator<SMOSequence> smoIt = ssh.smoSequences.iterator();
		Iterator<Schema> schemas = schemaHistory.schemas.iterator();
		
		System.out.println("Number of major and minor versions in schema history: " + 
				schemaHistory.size());
		
		//System.out.println(ssh);
		//System.out.println(ssh.toString());
		
		while(smoIt.hasNext()){
			SMOSequence ss = smoIt.next();
			Iterator<SMOImpl> sIt = ss.smos.iterator();
			while(sIt.hasNext()){
				Schema currentSchema = schemas.next();		
		//		System.out.println(currentSchema.toStringWithMetadata());
				SMOImpl smo = sIt.next();
		//		System.out.println(smo.toMetadataString());

				DEDSet ds = smo.getAllDEDs(currentSchema);			
				ds.sourceVersion  = smo.getSourceMajor();
				ds.minorSourceVersion = smo.getSourceMinor();
				ds.targetVersion = smo.getTargetMajor();
				ds.minorTargetVersion = smo.getTargetMinor();
				dedSetHistory.add(ds);
			}
		
		}
		
		// compose test
		
		
		DEDComposition dedComposeEngine = new DEDComposition(schemaHistory, dedSetHistory);
		DEDSet dedSet = dedComposeEngine.compose(41, 0, 42, 0);
		System.out.println("Finished DED composition task");
		System.out.println(dedSet);
		System.exit(1);
		
		//dedComposeEngine.composeAllMajorVersionsToTargetVersionForwardDEDs(31);
		//System.out.println(dedSetHistory.get(28, 30));
		
		// Step 2: set up rewriting engine - done only once
		PrimRewriteEngine pre = new PrimRewriteEngine(schemaHistory, dedSetHistory);
		
		
		
		// Step 3: rewriting procedure call - done repeatedly
		SQL inputSQL = getSQL();
		System.out.println(inputSQL);
		System.out.println("\nINTO query:");

		Stopwatch timer = new Stopwatch();
		timer.start();
		// modify the source and target to focus on the specific version
		// to see SMOS: 
		SQLUnion sqlUnion = null;
		try {
			sqlUnion = pre.rewrite(inputSQL, 41,0, 42,0);
		} catch (ErrorRewritingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		timer.stop();
		 
		System.out.print(sqlUnion);
		System.out.println("\nRewriting elapsed time: " + timer.getElapsedTime() + " ms");

	}
	private static SQL getSQL() {
		// v30
		SQL q30 = new SQL(
		"SELECT S0=u.user_id FROM user_rights u;");

		// v42
		SQL q42 = new SQL("SELECT S0=c.old_flags FROM cur1 c;");
		return q42;
	}
}
