package edu.ucla.cs.db.prism.backend.oldtestmain;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.Iterator;
import org.apache.log4j.Logger;


import edu.ucla.cs.db.prism.backend.exception.ErrorRewritingException;
import edu.ucla.cs.db.prism.backend.SQL.SQL;
import edu.ucla.cs.db.prism.backend.SQL.SQLUnion;
import edu.ucla.cs.db.prism.backend.ded.*;
import edu.ucla.cs.db.prism.backend.parser.parser.*;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.*;
import edu.ucla.cs.db.prism.backend.parser.visitor.*;
import edu.ucla.cs.db.prism.backend.rewrite.PrimRewriteEngine;
import edu.ucla.cs.db.prism.backend.schema.*;
import edu.ucla.cs.db.prism.backend.smo.*;
import edu.ucla.cs.db.prism.backend.smo.util.*;
import edu.ucla.cs.db.prism.backend.util.Stopwatch;
import edu.ucla.cs.db.prism.gui.server.DBLoader;
public class MainHSMOTest {

	//STATIC LOGGER
	static Logger logger = Logger.getLogger(DBLoader.class);
	
	public static void main(String[] args) {
		
		// Step 1.1: input parse
		SmoTranslateVisitor smotv = mainParse(args);
		SMOSequenceHistory ssh = smotv.smoSequenceHistory;
		//System.out.println(ssh.toString());
		Schema inputSchema = smotv.schema;
		//System.out.println(inputSchema);
		SQL inputSQL = getSQL();
		//System.out.println(inputSQL);
		//System.out.println("\nINTO query:");

		// Step 1.2: derived inputs
		SchemaHistory schemaHistory = new SchemaHistory(inputSchema, ssh);
		//System.out.println(schemaHistory);
		DEDSetHistory dedSetHistory = new DEDSetHistory();				
		//System.out.println("Number of major and minor versions in schema history: " + 
		//		schemaHistory.size());		
		
		Iterator<SMOSequence> smoIt = ssh.smoSequences.iterator();
		Iterator<Schema> schemas = schemaHistory.schemas.iterator();		
		while(smoIt.hasNext()){
			SMOSequence ss = smoIt.next();
			Iterator<SMOImpl> sIt = ss.smos.iterator();
			while(sIt.hasNext()){
				Schema currentSchema = schemas.next();		
				// System.out.println("BEFORE:\n" + currentSchema.toStringWithMetadata());
				SMOImpl smo = sIt.next();
				// System.out.println(smo.toMetadataString());
				DEDSet ds = smo.getAllDEDs(currentSchema);			
				// System.out.println("AFTER:\n" + currentSchema.toStringWithMetadata());
				ds.sourceVersion  = smo.getSourceMajor();
				ds.minorSourceVersion = smo.getSourceMinor();
				ds.targetVersion = smo.getTargetMajor();
				ds.minorTargetVersion = smo.getTargetMinor();
				dedSetHistory.add(ds);
			}
		
		}
		//System.out.println(dedSetHistory);
		
		// Step 2: set up rewriting engine - done only once
		PrimRewriteEngine pre = new PrimRewriteEngine(schemaHistory, dedSetHistory);		


		// Step 3: rewriting procedure call - done repeatedly
		Stopwatch timer = new Stopwatch();
		timer.start();
		// modify the source and target to focus on the specific version
		// to see SMOS: 
		SQLUnion sqlUnion = null;
		try {
			sqlUnion = pre.rewrite(inputSQL, 4,0, 5,0);
		} catch (ErrorRewritingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		timer.stop();
		 
		System.out.print(sqlUnion);
		System.out.println("\nRewriting elapsed time: " + timer.getElapsedTime() + " ms");
	}
	
	private static SmoTranslateVisitor mainParse(String args[]) {
		FileInputStream input = null;
		boolean packagingTakeInputFromCommandLineArgs = true;
		String inputPrimaScriptFileName = null;
		// step 1.1: fetch prima script filename to read
		try {
			if(packagingTakeInputFromCommandLineArgs && args.length > 0) {
				String filename = args[args.length-1];
				//System.out.println("Processing " + filename + " ...\n");
				input = new FileInputStream(filename);
			}
			else if(!packagingTakeInputFromCommandLineArgs) {
				//System.out.println("Processing " + inputPrimaScriptFileName + " ...\n");
				input = new FileInputStream(inputPrimaScriptFileName);
			}
			else {
				//usage();
				System.err.println("inputfile name not provided");
				System.exit(0);
			}
		} catch (FileNotFoundException e) {
			System.out.println("Error: input file is not found.");
			System.exit(-1);
		}

		// step 1.2: parse it
		PrimaRParser parser = new PrimaRParser(input);
		PrimaRScript script = null;
		try {
			script = parser.PrimaRScript();
		} catch(Exception e) {
			e.printStackTrace();
		}
		
		// step 1.3: restructure abstract syntax tree (AST) into our defined class objects
		SmoTranslateVisitor smotv = new SmoTranslateVisitor();
		script.accept(smotv);
		return smotv;
	}
	
	private static SQL getSQL() {
		// v30
		SQL q30 = new SQL(
		"SELECT S0=u.user_id FROM user_rights u;");

		// v42
		SQL q42 = new SQL("SELECT S0=c.old_flags FROM cur1 c;");
		
		// empdb v5
		SQL q5 = new SQL("SELECT S0=R0.salary FROM empacct R0;");
		return q5;
	}
}
