package edu.ucla.cs.db.prism.backend.oldtestmain;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.Iterator;
import java.util.Vector;

import org.apache.log4j.Logger;


import edu.ucla.cs.db.prism.backend.SQL.SQL;
import edu.ucla.cs.db.prism.backend.SQL.SQLUnion;
import edu.ucla.cs.db.prism.backend.ded.*;
import edu.ucla.cs.db.prism.backend.parser.parser.*;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.*;
import edu.ucla.cs.db.prism.backend.parser.visitor.*;
import edu.ucla.cs.db.prism.backend.rewrite.htable.*;
import edu.ucla.cs.db.prism.backend.schema.*;
import edu.ucla.cs.db.prism.backend.smo.*;
import edu.ucla.cs.db.prism.backend.smo.util.*;
import edu.ucla.cs.db.prism.backend.util.Stopwatch;
import edu.ucla.cs.db.prism.gui.server.DBLoader;
public class MainHTMappingTest {

	//STATIC LOGGER
	static Logger logger = Logger.getLogger(DBLoader.class);
	
	public static void main(String[] args) {
		
		// Step 1.1: input parse
		SmoTranslateVisitor smotv = mainParse(args);
		SMOSequenceHistory ssh = smotv.smoSequenceHistory;
		Schema inputSchema = smotv.schema;
		SQL inputSQL = getSQL();
		System.out.println(inputSQL);
		System.out.println(ssh);
		//System.exit(1);
		
		
		// Step 1.2: derived inputs
		SchemaHistory schemaHistory = new SchemaHistory(inputSchema, ssh);
		
		System.exit(1);
		
		declarePrimaryKeys(schemaHistory);
		DEDSetHistory dedSetHistory = new DEDSetHistory();				
		
		Iterator<SMOSequence> smoIt = ssh.smoSequences.iterator();
		Iterator<Schema> schemas = schemaHistory.schemas.iterator();		
		while(smoIt.hasNext()){
			SMOSequence ss = smoIt.next();
			Iterator<SMOImpl> sIt = ss.smos.iterator();
			while(sIt.hasNext()){
				Schema currentSchema = schemas.next();		
				// System.out.println("BEFORE:\n" + currentSchema.toStringWithMetadata());
				SMOImpl smo = sIt.next();
				// System.out.println(smo.toMetadataString());
				DEDSet ds = smo.getAllDEDs(currentSchema);			
				// System.out.println("AFTER:\n" + currentSchema.toStringWithMetadata());
				ds.sourceVersion  = smo.getSourceMajor();
				ds.minorSourceVersion = smo.getSourceMinor();
				ds.targetVersion = smo.getTargetMajor();
				ds.minorTargetVersion = smo.getTargetMinor();
				dedSetHistory.add(ds);
			}
		
		}
		
		// Step 2: set up rewriting engine - done only once
		PrimaSnapshotHQueryRewriteEngine pre = 
			new PrimaSnapshotHQueryRewriteEngine(schemaHistory, dedSetHistory);		


		// Step 3: rewriting procedure call - done repeatedly
		Stopwatch timer = new Stopwatch();
		timer.start();
		// modify the source and target to focus on the specific version
		// to see SMOS: 
		//System.out.println("inputSQL on H-table:\n" + inputSQL + "\n");
		
		SQLUnion sqlUnion = pre.rewrite(inputSQL, 1,0, 5,0);
		timer.stop();
		
		boolean db2FriendlyOuptut = true;
		String sqlUnionString = sqlUnion.toString();
		if(db2FriendlyOuptut) {
			sqlUnionString = sqlUnionString.replace('"', '\'');
			sqlUnionString = sqlUnionString.replace("S0=", "");
			sqlUnionString = sqlUnionString.replace("S1=", "");
			sqlUnionString = sqlUnionString.replace("S2=", "");
			sqlUnionString = sqlUnionString.replace("S3=", "");
			sqlUnionString = sqlUnionString.replace("job_", "v1job_");
			sqlUnionString = sqlUnionString.replace("empacct_", "v2empacct_");
			sqlUnionString = sqlUnionString.replace("engineerpersonnel_", "v1engineerpersonnel_");
			sqlUnionString = sqlUnionString.replace("otherpersonnel_", "v1otherpersonnel_");
			sqlUnionString = sqlUnionString.replace("dept_", "v3dept_");			
			sqlUnionString = sqlUnionString.replace("managerno", "mgrno");
			sqlUnionString += "\n;\n";
		}
		
		System.out.println("final rewritten query on H-table:\n" + sqlUnionString);
		System.out.println("\nRewriting elapsed time: " + timer.getElapsedTime() + " ms");
	}
	
	private static SmoTranslateVisitor mainParse(String args[]) {
		FileInputStream input = null;
		boolean packagingTakeInputFromCommandLineArgs = true;
		String inputPrimaScriptFileName = null;
		// step 1.1: fetch prima script filename to read
		try {
			if(packagingTakeInputFromCommandLineArgs && args.length > 0) {
				String filename = args[args.length-1];
				//System.out.println("Processing " + filename + " ...\n");
				input = new FileInputStream(filename);
			}
			else if(!packagingTakeInputFromCommandLineArgs) {
				//System.out.println("Processing " + inputPrimaScriptFileName + " ...\n");
				input = new FileInputStream(inputPrimaScriptFileName);
			}
			else {
				//usage();
				System.err.println("inputfile name not provided");
				System.exit(0);
			}
		} catch (FileNotFoundException e) {
			System.out.println("Error: input file is not found.");
			System.exit(-1);
		}

		// step 1.2: parse it
		PrimaRParser parser = new PrimaRParser(input);
		PrimaRScript script = null;
		try {
			script = parser.PrimaRScript();
		} catch(Exception e) {
			e.printStackTrace();
		}
		
		// step 1.3: restructure abstract syntax tree (AST) into our defined class objects
		SmoTranslateVisitor smotv = new SmoTranslateVisitor();
		script.accept(smotv);
		return smotv;
	}
	
	private static void declarePrimaryKeys(SchemaHistory schemaHistory) {
		// works for empdb v1~v5
		Schema schema = null;
		Table table = null;
	
		// v1
		schema = schemaHistory.getSchemaByVersion(1,0);
		table = schema.getTable("engineerpersonnel");
		table.setPrimaryKeys(new Vector<String>());
		table.getPrimaryKeysColumns().add("empno");

		table = schema.getTable("otherpersonnel");
		table.setPrimaryKeys(new Vector<String>());
		table.getPrimaryKeysColumns().add("empno");
		
		table = schema.getTable("job");
		table.setPrimaryKeys(new Vector<String>());
		table.getPrimaryKeysColumns().add("title");

		// v2
		schema = schemaHistory.getSchemaByVersion(2,0);
		table = schema.getTable("empacct");
		table.setPrimaryKeys(new Vector<String>());
		table.getPrimaryKeysColumns().add("empno");

		table = schema.getTable("job");
		table.setPrimaryKeys(new Vector<String>());
		table.getPrimaryKeysColumns().add("title");

		// v3
		schema = schemaHistory.getSchemaByVersion(3,0);
		table = schema.getTable("empacct");
		table.setPrimaryKeys(new Vector<String>());
		table.getPrimaryKeysColumns().add("empno");

		table = schema.getTable("job");
		table.setPrimaryKeys(new Vector<String>());
		table.getPrimaryKeysColumns().add("title");

		table = schema.getTable("dept");
		table.setPrimaryKeys(new Vector<String>());
		table.getPrimaryKeysColumns().add("deptno");

		// v4
		schema = schemaHistory.getSchemaByVersion(4,0);
		table = schema.getTable("empacct");
		table.setPrimaryKeys(new Vector<String>());
		table.getPrimaryKeysColumns().add("empno");

		table = schema.getTable("job");
		table.setPrimaryKeys(new Vector<String>());
		table.getPrimaryKeysColumns().add("title");

		table = schema.getTable("dept");
		table.setPrimaryKeys(new Vector<String>());
		table.getPrimaryKeysColumns().add("deptno");
		
		table = schema.getTable("empbio");
		table.setPrimaryKeys(new Vector<String>());
		table.getPrimaryKeysColumns().add("empno");
		
		// v5
		schema = schemaHistory.getSchemaByVersion(5,0);
		table = schema.getTable("empacct");
		table.setPrimaryKeys(new Vector<String>());
		table.getPrimaryKeysColumns().add("empno");

		table = schema.getTable("dept");
		table.setPrimaryKeys(new Vector<String>());
		table.getPrimaryKeysColumns().add("deptno");
		
		table = schema.getTable("empbio");
		table.setPrimaryKeys(new Vector<String>());
		table.getPrimaryKeysColumns().add("empno");
	
	}
	private static SQL getSQL() {
		// v30
		SQL q30 = new SQL(
		"SELECT S0=u.user_id FROM user_rights u;");

		// v42
		SQL q42 = new SQL("SELECT S0=c.old_flags FROM cur1 c;");
		
		
		SQL q1 = new SQL("SELECT S0=R0.salary " + 
				" FROM empacct_salary R0;");
		
		// empdb v5
		SQL q2 = new SQL("SELECT S0=R0.salary, S1=R1.deptno " + 
				" FROM empacct_salary R0, empacct_deptno R1 " + 
				" WHERE R0.empno = R1.empno;");
		
		
		// experiment 3/28/2008
		/*
		SQL expQ1 = new SQL(
		"SELECT S0=R0.managerno "+
		"FROM dept_managerno R0 "+
		"WHERE R0.deptno = \"d001\";"); 
		
		SQL expQ2= new SQL(
				"SELECT S0=R0.salary "+
				"FROM empacct_salary R0 "+
				"WHERE R0.empno = \"10004\";");
				
		SQL expQ3 = new SQL(
				"SELECT S0=R0.salary "+
				"FROM empacct_salary R0, empacct_title R1 "+
				"WHERE R0.empno = R1.empno and R1.title = \"assistant staff\";");		
				
		SQL expQ4 = new SQL(
				"SELECT S0=R0.salary "+
				"FROM empacct_salary R0;"
				);
		SQL expQ5 = new SQL(
				"SELECT S0=R0.empno "+
				"FROM empacct_deptno R0 " +
				"WHERE R0.deptno = \"d001\";"
			);
		SQL expQ6 = new SQL(
				"SELECT S0=R1.empno "+
				"FROM empacct_deptno R0, empacct_deptno R1 "+
				"WHERE R0.empno = \"10004\" and R0.deptno = R1.deptno;"
		);
		 */
		// 4/7/2008
		SQL expQ1Final = new SQL(
				"SELECT S0=R0.salary "+
				"FROM empacct_salary R0 "+
				"WHERE R0.empno = \"10004\" "//+
				//"  and R0.ts < \"2005-07-02\" and R0.te > \"2005-07-01\" "
				);
		
		SQL expQ2Final = new SQL(
//				"SELECT S0=count(R0.empno) "+
				"SELECT S0=R0.empno "+
				"FROM empacct_salary R0 "+
				"WHERE R0.salary > \"60000\" " //+
				//"  and R0.ts < \"2005-07-02\" and R0.te > \"2005-07-01\" "
		);
		/*
		SQL expQ3Final = new SQL(
//				"SELECT S0=avg(R0.salary) "+
				"SELECT S0=R0.salary "+
				"FROM empacct_salary R0 " //+
				//"WHERE R0.ts < \"2005-07-02\" and R0.te > \"2005-07-01\" "
		);
		*/
		SQL expQ3Final = new SQL(
				"SELECT S0=R0.salary "+
				"FROM empacct_salary R0 "+
				"WHERE R0.empno = \"10004\" " //+
				//"  and R0.ts < \"2005-12-01\" and R0.te > \"2005-07-01\" "
		);
		SQL expQ4Final = new SQL(
//				"SELECT S0=count(R0.empno) "+
				"SELECT S0=R0.empno "+
				"FROM empacct_salary R0 "+
				"WHERE R0.salary > \"60000\" " //+
				//"  and R0.ts < \"2005-12-01\" and R0.te > \"2005-07-01\" "
		);
		/*
		SQL expQ6Final = new SQL(
//				"SELECT S0=max(R2.salary-R1.salary) "+
				"SELECT S0=R2.salary, S1=R1.salary "+
				"FROM empacct_salary R1, empacct_salary R2 " +
				"WHERE R1.empno = R2.empno" //+
				//"  and R1.ts < \"2005-07-02\" and R1.te > \"2005-07-01\" "+
				//"  and R2.ts < \"2005-12-02\" and R2.te > \"2005-12-01\" "
		);
		*/
		SQL expQ5Final = new SQL(
				"SELECT S0=R1.managerno "+
				"FROM empacct_deptno R0, dept_managerno R1 "+
				"WHERE R0.empno = \"10004\" and R0.deptno = R1.deptno "//+
//				"  and R0.te > T1 and R0.ts < T2 "+
//				"  and R1.te > T1 and R1.ts < T2 "+
//				"  and R0.te > R1.ts and R0.ts < R1.te "				 
				);
		
		SQL expQ6Final = new SQL(
//				"SELECT S0=max(min(R0.te, R1.te)-max(R0.ts, R1.ts)) "+
				"SELECT R0.deptno "+
//				"SELECT S0=R0.empno "+
				"FROM empacct_deptno R0, empacct_deptno R1 "+
				"WHERE R0.deptno = R1.deptno and R0.empno != R1.empno "//+
//				"  and R0.te > T1 and R0.ts < T2 "+
//				"  and R1.te > T1 and R1.ts < T2 "+
//				"  and R0.te > R1.ts and R0.ts < R1.te "
				);

		SQL expQ7Final = new SQL(
//				"SELECT S0=R0.empno, S1=R0.salary, S2=R1.title, S3=R2.deptno, S4=max(R0.ts, R1.ts, R2.ts), S5=min(R0.te, R1.te, R2.te) "+
				"SELECT S0=R0.empno, S1=R0.salary, S2=R1.title, S3=R2.deptno "+
				"FROM empacct_salary R0, empacct_title R1, empacct_deptno R2 "+
				"WHERE R0.empno = \"10004\" "+
				"  and R0.empno = R1.empno and R1.empno = R2.empno "//+
//				"  and R0.te > T1 and R0.ts < T2 "+
//				"  and R1.te > T1 and R1.ts < T2 "+
//				"  and R2.te > T1 and R2.ts < T2 "+
//				"  and R0.te > R1.ts and R0.ts < R1.te "+
//				"  and R1.te > R2.ts and R1.ts < R2.te "+
//				"  and R2.te > R0.ts and R2.ts < R0.te "+
				);

		return expQ6Final;
	}

	
}
