package edu.ucla.cs.db.prism.backend.oldtestmain;

import it.polimi.elet.cautils.configurator.CACmdLineParser;
import it.polimi.elet.cautils.configurator.CAConfiguration;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.exception.ErrorRewritingException;
import edu.ucla.cs.db.prism.backend.SQL.RelationDeclaration;
import edu.ucla.cs.db.prism.backend.SQL.SQL;
import edu.ucla.cs.db.prism.backend.SQL.SQLUnion;
import edu.ucla.cs.db.prism.backend.SQL.SelectAttribute;
import edu.ucla.cs.db.prism.backend.ded.DED;
import edu.ucla.cs.db.prism.backend.ded.DEDSet;
import edu.ucla.cs.db.prism.backend.ded.DEDSetHistory;
import edu.ucla.cs.db.prism.backend.ded.QuantifiedDisjunctive;
import edu.ucla.cs.db.prism.backend.ded.Quantifier;
import edu.ucla.cs.db.prism.backend.parser.parser.*;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.*;
import edu.ucla.cs.db.prism.backend.parser.visitor.SmoTranslateVisitor;
import edu.ucla.cs.db.prism.backend.rewrite.PrimRewriteEngine;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.SchemaHistory;
import edu.ucla.cs.db.prism.backend.schema.Table;
import edu.ucla.cs.db.prism.backend.smo.util.Conjunctive;
import edu.ucla.cs.db.prism.backend.smo.util.Predicate;
import edu.ucla.cs.db.prism.backend.smo.util.Term;
import edu.ucla.cs.db.prism.backend.util.*;

public class MainREFTest {
	
	
	public static void main(String[] args) {
		
		SQL sql = generateSampleSQL();
		System.out.println(sql.toString() + "\n");
		int targetVersion = 1;
		int targetMinorVersion = 0;
		sql.convertInequalityToREF(targetVersion, targetMinorVersion);
		System.out.println(sql.toString() + "\n");
		
		sql.convertREFToInequality(targetVersion, targetMinorVersion);
		sql.where.removeTautologies();
		System.out.println(sql.toString() + "\n");
	}
	
	public static void testRewriteEngine(String[] args) {
		// Step 1: metadata loading
		//         sample metadata generator used for testing purpose
		SchemaHistory schemaHistory = generateSampleSchemaHistory();
		DEDSetHistory dedSetHistory = generateSampleDEDSetHistory();

		//System.out.println(schemaHistory);
		System.out.println(dedSetHistory);
		
		// Step 2: set up rewriting engine - done only once
		PrimRewriteEngine pre = new PrimRewriteEngine(schemaHistory, dedSetHistory);

		// Step 3: rewriting procedure call - done repeatedly
		SQL inputSQL = generateSampleSQL();
		//System.out.println(inputSQL);
		Stopwatch timer = new Stopwatch();
		timer.start();
		SQLUnion sqlUnion = null;
		try {
			sqlUnion = pre.rewrite(inputSQL, 1,0, 2,0);
		} catch (ErrorRewritingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		timer.stop();
		 
		System.out.print(sqlUnion);
		System.out.println("\nRewriting elapsed time: " + timer.getElapsedTime() + " ms");

	}
	
	
	public static SchemaHistory generateSampleSchemaHistory() {
		SchemaHistory schemaHistory = new SchemaHistory();
		Schema schema;
		Table table;
		
		schema = new Schema();
		schema.versionID = 1;
		schema.minorVersionID = 0;
		schemaHistory.schemas.add(schema);
		table = new Table("emp");
		schema.addTable(table);
		table.addColumn("eid");
		table.addColumn("title");
		table.addColumn("test");

		schema = new Schema();
		schema.versionID = 1;
		schema.minorVersionID = 1;
		schemaHistory.schemas.add(schema);
		table = new Table("empx");
		schema.addTable(table);
		table.addColumn("eid");
		table.addColumn("title");
		table.addColumn("test");

		schema = new Schema();
		schema.versionID = 2;
		schema.minorVersionID = 0;
		schemaHistory.schemas.add(schema);
		table = new Table("emp2");
		schema.addTable(table);
		table.addColumn("eid");
		table.addColumn("title");
		
		return schemaHistory;
	}
	
	private static DEDSetHistory generateSampleDEDSetHistory() {
		DEDSetHistory dedSetHistory = new DEDSetHistory();
		DEDSet dedSet;
		DED ded;
		
		//////////////////////////
		// dedSet from v1 to v1m1
		//////////////////////////
		dedSet = new DEDSet();
		dedSetHistory.add(dedSet);
		dedSet.sourceVersion = 1;
		dedSet.minorSourceVersion = 0;
		dedSet.targetVersion = 1;
		dedSet.minorTargetVersion = 1;
		
		
		ded = new DED();
		dedSet.add(ded);
		ded.dedName = "RENAME_TABLE_emp_C";
		ded.chaseDEDFlag = true; // chase direction = data-exchange direction
		
		Quantifier q1 = new Quantifier("all", "R0", "v1_emp");
		ded.globalQuantifiers.quantifiers.add(q1);
		
		QuantifiedDisjunctive tc1 = new QuantifiedDisjunctive(); 
		ded.thenClauses.add(tc1);

		q1 = new Quantifier("some", "R1", "v1m1_empx");
		tc1.quantifiers.add(q1);

		Conjunctive c1 = new Conjunctive();
		tc1.disjunctive.add(c1);
		c1.predicates = new Vector<Predicate>();

		Predicate p1 = new Predicate();
		c1.predicates.add(p1);
		p1.term1 = new Term("R0", "eid");
		p1.term2 = new Term("R1", "eid");
		p1.op = "=";
		
		p1 = new Predicate();
		c1.predicates.add(p1);
		p1.term1 = new Term("R0", "title");
		p1.term2 = new Term("R1", "title");
		p1.op = "=";

		// metadata about ded
		ded.chaseDEDFlag = true;
		ded.sourceRelations = new Vector<String>();
		ded.sourceRelations.add("v1_emp");
		ded.targetRelations = new Vector<String>();
		ded.targetRelations.add("v1m1_empx");

		
		
		ded = new DED();
		dedSet.add(ded);
		ded.dedName = "RENAME_TABLE_emp_BC";
		ded.chaseDEDFlag = false; // chase direction = data-exchange direction
		
		q1 = new Quantifier("all", "R0", "v1m1_empx");
		ded.globalQuantifiers.quantifiers.add(q1);
		
		tc1 = new QuantifiedDisjunctive(); 
		ded.thenClauses.add(tc1);

		q1 = new Quantifier("some", "R1", "v1_emp");
		tc1.quantifiers.add(q1);

		c1 = new Conjunctive();
		tc1.disjunctive.add(c1);
		c1.predicates = new Vector<Predicate>();

		p1 = new Predicate();
		c1.predicates.add(p1);
		p1.term1 = new Term("R0", "eid");
		p1.term2 = new Term("R1", "eid");
		p1.op = "=";
		
		p1 = new Predicate();
		c1.predicates.add(p1);
		p1.term1 = new Term("R0", "title");
		p1.term2 = new Term("R1", "title");
		p1.op = "=";

		// metadata about ded
		ded.chaseDEDFlag = false;
		ded.sourceRelations = new Vector<String>();
		ded.sourceRelations.add("v1m1_empx");
		ded.targetRelations = new Vector<String>();
		ded.targetRelations.add("v1_emp");

		
		//////////////////////////
		// dedSet from v1m1 (empx) to v2 (emp2)
		//////////////////////////
		dedSet = new DEDSet();
		dedSetHistory.add(dedSet);
		dedSet.sourceVersion = 1;
		dedSet.minorSourceVersion = 1;
		dedSet.targetVersion = 2;
		dedSet.minorTargetVersion = 0;
		
		
		ded = new DED();
		dedSet.add(ded);
		ded.dedName = "RENAME_TABLE_emp_C";
		ded.chaseDEDFlag = true; // chase direction = data-exchange direction
		
		q1 = new Quantifier("all", "R0", "v1m1_empx");
		ded.globalQuantifiers.quantifiers.add(q1);
		
		tc1 = new QuantifiedDisjunctive(); 
		ded.thenClauses.add(tc1);

		q1 = new Quantifier("some", "R1", "v2_emp2");
		tc1.quantifiers.add(q1);

		c1 = new Conjunctive();
		tc1.disjunctive.add(c1);
		c1.predicates = new Vector<Predicate>();

		p1 = new Predicate();
		c1.predicates.add(p1);
		p1.term1 = new Term("R0", "eid");
		p1.term2 = new Term("R1", "eid");
		p1.op = "=";
		
		p1 = new Predicate();
		c1.predicates.add(p1);
		p1.term1 = new Term("R0", "title");
		p1.term2 = new Term("R1", "title");
		p1.op = "=";

		// metadata about ded
		ded.chaseDEDFlag = true;
		ded.sourceRelations = new Vector<String>();
		ded.sourceRelations.add("v1m1_empx");
		ded.targetRelations = new Vector<String>();
		ded.targetRelations.add("v2_emp2");

		
		
		ded = new DED();
		dedSet.add(ded);
		ded.dedName = "RENAME_TABLE_emp_BC";
		ded.chaseDEDFlag = false; // chase direction = data-exchange direction
		
		q1 = new Quantifier("all", "R0", "v2_emp2");
		ded.globalQuantifiers.quantifiers.add(q1);
		
		tc1 = new QuantifiedDisjunctive(); 
		ded.thenClauses.add(tc1);

		q1 = new Quantifier("some", "R1", "v1m1_empx");
		tc1.quantifiers.add(q1);

		c1 = new Conjunctive();
		tc1.disjunctive.add(c1);
		c1.predicates = new Vector<Predicate>();

		p1 = new Predicate();
		c1.predicates.add(p1);
		p1.term1 = new Term("R0", "eid");
		p1.term2 = new Term("R1", "eid");
		p1.op = "=";
		
		p1 = new Predicate();
		c1.predicates.add(p1);
		p1.term1 = new Term("R0", "title");
		p1.term2 = new Term("R1", "title");
		p1.op = "=";

		// metadata about ded
		ded.chaseDEDFlag = false;
		ded.sourceRelations = new Vector<String>();
		ded.sourceRelations.add("v2_emp2");
		ded.targetRelations = new Vector<String>();
		ded.targetRelations.add("v1m1_empx");

		
		return dedSetHistory;
	}
	
	
	public static SQL generateSampleSQL() {
		SQL sql = new SQL();

		sql.select.selectAttributes.add(new SelectAttribute("S0", "R0", "eid"));
		sql.from.relationDeclarations.add(new RelationDeclaration("emp2", "R0"));

		Conjunctive conjunctive = new Conjunctive();
		sql.where.conjunctives.add(conjunctive);
		conjunctive.add(new Predicate("R0", "eid", "<", "R0", "title"));
		return sql;
	}
	
	
	
	
	
	
	public static SmoTranslateVisitor parsePRIMAScript(String[] args) {
		FileInputStream input = null;
		String inputFileName = "input/wikiall-20080128.txt";
		// step 1.1: fetch prima script filename to read
		try {
			input = new FileInputStream(inputFileName);
		} catch (FileNotFoundException e) {
			System.out.println("Error: input file is not found.");
			System.exit(-1);
		}

		// step 1.2: parse it
		PrimaRParser parser = new PrimaRParser(input);
		PrimaRScript script = null;
		try {
			script = parser.PrimaRScript();
		} catch(Exception e) {
			e.printStackTrace();
		}
		
		// step 1.3: restructure abstract syntax tree (AST) into our defined class objects
		SmoTranslateVisitor smotv = new SmoTranslateVisitor();
		script.accept(smotv);
		return smotv;
	}
	public static SmoTranslateVisitor parseSMO(String[] args) {
		FileInputStream input = null;
		//String inputFileName = "input/wikiall-20071214.txt";
		String inputFileName = "input/smo.txt";
		// step 1.1: fetch prima script filename to read
		try {
			input = new FileInputStream(inputFileName);
		} catch (FileNotFoundException e) {
			System.out.println("Error: input file is not found.");
			System.exit(-1);
		}

		// step 1.2: parse it
		PrimaRParser parser = new PrimaRParser(input);
		//PrimaRScript script = null;
		Smo smo = null;
		try {
			smo = parser.Smo();
			//script = parser.PrimaRScript();
		} catch(Exception e) {
			e.printStackTrace();
		}
		
		// step 1.3: restructure abstract syntax tree (AST) into our defined class objects
		SmoTranslateVisitor smotv = new SmoTranslateVisitor();
		//script.accept(smotv);
		smo.accept(smotv);
		return smotv;
	}

	
	private static SchemaHistory mainGlobalPreprocessing(SmoTranslateVisitor smotv) {
		// step 2.1: compute the inverse of SMOs to make SMO Sequence History complete
		//         in both forward and backward directions
		smotv.smoSequenceHistory.computeInverse();

		// step 2.2: compute schema history using SMOs (not views!)
		SchemaHistory schemaHistory = new SchemaHistory(smotv.schema, smotv.smoSequenceHistory);
		
		// step 2.3: version interval compilation finish (constant time operation)
		smotv.versionIntervalSet.SetEndTime();

		return schemaHistory;
	}
	
	public static void parseArgs(String[] args) {

		//Use CAConfigurator to load configuration from shell or xml-configuration file (-c inputfile)
		 CACmdLineParser par = null;
	 	 try {
			par = CACmdLineParser.getInstance();
		 } catch (InstantiationException e) {
			e.printStackTrace();
		 }
		//List here all the interesting parameters
		 par.addStringParameter('h',"host");	
		 par.addStringParameter('m',"metadbname");	
		 par.addStringParameter('u',"username");
		 par.addStringParameter('p',"password");
		 
		//parse them all 
		 CAConfiguration conf = par.parse(args);
		 
		//read the parameter you are interested in 
		 String host = (String) conf.getParameterValue("host");
		 String metadbname = (String) conf.getParameterValue("metadbname");
		 String username = (String) conf.getParameterValue("username");
		 String password = (String) conf.getParameterValue("password");
		 
		//use them
		 System.out.println("PRIM is about to be executed with the follwing parameters:");
		 System.out.println("  - host: " + host);		 
		 System.out.println("  - user: " + username);
		 System.out.println("  - pass: " + password);
		 System.out.println("  - metadb: " + metadbname);
		 
	}
	

}
