package edu.ucla.cs.db.prism.backend.oldtestmain;

import it.polimi.elet.cautils.configurator.CACmdLineParser;
import it.polimi.elet.cautils.configurator.CAConfiguration;

import java.sql.Connection;
import java.util.Iterator;
import java.util.Vector;

import org.apache.log4j.Logger;

import edu.ucla.cs.db.prism.backend.exception.ErrorRewritingException;
import edu.ucla.cs.db.prism.backend.SQL.SQL;
import edu.ucla.cs.db.prism.backend.SQL.SQLUnion;
import edu.ucla.cs.db.prism.backend.ded.*;
import edu.ucla.cs.db.prism.backend.rewrite.PrimRewriteEngine;
import edu.ucla.cs.db.prism.backend.schema.*;
import edu.ucla.cs.db.prism.backend.smo.*;
import edu.ucla.cs.db.prism.backend.smo.util.*;
import edu.ucla.cs.db.prism.backend.util.Log4JHelper;
import edu.ucla.cs.db.prism.backend.util.db.DBUtil;
import edu.ucla.cs.db.prism.gui.server.DBLoader;

public class MainSMOInverseGeneration {

	//STATIC LOGGER
	static Logger logger = Logger.getLogger(DBLoader.class);
	
	public static void main(String[] args) {
		
		  //LOG4J SET UP
        Log4JHelper.log4JSetup(logger);
       
       
		//Use CAConfigurator to load configuration from shell or xml-configuration file (-c inputfile)
		 CACmdLineParser par = null;
	 	 try {
			par = CACmdLineParser.getInstance();
		 } catch (InstantiationException e) {
			e.printStackTrace();
		 }
		//List here all the interesting parameters
		 par.addStringParameter('h',"host");	
		 par.addStringParameter('u',"dburl");
		 par.addStringParameter('m',"metadbname");	
		 par.addStringParameter('u',"username");
		 par.addStringParameter('p',"password");
		 par.addStringParameter('l',"logfile");
		 
		//parse them all 
		 CAConfiguration conf = par.parse(args);
		 
		//read the parameter you are interested in 
		 String host = (String) conf.getParameterValue("host");
		 String dburl = (String) conf.getParameterValue("dburl");
		 String metadbname = (String) conf.getParameterValue("metadbname");
		 String dbname = (String) conf.getParameterValue("dbname");
		 String username = (String) conf.getParameterValue("username");
		 String password = (String) conf.getParameterValue("password");
		 int targetMajor = Integer.parseInt((String)conf.getParameterValue("targetmajor"));
		 int targetMinor = Integer.parseInt((String) conf.getParameterValue("targetminor"));
		 int sourceMajor = Integer.parseInt((String)conf.getParameterValue("sourcemajor"));
		 int sourceMinor = Integer.parseInt((String) conf.getParameterValue("sourceminor"));		 
		 
		 
		//use them
		 System.out.println("PRIM rewriting query: ");
	//	 System.out.println("  - host: " + host);		
	//	 System.out.println("  - user: " + username);
	//	 System.out.println("  - metadb: " + metadbname);
		 
		// MySQL DRIVER LOADING 
       try {
           Class.forName("com.mysql.jdbc.Driver").newInstance();
       } catch (Exception ex) { ex.printStackTrace();}
	 

        Connection conn = DBUtil.connect(dburl,metadbname,username,password);
		    
        //System.out.println("Connected to the DB:" + dburl +" " +metadbname+" " +username);
		 
		 SchemaHistoryLoader shl = new SchemaHistoryLoader(conn,dbname);
		 SchemaHistory schemaHistory = shl.loadSchemaHistoryFromDB(true);

		 logger.info("Schema Loaded");
		
		 SMOSequenceHistoryLoader sshl = new SMOSequenceHistoryLoader(conn,dbname);
	     SMOSequenceHistory ssh = sshl.loadSMOSequenceHistoryFromDB(false);
		 
		DEDSetHistory dedSetHistory = new DEDSetHistory();
		
		Iterator<SMOSequence> smoIt = ssh.smoSequences.iterator();
		Iterator<Schema> schemas = schemaHistory.schemas.iterator();

		System.out.println("SCHEMA AND SMOs LOADED.");
		Vector<SMOSequence> inverse = new Vector<SMOSequence>();
		while(smoIt.hasNext()){
			SMOSequence ss = smoIt.next();
			Iterator<SMOImpl> sIt = ss.smos.iterator();
			while(sIt.hasNext()){
				Schema currentSchema = schemas.next();		
				SMOImpl smo = sIt.next();
				inverse.addAll(smo.getCandidateInverses(currentSchema));			
			}
		}
		
		
		Iterator<SMOSequence> invit = inverse.iterator();
		while(invit.hasNext()){
			SMOSequence ss = invit.next();
			Iterator<SMOImpl> smoit = ss.smos.iterator();
			while(smoit.hasNext()){
				SMOImpl smoimpl = smoit.next();
				System.out.println(smoimpl.toSQLInsertString("prim_metadb2.TEMPSMO", "wikipedia", 1));
			}
					
		}
	}


	private static Vector<String> generateAllMajorVersionView(String dbname, int targetMajor,
	int targetMinor, SchemaHistory schemaHistory,
	DEDSetHistory dedSetHistory) {

		Iterator<Schema> schemas;
		// Step 2: set up rewriting engine - done only once
		PrimRewriteEngine pre = new PrimRewriteEngine(schemaHistory, dedSetHistory);
		
		Vector<String> views = new Vector<String>();
		
		//Step 3: browse through the schema and create a view for each table of each version
		schemas = schemaHistory.schemas.iterator();
		while(schemas.hasNext()){
			Schema cs = schemas.next();	
			if(cs.versionID == 0)
				views.addAll(viewForSchema(dbname, pre, cs, targetMajor, targetMinor));
		}
		return views;
	}	


	private static Vector<String> generateViewsForVersion(String dbname, int sourceMajor, int sourceMinor, int targetMajor,
			int targetMinor, SchemaHistory schemaHistory,
			DEDSetHistory dedSetHistory) {
		
		Iterator<Schema> schemas;
		// Step 2: set up rewriting engine - done only once
		PrimRewriteEngine pre = new PrimRewriteEngine(schemaHistory, dedSetHistory);
		
		Vector<String> views = new Vector<String>();
 	
		//Step 3: browse through the schema and create a view for each table of each version
		schemas = schemaHistory.schemas.iterator();
		while(schemas.hasNext()){
			Schema cs = schemas.next();	
			if(cs.minorVersionID == sourceMinor && cs.versionID == sourceMajor)
			views.addAll(viewForSchema(dbname, pre, cs, targetMajor, targetMinor));
		}
		return views;
	}	

	private static Vector<String> viewForSchema(String dbname, PrimRewriteEngine pre, Schema cs, int targetMajor, int targetMinor) {
		Vector<String> views = new Vector<String>();
		Iterator<Table> tit = cs.getTables().iterator();
		while(tit.hasNext()){
			Table tab = tit.next();
			System.out.println("-- " + cs.versionID + "." + cs.minorVersionID + " -> " +  targetMajor + "." + targetMinor);			
			String view = "CREATE OR REPLACE VIEW " + dbname + "_" + cs.versionID +"." + tab.getTableName();
			view+= "(" + tab.getColumnsString() + ")";
			view+= " AS \n";
			String v =null;
			// COMPUTE THE BODY OF THE VIEW BY MEANS OF REWRITING
			SQL inputSQL = new SQL("SELECT " + tab.getColumnsStringMars() + " FROM " + tab.getTableName() + " t;");
			SQLUnion sqlUnion= null;
			try {
				sqlUnion = pre.rewrite(inputSQL, targetMajor, targetMinor, cs.versionID, cs.minorVersionID);
			} catch (ErrorRewritingException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			if(sqlUnion==null){
				String nulls = "";
				for(int i = 0; i< tab.getColumns().size(); i++){
					nulls +="null";
					if(i!=tab.getColumns().size())
						nulls+=",";
				}
				v += "SELECT "  + nulls + ";";
			}else{	
					
				v =view + sqlUnion.toViewString(dbname + "_main") + ";"; 
			}
			views.add(v);
			System.out.println(v);
		}
		return views;
	
	}
	
}
