package edu.ucla.cs.db.prism.backend.oldtestmain;

import java.util.Iterator;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.exception.ErrorRewritingException;
import edu.ucla.cs.db.prism.backend.SQL.RelationDeclaration;
import edu.ucla.cs.db.prism.backend.SQL.SQL;
import edu.ucla.cs.db.prism.backend.SQL.SQLUnion;
import edu.ucla.cs.db.prism.backend.SQL.SelectAttribute;
import edu.ucla.cs.db.prism.backend.ded.*;
import edu.ucla.cs.db.prism.backend.rewrite.PrimRewriteEngine;
import edu.ucla.cs.db.prism.backend.schema.*;
import edu.ucla.cs.db.prism.backend.smo.*;
import edu.ucla.cs.db.prism.backend.smo.util.*;
import edu.ucla.cs.db.prism.backend.util.Stopwatch;

public class MainSMOtoDEDTest {

	public static void main(String[] args) {
		
		//testRewriteEngine(SMOInterface.ADD_COL);
		//testRewriteEngine(SMOInterface.COPY_TAB);
		testRewriteEngine(SMOInterface.CREATE_TAB);
	/*	testRewriteEngine(SMOInterface.DECOMPOSE);
		testRewriteEngine(SMOInterface.DROP_COL);
		testRewriteEngine(SMOInterface.DROP_TAB);
		
		testRewriteEngine(SMOInterface.JOIN);
	
		testRewriteEngine(SMOInterface.MERGE);
		testRewriteEngine(SMOInterface.NOP);
		testRewriteEngine(SMOInterface.PARTITION);
		testRewriteEngine(SMOInterface.REN_COL);
		testRewriteEngine(SMOInterface.REN_TAB);
		*/
	}

	public static void testRewriteEngine(String type) {
		// Step 1: metadata loading
		//         sample metadata generator used for testing purpose
		SchemaHistory schemaHistory = generateSampleSchemaHistory(type);
		
		//System.out.println(schemaHistory);
		
		SMOSequenceHistory ssh = generateSMOSequenceHistory(type);		
		DEDSetHistory dedSetHistory = new DEDSetHistory();
		Iterator<SMOSequence> smoIt = ssh.smoSequences.iterator();
	
		System.out.println(ssh);
		//System.out.println(ssh.toString());
		
		while(smoIt.hasNext()){
			SMOSequence ss = smoIt.next();
			Iterator<SMOImpl> sIt = ss.smos.iterator();
			while(sIt.hasNext()){
				SMOImpl smo = sIt.next();
				DEDSet ds = smo.getAllDEDs(schemaHistory.schemas.firstElement().clone());
				System.out.println(smo.getDEDs(schemaHistory.schemas.firstElement().clone()));
				dedSetHistory.add(ds);
				
			}
		
		}

		//System.out.println(schemaHistory);
		//System.out.println(dedSetHistory);
		
		// Step 2: set up rewriting engine - done only once
		PrimRewriteEngine pre = new PrimRewriteEngine(schemaHistory, dedSetHistory);
		
		
		
		// Step 3: rewriting procedure call - done repeatedly
		SQL inputSQL = generateSampleSQL(type);
		Stopwatch timer = new Stopwatch();
		timer.start();
		SQLUnion sqlUnion= null;
		try {
			sqlUnion = pre.rewrite(inputSQL, 1,0, 2,0);
		} catch (ErrorRewritingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		timer.stop();
		 
		System.out.print(sqlUnion);
		System.out.println("\nRewriting elapsed time: " + timer.getElapsedTime() + " ms");

	}
	
	private static SMOSequenceHistory generateSMOSequenceHistory(String type) {
		
		SMOSequenceHistory ssh = new SMOSequenceHistory();
		SMOSequence ss = new SMOSequence();
		ss.sourceVersion=1;
		ss.targetVersion=2;
		SMOImpl smo = null;


		try {
			smo = SMOFactory.createSMO(type);
			smo.setSourceMajor(1);
			smo.setSourceMinor(0);
			smo.setTargetMajor(2);
			smo.setTargetMinor(0);
			
		} catch (SMOTypeUnknownException e) {
			e.printStackTrace();
		}


		if(type.equalsIgnoreCase(SMOInterface.REN_COL)){
			
			smo.setTableName("emp");
			smo.setColumnName("old_col");
			smo.setColumnName2("renamed_col");
			
		
		}else

		if(type.equalsIgnoreCase(SMOInterface.PARTITION)){
			
			smo.setTableName("emp");
			smo.setTableName2("emp_part1");
			smo.setTableName3("emp_part2");
			
			Disjunctive targetConditions2 = new Disjunctive();
			Disjunctive targetConditions3 = new Disjunctive();
			
			targetConditions2.add(new Conjunctive(new Predicate(new Term("emp","eid"),"=",new Term("const2"))));
			targetConditions3.add(new Conjunctive(new Predicate(new Term("emp","eid"),"=",new Term("const3"))));
			
			smo.setTargetConditions2(targetConditions2);
			smo.setTargetConditions3(targetConditions3);
		
		
		}else
		
		if(type.equalsIgnoreCase(SMOInterface.MERGE)){
			
			smo.setTableName("emp_to_merge1");
			smo.setTableName2("emp_to_merge2");
			smo.setTableName3("emp_to_merge2");
		
		}else
		
		if(type.equalsIgnoreCase(SMOInterface.DECOMPOSE)){
			
		smo.setTableName("emp");
		smo.setTableName2("emp_decomposed1");
		smo.setTableName3("emp_decomposed2");
		
		Vector<String> columnNameList2 = new Vector<String>();
		columnNameList2.add("eid");
		columnNameList2.add("title");
		smo.setAttributeList2(columnNameList2);

		
		Vector<String> columnNameList3 = new Vector<String>();
		columnNameList3.add("eid");
		columnNameList3.add("test");
		smo.setAttributeList3(columnNameList3);
		
		}else
			
		if(type.equalsIgnoreCase(SMOInterface.DROP_TAB)){
			smo.setTableName("emp");				
		}else
		
		if(type.equalsIgnoreCase(SMOInterface.NOP)){
			
		}else
		
		if(type.equalsIgnoreCase(SMOInterface.CREATE_TAB)){
			smo.setTableName("emp_created");
			Vector<String> v = new Vector<String>();
			v.add("a");
			v.add("b");
			v.add("c");
			smo.setColumnNameList(v);	
			
		
		}else
		
		if(type.equalsIgnoreCase(SMOInterface.DROP_COL)){
			smo.setTableName("emp");
			smo.setColumnName("tobedropped");
					
		}else 
		if(type.equalsIgnoreCase(SMOInterface.COPY_TAB)){
			smo.setTableName("emp");
			smo.setTableName2("emp_copy");
					
		}else 
		if(type.equalsIgnoreCase(SMOInterface.JOIN)){
			smo.setTableName("emp_to_join1");
			smo.setTableName2("emp_to_join2");
			smo.setTableName3("emp_joined");		
			Disjunctive targetConditions = new Disjunctive();
			Conjunctive o = new Conjunctive();
			Predicate p = new Predicate(new Term("emp_to_join1","myid"), "=", new Term("emp_to_join2","eid"));
			o.predicates.add(p);
			targetConditions.conjunctives.add(o);
			smo.setTargetConditions(targetConditions);
		
		}else if(type.equalsIgnoreCase(SMOInterface.REN_TAB)){
			smo.setTableName("emp");
			smo.setTableName2("emp2");
			
		}		
		if(type.equalsIgnoreCase(SMOInterface.ADD_COL)){
			smo.setTableName("emp");
			smo.setColumnName("added_col");
			smo.setDefaultConstant("cost");
		
		}
		
		ssh.smoSequences.add(ss);
		ss.smos.add(smo);
		
		return ssh;
	}

	public static SQL generateSampleSQL(String type) {
		SQL sql = new SQL();

		if(type.equalsIgnoreCase(SMOInterface.REN_COL)){
			sql.select.selectAttributes.add(new SelectAttribute("S0", "R0", "renamed_col"));
			sql.from.relationDeclarations.add(new RelationDeclaration("emp", "R0"));
		}
		
		if(type.equalsIgnoreCase(SMOInterface.PARTITION)){
			sql.select.selectAttributes.add(new SelectAttribute("S0", "R0", "test"));
			sql.from.relationDeclarations.add(new RelationDeclaration("emp_part1", "R0"));
		}
		
		if(type.equalsIgnoreCase(SMOInterface.MERGE)){
			sql.select.selectAttributes.add(new SelectAttribute("S0", "R0", "eid"));
			sql.from.relationDeclarations.add(new RelationDeclaration("emp_to_merge2", "R0"));
		}
		
		if(type.equalsIgnoreCase(SMOInterface.DECOMPOSE)){
			sql.select.selectAttributes.add(new SelectAttribute("S0", "R0", "test"));
			sql.from.relationDeclarations.add(new RelationDeclaration("emp_decomposed2", "R0"));
		}
		
		if(type.equalsIgnoreCase(SMOInterface.DROP_TAB)){
			sql.select.selectAttributes.add(new SelectAttribute("S0", "R0", "eid"));
			sql.from.relationDeclarations.add(new RelationDeclaration("emp_to_merge1", "R0"));
		}
		
		if(type.equalsIgnoreCase(SMOInterface.NOP)){
			sql.select.selectAttributes.add(new SelectAttribute("S0", "R0", "eid"));
			sql.from.relationDeclarations.add(new RelationDeclaration("emp", "R0"));
		}

		if(type.equalsIgnoreCase(SMOInterface.CREATE_TAB)){
			sql.select.selectAttributes.add(new SelectAttribute("S0", "R0", "a"));
			sql.from.relationDeclarations.add(new RelationDeclaration("emp_created", "R0"));
		}

		if(type.equalsIgnoreCase(SMOInterface.DROP_COL)){
			sql.select.selectAttributes.add(new SelectAttribute("S0", "R0", "eid"));
			sql.from.relationDeclarations.add(new RelationDeclaration("emp", "R0"));
		}
		
		if(type.equalsIgnoreCase(SMOInterface.COPY_TAB)){
			sql.select.selectAttributes.add(new SelectAttribute("S0", "R0", "eid"));
			sql.from.relationDeclarations.add(new RelationDeclaration("emp_copy", "R0"));
		}
		
		
		if(type.equalsIgnoreCase(SMOInterface.REN_TAB)){
			sql.select.selectAttributes.add(new SelectAttribute("S0", "R0", "eid"));
			sql.from.relationDeclarations.add(new RelationDeclaration("emp2", "R0"));
		}
		
		if(type.equalsIgnoreCase(SMOInterface.ADD_COL)){
			sql.select.selectAttributes.add(new SelectAttribute("S0", "R0", "added_col"));
			sql.from.relationDeclarations.add(new RelationDeclaration("emp", "R0"));
		}

		if(type.equalsIgnoreCase(SMOInterface.JOIN)){
			sql.select.selectAttributes.add(new SelectAttribute("S0", "R0", "title"));
			sql.from.relationDeclarations.add(new RelationDeclaration("emp_joined", "R0"));
		}
		return sql;
	}
	
	
	
	
	public static SchemaHistory generateSampleSchemaHistory(String type) {
		
		
		SchemaHistory schemaHistory = new SchemaHistory();
		Schema schema;
		Table table;
		

		if(type.equalsIgnoreCase(SMOInterface.REN_COL)){
			// SCHEMA 1.0
			schema = new Schema();
			schema.versionID = 1;
			schema.minorVersionID = 0;
			schemaHistory.schemas.add(schema);
		
			table = new Table("emp2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
			
			table = new Table("emp");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("old_col");

			
			// SCHEMA 1.1
			schema = new Schema();
			schema.versionID = 2;
			schema.minorVersionID = 0;
			schemaHistory.schemas.add(schema);
			
			table = new Table("emp");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("renamed_col");
		
			table = new Table("emp2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
			
		}
		
		if(type.equalsIgnoreCase(SMOInterface.PARTITION)){
			// SCHEMA 1.0
			schema = new Schema();
			schema.versionID = 1;
			schema.minorVersionID = 0;
			schemaHistory.schemas.add(schema);
		
			table = new Table("emp2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
			
			table = new Table("emp");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");

			
			// SCHEMA 1.1
			schema = new Schema();
			schema.versionID = 2;
			schema.minorVersionID = 0;
			schemaHistory.schemas.add(schema);
			
			table = new Table("emp");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		
			table = new Table("emp_part1");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
			
			table = new Table("emp_part2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		}
		
		if(type.equalsIgnoreCase(SMOInterface.MERGE)){
			// SCHEMA 1.0
			schema = new Schema();
			schema.versionID = 1;
			schema.minorVersionID = 0;
			schemaHistory.schemas.add(schema);
		
			table = new Table("emp2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
			
			table = new Table("emp_to_merge1");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		
			table = new Table("emp_to_merge2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");

			
			// SCHEMA 1.1
			schema = new Schema();
			schema.versionID = 2;
			schema.minorVersionID = 0;
			schemaHistory.schemas.add(schema);
			
			table = new Table("emp_to_merge2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		
			table = new Table("emp2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		}
		
		if(type.equalsIgnoreCase(SMOInterface.DECOMPOSE)){
			// SCHEMA 1.0
			schema = new Schema();
			schema.versionID = 1;
			schema.minorVersionID = 0;
			schemaHistory.schemas.add(schema);
		
			table = new Table("emp");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
			
			table = new Table("emp_to_merge1");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		
			table = new Table("emp_to_merge2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");

			
			// SCHEMA 1.1
			schema = new Schema();
			schema.versionID = 2;
			schema.minorVersionID = 0;
			schemaHistory.schemas.add(schema);
			
			table = new Table("emp_decomposed1");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			
			table = new Table("emp_decomposed2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("test");
			
			table = new Table("emp_to_merge1");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		
			table = new Table("emp_to_merge2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		}
		
		if(type.equalsIgnoreCase(SMOInterface.DROP_TAB)){
			// SCHEMA 1.0
			schema = new Schema();
			schema.versionID = 1;
			schema.minorVersionID = 0;
			schemaHistory.schemas.add(schema);
		
			table = new Table("emp");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
			
			table = new Table("emp_to_merge1");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		
			table = new Table("emp_to_merge2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");

			
			// SCHEMA 1.1
			schema = new Schema();
			schema.versionID = 2;
			schema.minorVersionID = 0;
			schemaHistory.schemas.add(schema);
			
			table = new Table("emp_to_merge1");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		
			table = new Table("emp_to_merge2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		}
		
		if(type.equalsIgnoreCase(SMOInterface.NOP)){
			// SCHEMA 1.0
			schema = new Schema();
			schema.versionID = 1;
			schema.minorVersionID = 0;
			schemaHistory.schemas.add(schema);
		
			table = new Table("emp");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
			
			table = new Table("emp_to_merge1");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		
			table = new Table("emp_to_merge2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");

			
			// SCHEMA 1.1
			schema = new Schema();
			schema.versionID = 2;
			schema.minorVersionID = 0;
			schemaHistory.schemas.add(schema);

			table = new Table("emp");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
			
			table = new Table("emp_to_merge1");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		
			table = new Table("emp_to_merge2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		}
		
		if(type.equalsIgnoreCase(SMOInterface.CREATE_TAB)){
			// SCHEMA 1.0
			schema = new Schema();
			schema.versionID = 1;
			schema.minorVersionID = 0;
			schemaHistory.schemas.add(schema);
		
			table = new Table("emp");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
			
			table = new Table("emp_to_merge1");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		
			table = new Table("emp_to_merge2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");

			
			// SCHEMA 1.1
			schema = new Schema();
			schema.versionID = 2;
			schema.minorVersionID = 0;
			schemaHistory.schemas.add(schema);

			table = new Table("emp");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");

			table = new Table("emp_created");
			schema.addTable(table);
			table.addColumn("a");
			table.addColumn("b");
			table.addColumn("c");
			
			table = new Table("emp_to_merge1");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		
			table = new Table("emp_to_merge2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		}
		
		if(type.equalsIgnoreCase(SMOInterface.DROP_COL)){
			// SCHEMA 1.0
			schema = new Schema();
			schema.versionID = 1;
			schema.minorVersionID = 0;
			schemaHistory.schemas.add(schema);
		
			table = new Table("emp");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
			table.addColumn("tobedropped");
			
			table = new Table("emp_to_merge1");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		
			table = new Table("emp_to_merge2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");

			
			// SCHEMA 1.1
			schema = new Schema();
			schema.versionID = 2;
			schema.minorVersionID = 0;
			schemaHistory.schemas.add(schema);

			table = new Table("emp");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
				
			table = new Table("emp_to_merge1");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		
			table = new Table("emp_to_merge2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		}
		
		if(type.equalsIgnoreCase(SMOInterface.COPY_TAB)){
			// SCHEMA 1.0
			schema = new Schema();
			schema.versionID = 1;
			schema.minorVersionID = 0;
			schemaHistory.schemas.add(schema);
		
			table = new Table("emp");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		    			
			table = new Table("emp_to_merge1");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		
			table = new Table("emp_to_merge2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");

			
			// SCHEMA 1.1
			schema = new Schema();
			schema.versionID = 2;
			schema.minorVersionID = 0;
			schemaHistory.schemas.add(schema);

			table = new Table("emp");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
			
			table = new Table("emp_copy");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
				
			table = new Table("emp_to_merge1");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		
			table = new Table("emp_to_merge2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		}
		
		if(type.equalsIgnoreCase(SMOInterface.REN_TAB)){
			// SCHEMA 1.0
			schema = new Schema();
			schema.versionID = 1;
			schema.minorVersionID = 0;
			schemaHistory.schemas.add(schema);
		
			table = new Table("emp");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		    			
			table = new Table("emp_to_merge1");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		
			table = new Table("emp_to_merge2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");

			
			// SCHEMA 1.1
			schema = new Schema();
			schema.versionID = 2;
			schema.minorVersionID = 0;
			schemaHistory.schemas.add(schema);
				
			table = new Table("emp2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
				
			table = new Table("emp_to_merge1");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		
			table = new Table("emp_to_merge2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		}
		
		
		if(type.equalsIgnoreCase(SMOInterface.ADD_COL)){
			// SCHEMA 1.0
			schema = new Schema();
			schema.versionID = 1;
			schema.minorVersionID = 0;
			schemaHistory.schemas.add(schema);
		
			table = new Table("emp");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		    			
			table = new Table("emp_to_merge1");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		
			table = new Table("emp_to_merge2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");

			
			// SCHEMA 1.1
			schema = new Schema();
			schema.versionID = 2;
			schema.minorVersionID = 0;
			schemaHistory.schemas.add(schema);
				
			table = new Table("emp");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
			table.addColumn("added_col");
			
			table = new Table("emp_to_merge1");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		
			table = new Table("emp_to_merge2");
			schema.addTable(table);
			table.addColumn("eid");
			table.addColumn("title");
			table.addColumn("test");
		}
		
		
		
		if(type.equalsIgnoreCase(SMOInterface.JOIN)){
		
		// SCHEMA 1.0
		schema = new Schema();
		schema.versionID = 1;
		schema.minorVersionID = 0;
		schemaHistory.schemas.add(schema);

		
		table = new Table("emp_to_merge1");
		schema.addTable(table);
		table.addColumn("eid");
		table.addColumn("title");
		table.addColumn("test");
	
		table = new Table("emp_to_merge2");
		schema.addTable(table);
		table.addColumn("eid");
		table.addColumn("title");
		table.addColumn("test");

		table = new Table("emp_to_join1");
		schema.addTable(table);
		table.addColumn("myid");
		table.addColumn("title");
	
		table = new Table("emp_to_join2");
		schema.addTable(table);
		table.addColumn("eid");
		table.addColumn("test");

		
		// SCHEMA 1.1
		schema = new Schema();
		schema.versionID = 2;
		schema.minorVersionID = 0;
		schemaHistory.schemas.add(schema);
		
		table = new Table("emp_joined");
		schema.addTable(table);
		table.addColumn("eid");
		table.addColumn("myid");
		table.addColumn("title");
		table.addColumn("test");
			
		table = new Table("emp_to_merge1");
		schema.addTable(table);
		table.addColumn("eid");
		table.addColumn("title");
		table.addColumn("test");
	
		table = new Table("emp_to_merge2");
		schema.addTable(table);
		table.addColumn("eid");
		table.addColumn("title");
		table.addColumn("test");
		}
		
		return schemaHistory;
	}
	
	
}
