package edu.ucla.cs.db.prism.backend.oldtestmain;

import java.util.Vector;

import edu.ucla.cs.db.prism.backend.schema.*;
import edu.ucla.cs.db.prism.backend.smo.*;
import edu.ucla.cs.db.prism.backend.smo.util.*;

public class MainSMOtoViewTest {

	public static void main(String[] args) {
		
		testADDCOLUMN();
		testCOPYTABLE();
		testCREATETABLE();
		testRENAMETABLE();
		testDROPCOLUMN();
		testDROPTABLE();
		testNOP();
		testRENAMECOLUMN();
		testPARTITIONTABLE();
		testMERGETABLE();
		testDECOMPOSETABLE();
		testJOINTABLE();
	}

	public static SchemaHistory generateSampleSchemaHistory() {
		SchemaHistory schemaHistory = new SchemaHistory();
		Schema schema;
		Table table;
		
		schema = new Schema();
		
		table = new Table("emp");
		schema.addTable(table);
		table.addColumn("eid");
		table.addColumn("title");
		table.addColumn("test");
		
		schema.versionID = 1;
		schema.minorVersionID = 0;
		schemaHistory.schemas.add(schema);
		table = new Table("emp_to_merge1");
		schema.addTable(table);
		table.addColumn("eid");
		table.addColumn("title");
		table.addColumn("test");
	
		table = new Table("emp_to_merge2");
		schema.addTable(table);
		table.addColumn("eid");
		table.addColumn("title");
		table.addColumn("test");

		table = new Table("emp_to_join1");
		schema.addTable(table);
		table.addColumn("eid");
		table.addColumn("test");
	
		table = new Table("emp_to_join2");
		schema.addTable(table);
		table.addColumn("eid");
		table.addColumn("title");

		
		
		schema = new Schema();
		schema.versionID = 1;
		schema.minorVersionID = 1;
		schemaHistory.schemas.add(schema);
		table = new Table("emp_joined");
		schema.addTable(table);
		table.addColumn("eid");
		table.addColumn("title");
		table.addColumn("test");
		

		
		
		return schemaHistory;
	}
	
	
	public static void testRENAMETABLE(){
		
		SMOImpl smo = null;
		try {
			smo = SMOFactory.createSMO(SMOInterface.REN_TAB);
			smo.setSourceMajor(2);
			smo.setSourceMinor(2);
			smo.setTargetMajor(2);
			smo.setTargetMinor(1);
			
		} catch (SMOTypeUnknownException e) {
			e.printStackTrace();
		}

		smo.setTableName("emp");
		smo.setTableName2("emp_renamed");
		
		System.out.println("-----------------------------------------");
		
		System.out.println(smo);
		
		// get a testing schema, emp(eid, title)
		SchemaHistory schemaHistory = generateSampleSchemaHistory();
		int[] version = {1,0}; // v1m0
		Schema schema = schemaHistory.getSchemaByVersion(version);
		
		// test
		System.out.println(smo.getSQLView(schema, "wikipedia_1", "wikipedia_2"));

		
		
	}
	
	public static void testADDCOLUMN(){
		
		SMOImpl smo = null;
		try {
			smo = SMOFactory.createSMO(SMOInterface.ADD_COL);
			smo.setSourceMajor(2);
			smo.setSourceMinor(2);
			smo.setTargetMajor(2);
			smo.setTargetMinor(1);
			
		} catch (SMOTypeUnknownException e) {
			e.printStackTrace();
		}
		smo.setFunctionName("myfunc");
		Vector<String> v = new Vector<String>();
		v.add("eid");
		smo.setFunctionArgs(v);
		//smo.setDefaultConstant("cost");
		smo.setTableName("emp");
		smo.setColumnName("added");
		
		System.out.println("-----------------------------------------");
		
		System.out.println(smo);
		
		// get a testing schema, emp(eid, title)
		SchemaHistory schemaHistory = generateSampleSchemaHistory();
		int[] version = {1,0}; // v1m0
		Schema schema = schemaHistory.getSchemaByVersion(version);
		

		// test
		System.out.println(smo.getSQLView(schema,"wikipedia_1","wikipedia_2"));
	}
	
	public static void testDROPCOLUMN(){
		
		SMOImpl smo = null;
		try {
			smo = SMOFactory.createSMO(SMOInterface.DROP_COL);
			smo.setSourceMajor(2);
			smo.setSourceMinor(2);
			smo.setTargetMajor(2);
			smo.setTargetMinor(1);
		} catch (SMOTypeUnknownException e) {
			e.printStackTrace();
		}

		smo.setTableName("emp");
		smo.setColumnName("eid");
		
		System.out.println("-----------------------------------------");
		System.out.println(smo);
		
		// get a testing schema, emp(eid, title)
		SchemaHistory schemaHistory = generateSampleSchemaHistory();
		int[] version = {1,0}; // v1m0
		Schema schema = schemaHistory.getSchemaByVersion(version);
		

		// test
		System.out.println(smo.getSQLView(schema, "wikipedia_1", "wikipedia_2"));
	}
	
	public static void testCREATETABLE(){
		
		SMOImpl smo = null;
		try {
			smo = SMOFactory.createSMO(SMOInterface.CREATE_TAB);
			smo.setSourceMajor(2);
			smo.setSourceMinor(2);
			smo.setTargetMajor(2);
			smo.setTargetMinor(1);
		} catch (SMOTypeUnknownException e) {
			e.printStackTrace();
		}

		smo.setTableName("emp_new");
		Vector<String> v = new Vector<String>();
		v.add("eid2");
		v.add("title2");
		v.add("test2");
		smo.setColumnNameList(v);
		
		System.out.println("-----------------------------------------");
		System.out.println(smo);
		
		// get a testing schema, emp(eid, title)
		SchemaHistory schemaHistory = generateSampleSchemaHistory();
		int[] version = {1,0}; // v1m0
		Schema schema = schemaHistory.getSchemaByVersion(version);
		

		// test
		System.out.println(smo.getSQLView(schema, "wikipedia_1", "wikipedia_2"));
	}
	
	public static void testDROPTABLE(){
		
		SMOImpl smo = null;
		try {
			smo = SMOFactory.createSMO(SMOInterface.DROP_TAB);
			smo.setSourceMajor(2);
			smo.setSourceMinor(2);
			smo.setTargetMajor(2);
			smo.setTargetMinor(1);
		} catch (SMOTypeUnknownException e) {
			e.printStackTrace();
		}

		smo.setTableName("emp");
		
		System.out.println("-----------------------------------------");
		System.out.println(smo);
		
		// get a testing schema, emp(eid, title)
		SchemaHistory schemaHistory = generateSampleSchemaHistory();
		int[] version = {1,0}; // v1m0
		Schema schema = schemaHistory.getSchemaByVersion(version);
		

		// test
		System.out.println(smo.getSQLView(schema, "wikipedia_1", "wikipedia_2"));
	}
	
	public static void testNOP(){
		
		SMOImpl smo = null;
		try {
			smo = SMOFactory.createSMO(SMOInterface.NOP);
			smo.setSourceMajor(2);
			smo.setSourceMinor(2);
			smo.setTargetMajor(2);
			smo.setTargetMinor(1);
		} catch (SMOTypeUnknownException e) {
			e.printStackTrace();
		}
		
		System.out.println("-----------------------------------------");
		System.out.println(smo);
		
		// get a testing schema, emp(eid, title)
		SchemaHistory schemaHistory = generateSampleSchemaHistory();
		int[] version = {1,0}; // v1m0
		Schema schema = schemaHistory.getSchemaByVersion(version);
		
		// test
		System.out.println(smo.getSQLView(schema, "wikipedia_1", "wikipedia_2"));

	}
	
	public static void testCOPYTABLE(){
		
		SMOImpl smo = null;
		try {
			smo = SMOFactory.createSMO(SMOInterface.COPY_TAB);
			smo.setSourceMajor(2);
			smo.setSourceMinor(2);
			smo.setTargetMajor(2);
			smo.setTargetMinor(1);
		} catch (SMOTypeUnknownException e) {
			e.printStackTrace();
		}

		smo.setTableName("emp");
		smo.setTableName2("emp_new");
		
		System.out.println("-----------------------------------------");
		System.out.println(smo);
		
		// get a testing schema, emp(eid, title)
		SchemaHistory schemaHistory = generateSampleSchemaHistory();
		int[] version = {1,0}; // v1m0
		Schema schema = schemaHistory.getSchemaByVersion(version);
		
		// test
		System.out.println(smo.getSQLView(schema, "wikipedia_1", "wikipedia_2"));
	}

	public static void testRENAMECOLUMN(){
		
		SMOImpl smo = null;
		try {
			smo = SMOFactory.createSMO(SMOInterface.REN_COL);
			smo.setSourceMajor(2);
			smo.setSourceMinor(2);
			smo.setTargetMajor(2);
			smo.setTargetMinor(1);
			
		} catch (SMOTypeUnknownException e) {
			e.printStackTrace();
		}

		smo.setTableName("emp");
		smo.setColumnName("eid");
		smo.setColumnName2("eid_renamed");
		
		System.out.println("-----------------------------------------");
		
		System.out.println(smo);
		
		// get a testing schema, emp(eid, title)
		SchemaHistory schemaHistory = generateSampleSchemaHistory();
		int[] version = {1,0}; // v1m0
		Schema schema = schemaHistory.getSchemaByVersion(version);
		
		// test
		System.out.println(smo.getSQLView(schema, "wikipedia_1", "wikipedia_2"));

	}

	public static void testPARTITIONTABLE(){
		
		SMOImpl smo = null;
		try {
			smo = SMOFactory.createSMO(SMOInterface.PARTITION);
			smo.setSourceMajor(2);
			smo.setSourceMinor(2);
			smo.setTargetMajor(2);
			smo.setTargetMinor(1);
			
		} catch (SMOTypeUnknownException e) {
			e.printStackTrace();
		}

		smo.setTableName("emp");
		smo.setTableName2("emp2");
		smo.setTableName3("emp3");
		
		
		Disjunctive targetConditions2 = new Disjunctive();
		Disjunctive targetConditions3 = new Disjunctive();
		
		targetConditions2.add(new Conjunctive(new Predicate(new Term("emp","eid"),"=",new Term("const2"))));
		targetConditions3.add(new Conjunctive(new Predicate(new Term("emp","eid"),"=",new Term("const3"))));
				
		smo.setTargetConditions2(targetConditions2);
		smo.setTargetConditions3(targetConditions3);
		
		System.out.println("-----------------------------------------");
		
		System.out.println(smo);
		
		// get a testing schema, emp(eid, title)
		SchemaHistory schemaHistory = generateSampleSchemaHistory();
		int[] version = {1,0}; // v1m0
		Schema schema = schemaHistory.getSchemaByVersion(version);
		
		// test
		System.out.println(smo.getSQLView(schema, "wikipedia_1", "wikipedia_2"));

	}

	public static void testMERGETABLE(){
		
		SMOImpl smo = null;
		try {
			smo = SMOFactory.createSMO(SMOInterface.MERGE);
			smo.setSourceMajor(2);
			smo.setSourceMinor(2);
			smo.setTargetMajor(2);
			smo.setTargetMinor(1);
			
		} catch (SMOTypeUnknownException e) {
			e.printStackTrace();
		}

		smo.setTableName("emp_to_merge1");
		smo.setTableName2("emp_to_merge2");
		smo.setTableName3("emp_merged");

		System.out.println("-----------------------------------------");
		
		System.out.println(smo);
		
		// get a testing schema, emp(eid, title)
		SchemaHistory schemaHistory = generateSampleSchemaHistory();
		int[] version = {1,0}; // v1m0
		Schema schema = schemaHistory.getSchemaByVersion(version);
		
		// test
		System.out.println(smo.getSQLView(schema, "wikipedia_1", "wikipedia_2"));
	}
	
	public static void testDECOMPOSETABLE(){
		
		SMOImpl smo = null;
		try {
			smo = SMOFactory.createSMO(SMOInterface.DECOMPOSE);
			smo.setSourceMajor(2);
			smo.setSourceMinor(2);
			smo.setTargetMajor(2);
			smo.setTargetMinor(1);
			
		} catch (SMOTypeUnknownException e) {
			e.printStackTrace();
		}

		smo.setTableName("emp");
		smo.setTableName2("emp_decomposed1");
		smo.setTableName3("emp_decomposed2");
		
		Vector<String> columnNameList2 = new Vector<String>();
		columnNameList2.add("eid");
		columnNameList2.add("title");
		smo.setAttributeList2(columnNameList2);

		
		Vector<String> columnNameList3 = new Vector<String>();
		columnNameList3.add("eid");
		columnNameList3.add("test");
		smo.setAttributeList3(columnNameList3);
		
		
		System.out.println("-----------------------------------------");
		
		System.out.println(smo);
		
		// get a testing schema, emp(eid, title)
		SchemaHistory schemaHistory = generateSampleSchemaHistory();
		int[] version = {1,0}; // v1m0
		Schema schema = schemaHistory.getSchemaByVersion(version);
		
		// test
		System.out.println(smo.getSQLView(schema, "wikipedia_1", "wikipedia_2"));
	}
	
	
	public static void testJOINTABLE(){
		
		SMOImpl smo = null;
		try {
			smo = SMOFactory.createSMO(SMOInterface.JOIN);
			smo.setSourceMajor(2);
			smo.setSourceMinor(2);
			smo.setTargetMajor(2);
			smo.setTargetMinor(1);
			
		} catch (SMOTypeUnknownException e) {
			e.printStackTrace();
		}

		smo.setTableName("emp_to_join1");
		smo.setTableName2("emp_to_join2");
		smo.setTableName3("emp_joined");
		
		
		
		Disjunctive targetConditions = new Disjunctive();
		Conjunctive o = new Conjunctive();
		Predicate p = new Predicate(new Term("emp_to_join1","eid"), "=", new Term("emp_to_join2","eid"));
		o.predicates.add(p);
		targetConditions.conjunctives.add(o);
		smo.setTargetConditions(targetConditions);
		
		
		System.out.println("-----------------------------------------");
		
		System.out.println(smo);
		
		// get a testing schema, emp(eid, title)
		SchemaHistory schemaHistory = generateSampleSchemaHistory();
		int[] version = {1,0}; // v1m0
		Schema schema = schemaHistory.getSchemaByVersion(version);
		
		// test
		System.out.println(smo.getSQLView(schema, "wikipedia_1", "wikipedia_2"));

	}
	
	
}
