package edu.ucla.cs.db.prism.backend.oldtestmain;
import java.io.*;
import java.util.*;


import edu.ucla.cs.db.prism.backend.parser.syntaxtree.*;
import edu.ucla.cs.db.prism.backend.parser.parser.*;
import edu.ucla.cs.db.prism.backend.parser.visitor.*;
import edu.ucla.cs.db.prism.backend.util.*;
import edu.ucla.cs.db.prism.backend.compose.DEDComposition;
import edu.ucla.cs.db.prism.backend.ded.*;
import edu.ucla.cs.db.prism.backend.exception.ErrorRewritingException;
import edu.ucla.cs.db.prism.backend.restructure.CommandRestructureException;
import edu.ucla.cs.db.prism.backend.rewrite.PrimRewriteEngine;
import edu.ucla.cs.db.prism.backend.schema.*;
import edu.ucla.cs.db.prism.backend.smo.*;
import edu.ucla.cs.db.prism.backend.smo.util.*;
import edu.ucla.cs.db.prism.backend.SQL.*;

public class MainScriptInput {
	// input: PRM script (schema, smos, XQuery input, command)
	// using parser
	
	final static boolean packagingTakeInputFromCommandLineArgs = true; // should set true, when creating jar
	static boolean optionSmoPruningFlag = false;
	static boolean optionSmoCompressFlag = false;
	static boolean optionShowRewrittenXQueryOnly = false;
	static boolean optionSourceVersionPruning = false;
	static boolean optionUnionOfJoin = false;
	static boolean optionDBOutput = false;

	final static boolean debuggingSkipChaseAndUsePreviousMarsResult = false; // "true" uses existing out.txt
	final static boolean debuggingShowTargetSchemaVersion = false;
	final static boolean debuggingShowSMOsUsed = false;
	final static boolean debuggingShowQueryFootprintSchemaHistory = false;

	//final static boolean postprocessingOptimized = true; - optimized by default now
	
	final static String inputPrimaScriptFileName = "sample-0.pri";
	//	final static String inputPrimaScriptFileName = "wikiall-20071029b.txt";
	//	final static String inputPrimaScriptFileName = "empdb-20071029.txt";
	
	public static void main(String[] args) {

		// step 1: experiment result recording setup
		PrimaExperimentResult expResult = new PrimaExperimentResult();
		expResult.exp_timestamp = new Date();

		// step 2: parse optimization options
		parseCommandLineArguments(args, expResult);
		
		// step 3: prima
		primaEntryPoint(args, expResult);

		// step 4: output experiment result
		if(optionShowRewrittenXQueryOnly) {
			if(expResult.rewritten_query_text != null)
				System.out.println(expResult.rewritten_query_text);
			else
				System.out.println("");
		}
		else if(optionDBOutput)
			System.out.println(expResult.toStringDBFriendly());
		else
			System.out.println(""
					//expResult.toStringHumanFriendly()
					);
	}


	private static void primaEntryPoint(String[] args, PrimaExperimentResult expResult) {
		Stopwatch primaTotalTimer = new Stopwatch();
		Stopwatch primaInputParseTimer = new Stopwatch();
		Stopwatch primaPreprocessingTimer = new Stopwatch();
		

		primaTotalTimer.start();
		
		/////////////////////////////////////////////////////////////
		// step 1: parsing
		/////////////////////////////////////////////////////////////
		primaInputParseTimer.start();
		SmoTranslateVisitor smotv = mainParse(args);
		expResult.prima_input_parse_time = primaInputParseTimer.stop();
		
		
		/////////////////////////////////////////////////////////////
		// step 2: global preprocessing - for the entire prima script
		/////////////////////////////////////////////////////////////
		primaPreprocessingTimer.start();
		SchemaHistory schemaHistory = mainGlobalPreprocessing(smotv);
		expResult.prima_input_parse_time += primaPreprocessingTimer.stop();
		
		
		/////////////////////////////////////////////////////////////
		// step 3: per-command preprocessing + execution
		/////////////////////////////////////////////////////////////

		for(int i=0; i<smotv.computeCommandSequence.size(); i++) {
			perCommandPreprocessingExecution(smotv, schemaHistory, smotv.computeCommandSequence.get(i), expResult);
		}
		
		expResult.prima_total_time = primaTotalTimer.stop();		
	}

	
	

	private static SmoTranslateVisitor mainParse(String args[]) {
		FileInputStream input = null;
		// step 1.1: fetch prima script filename to read
		try {
			if(packagingTakeInputFromCommandLineArgs && args.length > 0) {
				String filename = args[args.length-1];
				//System.out.println("Processing " + filename + " ...\n");
				input = new FileInputStream(filename);
			}
			else if(!packagingTakeInputFromCommandLineArgs) {
				//System.out.println("Processing " + inputPrimaScriptFileName + " ...\n");
				input = new FileInputStream(inputPrimaScriptFileName);
			}
			else {
				usage();
				System.exit(0);
			}
		} catch (FileNotFoundException e) {
			System.out.println("Error: input file is not found.");
			System.exit(-1);
		}

		// step 1.2: parse it
		PrimaRParser parser = new PrimaRParser(input);
		PrimaRScript script = null;
		try {
			script = parser.PrimaRScript();
		} catch(Exception e) {
			e.printStackTrace();
		}
		
		// step 1.3: restructure abstract syntax tree (AST) into our defined class objects
		SmoTranslateVisitor smotv = new SmoTranslateVisitor();
		script.accept(smotv);
		return smotv;
	}
	
	private static SchemaHistory mainGlobalPreprocessing(SmoTranslateVisitor smotv) {
		// step 2.1: compute the inverse of SMOs to make SMO Sequence History complete
		//         in both forward and backward directions
		//smotv.smoSequenceHistory.computeInverse();

		// step 2.2: compute schema history using SMOs (not views!)
		SchemaHistory schemaHistory = new SchemaHistory(smotv.schema, smotv.smoSequenceHistory);
		
		// step 2.3: version interval compilation finish (constant time operation)
		smotv.versionIntervalSet.SetEndTime();

		return schemaHistory;
	}
	

	private static void perCommandPreprocessingExecution(SmoTranslateVisitor smotv, SchemaHistory schemaHistory,
			ComputeCommand command, PrimaExperimentResult expResult) {
		
		Stopwatch primaPreprocessingTimer = new Stopwatch();

		primaPreprocessingTimer.start();
		////////////////////////////////////////////////////////////////////
		// step 3.1: query rewriting command restructure
		////////////////////////////////////////////////////////////////////
		String inputQueryName = 
			edu.ucla.cs.db.prism.backend.restructure.ComputeCommandRestructure.getInputQueryName(command);

		// step 3.1.1: input query must be found with inputQueryName
		SQLUnion sqlUnion = null;
		for(int j=0; j<smotv.sqlUnionSet.sqlUnions.size(); j++) {
			if(smotv.sqlUnionSet.sqlUnions.get(j).name.equals(inputQueryName)) {
				sqlUnion = smotv.sqlUnionSet.sqlUnions.get(j);
				break;
			}
		}
		if(sqlUnion == null) {
			System.err.println("Used query " + inputQueryName + " is not declared");
			System.exit(0);
		}
		
		// step 3.1.2: inputSQLUnion is needed for union-of-join optimization case only
		// so it may or may not be there
		// (it's needed because XQuery-to-XBind translator is not implemented yet)
		// 11-5-2007
		SQLUnion inputSQLUnion = null;
		for(int j=0; j<smotv.sqlUnionSet.sqlUnions.size(); j++) {
			if(smotv.sqlUnionSet.sqlUnions.get(j).name.equals(inputQueryName)) {
				inputSQLUnion = smotv.sqlUnionSet.sqlUnions.get(j);
				//XBindManage.simplifyInequalities(inputSQLUnion); //Quickfix
				break;
			}
		}
		SQL inputSQL = null;
		if(optionUnionOfJoin && inputSQLUnion == null) {
			System.err.println("unionofjoin option requires translated XBind query also for " + inputQueryName);
			System.exit(0);
		}
		else if(inputSQLUnion != null)
			// we don't take the union of SQL - we parse it but uses the only first portion only, for simplicity
			inputSQL = inputSQLUnion.sqls.get(0);  

		// step 3.1.3: parse the target version and source version(s)
		// minimize source version using the query's temporal constraints
		// if optimization is turned on
		int targetVersion = 
			edu.ucla.cs.db.prism.backend.restructure.ComputeCommandRestructure.getInputQuerySchemaVersion(command);
		expResult.target_version = targetVersion;
		if(debuggingShowTargetSchemaVersion) {
			//int version = targetVersion;
			//int version = 42;s
			//System.out.println("Schema V" + version + "\n" + schemaHistory.getSchemaByVersion(version));
			//for(version = 1; version<=10; version++)
			//	System.out.println("Schema V" + version + "\n" + 
			//			schemaHistory.getSchemaByVersion(version).getTable("ipblocks"));
		}

		Vector<Integer> sourceVersions = null;
		try {
			sourceVersions = edu.ucla.cs.db.prism.backend.restructure.ComputeCommandRestructure.getSourceSchemaVersions(command, 
					schemaHistory, smotv.smoSequenceHistory);
		} catch (CommandRestructureException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		//util.Sort.sortIntegerVector(sourceVersions);


		// DO REWRITING HERE!
		DEDSetHistory dedSetHistory = getDEDSetHistory(schemaHistory, smotv.smoSequenceHistory);

		// compose
		DEDComposition dedComposeEngine = new DEDComposition(schemaHistory, dedSetHistory);
		//DEDSet dedSet = dedComposeEngine.compose(168, 0, 171, 0);
		dedComposeEngine.composeAllMajorVersionsToTargetVersionForwardDEDs(171);
		System.out.println("Finished DED composition task");

		writeObject(dedSetHistory, "javaDump.obj");
		System.exit(0);

		//DEDSetHistory dedSetHistory = (DEDSetHistory)readObject("javaDump.obj");
		
		
		runRewriteEngine(schemaHistory, dedSetHistory, inputSQL, sourceVersions.get(0), targetVersion);
	}
	

	public static void runRewriteEngine(SchemaHistory schemaHistory, DEDSetHistory dedSetHistory, 
			SQL inputSQL, int sourceVersion, int targetVersion) {
		// Step 1: set up rewriting engine - done only once
		PrimRewriteEngine pre = new PrimRewriteEngine(schemaHistory, dedSetHistory);

		// Step 2: rewriting procedure call - done repeatedly
		Stopwatch timer = new Stopwatch();
		timer.start();
		SQLUnion sqlUnion= null;
		try {
			sqlUnion = pre.rewrite(inputSQL, sourceVersion, 0, targetVersion, 0);
		} catch (ErrorRewritingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		timer.stop();
		 
		System.out.println("\nRewriting elapsed time: " + timer.getElapsedTime() + " ms");
		System.out.print(sqlUnion);
	}

	
	private static void parseCommandLineArguments(String[] args, PrimaExperimentResult expResult) {
		// default ids
		expResult.dataset_id = "fake_data_set0";
		expResult.query_id = "fake_query0";
		
		/*
		for(int i=0; i<args.length; i++) {
			if(args[i].startsWith("-qid="))
				expResult.query_id = args[i].substring(5);
			else if(args[i].startsWith("-did="))
				expResult.dataset_id = args[i].substring(5);
			else if(args[i].equals("-vp"))
				optionSourceVersionPruning = true;
			else if(args[i].equals("-uj"))
				optionUnionOfJoin = true;
			else if(args[i].equals("-sp"))
				optionSmoPruningFlag = true;
			else if(args[i].equals("-sc"))
				optionSmoCompressFlag = true;
			else if(args[i].equals("-sq"))
				optionShowRewrittenXQueryOnly = true;
			else if(args[i].equals("-db"))
				optionDBOutput = true;
		}
		*/
		
		expResult.optimize_source_version_prune = optionSourceVersionPruning;
		expResult.optimize_union_of_join = optionUnionOfJoin;
		expResult.optimize_smo_prune = optionSmoPruningFlag;
		expResult.optimize_smo_compress = optionSmoCompressFlag;
	}
	
	public static void usage() {
		System.out.println("Usage: java -jar prima-0.7.jar [options] PRIMA-script-filename");
		System.out.println("Options:");
		System.out.println("\t\t-qid=[query_id] : query id");
		System.out.println("\t\t-did=[dataset_id] : dataset id");
		System.out.println("\t\t-vp : source Version Prune optimization");
		System.out.println("\t\t-uj : Union-of-Join optimization on input SQL when possible");
		System.out.println("\t\t-sp : Smo Prune optimization");
		System.out.println("\t\t-sc : Smo Compression optimization");
		System.out.println("\t\t-sq : Show the rewritten Query only");
		System.out.println("\t\t-db : output results in DB-friendly way");
	}

	
	public static DEDSetHistory getDEDSetHistory(SchemaHistory schemaHistory, SMOSequenceHistory ssh) {
		DEDSetHistory dedSetHistory = new DEDSetHistory();
		
		Iterator<SMOSequence> smoIt = ssh.smoSequences.iterator();
		Iterator<Schema> schemas = schemaHistory.schemas.iterator();
		
		//System.out.println("Number of major and minor versions in schema history: " + 
		//		schemaHistory.size());
		
		//System.out.println(ssh);
		//System.out.println(ssh.toString());
		
		while(smoIt.hasNext()){
			SMOSequence ss = smoIt.next();
			Iterator<SMOImpl> sIt = ss.smos.iterator();
			while(sIt.hasNext()){
				Schema currentSchema = schemas.next();		
		//		System.out.println(currentSchema.toStringWithMetadata());
				SMOImpl smo = sIt.next();
		//		System.out.println(smo.toMetadataString());

				DEDSet ds = smo.getAllDEDs(currentSchema);			
				ds.sourceVersion  = smo.getSourceMajor();
				ds.minorSourceVersion = smo.getSourceMinor();
				ds.targetVersion = smo.getTargetMajor();
				ds.minorTargetVersion = smo.getTargetMinor();
				dedSetHistory.add(ds);
			}
		
		}
		return dedSetHistory;
	}

	public static void writeObject(Object object, String filename) {
		File theFile = new File(filename);
		FileOutputStream outStream;             // generic stream to the file
		ObjectOutputStream objStream;           // stream for objects to the file

		try {
			   // setup a stream to a physical file on the filesystem
			   outStream = new FileOutputStream(theFile);

			   // attach a stream capable of writing objects to the stream that is
			   // connected to the file
			   objStream = new ObjectOutputStream(outStream);

			 	objStream.writeObject(object);
			 	objStream.close();

			   // ... other activities ...
			} catch(IOException e) {
			   System.err.println("Things not going as planned.");
			   e.printStackTrace();
			}   // catch
		}
	
	public static Object readObject(String filename) {
		File theFile = new File(filename);
		FileInputStream inStream;             // generic stream to the file
		ObjectInputStream objStream;           // stream for objects to the file
		Object object = null;
		
		try {
			   // setup a stream to a physical file on the filesystem
			   inStream = new FileInputStream(theFile);

			   // attach a stream capable of writing objects to the stream that is
			   // connected to the file
			   objStream = new ObjectInputStream(inStream);

			   object = objStream.readObject();
 		 	   objStream.close();

			   // ... other activities ...
			} catch(Exception e) {
			   System.err.println("Things not going as planned.");
			   e.printStackTrace();
			}   // catch
			return object;
		}
}

