package edu.ucla.cs.db.prism.backend.oldtestmain;

import it.polimi.elet.cautils.configurator.CACmdLineParser;
import it.polimi.elet.cautils.configurator.CAConfiguration;

import java.sql.Connection;
import java.util.Iterator;
import org.apache.log4j.Logger;

import edu.ucla.cs.db.prism.backend.exception.ErrorRewritingException;
import edu.ucla.cs.db.prism.backend.SQL.SQL;
import edu.ucla.cs.db.prism.backend.SQL.SQLUnion;
import edu.ucla.cs.db.prism.backend.ded.*;
import edu.ucla.cs.db.prism.backend.rewrite.PrimRewriteEngine;
import edu.ucla.cs.db.prism.backend.schema.*;
import edu.ucla.cs.db.prism.backend.smo.*;
import edu.ucla.cs.db.prism.backend.smo.util.*;
import edu.ucla.cs.db.prism.backend.util.Log4JHelper;
import edu.ucla.cs.db.prism.backend.util.Stopwatch;
import edu.ucla.cs.db.prism.backend.util.db.DBUtil;
import edu.ucla.cs.db.prism.gui.server.DBLoader;

public class MainWikipediaSMOTest {

	//STATIC LOGGER
	static Logger logger = Logger.getLogger(DBLoader.class);
	
	public static void main(String[] args) {
		
		  //LOG4J SET UP
        Log4JHelper.log4JSetup(logger);
       
       
		//Use CAConfigurator to load configuration from shell or xml-configuration file (-c inputfile)
		 CACmdLineParser par = null;
	 	 try {
			par = CACmdLineParser.getInstance();
		 } catch (InstantiationException e) {
			e.printStackTrace();
		 }
		//List here all the interesting parameters
		 par.addStringParameter('h',"host");	
		 par.addStringParameter('u',"dburl");
		 par.addStringParameter('m',"metadbname");	
		 par.addStringParameter('u',"username");
		 par.addStringParameter('p',"password");
		 par.addStringParameter('l',"logfile");
		 
		//parse them all 
		 CAConfiguration conf = par.parse(args);
		 
		//read the parameter you are interested in 
		 String host = (String) conf.getParameterValue("host");
		 String dburl = (String) conf.getParameterValue("dburl");
		 String metadbname = (String) conf.getParameterValue("metadbname");
		 String dbname = (String) conf.getParameterValue("dbname");
		 String username = (String) conf.getParameterValue("username");
		 String password = (String) conf.getParameterValue("password");
		 
		//use them
		 System.out.println("PRIM rewriting query: ");
	//	 System.out.println("  - host: " + host);		
	//	 System.out.println("  - user: " + username);
	//	 System.out.println("  - metadb: " + metadbname);
		 
		// MySQL DRIVER LOADING 
       try {
           Class.forName("com.mysql.jdbc.Driver").newInstance();
       } catch (Exception ex) { ex.printStackTrace();}
	 

        Connection conn = DBUtil.connect(dburl,metadbname,username,password);
		    
       System.out.println("Connected to the DB:" + dburl +" " +metadbname+" " +username);
		 
		 SchemaHistoryLoader shl = new SchemaHistoryLoader(conn,dbname);
		 SchemaHistory schemaHistory = shl.loadSchemaHistoryFromDB();

		 logger.info("Schema Loaded");
		
		 SMOSequenceHistoryLoader sshl = new SMOSequenceHistoryLoader(conn,dbname);
	     SMOSequenceHistory ssh = sshl.loadSMOSequenceHistoryFromDB();
		 
		DEDSetHistory dedSetHistory = new DEDSetHistory();
		
		Iterator<SMOSequence> smoIt = ssh.smoSequences.iterator();
		Iterator<Schema> schemas = schemaHistory.schemas.iterator();
		
		System.out.println("Number of major and minor versions in schema history: " + 
				schemaHistory.size());
		
		//System.out.println(ssh);
		//System.out.println(ssh.toString());
		
		while(smoIt.hasNext()){
			SMOSequence ss = smoIt.next();
			Iterator<SMOImpl> sIt = ss.smos.iterator();
			while(sIt.hasNext()){
				Schema currentSchema = schemas.next();		
		//		System.out.println(currentSchema.toStringWithMetadata());
				SMOImpl smo = sIt.next();
		//		System.out.println(smo.toMetadataString());

				DEDSet ds = smo.getAllDEDs(currentSchema);			
				ds.sourceVersion  = smo.getSourceMajor();
				ds.minorSourceVersion = smo.getSourceMinor();
				ds.targetVersion = smo.getTargetMajor();
				ds.minorTargetVersion = smo.getTargetMinor();
				dedSetHistory.add(ds);
			}
		
		}

		
		// Step 2: set up rewriting engine - done only once
		PrimRewriteEngine pre = new PrimRewriteEngine(schemaHistory, dedSetHistory);
		
		
		
		// Step 3: rewriting procedure call - done repeatedly

		
		/*	
		 //HYUN THIS IS ONE OF THE BIG CHALLENGING QUERIES
		 
		SQL inputSQL = new SQL("SELECT S0=p.page_namespace,"+
			"S1=p.page_title," +
			"S2=p.page_latest,"+
			"S3=r.rev_id,"+
			"S4=r.rev_page,"+
			"S5=r.rev_text_id,"+
			"S6=r.rev_comment,"+
			"S7=r.rev_user_text,"+
			"S8=r.rev_user,"+
			"S9=r.rev_minor_edit,"+
			"S10=r.rev_timestamp,"+
			"S11=r.rev_deleted  "+
		"FROM page p, revision r "+  
		"WHERE (p.page_latest=r.rev_id) AND "+
			  " r.rev_page = \"1\" AND "+
			   "(p.page_id=r.rev_page) ");
		*/
		
		
		// this is the simple version without join of the above
		// is should go through several complex SMOS to succeed in the rewriting

		SQL inputSQL = getSQL();
		System.out.println(inputSQL);
		
		System.out.println("\nINTO query:");
		Stopwatch timer = new Stopwatch();
		timer.start();
		// modify the source and target to focus on the specific version
		// to see SMOS: 
		SQLUnion sqlUnion= null;
		try {
			sqlUnion = pre.rewrite(inputSQL, 46, 3, 46, 4);
		} catch (ErrorRewritingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		timer.stop();
		 
		System.out.print(sqlUnion);
		System.out.println("\nRewriting elapsed time: " + timer.getElapsedTime() + " ms");

	}

	
	private static SQL getSQL() {

		// works on version 42-48
		SQL inputSQLv42 = new SQL("SELECT S0=p.page_id FROM page p");
		
		// works on version 41
		SQL inputSQLv41 = new SQL("SELECT S0=o.old_id FROM old o");
		
		SQL inputSQLQ1 = new SQL("SELECT S0=P.page_id FROM page P " +
            "WHERE P.page_namespace = \"0\" AND P.page_title = \"Main_Page\"");	
		
		SQL inputSQLQ2 = 
			new SQL("SELECT S0=P.page_namespace, S1=P.page_title, S2=P.page_latest, "+
       "S3=R.rev_id, S4=R.rev_page, "+
       //"S5=R.rev_text_id, "+
       "S6=R.rev_comment, " +
       "S7=R.rev_user_text, S8=R.rev_user, "+
       "S9=R.rev_minor_edit, S10=R.rev_timestamp " + 
       //"S11=R.rev_deleted " +  
       "FROM page P, revision R "+ 
       "WHERE R.rev_id=P.page_latest AND R.rev_page = \"1\" AND P.page_id=R.rev_page ");

		
		SQL inputSQLQ3 = new SQL(
			"SELECT S0=R.rev_timestamp "+ 
			"FROM revision R "+
			"WHERE R.rev_page = \"1\"");

		SQL inputSQLQ4 = new SQL(
		"SELECT type=pa.DoubleRedirects, "+ 
		"namespace=pa.page_namespace, "+
		"title=pa.page_title, "+
		"nsb=pb.page_namespace, "+
		"tb=pb.page_title, "+
		"nsc=pc.page_namespace, "+
		"nsc=pc.page_title "+
		"FROM pagelinks la, pagelinks lb, page pa, page pb, page pc "+ 
		"WHERE pa.page_is_redirect=\"1\" AND pb.page_is_redirect=\"1\" AND "+ 
		"      la.pl_from=pa.page_id AND la.pl_namespace=pb.page_namespace AND "+ 
		"      la.pl_title=pb.page_title AND lb.pl_from=pb.page_id AND "+
		"      lb.pl_namespace=pc.page_namespace AND lb.pl_title=pc.page_title");

		SQL inputSQLQ5 = new SQL(
				"SELECT S0=t.user_email, S1=t.user_id, S2=t.user_name, S3=t.user_newpassword, S4=t.user_options, S5=t.user_password, S6=t.user_real_name, S7=t.user_token, S8=t.user_touched FROM user t");
		
		// v32
		SQL q1 = new SQL("SELECT S0=u.user_token FROM user u;");
		
		// v41
		SQL q2 = new SQL("SELECT S0=w.wl_notificationtimestamp FROM watchlist w;");
		
		// v47
		SQL q3 = new SQL("SELECT S0=p.page_len FROM page p;");
		
		// choice
		return q3;
	}
}
