package edu.ucla.cs.db.prism.backend.restructure;

import edu.ucla.cs.db.prism.backend.parser.syntaxtree.Node;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.NodeChoice;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.NodeOptional;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.NodeSequence;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.NodeToken;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.arith_element;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.arith_expAnd;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.arith_expOr;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.arith_unary;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.expression;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.final_value;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.non_final_value;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.relational_exp;
import edu.ucla.cs.db.prism.backend.smo.util.Conjunctive;
import edu.ucla.cs.db.prism.backend.smo.util.Disjunctive;
import edu.ucla.cs.db.prism.backend.smo.util.Predicate;
import edu.ucla.cs.db.prism.backend.smo.util.Term;

public class ExpressionRestructure {
	
	// this function takes a parser-tree expression node and
	// produces a logical expression object, Disjunctive
	public static Disjunctive restructure(expression n) {
		return restructure(n.f0);
	}

	public static Disjunctive restructure(arith_expOr n) {
		Disjunctive disjunctive = new Disjunctive();
		disjunctive.add(restructure(n.f0));
		if(n.f1.present())
			disjunctive.addAll(
					restructure( (arith_expOr)(((NodeSequence)n.f1.node).nodes.get(1))) );
		return disjunctive;
	}

	public static Conjunctive restructure(arith_expAnd n) {
		Conjunctive conjunctive = new Conjunctive();
		if(n.f0.which == 0) {
			conjunctive.add(restructure( (relational_exp)((NodeSequence)n.f0.choice).nodes.get(0) ));
			NodeOptional n2 = (NodeOptional)((NodeSequence)n.f0.choice).nodes.get(1);
			if( n2.present() )
				conjunctive.addAll(restructure( (arith_expAnd)((NodeSequence)n2.node).nodes.get(1) ));
		}
		else {// which == 1
			conjunctive.disjunctives.add(restructure( (arith_expOr)((NodeSequence)n.f0.choice).nodes.get(1) ));
			NodeOptional n2 = (NodeOptional)((NodeSequence)n.f0.choice).nodes.get(3);
			if( n2.present() )
				conjunctive.addAll(restructure( (arith_expAnd)((NodeSequence)n2.node).nodes.get(1) ));
		}
		return conjunctive;
	}

	public static Predicate restructure(relational_exp n) {
		Predicate predicate = new Predicate();
		NodeChoice n2 = (NodeChoice)n.f1.node;
		if(n2 == null)
			return null;
			int whichChoice = n2.which;
		if(whichChoice == 0) {// EQUAL
			predicate.op = "=";
		}
		else if (whichChoice == 1) { // NOTEQ
			predicate.op = "!=";
		}
		else if (whichChoice == 2) { // LESS
			predicate.op = "<";
		}
		else if (whichChoice == 3) { // GREAT
			predicate.op = ">";
		}
		else if (whichChoice == 4) { // NOMORE
			predicate.op = "<=";
		}
		else if (whichChoice == 5) { // NOLESS
			predicate.op = ">=";
		}
		
		predicate.term1 = restructure(n.f0);
		predicate.term2 = restructure((arith_unary)((NodeSequence)n2.choice).nodes.get(1));
		
		return predicate;
	}
	
	// disallow +, -, *, /, %, negative
	public static Term restructure(arith_unary n) {
		NodeChoice n2 = ((arith_element)n.f0.choice).f0;
		if(n2.which == 0)
			return restructure((final_value)n2.choice);
		else // which == 1
			return restructure((non_final_value)n2.choice);
			
	}
	
	public static Term restructure(final_value n) {
		return new Term(((NodeToken)n.f0.choice).tokenImage);
	}
	
	
	public static Term restructure(non_final_value n) {
		
		if(n.f0.which == 0) {
	
			NodeSequence n2 = (NodeSequence)n.f0.choice;
			
			String tablename = ((NodeToken)n2.nodes.get(0)).tokenImage;
			Node no = ((NodeOptional)n2.nodes.get(1)).node;
			if(no != null){
				String columnname = ((NodeToken)((NodeSequence)no).nodes.get(1)).tokenImage;
				return new Term(tablename,columnname);
			}else{
				Term t = new Term();
				t.columnName = tablename; // this is the case in which the term is not table-specified	
				return t;
				
			}
			
			
		}
		else// modified on 6/14/2007 for xquery processing
			//return new Term(((NodeToken)n.f0.choice).tokenImage.toUpperCase());
			return new Term(((NodeToken)n.f0.choice).tokenImage);
	}
	
}
