package edu.ucla.cs.db.prism.backend.restructure;

import java.util.Vector;

import edu.ucla.cs.db.prism.backend.parser.syntaxtree.*;
//import edu.ucla.cs.db.prism.backend.smo.SMOImpl;
import edu.ucla.cs.db.prism.backend.icmo.*;
//import edu.ucla.cs.db.prism.backend.smo.util.SMOSequenceInterface;
//import edu.ucla.cs.db.prism.backend.smo.util.SMOFactory;
//import edu.ucla.cs.db.prism.backend.smo.util.SMOSequence;
//import edu.ucla.cs.db.prism.backend.smo.util.SMOTypeUnknownException;


public class IcmoRestructure {
	
	public static ICMOImpl restructure(Icmo IcmoNode)
	{
		//Hopefully no need to manage NOP
		NodeSequence icmo = (NodeSequence)IcmoNode.f0.choice;
		String icmoType1 = "";
		String icmoType2 = "";
		String icmoType0 = "";
		
		// Check for incomplete ICMO,  this is a first cut to fast-reject many incomplete sequences
		if(icmo.nodes.size() < 6){
			System.out.println("Not enough nodes to be an ICMO, early reject: " + icmo.nodes.toString());
			return null;
		}
		
		//Works in ALTER TABLE ABC ADD ***** KEY ....
		icmoType0 = getNodeValue(icmo, 3,true); //ADD
		if(icmoType0.equals("DROP"))
		{
			//To get the tokens PRIMARY/FOREIGN/VALUE and KEY/CONSTRAINT from the Optional node
			//Available at Base tokenno = 1 
			NodeOptional nn0 = ((NodeOptional)icmo.nodes.get(4));
			
			//ADDING several early checks to speed up parsing and avoid NullPointerExceptions
			if(((NodeChoice)nn0.node) == null)
				return null;
			
			NodeSequence nn1 = (NodeSequence)((NodeChoice)nn0.node).choice;
			icmoType1 = ((NodeToken)nn1.nodes.get(0)).tokenImage.toUpperCase().toString(); 
			icmoType2 = ((NodeToken)nn1.nodes.get(1)).tokenImage.toUpperCase().toString();
		}
		else
		{
			icmoType1 = getNodeValue(icmo, 4,true); //To get type of IC Primary/Foreign/Value
			icmoType2 = getNodeValue(icmo, 5,true); //Key/Constraint Keyword
		}
		
			
		ICMOImpl newIcmo = null;
		try
		{
			newIcmo = ICMOFactory.createICMO(icmoType0 + " " + icmoType1 + " " + icmoType2);			
			
		}
		catch(ICMOTypeUnknownException e)
		{
			e.printStackTrace();
		}
		
		if(newIcmo == null){
			return null;
		}	
		
		//Add Primary Key Code
		if(icmoType0.equals("ADD") && icmoType1.equals("PRIMARY"))
		{
			//Set Table name affected
			newIcmo.setTableName(getNodeValue(icmo, 2,false));
			//Set Primary Key ID such as P1 or PK1 in ALTER TABLE ABC ADD PRIMARY KEY PK1
			//newIcmo.id = getNodeValue(icmo, 6, false);
			
			//check for nulls (maybe redundant)
			if(newIcmo!=null && icmo.nodes!=null && 
				icmo.nodes.get(6) !=null && 
				((NodeOptional)icmo.nodes.get(6)).node!= null && 
				((NodeChoice)((NodeOptional)icmo.nodes.get(6)).node).choice!=null && 
				((NodeToken)((NodeChoice)((NodeOptional)icmo.nodes.get(6)).node).choice).tokenImage!= null){
			
				newIcmo.setId(((NodeToken)((NodeChoice)((NodeOptional)icmo.nodes.get(6)).node).choice).tokenImage.toString());
			}
			else{
				return null;
			}
			//set TYpe of enforcement of IC
			NodeOptional nn0 = ((NodeOptional)icmo.nodes.get(8));
			
			//CARLO: added check of empty choice, assuming conservative CHECK if missing token
			NodeToken nn1=null;
			if(((NodeChoice)nn0.node) != null)
				nn1 = (NodeToken)((NodeChoice)nn0.node).choice;
			
			
			if(nn1 != null)
			{
				if(nn1.tokenImage.toUpperCase().toString().equalsIgnoreCase(ICMOInterface.CHECK))
				{
					((AddPrimaryKey)newIcmo).check = true;
				}
				else if(nn1.tokenImage.toUpperCase().toString().equalsIgnoreCase(ICMOInterface.ENFORCE))
				{
					((AddPrimaryKey)newIcmo).enforce = true;
				}
				else if(nn1.tokenImage.toUpperCase().toString().equalsIgnoreCase(ICMOInterface.DECLARE))
				{
					((AddPrimaryKey)newIcmo).declare= true;
				}
			}
			else
			{
				((AddPrimaryKey)newIcmo).check = true;
			}
			NodeOptional n3 = (NodeOptional)icmo.nodes.get(7);
			
			if(n3.present()) 
			{
				NodeOptional n4 = (NodeOptional)((NodeSequence)n3.node).nodes.get(1);
				column_list n5 = (column_list)n4.node;

				Vector<String> columnNameList = new Vector<String>();
				//Vector<String> columnTypeList = new Vector<String>();
				//Set Columns affected
				((AddPrimaryKey)newIcmo).columns = columnNameList;

				
				while(n5 != null) 
				{
					columnNameList.add(n5.f0.tokenImage);
					if(!n5.f2.present()) 
					{
						n5 = null;
					}
					else {
						NodeSequence ns = (NodeSequence)n5.f2.node;
						n5 = (column_list)ns.nodes.get(1);
					}
				}
			}
		}
		else if(icmoType0.equals("DROP") && icmoType1.equals("PRIMARY"))
		{
			//DROP PRIMARY KEY PK1 FROM ABC
			//Adding PK1 to id attribute
			//((DropPrimaryKey)newIcmo).id = getNodeValue(icmo, 5,false);
			newIcmo.setId(((NodeToken)((NodeChoice)((NodeOptional)icmo.nodes.get(5)).node).choice).tokenImage.toString());
			//Adding Table name ABC 
			((DropPrimaryKey)newIcmo).setTableName(getNodeValue(icmo, 2,false));

		}
		else if(icmoType0.equals("DROP") && icmoType1.equals("FOREIGN"))
		{
			//DROP FOREIGN KEY FK1 FROM ABC
			//Adding FK1 to id attribute
			//((DropForeignKey)newIcmo).id = getNodeValue(icmo, 5,false);
			newIcmo.setId(((NodeToken)((NodeChoice)((NodeOptional)icmo.nodes.get(5)).node).choice).tokenImage.toString());
			//Adding Table name ABC 
			((DropForeignKey)newIcmo).setTableName(getNodeValue(icmo, 2,false));
		}
		else if(icmoType0.equals("DROP") && icmoType1.equals("VALUE"))
		{
			//DROP VALUE CONSTRAINT VC1 FROM ABC
			//Adding VC1 to id attribute
			//((DropValueConstraint)newIcmo).id = getNodeValue(icmo, 5,false);
			newIcmo.setId(((NodeToken)((NodeChoice)((NodeOptional)icmo.nodes.get(5)).node).choice).tokenImage.toString());
			//Adding Table name ABC 
			((DropValueConstraint)newIcmo).setTableName(getNodeValue(icmo, 2,false));
		}
		else if(icmoType0.equals("ADD") && icmoType1.equals("FOREIGN"))
		{
			//ALTER TABLE ABC ADD FOREIGN KEY FK1 (A,B) REFERENCES DEF (D,E) CHECK
			// 0      1    2   3     4     5   6    7        8      9    10    11
			//Set Table name affected
			newIcmo.setTableName(getNodeValue(icmo, 2,false));
			//Set Referencing Table name
			newIcmo.setTableName2(getNodeValue(icmo, 9,false));
			//Set Foreign Key ID such as FK1 in ALTER TABLE ABC ADD FOREIGN KEY FK1
			//newIcmo.id = getNodeValue(icmo, 6,false);
			newIcmo.setId(((NodeToken)((NodeChoice)((NodeOptional)icmo.nodes.get(6)).node).choice).tokenImage.toString());
			//set TYpe of enforcement of IC
			if((((NodeOptional)icmo.nodes.get(11)).node) != null)
			{
				NodeOptional nn0 = ((NodeOptional)icmo.nodes.get(11));
				NodeToken nn1 = (NodeToken)((NodeChoice)nn0.node).choice;
				if(nn1.tokenImage.toUpperCase().toString().equalsIgnoreCase(ICMOInterface.CHECK))
				{
					((AddForeignKey)newIcmo).check = true;
				}
				else if(nn1.tokenImage.toUpperCase().toString().equalsIgnoreCase(ICMOInterface.ENFORCE))
				{
					((AddForeignKey)newIcmo).enforce = true;
				}
				else if(nn1.tokenImage.toUpperCase().toString().equalsIgnoreCase(ICMOInterface.DECLARE))
				{
					((AddForeignKey)newIcmo).declare= true;
				}
			}
			else
			{
				((AddForeignKey)newIcmo).check = true;
			}
			
			//ColumnList Source
			NodeOptional n3 = (NodeOptional)icmo.nodes.get(7);
			if(n3.present()) 
			{
				NodeOptional n4 = (NodeOptional)((NodeSequence)n3.node).nodes.get(1);
				column_list n5 = (column_list)n4.node;

				Vector<String> columnNameList = new Vector<String>();
				//Vector<String> columnTypeList = new Vector<String>();
				//Set Columns affected
				((AddForeignKey)newIcmo).sourceColumns = columnNameList;

				
				while(n5 != null) 
				{
					columnNameList.add(n5.f0.tokenImage);
					if(!n5.f2.present()) 
					{
						n5 = null;
					}
					else {
						NodeSequence ns = (NodeSequence)n5.f2.node;
						n5 = (column_list)ns.nodes.get(1);
					}
				}
			}
			//ColumnList Target
			n3 = (NodeOptional)icmo.nodes.get(10);
			if(n3.present()) 
			{
				NodeOptional n4 = (NodeOptional)((NodeSequence)n3.node).nodes.get(1);
				column_list n5 = (column_list)n4.node;

				Vector<String> columnNameList = new Vector<String>();
				//Vector<String> columnTypeList = new Vector<String>();
				//Set Columns affected
				((AddForeignKey)newIcmo).targetColumns = columnNameList;

				
				while(n5 != null) 
				{
					columnNameList.add(n5.f0.tokenImage);
					if(!n5.f2.present()) 
					{
						n5 = null;
					}
					else {
						NodeSequence ns = (NodeSequence)n5.f2.node;
						n5 = (column_list)ns.nodes.get(1);
					}
				}
			}

		}
		else if(icmoType0.equals("ADD") && icmoType1.equals("VALUE"))
		{
			//ALTER TABLE ABC ADD VALUE CONSTRAINT VC1 (X>30 AND Y=40) DECLARE
			//  0     1    2   3    4        5      6          7          8
			//Set Table name affected
			newIcmo.setTableName(getNodeValue(icmo, 2,false));
			//Set Primary Key ID such as P1 or PK1 in ALTER TABLE ABC ADD PRIMARY KEY PK1
			//newIcmo.id = getNodeValue(icmo, 6,false);
			newIcmo.setId(((NodeToken)((NodeChoice)((NodeOptional)icmo.nodes.get(6)).node).choice).tokenImage.toString());
			//set TYpe of enforcement of IC
			NodeOptional nn0 = ((NodeOptional)icmo.nodes.get(8));
			if((NodeChoice)(nn0.node) != null)
			{
				NodeToken nn1 = (NodeToken)((NodeChoice)nn0.node).choice;
				if(nn1.tokenImage.toUpperCase().toString().equalsIgnoreCase(ICMOInterface.CHECK))
				{
					((AddValueConstraint)newIcmo).check = true;
				}
				else if(nn1.tokenImage.toUpperCase().toString().equalsIgnoreCase(ICMOInterface.ENFORCE))
				{
					((AddValueConstraint)newIcmo).enforce = true;
				}
				else if(nn1.tokenImage.toUpperCase().toString().equalsIgnoreCase(ICMOInterface.DECLARE))
				{
					((AddValueConstraint)newIcmo).declare= true;
				}
			}
			else
			{
				((AddValueConstraint)newIcmo).check = true;
			}
			//Set Condition string from expression
			//the ExpressionRestructure.restructure returns Disjunctive type - however the condition
			//attribute of AddValueCOnstraint class is type string - so converting
			//########!doubt shud we change the type of condition to disjunctive????#######
			//newSmo.setTargetConditions2(ExpressionRestructure.restructure((expression)smo.nodes.get(6)));
			((AddValueConstraint)newIcmo).condition = ExpressionRestructure.restructure((expression)icmo.nodes.get(7)).toString();
		}

		
		
		return newIcmo;
		
	
	}

	static String restructure(column_type n) 
	{
		String type = "";
		
		for(int i=0; i<n.f0.nodes.size(); i++) {
			NodeToken nt = (NodeToken)n.f0.nodes.get(i);
			type += nt.tokenImage;
			if(i<n.f0.nodes.size()-1)
				type += " ";
		}
		
		if(n.f1.present()) {
			type += "(";
			NodeSequence ns = (NodeSequence)n.f1.node;
			NodeToken integerNodeToken = (NodeToken)ns.nodes.get(1);
			type += integerNodeToken.tokenImage;
			type += ")";
		}
		
		for(int i=0; n.f2.present() && i<n.f2.nodes.size(); i++) {
			if(i==0) type += " ";
			NodeToken nt = (NodeToken)n.f2.nodes.get(i);
			type += nt.tokenImage;
			if(i<n.f0.nodes.size()-1)
				type += " ";
		}
		
		
		return type;
	}

	
	public static String getNodeValue(NodeSequence icmo, int tokenno, boolean capitalize)
	{
		String sRetVal = "";
		try
		{
			if(capitalize)
				sRetVal = ((NodeToken)icmo.nodes.get(tokenno)).tokenImage.toUpperCase().toString();
			else
				sRetVal = ((NodeToken)icmo.nodes.get(tokenno)).tokenImage.toString();
					
			
		}
		catch(Exception e)
		{
			e.printStackTrace();
		}
		return sRetVal;
	}

}
