package edu.ucla.cs.db.prism.backend.restructure;
import edu.ucla.cs.db.prism.backend.SQL.FromClause;
import edu.ucla.cs.db.prism.backend.SQL.RelationDeclaration;
import edu.ucla.cs.db.prism.backend.SQL.SQL;
import edu.ucla.cs.db.prism.backend.SQL.SQLUnion;
import edu.ucla.cs.db.prism.backend.SQL.SelectAttribute;
import edu.ucla.cs.db.prism.backend.SQL.SelectClause;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.*;

public class SQLRestructure {
	
	// This function accepts a parser-tree sql_union and
	// constructs a logical SQLUnionSet object

	private static SelectClause restructure(field_values n) {
		return restructure(n, 0);
	}
	private static SelectClause restructure(field_values n, int outputAttributeIndex) {
		
		SelectClause selectClause = new SelectClause();
		SelectAttribute sa = new SelectAttribute();
		
		field_value n2 = n.f0;
		if(n2.f0.present()) {
			NodeSequence ns = (NodeSequence)n2.f0.node;
			NodeToken nt = (NodeToken)ns.nodes.get(0);
			sa.outputAttributeName = nt.tokenImage;
		}
		else {
			sa.outputAttributeName = "S" + outputAttributeIndex++;
		}
		// S0 = , S1 = R.A, ... (the case of S0 here)
		if(!n2.f1.present()) {
			sa.constant = "\"\""; // empty string
		}
		else {
			NodeChoice nc = (NodeChoice)n2.f1.node;
			if(nc.which == 0) {// id=expression
				expression exp = (expression)nc.choice;
				if(isConstantExpression(exp)) {
					sa.constant = constantRestructure(exp);
				}
				else {
					NodeSequence n3 = simpleAttributeNameExpressionRestructure(exp);
					sa.relationAlias = ((NodeToken)n3.nodes.get(0)).tokenImage;
					NodeSequence n4 = (NodeSequence)((NodeOptional)n3.nodes.get(1)).node;
					sa.attribute = ((NodeToken)n4.nodes.get(1)).tokenImage;
				}
			}
			else { // id=aggregate(expression)
				NodeSequence n3 = (NodeSequence)nc.choice;
				atomic_aggregate n4 = (atomic_aggregate)n3.nodes.get(0);
				sa.aggregate = ((NodeToken)n4.f0.choice).tokenImage;
				
				NodeSequence n5 =
					simpleAttributeNameExpressionRestructure((expression)n3.nodes.get(2));
				sa.relationAlias = ((NodeToken)n5.nodes.get(0)).tokenImage;
				NodeSequence n6 = (NodeSequence)((NodeOptional)n5.nodes.get(1)).node;
				sa.attribute = ((NodeToken)n6.nodes.get(1)).tokenImage;
			}
		}
		
		selectClause.addSelectAttribute(sa);
		
		if(n.f1.present()) {
			NodeSequence n3 = (NodeSequence)n.f1.node;
			field_values n4 = (field_values)n3.nodes.get(1);
			SelectClause selectClausef1 = restructure(n4, outputAttributeIndex);
			selectClause.selectAttributes.addAll(selectClausef1.selectAttributes);
		}
		
		return selectClause;
	}

	private static FromClause restructure(bindings n) {
		// we assume n.f0 is a simple expression, namely relation name
		FromClause fromClause = new FromClause();
		
		String relationName = simpleRelationNameExpressionRestructure(n.f0);
		String id = n.f1.tokenImage;

		fromClause.addRelationDeclaration(new RelationDeclaration(relationName, id));
		
		if(n.f2.present()) {
			NodeSequence n2 = (NodeSequence)n.f2.node;
			bindings n3 = (bindings)n2.nodes.get(1);
			FromClause fromClausef2 = restructure(n3);
			fromClause.relationDeclarations.addAll(fromClausef2.relationDeclarations);
		}
		
		return fromClause;
	}
	
	private static String simpleRelationNameExpressionRestructure(expression n) {
		relational_exp n2 = (relational_exp)((NodeSequence)n.f0.f0.f0.choice).nodes.get(0);
		arith_element n3 = ((arith_element)n2.f0.f0.choice);
		non_final_value n4 = (non_final_value)n3.f0.choice;
		NodeSequence n5 = (NodeSequence)n4.f0.choice;
		NodeToken n6 = (NodeToken)n5.nodes.get(0);
		return n6.tokenImage;
	}
	
	// returns a node sequence with <ID> [ <DOT> <ID> ]
	private static NodeSequence simpleAttributeNameExpressionRestructure(expression n) {
		relational_exp n2 = (relational_exp)((NodeSequence)n.f0.f0.f0.choice).nodes.get(0);
		arith_element n3 = ((arith_element)n2.f0.f0.choice);
		non_final_value n4 = (non_final_value)n3.f0.choice;
		NodeSequence n5 = (NodeSequence)n4.f0.choice;
		
		return n5;
	}

	/**
	 * note that mars uses a bad form of constant output - no double quote surrounding
	 * we look for id-like constant
	 * @param n
	 * @return
	 */
	private static String constantRestructure(expression n) {
		relational_exp n2 = (relational_exp)((NodeSequence)n.f0.f0.f0.choice).nodes.get(0);
		arith_element n3 = ((arith_element)n2.f0.f0.choice);
		//void arith_element() :
		//{}
		//{
		//  final_value() | non_final_value()
		//}
		boolean isFinalValue = false;
		if(n3.f0.which == 0)
			isFinalValue = true;
		
		
		//void final_value() :
		//{}
		//{
		//  <INTEGER_VALUE> | <STRING_VALUE> 
		//}
		if(isFinalValue) {
			final_value nf = (final_value)n3.f0.choice;
			NodeToken nt = (NodeToken)nf.f0.choice;
			return "\"" + nt.tokenImage + "\"";
		}
		
		//void non_final_value() :
		//{}
		//{
		//  <ID> [ LOOKAHEAD(<DOT>) <DOT> <ID> ] | <NULL> | <MID>
		//}
		
		non_final_value n4 = (non_final_value)n3.f0.choice;
		if(n4.f0.which > 0) {// NULL or MID
			NodeToken nt = (NodeToken)n4.f0.choice;
			return "\"" + nt.tokenImage + "\"";
		}
		
		NodeSequence n5 = (NodeSequence)n4.f0.choice;
		NodeOptional n6 = (NodeOptional)n5.nodes.get(1);
		// if id.id -> non-constant
		// if id -> constant
		if(n6.present()) 
			return null;
		else {
			NodeToken nt = (NodeToken)n5.nodes.get(0);
			return "\"" + nt.tokenImage + "\"";
		}
		
	}
	
	private static boolean isConstantExpression(expression n) {
		relational_exp n2 = (relational_exp)((NodeSequence)n.f0.f0.f0.choice).nodes.get(0);
		arith_element n3 = ((arith_element)n2.f0.f0.choice);
		//void arith_element() :
		//{}
		//{
		//  final_value() | non_final_value()
		//}
		boolean isFinalValue = false;
		if(n3.f0.which == 0)
			isFinalValue = true;
		
		
		//void final_value() :
		//{}
		//{
		//  <INTEGER_VALUE> | <STRING_VALUE> 
		//}
		if(isFinalValue) {
			return true;
		}
		
		//void non_final_value() :
		//{}
		//{
		//  <ID> [ LOOKAHEAD(<DOT>) <DOT> <ID> ] | <NULL> | <MID>
		//}
		
		non_final_value n4 = (non_final_value)n3.f0.choice;
		if(n4.f0.which > 0) // NULL or MID
			return true;
		
		NodeSequence n5 = (NodeSequence)n4.f0.choice;
		NodeOptional n6 = (NodeOptional)n5.nodes.get(1);
		// if id.id -> non-constant
		// if id -> constant
		if(n6.present()) 
			return false;
		else
			return true;
	}
	public static SQL restructure(sql n) {
		SQL newSQL = new SQL();
		
		if(n == null)
			return newSQL;
		
		newSQL.select = restructure(n.f1);
		newSQL.from = restructure(n.f3);
		if(n.f4.present())
			newSQL.where = ExpressionRestructure.restructure(
					(expression)((NodeSequence)n.f4.node).nodes.get(1));
		return newSQL;
	}

	private static SQLUnion restructure(sql_union n) {
		SQLUnion newSQLUnionSet = new SQLUnion();
		
		if(n.f0.which == 0) { // sql_union
			SQLUnion sqlUnionSetf0 = 
				restructure((sql_union)((NodeSequence)n.f0.choice).nodes.get(1));
			newSQLUnionSet.sqls.addAll(sqlUnionSetf0.sqls);
		}
		else  { // which == 1, sql
			SQL sqlf0 = restructure((sql)n.f0.choice);
			newSQLUnionSet.sqls.add(sqlf0);
		}
		
		if(n.f1.present()) {
			NodeSequence n2 = (NodeSequence)n.f1.node;
			SQLUnion sqlUnionSetf1 = restructure((sql_union)n2.nodes.get(1));
			newSQLUnionSet.sqls.addAll(sqlUnionSetf1.sqls);
		}
		
		return newSQLUnionSet;
	}
	
	public static SQLUnion restructure(SQLQueryDeclaration n) {
		SQLUnion sqlUnion = restructure(n.f3);
		sqlUnion.name = n.f1.tokenImage;
		//System.out.println("Name: " + sqlUnion.name + "!");
		return sqlUnion;
	}

}
