package edu.ucla.cs.db.prism.backend.restructure;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.parser.syntaxtree.*;
import edu.ucla.cs.db.prism.backend.smo.SMOImpl;
import edu.ucla.cs.db.prism.backend.smo.util.SMOFactory;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequence;
import edu.ucla.cs.db.prism.backend.smo.util.SMOTypeUnknownException;

public class SmoRestructure {
	

	// This function accepts a parser-tree SMO sequence history
	// and then turns it into a logical SMO sequence history structure
	// usage:
	/*
	   public void visit(SmoDeclaration n) {
			   smoSequenceHistory.add(restructure.SmoRestructure.restructure(n));
		   }
	   */
	public static SMOSequence restructure(SmoDeclaration n) {
		int sourceVersion = Integer.parseInt(n.f3.tokenImage);
		int targetVersion = Integer.parseInt(n.f5.tokenImage);
		SMOSequence smoSequence = restructure(n.f9, sourceVersion, targetVersion);
		smoSequence.sourceVersion = sourceVersion;
		smoSequence.targetVersion = targetVersion;
		return smoSequence;
	}

	
	// This function accepts a parser-tree SMO sequence
	// and then turns it into a logical SMO sequence structure
	public static SMOSequence restructure(SmoList n, int sourceVersion, int targetVersion) {
		SMOSequence smoSequence = new SMOSequence();

		for(int i=0; i < n.f0.nodes.size(); i++) {
			Node n2 = ((NodeSequence) (n.f0.nodes.get(i))).elementAt(0);
			Smo smoNode = (Smo) n2;
			SMOImpl newSmo = restructure(smoNode);

			newSmo.setSourceMajor(sourceVersion);
			newSmo.setSourceMinor(i);
			if(i < n.f0.nodes.size()-1) {
				newSmo.setTargetMajor(sourceVersion);
				newSmo.setTargetMinor(i+1);
			}
			else {
				newSmo.setTargetMajor(targetVersion);
				newSmo.setTargetMinor(0);				
			}

			if(newSmo != null)
				smoSequence.add(newSmo);
		}
		
		return smoSequence;
	}

	
	public static SMOImpl restructure(Smo smoNode) {

		
		// for each smo, do the translation, except nop
		if(smoNode.f0.which == 11){
			
			try {
				return SMOFactory.createSMO("NOP");
			} catch (SMOTypeUnknownException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
		} // Carlo Management of NOP
		
		NodeSequence smo = (NodeSequence)smoNode.f0.choice;
		String smoType0 = ((NodeToken)smo.nodes.get(0)).tokenImage.toUpperCase();
		String smoType1 = ((NodeToken)smo.nodes.get(1)).tokenImage.toUpperCase();

		// CARLO C factory-based implementation
		SMOImpl newSmo = null;
		try {
			newSmo = SMOFactory.createSMO(smoType0 + " " + smoType1);
		} catch (SMOTypeUnknownException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		
		if(newSmo == null){
			return null;
		}	
		// end of carlo modification
		
		
		/* PREVIOUS HYUN IMPLEMENTATION
		SMOImpl newSmo = new SMOImpl();
		
		// added by Hyun 10-17-2007
		if(smoType0.equals("NOP")) {
			return null;
		}
		else
			newSmo.setOperatorType(smoType0 + " " + smoType1);
		
		*/
		
		
		if(smoType0.equals("CREATE") && smoType1.equals("TABLE")) {
			newSmo.setTableName(((NodeToken)smo.nodes.get(2)).tokenImage);
			NodeOptional n3 = (NodeOptional)smo.nodes.get(3);
			if(n3.present()) {
				NodeOptional n4 = (NodeOptional)((NodeSequence)n3.node).nodes.get(1);
				column_list n5 = (column_list)n4.node;

				Vector<String> columnNameList = new Vector<String>();
				Vector<String> columnTypeList = new Vector<String>();
				newSmo.setColumnNameList(columnNameList);
				newSmo.setColumnTypeList(columnTypeList);

				while(n5 != null) {
					columnNameList.add(n5.f0.tokenImage);
					if(n5.f1.present()) {
						columnTypeList.add(restructure((column_type)n5.f1.node));
					}
					else
						columnTypeList.add("varchar(255)"); // default data type
					
					if(!n5.f2.present()) {
						n5 = null;
					}
					else {
						NodeSequence ns = (NodeSequence)n5.f2.node;
						n5 = (column_list)ns.nodes.get(1);
					}
				}
			}
		}
		else if(smoType0.equals("DROP") && smoType1.equals("TABLE")) {
			newSmo.setTableName(((NodeToken)smo.nodes.get(2)).tokenImage);
		}
		else if(smoType0.equals("RENAME") && smoType1.equals("TABLE")) {
			newSmo.setTableName(((NodeToken)smo.nodes.get(2)).tokenImage);
			newSmo.setTableName2(((NodeToken)smo.nodes.get(4)).tokenImage);
		}
		else if(smoType0.equals("COPY") && smoType1.equals("TABLE")) {
			newSmo.setTableName(((NodeToken)smo.nodes.get(2)).tokenImage);
			newSmo.setTableName2(((NodeToken)smo.nodes.get(4)).tokenImage);
		}
		else if(smoType0.equals("PARTITION") && smoType1.equals("TABLE")) {
			newSmo.setTableName(((NodeToken)smo.nodes.get(2)).tokenImage);
			newSmo.setTableName2(((NodeToken)smo.nodes.get(4)).tokenImage);
			newSmo.setTableName3(((NodeToken)smo.nodes.get(8)).tokenImage);
			newSmo.setTargetConditions2(ExpressionRestructure.restructure((expression)smo.nodes.get(6)));
			newSmo.setTargetConditions3(ExpressionRestructure.restructure((expression)smo.nodes.get(10)));

		}
		else if(smoType0.equals("MERGE") && smoType1.equals("TABLE")) {
			// changed to allow no-with-clause
			newSmo.setTableName(((NodeToken)smo.nodes.get(2)).tokenImage);
			newSmo.setTableName2(((NodeToken)smo.nodes.get(5)).tokenImage);
			newSmo.setTableName3(((NodeToken)smo.nodes.get(8)).tokenImage);
			
			NodeOptional n3 = (NodeOptional)smo.nodes.get(3);
			if(n3.present()) {
				NodeSequence n4 = (NodeSequence)n3.node;
				newSmo.setTargetConditions2(ExpressionRestructure.restructure((expression)n4.nodes.get(1)));
			}
			else
				newSmo.setTargetConditions2(null);

			n3 = (NodeOptional)smo.nodes.get(6);
			if(n3.present()) {
				NodeSequence n4 = (NodeSequence)n3.node;
				newSmo.setTargetConditions2(ExpressionRestructure.restructure((expression)n4.nodes.get(1)));
			}
			else
				newSmo.setTargetConditions2(null);
		}
		else if(smoType0.equals("ADD") && smoType1.equals("COLUMN")) {
			newSmo.setColumnName(((NodeToken)smo.nodes.get(2)).tokenImage);
			newSmo.setTableName(((NodeToken)smo.nodes.get(6)).tokenImage);
			
			NodeOptional nodeOptionalType = (NodeOptional)smo.nodes.get(3);
			if(nodeOptionalType.present()) {
				column_type nodeType = (column_type)nodeOptionalType.node;
				newSmo.setColumnType(restructure(nodeType));
			}
			else
				newSmo.setColumnType("varchar(255)");
			
			// default constant
			NodeOptional n3 = (NodeOptional)smo.nodes.get(4);
			if(n3.present()) {
				NodeChoice n4 = (NodeChoice)((NodeSequence)n3.node).nodes.get(1);
				if(n4.which == 0) {// final_value()
					newSmo.setDefaultConstant(((NodeToken)((final_value)n4.choice).f0.choice).tokenImage);
				}
				else if(n4.which == 1) { // null
					newSmo.setDefaultConstant(null);
				}
				else if(n4.which == 2) { // function call as new column value (9/19/2007 by Hyun)
					function_call n5 = (function_call)n4.choice;
					if(n5.f0.which == 0) // general function names
						newSmo.setFunctionName(((NodeToken)n5.f0.choice).tokenImage);
					else if(n5.f0.which == 1) // function name equal to one of aggregate (e.g. max, min, concat)
						newSmo.setFunctionName(((NodeToken)((atomic_aggregate)n5.f0.choice).f0.choice).tokenImage);
					newSmo.setFunctionArgs(new Vector<String>());
			
					NodeOptional n6 = n5.f2;
					if(n6.present()) {// one or more arguments function
						id_list n7 = (id_list)n6.node;
						newSmo.getFunctionArgs().add(n7.f0.tokenImage); // arg0
						if(n7.f1.present()) {
							for(int j=0; j<n7.f1.nodes.size(); j++) {
								NodeSequence n8 = (NodeSequence)n7.f1.nodes.get(j);
								NodeToken n9 = (NodeToken)n8.nodes.get(1); //arg_i
								newSmo.getFunctionArgs().add(n9.tokenImage);
							}
						}
					}
				}
			}
			
		}
		else if(smoType0.equals("DROP") && smoType1.equals("COLUMN")) {
			newSmo.setColumnName(((NodeToken)smo.nodes.get(2)).tokenImage);
			newSmo.setTableName(((NodeToken)smo.nodes.get(4)).tokenImage);
		}
		else if(smoType0.equals("RENAME") && smoType1.equals("COLUMN")) {
			newSmo.setColumnName(((NodeToken)smo.nodes.get(2)).tokenImage);
			newSmo.setTableName(((NodeToken)smo.nodes.get(4)).tokenImage);
			newSmo.setColumnName2(((NodeToken)smo.nodes.get(6)).tokenImage);
		}
		// COPY COLUMN, MOVE COLUMN deprecated - Hyun 1/28/2008
		/*
		else if(smoType0.equals("COPY") && smoType1.equals("COLUMN")) {
			newSmo.setColumnName(((NodeToken)smo.nodes.get(2)).tokenImage);
			newSmo.setTableName(((NodeToken)smo.nodes.get(4)).tokenImage);
			newSmo.setTableName2(((NodeToken)smo.nodes.get(6)).tokenImage);
			NodeOptional whereClause = (NodeOptional)smo.nodes.get(7);
			if(whereClause.present())
				newSmo.setTargetConditions(ExpressionRestructure.restructure((expression)((NodeSequence)whereClause.node).nodes.get(1)));
				
		}
		else if(smoType0.equals("MOVE") && smoType1.equals("COLUMN")) {
			newSmo.setColumnName(((NodeToken)smo.nodes.get(2)).tokenImage);
			newSmo.setTableName(((NodeToken)smo.nodes.get(4)).tokenImage);
			newSmo.setTableName2(((NodeToken)smo.nodes.get(6)).tokenImage);				
			NodeOptional whereClause = (NodeOptional)smo.nodes.get(7);
			if(whereClause.present())
				newSmo.setTargetConditions(ExpressionRestructure.restructure((expression)((NodeSequence)whereClause.node).nodes.get(1)));
		}
		 */
		else if(smoType0.equals("JOIN") && smoType1.equals("TABLE")) {
			newSmo.setTableName(((NodeToken)smo.nodes.get(2)).tokenImage);
			newSmo.setTableName2(((NodeToken)smo.nodes.get(4)).tokenImage);
			newSmo.setTableName3(((NodeToken)smo.nodes.get(6)).tokenImage);
			NodeOptional whereClause = (NodeOptional)smo.nodes.get(7);
			if(whereClause.present())
				newSmo.setTargetConditions(ExpressionRestructure.restructure((expression)((NodeSequence)whereClause.node).nodes.get(1)));
		}
		else if(smoType0.equals("DECOMPOSE") && smoType1.equals("TABLE")) {
			newSmo.setTableName(((NodeToken)smo.nodes.get(2)).tokenImage);
			newSmo.setTableName2(((NodeToken)smo.nodes.get(4)).tokenImage);				
			newSmo.setTableName3(((NodeToken)smo.nodes.get(9)).tokenImage);				
			newSmo.setAttributeList2(restructure((id_list)smo.nodes.get(6)));
			newSmo.setAttributeList3(restructure((id_list)smo.nodes.get(11)));
		}
		
		return newSmo;
	}
	
	
	static Vector<String> restructure(id_list n) {
		Vector<String> idList = new Vector<String>();
		idList.add(n.f0.tokenImage); // first columnName
		if(n.f1.present()) {
			for(int j=0; j<n.f1.nodes.size(); j++) {
				NodeSequence n6 = (NodeSequence)n.f1.nodes.get(j);
				NodeToken n7 = (NodeToken)n6.nodes.get(1); //i-th columnName
				idList.add(n7.tokenImage);
			}
		}
		return idList;
	}
	static String restructure(column_type n) {
		String type = "";
		
		for(int i=0; i<n.f0.nodes.size(); i++) {
			NodeToken nt = (NodeToken)n.f0.nodes.get(i);
			type += nt.tokenImage;
			if(i<n.f0.nodes.size()-1)
				type += " ";
		}
		
		if(n.f1.present()) {
			type += "(";
			NodeSequence ns = (NodeSequence)n.f1.node;
			NodeToken integerNodeToken = (NodeToken)ns.nodes.get(1);
			type += integerNodeToken.tokenImage;
			type += ")";
		}
		
		for(int i=0; n.f2.present() && i<n.f2.nodes.size(); i++) {
			if(i==0) type += " ";
			NodeToken nt = (NodeToken)n.f2.nodes.get(i);
			type += nt.tokenImage;
			if(i<n.f0.nodes.size()-1)
				type += " ";
		}
		
		
		return type;
	}
}
