package edu.ucla.cs.db.prism.backend.restructure;

import java.util.Vector;

import edu.ucla.cs.db.prism.backend.parser.syntaxtree.Node;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.NodeChoice;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.NodeListOptional;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.NodeOptional;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.NodeSequence;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.NodeToken;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.XQueryDeclaration;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.expression;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.one_for_clause;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.path;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.xquery;
import edu.ucla.cs.db.prism.backend.xquery.ForClause;
import edu.ucla.cs.db.prism.backend.xquery.XQuery;

public class XQueryRestructure {
	static public XQuery restructure(XQueryDeclaration n) {
		XQuery newXQuery = restructure(n.f3);
		newXQuery.name = n.f1.tokenImage;
		return newXQuery;
	}
	
	static public XQuery restructure(xquery n) {
		XQuery newXQuery = new XQuery();
		newXQuery.type = n.f0.which;
		Node choice = n.f0.choice;
		
		if(newXQuery.type == 0) {
			newXQuery.path = ((NodeToken)choice).tokenImage;
		}
		else if(newXQuery.type == 1) {
			NodeSequence n2 = (NodeSequence)choice;
			String openingElementName = ((NodeToken)n2.elementAt(1)).tokenImage;
			String closingElementName = ((NodeToken)n2.elementAt(9)).tokenImage;
			if(!openingElementName.contentEquals(closingElementName)) {
				System.err.print("element not well-formed : " + openingElementName + ", " + 
						closingElementName);
				System.exit(-1);
			}
			newXQuery.containingElementName = openingElementName;
			newXQuery.containedXQueries = new Vector<XQuery>();
			newXQuery.containedXQueries.add(
					restructure((xquery)n2.elementAt(4)));
			NodeListOptional n3 = (NodeListOptional)n2.elementAt(5);
			for(int i=0; i<n3.size(); i++) {
				NodeSequence n4 = (NodeSequence)n3.elementAt(i);
				newXQuery.containedXQueries.add(
						restructure((xquery)n4.elementAt(1)));
			}
		}
		else { // type == 2
			NodeSequence n2 = (NodeSequence)choice;
			
			// for clause
			newXQuery.forClauses = new Vector<ForClause>();
			newXQuery.forClauses.add(
					restructure((one_for_clause)n2.elementAt(1)));
			
			NodeListOptional n3 = (NodeListOptional)n2.elementAt(2);
			for(int i=0; i<n3.size(); i++) {
				NodeSequence n4 = (NodeSequence)n3.elementAt(i);
				newXQuery.forClauses.add(
						restructure((one_for_clause)n4.elementAt(1)));
			}

			// where clause
			NodeOptional n5 = (NodeOptional)n2.elementAt(3);
			if(n5.present()) {
			NodeSequence n6 = (NodeSequence)n5.node;
				newXQuery.whereClause = 
					ExpressionRestructure.restructure(
						(expression)n6.elementAt(1));
			}
			
			// return clause
			newXQuery.returnClause = restructure((xquery)n2.elementAt(5));
		}
		
		return newXQuery;
	}
	
	public static ForClause restructure(one_for_clause n) {
		ForClause forClause = new ForClause();

		forClause.var = n.f0.tokenImage;
		
		forClause.valueSeq = new XQuery();
		forClause.valueSeq.type = 0;
		forClause.valueSeq.path = restructure(n.f2);
		return forClause;
	}

	public static String restructure(path n) {
		String str = "";
		
		NodeChoice n2 = n.f0;
		if(n2.which == 0) {
			NodeSequence n3 = (NodeSequence)n2.choice;
			for(int i=0; i<n3.size(); i++)
				str += ((NodeToken)n3.elementAt(i)).tokenImage;
		}
		else {
			NodeToken n3 = (NodeToken)n2.choice;
			str += n3.tokenImage;
		}
			
		str += n.f1.tokenImage + n.f2.tokenImage;
		return str;
	}

}
