package edu.ucla.cs.db.prism.backend.rewrite;

import java.util.Vector;

import edu.ucla.cs.db.prism.backend.ded.DEDSetHistory;
import edu.ucla.cs.db.prism.backend.exception.ErrorRewritingException;
import edu.ucla.cs.db.prism.backend.icmo.AddPrimaryKey;
import edu.ucla.cs.db.prism.backend.icmo.MOImpl;
import edu.ucla.cs.db.prism.backend.icmo.ICMOSMOSequence;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchemaHistory;

public class RewriteEngineAdaptor {

	
	ICSchema queriedSchema = null;
	ICSchema sourceSchema = null;
	ICMOSMOSequence sequence = null;
	ICSchemaHistory schemaHistory = null;
	DEDSetHistory deds = null;
	Vector<ICMOSMOSequence> portions = null;
	Vector<ICPrimRewriteEngine> engines = null;

	
	public RewriteEngineAdaptor(ICSchema queriedSchema, ICSchema sourceSchema, ICMOSMOSequence sequence, boolean isInverse){
		
		if(!isInverse){
			
			System.err.println("NOT IMPLEMENTED YET the NON INSERVE REWRITE");
			
		}
		
		schemaHistory = sequence.generateICSchemaHistory(sourceSchema, true);
		this.queriedSchema = queriedSchema;
		this.sourceSchema = sourceSchema;
		
		this.sequence = sequence;
		deds = sequence.getAllDED(sourceSchema,true);
		
		engines = new Vector<ICPrimRewriteEngine>();
		
		Vector<MOImpl> mos = sequence.getMos();
		int temp = 0;
		int i = 0;
		for(i = 0; i< mos.size(); i++){
			
			MOImpl icmo = mos.elementAt(i);
			
			if(icmo instanceof AddPrimaryKey && ((AddPrimaryKey)icmo).enforce == true){
				
				try {
					ICSchemaHistory partialhist = schemaHistory.schemaSet(schemaHistory.size()-(i+1), schemaHistory.size()-temp);
					partialhist.reOrder();
					DEDSetHistory partialdeds = new DEDSetHistory(deds.dedSets.subList(deds.dedSets.size()-(i),deds.dedSets.size()-temp));
					engines.add(new ICPrimRewriteEngine(partialhist, partialdeds));
					engines.add(new AddPkRewriteEngine((AddPrimaryKey)icmo, schemaHistory.schemas.get(i)));
				} catch (ErrorRewritingException e) {
					e.printStackTrace();
				}
				temp = i;
			}
				
		}
		try {
			ICSchemaHistory partialhist = schemaHistory.schemaSet(0, schemaHistory.size()-(i-1));
			partialhist.reOrder();
			DEDSetHistory partialdeds = new DEDSetHistory(deds.dedSets.subList(deds.dedSets.size()-(i),deds.dedSets.size()-temp));
			engines.add(new ICPrimRewriteEngine(partialhist, partialdeds));
		} catch (ErrorRewritingException e) {
			e.printStackTrace();
		}

	}
	
	
	public String rewrite(String query){
		
		String tempquery = null;
		try {
			//ICPrimRewriteEngine engine = new ICPrimRewriteEngine(schemaHistory,deds);
			//output = engine.rewriteWithNegationAndICMO(query, sourceSchema.versionID, sourceSchema.minorVersionID, queriedSchema.versionID, queriedSchema.minorVersionID);
			tempquery = query;
			
			for(ICPrimRewriteEngine eng:engines){
				tempquery = eng.rewriteWithNegationAndICMO(tempquery);
				System.out.println("\n\n" + tempquery);
			}
		
		} catch (ErrorRewritingException e) {
			e.printStackTrace();
		} 
		
		return tempquery;
	}
	
	public String toString(){
		
		String out = "================ \n"; 
		out+= "QUERIED SCHEMA:" + queriedSchema + "\n";
		out+= "SMO SEQUENCE:" + sequence +"\n";
		out+= "SCHEMA HISTORY:" + schemaHistory.toStringWithMetadata() +"\n";
		out+= "DED HISTORY:" + deds +"\n";
		out += "================ \n";
		return out;
	}
}
