package edu.ucla.cs.db.prism.backend.rewrite;

import static org.junit.Assert.*;

import java.util.Vector;

import org.junit.Test;

import edu.ucla.cs.db.prism.backend.icmo.AddPrimaryKey;
import edu.ucla.cs.db.prism.backend.icmo.ICMOFactory;
import edu.ucla.cs.db.prism.backend.icmo.ICMOSMOSequence;
import edu.ucla.cs.db.prism.backend.icmo.ICMOTypeUnknownException;
import edu.ucla.cs.db.prism.backend.icmo.MOFactory;
import edu.ucla.cs.db.prism.backend.icmo.MOImpl;
import edu.ucla.cs.db.prism.backend.icmo.exceptions.AddPrimaryKeyException;
import edu.ucla.cs.db.prism.backend.rewrite.update.UpdateRewriteEngine;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchemaHistory;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICTable;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.PrimaryKey;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.exceptions.IntegrityConstraintsExistsException;
import edu.ucla.cs.db.prism.backend.smo.exception.SMOParsingErrorException;
import edu.ucla.cs.db.prism.backend.smo.util.SMOFactory;

public class RewriteEngineAdaptorTest {
	
	ICSchema testschema;
	ICMOSMOSequence sequence;
	
	public void init(){
		
	    testschema = new ICSchema(0, 0, "testschema","", "","","", "0",null,null);
		
		ICTable test1 = new ICTable("test1");
		test1.addColumn("k1");
		test1.addColumn("k2");
		test1.addColumn("a");
		test1.addColumn("b");
		test1.addColumn("c");
		
		Vector<String> keys1 = new Vector<String>();
		keys1.add("k1");
		//keys1.add("k2");
		try {
			test1.addConstraint(new PrimaryKey("pk1",keys1));
		} catch (IntegrityConstraintsExistsException e2) {
			e2.printStackTrace();
		}
		
		ICTable test2 = new ICTable("test2");
		test2.addColumn("k1");
		test2.addColumn("d");
		test2.addColumn("e");
		test2.addColumn("f");
		
		
		testschema.addTable(test1);
		testschema.addTable(test2);
		
		sequence = new ICMOSMOSequence();
		sequence.sourceVersion = 0;
		sequence.targetVersion = 1;
		
		try {
	
			MOImpl mo = MOFactory.createMOFromString("DECOMPOSE TABLE test1 INTO test5(k1,a), test6(k1,k2,b,c)");
			mo.setSourceMajor(0);
			mo.setSourceMinor(0);
			mo.setTargetMajor(0);
			mo.setTargetMinor(1);
			sequence.add(mo);
			
			mo = MOFactory.createMOFromString("ALTER TABLE test5 DROP PRIMARY KEY pk1");
			mo.setSourceMajor(0);
			mo.setSourceMinor(1);
			mo.setTargetMajor(0);
			mo.setTargetMinor(2);
			sequence.add(mo);
			
			mo = MOFactory.createMOFromString("RENAME TABLE test5 INTO test7");
			mo.setSourceMajor(0);
			mo.setSourceMinor(2);
			mo.setTargetMajor(1);
			mo.setTargetMinor(0);
			sequence.add(mo);
			
			
		} catch (SMOParsingErrorException e) {
			e.printStackTrace();
		}
		
		
		
	}
	@Test
	public final void testRewrite() {
		
		init();
		String query = "SELECT t.c,t.a FROM test1 t WHERE (t.k1 = \"1\") ";
		//query = "SELECT x1_2.k1 FROM test1 x1_2, test1 x1 WHERE x1.k1 = x1_2.k1 AND x1.a != x1_2.a ";
		
		//FIXME MARS can't accept 2 identical constants in the target list
		//query = "SELECT \"1\", x0.k2, x1.a, x0.b, \"1\" FROM test1 x0, test1 x1 WHERE x1.k1 = x0.k1 and \"1\" = x0.c";
		
		System.out.println(query);
		ICSchemaHistory ics = sequence.generateICSchemaHistory(testschema.clone(), false);
		ICMOSMOSequence inverses = sequence.getICCandidateInverseSequence(testschema.clone());
		
		((AddPrimaryKey)inverses.getMos().get(1)).setEnforce();
		
		RewriteEngineAdaptor ure = new RewriteEngineAdaptor(testschema,ics.getMostRecent(), inverses, true);
		
		System.out.println(ure.rewrite(query));
		
		fail("Not yet implemented"); // TODO
	}

}
