package edu.ucla.cs.db.prism.backend.rewrite.htable;

import edu.ucla.cs.db.prism.backend.exception.ErrorRewritingException;
import edu.ucla.cs.db.prism.backend.SQL.*;
import edu.ucla.cs.db.prism.backend.ded.*;
import edu.ucla.cs.db.prism.backend.marsManage.MarsRunManage;
import edu.ucla.cs.db.prism.backend.rewrite.PrimRewriteEngine;
import edu.ucla.cs.db.prism.backend.schema.*;
import edu.ucla.cs.db.prism.backend.smo.util.*;
import edu.ucla.cs.db.prism.backend.util.REFConstant;

import java.util.*;

public class PrimaSnapshotHQueryRewriteEngine {
	final boolean checkInputQueryCorrectness = true;
	
	final boolean useTConstraints = true;
	final boolean useHConstraints = true;
	final boolean backchaseInHtoTRewriting = true;
	final boolean backchaseInTtoHRewriting = true;

	private SchemaHistory schemaHistory;
	private DEDSetHistory dedSetHistory;
	
	/**
	 * constructor
	 * @param schemaHistory
	 * @param dedSetHistory
	 */
	public PrimaSnapshotHQueryRewriteEngine(SchemaHistory schemaHistory, 
			DEDSetHistory dedSetHistory) {
		this.schemaHistory = schemaHistory;
		this.dedSetHistory = dedSetHistory;
	}
	
	public SQLUnion rewrite(SQL hSQLTargetVersion, int sourceVersion, int sourceMinorVersion, 
			int targetVersion, int targetMinorVersion) {
		
		// step 1: H-to-T rewriting, on target version
		SQLUnion tSQLUnionTargetVersion = 
			rewriteHtoT(hSQLTargetVersion, targetVersion, targetMinorVersion);
		System.out.println("tSQLUnion on Target Version:\n" + tSQLUnionTargetVersion + "\n");
		
		// step 2: target-to-source rewriting (T-to-T)
		PrimRewriteEngine pre = new PrimRewriteEngine(schemaHistory, dedSetHistory);
		SQLUnion tSQLUnionSourceVersion = null;
		try {
			tSQLUnionSourceVersion = pre.rewrite(tSQLUnionTargetVersion, 
					sourceVersion, sourceMinorVersion, targetVersion, targetMinorVersion);
		} catch (ErrorRewritingException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		System.out.println("tSQLUnion on Source Version:\n" + tSQLUnionSourceVersion + "\n");
		//System.exit(1);
		
		// step 3: T-to-H rewriting, on target version
		SQLUnion hSQLUnionSourceVersion = 
			rewriteTtoH(tSQLUnionSourceVersion, sourceVersion, sourceMinorVersion);
		//System.out.println("hSQLUnion on Source Version:\n" + hSQLUnionSourceVersion + "\n");
		
		
		return hSQLUnionSourceVersion;
	}

	public SQLUnion rewriteHtoT(SQL hSQL, int version, int minorVersion) {
		SQLUnion hSQLUnion = new SQLUnion();
		hSQLUnion.sqls.add(hSQL);
		SQLUnion rewrittenSQLUnion =
			rewriteHtoT(hSQLUnion, version, minorVersion);
		return rewrittenSQLUnion;
	}

	public SQLUnion rewriteHtoT(SQLUnion hSQLUnion, int version, int minorVersion) {
		if(hSQLUnion.name == null)
			hSQLUnion.name = "Q1";
		SQLUnionSet hSQLUnionSet = new SQLUnionSet();
		hSQLUnionSet.sqlUnions.add(hSQLUnion);
		SQLUnionSet rewrittenSQLUnionSet =
			rewriteHtoT(hSQLUnionSet, version, minorVersion);
		if(rewrittenSQLUnionSet == null || rewrittenSQLUnionSet.sqlUnions == null ||
				rewrittenSQLUnionSet.sqlUnions.size() == 0)
			return null;
		else
			return rewrittenSQLUnionSet.sqlUnions.get(0);
	}
	
	// add a prefix vh41m2
	public SQLUnionSet rewriteHtoT(SQLUnionSet hSQLUnionSet, int version, int minorVersion) {
		// step 1: generate DEDs
		Schema tSchema = schemaHistory.getSchemaByVersion(version, minorVersion);
		DEDSet dedSet = getTtoHMappings(tSchema); // data migration T-to-H = query rewriting H-to-T

		// step 1.1: add T-table constraints
		if(useTConstraints) {
			DEDSet tConstraints = getTConstraints(tSchema);
			dedSet.addAll(tConstraints);
		}
		
		// step 2: version-tag query		
		hSQLUnionSet.versionTag("vh" + version + "m" + minorVersion);

		// step 3: run mars
		// interface fitting
		DEDSetHistory dedSetHistory = new DEDSetHistory();
		dedSetHistory.add(dedSet);

		// prepare schemas, used for CODI schema type declaration
		SchemaHistory schemas = new SchemaHistory();
		Schema snapshotHTableSchema = getSnapshotHTableSchema(tSchema);
		schemas.addSchema(tSchema);
		schemas.addSchema(snapshotHTableSchema);
		
		// prepare sourceTablesForTranslate
		Vector<String> sourceTablesForTranslate = getSourceTablesForTranslate(
				tSchema, "v" + version + "m" + minorVersion);
		// include REFs coming from queries
		// quick-fix: to get REFs from a query, do REF conversion once more, before it's done by marsRunManage
		hSQLUnionSet.convertFunctionToREF(0, 0);
		hSQLUnionSet.convertInequalityToREF(0, 0);
		Vector<String> refRelationsInQuery = 
			REFConstant.getREFRelationNames(hSQLUnionSet);
		sourceTablesForTranslate.addAll(refRelationsInQuery);
		
		SQLUnionSet rewrittenSQLUnionSet =
			MarsRunManage.marsRunManage(hSQLUnionSet, 
					schemas, dedSetHistory, 
					0, 0, 0, 0, 
					sourceTablesForTranslate, backchaseInHtoTRewriting);
		
		// step 4: version-untag query
		if(rewrittenSQLUnionSet!=null)
			rewrittenSQLUnionSet.versionUntag();
		
		// step 5: minimize output queries remove tautologies
		//rewrittenSQLUnionSet.removeTautologies();
		
		return rewrittenSQLUnionSet;
	}
	
	

	public SQLUnion rewriteTtoH(SQL tSQL, int version, int minorVersion) {
		SQLUnion tSQLUnion = new SQLUnion();
		tSQLUnion.sqls.add(tSQL);
		SQLUnion rewrittenSQLUnion =
			rewriteTtoH(tSQLUnion, version, minorVersion);
		return rewrittenSQLUnion;
	}

	public SQLUnion rewriteTtoH(SQLUnion tSQLUnion, int version, int minorVersion) {
		if(tSQLUnion.name == null)
			tSQLUnion.name = "Q1";
		SQLUnionSet tSQLUnionSet = new SQLUnionSet();
		tSQLUnionSet.sqlUnions.add(tSQLUnion);
		SQLUnionSet rewrittenSQLUnionSet =
			rewriteTtoH(tSQLUnionSet, version, minorVersion);
		if(rewrittenSQLUnionSet == null || rewrittenSQLUnionSet.sqlUnions == null ||
				rewrittenSQLUnionSet.sqlUnions.size() == 0)
			return null;
		else
			return rewrittenSQLUnionSet.sqlUnions.get(0);
	}
	public SQLUnionSet rewriteTtoH(SQLUnionSet tSQLUnionSet, int version, int minorVersion) {
		// step 1: generate DEDs
		Schema tSchema = schemaHistory.getSchemaByVersion(version, minorVersion);
		DEDSet dedSet = getHtoTMappings(tSchema); // data migration T-to-H = query rewriting H-to-T
		
		// step 1.1: add H-table constraints
		if(useHConstraints) {
			DEDSet hConstraints = getHConstraints(tSchema);
			dedSet.addAll(hConstraints);
		}
		
		// step 2: version-tag query		
		tSQLUnionSet.versionTag("v" + version + "m" + minorVersion);

		// step 3: run mars
		// interface fitting
		DEDSetHistory dedSetHistory = new DEDSetHistory();
		dedSetHistory.add(dedSet);

		// prepare schemas, used for CODI schema type declaration
		SchemaHistory schemas = new SchemaHistory();
		Schema snapshotHTableSchema = getSnapshotHTableSchema(tSchema);
		schemas.addSchema(tSchema);
		schemas.addSchema(snapshotHTableSchema);
		
		// prepare sourceTablesForTranslate
		Vector<String> sourceTablesForTranslate = getSourceTablesForTranslate(
				snapshotHTableSchema, "vh" + version + "m" + minorVersion);
		// include REFs coming from queries
		// quick-fix: to get REFs from a query, do REF conversion once more, before it's done by marsRunManage
		tSQLUnionSet.convertFunctionToREF(0, 0);
		tSQLUnionSet.convertInequalityToREF(0, 0);
		Vector<String> refRelationsInQuery = 
			REFConstant.getREFRelationNames(tSQLUnionSet);
		sourceTablesForTranslate.addAll(refRelationsInQuery);

		SQLUnionSet rewrittenSQLUnionSet =
			MarsRunManage.marsRunManage(tSQLUnionSet, 
					schemas, dedSetHistory, 
					0, 0, 0, 0, 
					sourceTablesForTranslate, backchaseInTtoHRewriting);
		
		// step 4: version-untag query
		if(rewrittenSQLUnionSet!=null)
			rewrittenSQLUnionSet.versionUntag();
		

		// step 5: minimize output queries remove tautologies
		//rewrittenSQLUnionSet.removeTautologies();
		
		return rewrittenSQLUnionSet;
	}
	
	private DEDSet getTtoHMappings(Schema snapshotTSchema) {
		DEDSet dedSet = new DEDSet();
		for(int i=0; i<snapshotTSchema.size(); i++) {
			DEDSet tempDEDSet = getTtoHMappings(snapshotTSchema.getTable(i), 
					snapshotTSchema.versionID, snapshotTSchema.minorVersionID);
			dedSet.addAll(tempDEDSet);
		}
		return dedSet;
		
	}
	private DEDSet getTtoHMappings(Table table, int version, int minorVersion) {

		/*
		 *  T-to-H-DM
		 *	R(K,A,B) -> R_K(K)
		 *	R(K,A,B) -> R_A(K,A)
		 *	R(K,A,B) -> R_B(K,B)
		 *
		 *	T-to-H-QR
		 *	R_K(K)   -> R(K,A,B)
		 *	R_A(K,A) -> R(K,A,B)
		 *	R_B(K,B) -> R(K,A,B)
		 */
		DEDSet dedSet = new DEDSet();
		DED ded = null;
		QuantifiedDisjunctive tc = null;
		Predicate p = null;
		String dedNameCommon = "TtoH_v" + version + "m" + minorVersion + "_" + table.getTableName();

		String tVersionTag = "v" + version + "m" + minorVersion;
		String hVersionTag = "vh" + version + "m" + minorVersion;
		
		// step 1: for key attributes, create R(K,A,B)->R_K(K)
		// step 1.1: Data migration direction
		ded = new DED();
		dedSet.add(ded);
		String leastKeyColumns = table.getLeastPrimaryKeyColumn();
		ded.dedName = dedNameCommon + "_key_" + leastKeyColumns + "_DM";

		ded.globalQuantifiers.quantifiers.add(new Quantifier("all", "S0", tVersionTag + "_" + table.getTableName()));
		tc = new QuantifiedDisjunctive();
		ded.thenClauses.add(tc);
		tc.quantifiers.quantifiers.add(new Quantifier("some", "T0", hVersionTag + "_" + table.getTableName() + "_" + leastKeyColumns));
		tc.disjunctive.conjunctives.add(new Conjunctive());
		for(int j=0; j<table.getPrimaryKeysColumns().size(); j++) {
			p = new Predicate();
			tc.disjunctive.conjunctives.get(0).add(p);
			String keyColumn = table.getPrimaryKeysColumns().get(j);
			p.term1 = new Term("S0", keyColumn);
			p.op = "=";
			p.term2 = new Term("T0", keyColumn);
		}
		
		// step 1.2: Query rewriting direction
		ded = new DED();
		dedSet.add(ded);
		ded.dedName = dedNameCommon + "_key_" + leastKeyColumns + "_QR";

		ded.globalQuantifiers.quantifiers.add(new Quantifier("all", "T0", hVersionTag + "_" + table.getTableName() + "_" + leastKeyColumns));
		tc = new QuantifiedDisjunctive();
		ded.thenClauses.add(tc);
		tc.quantifiers.quantifiers.add(new Quantifier("some", "S0", tVersionTag + "_" + table.getTableName()));
		
		tc.disjunctive.conjunctives.add(new Conjunctive());
		for(int j=0; j<table.getPrimaryKeysColumns().size(); j++) {
			p = new Predicate();
			tc.disjunctive.conjunctives.get(0).add(p);
			String keyColumn = table.getPrimaryKeysColumns().get(j);
			p.term1 = new Term("S0", keyColumn);
			p.op = "=";
			p.term2 = new Term("T0", keyColumn);
		}
		
		// step 2: for each non-key attributes
		for(int i=0; i<table.getColumns().size(); i++) {
			if(table.isInPrimaryKey(table.getColumns().get(i)))
				continue;
			String nonKeyColumn = table.getColumns().get(i);
			
			// step 2.1: Data migration direction
			ded = new DED();
			dedSet.add(ded);
			ded.dedName = dedNameCommon + "_nonkey_" + nonKeyColumn + "_DM";

			ded.globalQuantifiers.quantifiers.add(new Quantifier("all", "S0", tVersionTag + "_" + table.getTableName()));
			tc = new QuantifiedDisjunctive();
			ded.thenClauses.add(tc);
			tc.quantifiers.quantifiers.add(new Quantifier("some", "T0", hVersionTag + "_" + table.getTableName() + "_" + nonKeyColumn));
			tc.disjunctive.conjunctives.add(new Conjunctive());
			for(int j=0; j<table.getPrimaryKeysColumns().size(); j++) {
				p = new Predicate();
				tc.disjunctive.conjunctives.get(0).add(p);
				String keyColumn = table.getPrimaryKeysColumns().get(j);
				p.term1 = new Term("S0", keyColumn);
				p.op = "=";
				p.term2 = new Term("T0", keyColumn);
			}
			// nonkey value equivalence
			p = new Predicate();
			tc.disjunctive.conjunctives.get(0).add(p);;
			p.term1 = new Term("S0", nonKeyColumn);
			p.op = "=";
			p.term2 = new Term("T0", nonKeyColumn);
			
			
			// step 2.2: Query rewriting direction
			ded = new DED();
			dedSet.add(ded);
			ded.dedName = dedNameCommon + "_nonkey_" + nonKeyColumn + "_QR";

			ded.globalQuantifiers.quantifiers.add(new Quantifier("all", "T0", hVersionTag + "_" + table.getTableName() + "_" + nonKeyColumn));
			tc = new QuantifiedDisjunctive();
			ded.thenClauses.add(tc);
			tc.quantifiers.quantifiers.add(new Quantifier("some", "S0", tVersionTag + "_" + table.getTableName()));
			tc.disjunctive.conjunctives.add(new Conjunctive());
			for(int j=0; j<table.getPrimaryKeysColumns().size(); j++) {
				p = new Predicate();
				tc.disjunctive.conjunctives.get(0).add(p);
				String keyColumn = table.getPrimaryKeysColumns().get(j);
				p.term1 = new Term("S0", keyColumn);
				p.op = "=";
				p.term2 = new Term("T0", keyColumn);
			}
			// nonkey value equivalence
			p = new Predicate();
			tc.disjunctive.conjunctives.get(0).add(p);;
			p.term1 = new Term("S0", nonKeyColumn);
			p.op = "=";
			p.term2 = new Term("T0", nonKeyColumn);
		}
		
		return dedSet;
	}
	private DEDSet getHtoTMappings(Schema snapshotTSchema) {
		DEDSet dedSet = new DEDSet();
		for(int i=0; i<snapshotTSchema.size(); i++) {
			DEDSet tempDEDSet = getHtoTMappings(snapshotTSchema.getTable(i), 
					snapshotTSchema.versionID, snapshotTSchema.minorVersionID);
			dedSet.addAll(tempDEDSet);
		}
		return dedSet;
		
	}
	private DEDSet getHtoTMappings(Table table, int version, int minorVersion) {

		/*
		 * H-to-T-DM
		 * R_K(K),R_A(K,A),R_B(K,B) -> R(K,A,B)
		 * 
		 * H-to-T-QR
		 * R(K,A,B) -> R_K(K),R_A(K,A),R_B(K,B)
		 */
		DEDSet dedSet = new DEDSet();
		DED ded = null;
		QuantifiedDisjunctive tc = null;
		Predicate p = null;
		String dedNameCommon = "HtoT_v" + version + "m" + minorVersion + "_" + table.getTableName();

		String tVersionTag = "v" + version + "m" + minorVersion;
		String hVersionTag = "vh" + version + "m" + minorVersion;
		
		// step 1: Data migration direction
		ded = new DED();
		dedSet.add(ded);
		String leastKeyColumns = table.getLeastPrimaryKeyColumn();
		ded.dedName = dedNameCommon + "_DM";

		ded.globalQuantifiers.quantifiers.add(new Quantifier("all", "S0", hVersionTag + "_" + table.getTableName() + "_" + leastKeyColumns));
		tc = new QuantifiedDisjunctive();
		ded.thenClauses.add(tc);
		tc.quantifiers.quantifiers.add(new Quantifier("some", "T0", tVersionTag + "_" + table.getTableName()));
		
		// initialize disjunctive
		ded.ifClause.disjunctive.conjunctives.add(new Conjunctive());
		tc.disjunctive.conjunctives.add(new Conjunctive());

		for(int j=0; j<table.getPrimaryKeysColumns().size(); j++) {
			p = new Predicate();
			tc.disjunctive.conjunctives.get(0).add(p);
			String keyColumn = table.getPrimaryKeysColumns().get(j);
			p.term1 = new Term("S0", keyColumn);
			p.op = "=";
			p.term2 = new Term("T0", keyColumn);
		}


		// for each R_column, 
		//  1) insert it to the if-clause, 
		//  2) add key equalities in if-clause
		//  3) add nonkey equality between if-clause and then-clause
		int nonKeyColumnIndex = 0;
		for(int i=0; i<table.getColumns().size(); i++) {
			if(table.isInPrimaryKey(table.getColumns().get(i)))
				continue;
			String nonKeyColumn = table.getColumns().get(i);
			nonKeyColumnIndex++;
			
			ded.globalQuantifiers.quantifiers.add(new Quantifier("all", "S" + nonKeyColumnIndex, hVersionTag + "_" + table.getTableName() + "_" + nonKeyColumn));
			for(int j=0; j<table.getPrimaryKeysColumns().size(); j++) {
				p = new Predicate();
				ded.ifClause.disjunctive.conjunctives.get(0).add(p);
				String keyColumn = table.getPrimaryKeysColumns().get(j);
				p.term1 = new Term("S0", keyColumn);
				p.op = "=";
				p.term2 = new Term("S" + nonKeyColumnIndex, keyColumn);
			}
			p = new Predicate();
			tc.disjunctive.conjunctives.get(0).add(p);
			p.term1 = new Term("S" + nonKeyColumnIndex, nonKeyColumn);
			p.op = "=";
			p.term2 = new Term("T0", nonKeyColumn);
		}
		
		
		
		
		
		// step 2: Query rewriting direction
		ded = new DED();
		dedSet.add(ded);
		ded.dedName = dedNameCommon + "_QR";

		ded.globalQuantifiers.quantifiers.add(new Quantifier("all", "T0", tVersionTag + "_" + table.getTableName()));
		tc = new QuantifiedDisjunctive();
		ded.thenClauses.add(tc);
		tc.quantifiers.quantifiers.add(new Quantifier("some", "S0", hVersionTag + "_" + table.getTableName() + "_" + leastKeyColumns));

		// initialize disjunctive
		ded.ifClause.disjunctive.conjunctives.add(new Conjunctive());
		tc.disjunctive.conjunctives.add(new Conjunctive());
		
		for(int j=0; j<table.getPrimaryKeysColumns().size(); j++) {
			p = new Predicate();
			tc.disjunctive.conjunctives.get(0).add(p);
			String keyColumn = table.getPrimaryKeysColumns().get(j);
			p.term1 = new Term("S0", keyColumn);
			p.op = "=";
			p.term2 = new Term("T0", keyColumn);
		}


		// for each R_column, 
		//  1) insert it to the if-clause, 
		//  2) add key equalities in if-clause
		//  3) add nonkey equality between if-clause and then-clause
		nonKeyColumnIndex = 0;
		for(int i=0; i<table.getColumns().size(); i++) {
			if(table.isInPrimaryKey(table.getColumns().get(i)))
				continue;
			String nonKeyColumn = table.getColumns().get(i);
			nonKeyColumnIndex++;
			
			tc.quantifiers.quantifiers.add(new Quantifier("some", "S" + nonKeyColumnIndex, hVersionTag + "_" + table.getTableName() + "_" + nonKeyColumn));
			for(int j=0; j<table.getPrimaryKeysColumns().size(); j++) {
				p = new Predicate();
				tc.disjunctive.conjunctives.get(0).add(p);
				String keyColumn = table.getPrimaryKeysColumns().get(j);
				p.term1 = new Term("S0", keyColumn);
				p.op = "=";
				p.term2 = new Term("S" + nonKeyColumnIndex, keyColumn);
			}
			p = new Predicate();
			tc.disjunctive.conjunctives.get(0).add(p);;
			p.term1 = new Term("S" + nonKeyColumnIndex, nonKeyColumn);
			p.op = "=";
			p.term2 = new Term("T0", nonKeyColumn);
		}
		return dedSet;
	}
	
	private DEDSet getTConstraints(Schema snapshotTSchema) {
		DEDSet allDEDs = new DEDSet();
		for(int i=0; i<snapshotTSchema.getTables().size(); i++) {
			DED ded = getTConstraints(snapshotTSchema.getTables().get(i), 
					snapshotTSchema.versionID, snapshotTSchema.minorVersionID);
			allDEDs.add(ded);
		}
		return allDEDs;
	}
	private DED getTConstraints(Table tTable, int version, int minorVersion) {
		// key constraints
		// PKt: R(K,A,B),S(K,A2,B2)->A=A2,B=B2
		DED ded = null;
		QuantifiedDisjunctive tc = null;
		Predicate p = null;
		String dedNameCommon = "T_v" + version + "m" + minorVersion + "_" + tTable.getTableName();
		
		String tVersionTag = "v" + version + "m" + minorVersion;

		// start a DED
		ded = new DED();
		ded.dedName = dedNameCommon + "_PrimaryKeyConstraint";

		// initialize a then-clause
		tc = new QuantifiedDisjunctive();
		ded.thenClauses.add(tc);
		
		// initialize disjunctive
		ded.ifClause.disjunctive.conjunctives.add(new Conjunctive());
		tc.disjunctive.conjunctives.add(new Conjunctive());

		Vector<String> keys = tTable.getPrimaryKeysColumns();
		
		// if-clause: if key columns are equal, for two tuples,
		ded.globalQuantifiers.quantifiers.add(new Quantifier("all", "S0", tVersionTag + "_" + tTable.getTableName()));
		ded.globalQuantifiers.quantifiers.add(new Quantifier("all", "S1", tVersionTag + "_" + tTable.getTableName()));
		for(int i=0; i<keys.size(); i++) {
			p = new Predicate();
			p.term1 = new Term("S0", keys.get(i));
			p.op = "=";
			p.term2 = new Term("S1", keys.get(i));
			ded.ifClause.disjunctive.conjunctives.get(0).add(p);
		}

		// then-clause: then the rest of the values are equal, too
		for(int i=0; i<tTable.getColumns().size(); i++) {
			if(tTable.isInPrimaryKey(tTable.getColumns().get(i)))
				continue;
			String nonKeyCol = tTable.getColumns().get(i);
			p = new Predicate();
			p.term1 = new Term("S0", nonKeyCol);
			p.op = "=";
			p.term2 = new Term("S1", nonKeyCol);
			tc.disjunctive.conjunctives.get(0).add(p);
		}
		return ded;
	}	
	private DEDSet getHConstraints(Schema snapshotTSchema) {
		DEDSet allDEDs = new DEDSet();
		for(int i=0; i<snapshotTSchema.getTables().size(); i++) {
			DEDSet tempDEDs = getHConstraints(snapshotTSchema.getTables().get(i),
					snapshotTSchema.versionID, snapshotTSchema.minorVersionID);
			allDEDs.addAll(tempDEDs);
		}
		return allDEDs;
	}
	private DEDSet getHConstraints(Table tTable, int version, int minorVersion) {
		DEDSet allDEDs = new DEDSet();
		for(int i=0; i<tTable.getColumns().size(); i++) {
			if(tTable.isInPrimaryKey(tTable.getColumns().get(i)))
				continue;
			String nonKeyCol = tTable.getColumns().get(i);
			DEDSet tempDEDs = getHConstraints(tTable, nonKeyCol, version, minorVersion);
			allDEDs.addAll(tempDEDs);
		}
		return allDEDs;
	}
	private DEDSet getHConstraints(Table tTable, String nonKeyCol, int version, int minorVersion) {
		/*
		 *   FKh: 
		 *   R(K) -> R_A(K,A)
         *   R_A(K,A) -> R(K)
         *   R(K) -> R_B(K,B)
         *   R_B(K,B) -> R(K)
		 */
		
		DEDSet dedSet = new DEDSet();
		DED ded = null;
		QuantifiedDisjunctive tc = null;
		Predicate p = null;
		String dedNameCommon = "H_v" + version + "m" + minorVersion + "_" + tTable.getTableName();

		String hVersionTag = "vh" + version + "m" + minorVersion;
		
		// Part 1: R(K) -> R_A(K,A)
		// start a DED
		ded = new DED();
		dedSet.add(ded);
		ded.dedName = dedNameCommon + "_" + nonKeyCol + "_ForeignKeyConstraint_KTtoNKT";

		// initialize a then-clause
		tc = new QuantifiedDisjunctive();
		ded.thenClauses.add(tc);
		
		// initialize disjunctive
		ded.ifClause.disjunctive.conjunctives.add(new Conjunctive());
		tc.disjunctive.conjunctives.add(new Conjunctive());
		
		// quantifiers
		Vector<String> keys = tTable.getPrimaryKeysColumns();
		String repKeyCol = tTable.getLeastPrimaryKeyColumn();
		ded.globalQuantifiers.quantifiers.add(new Quantifier("all", "S0", 
								hVersionTag + "_" + tTable.getTableName() + "_" + repKeyCol));
		tc.quantifiers.quantifiers.add(new Quantifier("some", "T0", 
				hVersionTag + "_" + tTable.getTableName() + "_" + nonKeyCol));
		// key = key in then-clause
		for(int i=0; i<keys.size(); i++) {
			p = new Predicate();
			p.term1 = new Term("S0", keys.get(i));
			p.op = "=";
			p.term2 = new Term("T0", keys.get(i));
			tc.disjunctive.conjunctives.get(0).add(p);
		}

		// Part 2: R_A(K,A) -> R(K)
		// start a DED
		ded = new DED();
		dedSet.add(ded);
		ded.dedName = dedNameCommon + "_" + nonKeyCol + "_ForeignKeyConstraint_NKTtoKT";

		// initialize a then-clause
		tc = new QuantifiedDisjunctive();
		ded.thenClauses.add(tc);
		
		// initialize disjunctive
		ded.ifClause.disjunctive.conjunctives.add(new Conjunctive());
		tc.disjunctive.conjunctives.add(new Conjunctive());
		
		// quantifiers
		keys = tTable.getPrimaryKeysColumns();
		repKeyCol = tTable.getLeastPrimaryKeyColumn();
		ded.globalQuantifiers.quantifiers.add(new Quantifier("all", "T0", 
				hVersionTag + "_" + tTable.getTableName() + "_" + nonKeyCol));
		tc.quantifiers.quantifiers.add(new Quantifier("some", "S0", 
				hVersionTag + "_" + tTable.getTableName() + "_" + repKeyCol));
		// key = key in then-clause
		for(int i=0; i<keys.size(); i++) {
			p = new Predicate();
			p.term1 = new Term("S0", keys.get(i));
			p.op = "=";
			p.term2 = new Term("T0", keys.get(i));
			tc.disjunctive.conjunctives.get(0).add(p);
		}

		return dedSet;
	}

	
	private Schema getSnapshotHTableSchema(Schema snapshotTSchema) {
		Schema snapshotHTableSchema = new Schema();
		for(int i=0; i<snapshotTSchema.size(); i++) {
			Vector<Table> snapshotHTables = 
				getSnapshotHTableSchema(snapshotTSchema.getTable(i));
			snapshotHTableSchema.getTables().addAll(snapshotHTables);
		}
		snapshotHTableSchema.versionID = snapshotTSchema.versionID;
		snapshotHTableSchema.minorVersionID = snapshotTSchema.minorVersionID;
		snapshotHTableSchema.isHTableSchema = true;
		return snapshotHTableSchema;
	}
	private Vector<Table> getSnapshotHTableSchema(Table tTable) {
		Vector<Table> tables = new Vector<Table>();
		
		Vector<String> keys = tTable.getPrimaryKeysColumns();
		String repKeyCol = tTable.getLeastPrimaryKeyColumn();
		if(keys == null || repKeyCol == null) {
			System.err.println("T-Table " + tTable.getTableName() + " does not have a key.");
			System.exit(1);
		}
		
		Table table = null;
		// key table
		table = new Table(tTable.getTableName() + "_" + repKeyCol);
		table.getColumns().addAll(keys);
		tables.add(table);
		
		// non-key table
		for(int i=0; i<tTable.getColumns().size(); i++) {
			if(keys.contains(tTable.getColumns().get(i)))
				continue;
			
			String nonKeyCol = tTable.getColumns().get(i);
			table = new Table(tTable.getTableName() + "_" + nonKeyCol);
			table.getColumns().addAll(keys);			
			table.getColumns().add(nonKeyCol);			
			tables.add(table);
		}
		
		return tables;
	}

	private Vector<String> getSourceTablesForTranslate(Schema sourceSchema,
			String versionTag) {
		Vector<String> sourceTables = new Vector<String>();
		for(int i=0; i<sourceSchema.getTables().size(); i++) {
			sourceTables.add(versionTag + "_" + sourceSchema.getTables().get(i).getTableName());
		}
		return sourceTables;
	}
}
