package edu.ucla.cs.db.prism.backend.rewrite.update;

import static org.junit.Assert.*;

import java.util.Vector;

import org.junit.Test;

import edu.ucla.cs.db.prism.backend.exception.SchemaErrorException;
import edu.ucla.cs.db.prism.backend.exception.UnimplementedFeatureException;
import edu.ucla.cs.db.prism.backend.icmo.AddPrimaryKey;
import edu.ucla.cs.db.prism.backend.icmo.ICMOSMOSequence;
import edu.ucla.cs.db.prism.backend.icmo.MOFactory;
import edu.ucla.cs.db.prism.backend.icmo.MOImpl;
import edu.ucla.cs.db.prism.backend.rewrite.update.exception.UnexpectedNegationException;
import edu.ucla.cs.db.prism.backend.schema.exception.IllegalSMOException;
import edu.ucla.cs.db.prism.backend.schema.exception.TableNotInSchemaException;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ForeignKey;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchemaHistory;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICTable;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.PrimaryKey;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.exceptions.IntegrityConstraintsExistsException;
import edu.ucla.cs.db.prism.backend.smo.exception.SMOParsingErrorException;
import edu.ucla.cs.db.prism.backend.smo.util.SMOFactory;
import edu.ucla.cs.db.prism.backend.util.Stopwatch;

public class UpdateRewriteEngineTest {

	ICSchema testschema = null;
	ICMOSMOSequence sequence = null;
	
	public void init(){
		
	    testschema = new ICSchema(0, 0, "testschema","", "","","", "0",null,null);
		
		ICTable test1 = new ICTable("test1");
		test1.addColumn("k1");
		test1.addColumn("k2");
		test1.addColumn("a");
		test1.addColumn("b");
		test1.addColumn("c");
		Vector<String> keys1 = new Vector<String>();
		keys1.add("k1");
		//keys1.add("k2");
		try {
			test1.addConstraint(new PrimaryKey("pk1",keys1));
		} catch (IntegrityConstraintsExistsException e2) {
			e2.printStackTrace();
		}
		
		ICTable test2 = new ICTable("test2");
		test2.addColumn("k1");
		test2.addColumn("a");
		test2.addColumn("b");
		test2.addColumn("c");
		test2.addColumn("fk1");
		
		// add a primary key
		Vector<String> keys2 = new Vector<String>();
		keys2.add("k1");
		try {
			test2.addConstraint(new PrimaryKey("pk2",keys2));
		} catch (IntegrityConstraintsExistsException e2) {
		
			e2.printStackTrace();
		}
		
		// add a foreign key to table test2
		Vector<String> foreignkey2 = new Vector<String>();
		foreignkey2.add("fk1");	
		Vector<String> ref2 = new Vector<String>();
		ref2.add("k1");
		try {
			test2.addConstraint(new ForeignKey("test2fk1", foreignkey2, "test1", ref2));
		} catch (IntegrityConstraintsExistsException e1) {
			e1.printStackTrace();
		}
		
		
		testschema.addTable(test1);
		testschema.addTable(test2);
	
		

		
	}

	public void initsequence(int i){
		
		
		sequence = new ICMOSMOSequence();
		sequence.sourceVersion = 0;
		sequence.targetVersion = 1;
		
		try {
			if(i==0){		
				MOImpl mo = SMOFactory.createSMOFromString("RENAME TABLE test1 INTO test4");
				mo.setSourceMajor(0);
				mo.setSourceMinor(0);
				mo.setTargetMajor(0);
				mo.setTargetMinor(1);
				sequence.add(mo);
				
				MOImpl mo2 = SMOFactory.createSMOFromString("DECOMPOSE TABLE test4 INTO test5(k1,a), test6(k1,k2,b,c)");
				mo2.setSourceMajor(0);
				mo2.setSourceMinor(1);
				mo2.setTargetMajor(1);
				mo2.setTargetMinor(0);
				sequence.add(mo2); 
			}
			else{

				MOImpl mo = MOFactory.createMOFromString("DECOMPOSE TABLE test1 INTO test5(k1,a), test6(k1,k2,b,c)");
				mo.setSourceMajor(0);
				mo.setSourceMinor(0);
				mo.setTargetMajor(0);
				mo.setTargetMinor(1);
				sequence.add(mo);
				
				mo = MOFactory.createMOFromString("ALTER TABLE test5 DROP PRIMARY KEY pk1");
				mo.setSourceMajor(0);
				mo.setSourceMinor(1);
				mo.setTargetMajor(0);
				mo.setTargetMinor(2);
				sequence.add(mo);
				
				mo = MOFactory.createMOFromString("RENAME TABLE test5 INTO test7");
				mo.setSourceMajor(0);
				mo.setSourceMinor(2);
				mo.setTargetMajor(1);
				mo.setTargetMinor(0);
				sequence.add(mo);

				
			}
		
		} catch (SMOParsingErrorException e) {
			e.printStackTrace();
		}
		
		
		
	}
	
	@Test
	public final void testRewriteUpdateStatement() {
	
		init();
		initsequence(1);
		//String statement = "DELETE FROM test1 WHERE k1=\"1\"";
		//String statement = "INSERT INTO test1 (k1,k2,a,b,c) VALUES(\"1\",\"2\",\"3\",\"4\",\"5\")";
		//String statement = "INSERT INTO test1 (k1,a,b)  SELECT k1,a,b FROM test2 WHERE b = \"7\"";
		
		String statement = "UPDATE test1 SET c = \"1\" WHERE k1=\"2\"";
		
		
		System.out.println(statement+"\n");
		
		ICSchemaHistory ics = sequence.generateICSchemaHistory(testschema.clone(), false);
		ICMOSMOSequence inverses = sequence.getICCandidateInverseSequence(testschema.clone());
		
		((AddPrimaryKey)inverses.getMos().get(1)).setEnforce();
		
		UpdateRewriteEngine ure = new UpdateRewriteEngine(testschema,ics.getMostRecent(), inverses, true);
				
		try {
			
			Stopwatch s = new Stopwatch();
			
			s.start();
			System.out.println(ure.rewriteUpdate(statement).toString());
			s.stop();
			System.out.println("COLD REWRITE: " + s.getElapsedTime() +"ms");
			
			s.reset();
			s.start();
			statement = "UPDATE test1 SET c = \"4\" WHERE k1=\"5\"";
			System.out.println(ure.rewriteUpdate(statement).toString());
			s.stop();
			System.out.println("HOT REWRITE: " + s.getElapsedTime() +"ms");
			
				
		} catch (TableNotInSchemaException e) {
			fail("something wrong");
			e.printStackTrace();			
		} catch (UnexpectedNegationException e) {
			fail("something wrong");
			e.printStackTrace();
		} catch (UnimplementedFeatureException e) {
			fail("unimplemented feature"); 
			e.printStackTrace();
		}
		
		
	
	}

}
