package edu.ucla.cs.db.prism.backend.schema;

import java.util.Collections;
import java.util.Vector;

//import com.sun.tools.javac.util.List;

import edu.ucla.cs.db.prism.backend.exception.SchemaErrorException;
import edu.ucla.cs.db.prism.backend.schema.exception.IllegalSMOException;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchemaHistory;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.exceptions.ExpressionParsignErrorException;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequenceInterface;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequenceHistory;

public class SchemaHistory {
	static final long serialVersionUID = 0;
	public Vector<Schema> schemas;
	
	public SchemaHistory () {
		schemas = new Vector<Schema>();
	}

	public SchemaHistory(Schema schema, SMOSequenceHistory smoSequenceHistory) {
		this(schema, smoSequenceHistory, null);
	}
	
	public SchemaHistory(Schema schema, SMOSequenceHistory smoSequenceHistory,
			SchemaHistory majorVersionSchemaHistory) {

		// Create all schema versions using the given schema and smoSequenceHistory
		// we replicate the given schema

		// initialize
		schemas = new Vector<Schema>();
		schemas.add(new Schema(schema));
		Schema maxVersionSchema = schema;
		//Schema minVersionSchema = schema;
		//boolean changeMade = true;
		int prevVersionID = -1;
		
		while(true) {
			// -> (forward) direction expansion
			SMOSequenceInterface smoSequenceFW = 
				smoSequenceHistory.get(maxVersionSchema.versionID, 
						maxVersionSchema.versionID+1);
			if(smoSequenceFW != null) {
				for(int i=0; i<smoSequenceFW.size(); i++) {
					maxVersionSchema = maxVersionSchema.clone();
					try {
						maxVersionSchema.applyOperator(smoSequenceFW.get(i));
					} catch (IllegalSMOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					} catch (ExpressionParsignErrorException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					} catch (SchemaErrorException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					schemas.add(maxVersionSchema);
				}
				
				// check using major version schema history (=reference)
				if(majorVersionSchemaHistory != null)
					checkComputedSchemaUsingReferenceSchema(maxVersionSchema,
							majorVersionSchemaHistory);
			}
			else
				break;
		}

	}
	
	public SchemaHistory (SchemaHistory srcSchemaHistory) {
		this.schemas = new Vector<Schema>();
		for(int i=0; i<srcSchemaHistory.size(); i++)
			this.addSchema(new Schema(srcSchemaHistory.getSchema(i)));
	}
	
	public void addSchema(Schema schema) {
		schemas.add(schema);
	}
	
	public Schema getSchema(int i) {
		return schemas.get(i);
	}

	public Schema getSchemaByVersion(int version,int minorVersion) {
		for(int i=0; i<size(); i++)
			if(schemas.get(i).versionID == version &&
					schemas.get(i).minorVersionID == minorVersion) // check for a major version only
				return schemas.get(i);
		return null;
	}

	public Schema getSchemaByVersion(int[] version) {
		for(int i=0; i<size(); i++)
			if(schemas.get(i).versionID == version[0] &&
					schemas.get(i).minorVersionID == version[1])
				return schemas.get(i);
		return null;
	}
	
	/*
	public Table getTable(String tablename) {
		for(int i=0; i<tables.size(); i++)
			if(tables.get(i).tableName.contentEquals(tablename))
				return tables.get(i);
		return null;
	}
	
	
	
	public int indexOf(String tableName) {
		for(int i=0; i<size(); i++)
			if(tables.get(i).tableName.contentEquals(tableName))
				return i;
		return -1;
	}

	public boolean containsColumn(String tablename, String columnname) {
		boolean found = false;
		Table table = null;
				
		for(int i=0; i<tables.size(); i++) {
			if(tables.get(i).tableName.contentEquals(tablename)) {
				table = tables.get(i);
				break;
			}
		}
		
		if(table == null)			
			return false;
		else if(table.indexOf(columnname) > -1)
			return true;
		else 
			return false;
	}
	 */
	
	public int size() {
		return schemas.size();
	}
	
	@Override
	public String toString() {
		String str = "";
		for(int i=0; i<schemas.size(); i++)
			str += schemas.get(i).toString() + "\n";
		return str;
	}
	
	public String toStringWithMetadata() {
		String str = "";
		for(int i=0; i<schemas.size(); i++)
			str += schemas.get(i).toStringWithMetadata() + "\n";
		return str;
	}

	public String metadata() {
		String str = "";
		for(int i=0; i<schemas.size(); i++)
			str += schemas.get(i).versionID +"." +schemas.get(i).minorVersionID+ "\n";
		return str;
	}
	
	public boolean equalsForCommonSchemaVersions(SchemaHistory schemaHistory2) {
		if(schemaHistory2 == null) 
			return true; // no common schema versions
		
		for(int i=0; i<this.schemas.size(); i++) {
			Schema schema = this.schemas.get(i);
			Schema schema2 =
				schemaHistory2.getSchemaByVersion(
						schema.versionID, schema.minorVersionID);
			if(schema2 == null)
				continue; // schemaHistory2 doesn't contain this.schema - ok
			else if(!schema.equals(schema2)) // if they are common, they should be equals
				return false;
		}

		// passed all test
		return true;
	}
	
	public boolean outputOldestNonEqualCommonSchemaVersions(SchemaHistory schemaHistory2) {
		if(schemaHistory2 == null) 
			return false; // no common schema versions

		Schema oldestMajorVersion = this.getOldestMajorVersion();
		int majorVersion = oldestMajorVersion.versionID;
		int minorVersion = 0;
		while(true) {
			Schema schema = getSchemaByVersion(majorVersion, minorVersion);
			if(schema == null) {
				minorVersion++;
				schema = getSchemaByVersion(majorVersion, minorVersion);
				if(schema == null) {
					majorVersion++;
					minorVersion = 0;
					schema = getSchemaByVersion(majorVersion, minorVersion);
				}
			}
			if(schema == null)
				break; // end of versions
			Schema schema2 = schemaHistory2.getSchemaByVersion(majorVersion, minorVersion);
			if(schema2 == null)
				continue; // schemaHistory2 doesn't have this.schema - ok
			else if(!schema.equals(schema2)) {
				System.out.println(schema);
				System.out.println(schema2);
				return true;
			}	
		}

		return false;
	}
	
	public Schema getOldestMajorVersion() {
		Schema oldestMajorVersion = null;
		for(int i=0; i<size(); i++) {
			Schema schema = this.schemas.get(i);
			if(schema.minorVersionID != 0) 
				continue;
			
			if(oldestMajorVersion == null)
				oldestMajorVersion = schema;
			else if(oldestMajorVersion.versionID > schema.versionID)
				oldestMajorVersion = schema;
		}
		return oldestMajorVersion;
	}
	
	protected void checkComputedSchemaUsingReferenceSchema(Schema computedSchema,
			SchemaHistory majorVersionSchemaHistory) {
		

		Schema referenceSchema =
			majorVersionSchemaHistory.getSchemaByVersion(
					computedSchema.versionID, computedSchema.minorVersionID);
		if(referenceSchema == null) // not found in reference
			return;
		
		else if(!referenceSchema.equals(computedSchema)) {
			System.err.println("Error in SMO: can't match " + 
					"the computed schema version with reference schema version");
			
			Schema[] diffs = referenceSchema.getDiffs(computedSchema);
			
			System.err.println("Reference Schema: v" +
					referenceSchema.versionID + "." +
					referenceSchema.minorVersionID + "\n" + 
					diffs[0]);
			System.err.println("Computed Schema: v" +
					computedSchema.versionID + "." +
					computedSchema.minorVersionID + "\n" + 
					diffs[1]);
			//System.exit(1);
		}
	}

	public boolean hasVersion (int versionID, int minorVersionID) {
		if(schemas.size() == 0) return false;
		for(int i=0; i<schemas.size(); i++) {
			Schema schema = schemas.get(i);
			if(schema.versionID == versionID && schema.minorVersionID == minorVersionID)
				return true;
		}
		return false;
	}

	public Schema getMostRecent() {
		
		if(this instanceof ICSchemaHistory)
			return ((ICSchemaHistory)this).schemas.lastElement();
		
		return schemas.lastElement();
	}

	public void reOrder() {
		Collections.sort(schemas.subList(0, schemas.size()));
	}

	public Vector<Schema> getSchemas() {
		Vector<Schema> ret = new Vector<Schema>();
		if(this instanceof ICSchemaHistory){
			for(Schema s :((ICSchemaHistory)this).schemas)
				ret.add(s);	
			return ret;
		}		
		return schemas;
	}
	

	public boolean setSchemas(Vector<Schema> in) {
		Vector<ICSchema> vet = new Vector<ICSchema>();
		if(this instanceof ICSchemaHistory){
			for(Schema s :in){
				if(s instanceof ICSchema)
					vet.add((ICSchema)s);
				else
					return false;
			}
			((ICSchemaHistory)this).schemas = vet;
			return true;
		}
		schemas = in;
		return true;
	}

}
