package edu.ucla.cs.db.prism.backend.schema;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.apache.log4j.Logger;
import java.util.Iterator;

import edu.ucla.cs.db.prism.backend.util.Log4JHelper;
import edu.ucla.cs.db.prism.backend.util.db.DBUtil;

public class SchemaHistoryLoader {
	
	static Logger logger = Logger.getLogger(SchemaHistoryLoader.class);
	boolean debug = false;
	protected Connection conn = null;
	protected String schemaName = null;
	
	public SchemaHistoryLoader(Connection conn, String schemaName){
		this.conn = conn;	
		this.schemaName = schemaName;
        Log4JHelper.log4JSetup(logger);		
	}
	
	
	public boolean storeSchemaHistoryToDB(SchemaHistory schemaHistory){
		
	
		String schemataQuery = "INSERT INTO SCHEMATA(schema_version, " +
											"schema_version_minor, " +
											"SCHEMA_NAME, " +
											"DEFAULT_CHARACTER_SET_NAME, " +
											"DEFAULT_COLLATION_NAME," +
											"SQL_PATH," +
											"comment," +
											"REVISION_NUMBER, " +
											"TIMESTAMP) " +
									"VALUES ";
		
		String tablesQuery = "INSERT INTO TABLES(schema_version, " +
												"schema_version_minor, " +
												"TABLE_SCHEMA, " +
												"TABLE_NAME) " +
									"VALUES ";
		String columnsQuery = "INSERT INTO COLUMNS(schema_version, " +
												"schema_version_minor, " +
												"TABLE_SCHEMA, " +
												"TABLE_NAME, " +
												"COLUMN_NAME, COLUMN_TYPE) " +
									"VALUES ";
									
		Iterator<Schema> schemaIt = schemaHistory.schemas.iterator();
		

	
		while(schemaIt.hasNext()){
			
			Schema schema = schemaIt.next();
			 System.out.println(schema.versionID + " " + schema.minorVersionID);

			int Major = schema.versionID;
			int Minor = schema.minorVersionID;
						
			schemataQuery += "('" + Major + "','" +
				     Minor + "','" +
				     schema.getSchemaName() + "','" +
				     schema.DEFAULT_CHARACTER_SET_NAME + "','" +
				     schema.DEFAULT_COLLATION_NAME + "','" +
				     schema.SQL_PATH + "','" +
				     schema.comment + "','" +
				     schema.REVISION_NUMBER + "','" +
				     schema.tstart +
				     "'),";
			
			
			Iterator<Table> tabit = schema.getTables().iterator();

			
			while(tabit.hasNext()){
				Table table = tabit.next();
				
				tablesQuery += "('" + Major + "','" +
								  	Minor + "','" +
								  	schemaName + "','" +
								  	table.getTableName() +"'),";
				Iterator<String> columnIt = table.getColumns().iterator();
				Iterator<String> typeIt = table.getColTypes().iterator();
				while(columnIt.hasNext()){
					String column = columnIt.next();
					String type = typeIt.next();
					columnsQuery += "('" + Major + "','" +
									  	Minor + "','" +
									  	schemaName + "','" +
									  	table.getTableName() + "','" + 
									  	column + "','"+ type +"'),";}

			}
			
			
		}
		columnsQuery = columnsQuery.substring(0,columnsQuery.length()-1)+ ";";
		tablesQuery = tablesQuery.substring(0,tablesQuery.length()-1)+ ";";
		schemataQuery = schemataQuery.substring(0, schemataQuery.length()-1) + ";";
		
		
		int schemaAff = DBUtil.executeSQLUpdate(schemataQuery,conn);
		int tableAff = DBUtil.executeSQLUpdate(tablesQuery,conn);
		int columnAff = DBUtil.executeSQLUpdate(columnsQuery,conn);

		logger.info("STORED IN DB " + schemaAff + " schemas, "  + tableAff + " tables, " + columnAff+ " columns" );
		
		return true;
	}
	/**
	 * retrocompatible
	 * @return
	 * @deprecated
	 */
	public SchemaHistory loadSchemaHistoryFromDB(){
		return loadSchemaHistoryFromDB(true);
	}
	public SchemaHistory loadSchemaHistoryFromDB(boolean forward){
		
		SchemaHistory sh = new SchemaHistory();

		String schemaListSQL ="SELECT sc.schema_version," +
									 "sc.schema_version_minor, " +
									 "sc.DEFAULT_CHARACTER_SET_NAME, " +
									 "sc.DEFAULT_COLLATION_NAME," +
									 "sc.SQL_PATH, " +
									 "sc.comment, " +
									 "sc.REVISION_NUMBER, " +
									 "sc.TIMESTAMP " +
		  					  "FROM SCHEMATA sc " +
		  					  "WHERE sc.schema_name = '" + schemaName +"' ";
		
		 if(forward) 					  
			 schemaListSQL+=  "ORDER BY schema_version, schema_version_minor;";
		 else
			 schemaListSQL+=  "ORDER BY schema_version DESC, schema_version_minor DESC;";
		 
		ResultSet schemaList = DBUtil.executeSQLQuery(schemaListSQL,conn);
		
		String schemaVersion = null;
		String schemaVersionMinor = null;
		Schema schema = null;

		try {
			while(schemaList.next()){
				schemaVersion = schemaList.getString("schema_version");
				schemaVersionMinor = schemaList.getString("schema_version_minor");
				
				schema = new Schema(schemaList.getInt("schema_version"),
									schemaList.getInt("schema_version_minor"),
									schemaName,
									schemaList.getString("DEFAULT_CHARACTER_SET_NAME"),
									schemaList.getString("DEFAULT_COLLATION_NAME"),
									schemaList.getString("SQL_PATH"),
									schemaList.getString("comment"),
									schemaList.getString("REVISION_NUMBER"),									
									schemaList.getTimestamp("TIMESTAMP"), null
									);
				schema.setSchemaName(schemaName);
				String tableListSQL = "SELECT table_name  " +
						      		  "FROM TABLES t " +
						      		  "WHERE  t.schema_version = '" + schemaVersion + "' AND " +
						      		  		 "t.schema_version_minor = '" +schemaVersionMinor + "' AND " + 
						      		  		 "t.table_schema = '" + schemaName + "';";
				
				ResultSet tableList = DBUtil.executeSQLQuery(tableListSQL,conn);
					
				String tableName = null;
				String columnQuery = null;
				
					while(tableList.next()){
	
						tableName = tableList.getString("table_name");
						print("\n"+schemaName + " " +schemaVersion+ "  " + tableName +"(");
						columnQuery = "SELECT  c.column_name, c.COLUMN_TYPE " +
									  "FROM COLUMNS c " +
									  "WHERE  c.schema_version = '" + schemaVersion + "' AND " +
									  		"c.schema_version_minor = '" + schemaVersionMinor + "' AND " +	
						      		  		"c.table_schema = '" + schemaName + "' AND "+
									  		"c.table_name = '" + tableName + "';"; 
						
						ResultSet columnList = DBUtil.executeSQLQuery(columnQuery,conn);			  		
						Table tt = new Table(tableName);
						String columnName = null;
						String columnType = null;
						while(columnList.next()){
							columnName = columnList.getString("column_name");
							columnType = columnList.getString("COLUMN_TYPE");		
							print(columnName + ",");
							tt.addColumn(columnName,columnType);		
						}
						print(")");
						schema.addTable(tt);
					}
	
				sh.addSchema(schema);
			}
			
		} catch (SQLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return sh;
	}
	
	private void print(String s){
		if(debug)
		System.out.print(s);
		
	}


}
