package edu.ucla.cs.db.prism.backend.schema;

import java.io.Serializable;
import java.util.Vector;

public class SchemaModifiedInfo extends Schema implements Serializable {
	static final long serialVersionUID = 0;

	public Vector<TableModifiedInfo> tableModifiedInfos;
	
	public SchemaModifiedInfo(int versionID) {
		super(versionID);
		super.setTables(null); // we won't use this
		tableModifiedInfos = new Vector<TableModifiedInfo>();
	}
	
	public SchemaModifiedInfo(SchemaModifiedInfo srcSchema) {
		super(srcSchema.versionID);
		super.setTables(null); // we won't use this
		tableModifiedInfos = new Vector<TableModifiedInfo>();
		for(int i=0; i<srcSchema.size(); i++)
			this.tableModifiedInfos.add(new TableModifiedInfo(srcSchema.getTable(i)));
	}

	public void addTable(TableModifiedInfo tableModifiedInfo) {
		tableModifiedInfos.add(tableModifiedInfo);
	}
	
	@Override
	public TableModifiedInfo getTable(int i) {
		return tableModifiedInfos.get(i);
	}
	
	@Override
	public TableModifiedInfo getTable(String tablename) {
		for(int i=0; i<tableModifiedInfos.size(); i++)
			if(tableModifiedInfos.get(i).getTableName().contentEquals(tablename))
				return tableModifiedInfos.get(i);
		return null;
	}
	
	public boolean getTableModifiedFlag(String tablename) {
		int i;
		for(i=0; i<tableModifiedInfos.size(); i++)
			if(tableModifiedInfos.get(i).getTableName().contentEquals(tablename))
				break;
		return tableModifiedInfos.get(i).tableModifiedFlag;
	}
	
	@Override
	public int indexOf(String tableName) {
		for(int i=0; i<size(); i++)
			if(tableModifiedInfos.get(i).getTableName().contentEquals(tableName))
				return i;
		return -1;
	}
	
	@Override
	public boolean containsTable(String tableName) {
		if(indexOf(tableName) >= 0) return true;
		else return false;
	}
	
	@Override
	public boolean containsColumn(String tableName, String columnName) {
		int tableIndex = this.indexOf(tableName);
		if(tableIndex == -1) return false;
		if(this.getTable(tableIndex).indexOf(columnName) >= 0)
			return true;
		else 
			return false;
	}
	
	@Override
	public int size() {
		return tableModifiedInfos.size();
	}
	
	@Override
	public String toString() {
		return tableModifiedInfos.toString();
	}
	
	// this makes a shallow copy of 'this' into newSchema
	public SchemaModifiedInfo schemaDifference(SchemaModifiedInfo schema2) {
		// return this - schema2 (set difference)

		// step 1: copy this' into newSchema
		SchemaModifiedInfo newSchema = new SchemaModifiedInfo(this);

		// step 2: from newSchema, remove columns, t.c is in schema2
		for(int i=0; i<newSchema.size(); i++) {
			int numColumns = newSchema.getTable(i).getNumColumns();
			for(int j=0; j<numColumns; j++) {
				if(schema2.containsColumn(newSchema.getTable(i).getTableName(),
										  newSchema.getTable(i).getColumn(j))) {
					newSchema.getTable(i).removeColumn(j);
					j--;
					numColumns--;
				}
			}			
		}
		// step 3: from newSchema, remove empty tables
		int numTables = newSchema.size();
		for(int i=0; i<numTables; i++) {
			if(newSchema.getTable(i).getNumColumns() == 0) {
				newSchema.tableModifiedInfos.remove(i);
				i--;
				numTables--;
			}
		}
		
		return newSchema;
	}
}
