/**
 * 
 */
package edu.ucla.cs.db.prism.backend.schema.integrityconstraints;

import java.util.Collections;
import java.util.List;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.exception.SchemaErrorException;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.SchemaHistory;
import edu.ucla.cs.db.prism.backend.schema.exception.IllegalSMOException;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.exceptions.ExpressionParsignErrorException;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequenceInterface;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequenceHistory;

/**
 * @author Carlo A. Curino (carlo@curino.us)
 *
 */
public class ICSchemaHistory extends SchemaHistory {
	public Vector<ICSchema> schemas;

	
	public ICSchemaHistory () {
		schemas = new Vector<ICSchema>();
	}

	public ICSchemaHistory(ICSchema schema, SMOSequenceHistory smoSequenceHistory) {
		this(schema, smoSequenceHistory, null);
	}
	
	public ICSchemaHistory(ICSchema schema, SMOSequenceHistory smoSequenceHistory,
			ICSchemaHistory majorVersionSchemaHistory) {

		// Create all schema versions using the given schema and smoSequenceHistory
		// we replicate the given schema

		// initialize
		schemas = new Vector<ICSchema>();
		schemas.add(new ICSchema(schema));
		ICSchema maxVersionSchema = schema;
		//Schema minVersionSchema = schema;
		//boolean changeMade = true;
		int prevVersionID = -1;
		
		while(true) {
			// -> (forward) direction expansion
			SMOSequenceInterface smoSequenceFW = 
				smoSequenceHistory.get(maxVersionSchema.versionID, 
						maxVersionSchema.versionID+1);
			if(smoSequenceFW != null) {
				for(int i=0; i<smoSequenceFW.size(); i++) {
					maxVersionSchema = maxVersionSchema.clone();
					try {
						maxVersionSchema.applyOperator(smoSequenceFW.get(i));
					} catch (IllegalSMOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					} catch (ExpressionParsignErrorException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					} catch (SchemaErrorException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					schemas.add(maxVersionSchema);
				}
				
				// check using major version schema history (=reference)
				if(majorVersionSchemaHistory != null)
					checkComputedSchemaUsingReferenceSchema(maxVersionSchema,
							majorVersionSchemaHistory);
			}
			else
				break;
		}

	}
	
	@Override
	public ICSchema getSchema(int i) {
		return schemas.get(i);
	}
	
	public ICSchemaHistory (ICSchemaHistory srcSchemaHistory) {
		this.schemas = new Vector<ICSchema>();
		for(int i=0; i<srcSchemaHistory.size(); i++)
			this.addSchema(new ICSchema(srcSchemaHistory.getSchema(i)));
	}
	
	public ICSchemaHistory(List<ICSchema> subList) {
		this.schemas = new Vector<ICSchema>();
		for(int i=0; i<subList.size(); i++)
			this.addSchema(new ICSchema(subList.get(i)));	
	}

	@Override
	public int size() {
		return schemas.size();
	}
	
	@Override
	public void reOrder() {
		Collections.sort(schemas.subList(0, schemas.size()));
	}
	
	@Override
	public String toStringWithMetadata() {
		String str = "";
		for(int i=0; i<schemas.size(); i++)
			str += schemas.get(i).toStringWithMetadata() + "\n";
		return str;
	}
	
	@Override
	public String toString() {
		String str = "";
		for(int i=0; i<schemas.size(); i++)
			str += schemas.get(i).toString() + "\n";
		return str;
	}
	
	public void addSchema(ICSchema schema) {
		schemas.add(schema);
	}
	
	@Override
	public ICSchema getMostRecent() {
		return schemas.lastElement();
	}
	
	@Override
	public ICSchema getSchemaByVersion(int version,int minorVersion) {
		for(int i=0; i<size(); i++)
			if(schemas.get(i).versionID == version &&
					schemas.get(i).minorVersionID == minorVersion) // check for a major version only
				return schemas.get(i);
		return null;
	}

	@Override
	public ICSchema getSchemaByVersion(int[] version) {
		for(int i=0; i<size(); i++)
			if(schemas.get(i).versionID == version[0] &&
					schemas.get(i).minorVersionID == version[1])
				return schemas.get(i);
		return null;
	}
	
	
	@Override
	public ICSchema getOldestMajorVersion() {
		ICSchema oldestMajorVersion = null;
		for(int i=0; i<size(); i++) {
			ICSchema schema = this.schemas.get(i);
			if(schema.minorVersionID != 0) 
				continue;
			
			if(oldestMajorVersion == null)
				oldestMajorVersion = schema;
			else if(oldestMajorVersion.versionID > schema.versionID)
				oldestMajorVersion = schema;
		}
		return oldestMajorVersion;
	}

	public ICSchemaHistory schemaSet(int start, int end) {
		Vector<ICSchema> v = new Vector<ICSchema>();
		for(int i=start; i<end;i++)
			v.add(schemas.get(i));
		return new ICSchemaHistory(v);
	}

	public ICSchema getOldest() {
		ICSchema oldest = null;
		for(int i=0; i<size(); i++) {
			ICSchema schema = this.schemas.get(i);

			if(oldest == null)
				oldest = schema;
			else if(oldest.versionID > schema.versionID || (oldest.versionID == schema.versionID && (oldest.minorVersionID > schema.minorVersionID)))
				oldest = schema;
		}
		return oldest;
	}

}
