/**
 * 
 */
package edu.ucla.cs.db.prism.backend.schema.integrityconstraints;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.schema.SchemaHistoryLoader;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.exceptions.ExpressionParsignErrorException;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.exceptions.IntegrityConstraintsExistsException;
import edu.ucla.cs.db.prism.backend.util.db.DBUtil;

/**
 * @author Carlo A. Curino (carlo@curino.us)
 *
 */
public class ICSchemaHistoryLoader extends SchemaHistoryLoader {

	/**
	 * @param conn
	 * @param schemaName
	 */
	public ICSchemaHistoryLoader(Connection conn, String schemaName) {
		super(conn, schemaName);
	}

	public ICSchemaHistory loadSchemaHistoryFromDB(boolean forward, boolean loadHistory, String version) throws ForeignKeyCreationException, ForeignKeyLoopException{
		
		ICSchemaHistory sh = new ICSchemaHistory();

		//if loadHistory is true, it means we need to load the entire history
		
		String schemaListSQL = "";
		if(loadHistory){
		
							
				schemaListSQL =" SELECT SCHEMA_UNIQUE_NAME, " +
									"sc.MAJOR_ORDINAL_VERSION, " +
									"sc.MINOR_ORDINAL_VERSION," +
									" ps.DEFAULT_CHARACTER_SET_NAME, " +
									"ps.DEFAULT_COLLATION_NAME,ps.SQL_PATH, " +
									"sc.COMMENT, " +
									"sc.SVN_VERSION, " +
									"sc.TSTART , " +
									"sc.TEND " +
							  " FROM PRISM_VERSIONS sc, " +
							  	   " PRISM_SCHEMATA ps " +
							  " WHERE ps.SCHEMA_NAME = sc.`SCHEMA_UNIQUE_NAME` AND " +
							         "sc.schema_name = '" + schemaName +"'";
							
		}else{
			
				schemaListSQL =" SELECT SCHEMA_UNIQUE_NAME, " +
										"sc.MAJOR_ORDINAL_VERSION, " +
										"sc.MINOR_ORDINAL_VERSION," +
										" ps.DEFAULT_CHARACTER_SET_NAME, " +
										"ps.DEFAULT_COLLATION_NAME,ps.SQL_PATH, " +
										"sc.COMMENT, " +
										"sc.SVN_VERSION, " +
										"sc.TSTART , " +
										"sc.TEND " +
								" FROM PRISM_VERSIONS sc, " +
									  "PRISM_SCHEMATA ps " +
								" WHERE ps.SCHEMA_NAME = sc.`SCHEMA_UNIQUE_NAME` AND " +
									   "sc.schema_name = '" + schemaName +"' AND " +
									   	"sc.MAJOR_ORDINAL_VERSION = '" +version +"'";
		}
								
		 if(forward) 					  
			 schemaListSQL+=  "ORDER BY MAJOR_ORDINAL_VERSION, MINOR_ORDINAL_VERSION;";
		 else
			 schemaListSQL+=  "ORDER BY MAJOR_ORDINAL_VERSION DESC, MINOR_ORDINAL_VERSION DESC;";
		 
		 
		ResultSet schemaList = DBUtil.executeSQLQuery(schemaListSQL,conn);
		String schemaVersion = null;
		String schemaVersionMinor = null;
		ICSchema schema = null;

		if(schemaList == null)
			System.out.println("The following query did not return anything: " + schemaListSQL);
		
		try {
			while(schemaList.next()){
				schemaVersion = schemaList.getString("MAJOR_ORDINAL_VERSION");
				schemaVersionMinor = schemaList.getString("MINOR_ORDINAL_VERSION");
				
				schema = new ICSchema(schemaList.getInt("MAJOR_ORDINAL_VERSION"),
									schemaList.getInt("MINOR_ORDINAL_VERSION"),
									schemaName,
									schemaList.getString("DEFAULT_CHARACTER_SET_NAME"),
									schemaList.getString("DEFAULT_COLLATION_NAME"),
									schemaList.getString("SQL_PATH"),
									schemaList.getString("COMMENT"),
									schemaList.getString("SVN_VERSION"),									
									schemaList.getTimestamp("TSTART"),
									schemaList.getTimestamp("TEND")
									);
				schema.setSchemaName(schemaName);
				
				String tableListSQL = "SELECT TABLE_NAME  " +
						      		  "FROM TABLES t " +
						      		  "WHERE  t.SCHEMA_VERSION = '" + schemaVersion + "' AND " +
						      		  		 "t.SCHEMA_VERSION_MINOR = '" +schemaVersionMinor + "' AND " + 
						      		  		 "t.TABLE_SCHEMA = '" + schemaName + "';";
				
				ResultSet tableList = DBUtil.executeSQLQuery(tableListSQL,conn);
				
				String tableName = null;
				String columnQuery = null;
				
					while(tableList.next()){
	
						tableName = tableList.getString("table_name");
						columnQuery = "SELECT  c.COLUMN_NAME, c.COLUMN_TYPE " +
									  "FROM COLUMNS c " +
									  "WHERE  c.SCHEMA_VERSION = '" + schemaVersion + "' AND " +
									  		"c.SCHEMA_VERSION_MINOR = '" + schemaVersionMinor + "' AND " +	
						      		  		"c.TABLE_SCHEMA = '" + schemaName + "' AND "+
									  		"c.TABLE_NAME = '" + tableName + "';"; 
						
						ResultSet columnList = DBUtil.executeSQLQuery(columnQuery,conn);			  		
						ICTable tt = new ICTable(tableName);
						String columnName = null;
						String columnType = null;
						while(columnList.next()){
							columnName = columnList.getString("COLUMN_NAME");
							columnType = columnList.getString("COLUMN_TYPE");		
							tt.addColumn(columnName,columnType);		
						}
						schema.addTable(tt);
						
						// ---- load primary keys ----
  						String primarykeyquery =" SELECT c.TABLE_NAME,COLUMN_NAME,c.CONSTRAINT_NAME "+
										  		" FROM PRISM_KEY_COLUMN_USAGE k, PRISM_TABLE_CONSTRAINTS c, PRISM_VERSIONS v" +
										  		" WHERE v.SCHEMA_UNIQUE_NAME = c.TABLE_SCHEMA" +
										  		" AND k.TABLE_NAME = c.TABLE_NAME" +
										  		" AND k.TABLE_SCHEMA = c.TABLE_SCHEMA" +
										  		" AND k.CONSTRAINT_NAME = c.CONSTRAINT_NAME" +
										  		" AND CONSTRAINT_TYPE = 'PRIMARY KEY'" +
										  		" AND v.MAJOR_ORDINAL_VERSION = '" + schemaVersion + "'" +
												" AND v.MINOR_ORDINAL_VERSION = '" + schemaVersionMinor + "'  " +
										  		" AND v.SCHEMA_NAME = '" + schemaName +"' "+ // changed to use correct fields
												" AND c.TABLE_NAME = '" + tableName + "'" +
												" ORDER BY COLUMN_NAME;"; 
  						  						
  						//System.out.println(primarykeyquery);
						ResultSet constraintslist = DBUtil.executeSQLQuery(primarykeyquery,conn);	
						
						Vector<String> colinkey = new Vector<String>();
						String consname = null;
						while(constraintslist.next()){
							consname = constraintslist.getString("CONSTRAINT_NAME");
							colinkey.add(constraintslist.getString("COLUMN_NAME"));
						}
						if(consname!= null && colinkey.size()>0) 
							tt.addConstraint(new PrimaryKey(consname,(Vector<String>)colinkey.clone()));
						
						//----------------------------
					
						

						// ----- Load Foreign Keys -----
						
						String foreignkeyquery = "SELECT k.CONSTRAINT_NAME,k.REFERENCED_TABLE_NAME,k.COLUMN_NAME,k.REFERENCED_COLUMN_NAME " +
												"FROM `PRISM_REFERENTIAL_CONSTRAINTS` r, PRISM_KEY_COLUMN_USAGE k, PRISM_VERSIONS v " +
												"WHERE v.SCHEMA_UNIQUE_NAME = k.CONSTRAINT_SCHEMA " +
												" AND v.MAJOR_ORDINAL_VERSION = '" + schemaVersion + "'" +
												" AND v.MINOR_ORDINAL_VERSION = '" + schemaVersionMinor + "'  " +
										  		" AND v.SCHEMA_NAME = '" + schemaName +"' "+ // changed to use correct fields
												" AND r.`CONSTRAINT_SCHEMA`=k.`CONSTRAINT_SCHEMA` " +
												" AND r.`CONSTRAINT_NAME` = k.`CONSTRAINT_NAME` " +
												" AND r.`TABLE_NAME` = k.`TABLE_NAME` " +
												" AND r.TABLE_NAME ='" + tableName +"';";
						
						//System.out.println(foreignkeyquery);

						constraintslist = DBUtil.executeSQLQuery(foreignkeyquery,conn);	
						Vector<String> sourcecolinkey = new Vector<String>();
						Vector<String> targetcolinkey = new Vector<String>();

						consname = "";
						String targettab ="";
						String oldtargettab = "";
						boolean first = true;
						
						//Manage multiple FK for one table
						while(constraintslist.next()){
						
							if(first){
								oldtargettab = constraintslist.getString("REFERENCED_TABLE_NAME");
								first = false;
							}
								
							targettab = constraintslist.getString("REFERENCED_TABLE_NAME");
							
							// if we are switching from a FK to the next load the first and reset
							if(!targettab.equals(oldtargettab)){
								if(consname!= null && sourcecolinkey.size()>0 && targetcolinkey.size()>0 ) 
								{	
									ForeignKey fk = new ForeignKey(consname,sourcecolinkey,oldtargettab,targetcolinkey);
									tt.addConstraint(fk);
								}else{
									
									throw new ForeignKeyCreationException("The foreign key stored in the DB is not loadable.");
									
								}
								sourcecolinkey = new Vector<String>();
								targetcolinkey = new Vector<String>();		
								oldtargettab = targettab;
							}
							consname = constraintslist.getString("CONSTRAINT_NAME");
							
							sourcecolinkey.add(constraintslist.getString("COLUMN_NAME"));
							targetcolinkey.add(constraintslist.getString("REFERENCED_COLUMN_NAME"));	
						
							
						
						}
						
						//manage the last step, adding the last FK
						if(consname!= null && sourcecolinkey.size()>0 && targetcolinkey.size()>0 ) 
						{	
							ForeignKey fk = new ForeignKey(consname,sourcecolinkey,targettab,targetcolinkey);
							tt.addConstraint(fk);
						}
						
	
						
						// ----- Load Values Constraints -----

						
						String checkquery = "SELECT t.CONSTRAINT_NAME,CHECK_CLAUSE " +
						"FROM `PRISM_TABLE_CONSTRAINTS` t, PRISM_CHECK_CONSTRAINTS c, PRISM_VERSIONS v  " +
						"WHERE v.SCHEMA_UNIQUE_NAME = t.CONSTRAINT_SCHEMA " +
						" AND v.MAJOR_ORDINAL_VERSION = '" + schemaVersion + "'" +
						" AND v.MINOR_ORDINAL_VERSION = '" + schemaVersionMinor + "'  " +
				  		" AND v.SCHEMA_NAME = '" + schemaName +"' AND "+ // changed to use correct fields
						"t.`CONSTRAINT_SCHEMA`=c.`CONSTRAINT_SCHEMA` AND " +
						"t.`CONSTRAINT_NAME` = c.`CONSTRAINT_NAME` AND " +
						"t.`TABLE_NAME` = c.`TABLE_NAME` AND " +
						"t.TABLE_NAME ='" + tableName +"';";
						
						
						
						constraintslist = DBUtil.executeSQLQuery(checkquery,conn);	
						String clause = "";
						if(constraintslist != null)
						while(constraintslist.next()){
							consname = constraintslist.getString("t.CONSTRAINT_NAME");
							clause = constraintslist.getString("CHECK_CLAUSE");
							ValueConstraint vc = new ValueConstraint(consname,clause);
							tt.addConstraint(vc);
						}
						
						//----------------------------
					
					
					
					}
	
					if(schema.foreignKeyLoop(null)){
						String loop="";
						for (ICTable i:schema.extractLoops()){
							loop+=i.getTableName() + " -> ";
							
						}
						System.err.println("WARNING: The schema you loaded contains a loop of foreign keys: " + loop);
						//throw new ForeignKeyLoopException("The schema you loaded contains a loop of foreign keys: " + loop);
					}
				sh.addSchema(schema);
			}
			
		} catch (SQLException e) {
			e.printStackTrace();
		}
		catch (ExpressionParsignErrorException e) {
			
			e.printStackTrace();
		}catch (IntegrityConstraintsExistsException e) {
			e.printStackTrace();
		}
		return sh;
	}
	
	
}
