package edu.ucla.cs.db.prism.backend.schema.integrityconstraints;

import java.io.StringReader;

import edu.ucla.cs.db.prism.backend.parser.parser.PrimaRParser;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.relational_exp;
import edu.ucla.cs.db.prism.backend.restructure.ExpressionRestructure;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.exceptions.ExpressionParsignErrorException;
import edu.ucla.cs.db.prism.backend.smo.util.Predicate;
import edu.ucla.cs.db.prism.backend.smo.util.Term;

public class ValueConstraint extends IntegrityConstraint {

	private Predicate p;
	/**
	 * 
	 * @param column
	 * @param operator
	 * @param value
	 */
	public ValueConstraint(String id, String table, String column, String operator, String value){
		this.setId(id);
		p = new Predicate();
		
		p.op = operator;
		
		Term t1 = new Term();
		t1.tableName = table;
		t1.columnName = column;
		p.term1 = t1;
		
		Term t2 = new Term();
		t2.constant = value;
		p.term2 = t2;
	}
	
	 
	public ValueConstraint(String id,Predicate pred) throws ComplexValueCosntraintNotImplementedException{
		this.setId(id);
		if(pred.op!=null && pred.term1.columnName!=null && pred.term2.constant!=null)
			this.p = pred.clone();
		else
			throw new ComplexValueCosntraintNotImplementedException("The input predicate represent complex condition not implemented for Value Cosntraints");
	
	}

	public ValueConstraint(ValueConstraint valueConstraint) throws ComplexValueCosntraintNotImplementedException {
		
		this.setId(valueConstraint.getId());
		this.p = valueConstraint.getAsPredicate().clone();
		
	}


	public ValueConstraint(String consname, String clause) throws ExpressionParsignErrorException {
		this.setId(consname);
	
		this.p = createPredicateFromString(clause);
		
	}
	
	
	
	public static Predicate createPredicateFromString(String string) throws ExpressionParsignErrorException {
		StringReader input = null;
		input = new StringReader(string);

		// step 1.2: parse it
		PrimaRParser parser = new PrimaRParser(input);
		//PrimaRScript script = null;
		relational_exp exp = null;
		try {
			exp = parser.relational_exp();
			//script = parser.PrimaRScript();
		} catch(Exception e) {
			e.printStackTrace();
		}
		// step 1.3: restructure abstract syntax tree (AST) into our defined class objects
		
		Predicate ptemp = ExpressionRestructure.restructure(exp);
		return ptemp;
	}
	


	@Override
	public String toString(){
		
		return p.toString();
	}

	/**
	 * @param column the column to set
	 */
	public void setColumn(String column) {
		p.term1.columnName = column;
	}


	/**
	 * Check whether the input column is constrained by this Constraint.
	 * @param column the column to set
	 */
	public boolean isConstrained(String column) {
		return p.term1.columnName.equals(column);
	}

	
	
	/**
	 * @return the column
	 */
	public String getColumn() {
		return p.term1.columnName;
	}


	/**
	 * @param operator the operator to set
	 */
	public void setOperator(String operator) {
		p.op = operator;
	}


	/**
	 * @return the operator
	 */
	public String getOperator() {
		return p.op;
	}


	public String getValue() {
		return p.term2.constant;
	}


	public void setValue(String value) {
		p.term2.constant = value;
	}


	public boolean isComplementary(ValueConstraint vc2) {
		String op1 = p.op;
		String val1 = p.term2.constant;
		String col1 = p.term1.columnName;
		String op2 = vc2.getOperator();
		String val2 = vc2.getValue();
		String col2 = vc2.getColumn();
		
		if(!col1.equals(col2))
			return false;
	
		
		if(val1.compareTo(val2) > 0){
			if(op1.equals("=")){
				if(op2.equals("="))
					return true;
				if(op2.equals(">"))
					return false;
				if(op2.equals("<"))
					return true;
				if(op2.equals(">="))
					return false;
				if(op2.equals("<="))
					return true;
			}
			if(op1.equals(">")){
				if(op2.equals("="))
					return true;
				if(op2.equals("<"))
					return true;
				if(op2.equals(">"))
					return false;	
				if(op2.equals("<="))
					return true;
				if(op2.equals(">="))
					return false;
			}	
			if(op1.equals("<")){
					return false;	
			}	
			
		}	
		
		
		if(val1.compareTo(val2) == 0){
			if(op1.equals("=")){
				if(op2.equals("="))
					return false;
				if(op2.equals(">"))
					return true;
				if(op2.equals("<"))
					return true;
				if(op2.equals(">="))
					return false;
				if(op2.equals("<="))
					return false;
			}
			if(op1.equals(">")){
				if(op2.equals("="))
					return true;
				if(op2.equals("<"))
					return true;
				if(op2.equals(">"))
					return false;	
				if(op2.equals("<="))
					return true;
				if(op2.equals(">="))
					return false;
			}	
			if(op1.equals("<")){
				if(op2.equals("="))
					return true;
				if(op2.equals("<"))
					return false;
				if(op2.equals(">"))
					return true;	
				if(op2.equals("<="))
					return false;
				if(op2.equals(">="))
					return true;	
			}	
		}
			
		if(val1.compareTo(val2) < 0){
			if(op1.equals("=")){
				if(op2.equals("="))
					return true;
				if(op2.equals(">"))
					return true;
				if(op2.equals("<"))
					return false;
				if(op2.equals(">="))
					return true;
				if(op2.equals("<="))
					return false;
			}
			if(op1.equals(">")){
					return false;
			}	
			if(op1.equals("<")){
				if(op2.equals("="))
					return true;
				if(op2.equals("<"))
					return false;
				if(op2.equals(">"))
					return true;	
				if(op2.equals("<="))
					return false;
				if(op2.equals(">="))
					return true;	
			}	
		
		}		
		return false;
	}


	public Predicate getAsPredicate() {
		return p;
	}	
	
    public void setTableName(String tn){
    	p.term1.tableName = tn;
    }

	@Override
	public ValueConstraint clone(){
		
		try {
			return new ValueConstraint(this);
		} catch (ComplexValueCosntraintNotImplementedException e) {
			e.printStackTrace();
		}
		return null;
		
	}
	
	@Override
	public boolean equals(IntegrityConstraint ic){
		
		if(!(ic instanceof ValueConstraint))
			return false;

		if(!p.equals(((ValueConstraint)ic).p))
			return false;
		return true;
				
	}


	public String getTableName() {
		
		return p.term1.tableName;
	}


	public static Predicate negatedPredicate(Predicate pr) throws ComplexValueCosntraintNotImplementedException {

		pr = pr.clone();
		if(pr.term1.columnName!=null && pr.term2.constant!=null){
			if(pr.op.equals("=")){
				pr.op = "!=";
				return pr;
			}	
			if(pr.op.equals("=")){
				pr.op = "!=";
				return pr;
			}				
			if(pr.op.equals("!=")){
				pr.op = "=";
				return pr;
			}			
			if(pr.op.equals(">")){
				pr.op = "<=";
				return pr;
			}	
			if(pr.op.equals("<")){
				pr.op = ">=";
				return pr;
			}	
			if(pr.op.equals(">=")){
				pr.op = "<";
				return pr;
			}	
			if(pr.op.equals("<=")){
				pr.op = ">";
				return pr;
			}				
		}
		throw new ComplexValueCosntraintNotImplementedException("The predicate you are trying to invert is too complex to be inverted at this point.");
	}
	
	
}	
