/**
 * 
 */
package edu.ucla.cs.db.prism.backend.schema.integrityconstraints.test;

import static org.junit.Assert.*;

import java.util.Vector;

import org.junit.Test;

import edu.ucla.cs.db.prism.backend.schema.exception.ColumnExistsException;
import edu.ucla.cs.db.prism.backend.schema.exception.ColumnNotInTableException;
import edu.ucla.cs.db.prism.backend.schema.exception.IllegalAddColumnArgumentException;
import edu.ucla.cs.db.prism.backend.schema.exception.SameColumnNameException;
import edu.ucla.cs.db.prism.backend.schema.exception.TableConstrainedException;
import edu.ucla.cs.db.prism.backend.schema.exception.TableExistsException;
import edu.ucla.cs.db.prism.backend.schema.exception.TableHasNoColumns;
import edu.ucla.cs.db.prism.backend.schema.exception.TableNotInSchemaException;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ForeignKey;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICTable;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.PrimaryKey;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ValueConstraint;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.exceptions.ColumnIsConstrainedException;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.exceptions.InboundForeignKeyOnMergeException;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.exceptions.InboundForeignKeyOnPartitionException;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.exceptions.IntegrityConstraintsExistsException;
import edu.ucla.cs.db.prism.backend.smo.SMOInterface;
import edu.ucla.cs.db.prism.backend.smo.exception.NoForeignKeysDefinedException;
import edu.ucla.cs.db.prism.backend.smo.exception.NoKeyInCommonInfPresException;
import edu.ucla.cs.db.prism.backend.smo.exception.NoPrimaryKeysDefinedException;
import edu.ucla.cs.db.prism.backend.smo.exception.noColumnsMatchInfPres;
import edu.ucla.cs.db.prism.backend.smo.exception.noCommonColumnInfPres;
import edu.ucla.cs.db.prism.backend.smo.exception.notInfPresException;
import edu.ucla.cs.db.prism.backend.smo.util.SMOFactory;
import edu.ucla.cs.db.prism.backend.smo.util.SMOTypeUnknownException;

/**
 * @author Carlo A. Curino (carlo@curino.us)
 *
 */
public class ICSchemaTest {

	ICSchema testschema = null;
	boolean verbose = false;
	
	public void init(){
		
	    testschema = new ICSchema(0, 0, "testschema","", "","","", "0",null,null);
		
		ICTable test1 = new ICTable("test1");
		test1.addColumn("k1");
		test1.addColumn("k2");
		test1.addColumn("a");
		test1.addColumn("b");
		test1.addColumn("c");
		Vector<String> keys1 = new Vector<String>();
		keys1.add("k1");
		keys1.add("k2");
		try {
			test1.addConstraint(new PrimaryKey("pk1",keys1));
		} catch (IntegrityConstraintsExistsException e2) {
			e2.printStackTrace();
		}
		
		ICTable test2 = new ICTable("test2");
		test2.addColumn("k1");
		test2.addColumn("a");
		test2.addColumn("b");
		test2.addColumn("c");
		test2.addColumn("fk1");
		test2.addColumn("fk2");
		
		// add a primary key
		Vector<String> keys2 = new Vector<String>();
		keys2.add("k1");
		try {
			test2.addConstraint(new PrimaryKey("pk2",keys2));
		} catch (IntegrityConstraintsExistsException e2) {
		
			e2.printStackTrace();
		}
		
		// add a foreign key to table test2
		Vector<String> foreignkey2 = new Vector<String>();
		foreignkey2.add("fk1");
		foreignkey2.add("fk2");
		Vector<String> ref2 = new Vector<String>();
		ref2.add("k1");
		ref2.add("k2");
		try {
			test2.addConstraint(new ForeignKey("test2fk1", foreignkey2, "test1", ref2));
		} catch (IntegrityConstraintsExistsException e1) {
			e1.printStackTrace();
		}
		
		ICTable test3 = new ICTable("test3");
		test3.addColumn("a");
		test3.addColumn("b");
		test3.addColumn("c");
		
		Vector<String> foreignkey3 = new Vector<String>();
		foreignkey3.add("c");
		Vector<String> ref3 = new Vector<String>();
		ref3.add("c");
		try {
			test3.addConstraint(new ForeignKey("test3fk1", foreignkey3, "test1", ref3));
		} catch (IntegrityConstraintsExistsException e) {
			e.printStackTrace();
		}
		
		
		ICTable tabletojoin = new ICTable("tabletojoin");
		tabletojoin.addColumn("d");
		tabletojoin.addColumn("e");
		tabletojoin.addColumn("f");
		tabletojoin.addColumn("fk1");
		tabletojoin.addColumn("fk2");
		
		// add a primary key
		Vector<String> tabletojoinkeys2 = new Vector<String>();
		tabletojoinkeys2.add("fk1");
		tabletojoinkeys2.add("fk2");
		try {
			tabletojoin.addConstraint(new PrimaryKey("pkttj",tabletojoinkeys2));
		} catch (IntegrityConstraintsExistsException e2) {
			e2.printStackTrace();
		}
		
		// add a foreign key to table test2
		Vector<String> foreignkeytojoin2 = new Vector<String>();
		foreignkeytojoin2.add("fk1");
		foreignkeytojoin2.add("fk2");
		Vector<String> reftojoin2 = new Vector<String>();
		reftojoin2.add("k1");
		reftojoin2.add("k2");
		try {
			tabletojoin.addConstraint(new ForeignKey("tabletojoinfk1", foreignkey2, "test1", ref2));
		} catch (IntegrityConstraintsExistsException e1) {
			e1.printStackTrace();
		}
		
		
		
		ICTable tabletomerge = new ICTable("tabletomerge");
		tabletomerge.addColumn("k1");
		tabletomerge.addColumn("k2");
		tabletomerge.addColumn("a");
		tabletomerge.addColumn("b");
		tabletomerge.addColumn("c");
		
		// add a primary key
		Vector<String> tabletomergefkeys2 = new Vector<String>();
		tabletomergefkeys2.add("k1");
		tabletomergefkeys2.add("k2");
		try {
			tabletomerge.addConstraint(new PrimaryKey("pkttm",tabletomergefkeys2));
		} catch (IntegrityConstraintsExistsException e2) {
			e2.printStackTrace();
		}
		
		testschema.addTable(test1);
		testschema.addTable(test2);
		testschema.addTable(test3);
		testschema.addTable(tabletojoin);
		testschema.addTable(tabletomerge);
	}
	
	private void setMergeValueConstraints(int i) throws IntegrityConstraintsExistsException {
		if(i==0){
			testschema.getTable("test1").addConstraint(new ValueConstraint("vct1","test1", "k1",">","0"));	
			testschema.getTable("tabletomerge").addConstraint(new ValueConstraint("vcttm","tabletomerge", "k1","<","0"));	
		}
		if(i==1){
			testschema.getTable("test1").addConstraint(new ValueConstraint("vct1","test1", "k2",">","5"));	
			testschema.getTable("tabletomerge").addConstraint(new ValueConstraint("vcttm","tabletomerge","k2","<","3"));	
		}
		if(i==2){
			testschema.getTable("test1").addConstraint(new ValueConstraint("vct1","test1", "k1","=","0"));	
			testschema.getTable("tabletomerge").addConstraint(new ValueConstraint("vcttm","tabletomerge","k1","=","1"));	
		}
		if(i==3){
			testschema.getTable("test1").addConstraint(new ValueConstraint("vct1","test1", "k1","<","0"));	
			testschema.getTable("tabletomerge").addConstraint(new ValueConstraint("vcttm","tabletomerge","k1",">=","0"));	
		}
	}
	
	private void initMergeOutboungGoodFK(){
		// add a foreign key to table test1 and tabletomerge
		Vector<String> foreignkey2 = new Vector<String>();
		foreignkey2.add("k1");
		foreignkey2.add("k2");
		
		
		Vector<String> foreignkeytomerge2 = new Vector<String>();
		Vector<String> ref2 = new Vector<String>();
		ref2 .add("fk1");
		ref2.add("fk2");
		try {
			testschema.getTable("test1").addConstraint(new ForeignKey("test1fk1", foreignkey2, "test2", ref2));
			testschema.getTable("tabletomerge").addConstraint(new ForeignKey("tabletomergefk1", foreignkey2, "test2", ref2));	
		} catch (IntegrityConstraintsExistsException e1) {
			e1.printStackTrace();
		}
	}
	

	private void initjoinFK() {
		// add a foreign key to table test1
		Vector<String> foreignkey2= new Vector<String>();
		foreignkey2.add("k1");
		foreignkey2.add("k2");
		Vector<String> ref2 = new Vector<String>();
		ref2.add("fk1");
		ref2.add("fk2");
		try {
			testschema.getTable("test1").addConstraint(new ForeignKey("test2fk1", foreignkey2, "tabletojoin", ref2));
		} catch (IntegrityConstraintsExistsException e1) {
			e1.printStackTrace();
		}		
	}
	
	/**
	 * Test method for {@link edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema#applyNopSMO(edu.ucla.cs.db.prism.backend.smo.SMOImpl)}.
	 */
	@Test
	public void testApplyNopSMO() {
		init();
		try {
			testschema.applyNopSMO(SMOFactory.createSMO(SMOInterface.NOP));
		} catch (SMOTypeUnknownException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Test method for {@link edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema#applyDropColumnSMO(edu.ucla.cs.db.prism.backend.smo.SMOImpl)}.
	 */
	@Test
	public void testApplyDropColumnSMO() {
		
		init();
		
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("DROP COLUMN randomname FROM test1"));
			fail("column should have not be removed because doesn't exist");
		} 
		catch(ColumnNotInTableException c){
			System.out.println("Test OK: randomanme doesn't exist");
		}
		catch (Exception e) {
			e.printStackTrace();
		}
			
		init();
		
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("DROP COLUMN k1 FROM test1"));
			fail("column should have not be removed because is key");
		} catch(ColumnIsConstrainedException c){
			System.out.println("Test OK: k1 is constrained and should not be removed");
		}
		catch (Exception e) {
			e.printStackTrace();
		}
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("DROP COLUMN c FROM test1"));
			fail("column should have not be removed because is target of foreign key");
		}catch(ColumnIsConstrainedException c){
			System.out.println("Test OK: k1 is constrained and should not be removed");
		} 
		catch (Exception e) {
			e.printStackTrace();
		}
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("DROP COLUMN a FROM test1"));
			if(testschema.getTable("test1").getColumnByName("a")!=null)
				fail("Column not removed while it should have");
		} catch (Exception e) {
			System.err.println(testschema.toString());
			e.printStackTrace();
			fail("Something went wrong when removing column, it should have been removed nicely");
		}	
	}

	/**
	 * Test method for {@link edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema#applyCreateTableSMO(edu.ucla.cs.db.prism.backend.smo.SMOImpl)}.
	 */
	@Test
	public void testApplyCreateTableSMO() {
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("CREATE TABLE test1(c,d,e)"));
			fail("Table test1 should not be added since already exist");
		} catch(TableExistsException t){
			System.out.println("Test OK: the table test1 already exists and cannot be added");
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something went wrong when trying to create the table");
		}	
		
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("CREATE TABLE test4(c,d,e)"));
			if(testschema.getTable("test4") == null)
				fail("Table test4 not added!");
			else
				System.out.println("Test OK: the table test4 added already exists and cannot be added");
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}	
		
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("CREATE TABLE test4(c,c,e)"));
			if(testschema.getTable("test4") != null)
				fail("Table test4 added while it should have failed!");
		}catch (ColumnExistsException c) {
			System.out.println("Test OK: the table test4 has duplicated columns and should not be added");
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}	
		
		
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("CREATE TABLE test4"));
			if(testschema.getTable("test4") != null)
				fail("Table test4 added while it should have failed!");
		}catch (TableHasNoColumns c) {
			System.out.println("Test OK: the table test4 has no columns and should not be added");
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}	
		
		
		
	}

	/**
	 * Test method for {@link edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema#applyDropTableSMO(edu.ucla.cs.db.prism.backend.smo.SMOImpl)}.
	 */
	@Test
	public void testApplyDropTableSMO() {
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("DROP TABLE test1"));
			if(testschema.getTable("test1")!=null)
				fail("Table removed while it should have not");
		} catch(TableConstrainedException t){
			System.out.println("Test OK: the table test1 is target of FK therefore is not removed");
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something went wrong when trying to remove table");
		}	
		
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("DROP TABLE test2"));
			if(testschema.getTable("test2")==null)
				System.out.println("Test OK: the table test2 has been removed");
		} catch(TableConstrainedException t){
			fail("The table is not constrained, while the system says it is.");
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something went wrong when trying to remove table");
		}	
	}
	

	/**
	 * Test method for {@link edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema#applyDropTableSMO(edu.ucla.cs.db.prism.backend.smo.SMOImpl)}.
	 */
	@Test
	public void testApplyAddColumnSMO() {
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("ADD COLUMN c INTO test7"));
			fail("Column c added to non existing table test7 while it should have failed");
		} catch(TableNotInSchemaException t){
			System.out.println("Test OK: the table test7 doesn't exists");
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something went wrong when trying to add column");
		}	
		
		
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("ADD COLUMN c INTO test1"));
			fail("Existing column c added to table test1. It should have failed");
		} catch(ColumnExistsException t){
			System.out.println("Test OK: the table test1 already has a column c");
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something went wrong when trying to add column");
		}	
		
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("ADD COLUMN d AS funct(g) INTO test1"));
			fail("Improper function accepted. It should have failed");
		} catch(IllegalAddColumnArgumentException t){
			System.out.println("Test OK: the table test1 does not have  g column used as argument by the SMO");
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something went wrong when trying to add column");
		}	
		
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("ADD COLUMN d AS funct(b,c) INTO test1"));
			if(testschema.getTable("test1").getColumnByName("d")==null)
				fail("Column should have been added");
		} 
		catch (Exception e) {
			e.printStackTrace();
			fail("Something went wrong when trying to add column");
		}
		
	}

	
	/**
	 * Test method for {@link edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema#applyRenameTableSMO(edu.ucla.cs.db.prism.backend.smo.SMOImpl)}.
	 */
	@Test
	public void testApplyRenameTableSMO(){
		
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("RENAME TABLE test1 INTO test2"));
			fail("Error, we are trying to make an illegal rename, this should have generated exception.");
		} catch(TableExistsException c){
			System.out.println("Test OK: the rename was illegal and generated the correct exception");
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}	
		
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("RENAME TABLE test4 INTO test5"));
			fail("Error, we are trying to make an illegal rename, this should have generated exception.");
		} catch(TableNotInSchemaException c){
			System.out.println("Test OK: the rename was illegal and generated the correct exception");
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}	
		
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("RENAME TABLE test1 INTO test4"));
			if(testschema.getTable("test1")!=null || testschema.getTable("test4")==null)
				fail("Error, correct rename did not succeed.");
			if(testschema.getTable("test2").getForeignKeyTargetting("test4")==null)
				fail("Error, fix of foreign key after rename did not succeed.");
			if(testschema.getTable("test2").getForeignKeyTargetting("test1")!=null)
				fail("Error, fix of foreign key after rename did not succeed.");			
			
		} catch(TableNotInSchemaException c){
			System.out.println("Test OK: the rename was illegal and generated the correct exception");
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}
		
		
		
	}
	
	/**
	 * Test method for {@link edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema#applyRenameColumnSMO(edu.ucla.cs.db.prism.backend.smo.SMOImpl)}.
	 */
	@Test	
	public void testApplyRenameColumnSMO(){

		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("RENAME COLUMN k3 IN  test1 TO k4"));
			fail("Error, we are trying to make an illegal rename, this should have generated exception.");
		} catch(ColumnNotInTableException c){
			System.out.println("Test OK: the rename was illegal and generated the correct exception");
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}	

		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("RENAME COLUMN k1 IN  test5 TO k2"));
			fail("Error, we are trying to make an illegal rename, this should have generated exception.");
		} catch(TableNotInSchemaException c){
			System.out.println("Test OK: the rename was illegal and generated the correct exception");
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}	
		
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("RENAME COLUMN k1 IN  test1 TO k2"));
			fail("Error, we are trying to make an illegal rename, this should have generated exception.");
		} catch(ColumnExistsException c){
			System.out.println("Test OK: the rename was illegal and generated the correct exception");
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}	
		
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("RENAME COLUMN k1 IN  test1 TO k3"));
			if(testschema.getTable("test1").getColumnByName("k3") == null || testschema.getTable("test1").getColumnByName("k1")!=null)
				fail("The column has not been renamed correctly, column name not changed");
			if(!testschema.getTable("test1").isInPrimaryKey("k3"))
				fail("The column has not been renamed correctly, primary key not changed");
			if(testschema.getTable("test1").isInPrimaryKey("k1"))
				fail("The column has not been renamed correctly, primary key not changed");
			
			Vector<ForeignKey> fk = testschema.getTable("test2").getForeignKeyTargetting("test1");
			if(!fk.elementAt(0).getTargetColumns().elementAt(0).equals("k3"))
				fail("The column has not been renamed correctly, external foreign key not changed");
			
			System.out.println("Test OK: rename column suceeded");
			if(verbose)
				System.out.println("\n" + testschema);
		
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}	
		
		
	}
	
	
	/**
	 * Test method for {@link edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema#testApplyCopyTableSMO(edu.ucla.cs.db.prism.backend.smo.SMOImpl)}.
	 */
	@Test	
	public void testApplyCopyTableSMO(){

		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("COPY TABLE test1 INTO test2"));
			fail("Error, we are trying to make an illegal copy, this should have generated exception.");
		} catch(TableExistsException c){
			System.out.println("Test OK: the copy was illegal and generated the correct exception");
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}
		
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("COPY TABLE test4 INTO test5"));
			fail("Error, we are trying to make an illegal copy, this should have generated exception.");
		} catch(TableNotInSchemaException c){
			System.out.println("Test OK: the copy was illegal and generated the correct exception");
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}

		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("COPY TABLE test1 INTO test4"));
			if(testschema.getTable("test1")==null || testschema.getTable("test4")==null)
				fail("Error, something did not work during copy, tables are missing.");
			else{ 
				testschema.getTable("test1").addConstraint(new ForeignKey("fk4",null,"test1",null));
				if(!testschema.getTable("test1").hasSameSignature(testschema.getTable("test4")) || !testschema.getTable("test1").hasSameConstraints(testschema.getTable("test4")))
					System.out.println("Test OK: the system correctly spotted differences between tables, introduced as a hack");
				else
					fail("The copied table differs from the original one");
			}
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}

		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("COPY TABLE test1 INTO test4"));
			if(testschema.getTable("test1")==null || testschema.getTable("test4")==null)
				fail("Error, something did not work during copy, tables are missing.");
			else{ 
				//testschema.getTable("test1").addConstraint(new ForeignKey("fk4",null,"test1",null));
				if(!testschema.getTable("test1").hasSameSignature(testschema.getTable("test4")))
					fail("The copied table signature differs from the original one");

				if(!testschema.getTable("test1").hasSameConstraints(testschema.getTable("test4")))
					fail("The copied table constraints differ from the original one");
				
					System.out.println("Test OK: copy succeeded");
					if(verbose)
						System.out.println("\n" + testschema);
				
			}			
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}
		
	}
	
	
	/**
	 * Test method for {@link edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema#testApplyDecomposeTableSMO(edu.ucla.cs.db.prism.backend.smo.SMOImpl)}.
	 */
	@Test	
	public void testApplyDecomposeTableSMO(){

		// testing wrong cases
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("DECOMPOSE TABLE test2 INTO t1(k1,k2,a,b),t2(k1,k2,c)"));
			fail("Error, we are trying to make an illegal decompose, this should have generated exception.");
		}catch(ColumnNotInTableException c){
			System.out.println("Test OK: the decompose failed correctly");
		} 
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}

		// testing wrong cases
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("DECOMPOSE TABLE test5 INTO t1(k1,k2,a,b),t2(k1,k2,c)"));
			fail("Error, we are trying to make an illegal decompose, this should have generated exception.");
		}catch(TableNotInSchemaException c){
			System.out.println("Test OK: the decompose failed correctly");
		} 
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}
		
		// testing wrong cases
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("DECOMPOSE TABLE test1 INTO test2(k1,k2,a,b),t3(k1,k2,c)"));
			fail("Error, we are trying to make an illegal decompose, this should have generated exception.");
		}catch(TableExistsException c){
			System.out.println("Test OK: the decompose failed correctly");
		} 
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}
		
		// testing wrong cases
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("DECOMPOSE TABLE test1 INTO t2(k1,k2,a,b),t3(c)"));
			fail("Error, we are trying to make an illegal decompose, this should have generated exception.");
		}catch(noCommonColumnInfPres c){
			System.out.println("Test OK: the decompose failed correctly");
		}catch(NoKeyInCommonInfPresException nk){
			System.out.println("Test OK: the decompose failed correctly");			
		} 
			
		
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}
		
		// testing wrong cases
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("DECOMPOSE TABLE test1 INTO t2(k1,k2,a,b),t3(k2,c)"));
			fail("Error, we are trying to make an illegal decompose, this should have generated exception.");
		}catch(NoKeyInCommonInfPresException c){
			System.out.println("Test OK: the decompose failed correctly");
		} 
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}
		
		// testing wrong cases
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("DECOMPOSE TABLE test1 INTO t2(k1,k2,a),t3(k1,k2,c)"));
			fail("Error, we are trying to make an illegal decompose, this should have generated exception.");
		}catch(noColumnsMatchInfPres c){
			System.out.println("Test OK: the decompose failed correctly");
		} 
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}
		
		// correct case
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("DECOMPOSE TABLE test1 INTO t1(k1,k2,a,b),t2(k1,k2,c)"));
			if(testschema.getTable("t1")!=null && 
			   testschema.getTable("t2")!=null && 
			   testschema.getTable("test1") == null &&
			   testschema.getTable("t1").getForeignKeyTargetting("t2").elementAt(0).isSourceColumn("k1") &&
			   testschema.getTable("t1").getForeignKeyTargetting("t2").elementAt(0).isSourceColumn("k2") &&
			   testschema.getTable("t1").getForeignKeyTargetting("t2").elementAt(0).isTargetColumn("k1") &&
			   testschema.getTable("t1").getForeignKeyTargetting("t2").elementAt(0).isTargetColumn("k2") &&
			   testschema.getTable("t2").getForeignKeyTargetting("t1").elementAt(0).isSourceColumn("k1") &&
			   testschema.getTable("t2").getForeignKeyTargetting("t1").elementAt(0).isSourceColumn("k2") &&
			   testschema.getTable("t2").getForeignKeyTargetting("t1").elementAt(0).isTargetColumn("k1") &&
			   testschema.getTable("t2").getForeignKeyTargetting("t1").elementAt(0).isTargetColumn("k2")){
						System.out.println("Test OK: the decompose has been applied correctly.");
						if(verbose)
							System.out.println("\n" + testschema);
			}	
			else
				fail("Error, we are trying to make an illegal decompose, this should have generated exception.");
		} 
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}
	}
	
	/**
	 * Test method for {@link edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema#testApplyJoinTableSMO(edu.ucla.cs.db.prism.backend.smo.SMOImpl)}.
	 */
	@Test	
	public void testApplyJoinTableSMO(){

		// testing failing cases
		init();
		initjoinFK();
		//induce an error of 
		testschema.getTable("tabletojoin").addColumn("a");

		try {
			testschema.applySMO(SMOFactory.createSMOFromString("JOIN TABLE test1,tabletojoin INTO test4 WHERE test1.k1=tabletojoin.fk1 AND test1.k2=tabletojoin.fk2"));
			fail("Error, we are trying to make an illegal JOIN, this should have generated exception.");
		}catch(SameColumnNameException c){
			System.out.println("Test OK: the JOIN failed correctly");
		} 
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}
		
		
		// testing failing cases
		init();
		initjoinFK();
	

		try {
			testschema.applySMO(SMOFactory.createSMOFromString("JOIN TABLE test1,tabletojoin INTO test3 WHERE test1.k1=test2.fk1 AND test1.k2=test2.fk2"));
			fail("Error, we are trying to make an illegal JOIN, this should have generated exception.");
		}catch(TableExistsException c){
			System.out.println("Test OK: the JOIN failed correctly");
		} 
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}

		
		// testing failing cases
		init();
		
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("JOIN TABLE test1,tabletojoin INTO joinedtable WHERE test1.k1=tabletojoin.fk1 AND test1.k2=tabletojoin.fk2"));
			fail("Error, we are trying to make an illegal JOIN, this should have generated exception.");
		}catch(NoForeignKeysDefinedException n){
			System.out.println("Test OK: the JOIN failed correctly");			
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}
		
		// testing failing cases
		init();
		initjoinFK();
		testschema.getTable("test1").removePrimaryKey();
		
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("JOIN TABLE test1,tabletojoin INTO joinedtable WHERE test1.k1=tabletojoin.fk1 AND test1.k2=tabletojoin.fk2"));
			fail("Error, we are trying to make an illegal JOIN, this should have generated exception.");
		}catch(NoPrimaryKeysDefinedException n){
			System.out.println("Test OK: the JOIN failed correctly");			
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}
		
		// testing failing cases
		init();
		initjoinFK();
		
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("JOIN TABLE test1,tabletojoin INTO joinedtable WHERE test1.k1=tabletojoin.fk1 AND test1.k2=tabletojoin.fk2"));
			if(testschema.getTable("test1")!=null || 
			   testschema.getTable("tabletojoin")!=null || 
			   testschema.getTable("joinedtable")== null)
				fail("The JOIN was supposed to work properly, but some of the tables was not generated or some input table not removed.");

			if(!testschema.getTable("test2").getForeignKeyTargetting("joinedtable").elementAt(0).isSourceColumn("fk1") ||
			   !testschema.getTable("test2").getForeignKeyTargetting("joinedtable").elementAt(0).isSourceColumn("fk2") ||
			   !testschema.getTable("test2").getForeignKeyTargetting("joinedtable").elementAt(0).isTargetColumn("k1") ||
			   !testschema.getTable("test2").getForeignKeyTargetting("joinedtable").elementAt(0).isTargetColumn("k2"))
				fail("The JOIN was supposed to work properly, but the inbound fk where not fixed properly.");
			
			if(!testschema.getTable("joinedtable").getPrimaryKey().isKey("k1") || !testschema.getTable("joinedtable").getPrimaryKey().isKey("k2"))
				fail("The JOIN was supposed to work properlu, but the primary key was not adapted properly.");
			System.out.println("The JOIN has been applied correctly");
			if(verbose)
				System.out.println("\n" + testschema);
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}
		
		
	}


	/**
	 * Test method for {@link edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema#testApplyMergeTableSMO(edu.ucla.cs.db.prism.backend.smo.SMOImpl)}.
	 */
	@Test	
	public void testApplyMergeTableSMO(){

		// testing failing cases
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("MERGE TABLE test1,tabletomerge INTO test2"));
			fail("Error, we are trying to make an illegal MERGE, this should have generated exception.");
		}catch(TableExistsException c){
			System.out.println("Test OK: the MERGE failed correctly");
		} 
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}
		
		// testing correct cases
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("MERGE TABLE test1,tabletomerge INTO test1"));
		}catch(notInfPresException c){
			System.out.println("Test OK: the MERGE failed correctly");
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}

		
		try {
			setMergeValueConstraints(0);
		} catch (IntegrityConstraintsExistsException e1) {
			fail("wrong value constraint added during setup");
			e1.printStackTrace();
		}
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("MERGE TABLE test1,tabletomerge INTO test5"));
		}catch(InboundForeignKeyOnMergeException i){
			System.out.println("Test OK: the MERGE failed correctly");
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}
		
		// testing failing cases
		for(int i = 0; i<=3;i++){
			init();
			try {
				setMergeValueConstraints(i);
				testschema.getTable("test2").removeForeignKeysByTarget("test1");
				testschema.getTable("test3").removeForeignKeysByTarget("test1");
				testschema.getTable("tabletojoin").removeForeignKeysByTarget("test1");
			} catch (IntegrityConstraintsExistsException e1) {
				fail("wrong value constraint added during setup");
				e1.printStackTrace();
			}
			try {
				testschema.applySMO(SMOFactory.createSMOFromString("MERGE TABLE test1,tabletomerge INTO test5"));
				//more checks
			}
			catch (Exception e) {
				e.printStackTrace();
				fail("Something unexpected went wrong when trying to create the table");
			}
		}
		// testing failing cases
		init();
		try {
			setMergeValueConstraints(0);
			testschema.getTable("test2").removeForeignKeysByTarget("test1");
			testschema.getTable("test3").removeForeignKeysByTarget("test1");
			testschema.getTable("tabletojoin").removeForeignKeysByTarget("test1");

		} catch (IntegrityConstraintsExistsException e1) {
			fail("wrong value constraint added during setup");
			e1.printStackTrace();
		}
		initMergeOutboungGoodFK();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("MERGE TABLE test1,tabletomerge INTO test5"));
			if(testschema.getTable("test1")!=null || testschema.getTable("tabletomerge")!=null || testschema.getTable("test5")==null)
				fail("The MERGE succeeded, but the tables have not been added/removed correctly!");
			
			if(testschema.getTable("test5")!=null)
				if(!testschema.getTable("test5").isInPrimaryKey("k1") || !testschema.getTable("test5").isInPrimaryKey("k2"))
					fail("The MERGE succeeded, but the primary key has not been propagated correctly");
			if(testschema.getTable("test5")!=null)
				if(!testschema.getTable("test5").isSourceofForeignKey("k1") || !testschema.getTable("test5").isSourceofForeignKey("k2"))
					fail("The MERGE succeeded, but the primary key has not been propagated correctly");
			
			System.out.println("Test Ok: the MERGE has been applied correctly");
			if(verbose)
				System.out.println("\n" + testschema);
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}
	}

	/**
	 * Test method for {@link edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema#testApplyPartitionTableSMO(edu.ucla.cs.db.prism.backend.smo.SMOImpl)}.
	 */
	@Test	
	public void testApplyPartitionTableSMO(){

		// testing failing cases
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("PARTITION TABLE test1 INTO test3 WITH test1.a>\"0\" ,part2 WITH test1.a<\"0\""));
			fail("Error, we are trying to make an illegal MERGE, this should have generated exception.");
		}catch(TableExistsException c){
			System.out.println("Test OK: the PARTITION failed correctly");
		} 
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}
		
		// testing correct cases
		init();
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("PARTITION TABLE test1 INTO part1 WITH test1.a>\"0\", part2 WITH test1.a>\"3\""));
		}catch(notInfPresException c){
			System.out.println("Test OK: the PARTITION failed correctly");
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}

		
		try {
			testschema.applySMO(SMOFactory.createSMOFromString("PARTITION TABLE test1 INTO part1 WITH test1.a>\"0\", part2 WITH test1.a<=\"0\""));
		}catch(InboundForeignKeyOnPartitionException i){
			System.out.println("Test OK: the PARTITION failed correctly");
		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}
		
		// testing failing cases
		init();
		testschema.getTable("test2").removeForeignKeysByTarget("test1");
		testschema.getTable("test3").removeForeignKeysByTarget("test1");
		testschema.getTable("tabletojoin").removeForeignKeysByTarget("test1");
		initjoinFK();
		try {
			
			testschema.applySMO(SMOFactory.createSMOFromString("PARTITION TABLE test1 INTO part1 WITH test1.a=\"0\", part2 WITH test1.a!=\"0\""));
			if(testschema.getTable("test1")!=null || testschema.getTable("part1")==null || testschema.getTable("part2")==null)
				fail("The PARTITION succeeded, but the tables have not been added/removed correctly!");
			
			if(testschema.getTable("part1")!=null)
				if(!testschema.getTable("part1").isInPrimaryKey("k1") || 
				   !testschema.getTable("part1").isInPrimaryKey("k2") || 
				   !testschema.getTable("part2").isInPrimaryKey("k1") || 
				   !testschema.getTable("part2").isInPrimaryKey("k2"))
						fail("The PARTITION succeeded, but the primary key has not been propagated correctly");
			
			if(testschema.getTable("part1")!=null)
				if(!testschema.getTable("part1").isSourceofForeignKey("k1") || 
				   !testschema.getTable("part1").isSourceofForeignKey("k2") ||
				   !testschema.getTable("part2").isSourceofForeignKey("k1") || 
				   !testschema.getTable("part2").isSourceofForeignKey("k2")  )
					fail("The PARTITION succeeded, but the primary key has not been propagated correctly\n\n" + testschema);
			System.out.println("Test Ok: the PARTITION has been applied correctly");
			if(verbose)
				System.out.println("\n" + testschema);

		}
		catch (Exception e) {
			e.printStackTrace();
			fail("Something unexpected went wrong when trying to create the table");
		}
	}



}
