/**
 * 
 */
package edu.ucla.cs.db.prism.backend.smo;

import java.util.Iterator;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.ded.DED;
import edu.ucla.cs.db.prism.backend.ded.DEDSet;
import edu.ucla.cs.db.prism.backend.ded.QuantifiedDisjunctive;
import edu.ucla.cs.db.prism.backend.ded.Quantifier;
import edu.ucla.cs.db.prism.backend.marsManage.exception.MarsConstantsSuffixException;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.Table;
import edu.ucla.cs.db.prism.backend.schema.exception.IllegalSMOException;
import edu.ucla.cs.db.prism.backend.smo.util.Conjunctive;
import edu.ucla.cs.db.prism.backend.smo.util.Predicate;
import edu.ucla.cs.db.prism.backend.smo.util.SMOFactory;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequence;
import edu.ucla.cs.db.prism.backend.smo.util.SMOTypeUnknownException;
import edu.ucla.cs.db.prism.backend.smo.util.Term;
import edu.ucla.cs.db.prism.backend.util.*;

/**
 * @author krl
 *
 */
public class AddColumnSMO extends SMOImpl {
	
	/**
	 * 
	 */
	public AddColumnSMO() {
		super("ADD COLUMN");
		setHasPerfectInverse(true);
		this.setColumnType("varchar(255)");
		setComplete(true);
	}

	public String toString() {
		String str = getOperatorType() + " ";	
		str += getColumnName();
		
		str += " " + getColumnType();
		
		if(getDefaultConstant() != null)
			str += " AS " + getDefaultConstant();
		else if(getFunctionName() != null) {
			str += " AS " + getFunctionName() + "(";
			if(getFunctionArgs() != null) {
				for(int i=0; i<getFunctionArgs().size(); i++) {
					str += getFunctionArgs().get(i);
					if(i<getFunctionArgs().size()-1)
						str += ", ";
				}
			}
			str += ")";
		}
		str += " INTO " + getTableName();
		return str + "\n";
	}
	
	
	public Vector<String> getModifiedInputSchemaParts() {
		Vector<String> schemaParts = new Vector<String>();
		// no modified input
		return schemaParts;	
	}	
	
	public Vector<String> getModifiedExistingTables(Schema queryFootprintTargetSchema) {
		Vector<String> modifiedExistingTables = new Vector<String>();
		modifiedExistingTables.add(getTableName());
		return modifiedExistingTables;
	}
	
	public SMOSequence getInverse(){
		SMOImpl newSMO = null;
		try {
			newSMO = SMOFactory.createSMO("DROP COLUMN");
		} catch (SMOTypeUnknownException e) {
			e.printStackTrace();
		}
		newSMO.setOperatorType(SMOInterface.DROP_COL);
		newSMO.setTableName(this.getTableName());
		newSMO.setColumnName(this.getColumnName());
		SMOSequence ss = new SMOSequence();
		ss.add(newSMO);
		
		// switch source and target and set them in the answer
		ss.sourceVersion=this.getTargetMajor();
		ss.targetVersion=this.getSourceMajor();
		newSMO.setSourceMajor(this.getTargetMajor());
		newSMO.setSourceMinor(this.getTargetMinor());
		newSMO.setTargetMajor(this.getSourceMajor());
		newSMO.setTargetMinor(this.getSourceMinor());
		return ss;
	}
	
	
	public Vector<SMOSequence> getCandidateInverses(Schema inputSchema) {
		SMOImpl newSMO = null;
		try {
			newSMO = SMOFactory.createSMO("DROP COLUMN");
		} catch (SMOTypeUnknownException e) {
			e.printStackTrace();
		}
		newSMO.setOperatorType(SMOInterface.DROP_COL);
		newSMO.setTableName(this.getTableName());
		newSMO.setColumnName(this.getColumnName());
		SMOSequence ss = new SMOSequence();
		ss.add(newSMO);
		
		// switch source and target and set them in the answer
		ss.sourceVersion=this.getTargetMajor();
		ss.targetVersion=this.getSourceMajor();
		newSMO.setSourceMajor(this.getTargetMajor());
		newSMO.setSourceMinor(this.getTargetMinor());
		newSMO.setTargetMajor(this.getSourceMajor());
		newSMO.setTargetMinor(this.getSourceMinor());
		
		Vector<SMOSequence> v = new Vector<SMOSequence>();
		v.add(ss);
		return v;
	}

	public DEDSet getAllDEDs(Schema inputSchema){
		
		Vector<String> excludeList = new Vector<String>();
		excludeList.add(getTableName());
		DEDSet ds = getDEDs(inputSchema.clone());
		ds.addAll(getIdentityDEDs(inputSchema,excludeList));
		
		return ds;
	}
	
	@Override
	public Vector<String> getIdentitySQLViews(Schema inputSchema, String dbname) {
	
		Vector<String> excludeList = new Vector<String>();
		excludeList.add(getTableName());
		
		return this.getIdentitySQLView(inputSchema, dbname, excludeList);
	}
	
	
	public DEDSet getDEDs(Schema inputSchema) {
		DEDSet ds = new DEDSet(getSourceMajor(),getSourceMinor(),getTargetMajor(), getTargetMinor());
		
		String dedId = null;
		String dedName = removeSpaces(getOperatorType()) + "_" + getTableName() + "_" + getColumnName();
		
		Vector<String> sourceTables = new Vector<String>();
		sourceTables.add(getTableName());
		Vector<String> targetTables = new Vector<String>();
		targetTables.add(getTableName());
	
		// DATA MIGRATION DED
		DED d = new DED(dedId, dedName + "_dm", true, sourceTables, targetTables, null);
		
		Quantifier q1 = new Quantifier("all", "R0", "v" + getSourceMajor() + "m" +getSourceMinor() + "_" + getTableName());
		d.globalQuantifiers.quantifiers.add(q1);
		
		Conjunctive c0 = new Conjunctive();
		d.ifClause.disjunctive.add(c0);		
		
		QuantifiedDisjunctive tc1 = new QuantifiedDisjunctive(); 
		d.thenClauses.add(tc1);

		q1 = new Quantifier("some", "R1", "v" + getTargetMajor() + "m" + getTargetMinor() + "_" + getTableName());
		tc1.quantifiers.add(q1);
		
		Conjunctive c1 = new Conjunctive();
		tc1.disjunctive.add(c1);
		c1.predicates = new Vector<Predicate>();
		
		Table inTable = inputSchema.getTable(getTableName());
		try {
			inputSchema.applySMO(this);
		} catch (IllegalSMOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Table outputTable = inputSchema.getTable(getTableName());
		
		Iterator<String> inColumns = inTable.getColumns().iterator();
		Iterator<String> outColumns = outputTable.getColumns().iterator();
		
		while(outColumns.hasNext()){					
			String outColumn = outColumns.next();
			// process columns that also existed in the previous version
			if(!outColumn.equalsIgnoreCase(getColumnName())){	
				Predicate p1 = new Predicate();
				c1.predicates.add(p1);
				p1.term2 = new Term("R1", outColumn);
				p1.term1 = new Term("R0", inColumns.next());
				p1.op = "=";
			}
			// process the newly added column
			else{			
				Predicate p1 = new Predicate();
				c1.predicates.add(p1);
				p1.term1 = new Term("R1", outColumn);
				p1.op = "=";
				
				if(this.getFunctionName()!=null){	
					// Relation-Encoded-Function - Hyun 2/12/2008
					// step 1: add a relation _refn_func R2 (arity: n+1)
					String functionName = this.getFunctionName();
					int numFunctionArgs = this.getFunctionArgs().size();
					q1 = new Quantifier("all", "R2", 
							REFConstant.getREF(numFunctionArgs, getTargetMajor(), getTargetMinor(), functionName));
					d.globalQuantifiers.quantifiers.add(q1);
					
					// step 2: add R1.added_col = R2.an
					p1.term2 = new Term("R2", "a" + numFunctionArgs);
					
					// step 3: add R2.a0 = R0.arg0, R2.a1 = R0.arg1, ...
					//         into if-clause, not then-clause (fix by Hyun on 4/9/2008)
					for(int i=0; i<numFunctionArgs; i++) {
						Predicate p3 = new Predicate();
						//c1.predicates.add(p3);
						c0.predicates.add(p3);
						p3.term1 = new Term("R2", "a" + i);
						p3.op = "=";
						p3.term2 = new Term("R0", this.getFunctionArgs().get(i));
					}
				}
				else{
					if(this.getDefaultConstant()==null)
						//p1.term2 = new Term("\"null_string\""); // temp hack
						p1.term2 = new Term("\"null\""); // temp hack
					else{
						if(this.getDefaultConstant().equalsIgnoreCase("\"\"")){
							//p1.term2 = new Term("\"empty_string\"");	
							p1.term2 = new Term("\"\"");
						}	
						else if(!this.getDefaultConstant().startsWith("\""))	
							p1.term2 = new Term("\""+this.getDefaultConstant()+"\"");	
						else
							p1.term2 = new Term(this.getDefaultConstant());	
								
					}
				}	
			}
		}
		
		ds.add(d);
		
		
		// QUERY REWRITING DED
		DED d2 = new DED(dedId, dedName + "_qr",false, sourceTables, targetTables, null);
		
		Quantifier q2 = new Quantifier("all", "R1", "v" + getTargetMajor() + "m" +getTargetMinor() + "_" + getTableName());
		d2.globalQuantifiers.quantifiers.add(q2);
		
		QuantifiedDisjunctive tc2 = new QuantifiedDisjunctive(); 
		d2.thenClauses.add(tc2);

		q2 = new Quantifier("some", "R0", "v" + getSourceMajor() + "m" + getSourceMinor() + "_" + getTableName());
		tc2.quantifiers.add(q2);
		
		Conjunctive c2 = new Conjunctive();
		tc2.disjunctive.add(c2);
		c2.predicates = new Vector<Predicate>();
		
		Iterator<String> inColumns2 = inTable.getColumns().iterator();
		Iterator<String> outColumns2 = outputTable.getColumns().iterator();
		
		while(outColumns2.hasNext()){					
			String outColumn2 = outColumns2.next();
			if(!outColumn2.equalsIgnoreCase(getColumnName())){	
				Predicate p2 = new Predicate();
				c2.predicates.add(p2);
				p2.term2 = new Term("R1", outColumn2);
				p2.term1 = new Term("R0", inColumns2.next());
				p2.op = "=";
			}
		
			else{		
				
				Predicate p2 = new Predicate();
				c2.predicates.add(p2);
				p2.term1 = new Term("R1", outColumn2);
				p2.op = "=";
				
				
				if(this.getFunctionName()!=null){	
					// Relation-Encoded-Function - Hyun 2/12/2008
					// step 1: add a relation _refn_func R2 (arity: n+1)
					String functionName = this.getFunctionName();
					int numFunctionArgs = this.getFunctionArgs().size();
					q2 = new Quantifier("some", "R2", 
							REFConstant.getREF(numFunctionArgs, getTargetMajor(), getTargetMinor(), functionName));
					tc2.quantifiers.add(q2);
					
					// step 2: add R1.added_col = R2.an
					p2.term2 = new Term("R2", "a" + numFunctionArgs);
					
					// step 3: add R2.a0 = R1.arg0, R2.a1 = R1.arg1, ...
					for(int i=0; i<numFunctionArgs; i++) {
						Predicate p3 = new Predicate();
						c2.predicates.add(p3);
						p3.term1 = new Term("R2", "a" + i);
						p3.op = "=";
						p3.term2 = new Term("R1", this.getFunctionArgs().get(i));
					}
				}
				else{
				
					
					
					if(this.getDefaultConstant()==null)
						//p2.term2 = new Term("\"null_string\""); // temp hack
						p2.term2 = new Term("\"null\"");
					else{
						if(this.getDefaultConstant().equalsIgnoreCase("\"\"")){
							//p2.term2 = new Term("\"empty_string\"");	
							p2.term2 = new Term("\"\"");
						}	
						else if(!this.getDefaultConstant().startsWith("\""))	
							p2.term2 = new Term("\""+this.getDefaultConstant()+"\"");	
						else
							p2.term2 = new Term(this.getDefaultConstant());	
								
					}
				}
			}
		}
		ds.add(d2);

		// hyun 2/27/2008
		// to fix mars specific fix, we add suffix to all constants
		try {
			edu.ucla.cs.db.prism.backend.marsManage.MarsSpecificFix.addSuffixToConstants(ds);
		} catch (MarsConstantsSuffixException e) {
			e.printStackTrace();
		}
		
		return ds;
	}

	public String getSQLView(Schema schema, String inputDBname, String outputDBName){
		String sql = "";
		
		
		/* 
		 * this is required to ensure correctness of multiple SMOS between 2 major 
		 * versions. Major Versions are represented by name convention in SCHEMA NAMEs, 
		 * while Minor Versions are represeted by name conventions in TABLE NAMES
		*/
		String sourceTabName =  this.getTableName(); 
		String targetTabName =  this.getTableName(); 
		
		if(this.getSourceMinor()>0)
			sourceTabName += "_" + this.getSourceMinor();
		if(this.getTargetMinor()>0)
			targetTabName += "_" + this.getTargetMinor();			
		
		
		sql += "CREATE VIEW " + outputDBName + "." + targetTabName + " AS ";
		if(this.getDefaultConstant()!=null)
			sql += "( SELECT t.*,'" + this.getDefaultConstant() + "' AS " + getColumnName(); 
		else if(this.getFunctionName()!=null){
			sql += "( SELECT t.*," + this.getFunctionName() + "(";
			Iterator<String> args = this.getFunctionArgs().iterator();
			while(args.hasNext()){
				sql+= args.next();
				if(args.hasNext())
					sql	+= ","; 
			}
			sql += ") AS " + getColumnName(); 
		}else{
			sql += "( SELECT t.*,'null' AS " + getColumnName(); 
		}	
		sql += " FROM " + inputDBname + "." + sourceTabName + " t );"; 
		return sql;
	}
	
	public String getSQLDDL(Schema schema, String outputDBname) {
		String sqlddl = "";
	
		sqlddl += "ALTER TABLE " + outputDBname + "." + this.getTableName()
				+ " ADD COLUMN " + getColumnName() + " " + getColumnType() + ";\n";
		
		if(this.getDefaultConstant()!=null) {
			sqlddl += "UPDATE " + outputDBname + "." + this.getTableName() + " SET ";
			sqlddl += getColumnName() + " = " + this.getDefaultConstant()+";\n";
		}
		else if(this.getFunctionName()!=null){
			sqlddl += "UPDATE " + outputDBname + "." + this.getTableName() + " SET ";
			sqlddl += getColumnName() + " = " + this.getFunctionName() + "(";
			Iterator<String> args = this.getFunctionArgs().iterator();
			while(args.hasNext()){
				sqlddl+= args.next();
				if(args.hasNext())
					sqlddl	+= ","; 
			}
			sqlddl += ");\n"; 
		}
		return sqlddl;
	}	
	
	
	@Override
	public boolean isRedundant() {
		if(this.getFunctionName()!=null)
			return true;
		else
			return false;
	}
	
}
