/**
 * 
 */
package edu.ucla.cs.db.prism.backend.smo;

import java.util.Iterator;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.ded.DED;
import edu.ucla.cs.db.prism.backend.ded.DEDSet;
import edu.ucla.cs.db.prism.backend.ded.QuantifiedDisjunctive;
import edu.ucla.cs.db.prism.backend.ded.Quantifier;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.Table;
import edu.ucla.cs.db.prism.backend.schema.exception.IllegalSMOException;
import edu.ucla.cs.db.prism.backend.smo.util.Conjunctive;
import edu.ucla.cs.db.prism.backend.smo.util.Predicate;
import edu.ucla.cs.db.prism.backend.smo.util.SMOFactory;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequence;
import edu.ucla.cs.db.prism.backend.smo.util.SMOTypeUnknownException;
import edu.ucla.cs.db.prism.backend.smo.util.Term;

/**
 * @author krl
 *
 */
public class CopyTableSMO extends SMOImpl {

	/**
	 * 
	 */
	public CopyTableSMO() {
		super("COPY TABLE");
		 setHasPerfectInverse(false);
	}

	@Override
	public String toString() {
		String str = getOperatorType() + " ";	
		str += getTableName() + " INTO " + getTableName2();
		return str + "\n";
	}
	
	
	@Override
	public Vector<String> getModifiedInputSchemaParts() {
		Vector<String> schemaParts = new Vector<String>();
			// no modified input
		return schemaParts;	
	}	
	
	@Override
	public Vector<String> getModifiedExistingTables(Schema queryFootprintTargetSchema) {
		Vector<String> modifiedExistingTables = new Vector<String>();
		// no modified input
		return modifiedExistingTables;
	}


	
	public Vector<SMOSequence> getCandidateInverses(Schema inputSchema) {
		Vector<SMOSequence> v = new Vector<SMOSequence>();
		
		// first SMO, typical one if copy table is used to create pure redundancy
		SMOSequence s = new SMOSequence();
	
		
		/* first mosto probable inverse the two copy are updated independently but 
		 * the tuple should "flow-back" in the same tuple -> MERGE*/
		SMOSequence s2 = new SMOSequence();
		SMOImpl newSMO2 = null;
		try {
			newSMO2 = SMOFactory.createSMO(SMOInterface.MERGE);
		} catch (SMOTypeUnknownException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		
		}	
		newSMO2.setTableName(this.getTableName());
		newSMO2.setTableName2(this.getTableName2());
		newSMO2.setTableName3(this.getTableName());
		
		
		// switch source and target and set them in the answer
		s2.sourceVersion=this.getTargetMajor();
		s2.targetVersion=this.getSourceMajor();
		newSMO2.setSourceMajor(this.getTargetMajor());
		newSMO2.setSourceMinor(this.getTargetMinor());
		newSMO2.setTargetMajor(this.getSourceMajor());
		newSMO2.setTargetMinor(this.getSourceMinor());
		s2.add(newSMO2);		
		v.add(s2);
		
		
		SMOImpl si = null;
		try {
			si = SMOFactory.createSMO("DROP TABLE");
		} catch (SMOTypeUnknownException ex) {
			// TODO Auto-generated catch block
			ex.printStackTrace();
		}
		si.setTableName(this.getTableName());
		
		// switch source and target and set them in the answer
		s.sourceVersion=this.getTargetMajor();
		s.targetVersion=this.getSourceMajor();
		si.setSourceMajor(this.getTargetMajor());
		si.setSourceMinor(this.getTargetMinor());
		si.setTargetMajor(this.getSourceMajor());
		si.setTargetMinor(this.getSourceMinor());
		s.add(si);
		v.add(s);
		
		return v;
	}
	
	public DEDSet getDEDs(Schema inputSchema) {
		DEDSet ds = new DEDSet(getSourceMajor(),getSourceMinor(),getTargetMajor(), getTargetMinor());
		
		String dedId = null;
		String dedName = removeSpaces(getOperatorType()) + "_" + getTableName();
		
		Vector<String> sourceTables = new Vector<String>();
		sourceTables.add(getTableName());
		Vector<String> targetTables = new Vector<String>();
		targetTables.add(getTableName2());
	
		copyDed(inputSchema, ds, dedId, dedName, true, sourceTables,
				targetTables);

		idDed(inputSchema, ds, dedId, dedName, true, sourceTables,
				sourceTables);

		copyDed(inputSchema, ds, dedId, dedName, false, targetTables,sourceTables);
		
		idDed(inputSchema, ds, dedId, dedName, false,sourceTables, sourceTables);	
		
		return ds;
	}
	
	@Override
	public DEDSet getAllDEDs(Schema inputSchema){
		
		Vector<String> excludeList = new Vector<String>();
		excludeList.add(getTableName());
		excludeList.add(getTableName2());
			
		DEDSet ds = getDEDs(inputSchema.clone());
		ds.addAll(getIdentityDEDs(inputSchema,excludeList));
		
		return ds;
	}
	
	@Override
	public boolean isRedundant() {
		return true;
	}
	
	@Override
	public Vector<String> getIdentitySQLViews(Schema inputSchema, String dbname) {
	
		Vector<String> excludeList = new Vector<String>();
		excludeList.add(getTableName());
		excludeList.add(getTableName2());
		
		return this.getIdentitySQLView(inputSchema, dbname, excludeList);
	}
	

	private void copyDed(Schema inputSchema, DEDSet ds, String dedId,
			String dedName, boolean chaseFlag, Vector<String> sourceTables,
			Vector<String> targetTables) {
	
		if(chaseFlag)
			dedName += "_copy_dm"; 
		else
			dedName += "_copy_qr"; 
		
		DED d = new DED(dedId, dedName, chaseFlag, sourceTables, targetTables, null);
		
		Quantifier q1 = null;
		
		if(chaseFlag)
			 q1 = new Quantifier("all", "R0", "v" + getSourceMajor() + "m" +getSourceMinor() + "_" + getTableName());
		else
			 q1 = new Quantifier("all", "R0", "v" + getTargetMajor() + "m" +getTargetMinor() + "_" + getTableName2());
				
		d.globalQuantifiers.quantifiers.add(q1);
		
		QuantifiedDisjunctive tc1 = new QuantifiedDisjunctive(); 
		d.thenClauses.add(tc1);

		if(chaseFlag)
			q1 = new Quantifier("some", "R1", "v" + getTargetMajor() + "m" + getTargetMinor() + "_" + getTableName2());
		else
			q1 = new Quantifier("some", "R1", "v" + getSourceMajor() + "m" + getSourceMinor() + "_" + getTableName());
		
		tc1.quantifiers.add(q1);
		
		Conjunctive c1 = new Conjunctive();
		tc1.disjunctive.add(c1);
		c1.predicates = new Vector<Predicate>();
		
		Table inTable = inputSchema.getTable(getTableName());
		inputSchema = inputSchema.clone();
		try {
			inputSchema.applySMO(this);
		} catch (IllegalSMOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Table outputTable = inputSchema.getTable(getTableName2());
		
		Iterator<String> inColumns = inTable.getColumns().iterator();
		Iterator<String> outColumns = outputTable.getColumns().iterator();
		
		while(inColumns.hasNext()){
			String inCol = inColumns.next();
			String outCol = outColumns.next();
			
			Predicate p1 = new Predicate();
			c1.predicates.add(p1);
			p1.term1 = new Term("R0", inCol);
			p1.term2 = new Term("R1", outCol);
			p1.op = "=";
						
		}
		
		ds.add(d);
	}
	
	private void idDed(Schema inputSchema, DEDSet ds, String dedId,
			String dedName, boolean chaseFlag, Vector<String> sourceTables,
			Vector<String> targetTables) {
		if(chaseFlag)
			dedName += "_id_dm"; 
		else
			dedName += "_id_qr"; 
		DED d = new DED(dedId, dedName, chaseFlag, sourceTables, targetTables, null);
		
		Quantifier q1 = null;
		
		if(chaseFlag)
			 q1 = new Quantifier("all", "R0", "v" + getSourceMajor() + "m" +getSourceMinor() + "_" + getTableName());
		else
			 q1 = new Quantifier("all", "R0", "v" + getTargetMajor() + "m" +getTargetMinor() + "_" + getTableName());
				
		d.globalQuantifiers.quantifiers.add(q1);
		
		QuantifiedDisjunctive tc1 = new QuantifiedDisjunctive(); 
		d.thenClauses.add(tc1);

		if(chaseFlag)
			q1 = new Quantifier("some", "R1", "v" + getTargetMajor() + "m" + getTargetMinor() + "_" + getTableName());
		else
			q1 = new Quantifier("some", "R1", "v" + getSourceMajor() + "m" + getSourceMinor() + "_" + getTableName());
			
		tc1.quantifiers.add(q1);
		
		Conjunctive c1 = new Conjunctive();
		tc1.disjunctive.add(c1);
		c1.predicates = new Vector<Predicate>();
		
		Table inTable = inputSchema.getTable(getTableName());

		Table outputTable = inputSchema.getTable(getTableName());
		
		Iterator<String> inColumns = inTable.getColumns().iterator();
		Iterator<String> outColumns = outputTable.getColumns().iterator();
		
		while(inColumns.hasNext()){
			String inCol = inColumns.next();
			String outCol = outColumns.next();
			
			Predicate p1 = new Predicate();
			c1.predicates.add(p1);
			p1.term1 = new Term("R0", inCol);
			p1.term2 = new Term("R1", outCol);
			p1.op = "=";
						
		}
		
		ds.add(d);
	}
	
	
	@Override
	public String getSQLView(Schema schema, String inputDBname, String outputDBName){
		String sql = "";
		
		
		/* 
		 * this is required to ensure correctness of multiple SMOS between 2 major 
		 * versions. Major Versions are represented by name convention in SCHEMA NAMEs, 
		 * while Minor Versions are represeted by name conventions in TABLE NAMES
		*/
		String sourceTabName1 =  this.getTableName(); 
		String targetTabName1 =  this.getTableName(); 
		String sourceTabName2 =  this.getTableName2(); 
		String targetTabName2 =  this.getTableName2(); 
		
		if(this.getSourceMinor()>0){
			sourceTabName1 += "_" + this.getSourceMinor();
			sourceTabName2 += "_" + this.getSourceMinor();
		}
		if(this.getTargetMinor()>0){
			targetTabName1 += "_" + this.getTargetMinor();	
			targetTabName2 += "_" + this.getTargetMinor();	
		}

		
		sql += "CREATE VIEW " + outputDBName + "." + targetTabName1 + " AS ";
		sql += "( SELECT t.* "; 
		sql += " FROM " + inputDBname + "." + sourceTabName1 + " t );\n"; 

		sql += "CREATE VIEW " + outputDBName + "." + targetTabName2 + " AS ";
		sql += "( SELECT t.* "; 
		sql += " FROM " + inputDBname + "." + sourceTabName1 + " t );\n"; 
		
		return sql;
	}
	
	@Override
	public String getSQLDDL(Schema schema, String outputDBname) {
		String sqlddl="";
		
		sqlddl += "CREATE TABLE " + outputDBname + "." + this.getTableName2() + " LIKE " + outputDBname + "." + this.getTableName() +";\n";
		sqlddl += "INSERT INTO " + outputDBname + "." + this.getTableName2() + " (SELECT * FROM " +outputDBname + "." + this.getTableName() + ");\n";
		
		
		return sqlddl;			
	}
	

}
