/**
 * 
 */
package edu.ucla.cs.db.prism.backend.smo;

import java.util.Iterator;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.ded.DED;
import edu.ucla.cs.db.prism.backend.ded.DEDSet;
import edu.ucla.cs.db.prism.backend.ded.QuantifiedDisjunctive;
import edu.ucla.cs.db.prism.backend.ded.Quantifier;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.smo.util.Conjunctive;
import edu.ucla.cs.db.prism.backend.smo.util.Predicate;
import edu.ucla.cs.db.prism.backend.smo.util.SMOFactory;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequence;
import edu.ucla.cs.db.prism.backend.smo.util.SMOTypeUnknownException;
import edu.ucla.cs.db.prism.backend.smo.util.Term;

/**
 * @author krl
 *
 */
public class CreateTableSMO extends SMOImpl {

	/**
	 * 
	 */
	public CreateTableSMO() {
		super("CREATE TABLE");
		 setHasPerfectInverse(true);

	}

	@Override
	public String toString() {
		String str = getOperatorType() + " ";	
		str += getTableName();
		if(getColumnNameList() != null && getColumnNameList().size() > 0) {
			for(int i=0; i<getColumnNameList().size(); i++) {
				if(i==0) str += "(";
				str += getColumnNameList().get(i);
				if(getColumnTypeList() != null && 
						getColumnTypeList().size() == getColumnNameList().size())
					str += " " + getColumnTypeList().get(i);
				if(i==getColumnNameList().size()-1) 
					str += ")";
				else 
					str += ", ";		
			}
		}
		return str + "\n";
	}
	
	
	@Override
	public Vector<String> getModifiedInputSchemaParts() {
		Vector<String> schemaParts = new Vector<String>();
			// no modified input
		return schemaParts;	
	}	
	
	@Override
	public Vector<String> getModifiedExistingTables(Schema queryFootprintTargetSchema) {
		Vector<String> modifiedExistingTables = new Vector<String>();
		// no modified input
		return modifiedExistingTables;
	}
	
	
	public Vector<SMOSequence> getCandidateInverses(Schema inputSchema) {
		Vector<SMOSequence> v = new Vector<SMOSequence>();
		SMOSequence s = new SMOSequence();
		SMOImpl si = null;
		try {
			si = SMOFactory.createSMO("DROP TABLE");
		} catch (SMOTypeUnknownException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		si.setTableName(this.getTableName());
		v.add(s);
		
		// switch source and target and set them in the answer
		s.sourceVersion=this.getTargetMajor();
		s.targetVersion=this.getSourceMajor();
		si.setSourceMajor(this.getTargetMajor());
		si.setSourceMinor(this.getTargetMinor());
		si.setTargetMajor(this.getSourceMajor());
		si.setTargetMinor(this.getSourceMinor());
		s.add(si);
		
		

		return v;
	}
	
	@Override
	public DEDSet getAllDEDs(Schema inputSchema){
		
		Vector<String> excludeList = new Vector<String>();
			
		DEDSet ds = getDEDs(inputSchema.clone());
		ds.addAll(getIdentityDEDs(inputSchema,excludeList));
		
		return ds;
	}
	
	
	@Override
	public Vector<String> getIdentitySQLViews(Schema inputSchema, String dbname) {
	
		Vector<String> excludeList = new Vector<String>();
		
		return this.getIdentitySQLView(inputSchema, dbname, excludeList);
	}
	
	public DEDSet getDEDs(Schema inputSchema) {
		DEDSet ds = new DEDSet(getSourceMajor(),getSourceMinor(),getTargetMajor(), getTargetMinor());
		
		
		String dedId = null;
		String dedName = removeSpaces(getOperatorType()) + "_" + getTableName();
		
		Vector<String> sourceTables = new Vector<String>();
		sourceTables.add(getTableName());
		
		Vector<String> targetTables = new Vector<String>();
		
		computeDed(inputSchema, ds, dedId, dedName, false, sourceTables,
				targetTables);
		return ds;
	}
	
	
	private void computeDed(Schema inputSchema, DEDSet ds, String dedId,
			String dedName, boolean chaseFlag, Vector<String> sourceTables,
			Vector<String> targetTables) {
			dedName += "_id_dm"; 
		DED d = new DED(dedId, dedName, chaseFlag, sourceTables, targetTables, null);
		
		Quantifier q1 = null;
		
		q1 = new Quantifier("all", "R0", "v" + getTargetMajor() + "m" +getTargetMinor() + "_" + getTableName());
				
		d.globalQuantifiers.quantifiers.add(q1);

		QuantifiedDisjunctive tc1 = new QuantifiedDisjunctive(); 
		d.thenClauses.add(tc1);
		
		Conjunctive c1 = new Conjunctive();
		tc1.disjunctive.add(c1);
		c1.predicates = new Vector<Predicate>();
		
		Predicate p = new Predicate();
		p.op = "=";
		p.term1 = new Term("\"1\"");
		p.term2 = new Term("\"2\"");
		c1.predicates.add(p);
		ds.add(d);
	}
	
	
	
	
	
	
	@Override
	public String getSQLView(Schema schema, String inputDBname, String outputDBName){
		String sql = "";
		String nulls="";
		String colList = "";
		int colnum = this.getColumnNameList().size();
		
		
		/* 
		 * this is required to ensure correctness of multiple SMOS between 2 major 
		 * versions. Major Versions are represented by name convention in SCHEMA NAMEs, 
		 * while Minor Versions are represeted by name conventions in TABLE NAMES
		*/
		String sourceTabName1 =  this.getTableName(); 
		String targetTabName1 =  this.getTableName(); 
		
		if(this.getSourceMinor()>0){
			sourceTabName1 += "_" + this.getSourceMinor();
		}
		if(this.getTargetMinor()>0){
			targetTabName1 += "_" + this.getTargetMinor();	
		}
		
		
		sql += "CREATE VIEW " + outputDBName + "." + targetTabName1 + "(";
		Iterator<String> cols = this.getColumnNameList().iterator();
		while (cols.hasNext()){
			sql+=cols.next();
			nulls += "null";
			if(cols.hasNext()){
				sql+=",";
				nulls+=",";
			}	
		}
		sql+=") AS ";
		sql += "( SELECT " + nulls + " );\n"; 

		
		return sql;
	}
		
	@Override
	public String getSQLDDL(Schema schema, String outputDBname) {
		String sqlddl="";
		
		sqlddl += "CREATE TABLE " + outputDBname + "." + this.getTableName() + "(";
		Iterator<String> cols = this.getColumnNameList().iterator();
		Iterator<String> colTypes = this.getColumnTypeList().iterator();
		while (cols.hasNext()) {
			sqlddl += cols.next()+" ";
			sqlddl += colTypes.next();
			if(cols.hasNext()){
				sqlddl += ",";
			}
		}
		sqlddl += ");\n";
		return sqlddl;
	}
	
}
