/**
 * 
 */
package edu.ucla.cs.db.prism.backend.smo;

import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.ded.DED;
import edu.ucla.cs.db.prism.backend.ded.DEDSet;
import edu.ucla.cs.db.prism.backend.ded.QuantifiedDisjunctive;
import edu.ucla.cs.db.prism.backend.ded.Quantifier;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.Table;
import edu.ucla.cs.db.prism.backend.schema.exception.IllegalSMOException;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.PrimaryKey;
import edu.ucla.cs.db.prism.backend.smo.exception.NoKeyInCommonInfPresException;
import edu.ucla.cs.db.prism.backend.smo.exception.NoTableInSchemaException;
import edu.ucla.cs.db.prism.backend.smo.exception.noColumnsMatchInfPres;
import edu.ucla.cs.db.prism.backend.smo.exception.noCommonColumnInfPres;
import edu.ucla.cs.db.prism.backend.smo.util.Conjunctive;
import edu.ucla.cs.db.prism.backend.smo.util.Disjunctive;
import edu.ucla.cs.db.prism.backend.smo.util.Predicate;
import edu.ucla.cs.db.prism.backend.smo.util.SMOFactory;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequence;
import edu.ucla.cs.db.prism.backend.smo.util.SMOTypeUnknownException;
import edu.ucla.cs.db.prism.backend.smo.util.Term;

/**
 * @author krl
 *
 */
public class DecomposeTableSMO extends SMOImpl {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * 
	 */
	public DecomposeTableSMO() {

		super("DECOMPOSE TABLE");
		 setHasPerfectInverse(true);

	}

	public String toString() {
		String str = getOperatorType() + " ";	
		str += getTableName() + " INTO " 
		    + getTableName2() + "(" + this.getAttributeList2String() + ")," 
		    + getTableName3() + "(" + this.getAttributeList3String() + ")";
		    //+ " WHERE " 
		    //+ this.getTargetConditions();
		return str + "\n";
	}
	


	public Vector<String> getModifiedInputSchemaParts() {
		Vector<String> schemaParts = new Vector<String>();
		schemaParts.add(getTableName());
		return schemaParts;	
	}	
	
	public Vector<String> getModifiedExistingTables(Schema queryFootprintTargetSchema) {
		Vector<String> modifiedExistingTables = new Vector<String>();
		modifiedExistingTables.add(getTableName());
		return modifiedExistingTables;
	}

	
	public Vector<SMOSequence> getCandidateInverses(Schema inputSchema) {
		
		if(assignedInverse != null){
			Vector<SMOSequence> ret = new Vector<SMOSequence>();
			ret.add(assignedInverse);
			return ret;
		}
		
		Vector<SMOSequence> v = new Vector<SMOSequence>();
		SMOSequence s = new SMOSequence();
		SMOImpl si = null;
		try {
			si = SMOFactory.createSMO("JOIN TABLE");
		} catch (SMOTypeUnknownException e) {
			e.printStackTrace();
		}
		si.setTableName(this.getTableName2());
		si.setTableName2(this.getTableName3());
		si.setTableName3(this.getTableName());
		
		Vector<String> columns1  = this.getAttributeList2();
		Vector<String> columns2  = this.getAttributeList3();
		
		Iterator<String> col1it =  columns1.iterator();
		Iterator<String> col2it =  columns2.iterator();
		
		Conjunctive conj = new Conjunctive();
	
		
		// debugged columns match plus use of new for with no iterator
		for(String c1: columns1)
			for(String c2:columns2){
				if(c1.equals(c2)){
					Predicate p = new Predicate();
					p.term1 = new Term(si.getTableName(),c1);
					p.term2 = new Term(si.getTableName2(),c2);
					p.op = "=";
					conj.predicates.add(p);
				}
				
			}
		
		/*
		while(col1it.hasNext()){
			String col1 = col1it.next();
			while(col2it.hasNext()){
				String col2 = col2it.next(); 
					if(col1.equalsIgnoreCase(col2)){
						Predicate p = new Predicate();
						p.term1 = new Term(si.getTableName(),col1);
						p.term2 = new Term(si.getTableName2(),col2);
						p.op = "=";
						conj.predicates.add(p);
					}
			}
		}*/
		
		Disjunctive dis = new Disjunctive();
		dis.conjunctives.add(conj);
		si.setTargetConditions(dis);

		// switch source and target and set them in the answer
		s.sourceVersion=this.getTargetMajor();
		s.targetVersion=this.getSourceMajor();
		si.setSourceMajor(this.getTargetMajor());
		si.setSourceMinor(this.getTargetMinor());
		si.setTargetMajor(this.getSourceMajor());
		si.setTargetMinor(this.getSourceMinor());
		

		s.add(si);
		v.add(s);
		return v;
	}
	
	public DEDSet getAllDEDs(Schema inputSchema){
		
		Vector<String> excludeList = new Vector<String>();
		excludeList.add(getTableName());
		excludeList.add(getTableName2());
		excludeList.add(getTableName3());
			
		DEDSet ds = getDEDs(inputSchema.clone());
		ds.addAll(getIdentityDEDs(inputSchema,excludeList));
		
		return ds;
	}
	
	@Override
	public Vector<String> getIdentitySQLViews(Schema inputSchema, String dbname) {
	
		Vector<String> excludeList = new Vector<String>();
		excludeList.add(getTableName());
		excludeList.add(getTableName2());
		excludeList.add(getTableName3());
		
		return this.getIdentitySQLView(inputSchema, dbname, excludeList);
	}
	
	
	
	
	public boolean isICInfPreserving(ICSchema curSchema) throws noColumnsMatchInfPres, noCommonColumnInfPres, NoTableInSchemaException, NoKeyInCommonInfPresException{
	
		isInfPreserving(curSchema);
		
		// we trust the assigned inverse to be infPres
		if(assignedInverse!=null){
			return true;
		}
		
		PrimaryKey sourcepk = curSchema.getTable(this.getTableName()).getPrimaryKey();
		Vector<String> overlapp = this.getOverlappingAttributes();		
		if(sourcepk==null)
			throw new NoKeyInCommonInfPresException("The table " + this.getTableName() + " does not have a primary key. There is no guarantee that the overlap between the output tables would join in a lossless way");

		if(!overlapp.containsAll(sourcepk.getFields()))
			throw new NoKeyInCommonInfPresException("Some of the fields of the PrimaryKey of table " + this.getTableName() + " are not in the overlap of attributes of the two output tables");
	
		return true;
		
	}
	
	
	
	
	private Vector<String> getOverlappingAttributes() {
		Vector<String> res = new Vector<String>();
		for(String att: this.getAttributeList2()){
			if(this.getAttributeList3().contains(att))
				res.add(att);
		}
		return res;
	}

	//added by swami Jan 20 '09
	@Override
	public boolean isInfPreserving(Schema curSchema) throws noColumnsMatchInfPres, noCommonColumnInfPres, NoTableInSchemaException{
		
		//Match all columns between Table 1 and Table 2, 3
		int index1 = curSchema.indexOf(this.getTableName());
		if(index1<0){
			throw new NoTableInSchemaException("Table " + this.getTableName() + " is not in the schema");
			
		}
	
		
		Table table = curSchema.getTable(index1);
		
		Vector<String> cols = table.getColumns();
		
		Vector<String> targetlists = (Vector<String>) this.getAttributeList2().clone();
		targetlists.addAll((Collection<? extends String>) this.getAttributeList3().clone());
		
		if(!targetlists.containsAll(cols))
			throw (new noColumnsMatchInfPres("Columns of source table missing in the destination(s)"));

/*		boolean flag = true;
		Vector<String> tbl1Cols = table.getColumns();
		//Vector<String> tbl2Cols = this.getAttributeList2().
		//Vector<String> tbl3Cols = table.getColumns();
		String tempCol;
		int j = 0;

		for(int i = 0; i< tbl1Cols.size();i++)
		{
			tempCol = tbl1Cols.elementAt(i);
			//need to check if this column is in any one of the decomposed tables mentioned
			for(j = 0; j< this.getAttributeList2().size(); j++)
			{
				if(tempCol == this.getAttributeList2().elementAt(j))
					break;
			}
			if(j< this.getAttributeList2().size())
			{
				continue;
			}
			
			for(j = 0; j< this.getAttributeList3().size(); j++)
			{
				if(tempCol == this.getAttributeList3().elementAt(j))
					break;
			}
			if(j>= this.getAttributeList3().size())
			{
				flag = false;
				break;
			}
			
		}
		
		
		if(flag == false)
		{
			//need to indicate that the information is not preserved as not all columns are decomposed
			//from source table to dest table
			//throw new Exception("Reason: All columns of source table not matched in destination tables");
			throw (new noColumnsMatchInfPres("Columns of source table missing in the destination(s)"));
			//return flag;
		}
		*/
		
		//See if there are any columns in Table 2 that matches with columns in table 3
		int i = 0;
		for(i = 0; i< this.getAttributeList2().size(); i++)
		{
			int j = 0;
			for(j = 0; j< this.getAttributeList3().size(); j++)
			{
				if(this.getAttributeList2().elementAt(i) == this.getAttributeList3().elementAt(j))
				{
					break;
				}
			}
			if(j < this.getAttributeList3().size())
			{
				//match already
				break;
			}
		}
		boolean flag = true;
		if(i >= this.getAttributeList2().size())
		{
			//full loop executed without a match
			flag = false;
			if(assignedInverse == null)
				throw (new noCommonColumnInfPres("Reason: The Destination tables must have atleast one matching column"));
		}
		
		return flag;
	}

	
	public DEDSet getDEDs(Schema inputSchema) {
		DEDSet ds = new DEDSet(getSourceMajor(),getSourceMinor(),getTargetMajor(), getTargetMinor());
		
		String dedId = null;
		String dedName = removeSpaces(getOperatorType()) + "_" + getTableName();
		
		Vector<String> sourceTables = new Vector<String>();
		sourceTables.add(getTableName());

	
		Vector<String> targetTables = new Vector<String>();
		targetTables.add(getTableName2());
		
		genDed(inputSchema.clone(), ds, dedId, dedName, true, sourceTables,targetTables,"t2");

		genDed(inputSchema.clone(), ds, dedId, dedName, false,targetTables,sourceTables,"t2");

		
		targetTables = new Vector<String>();
		targetTables.add(getTableName3());
		
		genDed(inputSchema.clone(), ds, dedId, dedName, true, sourceTables,	targetTables, "t3");

		genDed(inputSchema.clone(), ds, dedId, dedName, false, targetTables,sourceTables,"t3");	
		
		return ds;
	}

	private void genDed(Schema inputSchema, DEDSet ds, String dedId,
			String dedName, boolean chaseFlag, Vector<String> sourceTables,
			Vector<String> targetTables, String type) {
		
		String tt = null;
		if(type.equalsIgnoreCase("t2"))
			tt = getTableName2();
		else
			tt = getTableName3();
		
		dedName += "_" + tt;
		
		if(chaseFlag)
			dedName += "_dm"; 
		else
			dedName += "_qr"; 
		
		DED d = new DED(dedId, dedName, chaseFlag, sourceTables, targetTables, null);
		
		Quantifier q1 = null;
		
		if(chaseFlag)
			 q1 = new Quantifier("all", "R0", "v" + getSourceMajor() + "m" +getSourceMinor() + "_" + getTableName());
		else
			 q1 = new Quantifier("all", "R0", "v" + getTargetMajor() + "m" +getTargetMinor() + "_" + tt);
				
		d.globalQuantifiers.quantifiers.add(q1);
		
		QuantifiedDisjunctive tc1 = new QuantifiedDisjunctive(); 
		d.thenClauses.add(tc1);

		if(chaseFlag)
			q1 = new Quantifier("some", "R1", "v" + getTargetMajor() + "m" + getTargetMinor() + "_" + tt);
		else
			q1 = new Quantifier("some", "R1", "v" + getSourceMajor() + "m" + getSourceMinor() + "_" + getTableName());
		
		tc1.quantifiers.add(q1);
		
		Conjunctive c1 = new Conjunctive();
		tc1.disjunctive.add(c1);
		c1.predicates = new Vector<Predicate>();
		
		Table inTable = inputSchema.getTable(getTableName());
		try {
			inputSchema.applySMO(this);
		} catch (IllegalSMOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		Iterator<String> inColumns = inTable.getColumns().iterator();
		
		while(inColumns.hasNext()){
			String inCol = inColumns.next();
			if(inList(type,inCol)){
				Predicate p1 = new Predicate();
				c1.predicates.add(p1);
				p1.term1 = new Term("R0", inCol);
				p1.term2 = new Term("R1", inCol);
				p1.op = "=";
			}
		}
		ds.add(d);
	}

	private boolean inList(String type, String inCol) {
		Vector<String> v = null;
		if(type.equalsIgnoreCase("t2")){
			v= getAttributeList2();
		}
		else{
			v = getAttributeList3();
		}
		
		Iterator<String> it = v.iterator();
		while(it.hasNext())
			if(it.next().equalsIgnoreCase(inCol))
				return true;
		return false;
	}

	
	public String getSQLView(Schema schema, String inputDBname, String outputDBName){
		String sql = "";
		String nulls="";
		String collist = "";
		
		/* 
		 * this is required to ensure correctness of multiple SMOS between 2 major 
		 * versions. Major Versions are represented by name convention in SCHEMA NAMEs, 
		 * while Minor Versions are represeted by name conventions in TABLE NAMES
		*/
		String sourceTabName1 =  this.getTableName(); 
		String targetTabName1 =  this.getTableName(); 
		String sourceTabName2 =  this.getTableName2(); 
		String targetTabName2 =  this.getTableName2(); 
		String sourceTabName3 =  this.getTableName3(); 
		String targetTabName3 =  this.getTableName3(); 
		
		if(this.getSourceMinor()>0){
			sourceTabName1 += "_" + this.getSourceMinor();
			sourceTabName2 += "_" + this.getSourceMinor();
			sourceTabName3 += "_" + this.getSourceMinor();
		}
		if(this.getTargetMinor()>0){
			targetTabName1 += "_" + this.getTargetMinor();	
			targetTabName2 += "_" + this.getTargetMinor();
			targetTabName3 += "_" + this.getTargetMinor();
		}
	
		sql += "CREATE VIEW " + outputDBName + "." + targetTabName2 + "(";
		Iterator<String> cols = this.getAttributeList2().iterator();
		while (cols.hasNext()){
			collist+=cols.next();
			nulls += "null";
			if(cols.hasNext()){
				collist+=",";
				nulls+=",";
			}	
		}
		sql+=collist + ") AS ";
		sql += "( SELECT " + collist + " FROM " + inputDBname + "." + sourceTabName1 + " );\n"; 
		
		
		collist="";
		sql += "CREATE VIEW " + outputDBName + "." + targetTabName3 + "(";
		Iterator<String> cols2 = this.getAttributeList3().iterator();
		while (cols2.hasNext()){
			collist+=cols2.next();
			nulls += "null";
			if(cols2.hasNext()){
				collist+=",";
				nulls+=",";
			}	
		}
		sql+=collist + ") AS ";
		sql += "( SELECT " + collist + " FROM " + inputDBname + "." + sourceTabName1 + " );\n"; 
		
		
		return sql;
	}
	
	public String getSQLDDL(Schema schema, String outputDBname) {
		String sqlddl = "";		
		String colAndType2 = "";
		String collist2 ="";
		String collistNotIn2 = "";		
		String temp = "";
		boolean firstFlag=true;
		int i=0;
		
		Vector<String> tempList ;
		Vector<String> attributeList3 = getAttributeList3();
		Vector<String> attributeList2 = getAttributeList2();
				
		if(getTableName().equalsIgnoreCase(getTableName3())) {
			tempList = attributeList3;
			attributeList3 = attributeList2;
			attributeList2 = tempList;
		}
		Iterator<String> cols2 = attributeList3.iterator();

		//Iterator<String> colTypes2 = getAttributeTypeList().iterator();
		//schema.getTable(getTableName()).g
		
			
		while (cols2.hasNext()) {
			temp = cols2.next();
			
			collist2 += temp;
		//	CARLO FIX: table 3 is not in the schema to gather types use the input table types!!!
			//	colAndType2 += temp + " " + schema.getTable(getTableName3()).getColType(temp);
			colAndType2 += temp + " " + schema.getTable(getTableName()).getColType(temp);
			
			if (cols2.hasNext()) {
				collist2 += ",";
				colAndType2 += ",";
			}
			if (attributeList2.contains(temp)!=true ) {
				if(firstFlag!=true)
					collistNotIn2 += ",";
				collistNotIn2 += " DROP COLUMN " + temp;
				firstFlag=false;
			}
			
		}
		if (getTableName().equalsIgnoreCase(getTableName3())) {			
			sqlddl += "CREATE TABLE " + outputDBname + "." + getTableName2()
			+ " (" + colAndType2 + ");\n";
			sqlddl += "INSERT INTO " + outputDBname + "." + getTableName2()
			+ " (SELECT " + collist2 +" FROM " + outputDBname + "." 
			+ getTableName() + ");\n";
			sqlddl += "ALTER TABLE " + outputDBname + "." + getTableName()
			+ collistNotIn2 + ";\n";
			return sqlddl;
		}
		else if(!getTableName().equalsIgnoreCase(getTableName2())) {
			sqlddl += "RENAME TABLE " + outputDBname + "." + getTableName()
			+ " TO " + outputDBname + "." + getTableName2() + ";\n";
		}
		/*	RENAME TABLE T1 TO T2
		 * 	CREATE TABLE T3(...)
		 * 	INSERT INTO T3(...)
		 * 	ALTER TABLE T2 DROP COLUMN (...)
		 */
		
		sqlddl += "CREATE TABLE " + outputDBname + "." + getTableName3()
				+ " (" + colAndType2 + ");\n";
		sqlddl += "INSERT INTO " + outputDBname + "." + getTableName3()
				+ " (SELECT " + collist2 +" FROM " + outputDBname + "." 
				+ getTableName2() + ");\n";
		sqlddl += "ALTER TABLE " + outputDBname + "." + getTableName2()
				+ collistNotIn2 + ";\n";
		return sqlddl;
	}

}

