/**
 * 
 */
package edu.ucla.cs.db.prism.backend.smo;

import java.util.Iterator;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.ded.DED;
import edu.ucla.cs.db.prism.backend.ded.DEDSet;
import edu.ucla.cs.db.prism.backend.ded.QuantifiedDisjunctive;
import edu.ucla.cs.db.prism.backend.ded.Quantifier;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.Table;
import edu.ucla.cs.db.prism.backend.schema.exception.IllegalSMOException;
import edu.ucla.cs.db.prism.backend.smo.util.Conjunctive;
import edu.ucla.cs.db.prism.backend.smo.util.Predicate;
import edu.ucla.cs.db.prism.backend.smo.util.SMOFactory;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequence;
import edu.ucla.cs.db.prism.backend.smo.util.SMOTypeUnknownException;
import edu.ucla.cs.db.prism.backend.smo.util.Term;

/**
 * @author krl
 *
 */
public class DropColumnSMO extends SMOImpl {

	boolean hasPerfectInverse = false;
	/**
	 * 
	 */
	public DropColumnSMO() {
		super("DROP COLUMN");
		 setHasPerfectInverse(false);

	}
	
	@Override
	public boolean isInfPreserving() {
		return false;
	}
	//Added by swami - for Inf Pres
	@Override
	public boolean isInfPreserving(Schema curSchema) {
		return false;
	}

	@Override
	public String toString() {
		String str = getOperatorType() + " ";	
		str += getColumnName() + " FROM " + getTableName();
		return str + "\n";
	}
	
	
	@Override
	public Vector<String> getModifiedInputSchemaParts() {
		Vector<String> schemaParts = new Vector<String>();
		schemaParts.add(getTableName() + "." + getColumnName());
		return schemaParts;	
	}	
	
	@Override
	public Vector<String> getModifiedExistingTables(Schema queryFootprintTargetSchema) {
		Vector<String> modifiedExistingTables = new Vector<String>();
		modifiedExistingTables.add(getTableName());
		return modifiedExistingTables;
	}
	
	
	public Vector<SMOSequence> getCandidateInverses(Schema inputSchema) {
		Vector<SMOSequence> v = new Vector<SMOSequence>();
		
		
		// first most probable inverse
		SMOSequence s = new SMOSequence();
		SMOImpl newSMO = null;
		try {
			newSMO = SMOFactory.createSMO(SMOInterface.ADD_COL);
		} catch (SMOTypeUnknownException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		newSMO.setTableName(this.getTableName());
		newSMO.setColumnName(this.getColumnName());	
		//Carlo added
		newSMO.setColumnType(inputSchema.getTable(this.getTableName()).getColType(this.getColumnName()));
		
		// switch source and target and set them in the answer
		s.sourceVersion=this.getTargetMajor();
		s.targetVersion=this.getSourceMajor();
		newSMO.setSourceMajor(this.getTargetMajor());
		newSMO.setSourceMinor(this.getTargetMinor());
		newSMO.setTargetMajor(this.getSourceMajor());
		newSMO.setTargetMinor(this.getSourceMinor());
		newSMO.setComplete(false);
		s.add(newSMO);
		v.add(s);
		
		// second possible inverse -> nop -> no inverse -> query will break (intended behavior!)
		SMOSequence s2 = new SMOSequence();
		SMOImpl newSMO2 = null;
		try {
			newSMO2 = SMOFactory.createSMO(SMOInterface.NOP);
		} catch (SMOTypeUnknownException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		// switch source and target and set them in the answer
		s2.sourceVersion=this.getTargetMajor();
		s2.targetVersion=this.getSourceMajor();
		newSMO2.setSourceMajor(this.getTargetMajor());
		newSMO2.setSourceMinor(this.getTargetMinor());
		newSMO2.setTargetMajor(this.getSourceMajor());
		newSMO2.setTargetMinor(this.getSourceMinor());
		s2.add(newSMO2);
		
		v.add(s2);
		return v;
	}
	
	@Override
	public DEDSet getAllDEDs(Schema inputSchema){
		
		Vector<String> excludeList = new Vector<String>();
		excludeList.add(getTableName());
			
		DEDSet ds = getDEDs(inputSchema.clone());
		ds.addAll(getIdentityDEDs(inputSchema,excludeList));
		
		return ds;
	}

	@Override
	public Vector<String> getIdentitySQLViews(Schema inputSchema, String dbname) {
	
		Vector<String> excludeList = new Vector<String>();
		excludeList.add(getTableName());
		
		return this.getIdentitySQLView(inputSchema, dbname, excludeList);
	}
	
	public DEDSet getDEDs(Schema inputSchema) {
		DEDSet ds = new DEDSet(getSourceMajor(),getSourceMinor(),getTargetMajor(), getTargetMinor());
		
		String dedId = null;
		String dedName = removeSpaces(getOperatorType()) + "_" + getTableName() + "_" + getColumnName();
	
		Vector<String> sourceTables = new Vector<String>();
		sourceTables.add(getTableName());
		Vector<String> targetTables = new Vector<String>();
		targetTables.add(getTableName());
	
		DED d = new DED(dedId, dedName + "_qr", false,  targetTables,sourceTables, null);
	
		DED d2 = new DED(dedId, dedName + "_dm", true, sourceTables, targetTables, null);
		
		
		//DM
		Quantifier q2 = new Quantifier("all", "R0", "v" + getSourceMajor() + "m" +getSourceMinor() + "_" + getTableName());
		d2.globalQuantifiers.quantifiers.add(q2);

		QuantifiedDisjunctive tc2 = new QuantifiedDisjunctive(); 
		d2.thenClauses.add(tc2);

		q2 = new Quantifier("some", "R1", "v" + getTargetMajor() + "m" + getTargetMinor() + "_" + getTableName());
		tc2.quantifiers.add(q2);

		Conjunctive c2 = new Conjunctive();
		tc2.disjunctive.add(c2);
		c2.predicates = new Vector<Predicate>();

		
		
		//QR
		Quantifier q1 = new Quantifier("all", "R0", "v" + getTargetMajor() + "m" +getTargetMinor() + "_" + getTableName());
		d.globalQuantifiers.quantifiers.add(q1);
		
		QuantifiedDisjunctive tc1 = new QuantifiedDisjunctive(); 
		d.thenClauses.add(tc1);
		
		q1 = new Quantifier("some", "R1", "v" + getSourceMajor() + "m" + getSourceMinor() + "_" + getTableName());
		tc1.quantifiers.add(q1);

		Conjunctive c1 = new Conjunctive();
		tc1.disjunctive.add(c1);
		c1.predicates = new Vector<Predicate>();
		
		
		Table inTable = inputSchema.getTable(getTableName());
		try {
			inputSchema.applySMO(this);
		} catch (IllegalSMOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		Table outputTable = inputSchema.getTable(getTableName());

		Iterator<String> inColumns = inTable.getColumns().iterator();
		Iterator<String> outColumns = outputTable.getColumns().iterator();
		
		while(inColumns.hasNext()){			
			String inColumn = inColumns.next();
	
			
			if(!inColumn.equalsIgnoreCase(getColumnName())){
				String outCol = outColumns.next();
				Predicate p2 = new Predicate();
				c2.predicates.add(p2);
				p2.term1 = new Term("R0", inColumn);
				p2.term2 = new Term("R1",outCol );
				p2.op = "=";
				
				
				Predicate p1 = new Predicate();
				c1.predicates.add(p1);
				p1.term1 = new Term("R1", inColumn);
				p1.term2 = new Term("R0", outCol);
				p1.op = "=";
				
				
				
			}
		}
		

		ds.add(d2);
		ds.add(d);
		return ds;
	}
	
	@Override
	public String getSQLView(Schema schema, String inputDBname, String outputDBName){
		String sql = "";
		String collist = "";
	
		/* 
		 * this is required to ensure correctness of multiple SMOS between 2 major 
		 * versions. Major Versions are represented by name convention in SCHEMA NAMEs, 
		 * while Minor Versions are represeted by name conventions in TABLE NAMES
		*/
		String sourceTabName1 =  this.getTableName(); 
		String targetTabName1 =  this.getTableName(); 
		
		if(this.getSourceMinor()>0){
			sourceTabName1 += "_" + this.getSourceMinor();
		}
		if(this.getTargetMinor()>0){
			targetTabName1 += "_" + this.getTargetMinor();	
		}
	
		
		
		sql += "CREATE VIEW " + outputDBName + "." + targetTabName1 + "(";
		
		Iterator<Table> tit = schema.getTables().iterator();
		Iterator<String> cols =null;
		
		while(tit.hasNext()){
			Table tab = tit.next();
			if(tab.getTableName().equalsIgnoreCase(getTableName())){
		
				cols = tab.getColumns().iterator();
				break;
			}
		}

		
		while (cols.hasNext()){
			String temp = cols.next();
			if(temp==null)
				System.out.println("NULL COLUMN: " + this.toMetadataString() );
			if(!temp.equalsIgnoreCase(getColumnName())){
				collist+=temp;
				if(cols.hasNext()){
					collist+=",";
				}	
			}else{
				if(!cols.hasNext()){
					collist= collist.substring(0,collist.length()-1);
				}
				
			}
			
		}
		sql+=collist + ") AS ";
		sql += "( SELECT " + collist + " FROM " + inputDBname + "." + sourceTabName1 + " );\n"; 
			
		return sql;
	}
	
	@Override
	public String getSQLDDL(Schema schema, String outputDBname) {
		String sqlddl="";
		
		sqlddl += "ALTER TABLE " + outputDBname + "." + this.getTableName() + " DROP COLUMN " + getColumnName() +";\n";		
		return sqlddl;
	}	
	
}
