/**
 * 
 */
package edu.ucla.cs.db.prism.backend.smo;

import java.util.Vector;

import edu.ucla.cs.db.prism.backend.ded.DEDSet;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.Table;
import edu.ucla.cs.db.prism.backend.smo.util.SMOFactory;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequence;
import edu.ucla.cs.db.prism.backend.smo.util.SMOTypeUnknownException;

/**
 * @author krl
 *
 */
public class DropTableSMO extends SMOImpl {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * 
	 */
	public DropTableSMO() {
		super("DROP TABLE");
		 setHasPerfectInverse(false);

	}

	@Override
	public boolean isInfPreserving() {
		return false;
	}
	//Added by swami - for Inf Pres
	@Override
	public boolean isInfPreserving(Schema curSchema) {
		return false;
	}
	@Override
	public String toString() {
		String str = getOperatorType() + " ";	
		str += getTableName();
		return str + "\n";
	}
	
	
	@Override
	public Vector<String> getModifiedInputSchemaParts() {
		Vector<String> schemaParts = new Vector<String>();
		schemaParts.add(getTableName());
		return schemaParts;	
	}	

	@Override
	public Vector<String> getModifiedExistingTables(Schema queryFootprintTargetSchema) {
		Vector<String> modifiedExistingTables = new Vector<String>();
		modifiedExistingTables.add(getTableName());
		return modifiedExistingTables;
	}
	
	public Vector<SMOSequence> getCandidateInverses(Schema inputSchema) {
		Vector<SMOSequence> v = new Vector<SMOSequence>();
		
		
		// first main smo, the table is reacreted as an empty one with the same signature
		SMOSequence s = new SMOSequence();
		SMOImpl newSMO = null;
		try {
			newSMO = SMOFactory.createSMO("CREATE TABLE");
		} catch (SMOTypeUnknownException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		newSMO.setTableName(this.getTableName());
		
		//add the columns from the inputSchema
		Table t =  inputSchema.getTable(this.getTableName());
		System.out.println("INVERTING DROP of -" + this.getTableName() +"- " );
		if(t!=null){
			newSMO.setColumnNameList(t.getColumns());
			newSMO.setColumnTypeList(t.getColTypes());
		}
		
		
		// switch source and target and set them in the answer
		s.sourceVersion=this.getTargetMajor();
		s.targetVersion=this.getSourceMajor();
		newSMO.setSourceMajor(this.getTargetMajor());
		newSMO.setSourceMinor(this.getTargetMinor());
		newSMO.setTargetMajor(this.getSourceMajor());
		newSMO.setTargetMinor(this.getSourceMinor());
	
		s.add(newSMO);
		v.add(s);
		
		// second possible inverse -> nop -> no inverse -> query will break (intended behavior!)
		SMOSequence s2 = new SMOSequence();
		SMOImpl newSMO2 = null;
		try {
			newSMO2 = SMOFactory.createSMO(SMOInterface.NOP);
		} catch (SMOTypeUnknownException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		// switch source and target and set them in the answer
		s2.sourceVersion=this.getTargetMajor();
		s2.targetVersion=this.getSourceMajor();
		newSMO2.setSourceMajor(this.getTargetMajor());
		newSMO2.setSourceMinor(this.getTargetMinor());
		newSMO2.setTargetMajor(this.getSourceMajor());
		newSMO2.setTargetMinor(this.getSourceMinor());
		s2.add(newSMO2);
		
		v.add(s2);	
		return v;
	}
	
	@Override
	public DEDSet getAllDEDs(Schema inputSchema){
		
		Vector<String> excludeList = new Vector<String>();
		excludeList.add(getTableName());
			
		DEDSet ds = getDEDs(inputSchema.clone());
		ds.addAll(getIdentityDEDs(inputSchema,excludeList));
		
		return ds;
	}
	
	@Override
	public Vector<String> getIdentitySQLViews(Schema inputSchema, String dbname) {
	
		Vector<String> excludeList = new Vector<String>();
		excludeList.add(getTableName());
		
		return this.getIdentitySQLView(inputSchema, dbname, excludeList);
	}
	
	public DEDSet getDEDs(Schema inputSchema) {
		DEDSet ds = new DEDSet(getSourceMajor(),getSourceMinor(),getTargetMajor(), getTargetMinor());
		return ds;
	}
	
	@Override
	public String getSQLView(Schema schema, String inputDBname, String outputDBName){
		String sql = ""; // for a drop table SMO no view should be provided
		return sql;
	}
	
	@Override
	public String getSQLDDL(Schema schema, String outputDBname) {
		String sqlddl="";
		
		sqlddl += "DROP TABLE " + outputDBname + "." + this.getTableName()+ ";\n";
		return sqlddl;
	}
	
}
