/**
 * 
 */
package edu.ucla.cs.db.prism.backend.smo;

import java.util.Iterator;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.ded.DED;
import edu.ucla.cs.db.prism.backend.ded.DEDSet;
import edu.ucla.cs.db.prism.backend.ded.QuantifiedDisjunctive;
import edu.ucla.cs.db.prism.backend.ded.Quantifier;
import edu.ucla.cs.db.prism.backend.marsManage.exception.MarsConstantsSuffixException;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.Table;
import edu.ucla.cs.db.prism.backend.schema.exception.IllegalSMOException;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICTable;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ValueConstraint;
import edu.ucla.cs.db.prism.backend.smo.util.Conjunctive;
import edu.ucla.cs.db.prism.backend.smo.util.Predicate;
import edu.ucla.cs.db.prism.backend.smo.util.SMOFactory;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequence;
import edu.ucla.cs.db.prism.backend.smo.util.SMOTypeUnknownException;
import edu.ucla.cs.db.prism.backend.smo.util.Term;

/**
 * @author krl
 *
 */
public class MergeTableSMO extends SMOImpl {

	/**
	 * 
	 */
	public MergeTableSMO() {
		super("MERGE TABLE");
		 setHasPerfectInverse(false);

	}
	
	@Override
	public boolean isInfPreserving() {
		return false;
	}
	
	//Added by swami - for Inf Pres
	@Override
	public boolean isInfPreserving(Schema curSchema) {
		return false;
	}

	@Override
	public String toString() {
		String str = getOperatorType() + " ";	
		str += getTableName();
		if(getTargetConditions() != null)
			str += " WITH " + getTargetConditions().toString();
		str += ", " + getTableName2();
		if(getTargetConditions2() != null)
			str += " WITH " + getTargetConditions2().toString();
		str += " INTO " + getTableName3();
		return str + "\n";
	}
	
	
	@Override
	public Vector<String> getModifiedInputSchemaParts() {
		Vector<String> schemaParts = new Vector<String>();
		schemaParts.add(getTableName2());
		schemaParts.add(getTableName3());
		return schemaParts;	
	}	
	
	@Override
	public Vector<String> getModifiedExistingTables(Schema queryFootprintTargetSchema) {
		Vector<String> modifiedExistingTables = new Vector<String>();
		modifiedExistingTables.add(getTableName2());
		modifiedExistingTables.add(getTableName3());
		return modifiedExistingTables;
	}
	
	public Vector<SMOSequence> getCandidateInverses(Schema inputSchema) {
		Vector<SMOSequence> v = new Vector<SMOSequence>();
		
		// first candidate inverse, partition back... in general the conditions are not known!
		SMOSequence s = new SMOSequence();
		SMOImpl newSMO = null;
		try {
			newSMO = SMOFactory.createSMO(SMOInterface.PARTITION);
		} catch (SMOTypeUnknownException e) {
			e.printStackTrace();
		}
		newSMO.setTableName(this.getTableName3());
		newSMO.setTableName2(this.getTableName());
		newSMO.setTableName3(this.getTableName2());
		if(this.getTargetConditions2() == null || this.getTargetConditions3() == null) {
			newSMO.setComplete(false);
		}
		else{
			newSMO.setTargetConditions2(this.getTargetConditions2());
			newSMO.setTargetConditions3(this.getTargetConditions3());
		}
		// switch source and target and set them in the answer
		s.sourceVersion=this.getTargetMajor();
		s.targetVersion=this.getSourceMajor();
		newSMO.setSourceMajor(this.getTargetMajor());
		newSMO.setSourceMinor(this.getTargetMinor());
		newSMO.setTargetMajor(this.getSourceMajor());
		newSMO.setTargetMinor(this.getSourceMinor());
		s.add(newSMO);
		v.add(s);
		
		/* ---------------------------------------------------------------*/
		/* second candidate inverse, the merge is not reversible -> 
		 * DROP the MERGED table -> query will fail (intended behavior!)*/
		SMOSequence s2 = new SMOSequence();
		SMOImpl newSMO2 = null;
		try {
			newSMO2 = SMOFactory.createSMO(SMOInterface.DROP_TAB);
		} catch (SMOTypeUnknownException e) {
			e.printStackTrace();
		}
		newSMO2.setTableName(this.getTableName3());

		// switch source and target and set them in the answer
		s2.sourceVersion=this.getTargetMajor();
		s2.targetVersion=this.getSourceMajor();
		newSMO2.setSourceMajor(this.getTargetMajor());
		newSMO2.setSourceMinor(this.getTargetMinor());
		newSMO2.setTargetMajor(this.getSourceMajor());
		newSMO2.setTargetMinor(this.getSourceMinor());
		s2.add(newSMO2);
		v.add(s2);
		
		/* ---------------------------------------------------------------*/
		/* third candidate inverse, all the tuple belong to one of the 
		 * two tables (the first conventionally).
		 */
		SMOSequence s3 = new SMOSequence();
		SMOImpl newSMO3 = null;
		SMOImpl newSMO4 = null;
		try {
			newSMO3 = SMOFactory.createSMO(SMOInterface.REN_TAB);
			newSMO4 = SMOFactory.createSMO(SMOInterface.COPY_TAB);
		} catch (SMOTypeUnknownException e) {
			e.printStackTrace();
		}
		newSMO3.setTableName(this.getTableName3());
		newSMO3.setTableName2(this.getTableName());
		
		newSMO4.setTableName(newSMO3.getTableName2());
		newSMO4.setTableName2(this.getTableName2());
		
		// switch source and target and set them in the answer
		s3.sourceVersion=this.getTargetMajor();
		s3.targetVersion=this.getSourceMajor();
		newSMO3.setSourceMajor(this.getTargetMajor());
		newSMO3.setSourceMinor(this.getTargetMinor());
		newSMO3.setTargetMajor(this.getSourceMajor());
		newSMO3.setTargetMinor(this.getSourceMinor());
		
		newSMO4.setSourceMajor(this.getTargetMajor());
		newSMO4.setSourceMinor(this.getSourceMinor());
		newSMO4.setTargetMajor(this.getSourceMajor());
		newSMO4.setTargetMinor(this.getSourceMinor()-1);
		s3.add(newSMO3);
		s3.add(newSMO4);
		v.add(s3);		
		
		
		return v;
	}
	
	public DEDSet getAllDEDs(Schema inputSchema){
		
		Vector<String> excludeList = new Vector<String>();
		excludeList.add(getTableName());
		excludeList.add(getTableName2());
		excludeList.add(getTableName3());
			
		DEDSet ds = getDEDs(inputSchema.clone());
		ds.addAll(getIdentityDEDs(inputSchema,excludeList));
		
		return ds;
	}
	
	@Override
	public Vector<String> getIdentitySQLViews(Schema inputSchema, String dbname) {
	
		Vector<String> excludeList = new Vector<String>();
		excludeList.add(getTableName());
		excludeList.add(getTableName2());
		excludeList.add(getTableName3());
		
		return this.getIdentitySQLView(inputSchema, dbname, excludeList);
	}
	
	
	public DEDSet getDEDs(Schema inputSchema) {
		DEDSet ds = new DEDSet(getSourceMajor(),getSourceMinor(),getTargetMajor(), getTargetMinor());
		
		String dedId = null;
		String dedName = removeSpaces(getOperatorType()) + "_" + getTableName();
		
		Vector<String> sourceTables = new Vector<String>();
		sourceTables.add(getTableName());
		sourceTables.add(getTableName2());
		
		Vector<String> targetTables = new Vector<String>();
		targetTables.add(getTableName3());
	
		forwardDed(inputSchema.clone(), ds, dedId, dedName, true, sourceTables,
				targetTables, getTableName());

		forwardDed(inputSchema.clone(), ds, dedId, dedName, true, sourceTables,
				targetTables, getTableName2());		
		
		backwardDed(inputSchema.clone(), ds, dedId, dedName, false, targetTables, sourceTables,
				getTableName(),getTableName2());	
		
		// hyun 2/27/2008
		// to fix mars specific fix, we add suffix to all constants
		try {
			edu.ucla.cs.db.prism.backend.marsManage.MarsSpecificFix.addSuffixToConstants(ds);
		} catch (MarsConstantsSuffixException e) {
			e.printStackTrace();
		}

		return ds;
	}

	private void forwardDed(Schema inputSchema, DEDSet ds, String dedId,
			String dedName, boolean chaseFlag, Vector<String> sourceTables,
			Vector<String> targetTables, String tt) {
	
		dedName += "_" + tt;		
		dedName += "_dm"; 

		
		DED d = new DED(dedId, dedName, chaseFlag, sourceTables, targetTables, null);
		
		Quantifier q1 = null;
		
		q1 = new Quantifier("all", "R0", "v" + getSourceMajor() + "m" +getSourceMinor() + "_" + tt);
				
		d.globalQuantifiers.quantifiers.add(q1);
		
		QuantifiedDisjunctive tc1 = new QuantifiedDisjunctive(); 
		d.thenClauses.add(tc1);

		q1 = new Quantifier("some", "R1", "v" + getTargetMajor() + "m" + getTargetMinor() + "_" + getTableName3());
		
		tc1.quantifiers.add(q1);
		
		Conjunctive c1 = new Conjunctive();
		tc1.disjunctive.add(c1);
		c1.predicates = new Vector<Predicate>();
		
		Table inTable = inputSchema.getTable(tt);
		try {
			inputSchema.applySMO(this);
		} catch (IllegalSMOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Table outputTable = inputSchema.getTable(getTableName3());
		Iterator<String> inColumns = inTable.getColumns().iterator();
		Iterator<String> outColumns = outputTable.getColumns().iterator();
		
		while(inColumns.hasNext()){
			String inCol = inColumns.next();
			String outCol = outColumns.next();
			
			Predicate p1 = new Predicate();
			c1.predicates.add(p1);
			p1.term1 = new Term("R0", inCol);
			p1.term2 = new Term("R1", outCol);
			p1.op = "=";
		}
		ds.add(d);
	}
	
	private void backwardDed(Schema inputSchema, DEDSet ds, String dedId,
			String dedName, boolean chaseFlag, Vector<String> sourceTables,
			Vector<String> targetTables, String tt2, String tt3) {
	
		dedName += "_" + tt2 + "_" + tt3;
		dedName += "_qr"; 
		
		DED d = new DED(dedId, dedName, chaseFlag, sourceTables, targetTables, null);
		
		Quantifier q1 = null;
		
	   q1 = new Quantifier("all", "R0", "v" + getTargetMajor() + "m" +getTargetMinor() + "_" + getTableName3());
				
		d.globalQuantifiers.quantifiers.add(q1);
		
		QuantifiedDisjunctive tc1 = new QuantifiedDisjunctive(); 
		d.thenClauses.add(tc1);

		QuantifiedDisjunctive tc2 = new QuantifiedDisjunctive(); 
		d.thenClauses.add(tc2);

		
		q1 = new Quantifier("some", "R1", "v" + getSourceMajor() + "m" + getSourceMinor() + "_" + tt2);
		tc1.quantifiers.add(q1);

		Quantifier q2= new Quantifier("some", "R2", "v" + getSourceMajor() + "m" + getSourceMinor() + "_" + tt3);
		tc2.quantifiers.add(q2);
		
		Conjunctive c1 = new Conjunctive();
		tc1.disjunctive.add(c1);
		c1.predicates = new Vector<Predicate>();
		
		Conjunctive c2 = new Conjunctive();
		tc2.disjunctive.add(c2);
		c2.predicates = new Vector<Predicate>();
		
		Table inTable = inputSchema.getTable(tt2);
		try {
			inputSchema.applySMO(this);
		} catch (IllegalSMOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Table outputTable = inputSchema.getTable(getTableName3());
		
		Iterator<String> inColumns = inTable.getColumns().iterator();
		Iterator<String> outColumns = outputTable.getColumns().iterator();
		
		while(inColumns.hasNext()){
			String inCol = inColumns.next();
			String outCol = outColumns.next();
			
			Predicate p1 = new Predicate();
			c1.predicates.add(p1);
			p1.term1 = new Term("R0", inCol);
			p1.term2 = new Term("R1", outCol);
			p1.op = "=";
			
			Predicate p2 = new Predicate();
			c2.predicates.add(p2);
			p2.term1 = new Term("R0", inCol);
			p2.term2 = new Term("R2", outCol);
			p2.op = "=";			
			
		}
		ds.add(d);
	}
	
	public String getSQLView(Schema schema, String inputDBname, String outputDBName){
		String sql = "";
		
		/* 
		 * this is required to ensure correctness of multiple SMOS between 2 major 
		 * versions. Major Versions are represented by name convention in SCHEMA NAMEs, 
		 * while Minor Versions are represeted by name conventions in TABLE NAMES
		*/
		String sourceTabName1 =  this.getTableName(); 
		String targetTabName1 =  this.getTableName(); 
		String sourceTabName2 =  this.getTableName2(); 
		String targetTabName2 =  this.getTableName2(); 
		String sourceTabName3 =  this.getTableName3(); 
		String targetTabName3 =  this.getTableName3(); 
		
		if(this.getSourceMinor()>0){
			sourceTabName1 += "_" + this.getSourceMinor();
			sourceTabName2 += "_" + this.getSourceMinor();
			sourceTabName3 += "_" + this.getSourceMinor();
		}
		if(this.getTargetMinor()>0){
			targetTabName1 += "_" + this.getTargetMinor();	
			targetTabName2 += "_" + this.getTargetMinor();
			targetTabName3 += "_" + this.getTargetMinor();
		}
		
		sql += "CREATE VIEW " + outputDBName + "." + targetTabName3 + " AS \n";
		sql += "( SELECT * FROM " + inputDBname + "." + sourceTabName1 + " )\n";
		sql += " UNION DISTINCT \n"; 
		sql += "( SELECT * FROM " + inputDBname + "." + sourceTabName2 + " );\n"; 	
		return sql;
	}
		
	public String getSQLDDL(Schema schema, String outputDBname) {
		String sqlddl="";
		if(getTableName().equalsIgnoreCase(getTableName3())) {
			sqlddl += "INSERT INTO " + outputDBname + "." + this.getTableName3() + " (SELECT * FROM " + outputDBname + "." + this.getTableName2() + ");\n";
			sqlddl += "DROP TABLE " + outputDBname + "." + this.getTableName2() + ";\n";
			return sqlddl;
		}
		else if(getTableName2().equalsIgnoreCase(getTableName3())) {
			sqlddl += "INSERT INTO " + outputDBname + "." + this.getTableName3() + " (SELECT * FROM " + outputDBname + "." + this.getTableName() + ");\n";
			sqlddl += "DROP TABLE " + outputDBname + "." + this.getTableName() + ";\n";
			return sqlddl;
		}
		sqlddl += "RENAME TABLE " + outputDBname + "." + this.getTableName() + " TO " + outputDBname + "." + this.getTableName3() +";\n";
		sqlddl += "INSERT INTO " + outputDBname + "." + this.getTableName3() + " (SELECT * FROM " + outputDBname + "." + this.getTableName2() + ");\n";
		sqlddl += "DROP TABLE " + outputDBname + "." + this.getTableName2() + ";\n";
		return sqlddl;
	}

	public boolean isICInfPreserving(ICSchema schema) {

		ICTable tab1 = schema.getTable(this.getTableName());
		ICTable tab2 = schema.getTable(this.getTableName2());
		
		Vector<ValueConstraint> vc1list = tab1.getValueConstraint();
		Vector<ValueConstraint> vc2list = tab2.getValueConstraint();
		
		for(ValueConstraint vc1 : vc1list)
			for(ValueConstraint vc2 : vc2list)
				if(vc1.isComplementary(vc2))
					return true;
		return false;
	}		
}
