/**
 * 
 */
package edu.ucla.cs.db.prism.backend.smo;

import java.util.Iterator;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.ded.DED;
import edu.ucla.cs.db.prism.backend.ded.DEDSet;
import edu.ucla.cs.db.prism.backend.ded.QuantifiedDisjunctive;
import edu.ucla.cs.db.prism.backend.ded.Quantifier;
import edu.ucla.cs.db.prism.backend.marsManage.exception.MarsConstantsSuffixException;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.Table;
import edu.ucla.cs.db.prism.backend.schema.exception.IllegalSMOException;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema;
import edu.ucla.cs.db.prism.backend.smo.exception.ComplexPartitionConditionException;
import edu.ucla.cs.db.prism.backend.smo.util.Conjunctive;
import edu.ucla.cs.db.prism.backend.smo.util.Disjunctive;
import edu.ucla.cs.db.prism.backend.smo.util.Predicate;
import edu.ucla.cs.db.prism.backend.smo.util.SMOFactory;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequence;
import edu.ucla.cs.db.prism.backend.smo.util.SMOTypeUnknownException;
import edu.ucla.cs.db.prism.backend.smo.util.Term;
import edu.ucla.cs.db.prism.backend.util.REFConstant;
import edu.ucla.cs.db.prism.exception.NotImplementedException;

/**
 * @author krl
 *
 */
public class PartitionTableSMO extends SMOImpl {

	/**
	 * 
	 */
	public PartitionTableSMO() {
		super("PARTITION TABLE");
		 setHasPerfectInverse(true);
		 setComplete(true);
	}

	public String toString() {
		String str = getOperatorType() + " ";	
		str += getTableName() + " INTO " + getTableName2();
		if(getTargetConditions2() != null)
			str += " WITH " + getTargetConditions2().toString();
		str += ", " + getTableName3();
		if(getTargetConditions3() != null)
			str += " WITH " + getTargetConditions3().toString();
		return str + "\n";
	}
	
	
	public Vector<String> getModifiedInputSchemaParts() {
		Vector<String> schemaParts = new Vector<String>();
		schemaParts.add(getTableName());
		return schemaParts;	
	}	
	
	public Vector<String> getModifiedExistingTables(Schema queryFootprintTargetSchema) {
		Vector<String> modifiedExistingTables = new Vector<String>();
		modifiedExistingTables.add(getTableName());	
		return modifiedExistingTables;
	}
	
	public Vector<SMOSequence> getCandidateInverses(Schema inputSchema) {
		Vector<SMOSequence> v = new Vector<SMOSequence>();
		SMOSequence s = new SMOSequence();
		SMOImpl newSMO = null;
		try {
			newSMO = SMOFactory.createSMO(SMOInterface.MERGE);
		} catch (SMOTypeUnknownException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		newSMO.setTableName(this.getTableName3());
		newSMO.setTableName2(this.getTableName2());
		newSMO.setTableName3(this.getTableName());
		
		//FIXME check no condition needed
		//newSMO.setTargetConditions2(this.getTargetConditions2());
		//newSMO.setTargetConditions3(this.getTargetConditions3());
		
		// switch source and target and set them in the answer
		s.sourceVersion=this.getTargetMajor();
		s.targetVersion=this.getSourceMajor();
		newSMO.setSourceMajor(this.getTargetMajor());
		newSMO.setSourceMinor(this.getTargetMinor());
		newSMO.setTargetMajor(this.getSourceMajor());
		newSMO.setTargetMinor(this.getSourceMinor());
		

		s.add(newSMO);
		v.add(s);
		return v;
	}
	
	public DEDSet getDEDs(Schema inputSchema) {
		DEDSet ds = new DEDSet(getSourceMajor(),getSourceMinor(),getTargetMajor(), getTargetMinor());
		
		String dedId = null;
		String dedName = removeSpaces(getOperatorType()) + "_" + getTableName();
		
		Vector<String> sourceTables = new Vector<String>();
		sourceTables.add(getTableName());
		Vector<String> targetTables = new Vector<String>();
		targetTables.add(getTableName2());
	
		
		genDed(inputSchema.clone(), ds, dedId, dedName, true, sourceTables,
				targetTables, getTableName2(),getTargetConditions2());

		genDed(inputSchema.clone(), ds, dedId, dedName, false, targetTables, sourceTables,
				getTableName2(),getTargetConditions2());
	
		
		targetTables = new Vector<String>();
		targetTables.add(getTableName3());
	
		
		genDed(inputSchema.clone(), ds, dedId, dedName, true, sourceTables,
				targetTables, getTableName3(),getTargetConditions3());


		genDed(inputSchema.clone(), ds, dedId, dedName, false, targetTables, sourceTables,
				getTableName3(),getTargetConditions3());	
		
		// hyun 2/27/2008
		// to fix mars specific fix, we add suffix to all constants
		try {
			edu.ucla.cs.db.prism.backend.marsManage.MarsSpecificFix.addSuffixToConstants(ds);
		} catch (MarsConstantsSuffixException e) {
			e.printStackTrace();
		}

		return ds;
	}

	private void genDed(Schema inputSchema, DEDSet ds, String dedId,
			String dedName, boolean chaseFlag, Vector<String> sourceTables,
			Vector<String> targetTables,  String tt, Disjunctive cond) {
	
		dedName += "_" + tt;
		
		if(chaseFlag)
			dedName += "_dm"; 
		else
			dedName += "_qr"; 
		
		DED d = new DED(dedId, dedName, chaseFlag, sourceTables, targetTables, null);
		
		Quantifier q1 = null;
		
		if(chaseFlag)
			 q1 = new Quantifier("all", "R0", "v" + getSourceMajor() + "m" +getSourceMinor() + "_" + getTableName());
		else
			 q1 = new Quantifier("all", "R0", "v" + getTargetMajor() + "m" +getTargetMinor() + "_" + tt);
				
		d.globalQuantifiers.quantifiers.add(q1);
		
		QuantifiedDisjunctive tc1 = new QuantifiedDisjunctive(); 
		d.thenClauses.add(tc1);

		if(chaseFlag)
			q1 = new Quantifier("some", "R1", "v" + getTargetMajor() + "m" + getTargetMinor() + "_" + tt);
		else
			q1 = new Quantifier("some", "R1", "v" + getSourceMajor() + "m" + getSourceMinor() + "_" + getTableName());
		
		tc1.quantifiers.add(q1);
		
		Conjunctive c1 = new Conjunctive();
		tc1.disjunctive.add(c1);
		c1.predicates = new Vector<Predicate>();
		
		Table inTable = inputSchema.getTable(getTableName());
		try {
			inputSchema.applySMO(this);
		} catch (IllegalSMOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Table outputTable = inputSchema.getTable(tt);
		
		Iterator<String> inColumns = inTable.getColumns().iterator();
		Iterator<String> outColumns = outputTable.getColumns().iterator();
		
		while(inColumns.hasNext()){
			String inCol = inColumns.next();
			String outCol = outColumns.next();
			
			Predicate p1 = new Predicate();
			c1.predicates.add(p1);
			p1.term1 = new Term("R0", inCol);
			p1.term2 = new Term("R1", outCol);
			p1.op = "=";
		}
		if(chaseFlag){
		Iterator<Conjunctive> condIt = cond.conjunctives.iterator();
			while(condIt.hasNext()){
				Conjunctive conCond = condIt.next();
				Iterator<Predicate> cjIt = conCond.predicates.iterator();
				int index = 0;
				while(cjIt.hasNext()){
					Predicate pr = cjIt.next();
					
					boolean switched = false;
					if(pr.op.equals(">")) { // reverse disequalities for uniformity
						switched = true;
						pr.op = "<";
						Term term1 = pr.term1;
						pr.term1 = pr.term2;
						pr.term2 = term1;
					}
					
					if(pr.op.equalsIgnoreCase("<")){ // inequalities
						String relation = REFConstant.getREF(2, getTargetMajor(), getTargetMinor(), "less");
						String relationAlias = "y" + index;
						index++;
						q1 = new Quantifier("some",relationAlias ,  relation);
						tc1.quantifiers.add(q1);
						
						
						if(pr.term2.constant!=null || pr.term1.constant!=null){
							
							if(!switched){
								
								Term temp = pr.term2.clone();
								if(!temp.constant.startsWith("\""))	
									temp.constant = "\"" + temp.constant  + "\"";
	
								c1.predicates.add(new Predicate(new Term(relationAlias, "a0"), "=", new Term("R0",pr.term1.columnName)));
								c1.predicates.add(new Predicate(new Term(relationAlias, "a1"), "=",temp));
							}
							else{
								Term temp = pr.term1.clone();
								if(!temp.constant.startsWith("\""))	
									temp.constant = "\"" + temp.constant  + "\"";
	
								c1.predicates.add(new Predicate(new Term(relationAlias, "a0"), "=", new Term("R0",pr.term2.columnName)));
								c1.predicates.add(new Predicate(new Term(relationAlias, "a1"), "=",temp));			
							}
						}
						
						else{
							c1.predicates.add(new Predicate(new Term(relationAlias, "a0"), "=", new Term("R0",pr.term1.columnName)));
							c1.predicates.add(new Predicate(new Term(relationAlias, "a1"), "=", new Term("R0",pr.term2.columnName)));
						}
					
					
					}else{ // for normal equalities
						if(pr.term2.constant!=null){
							Term temp = pr.term2.clone();
							if(!temp.constant.startsWith("\""))	
								temp.constant = "\"" + temp.constant  + "\"";
							c1.predicates.add(new Predicate(new Term("R0",pr.term1.columnName),pr.op, temp));
						}
						else
							c1.predicates.add(new Predicate(new Term("R0",pr.term1.columnName),pr.op,pr.term2));
						}
					}
			}
		}
		ds.add(d);
	}
	
	
	public DEDSet getAllDEDs(Schema inputSchema){
		
		Vector<String> excludeList = new Vector<String>();
		excludeList.add(getTableName());
		excludeList.add(getTableName2());
		excludeList.add(getTableName3());
			
		DEDSet ds = getDEDs(inputSchema.clone());
		ds.addAll(getIdentityDEDs(inputSchema,excludeList));
		
		return ds;
	}
	
	
	@Override
	public Vector<String> getIdentitySQLViews(Schema inputSchema, String dbname) {
	
		Vector<String> excludeList = new Vector<String>();
		excludeList.add(getTableName());
		excludeList.add(getTableName2());
		excludeList.add(getTableName3());
		
		return this.getIdentitySQLView(inputSchema, dbname, excludeList);
	}
	
	public String getSQLView(Schema schema, String inputDBname, String outputDBName){
		String sql = "";
		
		/* 
		 * this is required to ensure correctness of multiple SMOS between 2 major 
		 * versions. Major Versions are represented by name convention in SCHEMA NAMEs, 
		 * while Minor Versions are represeted by name conventions in TABLE NAMES
		*/
		String sourceTabName1 =  this.getTableName(); 
		String targetTabName1 =  this.getTableName(); 
		String sourceTabName2 =  this.getTableName2(); 
		String targetTabName2 =  this.getTableName2(); 
		String sourceTabName3 =  this.getTableName3(); 
		String targetTabName3 =  this.getTableName3(); 
		
		if(this.getSourceMinor()>0){
			sourceTabName1 += "_" + this.getSourceMinor();
			sourceTabName2 += "_" + this.getSourceMinor();
			sourceTabName3 += "_" + this.getSourceMinor();
		}
		if(this.getTargetMinor()>0){
			targetTabName1 += "_" + this.getTargetMinor();	
			targetTabName2 += "_" + this.getTargetMinor();
			targetTabName3 += "_" + this.getTargetMinor();
		}
		
		String conditions = "";
		Iterator<Conjunctive> cond = this.getTargetConditions2().conjunctives.iterator();
		while(cond.hasNext()){
			Conjunctive c = cond.next();
			Iterator<Predicate> predicates = c.predicates.iterator();
			while(predicates.hasNext()){
				Predicate pr = predicates.next();
				if(pr.term2.constant==null){
					conditions+= "t1." + pr.term1.columnName + pr.op + "t2." + pr.term2.columnName;
				}else{
					conditions+= "t1." + pr.term1.columnName + pr.op + "'" + pr.term2.constant + "'";		
				}
				
			}
		}
		
		sql += "CREATE VIEW " + outputDBName + "." + targetTabName2 + " AS \n";
		sql += "( SELECT * FROM " + inputDBname + "." + sourceTabName1 + " t1 WHERE " + conditions + " );\n";
		
		conditions = "";
		cond = this.getTargetConditions3().conjunctives.iterator();
		while(cond.hasNext()){
			Conjunctive c = cond.next();
			Iterator<Predicate> predicates = c.predicates.iterator();
			while(predicates.hasNext()){
				Predicate pr = predicates.next();
				if(pr.term2.constant==null){
					conditions+= "t1." + pr.term1.columnName + pr.op + "t2." + pr.term2.columnName;
				}else{
					conditions+= "t1." + pr.term1.columnName + pr.op + "'" + pr.term2.constant + "'";
				}
			}
		}
		
		sql += "CREATE VIEW " + outputDBName + "." + targetTabName3 + " AS \n";
		sql += "( SELECT * FROM " + inputDBname + "." + sourceTabName1 + " t1 WHERE " + conditions + " );\n";
		
		return sql;
	}
		
	public String getSQLDDL(Schema schema, String outputDBname) {
		String sqlddl="";
		String temp="";
		String conditions="";
		Iterator<Conjunctive> cond;
		
		if(!getTableName().equalsIgnoreCase(getTableName2())) {
			
			sqlddl += "RENAME TABLE " + outputDBname + "." + getTableName() + " TO " + outputDBname + "." + getTableName2() +";\n";
		}
		sqlddl += "CREATE TABLE " + outputDBname + "." + getTableName3() + " LIKE " + outputDBname + "." + getTableName2() + ";\n";
		if(getTableName().equalsIgnoreCase(getTableName3())) {
			cond = this.getTargetConditions2().conjunctives.iterator();
		}else {		
		cond = this.getTargetConditions3().conjunctives.iterator();
		}
		
		while(cond.hasNext()){
			Conjunctive c = cond.next();
			Iterator<Predicate> predicates = c.predicates.iterator();
			while(predicates.hasNext()){
				Predicate pr = predicates.next();
				if(pr.term2.constant==null){
					conditions+= pr.term1.columnName + pr.op + pr.term2.columnName;
				}else{
					conditions+= pr.term1.columnName + pr.op + "'" + pr.term2.constant + "'";		
				}
				
			}
		}
		if(getTableName().equalsIgnoreCase(getTableName3())) {
			sqlddl += "INSERT INTO " + outputDBname + "." + getTableName2() + " (SELECT * FROM " + outputDBname + "." + getTableName() + " WHERE " + conditions + ");\n";
			sqlddl += "DELETE FROM " + outputDBname + "." + getTableName() + " WHERE " + conditions + ";\n";
			return sqlddl;
		}
		sqlddl += "INSERT INTO " + outputDBname + "." + this.getTableName3() + " (SELECT * FROM " + outputDBname + "." + getTableName2() + " WHERE " + conditions + ");\n";
		sqlddl += "DELETE FROM " + outputDBname + "." + this.getTableName2() + " WHERE " + conditions + ";\n";
		return sqlddl;
	}

	public boolean isICInfPreserving(ICSchema schema) throws NotImplementedException {

		//FIXME improve this condition check, to deal with disjunction and multiple conjuncts
		
		Disjunctive dis2 = getTargetConditions2(); 
		Disjunctive dis3 = getTargetConditions3(); 
		
		if(dis2.countPredicates() >1 || dis3.countPredicates() >1)
			throw new ComplexPartitionConditionException("The management of complex conditions is not fully implemented yet, we cannot check Information Preservation for this SMO");
		
		Predicate p2 = dis2.conjunctives.get(0).predicates.get(0);
		Predicate p3 = dis3.conjunctives.get(0).predicates.get(0);
		
		
		
		
		
		String col1 = p2.term1.columnName;
		String op1 = p2.op;
		String val1 = p2.term2.constant;
		
		String col2 = p3.term1.columnName;
		String op2 = p3.op;
		String val2 = p3.term2.constant;

		
		
		// very simple check, only verify whether the two conditions are expressed on the same column.
		if(!col1.equals(col2) || !p2.term1.tableName.equals(p3.term1.tableName))
			return false;
	
		
		if(val1.compareTo(val2) != 0){
			if(op1.equals("=")){
				if(op2.equals("!="))
					return true;
			}	
		}
		
		if(val1.compareTo(val2) == 0){
			if(op1.equals("=")){
				if(op2.equals("!="))
					return true;
				else
					return false;
			}
			if(op1.equals("!=")){
				if(op2.equals("="))
					return true;
				else
					return false;
			}
			if(op1.equals(">")){
				if(op2.equals("<="))
					return true;
				else
					return false;
			}	
			if(op1.equals("<")){
				if(op2.equals(">="))
					return true;
				else 
					return false;
			}	
			if(op1.equals(">=")){
				if(op2.equals("<"))
					return true;
				else
					return false;
			}	
			if(op1.equals("<=")){
				if(op2.equals(">"))
					return true;	
				else
					return false;	
			}	
			
		}
		return false;
	}	
		
	
}
