/**
 * 
 */
package edu.ucla.cs.db.prism.backend.smo;

import java.util.Iterator;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.ded.DED;
import edu.ucla.cs.db.prism.backend.ded.DEDSet;
import edu.ucla.cs.db.prism.backend.ded.QuantifiedDisjunctive;
import edu.ucla.cs.db.prism.backend.ded.Quantifier;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.Table;
import edu.ucla.cs.db.prism.backend.schema.exception.IllegalSMOException;
import edu.ucla.cs.db.prism.backend.smo.util.Conjunctive;
import edu.ucla.cs.db.prism.backend.smo.util.Predicate;
import edu.ucla.cs.db.prism.backend.smo.util.SMOFactory;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequence;
import edu.ucla.cs.db.prism.backend.smo.util.SMOTypeUnknownException;
import edu.ucla.cs.db.prism.backend.smo.util.Term;

/**
 * @author krl
 *
 */
public class RenameColumnSMO extends SMOImpl {

	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;


	/**
	 * 
	 */
	public RenameColumnSMO() {
		super("RENAME COLUMN");
		 setHasPerfectInverse(true);

	}

	@Override
	public String toString() {
		String str = getOperatorType() + " ";	
		str += getColumnName() + " IN " + getTableName() + " TO " + getColumnName2();
		return str + "\n";
	}
	
	
	@Override
	public Vector<String> getModifiedInputSchemaParts() {
		Vector<String> schemaParts = new Vector<String>();
		schemaParts.add(getTableName() + "." + getColumnName());
		return schemaParts;	
	}	
	
	@Override
	public Vector<String> getModifiedExistingTables(Schema queryFootprintTargetSchema) {
		Vector<String> modifiedExistingTables = new Vector<String>();
		modifiedExistingTables.add(getTableName());
		return modifiedExistingTables;
	}
	
	public Vector<SMOSequence> getCandidateInverses(Schema inputSchema) {
		Vector<SMOSequence> v = new Vector<SMOSequence>();
		SMOSequence s = new SMOSequence();
		SMOImpl newSMO = null;
		try {
			newSMO = SMOFactory.createSMO("RENAME COLUMN");
		} catch (SMOTypeUnknownException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		newSMO.setTableName(this.getTableName());
		newSMO.setColumnName(this.getColumnName2());
		newSMO.setColumnName2(this.getColumnName());
		newSMO.setColumnType(this.getColumnType());
		
		// switch source and target and set them in the answer
		s.sourceVersion=this.getTargetMajor();
		s.targetVersion=this.getSourceMajor();
		newSMO.setSourceMajor(this.getTargetMajor());
		newSMO.setSourceMinor(this.getTargetMinor());
		newSMO.setTargetMajor(this.getSourceMajor());
		newSMO.setTargetMinor(this.getSourceMinor());
		s.add(newSMO);
		v.add(s);
		return v;
	}
	
	
	@Override
	public DEDSet getAllDEDs(Schema inputSchema){
		
		Vector<String> excludeList = new Vector<String>();
		excludeList.add(getTableName());
			
		DEDSet ds = getDEDs(inputSchema.clone());
		ds.addAll(getIdentityDEDs(inputSchema,excludeList));
		
		return ds;
	}
	
	@Override
	public Vector<String> getIdentitySQLViews(Schema inputSchema, String dbname) {
	
		Vector<String> excludeList = new Vector<String>();
		excludeList.add(getTableName());
		
		return this.getIdentitySQLView(inputSchema, dbname, excludeList);
	}
	
	public DEDSet getDEDs(Schema inputSchema) {
		DEDSet ds = new DEDSet(getSourceMajor(),getSourceMinor(),getTargetMajor(), getTargetMinor());
		
		String dedId = null;
		String dedName = removeSpaces(getOperatorType()) + "_" + getTableName() + "_" +getColumnName();
		
	
		Vector<String> sourceTables = new Vector<String>();
		sourceTables.add(getTableName());
		Vector<String> targetTables = new Vector<String>();
		targetTables.add(getTableName());
	
		forwardDed(inputSchema, ds, dedId, dedName, true, sourceTables,
				targetTables);
		backwardDed(inputSchema, ds, dedId, dedName, false,targetTables, sourceTables);

		return ds;
	}

	
	private void forwardDed(Schema inputSchema, DEDSet ds, String dedId,
			String dedName, boolean chaseFlag, Vector<String> sourceTables,
			Vector<String> targetTables) {
		DED d = new DED(dedId, dedName + "_dm" , chaseFlag, sourceTables, targetTables, null);
		
		Quantifier q1 = new Quantifier("all", "R0", "v" + getSourceMajor() + "m" +getSourceMinor() + "_" + getTableName());
		d.globalQuantifiers.quantifiers.add(q1);
		
		QuantifiedDisjunctive tc1 = new QuantifiedDisjunctive(); 
		d.thenClauses.add(tc1);

		q1 = new Quantifier("some", "R1", "v" + getTargetMajor() + "m" + getTargetMinor() + "_" + getTableName());
		tc1.quantifiers.add(q1);
		
		Conjunctive c1 = new Conjunctive();
		tc1.disjunctive.add(c1);
		c1.predicates = new Vector<Predicate>();
		
		Table inTable = inputSchema.getTable(getTableName());
		
		Iterator<String> inColumns = inTable.getColumns().iterator();
		
		while(inColumns.hasNext()){
			String inCol = inColumns.next();
			
			if(!inCol.equalsIgnoreCase(getColumnName())){
				Predicate p1 = new Predicate();
				c1.predicates.add(p1);
				p1.term1 = new Term("R0", inCol);
				p1.term2 = new Term("R1", inCol);
				p1.op = "=";
			}
			else{
				Predicate p1 = new Predicate();
				c1.predicates.add(p1);
				p1.term1 = new Term("R0", inCol);
				p1.term2 = new Term("R1", getColumnName2());
				p1.op = "=";
			}
		}
		
		ds.add(d);
	}
	
	
	private void backwardDed(Schema inputSchema, DEDSet ds, String dedId,
			String dedName, boolean chaseFlag, Vector<String> sourceTables,
			Vector<String> targetTables) {
		DED d = new DED(dedId, dedName + "_qr", chaseFlag, sourceTables, targetTables, null);
		
		Quantifier q1 = new Quantifier("all", "R0", "v" + getTargetMajor() + "m" +getTargetMinor() + "_" + getTableName());
		d.globalQuantifiers.quantifiers.add(q1);
		
		QuantifiedDisjunctive tc1 = new QuantifiedDisjunctive(); 
		d.thenClauses.add(tc1);

		q1 = new Quantifier("some", "R1", "v" + getSourceMajor() + "m" + getSourceMinor() + "_" + getTableName());
		tc1.quantifiers.add(q1);
		
		Conjunctive c1 = new Conjunctive();
		tc1.disjunctive.add(c1);
		c1.predicates = new Vector<Predicate>();
		
		Table inTable = inputSchema.getTable(getTableName());
		try {
			inputSchema.applySMO(this);
		} catch (IllegalSMOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		Table outputTable = inputSchema.getTable(getTableName());
		
		Iterator<String> inColumns = inTable.getColumns().iterator();
		Iterator<String> outColumns = outputTable.getColumns().iterator();
		
		while(inColumns.hasNext()){
			String inCol = inColumns.next();
			String outCol = outColumns.next();
			if(!outCol.equalsIgnoreCase(getColumnName2())){
				Predicate p1 = new Predicate();
				c1.predicates.add(p1);
				p1.term2 = new Term("R1", inCol);
				p1.term1 = new Term("R0", outCol);
				p1.op = "=";
			}
			else{
				Predicate p1 = new Predicate();
				c1.predicates.add(p1);
				p1.term2 = new Term("R1", getColumnName());
				p1.term1 = new Term("R0", outCol);
				p1.op = "=";
			}
		}
		
		ds.add(d);
	}
	
	@Override
	public String getSQLView(Schema schema, String inputDBname, String outputDBName){
		String sql = "";
		String collist = "";
		String collist2 = "";
		
		/* 
		 * this is required to ensure correctness of multiple SMOS between 2 major 
		 * versions. Major Versions are represented by name convention in SCHEMA NAMEs, 
		 * while Minor Versions are represeted by name conventions in TABLE NAMES
		*/
		String sourceTabName1 =  this.getTableName(); 
		String targetTabName1 =  this.getTableName(); 
		String sourceTabName2 =  this.getTableName2(); 
		String targetTabName2 =  this.getTableName2(); 
		String sourceTabName3 =  this.getTableName3(); 
		String targetTabName3 =  this.getTableName3(); 
		
		if(this.getSourceMinor()>0){
			sourceTabName1 += "_" + this.getSourceMinor();
			sourceTabName2 += "_" + this.getSourceMinor();
			sourceTabName3 += "_" + this.getSourceMinor();
		}
		if(this.getTargetMinor()>0){
			targetTabName1 += "_" + this.getTargetMinor();	
			targetTabName2 += "_" + this.getTargetMinor();
			targetTabName3 += "_" + this.getTargetMinor();
		}
		
		sql += "CREATE VIEW " + outputDBName + "." + targetTabName1 + "(";
		
		Iterator<Table> tit = schema.getTables().iterator();
		Iterator<String> cols =null;
		while(tit.hasNext()){
			Table tab = tit.next();
			if(tab.getTableName().equalsIgnoreCase(getTableName())){
				cols = tab.getColumns().iterator();
				break;
			}
		}
		
		while (cols.hasNext()){
			String temp = cols.next();
			if(temp==null)
				System.out.println("NULL COLUMN: " + this.toMetadataString() );
			if(!temp.equalsIgnoreCase(getColumnName())){
				collist+=temp;
				collist2+=temp;
				if(cols.hasNext()){
					collist+=",";
					collist2+=",";
				}
			}else{
				collist+=getColumnName2();
				collist2+=getColumnName();
				if(cols.hasNext()){
					collist+=",";
					collist2+=",";	
				}
			}
		}
		sql+=collist + ") AS ";
		sql += "( SELECT " + collist2 + " FROM " + inputDBname + "." + sourceTabName1 + " );\n"; 
			
		return sql;
	}
	
	
	// getColumnType() method needed.
	@Override
	public String getSQLDDL(Schema schema, String outputDBname) {
		String sqlddl="";
		System.out.println("Column1:"+getColumnName());
		System.out.println("Column2:"+getColumnName2());
		
		if(typeofDBMS.equalsIgnoreCase("MYSQL")) {

			System.out.println("++ "+getColumnType());
			sqlddl += "ALTER TABLE " + outputDBname + "." + getTableName() + " CHANGE COLUMN " + getColumnName() +" "+getColumnName2()+" "+schema.getTable(getTableName()).getColType(getColumnName2())+";\n"; 
		}
		else if(typeofDBMS.equalsIgnoreCase("DB2")) {	
			sqlddl += "ALTER TABLE " + outputDBname + "." + this.getTableName() + " ADD COLUMN " + getColumnName2()+" "+ schema.getTable(getTableName2()).getColType(getColumnName2()) + ";\n";
			sqlddl += "UPDATE " + outputDBname + "." + this.getTableName() + " SET " + this.getColumnName2() + " = " + this.getColumnName() + ";\n";
			sqlddl += "ALTER TABLE " + outputDBname + "." + this.getTableName() + " DROP COLUMN " + this.getColumnName() + ";\n";
		}
		return sqlddl;
	}

}
