/**
 * 
 */
package edu.ucla.cs.db.prism.backend.smo;

import java.util.Iterator;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.ded.DED;
import edu.ucla.cs.db.prism.backend.ded.DEDSet;
import edu.ucla.cs.db.prism.backend.ded.QuantifiedDisjunctive;
import edu.ucla.cs.db.prism.backend.ded.Quantifier;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.Table;
import edu.ucla.cs.db.prism.backend.smo.util.Conjunctive;
import edu.ucla.cs.db.prism.backend.smo.util.Predicate;
import edu.ucla.cs.db.prism.backend.smo.util.SMOFactory;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequence;
import edu.ucla.cs.db.prism.backend.smo.util.SMOTypeUnknownException;
import edu.ucla.cs.db.prism.backend.smo.util.Term;

/**
 * @author krl
 *
 */
public class RenameTableSMO extends SMOImpl {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * 
	 */
	public RenameTableSMO() {
		super("RENAME TABLE");
		 setHasPerfectInverse(true);

	}

	@Override
	public String toString() {
		String str = getOperatorType() + " ";	
		str += getTableName() + " INTO " + getTableName2();
		return str + "\n";
	}
	
	@Override
	public Vector<String> getModifiedInputSchemaParts() {
		Vector<String> schemaParts = new Vector<String>();
			schemaParts.add(getTableName());
		return schemaParts;	
	}	
	
	@Override
	public Vector<String> getModifiedExistingTables(Schema queryFootprintTargetSchema) {
		Vector<String> modifiedExistingTables = new Vector<String>();
		modifiedExistingTables.add(getTableName());
		return modifiedExistingTables;
	}


	public Vector<SMOSequence> getCandidateInverses(Schema inputSchema) {
		Vector<SMOSequence> v = new Vector<SMOSequence>();
		SMOSequence s = new SMOSequence();
		SMOImpl newSMO = null;
		try {
			newSMO = SMOFactory.createSMO("RENAME TABLE");
		} catch (SMOTypeUnknownException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		newSMO.setTableName(this.getTableName2());
		newSMO.setTableName2(this.getTableName());	
		
		// switch source and target and set them in the answer
		s.sourceVersion=this.getTargetMajor();
		s.targetVersion=this.getSourceMajor();
		newSMO.setSourceMajor(this.getTargetMajor());
		newSMO.setSourceMinor(this.getTargetMinor());
		newSMO.setTargetMajor(this.getSourceMajor());
		newSMO.setTargetMinor(this.getSourceMinor());

		s.add(newSMO);
		v.add(s);
		return v;
	}
	
	@Override
	public DEDSet getAllDEDs(Schema inputSchema){
		
		Vector<String> excludeList = new Vector<String>();
		excludeList.add(getTableName());
		excludeList.add(getTableName2());
			
		DEDSet ds = getDEDs(inputSchema.clone());
		ds.addAll(getIdentityDEDs(inputSchema,excludeList));
		
		return ds;
	}
	
	
	@Override
	public Vector<String> getIdentitySQLViews(Schema inputSchema, String dbname) {
	
		Vector<String> excludeList = new Vector<String>();
		excludeList.add(getTableName());
		excludeList.add(getTableName2());
		
		return this.getIdentitySQLView(inputSchema, dbname, excludeList);
	}
	
	public DEDSet getDEDs(Schema inputSchema) {
		DEDSet ds = new DEDSet(getSourceMajor(),getSourceMinor(),getTargetMajor(), getTargetMinor());
		
		
		String dedId = null;
		String dedName = removeSpaces(getOperatorType()) + "_" + getTableName();
		
		Vector<String> sourceTables = new Vector<String>();
		sourceTables.add(getTableName());
		Vector<String> targetTables = new Vector<String>();
		targetTables.add(getTableName2());
	
		forwardDED(inputSchema, ds, dedId, dedName, true, sourceTables, targetTables);
		backwardDED(inputSchema, ds, dedId, dedName, false,  targetTables, sourceTables);	
		
		return ds;
	}



	private void backwardDED(Schema inputSchema, DEDSet ds, String dedId,
			String dedName, boolean chaseFlag, Vector<String> sourceTables,
			Vector<String> targetTables) {
		DED d = new DED(dedId, dedName + "_qr", chaseFlag, sourceTables, targetTables, null);
		
		// TODO CARLO CHECK THIS
		d.tableName =getTableName();
		
		d.sourceRelations = sourceTables;
		d.targetRelations = targetTables;
		
		Quantifier q1 = new Quantifier("all", "R0", "v" + getTargetMajor() + "m" +getTargetMinor() + "_" + getTableName2());
		d.globalQuantifiers.quantifiers.add(q1);
		
		QuantifiedDisjunctive tc1 = new QuantifiedDisjunctive(); 
		d.thenClauses.add(tc1);

		q1 = new Quantifier("some", "R1", "v" + getSourceMajor() + "m" + getSourceMinor() + "_" + getTableName());
		tc1.quantifiers.add(q1);

		Conjunctive c1 = new Conjunctive();
		tc1.disjunctive.add(c1);
		c1.predicates = new Vector<Predicate>();
		
		Table inTable = inputSchema.getTable(getTableName());
		Table outputTable = inputSchema.getTable(getTableName());

		Iterator<String> inColumns = inTable.getColumns().iterator();
		Iterator<String> outColumns = outputTable.getColumns().iterator();
		
		while(inColumns.hasNext()){
			Predicate p1 = new Predicate();
			c1.predicates.add(p1);
			p1.term1 = new Term("R0", inColumns.next());
			p1.term2 = new Term("R1", outColumns.next());
			p1.op = "=";
			
		}
		
		ds.add(d);
	}
	
	private void forwardDED(Schema inputSchema, DEDSet ds, String dedId,
			String dedName, boolean chaseFlag, Vector<String> sourceTables,
			Vector<String> targetTables) {
		DED d = new DED(dedId, dedName+"_dm", chaseFlag, sourceTables, targetTables, null);
		
		// TODO CARLO CHECK THIS
		d.tableName = getTableName();

		d.sourceRelations = sourceTables;
		d.targetRelations = targetTables;
		
		Quantifier q1 = new Quantifier("all", "R0", "v" + getSourceMajor() + "m" +getSourceMinor() + "_" + getTableName());
		d.globalQuantifiers.quantifiers.add(q1);
		
		QuantifiedDisjunctive tc1 = new QuantifiedDisjunctive(); 
		d.thenClauses.add(tc1);

		q1 = new Quantifier("some", "R1", "v" + getTargetMajor() + "m" + getTargetMinor() + "_" + getTableName2());
		tc1.quantifiers.add(q1);

		Conjunctive c1 = new Conjunctive();
		tc1.disjunctive.add(c1);
		c1.predicates = new Vector<Predicate>();
		
		Table inTable = inputSchema.getTable(getTableName());
		Table outputTable = inputSchema.getTable(getTableName());

		if(inTable==null){
			System.err.println("Looking for table: " + getTableName() + " into schema: " + inputSchema.toStringWithMetadata());
		}
		
		Iterator<String> inColumns = inTable.getColumns().iterator();
		Iterator<String> outColumns = outputTable.getColumns().iterator();
		
		while(inColumns.hasNext()){
			Predicate p1 = new Predicate();
			c1.predicates.add(p1);
			p1.term1 = new Term("R0", inColumns.next());
			p1.term2 = new Term("R1", outColumns.next());
			p1.op = "=";
			
		}
		
		ds.add(d);
	}
	
	@Override
	public String getSQLView(Schema schema, String inputDBname, String outputDBName){
		String sql = "";
		
		/* 
		 * this is required to ensure correctness of multiple SMOS between 2 major 
		 * versions. Major Versions are represented by name convention in SCHEMA NAMEs, 
		 * while Minor Versions are represeted by name conventions in TABLE NAMES
		*/
		String sourceTabName1 =  this.getTableName(); 
		String targetTabName1 =  this.getTableName(); 
		String sourceTabName2 =  this.getTableName2(); 
		String targetTabName2 =  this.getTableName2(); 
		
		if(this.getSourceMinor()>0){
			sourceTabName1 += "_" + this.getSourceMinor();
			sourceTabName2 += "_" + this.getSourceMinor();
		
		}
		if(this.getTargetMinor()>0){
			targetTabName1 += "_" + this.getTargetMinor();	
			targetTabName2 += "_" + this.getTargetMinor();
		}
	
		sql += "CREATE VIEW " + outputDBName + "." + targetTabName2 + " AS ";
		sql += "( SELECT t.* "; 
		sql += " FROM " + inputDBname + "." + sourceTabName1 + " t );\n"; 
		
		return sql;
	}
	
	@Override
	public String getSQLDDL(Schema schema, String outputDBname) {
		String sqlddl = "";
		
		sqlddl += "RENAME TABLE " + outputDBname + "." + this.getTableName() + " TO " + outputDBname + "." + this.getTableName2() +";\n";
		
		return sqlddl;
	}

		
}


