package edu.ucla.cs.db.prism.backend.smo;

import java.io.Serializable;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.ded.DED;
import edu.ucla.cs.db.prism.backend.ded.DEDSet;
import edu.ucla.cs.db.prism.backend.ded.QuantifiedDisjunctive;
import edu.ucla.cs.db.prism.backend.ded.Quantifier;
import edu.ucla.cs.db.prism.backend.icmo.MOImpl;
import edu.ucla.cs.db.prism.backend.icmo.MOInterface;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.Table;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICTable;
import edu.ucla.cs.db.prism.backend.smo.exception.NoTableInSchemaException;
import edu.ucla.cs.db.prism.backend.smo.exception.noColumnsMatchInfPres;
import edu.ucla.cs.db.prism.backend.smo.exception.noCommonColumnInfPres;
import edu.ucla.cs.db.prism.backend.smo.util.Conjunctive;
import edu.ucla.cs.db.prism.backend.smo.util.Disjunctive;
import edu.ucla.cs.db.prism.backend.smo.util.Predicate;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequence;
import edu.ucla.cs.db.prism.backend.smo.util.Term;

public abstract class SMOImpl extends MOImpl implements Serializable, SMOInterface, Comparable, MOInterface  {
	static final long serialVersionUID = 0;
	protected String typeofDBMS = "MYSQL";
	private boolean hasPerfectInverse = false;
	private boolean isComplete = true;
	private String operatorType;
	private String tableName3;
	private String tableName2;
	private String tableName;
	private String columnName2;
	private String columnName;
	private String columnType;

	private int smo_id = 0;
	private int source_Major = 0;
	private int source_Minor = 0;
	private int target_Major = 0;
	private int target_Minor = 0;
	private String comment = "";
	
	private SMOSequence inverse = null;
	
	// 10-17-2007 by Hyun 
	// added to support multiple column smo 
	// - create table T(C1, C2, ...)
	//public Vector<String> columnNames; - use columnNameList, instead
	// end
	
	private String defaultConstant = null; // added by Hyun (5/1/07) for ADD COLUMN
	private String functionName = null; // added by Hyun (9/18/07) for ADD COLUMN
	private Vector<String> functionArgs = null; // added by Hyun (9/18/07) for ADD COLUMN 
	//public Conjunctive targetConditions, targetConditions2, targetConditions3;
	private Disjunctive targetConditions3;
	private Disjunctive targetConditions2;
	private Disjunctive targetConditions;
	
	//public boolean finalSMO = false; // added by Hyun (5/3/07) for smo analysis and
									// view composition

	private Vector<String> attributeList2;
	private Vector<String> attributeList3;
	
	// 10-29-2007
	// used for SMO squeeze optimization (ADD COLUMN MULTI, DROP COLUMN MULTI, RENAME COLUMN MULTI
	private Vector<String> columnNameList;
	private Vector<String> columnNameList2;
	private Vector<String> defaultConstantList;
	
	// 2-10-08 by Ham
	// added column and attribute type list variable and (get,set)operations.
	private Vector<String> columnTypeList;
	
	// 2-11-08 by Carlo
	//removed... only CREATE TABLE and ADD COLUMN know about type, the rest is managed in the Schema and Table
	//	private Vector<String> attributeTypeList;
	//	private Vector<String> attributeTypeList2;
	
	public SMOImpl() {
	}
	
	public SMOImpl(String operatorType) {
		this.setOperatorType(operatorType);
		this.setColumnName2(null);
		this.setTargetConditions(null);
		this.setTargetConditions2(null);
		this.setTargetConditions3(null);
	}
	
	public void setTypeofDBMS(String typeofDBMS) {
		this.typeofDBMS = typeofDBMS;
	}
	
	/**
	 * This method should not be invoked, the overrided one by subclass should be used instead
	 * @deprecated
	 */
	public String toString() {
		String str = getOperatorType() + " ";
		
		if(getOperatorType().contentEquals("CREATE TABLE")) {
			str += getTableName();
			if(getColumnNameList() != null && getColumnNameList().size() > 0) {
				for(int i=0; i<getColumnNameList().size(); i++) {
					if(i==0) str += "(";
					str += getColumnNameList().get(i);
					if(i==getColumnNameList().size()-1) 
						str += ")";
					else 
						str += ", ";						
				}
			}
		}
		
		else if(getOperatorType().contentEquals("DROP TABLE")) {
			str += getTableName();
		}
		else if(getOperatorType().contentEquals("RENAME TABLE")) {
			str += getTableName() + " TO " + getTableName2();
		}
		else if(getOperatorType().contentEquals("COPY TABLE")) {
			str += getTableName() + " TO " + getTableName2();
		}
		else if(getOperatorType().contentEquals("PARTITION TABLE")) {
			str += getTableName() + " TO " + getTableName2();
			if(getTargetConditions2() != null)
				str += " WITH " + getTargetConditions2().toString();
			str += ", " + getTableName3();
			if(getTargetConditions3() != null)
				str += " WITH " + getTargetConditions3().toString();
		}
		else if(getOperatorType().contentEquals("MERGE TABLE")) {
			str += getTableName2();
			if(getTargetConditions2() != null)
				str += " WITH " + getTargetConditions2().toString();
			str += ", " + getTableName3();
			if(getTargetConditions3() != null)
				str += " WITH " + getTargetConditions3().toString();
			str += " INTO " + getTableName();
		}
		else if(getOperatorType().contentEquals("ADD COLUMN")) {
			str += getColumnName();
			if(getDefaultConstant() != null)
				str += " AS " + getDefaultConstant();
			else if(getFunctionName() != null) {
				str += " AS " + getFunctionName() + "(";
				if(getFunctionArgs() != null) {
					for(int i=0; i<getFunctionArgs().size(); i++) {
						str += getFunctionArgs().get(i);
						if(i<getFunctionArgs().size()-1)
							str += ", ";
					}
				}
				str += ")";
			}
			str += " INTO " + getTableName();
		}
		else if(getOperatorType().contentEquals("DROP COLUMN")) {
			str += getColumnName() + " FROM " + getTableName();
		}
		else if(getOperatorType().contentEquals("RENAME COLUMN")) {
			str += getColumnName() + " IN " + getTableName() + " TO " + getColumnName2();
		}
		else if(getOperatorType().contentEquals("COPY COLUMN")) {
			str += getColumnName() + " FROM " + getTableName() + " TO " + getTableName2();
			if(getTargetConditions() != null)
				str += " WHERE " + getTargetConditions().toString();
		}
		else if(getOperatorType().contentEquals("COPYBACK COLUMN")) {
			str += getColumnName() + " FROM " + getTableName() + " TO " + getTableName2();
			if(getTargetConditions() != null)
				str += " WHERE " + getTargetConditions().toString();
		}
		else if(getOperatorType().contentEquals("MOVE COLUMN")) {
			str += getColumnName() + " FROM " + getTableName() + " TO " + getTableName2();
			if(getTargetConditions() != null)
				str += " WHERE " + getTargetConditions().toString();
		}
		else if(getOperatorType().contentEquals("ADD COLUMN MULTI")) {
			for(int i=0; i<getColumnNameList().size(); i++) {
				String columnName = getColumnNameList().get(i);
				String defaultConstant = getDefaultConstantList().get(i);
				str += columnName;
				if(defaultConstant != null)
					str += " AS " + defaultConstant;
				if(i<getColumnNameList().size()-1)
					str += ", ";
			}
			str += " INTO " + getTableName();
		}
		else if(getOperatorType().contentEquals("DROP COLUMN MULTI")) {
			for(int i=0; i<getColumnNameList().size(); i++) {
				String columnName = getColumnNameList().get(i);
				str += columnName;
				if(i<getColumnNameList().size()-1)
					str += ", ";
			}
			str += " FROM " + getTableName();
		}
		else if(getOperatorType().contentEquals("RENAME COLUMN MULTI")) {
			for(int i=0; i<getColumnNameList().size(); i++) {
				String columnName = getColumnNameList().get(i);
				String columnName2 = getColumnNameList2().get(i);
				str += columnName + " IN " + getTableName() + " TO " + columnName2;
				if(i<getColumnNameList().size()-1)
					str += ", ";
			}
		}
		else {
			str += "Bad type SMO";
		}

		return str;
	}
	

	/**
	 * This method should not be invoked, the overrided one by subclass should be used instead
	 * @deprecated
	 */
	public Vector<String> getModifiedInputSchemaParts() {
		Vector<String> schemaParts = new Vector<String>();
		
		if(getOperatorType().contentEquals("CREATE TABLE")) {
			// no modified input
		}
		else if(getOperatorType().contentEquals("DROP TABLE")) {
			schemaParts.add(getTableName());
		}
		else if(getOperatorType().contentEquals("RENAME TABLE")) {
			schemaParts.add(getTableName());
		}
		else if(getOperatorType().contentEquals("COPY TABLE")) {
			// no modified input
		}
		else if(getOperatorType().contentEquals("PARTITION TABLE")) {
			schemaParts.add(getTableName());
		}
		else if(getOperatorType().contentEquals("MERGE TABLE")) {
			schemaParts.add(getTableName2());
			schemaParts.add(getTableName3());
		}
		else if(getOperatorType().contentEquals("ADD COLUMN")) {
			// no modified input
		}
		else if(getOperatorType().contentEquals("ADD COLUMN MULTI")) {
			// no modified input
		}
		else if(getOperatorType().contentEquals("DROP COLUMN")) {
			schemaParts.add(getTableName() + "." + getColumnName());
		}
		else if(getOperatorType().contentEquals("DROP COLUMN MULTI")) {
			for(int i=0; i<getColumnNameList().size(); i++)
				schemaParts.add(getTableName() + "." + getColumnNameList().get(i));
		}
		else if(getOperatorType().contentEquals("RENAME COLUMN")) {
			schemaParts.add(getTableName() + "." + getColumnName());
		}
		else if(getOperatorType().contentEquals("RENAME COLUMN MULTI")) {
			for(int i=0; i<getColumnNameList().size(); i++)
				schemaParts.add(getTableName() + "." + getColumnNameList().get(i));
		}
		else if(getOperatorType().contentEquals("COPY COLUMN")) {
			// no modified input
		}
		else if(getOperatorType().contentEquals("MOVE COLUMN")) {
			schemaParts.add(getTableName() + "." + getColumnName());
		}
		else { // error
			schemaParts = null;
		}

		return schemaParts;
	}
	
	/**
	 * This method should not be invoked, the overrided one by subclass should be used instead
	 * @deprecated
	 */
	public Vector<String> getModifiedExistingTables(Schema queryFootprintTargetSchema) {
		Vector<String> modifiedExistingTables = new Vector<String>();
		
		if(getOperatorType().contentEquals("CREATE TABLE")) {
			// no modified input
		}
		else if(getOperatorType().contentEquals("DROP TABLE")) {
			modifiedExistingTables.add(getTableName());
		}
		else if(getOperatorType().contentEquals("RENAME TABLE")) {
			modifiedExistingTables.add(getTableName());
		}
		else if(getOperatorType().contentEquals("COPY TABLE")) {
			// no modified input
		}
		else if(getOperatorType().contentEquals("PARTITION TABLE")) {
			modifiedExistingTables.add(getTableName());
		}
		else if(getOperatorType().contentEquals("MERGE TABLE")) {
			modifiedExistingTables.add(getTableName2());
			modifiedExistingTables.add(getTableName3());
		}
		else if(getOperatorType().contentEquals("ADD COLUMN")) {
			modifiedExistingTables.add(getTableName());
		}
		else if(getOperatorType().contentEquals("ADD COLUMN MULTI")) {
			modifiedExistingTables.add(getTableName());
		}
		else if(getOperatorType().contentEquals("DROP COLUMN")) {
			modifiedExistingTables.add(getTableName());
		}
		else if(getOperatorType().contentEquals("DROP COLUMN MULTI")) {
			modifiedExistingTables.add(getTableName());
		}
		else if(getOperatorType().contentEquals("RENAME COLUMN")) {
			modifiedExistingTables.add(getTableName());
		}
		else if(getOperatorType().contentEquals("RENAME COLUMN MULTI")) {
			modifiedExistingTables.add(getTableName());
		}
		else if(getOperatorType().contentEquals("COPY COLUMN")) {
			// 10-26-2007 hyun - not modified, but the following rule is
			// generated by Copy Column to avoid redundant generation for tableName in
			// sourceVersion
			// tableName,tableName2' (target version) <- tableName,tableName2 (source version)
			if(queryFootprintTargetSchema.getTable(getTableName2()) != null)
				modifiedExistingTables.add(getTableName());
			// end
			modifiedExistingTables.add(getTableName2());
		}
		else if(getOperatorType().contentEquals("MOVE COLUMN")) {
			modifiedExistingTables.add(getTableName());
			modifiedExistingTables.add(getTableName2());
		}
		else { // error
			modifiedExistingTables = null;
		}

		return modifiedExistingTables;
	}
	
	
	public abstract DEDSet getAllDEDs(Schema inputSchema);
	
	/**
	 * this is implmeneted by each SMOImpl extender by invoking getIdentitySQLView with proper exclude list
	 * @param inputSchema
	 * @param dbname
	 * @return
	 */
	public abstract Vector<String> getIdentitySQLViews(Schema inputSchema, String dbname);

	
	public DEDSet getIdentityDEDs(Schema inputSchema, Vector<String> excludeList){
		
		DEDSet ds = new DEDSet(getSourceMajor(),getSourceMinor(),getTargetMajor(), getTargetMinor());
		String dedId = null;
			
		if(inputSchema instanceof ICSchema){
			Iterator<ICTable> tabIt = ((ICSchema)inputSchema).ictables.iterator();
			while(tabIt.hasNext()){
				ICTable tab = tabIt.next();
				if(!isExcluded(tab.getTableName(),excludeList)){
					creatIDDED(inputSchema, ds, tab.getTableName(), dedId, false);
					creatIDDED(inputSchema, ds, tab.getTableName(), dedId, true);
				}	
			}
		}else{	
			Iterator<Table> tabIt = inputSchema.getTables().iterator();
			while(tabIt.hasNext()){
				Table tab = tabIt.next();
				if(!isExcluded(tab.getTableName(),excludeList)){
					creatIDDED(inputSchema, ds, tab.getTableName(), dedId, false);
					creatIDDED(inputSchema, ds, tab.getTableName(), dedId, true);
				}	
			}
		}
		return ds;
	}
	

	/**
	 * this is invoked by each SMOImpl extender to get the identity views
	 * @param s
	 * @param dbname
	 * @param excludeList
	 * @return
	 */
	protected Vector<String> getIdentitySQLView(Schema s, String dbname,Vector<String> excludeList) {
		Vector<String> out = new Vector();
		Iterator<Table> tabIt = s.getTables().iterator();
		
		while(tabIt.hasNext()){
			Table tab = tabIt.next();
			
			if(!isExcluded(tab.getTableName(),excludeList)){
				String view = "CREATE VIEW " + dbname + "_" + this.getTargetMajor() + "." + tab.getTableName();
					if(this.getTargetMinor()!=0)
						view+=  "_" + this.getTargetMinor();
				view += " AS ( SELECT * FROM ";
				view+= dbname + "_" + this.getSourceMajor() + "." + tab.getTableName();
				if(this.getSourceMinor()!=0)
					view+=  "_" + this.getSourceMinor();
				view+=" );";
				out.add(view);
			}	
		}
		return out;
	}
	



	public static boolean isExcluded(String tableName,Vector<String> excludeList) {
		Iterator<String> it = excludeList.iterator();
		while(it.hasNext())
			if(it.next().equalsIgnoreCase(tableName))	
				return true;
		return false;
	}

	private void creatIDDED(Schema inputSchema, DEDSet ds, String tabName,
			String dedId, boolean chaseFlag) {
		
		String dedName = "ID_" + tabName;
		Vector<String> sourceTables = new Vector<String>();
		sourceTables.add(tabName);
		Vector<String> targetTables = new Vector<String>();
		targetTables.add(tabName);
		DED d = null;
		if(chaseFlag)
			d = new DED(dedId, dedName + "_dm", chaseFlag, sourceTables, targetTables, null);
		else
			d = new DED(dedId, dedName + "_qr", chaseFlag, sourceTables, targetTables, null);
		
		Quantifier q1 = null;
		if(chaseFlag)
			q1= new Quantifier("all", "R0", "v" + getSourceMajor() + "m" +getSourceMinor() + "_" + tabName);
		else
			q1= new Quantifier("all", "R0", "v" + getTargetMajor() + "m" +getTargetMinor() + "_" + tabName);
				
		d.globalQuantifiers.quantifiers.add(q1);
		
		QuantifiedDisjunctive tc1 = new QuantifiedDisjunctive(); 
		d.thenClauses.add(tc1);
		
		if(chaseFlag)
			q1 = new Quantifier("some", "R1", "v" + getTargetMajor() + "m" + getTargetMinor() + "_" + tabName);
		else
			q1 = new Quantifier("some", "R1", "v" + getSourceMajor() + "m" + getSourceMinor() + "_" + tabName);
		
		tc1.quantifiers.add(q1);

		Conjunctive c1 = new Conjunctive();
		tc1.disjunctive.add(c1);
		c1.predicates = new Vector<Predicate>();
		Table inTable = inputSchema.getTable(tabName);
		Iterator<String> inColumns = inTable.getColumns().iterator();
		while(inColumns.hasNext()){
			Predicate p1 = new Predicate();
			String col = inColumns.next();
			c1.predicates.add(p1);
			p1.term1 = new Term("R0", col);
			p1.term2 = new Term("R1", col);
			p1.op = "=";

		}
		
		ds.add(d);
	}
	
	
	
	
	
	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#getSmo_id()
	 */
	public int getSmo_id() {
		return smo_id;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#setSmo_id(int)
	 */
	public void setId(int smo_id) {
		this.smo_id = smo_id;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#getSource_Major()
	 */
	public int getSourceMajor() {
		return source_Major;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#setSource_Major(int)
	 */
	public void setSourceMajor(int source_Major) {
		this.source_Major = source_Major;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#getSource_Minor()
	 */
	public int getSourceMinor() {
		return source_Minor;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#setSource_Minor(int)
	 */
	public void setSourceMinor(int source_Minor) {
		this.source_Minor = source_Minor;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#getTarget_Major()
	 */
	public int getTargetMajor() {
		return target_Major;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#setTarget_Major(int)
	 */
	public void setTargetMajor(int target_Major) {
		this.target_Major = target_Major;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#getTarget_Minor()
	 */
	public int getTargetMinor() {
		return target_Minor;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#setTarget_Minor(int)
	 */
	public void setTargetMinor(int target_Minor) {
		this.target_Minor = target_Minor;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#getComment()
	 */
	public String getComment() {
		return comment;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#setComment(java.lang.String)
	 */
	public void setComment(String comment) {
		this.comment = comment;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#setOperatorType(java.lang.String)
	 */
	public void setOperatorType(String operatorType) {
		this.operatorType = operatorType;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#getOperatorType()
	 */
	public String getOperatorType() {
		return operatorType;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#setTableName(java.lang.String)
	 */
	public void setTableName(String tableName) {
		this.tableName = tableName;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#getTableName()
	 */
	public String getTableName() {
		return tableName;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#setTableName2(java.lang.String)
	 */
	public void setTableName2(String tableName2) {
		this.tableName2 = tableName2;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#getTableName2()
	 */
	public String getTableName2() {
		return tableName2;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#setTableName3(java.lang.String)
	 */
	public void setTableName3(String tableName3) {
		this.tableName3 = tableName3;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#getTableName3()
	 */
	public String getTableName3() {
		return tableName3;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#setColumnName(java.lang.String)
	 */
	public void setColumnName(String columnName) {
		this.columnName = columnName;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.db.prim.smo.SMOInterface#getColumnName()
	 */
	public String getColumnName() {
		return columnName;
	}

	public void setColumnName2(String columnName2) {
		this.columnName2 = columnName2;
	}

	public String getColumnName2() {
		return columnName2;
	}

	public void setColumnType(String columnType) {
		this.columnType = columnType;
	}
	
	public String getColumnType() {
		return columnType;
	}
	
	public void setDefaultConstant(String defaultConstant) {
		this.defaultConstant = defaultConstant;
	}

	public String getDefaultConstant() {
		return defaultConstant;
	}

	public void setFunctionName(String functionName) {
		this.functionName = functionName;
	}

	public String getFunctionName() {
		return functionName;
	}

	public void setFunctionArgs(Vector<String> functionArgs) {
		this.functionArgs = functionArgs;
	}

	public Vector<String> getFunctionArgs() {
		return functionArgs;
	}
	
	
	protected Vector<Term> getFunctionArgsAsTerm() {
		Vector<Term> vt = new Vector<Term>();
		java.util.Iterator<String> it = functionArgs.iterator();
		while(it.hasNext())
			vt.add(new Term(getTableName(),it.next()));
		return vt;
	}

	public void setTargetConditions(Disjunctive targetConditions) {
		this.targetConditions = targetConditions;
	}

	public Disjunctive getTargetConditions() {
		return targetConditions;
	}

	public void setTargetConditions2(Disjunctive targetConditions2) {
		this.targetConditions2 = targetConditions2;
	}

	public Disjunctive getTargetConditions2() {
		return targetConditions2;
	}

	public void setTargetConditions3(Disjunctive targetConditions3) {
		this.targetConditions3 = targetConditions3;
	}

	public Disjunctive getTargetConditions3() {
		return targetConditions3;
	}

	public void setAttributeList2(Vector<String> attributeList2) {
		this.attributeList2 = attributeList2;
	}

	public Vector<String> getAttributeList2() {
		return attributeList2;
	}
	
	public String getAttributeList2String() {
		Iterator<String> it = attributeList2.iterator();
		String out="";
		while(it.hasNext()){
			out+=it.next();
			if(it.hasNext())
				out+=", ";
			
		}
		return out;
	}

	
	public void setAttributeList3(Vector<String> attributeList3) {
		this.attributeList3 = attributeList3;
	}

	public Vector<String> getAttributeList3() {
		return attributeList3;
	}
	

	public String getAttributeList3String() {
		Iterator<String> it = attributeList3.iterator();
		String out="";
		while(it.hasNext()){
			out+=it.next();
			if(it.hasNext())
				out+=", ";
			
		}
		return out;
	}

	public void setColumnNameList(Vector<String> columnNameList) {
		this.columnNameList = columnNameList;
	}

	public Vector<String> getColumnNameList() {
		return columnNameList;
	}

	public void setColumnNameList2(Vector<String> columnNameList2) {
		this.columnNameList2 = columnNameList2;
	}

	public Vector<String> getColumnNameList2() {
		return columnNameList2;
	}
	
	public void setColumnTypeList(Vector<String> columnTypeList) {
		this.columnTypeList = columnTypeList;
	}
	public Vector<String> getColumnTypeList() {
		
		//workaround for temporary limitations of the parser
		if(columnTypeList==null){
			Vector<String> temp=new Vector<String>();
			for(int i=0;i<this.columnNameList.size();i++)
				temp.add("varchar(255)");
			return temp;
		}
			
		return columnTypeList;
	}
 
	public void setDefaultConstantList(Vector<String> defaultConstantList) {
		this.defaultConstantList = defaultConstantList;
	}

	public Vector<String> getDefaultConstantList() {
		return defaultConstantList;
	}

	public void setInverse(SMOSequence inverse) {
		this.inverse = inverse;
	}
	
	public SMOSequence getInverse(){
		return inverse;
	}
	
	public static String removeSpaces(String s) {
		  StringTokenizer st = new StringTokenizer(s," ",false);
		  String t="";
		  while (st.hasMoreElements()) t += st.nextElement();
		  return t;
	}

	public boolean hasPerfectInverse() {
		return hasPerfectInverse;
	}
	
	public boolean isInfPreserving() {
		return true;
	}
	//Added by swami - for Inf Pres
	public boolean isInfPreserving(Schema curSchema) throws noColumnsMatchInfPres, noCommonColumnInfPres, NoTableInSchemaException{
		
		return true;
		
	}

	public void setHasPerfectInverse(boolean hasPerfectInverse) {
		this.hasPerfectInverse = hasPerfectInverse;
	}


	public String getSQLView(Schema schema, String inputDB, String outputDB) {
		// TODO Auto-generated method stub
		return null;
	}
	
	//
	public String getSQLDDL(Schema schema, String outputDBname) {
		return null;
	}
	

	public String toMetadataString() {
		return this.source_Major + "." + this.source_Minor + "-" + this.target_Major + "." +this.target_Minor + " " + toString() ;
	}

	public String toSQLInsertString(String metatableName, String dbname, int is_inverse) {
		return "INSERT INTO " + metatableName + "(Source_Major, Source_Minor, Target_Major, " +
				"Target_Minor, schema_name, SMO_TEXT, SMO_TYPE, IS_INVERSE)\n  VALUES('" +
					this.source_Major + "','" + this.source_Minor + "','" + this.target_Major + "','" +this.target_Minor + "','" +
					dbname + "','"+ toString() + "','" + this.operatorType + "','" + is_inverse + "');" ;
	}

	/**
	 * Return if the current SMO is redundant, by default no.
	 * @return
	 */
	public boolean isRedundant() {
		return false;
	}
	

	public int compareTo(Object o) {
		SMOImpl s = (SMOImpl) o;
		if(s.source_Major<this.source_Major)
			return 1;
		if(s.source_Major==this.source_Major && s.source_Minor < this.source_Minor)
			return 1;
		if(s.source_Major==this.source_Major && s.source_Minor == this.source_Minor)
			return 0;
		
		return -1;
	}

	public boolean isComplete() {
		return isComplete;
	}

	public void setComplete(boolean isComplete) {
		this.isComplete = isComplete;
	}
	
	
	public SMOSequence getBestCandidateInverses(Schema inputSchema) {
		return getCandidateInverses(inputSchema).firstElement();
	}


	
}
