package edu.ucla.cs.db.prism.backend.smo.util;
import java.io.Serializable;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.ded.QuantifiedDisjunctive;
import edu.ucla.cs.db.prism.backend.ded.QuantifierSet;
import edu.ucla.cs.db.prism.backend.temporalOptimize.RVSet;
import edu.ucla.cs.db.prism.backend.temporalOptimize.VersionIntervalSet;
import edu.ucla.cs.db.prism.backend.xquery.VarInfo;

public class Conjunctive  implements Serializable {
	static final long serialVersionUID = 0;
	// set of predicates ANDed together
	public Vector<Predicate> predicates;
	public Vector<Disjunctive> disjunctives;
	
	public Conjunctive () {
		predicates = new Vector<Predicate>();
		disjunctives = new Vector<Disjunctive>(); // disable this line to disallow disjunctive within conjunctive
	}
	
	public Conjunctive (Predicate predicate) {
		predicates = new Vector<Predicate>();
		predicates.add(predicate);
		disjunctives = new Vector<Disjunctive>();
	}
	
	public void addAllClone(Conjunctive conjunctive2) { // needs to do some work on join table list
		Conjunctive conjunctive2Clone = conjunctive2.clone();
		this.predicates.addAll(conjunctive2Clone.predicates);
		this.disjunctives.addAll(conjunctive2Clone.disjunctives);
	}

	public void addAll(Conjunctive conjunctive2) {
		this.predicates.addAll(conjunctive2.predicates);
		this.disjunctives.addAll(conjunctive2.disjunctives);
	}
	
	
	@Override
	public Conjunctive clone() {
		Conjunctive newConjunctive = new Conjunctive();
		for(int i=0; i<this.predicates.size(); i++)
			newConjunctive.predicates.add(this.predicates.get(i).clone());
		for(int i=0; i<this.disjunctives.size(); i++)
			newConjunctive.disjunctives.add(this.disjunctives.get(i).clone());
		
		return newConjunctive;
	}
	
	public int size() {
		return predicates.size();
	}
	
	public boolean add(Predicate predicate) {
		return predicates.add(predicate);
	}
	public void addFirst(Predicate predicate) {
		predicates.add(0, predicate);
	}
	public Predicate get(int index) {
		return predicates.get(index);
	}
	public Predicate remove(int index) {
		return predicates.remove(index);
	}
	
	public Term getNonquantifiedTerm(QuantifierSet quantifiers) {
		for(int i=0; i<size(); i++) {
			Term term = predicates.get(i).getNonquantifiedTerm(quantifiers);
			if(term != null)
				return term;
		}
		for(int i=0; i<disjunctives.size(); i++) {
			Term term = disjunctives.get(i).getNonquantifiedTerm(quantifiers);
			if(term != null)
				return term;
		}
		return null;
	}
	
	
	
	public void renameTableNames(String oldTableName, String newTableName) {
		// SMO doesn't use table name alias - change table names to proper aliases
		for(int j=0; j<this.size(); j++) {
			if(this.get(j).term1.tableName != null) {
				if(this.get(j).term1.tableName.contentEquals(oldTableName))
					this.get(j).term1.tableName = newTableName;
			}

			if(this.get(j).term2.tableName != null) {
				if(this.get(j).term2.tableName.contentEquals(oldTableName))
					this.get(j).term2.tableName = newTableName;
			}
		}
		
		for(int i=0; i<this.disjunctives.size(); i++)
			this.disjunctives.get(i).renameTableNames(oldTableName, newTableName);
	}
	
	
	public boolean hasTable(String tableAlias) {
		for(int j=0; j<this.size(); j++) {
			if(this.get(j).term1.tableName == null) {
				if(this.get(j).term1.tableName.contentEquals(tableAlias))
					return true;
			}
			
			if(this.get(j).term2.tableName != null) {
				if(this.get(j).term2.tableName.contentEquals(tableAlias))
					return true;
			}
		}
		return false;
	}
	
	public void renameRelationVariable(String varPrefix, int varIndexIncrement, 
			String varToReplace, String newVarName) {
		
		for(int i=0; i<predicates.size(); i++)
			predicates.get(i).renameRelationVariable(varPrefix, varIndexIncrement,
					varToReplace, newVarName);
		
		for(int i=0; i<disjunctives.size(); i++)
			disjunctives.get(i).renameRelationVariable(varPrefix, varIndexIncrement,
					varToReplace, newVarName);
	}
	
	// if this conjunctive has a term with tableName as a table name,
	// then replace it with "term" by referring to view (it has tableName.attr = term)
	public void replaceTerms(String tableName, QuantifiedDisjunctive viewThenClause) {
		for(int i=0; i<predicates.size(); i++)
			predicates.get(i).replaceTerms(tableName, viewThenClause);
		
		for(int i=0; i<disjunctives.size(); i++)
			disjunctives.get(i).replaceTerms(tableName, viewThenClause);
	}
	
	public Term getTermEqualToRA(String tableName, String columnName, boolean removePredicate) {
		for(int i=0; i<predicates.size(); i++) {
			Predicate p = predicates.get(i);
			if(!p.op.equals("=")) continue;
			boolean found = false;
			Term termToReturn = null;
			
			// if columnName == null don't check columnName
			if(p.term1.constant == null &&
					p.term1.tableName.equals(tableName) &&
					(columnName == null || p.term1.columnName.equals(columnName))) {
				found = true;
				termToReturn = p.term2;
			}
			else if(p.term2.constant == null &&
					p.term2.tableName.equals(tableName) &&
					(columnName == null || p.term2.columnName.equals(columnName))) {
				found = true;
				termToReturn = p.term1;
			}
			if(found) {
				if(removePredicate) predicates.remove(i);
				return termToReturn;
			}
		}
		for(int i=0; i<disjunctives.size(); i++) {
			Term term = disjunctives.get(i).getTermEqualToRA(tableName, columnName, removePredicate);
			if(term != null) return term;
		}
		return null;
	}
	
	/*
	public Vector<Term> getTermEQualToRAClosure(String tableName, String columnName) {
		Vector<Term> terms = new Vector<Term>();
		terms.add(new Term(tableName, columnName));

		while(true) {
			boolean termAdded = false;
			for(int i=0; i<this.size(); i++) {
				if(!this.get(i).op.equals("=")) continue;
				Term term1 = this.get(i).term1;
				Term term2 = this.get(i).term2;
				
				int termsSize = terms.size();
				for(int j=0; j<termsSize; j++) {
					if(term1.equals(terms.get(j)) && !terms.contains(term2)) {
						terms.add(term2);
						termAdded = true;
						//System.out.println("term1 match in " + this.get(i).toString() 
						//		+ " -> terms: " + terms.toString());
						break;
					}
					else if(term2.equals(terms.get(j)) && !terms.contains(term1)) {
						terms.add(term1);
						termAdded = true;
						//System.out.println("term2 match in " + this.get(i).toString()
						//		+ " -> terms: " + terms.toString());
						break;
					}
				}
			}
			
			if(!termAdded)
				break;
		}
		
		return terms;
	}
	*/
	public Vector<Term> getTermEQualToRAClosure(String tableName, String columnName) {
		return getEqualTermClosure(new Term(tableName, columnName));
	}
	
	
	
	
	public Vector<Term> getEqualTermClosure(Term term) {
		Vector<Term> terms = new Vector<Term>();
		terms.add(term.clone());

		while(true) {
			boolean termAdded = false;
			for(int i=0; i<this.size(); i++) {
				if(!this.get(i).op.equals("=")) continue;
				Term term1 = this.get(i).term1;
				Term term2 = this.get(i).term2;
				
				int termsSize = terms.size();
				for(int j=0; j<termsSize; j++) {
					if(term1.equals(terms.get(j)) && !terms.contains(term2)) {
						terms.add(term2);
						term2.chosenAsEqual = true;
						termAdded = true;
						//System.out.println("term1 match in " + this.get(i).toString() 
						//		+ " -> terms: " + terms.toString());
						break;
					}
					else if(term2.equals(terms.get(j)) && !terms.contains(term1)) {
						terms.add(term1);
						term1.chosenAsEqual = true;
						termAdded = true;
						//System.out.println("term2 match in " + this.get(i).toString()
						//		+ " -> terms: " + terms.toString());
						break;
					}
				}
			}
			
			if(!termAdded)
				break;
		}
		
		return terms;
	}
	
	// utilize Term.chosenAsEqual flag
	// skip testing it if it's true - already examined and included in the previous while loop
	public Vector<Term> getEqualTermClosureOptimized(Term term) {
		Vector<Term> terms = new Vector<Term>();
		terms.add(term.clone());

		//int count=0;
		while(true) {
			//count++;
			//System.out.println("looping " + count);
			boolean termAdded = false;
			for(int i=0; i<this.size(); i++) {
				if(!this.get(i).op.equals("=")) continue;
				Term term1 = this.get(i).term1;
				Term term2 = this.get(i).term2;
				
				if(term1.chosenAsEqual && term2.chosenAsEqual)
					continue;
				
				int termsSize = terms.size();
				for(int j=0; j<termsSize; j++) {
					if(!term2.chosenAsEqual && 
							term1.equals(terms.get(j))) {
						term1.chosenAsEqual = true;
						term2.chosenAsEqual = true;

						if(!terms.contains(term2)) {
							terms.add(term2);
							termsSize++;
							termAdded = true;
						}
						//System.out.println("term1 match in " + this.get(i).toString() 
						//		+ " -> terms: " + terms.toString());
						break;
					}
					
					else if(!term1.chosenAsEqual &&
							term2.equals(terms.get(j))) {
						
						term1.chosenAsEqual = true;
						term2.chosenAsEqual = true;
						
						if(!terms.contains(term1)) {
							terms.add(term1);
							termsSize++;
							termAdded = true;
						}
						//System.out.println("term2 match in " + this.get(i).toString()
						//		+ " -> terms: " + terms.toString());
						break;
					}
				}
			}
			
			if(!termAdded)
				break;
		}
		
		return terms;
	}
	
	/**
	 * utilize R0.el = R1.el to infer R0.ts=R1.ts, R0.te=R1.te, R0.text=R1.text
	 * it's a quite loose rule... (more like a quick-and-dirty fix)
	 * @param tableName
	 * @param columnName
	 * @return
	 */
	public Vector<Term> getTermEQualToRAClosureUsingElementEqual(String tableName, String columnName) {
		Vector<Term> terms = new Vector<Term>();
		terms.add(new Term(tableName, columnName));

		while(true) {
			boolean termAdded = false;
			for(int i=0; i<this.size(); i++) {
				if(!this.get(i).op.equals("=")) continue;
				Term term1 = this.get(i).term1;
				Term term2 = this.get(i).term2;
				
				int termsSize = terms.size();
				for(int j=0; j<termsSize; j++) {
					if(term1.equals(terms.get(j)) && !terms.contains(term2)) {
						if(term2.isConstantTerm()) {
							terms.add(0, term2);
							return terms; // stop here
						}
						else
							terms.add(term2);
						term2.chosenAsEqual = true;
						termAdded = true;
						//System.out.println("term1 match in " + this.get(i).toString() 
						//		+ " -> terms: " + terms.toString());
						break;
					}
					else if(term2.equals(terms.get(j)) && !terms.contains(term1)) {
						if(term1.isConstantTerm()) {
							terms.add(0, term1);
							return terms; // stop here
						}
						else
							terms.add(term1);
						term1.chosenAsEqual = true;
						termAdded = true;
						//System.out.println("term2 match in " + this.get(i).toString()
						//		+ " -> terms: " + terms.toString());
						break;
					}
					
					else if(term1.isRATerm() && term2.isRATerm() &&
							term1.tableName.equals(terms.get(j).tableName) && 
							term1.columnName.equals("el") &&
							term2.columnName.equals("el")) {
						Term newTerm = new Term(term2.tableName, columnName); // ts, te, text

						if(!terms.contains(newTerm)) {
							terms.add(newTerm);
							term2.chosenAsEqual = true;
							termAdded = true;
							break;
						}
					}
					else if(term1.isRATerm() && term2.isRATerm() &&
							term2.tableName.equals(terms.get(j).tableName) && 
							term1.columnName.equals("el") &&
							term2.columnName.equals("el")) {
						Term newTerm = new Term(term1.tableName, columnName); // ts, te, text

						if(!terms.contains(newTerm)) {
							terms.add(newTerm);
							term1.chosenAsEqual = true;
							termAdded = true;
							break;
						}
					}
					
				}
			}
			
			if(!termAdded)
				break;
		}
		
		return terms;
	}
	
	
	// if a<b. b<c are in, produce a<c as well
	public Conjunctive getComparisonClosure() {
		Conjunctive newConjunctive = this.clone();
		
		while(true) {
			boolean termAdded = false;
			for(int i=0; i<this.size(); i++) {
				if(!this.get(i).op.equals("<"))
					continue;
				for(int j=i+1; j<this.size(); j++) {
					if(this.get(i).term2.equals(this.get(j).term1)) {
						// add predicat_i.term1 < predicate_j.term2, if not already in
						Predicate newPredicate = 
							new Predicate(this.get(i).term1, "<", this.get(j).term2);
						if(!this.contains(newPredicate))
							this.add(newPredicate);
					}
					if(this.get(j).term2.equals(this.get(i).term1)) {
						// add predicat_j.term1 < predicate_i.term2, if not already in
						Predicate newPredicate = 
							new Predicate(this.get(j).term1, "<", this.get(i).term2);
						if(!this.contains(newPredicate)) {
							termAdded = true;
							this.add(newPredicate);
						}
					}
					
				}
			}
			
			if(!termAdded)
				break;
		}
		return newConjunctive;
	}
	
	// non-recursive - consider only the predicates directly under this
	//                 doesn't cosider the disjunctives inside conjunctive
	public boolean contains(Predicate p) {
		for(int i=0; i<this.size(); i++)
			if(this.get(i).equals(p))
				return true;
		return false;
	}
	/**
	 * add related terms to terms using this conjunctive's predicates
	 * term2 is a related terms of term1, if there's a predicate p, term1 op term2 (or term2 op term1)
	 * op: "=", "<"
	 * 
	 * @param terms
	 * @return all predicates used as above
	 */
	public Conjunctive addRelatedTerms(Vector<Term> terms) {
		Conjunctive closurePredicates = new Conjunctive();
		
		while(true) {
			boolean termAdded = false;
			for(int i=0; i<this.size(); i++) {
				// step 1: when arguments are in terms, function call is included into terms
				if(this.get(i).term1.usingOnly(terms) &&
						!this.get(i).term1.isConstantTerm() &&
						!terms.contains(this.get(i).term1)) {
					termAdded = true;
					terms.add(this.get(i).term1);
					//closurePredicates.add(this.get(i));
				}
				else if(this.get(i).term2.usingOnly(terms) &&
						!this.get(i).term2.isConstantTerm() &&
						!terms.contains(this.get(i).term2)) {
					termAdded = true;
					terms.add(this.get(i).term2);
					//closurePredicates.add(this.get(i));
				}
				
				// step 2: two-term relationship
				if(terms.contains(this.get(i).term1) && 
						!terms.contains(this.get(i).term2)) {
					termAdded = true;
					terms.add(this.get(i).term2); // TODO: unnest functions and add them too
					closurePredicates.add(this.get(i));
				}
				else if(terms.contains(this.get(i).term2) && 
						!terms.contains(this.get(i).term1)) {
					termAdded = true;
					terms.add(this.get(i).term1);
					closurePredicates.add(this.get(i));
				}
				
				// step 3: two terms are in "terms", but this predicate is a new relationship
				if(terms.contains(this.get(i).term1) && 
				   terms.contains(this.get(i).term2) &&
				   !closurePredicates.contains(this.get(i))) {
					closurePredicates.add(this.get(i));
				}
			}
			if(!termAdded)
				break;
		}

		return closurePredicates; // also, terms is updated, which is another return value
	}

	/*
	 * op1: "=", op2: "="
	 * a = b, a = c -> b=c
	 * a = b, c = a -> b=c
	 * a = b, b = c -> a=c
	 * a = b, c = b -> a=c
	 * 
	 * op1: "=", op2: "<"
	 * a = b, a < c -> b<c
	 * a = b, c < a -> c<b
	 * a = b, b < c -> a<c
	 * a = b, c < b -> c<a
	 * 
	 * op1: "<", op2: "<"
	 * a<b, a<c -> N/A
	 * a<b, c<a -> c<b
	 * a<b, b<c -> a<c
	 * a<b, c<b -> N/A
	 * 
	 *  case 1: a=b, b=c -> a=c
	 *  case 2: a<b, b<c -> a<c
	 *  case 3: a<b, b=c -> a<c
	 *  case 4 :a=b, b<c -> a<c
	 */	
	public Conjunctive getEqualityAndComparisonClosure() {
		Conjunctive newConjunctive = this.clone();
		
		while(true) {
			boolean termAdded = false;
			int newConjunctiveSize = newConjunctive.size();
			for(int i=0; i<newConjunctiveSize; i++) {
				for(int j=i+1; j<newConjunctiveSize; j++) {
					Predicate p1 = newConjunctive.get(i);
					Predicate p2 = newConjunctive.get(j);
					Predicate p3 = null;
					// swap order if op1: "<", op2: "="
					if(p1.op.equals("<") && p2.op.equals("=")) {
						Predicate tempP = p1;
						p1 = p2;
						p2= tempP;
					}

					if(p1.op.equals("=") && p2.op.equals("=")) {
						if(p1.term1.equals(p2.term1))
							p3 = new Predicate(p1.term2, "=", p2.term2);
						else if(p1.term1.equals(p2.term2))
							p3 = new Predicate(p1.term2, "=", p2.term1);
						else if(p1.term2.equals(p2.term1))
							p3 = new Predicate(p1.term1, "=", p2.term2);
						else if(p1.term2.equals(p2.term2))
							p3 = new Predicate(p1.term1, "=", p2.term1);
					}
					else if(p1.op.equals("=") && p2.op.equals("<")) {
						if(p1.term1.equals(p2.term1))
							p3 = new Predicate(p1.term2, "<", p2.term2);
						else if(p1.term1.equals(p2.term2))
							p3 = new Predicate(p2.term1, "<", p1.term2);
						else if(p1.term2.equals(p2.term1))
							p3 = new Predicate(p1.term1, "<", p2.term2);
						else if(p1.term2.equals(p2.term2))
							p3 = new Predicate(p2.term1, "<", p1.term1);
					}
					else if(p1.op.equals("<") && p2.op.equals("<")) {
						if(p1.term1.equals(p2.term2))
							p3 = new Predicate(p2.term1, "<", p1.term2);
						else if(p1.term2.equals(p2.term1))
							p3 = new Predicate(p1.term1, "<", p2.term2);
					}
					
					
					if(p3 != null && !newConjunctive.contains(p3)) {
						termAdded = true;
						newConjunctive.add(p3);
						newConjunctiveSize++;
					}
				}
			}
			//System.out.println("newConjunctive : " + newConjunctive.toString() + "\n\n\n");
			if(!termAdded)
				break;
		}
		
		return newConjunctive;
	}
	
	public Vector<Term> getAllTermsInEquality() {
		Vector<Term> terms = new Vector<Term>();
		
		for(int i=0; i<predicates.size(); i++) {
			Predicate p = predicates.get(i);
			if(!p.op.equals("=")) continue;
			terms.add(p.term1);
			terms.add(p.term2);
		}
		
		return terms;		
	}
	
	// similar to getAllTermInEquality() except that this returns also
	// the function argument terms
	public Vector<Term> getAllTermsDeepInEquality() {
		Vector<Term> terms = new Vector<Term>();
		
		for(int i=0; i<predicates.size(); i++) {
			Predicate p = predicates.get(i);
			if(!p.op.equals("=")) continue;
			
			if(p.term1.isFunctionTerm()) {
				terms.addAll(p.term1.getAllTermsDeep());
				//System.out.println(p.term1.getAllTermsDeep());
			}
			else
				terms.add(p.term1);

			if(p.term2.isFunctionTerm()) {
				terms.addAll(p.term2.getAllTermsDeep());
				//System.out.println(p.term2.getAllTermsDeep());
			}
			else
				terms.add(p.term2);
		}
		
		return terms;		
	}
	

	
	/*
	public Conjunctive getEqualityAndComparisonClosure() {
		Conjunctive newConjunctive = this.clone();
		
		while(true) {
			boolean termAdded = false;
			int newConjunctiveSize = newConjunctive.size();
			for(int i=0; i<newConjunctiveSize; i++) {
				for(int j=i+1; j<newConjunctiveSize; j++) {
					Predicate p1 = null;
					Predicate p2 = null; // we set it so that p1.term2.equals(p2.term1)
					if(newConjunctive.get(i).term2.equals(newConjunctive.get(j).term1) && 
					  !newConjunctive.get(i).term1.equals(newConjunctive.get(j).term2)) {
						p1 = newConjunctive.get(i);
						p2 = newConjunctive.get(j);
					}
					else if(newConjunctive.get(j).term2.equals(newConjunctive.get(i).term1) &&
					  !newConjunctive.get(j).term1.equals(newConjunctive.get(i).term2)) {
						p1 = newConjunctive.get(j);
						p2 = newConjunctive.get(i);
					}
					System.out.println("i=" + i + " j=" + j);
					if(p1 == null || p2 == null) {
						System.out.println("both are nulls");
						continue;
					}

					Predicate newPredicate = null;
					if(p1.op.equals("=") && p2.op.equals("=")) // case 1
						newPredicate = new Predicate(p1.term1, "=", p2.term2);
					else // case 2, 3, 4
						newPredicate = new Predicate(p1.term1, "<", p2.term2);
					
					if(!newConjunctive.contains(newPredicate)) {
						termAdded = true;
						newConjunctive.add(newPredicate);
						newConjunctiveSize++;
					}
				}
			}
			System.out.println("newConjunctive : " + newConjunctive.toString() + "\n\n\n");
			if(!termAdded)
				break;
		}
		
		return newConjunctive;
	}
*/
	
	public int countPredicates() {
		int count = predicates.size();
		for(int i=0; i<disjunctives.size(); i++)
			count += disjunctives.get(i).countPredicates();
		return count;
	}
	
	public boolean isUsingVariable(String var) {
		for(int i=0; i<predicates.size(); i++)
			if(predicates.get(i).isUsingVariable(var))
				return true;
		for(int i=0; i<disjunctives.size(); i++)
			if(disjunctives.get(i).isUsingVariable(var))
				return true;
		return false;
	}
	
	// remove empty conjunctives
	public void simplify() {
		int size = disjunctives.size();
		//System.out.println("predicate size within conjunctive: " + predicates.size());
		//System.out.println("disjunctive size within conjunctive: " + size);
		for(int i=0; i<size; i++) {
			disjunctives.get(i).simplify();
			if(disjunctives.get(i).conjunctives.size() == 0) {
				disjunctives.remove(i);
				i--;
				size--;
			}
			else if(disjunctives.get(i).conjunctives.size() == 1) {
				predicates.addAll(disjunctives.get(i).conjunctives.get(0).predicates);
				disjunctives.remove(i);
				i--;
				size--;
			}
		}
	}

	public void removeTautologies() {
		int size = this.size();
		for(int i=0; i<size; i++) {
			if(this.get(i).isTautology()) {
				this.remove(i);
				i--;
				size--;
			}
		}
	}
	
	public RVSet computeRVSetFromTemporalSelection(Vector<String> relationAliases, 
			VersionIntervalSet viSet, VarInfo varInfo) {
		RVSet rvSet = new RVSet();
		for(int i=0; i<predicates.size(); i++) {
			RVSet tempRVSet = predicates.get(i).computeRVSetFromTemporalSelection(viSet, varInfo);
			if(tempRVSet != null)
				rvSet = rvSet.setUnionElementIntersection(tempRVSet);
		}
		for(int i=0; i<disjunctives.size(); i++) {
			RVSet tempRVSet = disjunctives.get(i).computeRVSetFromTemporalSelection(
					relationAliases, viSet, varInfo);
			rvSet = rvSet.setUnionElementIntersection(tempRVSet);
		}
		RVSet rvSetFull = new RVSet(relationAliases, viSet);
		return rvSet.setUnionElementIntersection(rvSetFull);
	}
	
	public RVSet computeRVSetFromTemporalJoin(RVSet prevUniverse, VarInfo varInfo) {
		RVSet rvSet = new RVSet();
		for(int i=0; i<predicates.size(); i++) {
			RVSet tempRVSet = predicates.get(i).computeRVSetFromTemporalJoin(prevUniverse, varInfo);
			if(tempRVSet != null)
				rvSet = rvSet.setUnionElementIntersection(tempRVSet);
		}
		for(int i=0; i<disjunctives.size(); i++) {
			RVSet tempRVSet = disjunctives.get(i).computeRVSetFromTemporalJoin(
					prevUniverse, varInfo);
			rvSet = rvSet.setUnionElementIntersection(tempRVSet);
		}
		return rvSet.setUnionElementIntersection(prevUniverse);
	}

	//public String toStringOld() {
	//	return predicates.toString();
	//}
	@Override
	public String toString() {
		String str="";
		int numQuantifiersInLine = 3;
		

		for(int i=0; i<predicates.size(); i++) {
			str += predicates.get(i).toString();

			if(i < predicates.size()-1) {
				str += " and ";
				if(i % numQuantifiersInLine == numQuantifiersInLine-1)
					str += "\n";
			}
		}
		
		if(predicates.size() > 0 && disjunctives.size() > 0) 
			str += " and ";
		
		//str += disjunctives.size();
		
		for(int i=0; i<disjunctives.size(); i++) {
			str += "(" + disjunctives.get(i).toString() + ")";
			if(i<disjunctives.size()-1)
				str += " and ";
		}
		
		return str;
	}
}
