package edu.ucla.cs.db.prism.backend.smo.util;

import java.io.Serializable;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.SQL.SelectAttribute;
import edu.ucla.cs.db.prism.backend.ded.QuantifiedDisjunctive;
import edu.ucla.cs.db.prism.backend.ded.QuantifierSet;
import edu.ucla.cs.db.prism.backend.temporalOptimize.RVSet;
import edu.ucla.cs.db.prism.backend.temporalOptimize.VersionIntervalSet;
import edu.ucla.cs.db.prism.backend.xquery.VarInfo;

public class Disjunctive  implements Serializable {
	static final long serialVersionUID = 0;
	// set of predicates ANDed together
	public Vector<Conjunctive> conjunctives;
	
	public Disjunctive () {
		conjunctives = new Vector<Conjunctive>();
		//conjunctives.add(new Conjunctive()); // temporary fix; may not be good to have it
	}

	public Disjunctive (Predicate p) {
		conjunctives = new Vector<Conjunctive>();
		conjunctives.add(new Conjunctive());
		conjunctives.get(0).add(p);
	
	}

	
	public void addAllClone(Disjunctive disjunctive2) { // needs to do some work on join table list
		Disjunctive disjunctive2Clone = disjunctive2.clone();
		this.conjunctives.addAll(disjunctive2Clone.conjunctives);
	}

	public void addAll(Disjunctive disjunctive2) {
		this.conjunctives.addAll(disjunctive2.conjunctives);
	}
	
	
	
	@Override
	public Disjunctive clone() {
		Disjunctive newDisjunctive = new Disjunctive();
		for(int i=0; i<this.conjunctives.size(); i++)
			newDisjunctive.conjunctives.add(this.conjunctives.get(i).clone());
		return newDisjunctive;
	}
	
	public int size() {
		return conjunctives.size();
	}
	
	public boolean add(Conjunctive conjunctive) {
		return conjunctives.add(conjunctive);
	}
	public void addFirst(Conjunctive conjunctive) {
		conjunctives.add(0, conjunctive);
	}
	public Conjunctive get(int index) {
		return conjunctives.get(index);
	}
	public Conjunctive remove(int index) {
		return conjunctives.remove(index);
	}
	
	public Term getNonquantifiedTerm(QuantifierSet quantifiers) {
		for(int i=0; i<size(); i++) {
			Term term = conjunctives.get(i).getNonquantifiedTerm(quantifiers);
			if(term != null)
				return term;
		}
		return null;
	}
	

	public void renameTableNames(String oldTableName, String newTableName) {
		// SMO doesn't use table name alias - change table names to proper aliases
		for(int i=0; i<this.conjunctives.size(); i++) {
			this.conjunctives.get(i).renameTableNames(oldTableName, newTableName);
		}
	}
	
	
	public void renameRelationVariable(String varPrefix, int varIndexIncrement, 
										String varToReplace, String newVarName) {
		for(int i=0; i<conjunctives.size(); i++)
			conjunctives.get(i).renameRelationVariable(varPrefix, varIndexIncrement,
					varToReplace, newVarName);
	}
	
	
	// if this disjunctive has a term with tableName as a table name,
	// then replace it with "term" by referring to view (it has tableName.attr = term)
	public void replaceTerms(String tableName, QuantifiedDisjunctive viewThenClause) {
		for(int i=0; i<conjunctives.size(); i++)
			conjunctives.get(i).replaceTerms(tableName, viewThenClause);
	}
	
	public Term getTermEqualToRA(String tableName, String columnName, boolean removePredicate) {
		for(int i=0; i<conjunctives.size(); i++) {
			Term term = conjunctives.get(i).getTermEqualToRA(tableName, columnName, removePredicate);
			if(term != null) return term;
		}
		return null;
	}
	
	public int countPredicates() {
		int count = 0;
		for(int i=0; i<conjunctives.size(); i++)
			count += conjunctives.get(i).countPredicates();
		return count;
	}
	
	public boolean isUsingVariable(String var) {
		for(int i=0; i<conjunctives.size(); i++)
			if(conjunctives.get(i).isUsingVariable(var))
				return true;
		return false;
	}
	
	// remove empty conjunctives
	public void simplify() {
		//System.out.println("simplifying disjunctive");
		int size = conjunctives.size();
		for(int i=0; i<size; i++) {
			conjunctives.get(i).simplify();
			if(conjunctives.get(i).predicates.size() == 0 &&
					conjunctives.get(i).disjunctives.size() == 0) {
				conjunctives.remove(i);
				i--;
				size--;
			}
		}
	}
	
	public void removeTautologies() {
		int size = conjunctives.size();
		for(int i=0; i<size; i++) {
			conjunctives.get(i).removeTautologies();
			if(conjunctives.get(i).predicates.size() == 0 &&
					conjunctives.get(i).disjunctives.size() == 0) {
				conjunctives.remove(i);
				i--;
				size--;
			}
				
		}
	}
	
	public RVSet computeRVSetFromTemporalSelection(Vector<String> relationAliases, 
			VersionIntervalSet viSet, VarInfo varInfo) {
		RVSet rvSet = new RVSet();
		if(conjunctives.size() == 0)
			return 	new RVSet(relationAliases, viSet); // full RV set
		
		for(int i=0; i<conjunctives.size(); i++) {
			RVSet tempRVSet = conjunctives.get(i).computeRVSetFromTemporalSelection(
					relationAliases, viSet, varInfo);
			rvSet = rvSet.union(tempRVSet);
		}
		
		return rvSet;
	}
	
	public RVSet computeRVSetFromTemporalJoin(RVSet prevUniverse, VarInfo varInfo) {
		RVSet rvSet = new RVSet();
		if(conjunctives.size() == 0)
			return prevUniverse; // full RV set
		
		for(int i=0; i<conjunctives.size(); i++) {
			RVSet tempRVSet = conjunctives.get(i).computeRVSetFromTemporalJoin(
					prevUniverse, varInfo);
			rvSet = rvSet.union(tempRVSet);
		}
		
		return rvSet;
	}
	
	/*
	public void renameTableNames(String oldTableName, String newTableName) {
		// SMO doesn't use table name alias - change table names to proper aliases
		for(int j=0; j<this.size(); j++) {
			if(this.get(j).tableName1.contentEquals(oldTableName))
				this.get(j).tableName1 = newTableName;

			if(this.get(j).tableName2 == null) continue;
			
			if(this.get(j).tableName2.contentEquals(oldTableName))
				this.get(j).tableName2 = newTableName;
		}
	}
	
	public boolean hasTable(String tableAlias) {
		for(int j=0; j<this.size(); j++) {
			if(this.get(j).tableName1.contentEquals(tableAlias))
				return true;

			if(this.get(j).tableName2 == null) continue;
			
			if(this.get(j).tableName2.contentEquals(tableAlias))
				return true;
		}
		return false;
	}
	
	public String toStringOld() {
		return predicates.toString();
	}
	*/
	@Override
	public String toString() {
		String str="";
		int numQuantifiersInLine = 3;
		

		for(int i=0; i<conjunctives.size(); i++) {
			if(conjunctives.size() > 1)
				str += "(";
			str += conjunctives.get(i).toString();
			if(conjunctives.size() > 1)
				str += ")";

			if(i < conjunctives.size()-1) {
				str += " OR ";
				if(i % numQuantifiersInLine == numQuantifiersInLine-1)
					str += "\n";
			}
		}
		return str;
	}

	public Vector<Predicate> getPredicatesByAttribute(SelectAttribute attribute) {
		
		Vector<Predicate> v = getAllPredicates();
		Vector<Predicate> out = new Vector<Predicate>();
		for(Predicate p:v){
			
			if(attribute.attribute.equals(p.term1.columnName) && attribute.relationAlias.equals(p.term1.tableName)){
				out.add(p);
			}else{
				if(attribute.attribute.equals(p.term2.columnName) && attribute.relationAlias.equals(p.term2.tableName)){
					out.add(p);
				}
			}
		}
		return out;
	}	
	
	/**
	 * Recursive function gathering all the predicates and returning them as a flat Vector
	 * @return
	 */
	public Vector<Predicate> getAllPredicates() {
		Vector<Predicate> v = new Vector<Predicate>();
 		for(Conjunctive c:conjunctives){
			for(Predicate p:c.predicates){
				v.add(p);				
			}		
			for(Disjunctive d:c.disjunctives){
				v.addAll(d.getAllPredicates());
			}
 		}
		return v;
	}
	
}
