package edu.ucla.cs.db.prism.backend.smo.util;

import java.io.Serializable;

import edu.ucla.cs.db.prism.backend.ded.QuantifiedDisjunctive;
import edu.ucla.cs.db.prism.backend.ded.QuantifierSet;
import edu.ucla.cs.db.prism.backend.temporalOptimize.RV;
import edu.ucla.cs.db.prism.backend.temporalOptimize.RVSet;
import edu.ucla.cs.db.prism.backend.temporalOptimize.VersionInterval;
import edu.ucla.cs.db.prism.backend.temporalOptimize.VersionIntervalSet;
import edu.ucla.cs.db.prism.backend.xquery.VarInfo;
import edu.ucla.cs.db.prism.backend.xquery.VarInfoUnit;

public class Predicate  implements Serializable {
	static final long serialVersionUID = 0;
	// tableName1.columnName1 = tableName2.columnName2;
	
	/*
	public String columnName1;
	public String tableName1;
	*/
	public Term term1;
	public String op;
	public Term term2;
	
	
	/*
	public String columnName2;
	public String tableName2;
	public String constant;
	*/
	
	public Predicate(String tableName1, String columnName1, String op,
						 String tableName2, String columnName2) {
		this.term1 = new Term(tableName1, columnName1);
		this.op = op;
		this.term2 = new Term(tableName2, columnName2);
	}
	
	public Predicate(String tableName1, String columnName1, String op,
							String constant) {

		this.term1 = new Term(tableName1, columnName1);
		this.op = op;
		this.term2 = new Term(constant);
	}
	public Predicate(String tableName1, String columnName1, String op, Term term2) {
		this.term1 = new Term(tableName1, columnName1);
		this.op = op;
		this.term2 = term2;
		
	}
	
	
	public Predicate(Term term1, String op, Term term2) {
		this.term1 = term1;
		this.op = op;
		this.term2 = term2;
	}
	
	public Predicate() {
		this.term1 = null;
		this.op = null;
		this.term2 = null;
	}
	
	
	public Predicate(Predicate src) {
		if(src.term1 == null || src.term2 == null)
			System.out.println(src);
		
		this.term1 = new Term(src.term1);
		this.op = new String(src.op);
		this.term2 = new Term(src.term2);
	}
	
	public Predicate clone() {
		//System.out.println("predicate cloning");
		return new Predicate(this);
	}

	public boolean isTautology() {
		// t1.c1 = t2.c2, where t1=t2, c1=c2
		// constant1 = constant2, where constant1 = constant2

		if(!this.op.contentEquals("="))
			return false;
		
		/*
		if(term1.constant == null && term2.constant == null)
			return term1.tableName.equals(term2.tableName) &&
					term1.columnName.equals(term2.columnName);
		else if(term1.constant != null && term2.constant != null)
			return term1.constant.equals(term2.constant);
		else
			return false;
			*/
		// updated on 9-14-2007 by hyun
		else
			return this.term1.equals(this.term2);
	}
	
	public boolean contentEquals(Predicate predicate) {
		if(predicate == null) return false; // "this" is not null

		// 1. test
		// case 1 (p1.t1 = p2.t1 and p1.t2 = p2.t2) or
		// case 2 (p1.t1 = p2.t2 and p1.t2 = p2.t1)
		// if fails, return false
		
		// 2. for case 1, p1.op and p2.op should be the same
		//    for case 2, p1.op and p2.op should be the opposite 
		
		boolean case1valid = false;
		boolean case2valid = false;

		
		if(this.term1.contentEquals(predicate.term1) &&
				this.term2.contentEquals(predicate.term2))
			case1valid = true;
		
		if(this.term1.contentEquals(predicate.term2) &&
				this.term2.contentEquals(predicate.term1))
			case2valid = true;
		
		
		if(case1valid)
			return this.op.equals(predicate.op);
		else if(case2valid) {
			if(this.op.equals("="))
				return predicate.op.equals("=");
			else if(this.op.equals("!="))
				return predicate.op.equals("!=");
			else if(this.op.equals("<"))
				return predicate.op.equals(">");
			else if(this.op.equals(">"))
				return predicate.op.equals("<");
			else if(this.op.equals("<="))
				return predicate.op.equals("<=");
			else if(this.op.equals(">="))
				return predicate.op.equals(">=");
		}

		return false;
	}
	
	
	public Term getNonquantifiedTerm(QuantifierSet quantifiers) {
		
		if(term1.hasNonquantifiedRelation(quantifiers))
			return term1;
		else if(term2.hasNonquantifiedRelation(quantifiers))
			return term2;
		else
			return null;
	}
	
	public void renameRelationVariable(String varPrefix, int varIndexIncrement, 
			String varToReplace, String newVarName) {

		term1.renameRelationVariable(varPrefix, varIndexIncrement,
					varToReplace, newVarName);
		term2.renameRelationVariable(varPrefix, varIndexIncrement,
				varToReplace, newVarName);
	}
	
	// if this predicate has a term with tableName as a table name,
	// then replace it with "term" by referring to view (it has tableName.attr = term)
	public void replaceTerms(String tableName, QuantifiedDisjunctive viewThenClause) {
		Term termToReplace = null;
		int whichTermToReplace = 0;
		if(term1.constant == null &&
				term1.tableName.equals(tableName)) {
			termToReplace = term1;
			whichTermToReplace = 1;
		}
		else if(term2.constant == null &&
				term2.tableName.equals(tableName)) {
			termToReplace = term2;
			whichTermToReplace = 2;
		}
		
		if(termToReplace == null) return;
		
		//Term newTerm = DED.getTermEqualToSxAttr(viewThenClause, 
		//				termToReplace.tableName, termToReplace.columnName);
		Term newTerm = viewThenClause.disjunctive.getTermEqualToRA(
				termToReplace.tableName, termToReplace.columnName, false);
		
		/*
		if(newTerm == null) {
			System.out.println(tableName);
			System.out.println(viewThenClause);
			System.out.println("null newTerm");
			System.exit(0);
		}
		*/
			
		
		if(whichTermToReplace == 1)
			term1 = newTerm;
		else
			term2 = newTerm;
	}
	
	
	public boolean isUsingVariable(String var) {
		if(term1.constant == null && term1.tableName.equals(var))
			return true;
		else if(term2.constant == null && term2.tableName.equals(var))
			return true;
		else
			return false;
	}
	
	
	private RVSet computeRVSetFromSQLTemporalSelection(VersionIntervalSet viSet) throws PredicateException {
		// here we only look at temporal selection case,
		// case: R.(ts|te) op TimeConstant

		boolean caseFound = false;
		Term constantTerm = null;
		Term RATerm = null;
		String RAopConstant = null; // op reversed if term1 = constant, term2 = RATerm
		if(term1.constant == null && term2.constant != null) {
			if(term1.columnName.equals("ts") | term1.columnName.equals("te")) {
				caseFound = true;
				constantTerm = term2;
				RATerm = term1;
				RAopConstant = op;
			}
		}
		else if(term1.constant != null && term2.constant == null) {
			if(term2.columnName.equals("ts") | term2.columnName.equals("te")) {
				caseFound = true;
				constantTerm = term1;
				RATerm = term2;
				RAopConstant = reverseOp(op);
			}
		}
		if(!caseFound)
			return null;
		
		RVSet rvSet = new RVSet();
		// OLD
		// case A1: R.ts < T -> all versions
		// case A2: R.ts = T -> {CV(T), ..., N}, also CV(T-1) if T=CV(T)ts
		// case A3: R.ts > T -> {CV(T), ..., N)
		// case A4: R.ts <= T -> same as case A1 U case A2 = case A1
		// case A5: R.ts >= T -> same as case A2 U case A3 = case A2
		// case A6: R.ts != T -> all versions

		// case B1: R.te < T -> {1, ..., CV(T)} 
		// case B2: R.te = T -> {1, ..., CV(T)}, also CV(T+1) if T=CV(T)te
		// case B3: R.te > T -> all versions
		// case B4: R.te <= T -> same as case B1 U case B2 = case B2
		// case B5: R.te >= T -> same as case B2 U case B3 = case B3
		// case B6: R.te != T -> all versions

		// NEW - 20081015 updated for CNesT rewriting
		// case A1: R.ts < T -> all versions
		// case A2: R.ts = T -> {CV(T), ..., N}
		// case A3: R.ts > T -> {CV(T), ..., N)
		// case A4: R.ts <= T -> same as case A1 U case A2 = case A1
		// case A5: R.ts >= T -> same as case A2 U case A3 = case A2
		// case A6: R.ts != T -> all versions

		// case B1: R.te < T -> {1, ..., CV(T)}, CV(T) can be pruned if T=CV(T).ts 
		// case B2: R.te = T -> CV(T) if T>CV(T).ts, CV(T)-1 if T=CV(T).ts
		// case B3: R.te > T -> {CV(T), ..., N}
		// case B4: R.te <= T -> same as case B1 U case B2 = case B1
		// case B5: R.te >= T -> same as case B2 U case B3 = {CV(T), ..., N} and also CV(T)-1, if T=CV(T).ts
		// case B6: R.te != T -> all versions
		
		
		int firstVersion = viSet.getFirstVersion().version;
		int lastVersion = viSet.getLastVersion().version;
		VersionInterval vcT = viSet.getByTime(constantTerm.constant);
		//System.out.println(constantTerm.constant + ": " + vcT);
		if(vcT == null) {
			System.err.println("Error: invalid time instance");
			throw new PredicateException("Error: invalid time instance");
			//System.exit(0);
		}
		int versionContainingT = vcT.version;

		if(RATerm.columnName.equals("ts")) {
			// case A1
			if(RAopConstant.equals("<")) { 
				// all versions
				rvSet.rvs.add(new RV(RATerm.tableName, firstVersion, lastVersion));
			}
			// case A2
			else if(RAopConstant.equals("=")) { 
				rvSet.rvs.add(new RV(RATerm.tableName, versionContainingT, lastVersion));
			}
			// case A3
			else if(RAopConstant.equals(">")) {
				rvSet.rvs.add(new RV(RATerm.tableName, versionContainingT, lastVersion));
			}
			// case A4
			else if(RAopConstant.equals("<=")) { 
				// all versions
				rvSet.rvs.add(new RV(RATerm.tableName, firstVersion, lastVersion));
			}
			// case A5
			else if(RAopConstant.equals(">=")) { 
				rvSet.rvs.add(new RV(RATerm.tableName, versionContainingT, lastVersion));
			}
			// case A6
			else if(RAopConstant.equals("!=")) { 
				// all versions
				rvSet.rvs.add(new RV(RATerm.tableName, firstVersion, lastVersion));
			}
		}
		else if(RATerm.columnName.equals("te")) {
			// case B1
			if(RAopConstant.equals("<")) {
				if(constantTerm.constant.equals(vcT.startTime))
					rvSet.rvs.add(new RV(RATerm.tableName, firstVersion, versionContainingT-1));
				else
					rvSet.rvs.add(new RV(RATerm.tableName, firstVersion, versionContainingT));
			}
			// case B2
			else if(RAopConstant.equals("=")) {
				if(constantTerm.constant.equals(vcT.startTime))
					rvSet.rvs.add(new RV(RATerm.tableName, versionContainingT-1, versionContainingT-1));
				else
					rvSet.rvs.add(new RV(RATerm.tableName, versionContainingT, versionContainingT));
			}
			// case B3
			else if(RAopConstant.equals(">")) {
				rvSet.rvs.add(new RV(RATerm.tableName, versionContainingT, lastVersion));
			}
			// case B4
			else if(RAopConstant.equals("<=")) {
				if(constantTerm.constant.equals(vcT.startTime))
					rvSet.rvs.add(new RV(RATerm.tableName, firstVersion, versionContainingT-1));
				else
					rvSet.rvs.add(new RV(RATerm.tableName, firstVersion, versionContainingT));
			}
			// case B5
			else if(RAopConstant.equals(">=")) {
				if(constantTerm.constant.equals(vcT.startTime))
					rvSet.rvs.add(new RV(RATerm.tableName, versionContainingT-1, lastVersion));
				else
					rvSet.rvs.add(new RV(RATerm.tableName, versionContainingT, lastVersion));
			}
			// case B6
			else if(RAopConstant.equals("!=")) {
				rvSet.rvs.add(new RV(RATerm.tableName, firstVersion, lastVersion));
			}
		}
		
		return rvSet;
	}
	
	public String reverseOp(String op) {
		if(op.equals("=") || op.equals("!="))
			return op;
		else if(op.equals(">"))
				return "<";
		else if(op.equals(">="))
			return "<=";
		else if(op.equals("<"))
			return ">";
		else if(op.equals("<="))
			return ">=";
		else
			return null;
	}
	
	/*
	public Vector<Integer> allIntegersBetween(int n1, int n2) {
		if(n1>n2) return null;
		Vector<Integer> result = new Vector<Integer>();
		for(int i=n1; i<=n2; i++)
			result.add(new Integer(i));
		return result;
	}
	*/

	// inputRVSet: given from the above level
	// return value: (possibly) reduce inputRVSet and return it
	// return null if no reduction done
	private RVSet computeRVSetFromSQLTemporalJoin(RVSet prevUniverse) {
		// we look at tempora join case
		// case : R1.(ts|te) op R2.(ts|te)
		// we (possibly) reduce RV of R1 and/or R2
		boolean caseFound = false;
		if(term1.constant == null && term2.constant == null) {
			if( (term1.columnName.equals("ts") | term1.columnName.equals("te")) &&
					(term1.columnName.equals("ts") | term1.columnName.equals("te")) )
				caseFound = true;
		}
		if(!caseFound)
			return null;
		
		RVSet rvSet1 = computeRVSetForR1FromSQLTemporalJoin(term1, term2, op, prevUniverse);
		RVSet rvSet2 = computeRVSetForR1FromSQLTemporalJoin(term2, term1, reverseOp(op), prevUniverse);

		if(rvSet1 == null) // returns null if the result is same as prevUniverse
			return rvSet2;
		else if(rvSet2 == null)
			return rvSet1;
		else
			return rvSet1.setUnionElementIntersection(rvSet2);
	}
	
	private RVSet computeRVSetForR1FromSQLTemporalJoin(Term r1, Term r2, String op, RVSet prevUniverse) {
		// to simplify cases
		// technique 1: != has no effect on reduction, so return null
		if(op.equals("!=")) 
			return null;
		// technique 2.1: >= is equal to > or =
		else if(op.equals(">=")) {
			RVSet rvSet1 = computeRVSetForR1FromSQLTemporalJoin(r1, r2, ">", prevUniverse);
			RVSet rvSet2 = computeRVSetForR1FromSQLTemporalJoin(r1, r2, "=", prevUniverse);
			if(rvSet1 == null || rvSet2 == null) // returns null if the result is same as prevUniverse
				return null;
			else
				return rvSet1.union(rvSet2);
		}
		// technique 2.2: <= is equal to < or =
		else if(op.equals("<=")) {
			RVSet rvSet1 = computeRVSetForR1FromSQLTemporalJoin(r1, r2, "<", prevUniverse);
			RVSet rvSet2 = computeRVSetForR1FromSQLTemporalJoin(r1, r2, "=", prevUniverse);
			if(rvSet1 == null || rvSet2 == null) // returns null if the result is same as prevUniverse
				return null;
			else
				return rvSet1.union(rvSet2);
		}
		
		// convention: r1.table = R1, r2.table = R2
		//				RV1 = RV(R1), RV2 = RV(R2).
		//				VB1i (VB2i) = i-th version block of RV(R1) (resp. RV(R2))
		//
		// Latest Observation
		// case 1. R1.ts < R2.(ts|te) -> VB1i= VB1i if a version of RV2 intersecting or following VB1i exists, empty set otherwise
		// case 2. R1.ts = R2.(ts|te) -> VB1i= earliest VB1i^RV2 and the following versions of VB1i
		// case 3. R1.ts > R2.(ts|te) -> VB1i= VB1i^(RV2 U all-following(RV2))
		
		// case 4. R1.te < R2.(ts|te) -> VB1i= VB1i^(RV2 U all-precedding(RV2))
		// case 5. R1.te = R2.(ts|te) -> VB1i= last VB1i^RV2 and the preceeding versions of VB1i
		// case 6. R1.te > R2.(ts|te) -> VB1i= VB1i if a version of RV2 intersecting or preceeding VB1i exists, empty set otherwise

		RVSet rvSet = new RVSet(prevUniverse);
		RV rvR1 = rvSet.get(r1.tableName);
		RV rvR2 = rvSet.get(r2.tableName);
		if(r1.columnName.equals("ts")) {
			if(op.equals("<")) {
				rvR1.inPlaceAllOrNoneByR2IntersectingOrFollowingForEachBlock(rvR2);
			}
			else if(op.equals("=")) {
				rvR1.inPlaceEarliestIntersectionAndFollowingForEachBlock(rvR2);
			}
			else if(op.equals(">")) {
				rvR1.inPlaceIntersectionRV2AndGreater(rvR2);
			}
		}
		else if(r1.columnName.equals("te")) {
			if(op.equals("<")) {
				rvR1.inPlaceIntersectionRV2AndLess(rvR2);
			}
			else if(op.equals("=")) {
				rvR1.inPlaceLastIntersectionAndPreceedingForEachBlock(rvR2);
			}
			else if(op.equals(">")) {
				rvR1.inPlaceAllOrNoneByR2IntersectingOrPreceedingForEachBlock(rvR2);
			}
		}
		else // this shouldn't occur
			return null;
		
		
		return rvSet;
	}
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		
		

	private RVSet computeRVSetFromXQueryTemporalSelection(VersionIntervalSet viSet, VarInfo varInfo) throws PredicateException {
		// here we only look at temporal selection case,
		// case: R.(ts|te) op TimeConstant

		boolean caseFound = false;
		//Term varTerm = null;
		Term constantTerm = null;
		String varOpConstant = null; // op reversed if term1 = constant, term2 = RATerm

		VarInfoUnit varInfoUnit = null;
		
		if(term1.constant != null && term2.constant != null) {
			if(term1.constant.startsWith("$") && !term2.constant.startsWith("$")) {
				varInfoUnit = varInfo.get(term1.constant);
				if(varInfoUnit.isAttr &&
				   (varInfoUnit.nodeName.equals("@ts") || varInfoUnit.nodeName.equals("@te"))) {
					caseFound = true;
					//varTerm = term1;
					varOpConstant = op;
					constantTerm = term2;
				}
			}
			else if(!term1.constant.startsWith("$") && term2.constant.startsWith("$")) {
				varInfoUnit = varInfo.get(term2.constant);
				if(varInfoUnit.isAttr &&
				   (varInfoUnit.nodeName.equals("@ts") || varInfoUnit.nodeName.equals("@te")))
					caseFound = true;
				//varTerm = term2;
				varOpConstant = reverseOp(op);
				constantTerm = term1;
			}
		}

		if(!caseFound)
			return null;
				
		RVSet rvSet = new RVSet();
		// case A1: $ts < T -> all versions
		// case A2: $ts = T -> {CV(T), ..., N}, also CV(T-1) if T=CV(T)ts
		// case A3: $ts > T -> {CV(T), ..., N)
		// case A4: $ts <= T -> same as case A1 U case A2 = case A1
		// case A5: $ts >= T -> same as case A2 U case A3 = case A2
		// case A6: $ts != T -> all versions

		// case B1: $te < T -> {1, ..., CV(T)} 
		// case B2: $te = T -> {1, ..., CV(T)}, also CV(T+1) if T=CV(T)te
		// case B3: $te > T -> all versions
		// case B4: $te <= T -> same as case B1 U case B2 = case B2
		// case B5: $te >= T -> same as case B2 U case B3 = case B3
		// case B6: $te != T -> all versions
		
		int firstVersion = viSet.getFirstVersion().version;
		int lastVersion = viSet.getLastVersion().version;
		VersionInterval vcT = viSet.getByTime(constantTerm.constant);
		//System.out.println(constantTerm.constant + ": " + vcT);
		if(vcT == null) {
			System.err.println("Error: invalid time instance");
			throw new PredicateException("Error: invalid time instance");
			//System.exit(0);
		}
		int versionContainingT = vcT.version;

		if(varInfoUnit.nodeName.equals("@ts")) {
			// case A1, A4, A6
			if(varOpConstant.equals("<") ||
					varOpConstant.equals("<=") ||
					varOpConstant.equals("!=")) { 
				// all versions
				rvSet.rvs.add(new RV(varInfoUnit.parVar, firstVersion, lastVersion));
			}
			// case A2, A5
			else if(varOpConstant.equals("=") ||
					varOpConstant.equals(">=")) { 
				if(constantTerm.constant.equals(vcT.startTime) && 
						versionContainingT > firstVersion)
					rvSet.rvs.add(new RV(varInfoUnit.parVar, versionContainingT-1, lastVersion));				
				else
					rvSet.rvs.add(new RV(varInfoUnit.parVar, versionContainingT, lastVersion));
			}
			// case A3
			else if(varOpConstant.equals(">")) {
				//System.out.println("right case");
				rvSet.rvs.add(new RV(varInfoUnit.parVar, versionContainingT, lastVersion));
				//System.out.println(rvSet);
			}
		}
		else if(varInfoUnit.nodeName.equals("@ts")) {
			// case B1
			if(varOpConstant.equals("<")) {
				rvSet.rvs.add(new RV(varInfoUnit.parVar, firstVersion, versionContainingT));
			}
			// case B2, B4
			else if(varOpConstant.equals("=") ||
					varOpConstant.equals("<=")) {
				if(constantTerm.constant.equals(vcT.endTime) &&
						versionContainingT < lastVersion)
					rvSet.rvs.add(new RV(varInfoUnit.parVar, firstVersion, versionContainingT+1));
				else
					rvSet.rvs.add(new RV(varInfoUnit.parVar, firstVersion, versionContainingT));
			}
			// case B3, B5, B6
			else if(varOpConstant.equals(">") ||
					varOpConstant.equals(">=")) {
				rvSet.rvs.add(new RV(varInfoUnit.parVar, firstVersion, lastVersion));
			}
		}
		
		return rvSet;
	}
	
	
	
	
	
	
	private RVSet computeRVSetFromXQueryTemporalJoin(RVSet prevUniverse, VarInfo varInfo) {
		// here we look at tempora join case
		// case : ($ts1|$te1) op ($ts2|$te2)
		// we (possibly) reduce RV of R1 and/or R2
		boolean caseFound = false;
		VarInfoUnit varInfoUnit1 = null;
		VarInfoUnit varInfoUnit2 = null;
		if(term1.constant != null && term2.constant != null) {
			if(term1.constant.startsWith("$") && term2.constant.startsWith("$")) {
				varInfoUnit1 = varInfo.get(term1.constant);
				varInfoUnit2 = varInfo.get(term2.constant);
				if(varInfoUnit1.isAttr &&
				   (varInfoUnit1.nodeName.equals("@ts") || varInfoUnit1.nodeName.equals("@te")) &&
				   varInfoUnit2.isAttr &&
				   (varInfoUnit2.nodeName.equals("@ts") || varInfoUnit2.nodeName.equals("@te")))
					caseFound = true;
			}
		}
		
		if(!caseFound)
			return null;
		
		RVSet rvSet1 = computeRVSetForR1FromXQueryTemporalJoin(term1, term2, op, prevUniverse, varInfo);
		RVSet rvSet2 = computeRVSetForR1FromXQueryTemporalJoin(term2, term1, reverseOp(op), prevUniverse, varInfo);

		if(rvSet1 == null) // returns null if the result is same as prevUniverse
			return rvSet2;
		else if(rvSet2 == null)
			return rvSet1;
		else
			return rvSet1.setUnionElementIntersection(rvSet2);
	}
	
	private RVSet computeRVSetForR1FromXQueryTemporalJoin(Term var1, Term var2, String op, RVSet prevUniverse, 
			VarInfo varInfo) {
		// to simplify cases
		// technique 1: != has no effect on reduction, so return null
		if(op.equals("!=")) 
			return null;
		// technique 2.1: >= is equal to > or =
		else if(op.equals(">=")) {
			RVSet rvSet1 = computeRVSetForR1FromXQueryTemporalJoin(var1, var2, ">", prevUniverse, varInfo);
			RVSet rvSet2 = computeRVSetForR1FromXQueryTemporalJoin(var1, var2, "=", prevUniverse, varInfo);
			if(rvSet1 == null || rvSet2 == null) // returns null if the result is same as prevUniverse
				return null;
			else
				return rvSet1.union(rvSet2);
		}
		// technique 2.2: <= is equal to < or =
		else if(op.equals("<=")) {
			RVSet rvSet1 = computeRVSetForR1FromXQueryTemporalJoin(var1, var2, "<", prevUniverse, varInfo);
			RVSet rvSet2 = computeRVSetForR1FromXQueryTemporalJoin(var1, var2, "=", prevUniverse, varInfo);
			if(rvSet1 == null || rvSet2 == null) // returns null if the result is same as prevUniverse
				return null;
			else
				return rvSet1.union(rvSet2);
		}
		
		// convention: r1.table = R1, r2.table = R2
		//				RV1 = RV(R1), RV2 = RV(R2).
		//				VB1i (VB2i) = i-th version block of RV(R1) (resp. RV(R2))
		//
		// Latest Observation
		// case 1. $ts1 < ($ts2|$te2) -> VB1i= VB1i if a version of RV2 intersecting or following VB1i exists, empty set otherwise
		// case 2. $ts1 = ($ts2|$te2) -> VB1i= earliest VB1i^RV2 and the following versions of VB1i
		// case 3. $ts1 > ($ts2|$te2) -> VB1i= VB1i^(RV2 U all-following(RV2))
		
		// case 4. $te1 < ($ts2|$te2) -> VB1i= VB1i^(RV2 U all-precedding(RV2))
		// case 5. $te1 = ($ts2|$te2) -> VB1i= last VB1i^RV2 and the preceeding versions of VB1i
		// case 6. $te1 > ($ts2|$te2) -> VB1i= VB1i if a version of RV2 intersecting or preceeding VB1i exists, empty set otherwise

		RVSet rvSet = new RVSet(prevUniverse);
		RV rvVar1 = rvSet.get(varInfo.get(var1.constant).parVar);
		RV rvVar2 = rvSet.get(varInfo.get(var2.constant).parVar);
		if(varInfo.get(var1.constant).nodeName.equals("@ts")) {
			if(op.equals("<")) {
				rvVar1.inPlaceAllOrNoneByR2IntersectingOrFollowingForEachBlock(rvVar2);
			}
			else if(op.equals("=")) {
				rvVar1.inPlaceEarliestIntersectionAndFollowingForEachBlock(rvVar2);
			}
			else if(op.equals(">")) {
				rvVar1.inPlaceIntersectionRV2AndGreater(rvVar2);
			}
		}
		else if(varInfo.get(var1.constant).nodeName.equals("@te")) {
			if(op.equals("<")) {
				rvVar1.inPlaceIntersectionRV2AndLess(rvVar2);
			}
			else if(op.equals("=")) {
				rvVar1.inPlaceLastIntersectionAndPreceedingForEachBlock(rvVar2);
			}
			else if(op.equals(">")) {
				rvVar1.inPlaceAllOrNoneByR2IntersectingOrPreceedingForEachBlock(rvVar2);
			}
		}
		else // this shouldn't occur
			return null;
		
		
		return rvSet;
	}
		
		
		
		
	
	
	
	public RVSet computeRVSetFromTemporalSelection(VersionIntervalSet viSet, VarInfo varInfo) {
		if((term1.constant != null && term1.constant.startsWith("$")) ||
		   (term2.constant != null && term2.constant.startsWith("$")))
			try {
				return computeRVSetFromXQueryTemporalSelection(viSet, varInfo);
			} catch (PredicateException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}
		else
			try {
				return computeRVSetFromSQLTemporalSelection(viSet);
			} catch (PredicateException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		return null;
	}
	
	public RVSet computeRVSetFromTemporalJoin(RVSet prevUniverse, VarInfo varInfo) {
		if((term1.constant != null && term1.constant.startsWith("$")) ||
		   (term2.constant != null && term2.constant.startsWith("$")))
			return computeRVSetFromXQueryTemporalJoin(prevUniverse, varInfo);
		else
			return computeRVSetFromSQLTemporalJoin(prevUniverse);
	}
	
	public boolean equals(Object object) {
		return equals((Predicate)object);
	}
	public boolean equals(Predicate p) {
		return (this.term1.equals(p.term1) && this.op.equals(p.op) && this.term2.equals(p.term2))
				|| (this.term1.equals(p.term2) && this.op.equals(reverseOp(p.op)) && this.term2.equals(p.term1));
	}
	/**
	 * Test if this predicate is more restrictive than p, which means that
	 * if this predicate evaluates to true, then p always evaluates to true
	 * @param p
	 * @return true if we can guarantee that this predicate is more restrictive that p
	 *         false otherwise
	 */
	public boolean isGuaranteedToBeMoreRestrictiveThan(Predicate p) {
		// we test the case of temporal overlap of multiple relations in the form of
		// max(a1, a2, ..., an) < min(b1, b2, ..., bm)
		// if p is max(a1, a2, ..., an) < min(b1, b2, ..., bm) and
		// this predicate is max(a1, a2, ..., an, c1, ..., ck) < min(b1, b2, ..., bm, d1, ..., dl)
		// then this predicate is more restrictive than p
		if(!this.op.equals("<") || !p.op.equals("<"))
			return false;
		else if(!this.term1.isFunctionTerm() || !this.term2.isFunctionTerm() ||
				!p.term1.isFunctionTerm() || !p.term2.isFunctionTerm())
			return false;
		else if(!this.term1.function.equals("max") || !this.term2.function.equals("min") ||
				!p.term1.function.equals("max") || !p.term2.function.equals("min"))
			return false;

		for(int i=0; i<p.term1.functionArgs.size(); i++) {
			Term argTerm = p.term1.functionArgs.get(i);
			if(!this.term1.functionArgs.contains(argTerm))
				return false;
		}
		for(int i=0; i<p.term2.functionArgs.size(); i++) {
			Term argTerm = p.term2.functionArgs.get(i);
			if(!this.term2.functionArgs.contains(argTerm))
				return false;
		}
		return true;
	}
	
	
	public String toString() {
		//return term1.toString() + " " + op + " " + term2.toString();
		if(term2 == null)
			return term1 + " " + op + " " + "realnull";	
			
		return term1 + " " + op + " " + term2;	
	}
}
