package edu.ucla.cs.db.prism.backend.smo.util;

import java.io.StringReader;

import edu.ucla.cs.db.prism.backend.parser.parser.PrimaRParser;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.Smo;
import edu.ucla.cs.db.prism.backend.parser.visitor.SmoTranslateVisitor;
import edu.ucla.cs.db.prism.backend.smo.AddColumnSMO;
import edu.ucla.cs.db.prism.backend.smo.CopyTableSMO;
import edu.ucla.cs.db.prism.backend.smo.CreateTableSMO;
import edu.ucla.cs.db.prism.backend.smo.DecomposeTableSMO;
import edu.ucla.cs.db.prism.backend.smo.DropColumnSMO;
import edu.ucla.cs.db.prism.backend.smo.DropTableSMO;
import edu.ucla.cs.db.prism.backend.smo.JoinTableSMO;
import edu.ucla.cs.db.prism.backend.smo.MergeTableSMO;
import edu.ucla.cs.db.prism.backend.smo.NopSMO;
import edu.ucla.cs.db.prism.backend.smo.PartitionTableSMO;
import edu.ucla.cs.db.prism.backend.smo.RenameColumnSMO;
import edu.ucla.cs.db.prism.backend.smo.RenameTableSMO;
import edu.ucla.cs.db.prism.backend.smo.SMOImpl;
import edu.ucla.cs.db.prism.backend.smo.SMOInterface;
import edu.ucla.cs.db.prism.backend.smo.exception.SMOParsingErrorException;

public class SMOFactory {

	public static SMOImpl createSMO(String string) throws SMOTypeUnknownException {
		
		if(string.equalsIgnoreCase(SMOInterface.CREATE_TAB))
			return new CreateTableSMO();
		if(string.equalsIgnoreCase(SMOInterface.DROP_TAB))
			return new DropTableSMO();
		if(string.equalsIgnoreCase(SMOInterface.REN_TAB))
			return new RenameTableSMO();
		if(string.equalsIgnoreCase(SMOInterface.COPY_TAB))
			return new CopyTableSMO();
		if(string.equalsIgnoreCase(SMOInterface.MERGE))
			return new MergeTableSMO();
		if(string.equalsIgnoreCase(SMOInterface.PARTITION))
			return new PartitionTableSMO();		
		if(string.equalsIgnoreCase(SMOInterface.JOIN))
			return new JoinTableSMO();
		if(string.equalsIgnoreCase(SMOInterface.DECOMPOSE))
			return new DecomposeTableSMO();
		if(string.equalsIgnoreCase(SMOInterface.REN_COL))
			return new RenameColumnSMO();
		if(string.equalsIgnoreCase(SMOInterface.ADD_COL))
			return new AddColumnSMO();
		if(string.equalsIgnoreCase(SMOInterface.DROP_COL))
			return new DropColumnSMO();
		if(string.equalsIgnoreCase(SMOInterface.NOP))
			return new NopSMO();		
		throw new SMOTypeUnknownException(string);
		
	}

	public static SMOImpl createSMOFromString(String string) throws SMOParsingErrorException {
		StringReader input = null;
		input = new StringReader(string);

		// step 1.2: parse it
		PrimaRParser parser = new PrimaRParser(input);
		//PrimaRScript script = null;
		Smo smo = null;
		try {
			smo = parser.Smo();
			//script = parser.PrimaRScript();
		} catch(Exception e) {
			e.printStackTrace();
		}
		
		// step 1.3: restructure abstract syntax tree (AST) into our defined class objects
		SmoTranslateVisitor smotv = new SmoTranslateVisitor();
		//script.accept(smotv);
		//System.out.println(inString);
		if(smo != null)
			smo.accept(smotv);
		else{
			throw new SMOParsingErrorException("The input text: " + string + " cannot be parsed correctly.");
			
		}
		smotv.oneSMO.setComplete(true);
		return smotv.oneSMO;
	}
	

}
