package edu.ucla.cs.db.prism.backend.smo.util;

import java.io.Serializable;
import java.util.Iterator;
import java.util.Vector;

//import com.sun.tools.javac.util.List;

import edu.ucla.cs.db.prism.backend.ded.DEDSet;
import edu.ucla.cs.db.prism.backend.ded.DEDSetHistory;
import edu.ucla.cs.db.prism.backend.icmo.MOImpl;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.SchemaHistory;
import edu.ucla.cs.db.prism.backend.schema.exception.IllegalSMOException;
import edu.ucla.cs.db.prism.backend.smo.SMOImpl;
import edu.ucla.cs.db.prism.backend.smo.SMOInterface;

public class SMOSequence implements Serializable, SMOSequenceInterface {
	// A sequence of SMO and related methods
	static final long serialVersionUID = 0;
	public int sourceVersion; 
	public int targetVersion; 
	public Vector<SMOImpl> smos;
	
	public SMOSequence() {
		smos = new Vector<SMOImpl>();
	}
	
	public SMOSequence(SMOSequence newsmos) {
		this.smos = (Vector<SMOImpl>) newsmos.smos.clone();
		sourceVersion = newsmos.sourceVersion;
		targetVersion = newsmos.targetVersion;
	}


	/* (non-Javadoc)
	 * @see edu.ucla.cs.db.prism.backend.smo.util.MOSequenceInterface#getInverseSequence()
	 */
	public SMOSequence getInverseSequence() {
		SMOSequence inverseSMOSequence = new SMOSequence();
		
		// MODIFIED BY CARLO CHECK
		for(int i=0; i<smos.size(); i++){
			SMOSequence temp = smos.get(i).getInverse();
			//TODO remove printout
			System.out.println(temp.toString());
			inverseSMOSequence.smos.addAll(temp.smos);
		}
		inverseSMOSequence.sourceVersion = this.targetVersion;
		inverseSMOSequence.targetVersion = this.sourceVersion;
		return inverseSMOSequence;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.cs.db.prism.backend.smo.util.MOSequenceInterface#size()
	 */
	public int size() {
		return smos.size();
	}
	
	/* (non-Javadoc)
	 * @see edu.ucla.cs.db.prism.backend.smo.util.MOSequenceInterface#add(edu.ucla.cs.db.prism.backend.smo.SMOImpl)
	 */
	public boolean add(SMOImpl smo) {
		return smos.add(smo);
	}
	/* (non-Javadoc)
	 * @see edu.ucla.cs.db.prism.backend.smo.util.MOSequenceInterface#add(int, edu.ucla.cs.db.prism.backend.smo.SMOImpl)
	 */
	public void add(int index, SMOImpl smo) {
		smos.add(index, smo);
		return;
	}
	
	/* (non-Javadoc)
	 * @see edu.ucla.cs.db.prism.backend.smo.util.MOSequenceInterface#get(int)
	 */
	public MOImpl get(int index) {
		return smos.get(index);
	}
	/* (non-Javadoc)
	 * @see edu.ucla.cs.db.prism.backend.smo.util.MOSequenceInterface#remove(int)
	 */
	public SMOInterface remove(int index) {
		return smos.remove(index);
	}
	/* (non-Javadoc)
	 * @see edu.ucla.cs.db.prism.backend.smo.util.MOSequenceInterface#toString()
	 */
	@Override
	public String toString() {
		String str = "";
		str += "v" + this.sourceVersion;
		str += " - v" + this.targetVersion;
		str += smos.toString();
		return str;
	}
	
	/* (non-Javadoc)
	 * @see edu.ucla.cs.db.prism.backend.smo.util.MOSequenceInterface#hasVersions(int, int)
	 */
	public boolean hasVersions(int sourceVersion, int targetVersion) {
		return (sourceVersion == this.sourceVersion) && 
				(targetVersion == this.targetVersion);
	}
	
	// check by version numbers (assume that the contents are correct)
	/* (non-Javadoc)
	 * @see edu.ucla.cs.db.prism.backend.smo.util.MOSequenceInterface#isInverseOf(edu.ucla.cs.db.prism.backend.smo.util.SMOSequence)
	 */
	public boolean isInverseOf(SMOSequence smoSequence2) {
		return this.hasVersions(smoSequence2.targetVersion, smoSequence2.sourceVersion);
	}
	
	// 1. squeeze multiple add columns into a single table into one "ADD COLUMN MULTI"
	// 2. squeeze multiple drop columns into a single table into one "DROP COLUMN MULTI"
	// 3. squeeze multiple rename columns into a single table into one "RENAME COLUMN MULTI"
	
	/* (non-Javadoc)
	 * @see edu.ucla.cs.db.prism.backend.smo.util.MOSequenceInterface#compressSMOs()
	 */
	public int compressSMOs() {
		
		int addColumnReductionCount = compressSMOsOfOneType("ADD COLUMN");
		int dropColumnReductionCount = compressSMOsOfOneType("DROP COLUMN");
		int renameColumnReductionCount = compressSMOsOfOneType("RENAME COLUMN");

		int smosReductionCount = 
			addColumnReductionCount +
			dropColumnReductionCount +
			renameColumnReductionCount;
		
		return smosReductionCount;
	}
	
	private int compressSMOsOfOneType(String smoType) {
		int totalSMOReductionCount = 0; // 3 smos into 1 smo gives us smoReductionCount = 2

		int firstSMOToSqueeze = -1;
		String smoTableToSqueeze = null;
		int smosSize = smos.size();
		for(int i=0; i<smosSize; i++) {
			SMOImpl smo = smos.get(i);
			boolean wantedTypeFlag = smo.getOperatorType().equals(smoType);
			 // we don't squeeze ADD COLUMN with function, which is possible
			if(smo.getOperatorType().equals("ADD COLUMN") && smo.getFunctionName() != null)
				wantedTypeFlag = false;
			
			boolean wantedTypeAndTableFlag = false;
			if(wantedTypeFlag && firstSMOToSqueeze == -1)
				wantedTypeAndTableFlag = true;
			else if(wantedTypeFlag && firstSMOToSqueeze > -1)
				wantedTypeAndTableFlag = smo.getTableName().equals(smoTableToSqueeze);
			else // if(!wantedTypeFlag)
				wantedTypeAndTableFlag = false;
			
			
			if(i < smosSize-1) {
				if(wantedTypeAndTableFlag && firstSMOToSqueeze == -1) {
					// new squeeze-sequence begins here
					firstSMOToSqueeze = i;
					smoTableToSqueeze = smo.getTableName();
				}
				else if(wantedTypeAndTableFlag && firstSMOToSqueeze > -1) {
					// squeeze-sequence continues
					// do nothing
				}
				else if(!wantedTypeAndTableFlag && firstSMOToSqueeze == -1) {
					// no existing squeeze-sequence, not startly newly
					// do nothing
				}
				else { // if(!wantedTypeFlag && firstSMOToSzueeze > -1)
					// existing squeeze-sequence ends, excluding this smo
					if(i == firstSMOToSqueeze + 1) {
						firstSMOToSqueeze = -1;
						continue; // only one smo to squeeze, can't squeeze
					}
					performCompressSMOs(smoType, firstSMOToSqueeze, i);
					int smoReductionCount = i - firstSMOToSqueeze - 1 ;
					totalSMOReductionCount += smoReductionCount;
					i -= smoReductionCount;
					smosSize -= smoReductionCount;
					firstSMOToSqueeze = -1; // reset
				}
			}
			else { // if(i == smosSize-1) {
				// other cases don't need a special attention, except the following
				if(wantedTypeAndTableFlag && firstSMOToSqueeze > -1) {
					// existing squeeze-sequence ends, INCLUDING this smo
					// here we have always 2 or more smos
					performCompressSMOs(smoType, firstSMOToSqueeze, i+1);
					int smoReductionCount = i+1 - firstSMOToSqueeze - 1 ;
					totalSMOReductionCount += smoReductionCount;
					i -= smoReductionCount;
					smosSize -= smoReductionCount;
					firstSMOToSqueeze = -1; // reset
				}
				else if(!wantedTypeAndTableFlag && firstSMOToSqueeze > -1) {
					// existing squeeze-sequence ends, excluding this smo
					if(i == firstSMOToSqueeze + 1) 
						continue; // only one smo to squeeze, can't squeeze
					performCompressSMOs(smoType, firstSMOToSqueeze, i);
					int smoReductionCount = i - firstSMOToSqueeze - 1 ;
					totalSMOReductionCount += smoReductionCount;
					i -= smoReductionCount;
					smosSize -= smoReductionCount;
					firstSMOToSqueeze = -1; // reset
				}
			}
		}
		
		return totalSMOReductionCount;
	}
	
	// squeeze until smos.get(first) through smo.get(last-1)
	private void performCompressSMOs(String smoType, int first, int last) {
		boolean columnName2Used = true;
		if(smoType.equals("ADD COLUMN") || smoType.equals("DROP COLUMN"))
			columnName2Used = false;
		
		boolean defaultConstantUsed = false;
		if(smoType.equals("ADD COLUMN"))
			defaultConstantUsed = true;
		
		
		// CARLO C modified to use factory
		//SMOImpl newSMO = new SMOImpl();
		//newSMO.setOperatorType(smoType + " MULTI");
		SMOImpl newSMO = null;
		try {
			newSMO = SMOFactory.createSMO(smoType + " MULTI");
		} catch (SMOTypeUnknownException e) {
			e.printStackTrace();
		}
		newSMO.setTableName(smos.get(first).getTableName());
		newSMO.setColumnNameList(new Vector<String>(last-first-1));
		if(columnName2Used)
			newSMO.setColumnNameList2(new Vector<String>(last-first-1));
		if(defaultConstantUsed)
			newSMO.setDefaultConstantList(new Vector<String>(last-first-1));
		
		for(int i=first; i<last; i++) {
			SMOImpl smo = smos.get(i);
			newSMO.getColumnNameList().add(smo.getColumnName());
			if(columnName2Used) {
				newSMO.getColumnNameList2().add(smo.getColumnName2());
			}
			if(defaultConstantUsed) {
				newSMO.getDefaultConstantList().add(smo.getDefaultConstant());
			}
		}
		
		for(int i=last-1; i>=first; i--)
			smos.remove(i);
		smos.add(first, newSMO);
	}

	/* (non-Javadoc)
	 * @see edu.ucla.cs.db.prism.backend.smo.util.MOSequenceInterface#removeReassignVersions(int)
	 */
	public MOImpl removeReassignVersions(int index) {
		for(int i = index; i< smos.size();i++){
			smos.get(i).setSourceMinor(smos.get(i).getSourceMinor()-1);
			
			// LAST SMOS SHOULD NOT HAVE INDEX CHANGED
			if(i!= smos.size()-1){
				smos.get(i).setTargetMinor(smos.get(i).getTargetMinor()-1);
			}
		}
		
		return smos.remove(index);
	}

	
	/* (non-Javadoc)
	 * @see edu.ucla.cs.db.prism.backend.smo.util.MOSequenceInterface#getCandidateInverseSequence(edu.ucla.cs.db.prism.backend.schema.Schema)
	 */
	public SMOSequence getCandidateInverseSequence(Schema schema) {
		Schema inputSchema = schema.clone();
		SMOSequence inverseSMOSequence = new SMOSequence();
		
		// FIXME this returns only the first candidate
		for(int i=0; i<smos.size(); i++){
			SMOSequence temp = smos.get(i).getCandidateInverses(inputSchema).firstElement();
			try {
				inputSchema.applySMO(smos.get(i));
			} catch (IllegalSMOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}			
			inverseSMOSequence.smos.addAll(temp.smos);
		}
		inverseSMOSequence.sourceVersion = this.targetVersion;
		inverseSMOSequence.targetVersion = this.sourceVersion;
		return inverseSMOSequence;
	}

	
	/* (non-Javadoc)
	 * @see edu.ucla.cs.db.prism.backend.smo.util.MOSequenceInterface#getAllDED(edu.ucla.cs.db.prism.backend.schema.Schema, boolean)
	 */
	public DEDSetHistory getAllDED(Schema schema, boolean isInverse) {
		
		Schema curSchema = schema.clone();
		DEDSetHistory dedSetHistory = new DEDSetHistory();

		int schmaj = schema.versionID;
		int schmin = schema.minorVersionID;
		int startingsmo = -1;
		// we find the SMOs from which is appropriate to start applying the sequence, i.e., the one with source version equal to the schema version
		for(int i = 0; i< smos.size(); i++){
			if(schmaj == smos.elementAt(i).getSourceMajor() && schmin == smos.elementAt(i).getSourceMinor()){
				startingsmo = i;
				break;
			}
		}
		
		
		
		if(isInverse){
			for(int i = startingsmo; i>= 0; i--){
				SMOImpl smo = smos.elementAt(i);
				//System.out.println("Generating DEDS for " + smo.toMetadataString() + "SCHEMA: 48_" + curSchema.minorVersionID );
				DEDSet ds = smo.getAllDEDs(curSchema);
				try {
					curSchema.applySMO(smo);
				} catch (IllegalSMOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				dedSetHistory.add(ds);		
			}
		}else{
			for(int i = startingsmo; i<smos.size(); i++){
				SMOImpl smo = smos.elementAt(i);
				DEDSet ds = smo.getAllDEDs(curSchema);
				try {
					curSchema.applySMO(smo);
				} catch (IllegalSMOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				dedSetHistory.add(ds);		
			}
			
			
		}
			
		return dedSetHistory;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.cs.db.prism.backend.smo.util.MOSequenceInterface#generateSchemaHistory(edu.ucla.cs.db.prism.backend.schema.Schema, boolean)
	 */
	public SchemaHistory generateSchemaHistory(Schema input, boolean isInverse) {
		
		Schema temp = input.clone();
		SchemaHistory sh = new SchemaHistory();
		
		int schmaj = input.versionID;
		int schmin = input.minorVersionID;
		int startingsmo = -1;
		// we find the SMOs from which is appropriate to start applying the sequence, i.e., the one with source version equal to the schema version
		for(int i = 0; i< smos.size(); i++){
			if(schmaj == smos.elementAt(i).getSourceMajor() && schmin == smos.elementAt(i).getSourceMinor()){
				startingsmo = i;
				break;
			}
		}
		
		//System.out.println("APPLYING INVERSE.generateSchemaHistory FROM: " + startingsmo + " TO 0");
		
		if(isInverse){
			//we process the SMOSequence in inverse direction (from the last to the first element)
			for(int i = startingsmo; i>=0; i--){
				//System.out.println("PROCESSING SMO: " + smos.elementAt(i).toMetadataString());
				sh.addSchema(temp.clone()); 
				SMOImpl smo = smos.elementAt(i);
				try {
					temp.applySMO(smo);
				} catch (IllegalSMOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
			sh.addSchema(temp);
		}else{
			// otherwise we process the sequence forward
			//Added by Swami - Incase of deleting the only smo
			//startingsmo value will be -1 and so throws error
			//Also changed some code in SMODesign to accomodate unexisting schema error
			if(startingsmo >=0)
			{
				for(int i = startingsmo; i < smos.size();i++){
					sh.addSchema(temp.clone());
					SMOImpl smo = smos.elementAt(i);
					try {
						temp.applySMO(smo);
					} catch (IllegalSMOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
				sh.addSchema(temp);
			}
			else
			{
				sh.addSchema(temp);
				
			}
			//Ended - Swami
		} 
		
		//we return the new schemaHistory
		return sh;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.cs.db.prism.backend.smo.util.MOSequenceInterface#getSMOBySource(int, int)
	 */
	public MOImpl getSMOBySource(int major, int minor) {
	
		for(int i=0;i<smos.size();i++){
			if(smos.get(i).getSourceMajor() == major && smos.get(i).getSourceMinor() == minor)
				return smos.get(i);
			
		}
		return null;
	}
	
	/* (non-Javadoc)
	 * @see edu.ucla.cs.db.prism.backend.smo.util.MOSequenceInterface#getSMOIndexBySource(int, int)
	 */
	public int getSMOIndexBySource(int major, int minor) {
		
		for(int i=0;i<smos.size();i++){
			if(smos.get(i).getSourceMajor() == major && smos.get(i).getSourceMinor() == minor)
				return i;
			
		}
		return -1;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.cs.db.prism.backend.smo.util.MOSequenceInterface#getHighestMinor(int)
	 */
	public int getHighestMinor(int major) {
		int ret = 0;
		for(int i=0;i<smos.size();i++){
			if(smos.get(i).getSourceMajor() == major && smos.get(i).getSourceMinor() > ret)
				ret = smos.get(i).getSourceMinor();
		}
		return ret;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.cs.db.prism.backend.smo.util.MOSequenceInterface#isComplete()
	 */
	public boolean isComplete() {
		Iterator<SMOImpl> it = smos.iterator();	
		while(it.hasNext()){
			if(!it.next().isComplete())
				return false;
		}
		return true;
	}

	/* (non-Javadoc)
	 * @see edu.ucla.cs.db.prism.backend.smo.util.MOSequenceInterface#getCommonHistoryIndex(edu.ucla.cs.db.prism.backend.smo.util.SMOSequence, boolean)
	 */
	public int getCommonHistoryIndex(SMOSequence newsmos, boolean isInverse) {
		int index=0;
		int limit = 0;
		if(smos.size()<newsmos.smos.size())
			limit = smos.size();
		else
			limit = newsmos.smos.size();
		
		if(isInverse){
			for(int i=0; i<limit; i++){
				if(smos.elementAt(i).equals(newsmos.smos.elementAt(i)))
					index++;
				else
					break;
			}
		}else{
			for(int i=limit-1; i>=0; i--){
				if(smos.elementAt(i).equals(newsmos.smos.elementAt(i)))
					index++;
				else
					break;
			}	
		}
		
		return index;
	}


	
	/*
	
	public static SMOSequence createSampleSMOSequence1() {
		SMOSequence smos = new SMOSequence();
		SMO smo = null;
		Predicate targetCondition = null;
		
		smo = new SMO("RENAME COLUMN");
		smo.tableName = "employee";
		smo.columnName = "name";
		smo.columnName2 = "ename";
		smos.add(smo);

		smo = new SMO("RENAME TABLE");
		smo.tableName = "salary";
		smo.tableName2 = "salarychart";
		smos.add(smo);
		
		smo = new SMO("PARTITION TABLE");
		smo.tableName = "employee";
		smo.tableName2 = "high-emp-med";
		smo.tableName3 = "low-emp-med";
		smos.add(smo);
		
		smo = new SMO("MERGE TABLE");
		smo.tableName = "salary-all";
		smo.tableName2 = "salarychart";

		targetCondition = new Predicate("salarychart", "job", "=", "\'production\'");
		smo.targetConditions2 = new Conjunctive();
		smo.targetConditions2.add(targetCondition);
		smo.tableName3 = "salary-contract";
		targetCondition = new Predicate("salary-contract", "job", "=", "\'contract\'");
		smo.targetConditions3 = new Conjunctive();
		smo.targetConditions3.add(targetCondition);
		smos.add(smo);
		
		smo = new SMO("DROP TABLE");
		smo.tableName = "salary-all";
		smos.add(smo);
		
		smo = new SMO("DROP COLUMN");
		smo.tableName = "high-emp-med";
		smo.columnName = "ethnicity";
		smos.add(smo);
		
		smo = new SMO("RENAME COLUMN");
		smo.tableName = "salary-all";
		smo.columnName = "job";
		smo.columnName2 = "job-title";
		smos.add(smo);
		
		return smos;
	}

	public static SMOSequence createSampleSMOSequence() {
		SMOSequence smos = new SMOSequence();
		SMO smo = null;
		Predicate targetCondition = null;
		
		// smo group 1 (5 smos)
		smo = new SMO("CREATE TABLE");
		smo.tableName = "emp-med";
		smos.add(smo);

		smo = new SMO("COPY COLUMN");
		smo.tableName = "emp";
		smo.columnName = "eid";
		smo.tableName2 = "emp-med";
		smos.add(smo);

		smo = new SMO("MOVE COLUMN");
		smo.tableName = "emp";
		smo.columnName = "weight";
		smo.tableName2 = "emp-med";
		targetCondition = new Predicate("emp", "eid", "=", "emp-med", "eid");
		smo.targetConditions = new Conjunctive();
		smo.targetConditions.add(targetCondition);
		smos.add(smo);

		smo = new SMO("MOVE COLUMN");
		smo.tableName = "emp";
		smo.columnName = "bp";
		smo.tableName2 = "emp-med";
		targetCondition = new Predicate("emp", "eid", "=", "emp-med", "eid");
		smo.targetConditions = new Conjunctive();
		smo.targetConditions.add(targetCondition);
		smos.add(smo);
		
		smo = new SMO("RENAME TABLE");
		smo.tableName = "emp";
		smo.tableName2 = "emp-acct";
		smos.add(smo);

		// smo group 2 (3 smos)
		smo = new SMO("PARTITION TABLE");
		smo.tableName = "emp-med";
		smo.tableName2 = "high-emp-med";
		smo.targetConditions2 = new Conjunctive();
		targetCondition = new Predicate("high-emp-med", "eid", "=", "emp-acct", "eid");
		smo.targetConditions2.add(targetCondition);
		targetCondition = new Predicate("emp-acct", "job", "=", "\'production\'");
		smo.targetConditions2.add(targetCondition);

		smo.tableName3 = "low-emp-med";
		smo.targetConditions3 = new Conjunctive();
		targetCondition = new Predicate("low-emp-med", "eid", "=", "emp-acct", "eid");
		smo.targetConditions3.add(targetCondition);
		targetCondition = new Predicate("emp-acct", "job", "<>", "\'production\'");
		smo.targetConditions3.add(targetCondition);
		smos.add(smo);
		
		smo = new SMO("ADD COLUMN");
		smo.tableName = "high-emp-med";
		smo.columnName = "lung";
		smos.add(smo);

		smo = new SMO("ADD COLUMN");
		smo.tableName = "high-emp-med";
		smo.columnName = "skin";
		smos.add(smo);

		// smo group 3 (1 smo)
		smo = new SMO("DROP COLUMN");
		smo.tableName = "emp-acct";
		smo.columnName = "ethnicity";
		smos.add(smo);
		
		// smo group 4 (3 smos)
		smo = new SMO("COPY COLUMN");
		smo.tableName = "salary";
		smo.columnName = "amount";
		smo.tableName2 = "emp-acct";
		targetCondition = new Predicate("salary", "job", "=", "emp-acct", "job");
		smo.targetConditions = new Conjunctive();
		smo.targetConditions.add(targetCondition);
		smos.add(smo);
		
		smo = new SMO("RENAME TABLE");
		smo.tableName = "salary";
		smo.tableName2 = "job-salary-ref";
		smos.add(smo);
		
		smo = new SMO("RENAME COLUMN");
		smo.tableName = "emp-acct";
		smo.columnName = "amount";
		smo.columnName2 = "salary";
		smos.add(smo);

		
		//smo group 5 (3 smos)
		smo = new SMO("CREATE TABLE");
		smo.tableName = "car-registration";
		smos.add(smo);

		smo = new SMO("COPY COLUMN");
		smo.tableName = "emp-acct";
		smo.columnName = "eid";
		smo.tableName2 = "car-registration";
		smos.add(smo);
		
		smo = new SMO("MOVE COLUMN");
		smo.tableName = "emp-acct";
		smo.columnName = "carplate";
		smo.tableName2 = "car-registration";
		targetCondition = new Predicate("emp-acct", "eid", "=", "car-registration", "eid");
		smo.targetConditions = new Conjunctive();
		smo.targetConditions.add(targetCondition);
		smos.add(smo);

		
		return smos;
	}

	// currently used
	public static SMOSequence createSampleSMOSequence2() {
		SMOSequence smos = new SMOSequence();
		SMO smo = null;
		Predicate targetCondition = null;
		
		// smo group 2 (3 smos)
		smo = new SMO("PARTITION TABLE");
		smo.tableName = "emp-med";
		smo.tableName2 = "high-emp-med";
		smo.targetConditions2 = new Conjunctive();
		targetCondition = new Predicate("high-emp-med", "eid", "=", "emp-acct", "eid");
		smo.targetConditions2.add(targetCondition);
		targetCondition = new Predicate("emp-acct", "job", "=", "\"production\"");
		smo.targetConditions2.add(targetCondition);

		smo.tableName3 = "low-emp-med";
		smo.targetConditions3 = new Conjunctive();
		targetCondition = new Predicate("low-emp-med", "eid", "=", "emp-acct", "eid");
		smo.targetConditions3.add(targetCondition);
		targetCondition = new Predicate("emp-acct", "job", "=", "\"office\"");
		smo.targetConditions3.add(targetCondition);
		smos.add(smo);
		
		smo = new SMO("ADD COLUMN");
		smo.tableName = "high-emp-med";
		smo.columnName = "lung";
		smos.add(smo);

		smo = new SMO("ADD COLUMN");
		smo.tableName = "high-emp-med";
		smo.columnName = "skin";
		smos.add(smo);

		// smo group 3 (1 smo)
		smo = new SMO("DROP COLUMN");
		smo.tableName = "emp-acct";
		smo.columnName = "ethnicity";
		smos.add(smo);
		
		// smo group 4 (3 smos)
		smo = new SMO("COPY COLUMN");
		smo.tableName = "salary";
		smo.columnName = "amount";
		smo.tableName2 = "emp-acct";
		targetCondition = new Predicate("salary", "job", "=", "emp-acct", "job");
		smo.targetConditions = new Conjunctive();
		smo.targetConditions.add(targetCondition);
		smos.add(smo);
		
		smo = new SMO("RENAME TABLE");
		smo.tableName = "salary";
		smo.tableName2 = "job-salary-ref";
		smos.add(smo);
		
		smo = new SMO("RENAME COLUMN");
		smo.tableName = "emp-acct";
		smo.columnName = "amount";
		smo.columnName2 = "salary";
		smos.add(smo);
		
		
		//smo group 5 (3 smos)
		smo = new SMO("CREATE TABLE");
		smo.tableName = "car-registration";
		smos.add(smo);

		smo = new SMO("COPY COLUMN");
		smo.tableName = "emp-acct";
		smo.columnName = "eid";
		smo.tableName2 = "car-registration";
		smos.add(smo);
		
		smo = new SMO("MOVE COLUMN");
		smo.tableName = "emp-acct";
		smo.columnName = "carplate";
		smo.tableName2 = "car-registration";
		targetCondition = new Predicate("emp-acct", "eid", "=", "car-registration", "eid");
		smo.targetConditions = new Conjunctive();
		smo.targetConditions.add(targetCondition);
		smos.add(smo);

		
		return smos;
	}

	public static SMOSequence createSmallSampleSMOSequence() {
		SMOSequence smos = new SMOSequence();
		SMO smo = null;
		Predicate targetCondition = null;
		
		// smo group 1 (5 smos)

		smo = new SMO("COPY COLUMN");
		smo.tableName = "salary";
		smo.columnName = "amount";
		smo.tableName2 = "emp";
		targetCondition = new Predicate("salary", "job", "=", "emp", "job");
		smo.targetConditions = new Conjunctive();
		smo.targetConditions.add(targetCondition);
		smos.add(smo);
		
		return smos;
	}

	// 2/7/2007 - create table, add column testing
	public static SMOSequence createTestingSMOSequence1() {
		SMOSequence smos = new SMOSequence();
		SMO smo = null;
		Predicate targetCondition = null;
		

		smo = new SMO("CREATE TABLE");
		smo.tableName = "manager";
		smo.targetConditions = new Conjunctive();
		smos.add(smo);

		smo = new SMO("ADD COLUMN");
		smo.tableName = "manager";
		smo.columnName = "eid";
		smo.targetConditions = new Conjunctive();
		smos.add(smo);

		smo = new SMO("ADD COLUMN");
		smo.tableName = "manager";
		smo.columnName = "mid";
		smo.targetConditions = new Conjunctive();
		smos.add(smo);

		return smos;
	}
	 */
}
