package edu.ucla.cs.db.prism.backend.smo.util;

import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.exception.IllegalSMOException;
import edu.ucla.cs.db.prism.backend.smo.SMOImpl;

public class SMOSequenceHistory {
	public Vector<SMOSequence> smoSequences;
	
	public SMOSequenceHistory() {
		smoSequences = new Vector<SMOSequence>();
	}

	public int size() {
		return smoSequences.size();
	}
	
	public boolean add(SMOSequence sequenceInterface) {
		return smoSequences.add(sequenceInterface);
	}
	
	public SMOSequenceInterface get(int index) {
		return smoSequences.get(index);
	}
	
	
	public SMOSequenceInterface get(int sourceVersion, int targetVersion) {
		for(int i=0; i<size(); i++)
			if(smoSequences.get(i).hasVersions(sourceVersion, targetVersion))
				return smoSequences.get(i);
		
		return null;
	}

	
	// fully computes the inverse of everything
	public void computeInverse() {
		int size = smoSequences.size();
		
		for(int i=0; i<size; i++) {
			boolean inverseFound = false;
			for(int j=0; j<size; j++) {
				if(i==j) 
					continue; // one can't be its own inverse
				else if(smoSequences.get(i).isInverseOf(smoSequences.get(j))) {
					inverseFound = true;
					break;
				}
			}
			if(!inverseFound) {
				smoSequences.add(smoSequences.get(i).getInverseSequence());
				size++;
			}
		}
	}
	
	public SMOSequenceInterface remove(int index) {
		return smoSequences.remove(index);
	}
	

	public Vector<Integer> getAllSourceVersions() {
		Vector<Integer> sourceVersions = new Vector<Integer>();
		for(int i=0; i<smoSequences.size(); i++) {
			Integer sourceVersion = new Integer(smoSequences.get(i).sourceVersion);
			if(!sourceVersions.contains(sourceVersion))
				sourceVersions.add(sourceVersion);
		}
		
		return sourceVersions;
			
	}
	
	public SMOSequenceHistory getForwardDirectionPartialHistory(int minVersion, int maxVersion) {
		SMOSequenceHistory smoSequenceHistory = new SMOSequenceHistory();
		for(int i=0; i<smoSequences.size(); i++) {
			SMOSequence smoSequence = smoSequences.get(i);
			if(smoSequence.sourceVersion < smoSequence.targetVersion && // forward direction only
					minVersion <= smoSequence.sourceVersion &&
					smoSequence.targetVersion <= maxVersion)
				smoSequenceHistory.add(smoSequence);
		}
		return smoSequenceHistory;
		
	}
	
	public int getNumberOfAllSMOs() {
		int count = 0;
		for(int i=0; i<smoSequences.size(); i++)
			count += smoSequences.get(i).smos.size();
		return count;
	}
	
	public int compressSMOs() {
		int totalSMOReductionCount = 0;
		for(int i=0; i<smoSequences.size(); i++)
			totalSMOReductionCount += smoSequences.get(i).compressSMOs();
		return totalSMOReductionCount;	
	}
	
	@Override
	public String toString() {
		String str = "";
		for(int i=0; i<this.size(); i++) {
			str += this.get(i).toString() + "\n";
		}
		return str;
	}
	
	public String toStringNotEmpty() {
		String str = "";
		for(int i=0; i<this.size(); i++) {
			if(this.get(i).size() == 0)
				continue;
			str += this.get(i).toString() + "\n";
		}
		return str;
	}

	public Vector<String> getSQLViewsNoID(Schema s, String dbname) {
		Vector<String> v = new Vector<String>();
		Iterator<SMOSequence> sit = this.smoSequences.iterator();
		while(sit.hasNext()){
			SMOSequence ss = sit.next();
			Iterator<SMOImpl> smoit = ss.smos.iterator();
			SMOImpl smo = null;
			while(smoit.hasNext()){
				if(smo!=null)
					try {
						s.applySMO(smo);
					} catch (IllegalSMOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				smo = smoit.next();
				v.add(smo.getSQLView(s, dbname + "_"+ smo.getSourceMajor(), dbname + "_" + smo.getTargetMajor()));
			}
			try {
				s.applySMO(smo);
			} catch (IllegalSMOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return v;
	}
	
	public Vector<String> getSQLViewsAll(Schema s, String dbname) {
		Vector<String> v = new Vector<String>();
		Iterator<SMOSequence> sit = this.smoSequences.iterator();
		while(sit.hasNext()){
			SMOSequence ss = sit.next();
			Iterator<SMOImpl> smoit = ss.smos.iterator();
			SMOImpl smo = null;
			while(smoit.hasNext()){
				if(smo!=null)
					try {
						s.applySMO(smo);
					} catch (IllegalSMOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				smo = smoit.next();
				v.add(smo.getSQLView(s, dbname + "_"+ smo.getSourceMajor(), dbname + "_" + smo.getTargetMajor()));
				v.addAll(smo.getIdentitySQLViews(s, dbname));
			}
			try {
				s.applySMO(smo);
			} catch (IllegalSMOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return v;
	}

	public Vector<String> getSQLDDL(Schema s, String dbname) {
		Vector<String> v = new Vector<String>();
		Iterator<SMOSequence> sit = this.smoSequences.iterator();
		while(sit.hasNext()){
			SMOSequence ss = sit.next();
			Iterator<SMOImpl> smoit = ss.smos.iterator();
			SMOImpl smo = null;
			while(smoit.hasNext()){
				if(smo!=null)
					try {
						s.applySMO(smo);
					} catch (IllegalSMOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				smo = smoit.next();
				//System.out.println(s.getAllTables().toString());
				//v.add(smo.getSQLDDL(s, dbname + "_"+smo.getTargetMajor()));
				v.add(smo.getSQLDDL(s, dbname));
				
				//System.out.println(v.toString());
			}
			try {
				s.applySMO(smo);
			} catch (IllegalSMOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return v;
	}

	public Vector<String> getIdentityViews(Schema s, String dbname) {
		Vector<String> v = new Vector<String>();
		Iterator<SMOSequence> sit = this.smoSequences.iterator();
		while(sit.hasNext()){
			SMOSequence ss = sit.next();
			Iterator<SMOImpl> smoit = ss.smos.iterator();
			while(smoit.hasNext()){
				SMOImpl smo = smoit.next();
				v.addAll(smo.getIdentitySQLViews(s, dbname));
			}
		}
		return v;
	}
}
