package edu.ucla.cs.db.prism.backend.smo.util;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;

import edu.ucla.cs.db.prism.backend.icmo.ICMOSMOSequence;
import edu.ucla.cs.db.prism.backend.icmo.MOFactory;
import edu.ucla.cs.db.prism.backend.icmo.MOImpl;
import edu.ucla.cs.db.prism.backend.smo.SMOImpl;
import edu.ucla.cs.db.prism.backend.smo.SMOInterface;
import edu.ucla.cs.db.prism.backend.smo.exception.SMOParsingErrorException;
import edu.ucla.cs.db.prism.backend.util.db.DBUtil;

public class SMOSequenceHistoryLoader {

	boolean debug = true;
	Connection conn = null;
	String schemaName = null;
	
	public SMOSequenceHistoryLoader(Connection conn, String schemaName){
		this.conn = conn;	
		this.schemaName = schemaName;
	}
	
	
	/**
	 * Retrocompatible
	 * @return
	 * @deprecated
	 */
	@Deprecated
	public SMOSequenceHistory loadSMOSequenceHistoryFromDB(){
		return loadSMOSequenceHistoryFromDB(false);
	}
	
	
	/**
	 * 
	 * @param inverse
	 * @return and SMO or ICMO sequence history
	 */
	public SMOSequenceHistory loadSMOSequenceHistoryFromDB(boolean inverse){
		
		int inv = 0;
		if (inverse)
			inv=1;
		SMOSequenceHistory ssh = new SMOSequenceHistory();
		
		String smoSequenceHistorySQL =  "SELECT s.SMO_ID, s.Source_Major, s.Source_Minor, s.Target_Major, s.Target_Minor, SMO_TEXT, SMO_COMMENT " +
										"FROM SMOS s " +
										"WHERE IS_INVERSE=" +inv+" AND schema_name = '" + schemaName +"' ";
		if(inverse)
			smoSequenceHistorySQL +="ORDER BY Target_Major DESC, Target_Minor DESC;";
		else		
			smoSequenceHistorySQL +="ORDER BY Source_Major, Source_Minor;";
		
		
		ResultSet smoSequenceHistory = DBUtil.executeSQLQuery(smoSequenceHistorySQL,conn);
		
		ICMOSMOSequence ss = new ICMOSMOSequence();
		
		int source_old_major = -1;
		int source_major = -1;
		int source_old_minor = -1;
		int source_minor = -1;
		int target_major = -1;
		int target_old_major = -1;
			
		try {
			while(smoSequenceHistory.next()){
				source_major = smoSequenceHistory.getInt("Source_Major");
				source_minor = smoSequenceHistory.getInt("Source_Minor");
				target_major = smoSequenceHistory.getInt("Target_Major");
				
				if(source_old_major != source_major){
					ss.targetVersion=target_old_major;
					ss = new ICMOSMOSequence();
					ss.sourceVersion=source_major;
					ssh.add(ss);
				}
				
				MOImpl smo = MOFactory.createMOFromString(smoSequenceHistory.getString("SMO_TEXT"));
				
				if(smo instanceof SMOImpl)
				if(((SMOImpl)smo).getColumnType()==null)
					((SMOImpl)smo).setColumnType("varchar(255)");	
				
				smo.setId(smoSequenceHistory.getInt("SMO_ID"));
				smo.setSourceMajor(smoSequenceHistory.getInt("Source_Major"));
				smo.setSourceMinor(smoSequenceHistory.getInt("Source_Minor"));
				smo.setTargetMajor(smoSequenceHistory.getInt("Target_Major"));
				smo.setTargetMinor(smoSequenceHistory.getInt("Target_Minor"));
				smo.setComment(smoSequenceHistory.getString("SMO_COMMENT"));

				ss.add(smo);
				source_old_major = source_major;
				source_old_minor = source_minor;
				target_old_major = target_major;
			}
		} catch (SQLException e) {
			e.printStackTrace();
		} catch (SMOParsingErrorException e) {
			e.printStackTrace();
		}

		ss.targetVersion=target_major;
		
		if(ssh.smoSequences.size() == 0){
			
			
		}
		
		return ssh;
	}	
	
	public boolean storeSMOSequenceHistoryToDB(SMOSequenceHistory smoSequenceHistory, boolean generateIDs){
		String query = null;
		if(!generateIDs)	
	    	 query = "INSERT INTO SMOS(SMO_ID, Source_Major, Source_Minor, Target_Major, Target_Minor, schema_name, SMO_TEXT, SMO_COMMENT,SMO_TYPE) VALUES ";
	    else
	    	 query = "INSERT INTO SMOS(Source_Major, Source_Minor, Target_Major, Target_Minor, schema_name, SMO_TEXT, SMO_COMMENT,SMO_TYPE) VALUES ";
	    
		java.util.Iterator<SMOSequence> it = smoSequenceHistory.smoSequences.iterator();
		int count=0;
		while(it.hasNext()){
			SMOSequence sms = it.next();
			java.util.Iterator<SMOImpl> smit = sms.smos.iterator();
			while(smit.hasNext()){
				count++;
				SMOInterface s = smit.next();
				if(!generateIDs)
					query+="('" + s.getSmo_id() + "','";
				else
					query+="('";
				
				query+= s.getSourceMajor() +
					   "','" + s.getSourceMinor() +
					   "','" + s.getTargetMajor() +
					   "','" + s.getTargetMinor() +
					   "','" + schemaName +
					   "','" + s.toString() +
					   "','" + s.getComment() +
					   "','" + s.getOperatorType() +"'),";
			}
			
		}
		query = query.substring(0, query.length()-1) + ";";
		if(count!=DBUtil.executeSQLUpdate(query,conn))
			return true;
		
		return false;
	}
	
	
	
	public static SMOImpl parseSMOText(String inString) throws SMOParsingErrorException {
		return SMOFactory.createSMOFromString(inString);
	}
	
	
}
