package edu.ucla.cs.db.prism.backend.smo.util;

import edu.ucla.cs.db.prism.backend.ded.DEDSetHistory;
import edu.ucla.cs.db.prism.backend.icmo.MOImpl;
import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.SchemaHistory;
import edu.ucla.cs.db.prism.backend.smo.SMOImpl;
import edu.ucla.cs.db.prism.backend.smo.SMOInterface;

public interface SMOSequenceInterface {

	public abstract SMOSequenceInterface getInverseSequence();

	public abstract int size();

	public abstract boolean add(SMOImpl smo);

	public abstract void add(int index, SMOImpl smo);

	public abstract MOImpl get(int index);

	public abstract SMOInterface remove(int index);

	public abstract String toString();

	public abstract boolean hasVersions(int sourceVersion, int targetVersion);

	// check by version numbers (assume that the contents are correct)
	public abstract boolean isInverseOf(SMOSequence smoSequence2);

	public abstract int compressSMOs();

	public abstract MOImpl removeReassignVersions(int index);

	/**
	 * return the sequence of inverse candidates (first choice) 
	 * @param schema
	 * @return
	 */
	public abstract SMOSequenceInterface getCandidateInverseSequence(Schema schema);

	/**
	 * get all the DEDs starting from the given schema version and according to the specified directions all the way throughout the history
	 * @param schema
	 * @param isInverse
	 * @return
	 */
	public abstract DEDSetHistory getAllDED(Schema schema, boolean isInverse);

	/**
	 * Apply itself to the input schema and generate a SchemaHistory. Start applying the SMOs in the required direction from the provided schema version on. Only the correct part of the sequence is applied.
	 * @param temp
	 * @return
	 */
	public abstract SchemaHistory generateSchemaHistory(Schema input,
			boolean isInverse);

	public abstract MOImpl getSMOBySource(int major, int minor);

	public abstract int getSMOIndexBySource(int major, int minor);

	public abstract int getHighestMinor(int major);

	public abstract boolean isComplete();

	/**
	 * Check until which point two sequences are indentical. Used to cache query results
	 * @param newsmos
	 * @param isInverse
	 * @return
	 */
	public abstract int getCommonHistoryIndex(SMOSequence newsmos,
			boolean isInverse);

}