package edu.ucla.cs.db.prism.backend.smo.util;

import java.util.Hashtable;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.SQL.RelationDeclaration;
import edu.ucla.cs.db.prism.backend.ded.QuantifierSet;
import edu.ucla.cs.db.prism.backend.xquery.ForClause;
import java.io.*;

public class Term implements Serializable {
	static final long serialVersionUID = 1;
	public String columnName;
	public String tableName;
	public String constant; // if constant is null then this term is table.column
							// in xquery, constant is used to represent variable
							// (e.g. $t1)
	public String function;
	public Vector<Term> functionArgs;
	public String xmlAttributeName;
	
	final static boolean optionXQueryMaxMinFix = true;
	final static boolean optionUseLocalNamespaceForXQueryUDF = true;

	// 10-10-2007 Hyun, 
	// used in marsManage.grev.ChasePostProcessing.updateColumnLevelXQueryWhereClauseOptimized()
	public boolean chosenAsEqual; 
	
	public Term(String tableName, String columnName, String xmlAttributeName) {
		this.columnName = columnName;
		this.tableName = tableName;
		this.constant = null;
		this.function = null;
		this.functionArgs = null;
		this.xmlAttributeName = xmlAttributeName;
	}
	
	public Term(String tableName, String columnName) {
		this.columnName = columnName;
		this.tableName = tableName;
		this.constant = null;
		this.function = null;
		this.functionArgs = null;
		this.xmlAttributeName = null;
	}

	public Term(String constant) {
		this.columnName = null;
		this.tableName = null;
		this.constant = constant;
		this.function = null;
		this.functionArgs = null;
		this.xmlAttributeName = null;
	}

	public Term() {
		this.columnName = null;
		this.tableName = null;
		this.constant = null;
		this.functionArgs = new Vector<Term>();
		this.xmlAttributeName = null;
	}

	public Term(Term src) {
		this.set(src);
	}

	// set this term as a clone of src
	public void set(Term src) {
		
		// in case we have Value Constraints
		if(src.columnName!=null && src.tableName==null) {
			this.columnName = new String(src.columnName);
			this.tableName = null;
			this.constant = null;
			this.function = null;
			this.functionArgs = null;
			this.xmlAttributeName = null;
		}
		
		
		if(src.isRATerm()) {
			this.columnName = new String(src.columnName);
			this.tableName = new String(src.tableName);
			this.constant = null;
			this.function = null;
			this.functionArgs = null;
			this.xmlAttributeName = null;
		}
		else if(src.isConstantTerm()){ 
			this.columnName = null;
			this.tableName = null;
			this.constant = new String(src.constant);
			this.function = null;
			this.functionArgs = null;
			this.xmlAttributeName = null;
		}
		else if(src.isFunctionTerm()) {
			this.columnName = null;
			this.tableName = null;
			this.constant = null;
			this.function = new String(src.function);
			this.functionArgs = new Vector<Term>(src.functionArgs.size());
			for(int i=0; i<src.functionArgs.size(); i++)
				this.functionArgs.add(new Term(src.functionArgs.get(i)));
			this.xmlAttributeName = null;
		}
		else if(src.isXMLAttributeTerm()) {
			this.columnName = new String(src.columnName);
			this.tableName = new String(src.tableName);
			this.constant = null;
			this.function = null;
			this.functionArgs = null;
			this.xmlAttributeName = new String(src.xmlAttributeName);
		}
	}
	
	public Term clone() {
		//System.out.println("predicate cloning");
		return new Term(this);
	}


	
	public boolean hasNonquantifiedRelation(QuantifierSet quantifiers) {
		if(constant != null)
			return false;
		else if(quantifiers.hasVariable(tableName) == false) // using non-variable
			return true;
		else
			return false;
	}

	public void renameRelationVariable(String varPrefix, int varIndexIncrement, 
			String varToReplace, String newVarName) {

		if(constant != null) return;
		
		// case 1: "Sn" format -> "Sm" where m=n+varIndexIncrement
		if(varPrefix != null && tableName.substring(0,1).equals(varPrefix)) {
			tableName = varPrefix + 
				(Integer.parseInt(tableName.substring(1)) + varIndexIncrement);
		}
		
		// case 2: varToReplace -> newVarName (whole match & replace)
		else if(varToReplace != null && tableName.equals(varToReplace))
			tableName = newVarName;
	}

	
	public boolean isRATerm() {
		return (tableName != null && columnName != null);
	}

	public boolean isConstantTerm() {
		return constant != null;
	}

	public boolean isFunctionTerm() {
		return function != null && functionArgs != null;
	}
	
	public boolean isNestedFunctionTerm() {
		if(!isFunctionTerm())
			return false;
		for(int i=0; i<functionArgs.size(); i++) {
			if(functionArgs.get(i).isFunctionTerm())
				return true;
		}
		return false;
	}
	
	public boolean isXMLAttributeTerm() {
		return xmlAttributeName != null;
	}

	public boolean contentEquals(Term term2) {
		return this.equals(term2);
	}
	
	public boolean equals(Term term2) {
		if(term2 == null) return false; // "this" is not null

		if(this.isRATerm() && term2.isRATerm())
			return this.tableName.equals(term2.tableName) && 
					this.columnName.equals(term2.columnName);
		else if(this.isConstantTerm() && term2.isConstantTerm())
			return this.constant.equals(term2.constant);
		else if(this.isFunctionTerm() && term2.isFunctionTerm()) {
			boolean equalFlag = 
				this.function.equals(term2.function)
				&& (this.functionArgs.size() == term2.functionArgs.size());
			if(!equalFlag)
				return false; // if false, return false
			for(int i=0; i<this.functionArgs.size(); i++)
				if(!this.functionArgs.get(i).equals(term2.functionArgs.get(i)))
					return false;
			return true; // passed all equality check for function case
		}
		else // if type differ
			return false;
	}
	
	public boolean equals(Object o) {
		return equals((Term)o);
	}
	
	public String toString() {
		String str = null;
		if(xmlAttributeName!=null)
			return tableName + "." + columnName + " as \"" + xmlAttributeName + "\"";
		else if(isConstantTerm())
			return constant;
		else if(isRATerm())
			return tableName + "." + columnName;
		else if (columnName!= null && tableName == null) { // this is the case in which we have non qualified terms might be removed?
			return  columnName;
		}
		else if(isFunctionTerm()) {
			str = function + "(";
			if(functionArgs != null)
				for(int i=0; i<functionArgs.size(); i++) {
					str += functionArgs.get(i).toString();
					if(i<functionArgs.size()-1)
						str += ", ";
				}
			str += ")";
			return str;
		}else if(tableName!=null && columnName == null){
			return tableName;
			
		}
		return str;
	}
	
	
	public String translateTsTeMappingExpression(String parentVarPrefix, // "$vsTable" of "$vsTable2"
			Vector<RelationDeclaration> vsTsSet, Vector<RelationDeclaration> vsTeSet) {
		if(isConstantTerm())
			return constant;
		else if(isRATerm()) {
			if(!this.columnName.equals("a2")) // it should be attr.a2 
				return null;
			for(int i=0; i<vsTsSet.size(); i++) {
				if(this.tableName.equals(vsTsSet.get(i).relationAlias))
					return parentVarPrefix + i + "/@ts";
				else if(this.tableName.equals(vsTeSet.get(i).relationAlias))
					return parentVarPrefix + i + "/@te";
			}
			return null; // shouldn't reach here
		}
		else if(isFunctionTerm()) {
			String str = "";
			if(optionUseLocalNamespaceForXQueryUDF && 
					!function.equals("max") && !function.equals("min"))
				str += "local:";
			str += function + "(";
			if(optionXQueryMaxMinFix && (function.equals("max") || function.equals("min")))
				str += "(";
			for(int i=0; i<functionArgs.size(); i++) {
				String subExpr = functionArgs.get(i).translateTsTeMappingExpression(parentVarPrefix, vsTsSet, vsTeSet);
				if(subExpr == null) {
					return null;
				}
				else {
					str += subExpr;
					if(optionXQueryMaxMinFix && (function.equals("max") || function.equals("min")))
						str += " cast as xs:string";
				}
				if(i<functionArgs.size()-1)
					str += ", ";
			}
			str += ")";
			if(optionXQueryMaxMinFix && (function.equals("max") || function.equals("min")))
				str += ")";
			return str;
		}
		else
			return null;
	}
	
	public String translateTsTeElemMappingExpressionGReV(String parentVarPrefix, // "$vsTable" of "$vsTable2"
			Vector<RelationDeclaration> vsElemSet) {
		// here in GReV, we pass values such that vsTeSet is not needed and made null
		
		if(isConstantTerm())
			return constant;
		else if(isRATerm() && vsElemSet != null) {
			for(int i=0; i<vsElemSet.size(); i++) {
				if(this.tableName.equals(vsElemSet.get(i).relationAlias)) {
					if(this.columnName.equals("ts"))
						return parentVarPrefix + i + "/@ts";
					else if(this.columnName.equals("te"))
						return parentVarPrefix + i + "/@te";
					else if(this.columnName.equals("text"))
						return parentVarPrefix + i + "/text()";
					else if(this.columnName.equals("el"))
						return parentVarPrefix + i;
					else
						return null; // bad columnName (not ts, te, el)
				}
			}
			return null; // not found error
		}
		else if(isFunctionTerm()) {
			String str = "";
			if(optionUseLocalNamespaceForXQueryUDF && 
					!function.equals("max") && !function.equals("min"))
				str += "local:";
			str += function + "(";
			if(optionXQueryMaxMinFix && (function.equals("max") || function.equals("min")))
				str += "(";
			for(int i=0; i<functionArgs.size(); i++) {
				String subExpr = functionArgs.get(i).translateTsTeElemMappingExpressionGReV(parentVarPrefix, vsElemSet);
				if(subExpr == null) {
					return null;
				}
				else {
					str += subExpr;
					if(optionXQueryMaxMinFix && (function.equals("max") || function.equals("min")))
						str += " cast as xs:string";
				}
				if(i<functionArgs.size()-1)
					str += ", ";
			}
			str += ")";
			if(optionXQueryMaxMinFix && (function.equals("max") || function.equals("min")))
				str += ")";
			return str;
		}
		else
			return null;
		
	}
	
	/*
	public String translateTsTeMappingExpressionForColumnElement(String rowVarPrefix, // "$vsTable" of "$vsTable2"
			Vector<Vector<RelationDeclaration>> vsTsSetSet, 
			Vector<Vector<RelationDeclaration>> vsTeSetSet,
			Vector<Vector<String>> vsColumnNameSetSet,
			Vector<ForClause> forClauses) {
		
		String colVarPrefix = "$vsCol";
		
		if(isConstantTerm())
			return constant;
		else if(isRATerm()) {
			if(!this.columnName.equals("a2")) // it should be attr.a2 
				return null;
			for(int i=0; i<vsTsSetSet.size(); i++) {
				for(int j=0; j<vsTsSetSet.get(i).size(); j++) {
					if(this.tableName.equals(vsTsSetSet.get(i).get(j).relationAlias)) {
						String var = colVarPrefix + i + "_" + j;
						String path = rowVarPrefix + i + "/" + vsColumnNameSetSet.get(i).get(j);
						ForClause forClause = new ForClause(var, path);
						forClause.relationAttribute = this.tableName + ".a2"; // (attr r1).a2
						if(!forClauses.contains(forClause))
							forClauses.add(forClause);
						return var + "/@ts";
					}
					else if(this.tableName.equals(vsTeSetSet.get(i).get(j).relationAlias)) {
						String var = colVarPrefix + i + "_" + j;
						String path = rowVarPrefix + i + "/" + vsColumnNameSetSet.get(i).get(j);
						ForClause forClause = new ForClause(var, path);
						forClause.relationAttribute = this.tableName + ".a2"; // (attr r1).a2
						if(!forClauses.contains(forClause))
							forClauses.add(forClause);
						return var + "/@te";
					}
				}
			}
			return null; // shouldn't reach here
		}
		else if(isFunctionTerm()) {
			String str = function + "(";
			for(int i=0; i<functionArgs.size(); i++) {
				str += functionArgs.get(i).translateTsTeMappingExpressionForColumnElement(
						rowVarPrefix, vsTsSetSet, vsTeSetSet, vsColumnNameSetSet, forClauses);
				if(i<functionArgs.size()-1)
					str += ", ";
			}
			str += ")";
			return str;
		}
		else
			return null;
	}
	
	public String translateTextMappingExpression(String rowVarPrefix, // "$vsTable" of "$vsTable2"
			Vector<Vector<RelationDeclaration>> vsTextSetSet,
			Vector<Vector<String>> vsColumnNameSetSet,
			Vector<ForClause> forClauses) {
		
		String colVarPrefix = "$vsCol";
		if(isConstantTerm())
			return constant;
		else if(isRATerm()) {
			if(!this.columnName.equals("a1")) // it should be text.a1
				return null;
			for(int i=0; i<vsTextSetSet.size(); i++) {
				for(int j=0; j<vsTextSetSet.get(i).size(); j++) {
					if(this.tableName.equals(vsTextSetSet.get(i).get(j).relationAlias)) {
						String var = colVarPrefix + i + "_" + j;
						String path = rowVarPrefix + i + "/" + vsColumnNameSetSet.get(i).get(j);
						ForClause forClause = new ForClause(var, path);
						forClause.relationAttribute = this.tableName + ".a1"; // (text r1).a1
						if(!forClauses.contains(forClause))
							forClauses.add(forClause);
						return var + "/text()";
					}
				}
			}
			return null; // shouldn't reach here
		}
		else if(isFunctionTerm()) {
			String str = function + "(";
			for(int i=0; i<functionArgs.size(); i++) {
				str += functionArgs.get(i).translateTextMappingExpression(
						rowVarPrefix, vsTextSetSet, vsColumnNameSetSet, forClauses);
				if(i<functionArgs.size()-1)
					str += ", ";
			}
			str += ")";
			return str;
		}
		else
			return null;
	}
	*/
	

	public String translateTsTeMappingExpressionForColumnElement(String rowVarPrefix, // "$vsTable" of "$vsTable2"
			Vector<Vector<RelationDeclaration>> vsTsSetSet, 
			Vector<Vector<RelationDeclaration>> vsTeSetSet,
			Vector<Vector<String>> vsColumnNameSetSet,
			Vector<ForClause> forClauses,
			Vector<Term> sqlTermsUsed) {
		return translateMappingExpressionForColumnElement(rowVarPrefix,
				vsTsSetSet, 
				vsTeSetSet,
				null,
				vsColumnNameSetSet,
				forClauses,
				sqlTermsUsed);
	}

	public String translateTextMappingExpression(String rowVarPrefix, // "$vsTable" of "$vsTable2"
			Vector<Vector<RelationDeclaration>> vsTextSetSet,
			Vector<Vector<String>> vsColumnNameSetSet,
			Vector<ForClause> forClauses,
			Vector<Term> sqlTermsUsed) {
		return translateMappingExpressionForColumnElement(rowVarPrefix,
				null,
				null,
				vsTextSetSet,
				vsColumnNameSetSet,
				forClauses,
				sqlTermsUsed);
		
	}
	/*
	public String translateTsTeMappingExpressionForColumnElementGReV(String rowVarPrefix, // "$vsTable" of "$vsTable2"
			Vector<Vector<RelationDeclaration>> vsTsSetSet, 
			Vector<Vector<RelationDeclaration>> vsTeSetSet,
			Vector<Vector<String>> vsColumnNameSetSet,
			Vector<ForClause> forClauses,
			Vector<Term> sqlTermsUsed) {
		return translateMappingExpressionForColumnElementGReV(rowVarPrefix,
				vsTsSetSet, 
				vsTeSetSet,
				null,
				vsColumnNameSetSet,
				forClauses,
				sqlTermsUsed);
	}

	public String translateTextMappingExpressionGReV(String rowVarPrefix, // "$vsTable" of "$vsTable2"
			Vector<Vector<RelationDeclaration>> vsTextSetSet,
			Vector<Vector<String>> vsColumnNameSetSet,
			Vector<ForClause> forClauses,
			Vector<Term> sqlTermsUsed) {
		return translateMappingExpressionForColumnElementGReV(rowVarPrefix,
				null,
				null,
				vsTextSetSet,
				vsColumnNameSetSet,
				forClauses,
				sqlTermsUsed);
		
	}
	*/
	/**
	 * This incorporates translateTsTeMappingExpressionForColumnElement() and translateTextMappingExpression() above
	 * 
	 * @param rowVarPrefix
	 * @param vsTsSetSet
	 * @param vsTeSetSet
	 * @param vsTextSetSet
	 * @param vsColumnNameSetSet
	 * @param forClauses
	 * @return
	 */
	public String translateMappingExpressionForColumnElement(String rowVarPrefix, // "$vsTable" of "$vsTable2"
			Vector<Vector<RelationDeclaration>> vsTsSetSet, 
			Vector<Vector<RelationDeclaration>> vsTeSetSet,
			Vector<Vector<RelationDeclaration>> vsTextSetSet,
			Vector<Vector<String>> vsColumnNameSetSet,
			Vector<ForClause> forClauses,
			Vector<Term> sqlTermsUsed) // record what terms are found in this possibly complex term
	{
		
		String colVarPrefix = "$vsCol";
		
		if(isConstantTerm())
			return constant;
		else if(isRATerm() && this.columnName.equals("a2") && vsTsSetSet != null && vsTeSetSet != null) {// attr.a2 
			for(int i=0; i<vsTsSetSet.size(); i++) {
				for(int j=0; j<vsTsSetSet.get(i).size(); j++) {
					if(this.tableName.equals(vsTsSetSet.get(i).get(j).relationAlias)) {
						String var = colVarPrefix + i + "_" + j;
						String path = rowVarPrefix + i + "/" + vsColumnNameSetSet.get(i).get(j);
						ForClause forClause = new ForClause(var, path);
						//forClause.relationAttribute = this.tableName + ".a2"; // (attr r1).a2
						if(sqlTermsUsed != null && !sqlTermsUsed.contains(this))
							sqlTermsUsed.add(this);
						if(!forClauses.contains(forClause))
							forClauses.add(forClause);
						return var + "/@ts";
					}
					else if(this.tableName.equals(vsTeSetSet.get(i).get(j).relationAlias)) {
						String var = colVarPrefix + i + "_" + j;
						String path = rowVarPrefix + i + "/" + vsColumnNameSetSet.get(i).get(j);
						ForClause forClause = new ForClause(var, path);
						//forClause.relationAttribute = this.tableName + ".a2"; // (attr r1).a2
						if(sqlTermsUsed != null && !sqlTermsUsed.contains(this))
							sqlTermsUsed.add(this);
						if(!forClauses.contains(forClause))
							forClauses.add(forClause);
						return var + "/@te";
					}
				}
			}
			return null; // shouldn't reach here
		}
		else if(isRATerm() && this.columnName.equals("a1") && vsTextSetSet != null) {// text.a1
			for(int i=0; i<vsTextSetSet.size(); i++) {
				for(int j=0; j<vsTextSetSet.get(i).size(); j++) {
					if(this.tableName.equals(vsTextSetSet.get(i).get(j).relationAlias)) {
						String var = colVarPrefix + i + "_" + j;
						String path = rowVarPrefix + i + "/" + vsColumnNameSetSet.get(i).get(j);
						ForClause forClause = new ForClause(var, path);
						//forClause.relationAttribute = this.tableName + ".a1"; // (text r1).a1
						if(sqlTermsUsed != null && !sqlTermsUsed.contains(this))
							sqlTermsUsed.add(this);
						if(!forClauses.contains(forClause))
							forClauses.add(forClause);
						return var + "/text()";
					}
				}
			}
			return null; // shouldn't reach here
		}
		else if(isFunctionTerm()) {
			String str = "";
			if(optionUseLocalNamespaceForXQueryUDF && 
					!function.equals("max") && !function.equals("min"))
				str += "local:";
			str += function + "(";
			if(optionXQueryMaxMinFix && (function.equals("max") || function.equals("min")))
				str += "(";
			for(int i=0; i<functionArgs.size(); i++) {
				String subExpr = functionArgs.get(i).translateMappingExpressionForColumnElement(
						rowVarPrefix, vsTsSetSet, vsTeSetSet, vsTextSetSet, vsColumnNameSetSet, 
						forClauses, sqlTermsUsed);
				if(subExpr == null) {
					return null;
				}
				else {
					str += subExpr;
					if(optionXQueryMaxMinFix && (function.equals("max") || function.equals("min")))
						str += " cast as xs:string";
				}
				if(i<functionArgs.size()-1)
					str += ", ";
			}
			str += ")";
			if(optionXQueryMaxMinFix && (function.equals("max") || function.equals("min")))
				str += ")";
			return str;
		}
		else
			return null; // shouldn't fall here - error in xbind (R.a0 doesn't have any actual data)
	}
	
	/**
	 * This incorporates translateTsTeMappingExpressionForColumnElement() and translateTextMappingExpression() above
	 * 
	 * @param rowVarPrefix
	 * @param vsTsSetSet
	 * @param vsTeSetSet
	 * @param vsTextSetSet
	 * @param vsColumnNameSetSet
	 * @param forClauses
	 * @return
	 */
	public String translateMappingExpressionForColumnElementGReV(String rowVarPrefix, // "$vsTable" of "$vsTable2"
			Vector<Vector<RelationDeclaration>> vsElemSetSet,
			Vector<Vector<String>> vsColumnNameSetSet,
			Vector<ForClause> forClauses,
			Vector<Term> sqlTermsUsed) // record what terms are found in this possibly complex term
	{
		// TODO: to alter this according to GReV
		
		String colVarPrefix = "$vsCol";
		
		if(isConstantTerm())
			return constant;
		else if(isRATerm() && vsElemSetSet != null) {
			for(int i=0; i<vsElemSetSet.size(); i++) {
				for(int j=0; j<vsElemSetSet.get(i).size(); j++) {
					if(!this.tableName.equals(vsElemSetSet.get(i).get(j).relationAlias))
						continue;
					String var = colVarPrefix + i + "_" + j;
					String path = rowVarPrefix + i + "/" + vsColumnNameSetSet.get(i).get(j);
					ForClause forClause = new ForClause(var, path);
					if(sqlTermsUsed != null && !sqlTermsUsed.contains(this))
						sqlTermsUsed.add(this);
					if(forClause != null  && !forClauses.contains(forClause))
						forClauses.add(forClause);
					
					if(this.columnName.equals("ts"))
						return var + "/@ts";
					else if(this.columnName.equals("te"))
						return var + "/@te";
					else if(this.columnName.equals("text"))
						return var + "/text()";
					else if(this.columnName.equals("el"))
						return var;
					else
						return null; // error, bad column name (not ts, te, text, el)
				}
			}
			return null; // not in vsElemSetSet
		}
		else if(isFunctionTerm()) {
			String str = "";
			if(optionUseLocalNamespaceForXQueryUDF && 
					!function.equals("max") && !function.equals("min"))
				str += "local:";
			str += function + "(";
			// xquery max takes a sequence of elements
			if(optionXQueryMaxMinFix && (function.equals("max") || function.equals("min")))
				str += "(";
			//System.out.println("function!: " + this.toString());
			for(int i=0; i<functionArgs.size(); i++) {
				String subExpr = functionArgs.get(i).translateMappingExpressionForColumnElementGReV(
						rowVarPrefix, vsElemSetSet, vsColumnNameSetSet, 
						forClauses, sqlTermsUsed);
				if(subExpr == null) {
					return null;
				}
				else {
					str += subExpr;
					if(optionXQueryMaxMinFix && (function.equals("max") || function.equals("min")))
						str += " cast as xs:string";
				}
				if(i<functionArgs.size()-1)
					str += ", ";
			}
			str += ")";
			if(optionXQueryMaxMinFix && (function.equals("max") || function.equals("min")))
				str += ")";

			return str;
		}
		else
			return null; // shouldn't fall here - error in xbind (R.a0 doesn't have any actual data)

	}
	
	// 11-6-2007 new version of translateMappingExpressionForColumnElementGReV()
	// for queryRewritingPostprocessing
	public String translateMappingExpressionForColumnElementGReVQR(
			Hashtable<String, String> sqlVarToXQueryVar) // record what terms are found in this possibly complex term
	{
		if(isConstantTerm())
			return constant;
		else if(isRATerm() && sqlVarToXQueryVar != null) {
			String xqueryVar = sqlVarToXQueryVar.get(this.tableName);
			if(xqueryVar == null) return null; // tableName (= relationAlias) not found
			
			if(this.columnName.equals("ts"))
				return xqueryVar + "/@ts";
			else if(this.columnName.equals("te"))
				return xqueryVar + "/@te";
			else if(this.columnName.equals("text"))
				return xqueryVar + "/text()";
			else if(this.columnName.equals("el"))
				return xqueryVar;
			else
				return null; // error, bad column name (not ts, te, text, el)
		}
		else if(isFunctionTerm()) {
			String str = "";
			if(optionUseLocalNamespaceForXQueryUDF && 
					!function.equals("max") && !function.equals("min"))
				str += "local:";
			str += function + "(";
			if(optionXQueryMaxMinFix && (function.equals("max") || function.equals("min")))
				str += "(";
			for(int i=0; i<functionArgs.size(); i++) {
				String subExpr = functionArgs.get(i).translateMappingExpressionForColumnElementGReVQR(
						sqlVarToXQueryVar);
				if(subExpr == null) {
					return null;
				}
				else {
					str += subExpr;
					if(optionXQueryMaxMinFix && (function.equals("max") || function.equals("min")))
						str += " cast as xs:string";
				}
				if(i<functionArgs.size()-1)
					str += ", ";
			}
			str += ")";
			if(optionXQueryMaxMinFix && (function.equals("max") || function.equals("min")))
				str += ")";
			return str;
		}
		else
			return null; // shouldn't fall here - error in xbind (R.a0 doesn't have any actual data)
	}
	
	
	
	/**
 	 * look for relationAlias.ak and replace it with the clone of newTerms[k]
 	 * used for relationEncodedFunction decoding
	 * @param relationAlis
	 * @param newTerms
	 */
	public void replaceFunctionArg(String relationAlias, Term[] newTerms) {
		if(!this.isFunctionTerm())
			return;
		for(int i=0; i<this.functionArgs.size(); i++) {
			if(this.functionArgs.get(i).isRATerm() && 
					this.functionArgs.get(i).tableName.equals(relationAlias)) {
				int attributeIndex =
					Integer.parseInt(this.functionArgs.get(i).columnName.substring(1)); // "1" in "a1"
				this.functionArgs.remove(i);
				this.functionArgs.add(i, newTerms[attributeIndex].clone());
			}
			else if(this.functionArgs.get(i).isFunctionTerm())
				this.functionArgs.get(i).replaceFunctionArg(relationAlias, newTerms);	
		}
	}
	
	public Vector<Term> getAllFunctionMembers() {
		if(!this.isFunctionTerm())
			return null;
		
		Vector<Term> terms = new Vector<Term>();
		for(int i=0; i<this.functionArgs.size(); i++) {
			terms.add(this.functionArgs.get(i));
			Vector<Term> tempTerms = this.functionArgs.get(i).getAllFunctionMembers();
			if(tempTerms != null)
				terms.addAll(tempTerms);
		}
		
		return terms;
	}

	// return ground terms only
	public Vector<Term> getAllFunctionMembersGroundTerms() {
		if(!this.isFunctionTerm())
			return null;
		
		Vector<Term> terms = new Vector<Term>();
		for(int i=0; i<this.functionArgs.size(); i++) {
			if(!this.functionArgs.get(i).isFunctionTerm())
				terms.add(this.functionArgs.get(i));
			Vector<Term> tempTerms = this.functionArgs.get(i).getAllFunctionMembersGroundTerms();
			if(tempTerms != null)
				terms.addAll(tempTerms);
		}
		
		return terms;
	}
	
	public boolean simplifyOneArgMaxMinFunction() {
		boolean anyUpdate = false;
		if(!this.isFunctionTerm())
			return false;
		else if(!this.function.equals("max") &&
				!this.function.equals("min"))
			return false;

		// recursive simplification
		for(int i=0; i<this.functionArgs.size(); i++) {
			anyUpdate =
				this.functionArgs.get(i).simplifyOneArgMaxMinFunction() || anyUpdate;
		}
		// this function
		if(this.functionArgs.size() == 1) {
			Term singleArg = this.functionArgs.get(0);
			this.set(singleArg);
			anyUpdate = true;
		}
		return anyUpdate;
	}
	
	public boolean simplifyRedundantArgsMaxMinFunction() {
		boolean anyUpdate = false;
		if(!this.isFunctionTerm())
			return false;
		else if(!this.function.equals("max") &&
				!this.function.equals("min"))
			return false;

		// recursive simplification
		for(int i=0; i<this.functionArgs.size(); i++) {
			anyUpdate =
				this.functionArgs.get(i).simplifyRedundantArgsMaxMinFunction() || anyUpdate;
		}
		if(this.functionArgs.size() < 2)
			return anyUpdate;
		
		// this function
		int size = this.functionArgs.size();
		for(int i=0; i<size-1; i++) {
			Term arg1 = functionArgs.get(i);
			for(int j=i+1; j<size; j++) {
				Term arg2 = functionArgs.get(j);
				if(arg1.equals(arg2)) {
					functionArgs.remove(j);
					j--;
					size--;
					anyUpdate = true;
				}
			}
		}
		
		return anyUpdate;
	}
	
	public boolean simplifyNestedMaxFunction() {
		if(!this.isFunctionTerm() || !this.function.equals("max"))
			return false;
		boolean anyUpdate = false;
		
		int functionArgsSize = this.functionArgs.size();
		for(int i=0; i<functionArgsSize; i++) {
			Term argTerm = this.functionArgs.get(i);
			//System.out.println("argTerm: " + argTerm);
			if(argTerm.isFunctionTerm() && argTerm.function.equals("max")) {
				// recursive call
				anyUpdate = argTerm.simplifyNestedMaxFunction() || anyUpdate;
				
				// flattening
				for(int j=0; j<argTerm.functionArgs.size(); j++) {
					Term argTermArgTerm = argTerm.functionArgs.get(j);
					if(this.functionArgs.contains(argTermArgTerm))
						continue;
					this.functionArgs.add(argTermArgTerm);
					anyUpdate = true;
					functionArgsSize++;
				}
				this.functionArgs.remove(i);
				functionArgsSize--;
				i--;
			}
		}
		//System.out.println("simplifyMaxFunction done");
		return anyUpdate;
	}
	
	public boolean simplifyNestedMinFunction() {
		if(!this.isFunctionTerm() || !this.function.equals("min"))
			return false;
		boolean anyUpdate = false;
		
		int functionArgsSize = this.functionArgs.size();
		for(int i=0; i<functionArgsSize; i++) {
			Term argTerm = this.functionArgs.get(i);
			//System.out.println("argTerm: " + argTerm);
			if(argTerm.isFunctionTerm() && argTerm.function.equals("min")) {
				// recursive call
				anyUpdate = argTerm.simplifyNestedMinFunction() || anyUpdate;
				
				// flattening
				for(int j=0; j<argTerm.functionArgs.size(); j++) {
					Term argTermArgTerm = argTerm.functionArgs.get(j);
					if(this.functionArgs.contains(argTermArgTerm))
						continue;
					this.functionArgs.add(argTermArgTerm);
					anyUpdate = true;
					functionArgsSize++;
				}
				this.functionArgs.remove(i);
				functionArgsSize--;
				i--;
			}
		}
		//System.out.println("simplifyMaxFunction done");
		return anyUpdate;
	}
	
	public boolean simplifyMaxFunction() {
		boolean anyUpdate = false;
		
		while(true) {
			boolean submoduleUpdate = false;

			submoduleUpdate = simplifyNestedMaxFunction() || submoduleUpdate;
			submoduleUpdate = simplifyRedundantArgsMaxMinFunction() || submoduleUpdate;
			submoduleUpdate = simplifyOneArgMaxMinFunction() || submoduleUpdate;
			
			if(submoduleUpdate)
				anyUpdate = true;
			else
				break;
		}
		return anyUpdate;
	}

	public boolean simplifyMinFunction() {
		boolean anyUpdate = false;
		
		while(true) {
			boolean submoduleUpdate = false;

			submoduleUpdate = simplifyNestedMinFunction() || submoduleUpdate;
			submoduleUpdate = simplifyRedundantArgsMaxMinFunction() || submoduleUpdate;
			submoduleUpdate = simplifyOneArgMaxMinFunction() || submoduleUpdate;
			
			if(submoduleUpdate)
				anyUpdate = true;
			else
				break;
		}
		return anyUpdate;		
	}
	
	// test to see if all terms used in this (R.A or function arguments) are in givenTerms
	public boolean usingOnly(Vector<Term> givenTerms) {
		if(this.isConstantTerm())
			return true;
		else if(this.isRATerm()) {
			return givenTerms.contains(this);
		}
		else if(this.isFunctionTerm()) {
			for(int i=0; i<this.functionArgs.size(); i++) {
				boolean test = this.functionArgs.get(i).usingOnly(givenTerms);
				if(!test)
					return false;
			}
			return true;
		}
		else
			return true; // shouldn't arrive here - what is this case?
	}
	
	// 10-22-2007 
	 public int hashCode() { 
		    int hash = 1;


			if(this.isRATerm()) {
				hash = hash * 31 + this.tableName.hashCode(); 
				hash = hash * 31 + this.columnName.hashCode();
			}
			else if(this.isConstantTerm()) {
				hash = hash * 31 + this.constant.hashCode();
			}	
			else if(this.isFunctionTerm()) {
				hash = hash * 31 + this.function.hashCode();
				for(int i=0; i<this.functionArgs.size(); i++) {
					hash = hash * 31 + this.functionArgs.get(i).hashCode();
				}
			}
			else // if type differ
				hash = 0;
			
			return hash;	
	}

	public Vector<Term> getAllTermsDeep() {
		Vector<Term> terms = new Vector<Term>();
		terms.add(this);
		if(!this.isFunctionTerm())
			return terms;
		
		for(int i=0; i<this.functionArgs.size(); i++) {
			terms.addAll(this.functionArgs.get(i).getAllTermsDeep());
		}
		return terms;
	}
	
}
