package edu.ucla.cs.db.prism.backend.smo.util;

import java.util.Vector;

import edu.ucla.cs.db.prism.backend.SQL.RelationDeclaration;
import edu.ucla.cs.db.prism.backend.ded.QuantifierSet;
import edu.ucla.cs.db.prism.backend.xquery.ForClause;


public class TermSimplified {
	public String columnName;
	public String tableName;
	public String constant; // if constant is null then this term is table.column
							// in xquery, constant is used to represent variable
							// (e.g. $t1)
	public String function;
	public Vector<TermSimplified> functionArgs;

	// 10-10-2007 Hyun, 
	// used in marsManage.grev.ChasePostProcessing.updateColumnLevelXQueryWhereClauseOptimized()
	public boolean chosenAsEqual; 
	
	public TermSimplified(String tableName, String columnName) {
		this.columnName = columnName;
		this.tableName = tableName;
		this.constant = null;
	}

	public TermSimplified(String constant) {
		this.columnName = null;
		this.tableName = null;
		this.constant = constant;
	}

	public TermSimplified() {
		this.columnName = null;
		this.tableName = null;
		this.constant = null;
	}

	public TermSimplified(TermSimplified src) {
		this.set(src);
	}

	// set this term as a clone of src
	public void set(TermSimplified src) {
		if(src.isRATerm()) {
			this.columnName = new String(src.columnName);
			this.tableName = new String(src.tableName);
			this.constant = null;
		}
		else if(src.isConstantTerm()){ 
			this.columnName = null;
			this.tableName = null;
			this.constant = new String(src.constant);
		}
		else if(src.isFunctionTerm()) {
			this.function = new String(src.function);
			this.functionArgs = new Vector<TermSimplified>();
			for(int i=0; i<src.functionArgs.size(); i++)
				this.functionArgs.add(new TermSimplified(src.functionArgs.get(i)));
		}
	}
	
	@Override
	public TermSimplified clone() {
		//System.out.println("predicate cloning");
		return new TermSimplified(this);
	}


	
	public boolean hasNonquantifiedRelation(QuantifierSet quantifiers) {
		if(constant != null)
			return false;
		else if(quantifiers.hasVariable(tableName) == false) // using non-variable
			return true;
		else
			return false;
	}

	public void renameRelationVariable(String varPrefix, int varIndexIncrement, 
			String varToReplace, String newVarName) {

		if(constant != null) return;
		
		// case 1: "Sn" format -> "Sm" where m=n+varIndexIncrement
		if(varPrefix != null && tableName.substring(0,1).equals(varPrefix)) {
			tableName = varPrefix + 
				(Integer.parseInt(tableName.substring(1)) + varIndexIncrement);
		}
		
		// case 2: varToReplace -> newVarName (whole match & replace)
		else if(varToReplace != null && tableName.equals(varToReplace))
			tableName = newVarName;
	}

	
	public boolean isRATerm() {
		return (tableName != null && columnName != null);
	}

	public boolean isConstantTerm() {
		return constant != null;
	}

	public boolean isFunctionTerm() {
		return function != null && functionArgs != null;
		
	}
	public boolean isNestedFunctionTerm() {
		if(!isFunctionTerm())
			return false;
		for(int i=0; i<functionArgs.size(); i++) {
			if(functionArgs.get(i).isFunctionTerm())
				return true;
		}
		return false;
	}

	public boolean contentEquals(TermSimplified term2) {
		return this.equals(term2);
	}
	
	public boolean equals(TermSimplified term2) {
		if(term2 == null) return false; // "this" is not null
		if(this.isConstantTerm() && term2.isConstantTerm())
			return this.constant.equals(term2.constant);
		else if(this.isRATerm() && term2.isRATerm())
			return this.tableName.equals(term2.tableName) && 
					this.columnName.equals(term2.columnName);
		else if(this.isFunctionTerm() && term2.isFunctionTerm()) {
			boolean equalFlag = 
				this.function.equals(term2.function)
				&& (this.functionArgs.size() == term2.functionArgs.size());
			if(!equalFlag)
				return false; // if false, return false
			for(int i=0; i<this.functionArgs.size(); i++)
				if(!this.functionArgs.get(i).equals(term2.functionArgs.get(i)))
					return false;
			return true; // passed all equality check for function case
		}
		else // if type differ
			return false;
	}
	
	@Override
	public boolean equals(Object o) {
		return equals((TermSimplified)o);
	}
	
	@Override
	public String toString() {
		String str = null;
		if(isConstantTerm())
			str = constant;
		else if(isRATerm())
			str = tableName + "." + columnName;
		else if(isFunctionTerm()) {
			str = function + "(";
			if(functionArgs != null)
				for(int i=0; i<functionArgs.size(); i++) {
					str += functionArgs.get(i).toString();
					if(i<functionArgs.size()-1)
						str += ", ";
				}
			str += ")";
		}
		return str;
	}
	
	
	public String translateTsTeMappingExpression(String parentVarPrefix, // "$vsTable" of "$vsTable2"
			Vector<RelationDeclaration> vsTsSet, Vector<RelationDeclaration> vsTeSet) {
		if(isConstantTerm())
			return constant;
		else if(isRATerm()) {
			if(!this.columnName.equals("a2")) // it should be attr.a2 
				return null;
			for(int i=0; i<vsTsSet.size(); i++) {
				if(this.tableName.equals(vsTsSet.get(i).relationAlias))
					return parentVarPrefix + i + "/@ts";
				else if(this.tableName.equals(vsTeSet.get(i).relationAlias))
					return parentVarPrefix + i + "/@te";
			}
			return null; // shouldn't reach here
		}
		else if(isFunctionTerm()) {
			String str = function + "(";
			for(int i=0; i<functionArgs.size(); i++) {
				str += functionArgs.get(i).translateTsTeMappingExpression(parentVarPrefix, vsTsSet, vsTeSet);
				if(i<functionArgs.size()-1)
					str += ", ";
			}
			str += ")";
			return str;
		}
		else
			return null;
	}
	
	public String translateTsTeElemMappingExpressionGReV(String parentVarPrefix, // "$vsTable" of "$vsTable2"
			Vector<RelationDeclaration> rdSet) {
		// here in GReV, we pass values such that vsTeSet is not needed and made null
		
		if(isConstantTerm())
			return constant;
		else if(isRATerm()) {
			if(!this.columnName.equals("ts") && !this.columnName.equals("te") && !this.columnName.equals("el")) // it should be attr.a2 
				return null;

			for(int i=0; i<rdSet.size(); i++) {
				if(this.tableName.equals(rdSet.get(i).relationAlias) && this.columnName.equals("ts"))
					return parentVarPrefix + i + "/@ts";
				else if(this.tableName.equals(rdSet.get(i).relationAlias) && this.columnName.equals("te"))
					return parentVarPrefix + i + "/@te";
				else if(this.tableName.equals(rdSet.get(i).relationAlias) && this.columnName.equals("el"))
					return parentVarPrefix + i;
			}
			return null; // shouldn't reach here
		}
		else if(isFunctionTerm()) {
			String str = function + "(";
			for(int i=0; i<functionArgs.size(); i++) {
				str += functionArgs.get(i).translateTsTeElemMappingExpressionGReV(parentVarPrefix, rdSet);
				if(i<functionArgs.size()-1)
					str += ", ";
			}
			str += ")";
			return str;
		}
		else
			return null;
	}
	
	/*
	public String translateTsTeMappingExpressionForColumnElement(String rowVarPrefix, // "$vsTable" of "$vsTable2"
			Vector<Vector<RelationDeclaration>> vsTsSetSet, 
			Vector<Vector<RelationDeclaration>> vsTeSetSet,
			Vector<Vector<String>> vsColumnNameSetSet,
			Vector<ForClause> forClauses) {
		
		String colVarPrefix = "$vsCol";
		
		if(isConstantTerm())
			return constant;
		else if(isRATerm()) {
			if(!this.columnName.equals("a2")) // it should be attr.a2 
				return null;
			for(int i=0; i<vsTsSetSet.size(); i++) {
				for(int j=0; j<vsTsSetSet.get(i).size(); j++) {
					if(this.tableName.equals(vsTsSetSet.get(i).get(j).relationAlias)) {
						String var = colVarPrefix + i + "_" + j;
						String path = rowVarPrefix + i + "/" + vsColumnNameSetSet.get(i).get(j);
						ForClause forClause = new ForClause(var, path);
						forClause.relationAttribute = this.tableName + ".a2"; // (attr r1).a2
						if(!forClauses.contains(forClause))
							forClauses.add(forClause);
						return var + "/@ts";
					}
					else if(this.tableName.equals(vsTeSetSet.get(i).get(j).relationAlias)) {
						String var = colVarPrefix + i + "_" + j;
						String path = rowVarPrefix + i + "/" + vsColumnNameSetSet.get(i).get(j);
						ForClause forClause = new ForClause(var, path);
						forClause.relationAttribute = this.tableName + ".a2"; // (attr r1).a2
						if(!forClauses.contains(forClause))
							forClauses.add(forClause);
						return var + "/@te";
					}
				}
			}
			return null; // shouldn't reach here
		}
		else if(isFunctionTerm()) {
			String str = function + "(";
			for(int i=0; i<functionArgs.size(); i++) {
				str += functionArgs.get(i).translateTsTeMappingExpressionForColumnElement(
						rowVarPrefix, vsTsSetSet, vsTeSetSet, vsColumnNameSetSet, forClauses);
				if(i<functionArgs.size()-1)
					str += ", ";
			}
			str += ")";
			return str;
		}
		else
			return null;
	}
	
	public String translateTextMappingExpression(String rowVarPrefix, // "$vsTable" of "$vsTable2"
			Vector<Vector<RelationDeclaration>> vsTextSetSet,
			Vector<Vector<String>> vsColumnNameSetSet,
			Vector<ForClause> forClauses) {
		
		String colVarPrefix = "$vsCol";
		if(isConstantTerm())
			return constant;
		else if(isRATerm()) {
			if(!this.columnName.equals("a1")) // it should be text.a1
				return null;
			for(int i=0; i<vsTextSetSet.size(); i++) {
				for(int j=0; j<vsTextSetSet.get(i).size(); j++) {
					if(this.tableName.equals(vsTextSetSet.get(i).get(j).relationAlias)) {
						String var = colVarPrefix + i + "_" + j;
						String path = rowVarPrefix + i + "/" + vsColumnNameSetSet.get(i).get(j);
						ForClause forClause = new ForClause(var, path);
						forClause.relationAttribute = this.tableName + ".a1"; // (text r1).a1
						if(!forClauses.contains(forClause))
							forClauses.add(forClause);
						return var + "/text()";
					}
				}
			}
			return null; // shouldn't reach here
		}
		else if(isFunctionTerm()) {
			String str = function + "(";
			for(int i=0; i<functionArgs.size(); i++) {
				str += functionArgs.get(i).translateTextMappingExpression(
						rowVarPrefix, vsTextSetSet, vsColumnNameSetSet, forClauses);
				if(i<functionArgs.size()-1)
					str += ", ";
			}
			str += ")";
			return str;
		}
		else
			return null;
	}
	*/
	

	public String translateTsTeMappingExpressionForColumnElement(String rowVarPrefix, // "$vsTable" of "$vsTable2"
			Vector<Vector<RelationDeclaration>> vsTsSetSet, 
			Vector<Vector<RelationDeclaration>> vsTeSetSet,
			Vector<Vector<String>> vsColumnNameSetSet,
			Vector<ForClause> forClauses,
			Vector<TermSimplified> sqlTermsUsed) {
		return translateMappingExpressionForColumnElement(rowVarPrefix,
				vsTsSetSet, 
				vsTeSetSet,
				null,
				vsColumnNameSetSet,
				forClauses,
				sqlTermsUsed);
	}

	public String translateTextMappingExpression(String rowVarPrefix, // "$vsTable" of "$vsTable2"
			Vector<Vector<RelationDeclaration>> vsTextSetSet,
			Vector<Vector<String>> vsColumnNameSetSet,
			Vector<ForClause> forClauses,
			Vector<TermSimplified> sqlTermsUsed) {
		return translateMappingExpressionForColumnElement(rowVarPrefix,
				null,
				null,
				vsTextSetSet,
				vsColumnNameSetSet,
				forClauses,
				sqlTermsUsed);
		
	}
	public String translateTsTeMappingExpressionForColumnElementGReV(String rowVarPrefix, // "$vsTable" of "$vsTable2"
			Vector<Vector<RelationDeclaration>> vsTsSetSet, 
			Vector<Vector<RelationDeclaration>> vsTeSetSet,
			Vector<Vector<String>> vsColumnNameSetSet,
			Vector<ForClause> forClauses,
			Vector<TermSimplified> sqlTermsUsed) {
		return translateMappingExpressionForColumnElementGReV(rowVarPrefix,
				vsTsSetSet, 
				vsColumnNameSetSet,
				forClauses,
				sqlTermsUsed);
	}

	public String translateTextMappingExpressionGReV(String rowVarPrefix, // "$vsTable" of "$vsTable2"
			Vector<Vector<RelationDeclaration>> vsTextSetSet,
			Vector<Vector<String>> vsColumnNameSetSet,
			Vector<ForClause> forClauses,
			Vector<TermSimplified> sqlTermsUsed) {
		return translateMappingExpressionForColumnElementGReV(rowVarPrefix,
				vsTextSetSet,
				vsColumnNameSetSet,
				forClauses,
				sqlTermsUsed);
		
	}
	/**
	 * This incorporates translateTsTeMappingExpressionForColumnElement() and translateTextMappingExpression() above
	 * 
	 * @param rowVarPrefix
	 * @param vsTsSetSet
	 * @param vsTeSetSet
	 * @param vsTextSetSet
	 * @param vsColumnNameSetSet
	 * @param forClauses
	 * @return
	 */
	public String translateMappingExpressionForColumnElement(String rowVarPrefix, // "$vsTable" of "$vsTable2"
			Vector<Vector<RelationDeclaration>> vsTsSetSet, 
			Vector<Vector<RelationDeclaration>> vsTeSetSet,
			Vector<Vector<RelationDeclaration>> vsTextSetSet,
			Vector<Vector<String>> vsColumnNameSetSet,
			Vector<ForClause> forClauses,
			Vector<TermSimplified> sqlTermsUsed) // record what terms are found in this possibly complex term
	{
		
		String colVarPrefix = "$vsCol";
		
		if(isConstantTerm())
			return constant;
		else if(isRATerm() && this.columnName.equals("a2") && vsTsSetSet != null && vsTeSetSet != null) {// attr.a2 
			for(int i=0; i<vsTsSetSet.size(); i++) {
				for(int j=0; j<vsTsSetSet.get(i).size(); j++) {
					if(this.tableName.equals(vsTsSetSet.get(i).get(j).relationAlias)) {
						String var = colVarPrefix + i + "_" + j;
						String path = rowVarPrefix + i + "/" + vsColumnNameSetSet.get(i).get(j);
						ForClause forClause = new ForClause(var, path);
						//forClause.relationAttribute = this.tableName + ".a2"; // (attr r1).a2
						if(sqlTermsUsed != null && !sqlTermsUsed.contains(this))
							sqlTermsUsed.add(this);
						if(!forClauses.contains(forClause))
							forClauses.add(forClause);
						return var + "/@ts";
					}
					else if(this.tableName.equals(vsTeSetSet.get(i).get(j).relationAlias)) {
						String var = colVarPrefix + i + "_" + j;
						String path = rowVarPrefix + i + "/" + vsColumnNameSetSet.get(i).get(j);
						ForClause forClause = new ForClause(var, path);
						//forClause.relationAttribute = this.tableName + ".a2"; // (attr r1).a2
						if(sqlTermsUsed != null && !sqlTermsUsed.contains(this))
							sqlTermsUsed.add(this);
						if(!forClauses.contains(forClause))
							forClauses.add(forClause);
						return var + "/@te";
					}
				}
			}
			return null; // shouldn't reach here
		}
		else if(isRATerm() && this.columnName.equals("a1") && vsTextSetSet != null) {// text.a1
			for(int i=0; i<vsTextSetSet.size(); i++) {
				for(int j=0; j<vsTextSetSet.get(i).size(); j++) {
					if(this.tableName.equals(vsTextSetSet.get(i).get(j).relationAlias)) {
						String var = colVarPrefix + i + "_" + j;
						String path = rowVarPrefix + i + "/" + vsColumnNameSetSet.get(i).get(j);
						ForClause forClause = new ForClause(var, path);
						//forClause.relationAttribute = this.tableName + ".a1"; // (text r1).a1
						if(sqlTermsUsed != null && !sqlTermsUsed.contains(this))
							sqlTermsUsed.add(this);
						if(!forClauses.contains(forClause))
							forClauses.add(forClause);
						return var + "/text()";
					}
				}
			}
			return null; // shouldn't reach here
		}
		else if(isFunctionTerm()) {
			String str = function + "(";
			for(int i=0; i<functionArgs.size(); i++) {
				String subExpr = functionArgs.get(i).translateMappingExpressionForColumnElement(
						rowVarPrefix, vsTsSetSet, vsTeSetSet, vsTextSetSet, vsColumnNameSetSet, 
						forClauses, sqlTermsUsed);
				if(subExpr == null) 
					return null;
				else
					str += subExpr;
				if(i<functionArgs.size()-1)
					str += ", ";
			}
			str += ")";
			return str;
		}
		else
			return null; // shouldn't fall here - error in xbind (R.a0 doesn't have any actual data)
	}
	
	/**
	 * This incorporates translateTsTeMappingExpressionForColumnElement() and translateTextMappingExpression() above
	 * 
	 * @param rowVarPrefix
	 * @param vsTsSetSet
	 * @param vsTeSetSet
	 * @param vsTextSetSet
	 * @param vsColumnNameSetSet
	 * @param forClauses
	 * @return
	 */
	public String translateMappingExpressionForColumnElementGReV(String rowVarPrefix, // "$vsTable" of "$vsTable2"
			Vector<Vector<RelationDeclaration>> rdSetSet, 
			Vector<Vector<String>> vsColumnNameSetSet,
			Vector<ForClause> forClauses,
			Vector<TermSimplified> sqlTermsUsed) // record what terms are found in this possibly complex term
	{
		// TODO: to alter this according to GReV
		
		String colVarPrefix = "$vsCol";
		
		if(isConstantTerm())
			return constant;
		else if(rdSetSet != null && isRATerm() && 
				(this.columnName.equals("ts") || 
				 this.columnName.equals("te") ||
				 this.columnName.equals("text") ||
				 this.columnName.equals("el"))) { 
			for(int i=0; i<rdSetSet.size(); i++) {
				for(int j=0; j<rdSetSet.get(i).size(); j++) {
					if(!this.tableName.equals(rdSetSet.get(i).get(j).relationAlias))
						continue;
					
					if(this.columnName.equals("ts") || this.columnName.equals("te")) {
						String var = colVarPrefix + i + "_" + j;
						String path = rowVarPrefix + i + "/" + vsColumnNameSetSet.get(i).get(j);
						ForClause forClause = new ForClause(var, path);
						if(sqlTermsUsed != null && !sqlTermsUsed.contains(this))
							sqlTermsUsed.add(this);
						if(forClause != null  && !forClauses.contains(forClause))
							forClauses.add(forClause);
						return var + "/@" + this.columnName;
					}
					else if(this.columnName.equals("text")) {
						String var = colVarPrefix + i + "_" + j;
						String path = rowVarPrefix + i + "/" + vsColumnNameSetSet.get(i).get(j);
						ForClause forClause = new ForClause(var, path);
						if(sqlTermsUsed != null && !sqlTermsUsed.contains(this))
							sqlTermsUsed.add(this);
						if(forClause != null  && !forClauses.contains(forClause))
							forClauses.add(forClause);
						return var + "/text()";
					}
					else { //if(this.columnName.equals("el")) {
						String var = colVarPrefix + i + "_" + j;
						String path = rowVarPrefix + i + "/" + vsColumnNameSetSet.get(i).get(j);
						ForClause forClause = new ForClause(var, path);
						if(sqlTermsUsed != null && !sqlTermsUsed.contains(this))
							sqlTermsUsed.add(this);
						if(forClause != null  && !forClauses.contains(forClause))
							forClauses.add(forClause);
						return var;
					}
				}
			}
			return null; // shouldn't reach here
		}
		else if(isFunctionTerm()) {
			String str = function + "(";
			//System.out.println("function!: " + this.toString());
			for(int i=0; i<functionArgs.size(); i++) {
				String subExpr = functionArgs.get(i).translateMappingExpressionForColumnElementGReV(
						rowVarPrefix, rdSetSet, vsColumnNameSetSet, 
						forClauses, sqlTermsUsed);
				if(subExpr == null) {
					//System.out.println("Function subexpr NULL: " + functionArgs.get(i));
					//System.out.println("vsTextSetSet: " + vsTextSetSet);
					return null;
				}
				else
					str += subExpr;
				if(i<functionArgs.size()-1)
					str += ", ";
			}
			str += ")";
			return str;
		}
		else
			return null; // shouldn't fall here - error in xbind (R.a0 doesn't have any actual data)
	}
	
	/**
 	 * look for relationAlias.ak and replace it with the clone of newTerms[k]
 	 * used for relationEncodedFunction decoding
	 * @param relationAlis
	 * @param newTerms
	 */
	public void replaceFunctionArg(String relationAlias, TermSimplified[] newTerms) {
		if(!this.isFunctionTerm())
			return;
		for(int i=0; i<this.functionArgs.size(); i++) {
			if(this.functionArgs.get(i).isRATerm() && 
					this.functionArgs.get(i).tableName.equals(relationAlias)) {
				int attributeIndex =
					Integer.parseInt(this.functionArgs.get(i).columnName.substring(1)); // "1" in "a1"
				this.functionArgs.remove(i);
				this.functionArgs.add(i, newTerms[attributeIndex].clone());
			}
			else if(this.functionArgs.get(i).isFunctionTerm())
				this.functionArgs.get(i).replaceFunctionArg(relationAlias, newTerms);	
		}
	}
	
	public Vector<TermSimplified> getAllFunctionMembers() {
		if(!this.isFunctionTerm())
			return null;
		
		Vector<TermSimplified> terms = new Vector<TermSimplified>();
		for(int i=0; i<this.functionArgs.size(); i++) {
			terms.add(this.functionArgs.get(i));
			Vector<TermSimplified> tempTerms = this.functionArgs.get(i).getAllFunctionMembers();
			if(tempTerms != null)
				terms.addAll(tempTerms);
		}
		
		return terms;
	}

	// return ground terms only
	public Vector<TermSimplified> getAllFunctionMembersGroundTerms() {
		if(!this.isFunctionTerm())
			return null;
		
		Vector<TermSimplified> terms = new Vector<TermSimplified>();
		for(int i=0; i<this.functionArgs.size(); i++) {
			if(!this.functionArgs.get(i).isFunctionTerm())
				terms.add(this.functionArgs.get(i));
			Vector<TermSimplified> tempTerms = this.functionArgs.get(i).getAllFunctionMembersGroundTerms();
			if(tempTerms != null)
				terms.addAll(tempTerms);
		}
		
		return terms;
	}

	public void simplifyMaxFunction() {
		if(!this.isFunctionTerm() || !this.function.equals("max"))
			return;

		//System.out.println(this);

		// type 1: simplify single argument max
		/*
		if(this.functionArgs.size() == 1) {
			Term singleArg = this.functionArgs.get(0);
			this.set(singleArg);
			this.simplifyMaxFunction();
		}
		*/
			
		
		// type 2: simplify nested max
		int functionArgsSize = this.functionArgs.size();
		for(int i=0; i<functionArgsSize; i++) {
			TermSimplified argTerm = this.functionArgs.get(i);
			//System.out.println("argTerm: " + argTerm);
			if(argTerm.isFunctionTerm() && argTerm.function.equals("max")) {
				argTerm.simplifyMaxFunction();
				for(int j=0; j<argTerm.functionArgs.size(); j++) {
					TermSimplified argTermArgTerm = argTerm.functionArgs.get(j);
					if(this.functionArgs.contains(argTermArgTerm))
						continue;
					this.functionArgs.add(argTermArgTerm); // flattening
					functionArgsSize++;
				}
				this.functionArgs.remove(i);
				functionArgsSize--;
				i--;
			}
		}
		
		//System.out.println("simplifyMaxFunction done");
	}
	
	public void simplifyMinFunction() {
		if(!this.isFunctionTerm() || !this.function.equals("min"))
			return;
		
		// type 1: simplify single argument max
		/*
		if(this.functionArgs.size() == 1) {
			this.set(this.functionArgs.get(0));
			this.simplifyMaxFunction();
		}
		*/
			
		
		// type 2: simplify nested min
		int functionArgsSize = this.functionArgs.size();
		for(int i=0; i<functionArgsSize; i++) {
			TermSimplified argTerm = this.functionArgs.get(i);
			//System.out.println("argTerm: " + argTerm);
			if(argTerm.isFunctionTerm() && argTerm.function.equals("min")) {
				argTerm.simplifyMaxFunction();
				for(int j=0; j<argTerm.functionArgs.size(); j++) {
					TermSimplified argTermArgTerm = argTerm.functionArgs.get(j);
					if(this.functionArgs.contains(argTermArgTerm))
						continue;
					this.functionArgs.add(argTermArgTerm); // flattening
					functionArgsSize++;
				}
				this.functionArgs.remove(i);
				functionArgsSize--;
				i--;
			}
		}
	}
	
	// test to see if all terms used in this (R.A or function arguments) are in givenTerms
	public boolean usingOnly(Vector<TermSimplified> givenTerms) {
		if(this.isConstantTerm())
			return true;
		else if(this.isRATerm()) {
			return givenTerms.contains(this);
		}
		else if(this.isFunctionTerm()) {
			for(int i=0; i<this.functionArgs.size(); i++) {
				boolean test = this.functionArgs.get(i).usingOnly(givenTerms);
				if(!test)
					return false;
			}
			return true;
		}
		else
			return true; // shouldn't arrive here - what is this case?
	}
	
}
