package edu.ucla.cs.db.prism.backend.temporalOptimize;
import java.util.Vector;

public class RV {
	// RV: Required Versions for each relation alias in SQL From Clause
	public String relationAlias;
	public Vector<Integer> versions;
	
	
	public RV() {
		this.relationAlias = null;
		versions = new Vector<Integer>();
	}
	
	public RV(RV rv) {
		this.relationAlias = rv.relationAlias;
		this.versions = new Vector<Integer>();
		for(int i=0; i<rv.versions.size(); i++)
			this.versions.add(new Integer(rv.versions.get(i).intValue()));
	}
	
	public RV(String relationAlias, int firstVersion, int lastVersion) {
		this.relationAlias = relationAlias;
		this.versions = new Vector<Integer>();
		if(firstVersion <= lastVersion)
			for(int i=firstVersion; i<=lastVersion; i++)
				this.versions.add(new Integer(i));
	}
	
	public RV(String relationAlias, Vector<Integer> versions2) {
		this.relationAlias = relationAlias;
		this.versions = new Vector<Integer>();
		for(int i=0; i<versions2.size(); i++)
			this.versions.add(new Integer(versions2.get(i).intValue()));
		this.sortVersions();
	}
	
	public boolean contains(int version) {
		for(int i=0; i<versions.size(); i++)
			if(versions.get(i).intValue() == version)
				return true;				
		return false;
	}
	
	public RV intersection(RV rv2) {
		if(rv2 == null)
			return null;
		if(!this.relationAlias.equals(rv2.relationAlias))
			return null;
		RV rv3 = new RV();
		rv3.relationAlias = this.relationAlias;
		
		for(int i=0; i<this.versions.size(); i++) {
			int version = this.versions.get(i).intValue();
			if(rv2.contains(version))
				rv3.versions.add(new Integer(version));
		}
		rv3.sortVersions();
		return rv3;
	}
	
	static public Vector<Integer> intersection(Vector<Integer> numbers1, Vector<Integer> numbers2) {
		if(numbers2 == null)
			return null;
		Vector<Integer> numbers3 = new Vector<Integer>();
		
		for(int i=0; i<numbers1.size(); i++) {
			Integer version = numbers1.get(i);
			if(numbers2.contains(version))
				numbers3.add(new Integer(version));
		}
		edu.ucla.cs.db.prism.backend.util.Sort.sortIntegerVector(numbers3);
		return numbers3;
	}
	
	public RV differentRelationAliasesIntersection(RV rv2) {
		if(rv2 == null)
			return null;
		//if(!this.relationAlias.equals(rv2.relationAlias))
		//	return null;
		RV rv3 = new RV();
		rv3.relationAlias = this.relationAlias;
		
		for(int i=0; i<this.versions.size(); i++) {
			int version = this.versions.get(i).intValue();
			if(rv2.contains(version))
				rv3.versions.add(new Integer(version));
		}
		rv3.sortVersions();
		return rv3;
	}
	
	// update this = this.intersection(rv2)
	public void inPlaceDifferentRelationAliasesIntersection(RV rv2) {
		RV newRV = this.differentRelationAliasesIntersection(rv2);
		this.versions = newRV.versions;
	}
	
	public RV union(RV rv2) {
		if(rv2 == null)
			return this;
		if(!this.relationAlias.equals(rv2.relationAlias))
			return null;
		RV rv3 = new RV(this);
		
		for(int i=0; i<rv2.versions.size(); i++) {
			int version = rv2.versions.get(i).intValue();
			if(!rv3.contains(version))
				rv3.versions.add(new Integer(version));
		}
		rv3.sortVersions();
		return rv3;
	}
	
	public RV inPlaceIntersectionRV2AndGreater(RV rv2) {
		// returns this ^ (rv2 and greater versiosn than rv2)

		// if rv2 is empty
		if(rv2.versions.size()==0) {
			this.versions = new Vector<Integer>();
			return this;
		}
			
		int rv2MinVersion = rv2.versions.get(0).intValue();

		int size = this.versions.size();
		for(int i=0; i<size; i++) {
			if(this.versions.get(i).intValue() < rv2MinVersion) {
				this.versions.remove(i);
				i--;
				size--;
			}
		}
		return this;
	}

	public RV inPlaceIntersectionRV2AndLess(RV rv2) {
		// returns this ^ (rv2 and less versions than rv2)
		// if rv2 is empty
		if(rv2.versions.size()==0) {
			this.versions = new Vector<Integer>();
			return this;
		}

		int rv2MaxVersion = rv2.versions.get(rv2.versions.size()-1).intValue();

		int size = this.versions.size();
		for(int i=0; i<size; i++) {
			if(this.versions.get(i).intValue() > rv2MaxVersion) {
				this.versions.remove(i);
				i--;
				size--;
			}
		}
		return this;
	}
	
	// case 1. R1.ts < R2.(ts|te) -> VB1i= VB1i if a version of RV2 intersecting or following VB1i exists, empty set otherwise
	// case 2. R1.ts = R2.(ts|te) -> VB1i= earliest VB1i^RV2 and the following versions of VB1i
	// case 3. R1.ts > R2.(ts|te) -> VB1i= VB1i^(RV2 U all-following(RV2))
	
	// case 4. R1.te < R2.(ts|te) -> VB1i= VB1i^(RV2 U all-precedding(RV2))
	// case 5. R1.te = R2.(ts|te) -> VB1i= last VB1i^RV2 and the preceeding versions of VB1i
	// case 6. R1.te > R2.(ts|te) -> VB1i= VB1i if a version of RV2 intersecting or preceeding VB1i exists, empty set otherwise
	
	// case 1's result
	public RV inPlaceAllOrNoneByR2IntersectingOrFollowingForEachBlock(RV rv2) {
		if(this.versions.size() == 0) return this;
		else if(rv2.versions.size() == 0) { // if rv2 is empty, delete all and return empty
			versions = new Vector<Integer>();
			return this;
		}
		
		int beginningOfVB = 0;
		int endOfVB = -1;
		while(true) {
			endOfVB = getLastIndexOfThisVersionBlock(versions, beginningOfVB);
			if(endOfVB==-1) break; // finished navigating VBs
			
			// if there's no RV2 version intersecting or following, 
			// delete this version block
			if(versions.get(beginningOfVB) > rv2.getLastVersion()) {
				for(int i=beginningOfVB; i<=endOfVB; i++)
					versions.remove(beginningOfVB);
			}
			else // there is RV2 version intersecting of following, go to next block
				beginningOfVB = endOfVB+1;
		}
		return this;
	}
	
	// case 6's result
	public RV inPlaceAllOrNoneByR2IntersectingOrPreceedingForEachBlock(RV rv2) {
		if(this.versions.size() == 0) return this;
		else if(rv2.versions.size() == 0) { // if rv2 is empty, delete all and return empty
			versions = new Vector<Integer>();
			return this;
		}
		
		int beginningOfVB = 0;
		int endOfVB = -1;
		while(true) {
			endOfVB = getLastIndexOfThisVersionBlock(versions, beginningOfVB);
			if(endOfVB==-1) break; // finished navigating VBs
			
			// if there's no RV2 version intersecting or preceeding, 
			// delete this version block
			if(versions.get(endOfVB) < rv2.getFirstVersion()) {
				for(int i=beginningOfVB; i<=endOfVB; i++)
					versions.remove(beginningOfVB);
			}
			else // there is RV2 version intersecting of following, go to next block
				beginningOfVB = endOfVB+1;
		}
		return this;
	}
	
	// case 2's result
	public RV inPlaceEarliestIntersectionAndFollowingForEachBlock(RV rv2) {
		if(this.versions.size() == 0) return this;
		else if(rv2.versions.size() == 0) { // if rv2 is empty, delete all and return empty
			versions = new Vector<Integer>();
			return this;
		}
		
		int beginningOfVB = 0;
		int endOfVB = -1;
		while(true) {
			endOfVB = getLastIndexOfThisVersionBlock(versions, beginningOfVB);
			if(endOfVB==-1) break; // finished navigating VBs
			
			for(int i=beginningOfVB; i<=endOfVB; i++) {
				if(rv2.contains(versions.get(i)))
					break; // the rest of the block survives
				else { // remove this
					versions.remove(i);
					i--;
					endOfVB--;
				}
			}
			
			// go to next block
			beginningOfVB = endOfVB+1;
		}
		return this;
	}
	
	// case 5's result
	public RV inPlaceLastIntersectionAndPreceedingForEachBlock(RV rv2) {
		if(this.versions.size() == 0) return this;
		else if(rv2.versions.size() == 0) { // if rv2 is empty, delete all and return empty
			versions = new Vector<Integer>();
			return this;
		}
		
		int beginningOfVB = 0;
		int endOfVB = -1;
		while(true) {
			endOfVB = getLastIndexOfThisVersionBlock(versions, beginningOfVB);
			if(endOfVB==-1) break; // finished navigating VBs
			
			for(int i=endOfVB; i>=beginningOfVB; i--) {
				if(rv2.contains(versions.get(i)))
					break; // the rest of the block survives
				else { // remove this
					versions.remove(i);
					//i--;
					endOfVB--;
				}
			}
			
			// go to next block
			beginningOfVB = endOfVB+1;
		}
		return this;
	}
	
	private int getLastIndexOfThisVersionBlock(Vector<Integer> versions, int beginningOfThisVersionBlock) {
		if(beginningOfThisVersionBlock >= versions.size())
			return -1; // this version block does not exist; finished navigating VBs
		
		for(int i=beginningOfThisVersionBlock; i<versions.size()-1; i++) {
			if(versions.get(i).intValue()+1 < versions.get(i+1).intValue()) {
				return i;
			}
		}
		return versions.size()-1; // last index
	}
	
	public boolean equals(RV rv2) {
		if(!this.relationAlias.equals(rv2.relationAlias))
			return false;
		if(this.versions.size() != rv2.versions.size())
			return false;
		for(int i=0; i<this.versions.size(); i++) { // assume sorted
			if(this.versions.get(i).intValue() != 
				rv2.versions.get(i).intValue())
				return false;
		}
		return true;
	}
	
	// we assume sorted versions
	public int getFirstVersion() {
		if(versions.size() == 0) return -1;
		else
			return versions.get(0);
	}
	
	// we assume sorted versions
	public int getLastVersion() {
		if(versions.size() == 0) return -1;
		else
			return versions.get(versions.size()-1);
	}
	
	public void sortVersions() {
		// sort versions in ascending order
		for(int i=0; i<versions.size(); i++) {
			for(int j=i+1; j<versions.size(); j++) {
				if(versions.get(i).intValue() > versions.get(j).intValue()) {
					Integer tempj = versions.remove(j);
					Integer tempi = versions.remove(i);
					versions.add(i, tempj);
					versions.add(j, tempi);
				}
			}
		}
	}

	@Override
	public String toString() {
		return relationAlias + " : " + versions.toString();
	}
}
