package edu.ucla.cs.db.prism.backend.temporalOptimize;
import java.util.Vector;

public class RVSet {
	public Vector<RV> rvs;
	
	public RVSet() {
		rvs = new Vector<RV>();
	}

	public RVSet(RVSet rvSet) {
		rvs = new Vector<RV>();
		for(int i=0; i<rvSet.rvs.size(); i++)
			rvs.add(new RV(rvSet.rvs.get(i)));
	}
	
	/*
	public RVSet(SQL sql, VersionIntervalSet viSet) {
		rvs = new Vector<RV>();
		for(int i=0; i<sql.from.relationDeclarations.size(); i++) {
			RV rv = new RV();
			rv.relationAlias = sql.from.relationDeclarations.get(i).relationAlias;
			rv.versions = viSet.getAllVersions();
			rvs.add(rv);
		}
	}
	*/
	
	public RVSet(Vector<String> relationAliases, VersionIntervalSet VISet) {
		rvs = new Vector<RV>();
		for(int i=0; i<relationAliases.size(); i++) {
			RV rv = new RV();
			rv.relationAlias = relationAliases.get(i);
			rv.versions = VISet.getAllVersions();
			rvs.add(rv);
		}
	}
	
	public RVSet deepCopy() {
		return new RVSet(this);
	}

	public RV get(String relationAlias) {
		for(int i=0; i<rvs.size(); i++)
			if(rvs.get(i).relationAlias.equals(relationAlias))
				return rvs.get(i);
		return null;
	}

	public RV remove(String relationAlias) {
		for(int i=0; i<rvs.size(); i++)
			if(rvs.get(i).relationAlias.equals(relationAlias))
				return rvs.remove(i);
		return null;
	}
	
	
	// set intersection, element intersection
	public RVSet intersection(RVSet rvSet2) {
		RVSet rvSet3 = new RVSet();
		for(int i=0; i<this.rvs.size(); i++) {
			RV rv1 = this.rvs.get(i);
			RV rv2 = rvSet2.get(rv1.relationAlias);
			if(rv2 == null) // rv1 not found in rvSet2
				continue;
			rvSet3.rvs.add(rv1.intersection(rv2));
		}
		return rvSet3;
	}

	// set intersection, element intersection
	public RVSet intersection(Vector<Integer> versions) {
		RVSet rvSet3 = new RVSet();
		for(int i=0; i<this.rvs.size(); i++) {
			RV rv1 = this.rvs.get(i);
			RV rv2 = new RV();
			rv2.relationAlias = rv1.relationAlias;
			rv2.versions = versions;
			rvSet3.rvs.add(rv1.intersection(rv2));
		}
		return rvSet3;
	}
	
	// set union, element union
	public RVSet union(RVSet rvSet2) {
		RVSet rvSet3 = new RVSet();
		for(int i=0; i<this.rvs.size(); i++) {
			RV rv1 = this.rvs.get(i);
			RV rv2 = rvSet2.get(rv1.relationAlias);
			if(rv2 == null) // rv1 not found in rvSet2
				rvSet3.rvs.add(new RV(rv1));
			else
				rvSet3.rvs.add(rv1.union(rv2));
		}
		for(int i=0; i<rvSet2.rvs.size(); i++) {
			RV rv2 = rvSet2.rvs.get(i);
			RV rv1 = this.get(rv2.relationAlias);
			if(rv1 == null) // rv2 not found in rvSet1 (this)
				rvSet3.rvs.add(new RV(rv2));
		}
		return rvSet3;
	}
	
	// set union, element intersection 
	// - when there's only one element with a relationAlias, include it in the result
	// - when two elements share a relationAlias, do intersection between them
	public RVSet setUnionElementIntersection(RVSet rvSet2) {
		RVSet rvSet3 = new RVSet();
		for(int i=0; i<this.rvs.size(); i++) {
			RV rv1 = this.rvs.get(i);
			RV rv2 = rvSet2.get(rv1.relationAlias);
			if(rv2 == null) // rv1 not found in rvSet2
				rvSet3.rvs.add(new RV(rv1));
			else
				rvSet3.rvs.add(rv1.intersection(rv2));
		}
		for(int i=0; i<rvSet2.rvs.size(); i++) {
			RV rv2 = rvSet2.rvs.get(i);
			RV rv1 = this.get(rv2.relationAlias);
			if(rv1 == null) // rv2 not found in rvSet1 (this)
				rvSet3.rvs.add(new RV(rv2));
		}
		return rvSet3;
	}
	
	public boolean equals(RVSet rvSet2) {
		if(this.rvs.size() != rvSet2.rvs.size())
			return false;
		for(int i=0; i<this.rvs.size(); i++) {
			RV rv1 = this.rvs.get(i);
			RV rv2 = rvSet2.get(rv1.relationAlias);
			if(rv2 == null) return false;
			if(!rv1.equals(rv2)) return false;
		}
		
		return true;
	}
	
	public boolean hasEmptyRV() {
		for(int i=0; i<rvs.size(); i++)
			if(rvs.get(i).versions.size() == 0)
				return true;
		return false;
	}
	
	public Vector<Integer> getAllVersionsUnioned() {
		Vector<Integer> versionSet = new Vector<Integer>();
		for(int i=0; i<rvs.size(); i++) {
			RV rv = rvs.get(i);
			for(int j=0; j<rv.versions.size(); j++) {
				Integer version = rv.versions.get(j);
				if(!versionSet.contains(version))
					versionSet.add(version);
			}
		}
		edu.ucla.cs.db.prism.backend.util.Sort.sortIntegerVector(versionSet);
		return versionSet;
	}
	
	@Override
	public String toString() {
		String str = "";
		for(int i=0; i<rvs.size(); i++) {
			str += rvs.get(i).toString() + "\n";
		}
		return str;
	}

}
