package edu.ucla.cs.db.prism.backend.temporalOptimize;
import java.util.Vector;

public class VersionIntervalSet {
	public Vector<VersionInterval> versionIntervals;

	public VersionIntervalSet () {
		versionIntervals = new Vector<VersionInterval>();
	}
	
	public VersionInterval getByTime(String timeInstance) {
		for(int i=0; i<versionIntervals.size(); i++) {
			if(versionIntervals.get(i).contains(timeInstance))
				return versionIntervals.get(i);
		}
		
		return null; // not found
	}
	
	public VersionInterval getByVersion(int version) {
		for(int i=0; i<versionIntervals.size(); i++) {
			if(versionIntervals.get(i).version == version)
				return versionIntervals.get(i);
		}
		return null;
	}
	
	// this needs be called only once after setting start times
	public void SetEndTime() {
		// step 1: find max version
		int maxVersion = this.getLastVersion().version;
		
		for(int i=0; i<versionIntervals.size(); i++) {
			int thisVersion = versionIntervals.get(i).version;
			// last version's end time is "now"
			if(thisVersion == maxVersion)
				versionIntervals.get(i).endTime = VersionInterval.maximumTimeInstance;

			else {
				VersionInterval nextVI = this.getByVersion(thisVersion+1);
				if(nextVI == null) {
					System.err.println("Error: discontinued version time information");
					System.exit(0);
				}
				else
					versionIntervals.get(i).endTime = nextVI.startTime;
			}
		}
	}
	
	public Vector<Integer> getAllVersions() {
		Vector<Integer> allVersions = new Vector<Integer>();
		for(int i=0; i<versionIntervals.size(); i++)
			allVersions.add(new Integer(versionIntervals.get(i).version));
		return allVersions;
	}
	
	public VersionInterval getFirstVersion() {
		VersionInterval vi = versionIntervals.get(0);
		for(int i=1; i<versionIntervals.size(); i++) 
			if(versionIntervals.get(i).version < vi.version) 
				vi = versionIntervals.get(i);
		return vi;
	}
	
	public VersionInterval getLastVersion() {
		VersionInterval vi = versionIntervals.get(0);
		for(int i=1; i<versionIntervals.size(); i++) 
			if(versionIntervals.get(i).version > vi.version) 
				vi = versionIntervals.get(i);
		return vi;
	}
	
	@Override
	public String toString() {
		String str = "";
		for(int i=0; i<versionIntervals.size(); i++) {
			str += versionIntervals.get(i).toString() + "\n";
		}
		return str;
	}	
}
