package edu.ucla.cs.db.prism.backend.util;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Vector;

public class PrimaExperimentResult {
	//public String exp_id; // generated as query_id + timestamp in millisecond
	public Date exp_timestamp;
	public String dataset_id;
	public String query_id;
	public int target_version;
	public Vector<Integer> source_version_set;
	public boolean optimize_source_version_prune;
	public boolean optimize_union_of_join;
	public boolean optimize_smo_prune;
	public boolean optimize_smo_compress;

	public boolean is_empty_rewriting;
	public String rewritten_query_text;
	public int num_effective_smos;
	public long prima_input_parse_time; // millisecond
	public long prima_preprocessing_time; // millisecond
	public long prima_postprocessing_time; // millisecond
	public long prima_total_time; // millisecond
	public long mars_input_parse_time; // millisecond
	public long mars_semex_time; // millisecond
	public long mars_chase_time; // millisecond
	public long mars_total_time; // millisecond
	public long mars_result_parse_time; // millisecond
	public long optimize_source_version_prune_cost_time; // millisecond
	public long optimize_union_of_join_cost_time; // millisecond
	public long optimize_smo_prune_cost_time; // millisecond
	public long optimize_smo_compress_cost_time; // millisecond
	
	
	// converter
	DateFormat dateFormat;
	
	public PrimaExperimentResult() {
		// MySQL wants 'YYYY-MM-DD HH:MM:SS

	}
	public String toStringHumanFriendly() {
		String str = "";

		// input
		dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
		String exp_id = query_id + "_" + dateFormat.format(exp_timestamp);
		str += "exp_id: " + exp_id + "\n";
		dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		str += "exp_timestamp: " + dateFormat.format(exp_timestamp) + "\n";
		str += "dataset_id: " + dataset_id + "\n";
		str += "query_id: " + query_id + "\n";
		str += "target_version: " + target_version + "\n";
		str += "source_versions: " + source_version_set.toString() + "\n";
		str += "optimize_source_version_prune: " + optimize_source_version_prune + "\n";
		str += "optimize_union_of_join: " + optimize_union_of_join + "\n";
		str += "optimize_smo_prune: " + optimize_smo_prune + "\n";
		str += "optimize_smo_compress: " + optimize_smo_compress + "\n";

		// output
		str += "Is rewritten query empty: " + is_empty_rewriting + "\n";
		str += "rewritten_query_text: \n" + rewritten_query_text + "\n";
		str += "Number of effective SMOs: " + num_effective_smos + "\n";
		str += "PRIMA input parse time: " + prima_input_parse_time + " ms\n";
		str += "PRIMA preprocessing time: " + prima_preprocessing_time + " ms\n";
		str += "PRIMA postprocessing time: " + prima_postprocessing_time + " ms\n";
		str += "PRIMA total time: " + prima_total_time + " ms\n";
		str += "MARS input parse time: " + mars_input_parse_time + " ms\n";
		str += "MARS semantic + execution time: " + mars_semex_time + " ms\n";
		str += "MARS chase time: " + mars_chase_time + " ms\n";
		str += "MARS total time: " + mars_total_time + " ms\n";
		str += "MARS result parse time: " + mars_result_parse_time + " ms\n";
		str += "Source version prune optimization cost: " + optimize_source_version_prune_cost_time + " ms\n";
		str += "Union-of-Join optimization Cost: " + optimize_union_of_join_cost_time + " ms\n";
		str += "SMO prune optimization cost: " + optimize_smo_prune_cost_time + " ms\n";
		str += "SMO compress optimization cost: " + optimize_smo_compress_cost_time + " ms\n";
		return str;
	}
	
	public String toString() {
		return toStringDBFriendly();
	}

	public String toStringDBFriendly() {
		String str = "";
		
		// experiment setup
		str += "INSERT INTO rewriting_performance VALUE (";
		dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
		String exp_id = query_id + "_" + dateFormat.format(exp_timestamp);
		str += "'" + exp_id + "', ";
		dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		str += "'" + dateFormat.format(exp_timestamp) + "', ";
		str += "'" + dataset_id + "', ";
		str += "'" + query_id + "', ";
		str += target_version + ", ";
		//str += source_version_set + ", ";
		if(optimize_source_version_prune) str += "1, ";
		else str += "0, ";
		if(optimize_union_of_join) str += "1, ";
		else str += "0, ";
		if(optimize_smo_prune) str += "1, ";
		else str += "0, ";
		if(optimize_smo_compress) str += "1, ";
		else str += "0, ";
		
		// experiment results
		//str += escapeTextMySQL(rewritten_query_text) + ", ";

		if(is_empty_rewriting) str += "1, ";
		else str += "0, ";
		str += "'" + escapeQuote(rewritten_query_text) +  "', ";
		str += num_effective_smos + ", ";
		str += prima_input_parse_time + ", ";
		str += prima_preprocessing_time + ", ";
		str += prima_postprocessing_time + ", ";
		str += prima_total_time + ", ";
		str += mars_input_parse_time + ", ";
		str += mars_semex_time + ", ";
		str += mars_chase_time + ", ";
		str += mars_total_time + ", ";
		str += mars_result_parse_time + ", ";
		str += optimize_source_version_prune_cost_time + ", ";
		str += optimize_union_of_join_cost_time + ", ";
		str += optimize_smo_prune_cost_time + ", ";
		str += optimize_smo_compress_cost_time;
		str += ");\n";
		
		if(source_version_set == null) return str;
		
		
		// source_version_set table
		for(int i=0; i<source_version_set.size(); i++) {
			String str2 = "INSERT INTO source_version_set VALUE (";
			str2 += "'" + exp_id + "', ";
			str2 += source_version_set.get(i);
			str2 += ");\n";
			str += str2;
		}
		return str;
	}
	
	String escapeQuote(String input) {
		if(input == null) return null;
		String str = "";
		for(int i=0; i<input.length(); i++) {
			if(input.substring(i, i+1).equals("'"))
				str += "\\";
			str += input.substring(i, i+1);
		}
		return str;
	}
}
