package edu.ucla.cs.db.prism.backend.util;

import edu.ucla.cs.db.prism.backend.SQL.*;

import java.util.*;

public class REFConstant {
	// 3/7/2007 - to make all REF unique, we use the following format
	// _ref3_v46m2_FUNCNAME
	
	private final static String relationEncodedFunctionPrefix = "_ref";
	private final static String prefixSep = "_";

	
	public static String getRelationEncodedFunctionPrefix() {
		return relationEncodedFunctionPrefix;
	}
	
	/**
	 * given 5, func, return _ref5_v46m2_func
	 * @param numFunctionArgs
	 * @param functionName
	 * @return
	 */
	public static String getREF(int numFunctionArgs, int targetVersion, int targetVersionMinor, String functionName) {
		return getFullRelationEncodedFunctionPrefix(numFunctionArgs, targetVersion, targetVersionMinor) + functionName;
	}

	/**
	 * given 5, 46, 2, return _ref5_v46m2_
	 * @param numFunctionArgs
	 * @return
	 */
	public static String getFullRelationEncodedFunctionPrefix(int numFunctionArgs, int targetVersion, int targetVersionMinor) {
		return relationEncodedFunctionPrefix + numFunctionArgs + prefixSep +
		"v" + targetVersion + "m" + targetVersionMinor + prefixSep;
	}
	
	/**
	 * given _ref5_v46m2_func, return 5
	 * @param refRelation
	 * @return
	 */
	public static int getNumOfArgs(String refRelation) {
		if(refRelation == null) 
			return -1;
		if(!refRelation.startsWith(relationEncodedFunctionPrefix))
			return -1;
		int refPrefixLen = relationEncodedFunctionPrefix.length();
		
		// substr = refRelation - "_ref" (prefix stripped)
		String refRelationPrefixStripped = refRelation.substring(refPrefixLen);
		int prefixSepIndex = refRelationPrefixStripped.indexOf(prefixSep);
		if(prefixSepIndex <= 0)
			return -1;
		
		String argNumStr = refRelationPrefixStripped.substring(0, prefixSepIndex);
		int argNum = Integer.parseInt(argNumStr);
		return argNum;
	}
	
	/**
	 * given _ref5_v46m2_func, return func
	 * @param refRelation
	 * @return 
	 */
	public static String getFunctionName(String refRelation) {
		int refPrefixLen = relationEncodedFunctionPrefix.length();
		String substr = refRelation.substring(refPrefixLen); // strip "_ref"
		int prefixSepIndex = substr.indexOf(prefixSep);
		substr = substr.substring(prefixSepIndex+1); // strip "5_";
		prefixSepIndex = substr.indexOf(prefixSep);
		substr = substr.substring(prefixSepIndex+1); // strip "v46m2_"
		return substr;
	}
	
	public static boolean hasREFPrefix(String relationName) {
		if(relationName == null) 
			return false;
		if(relationName.startsWith(relationEncodedFunctionPrefix))
			return true;
		else
			return false;
	}
	
	public static boolean isInequalityREF(String refPrefixedRelationName) {
		if(!refPrefixedRelationName.startsWith(relationEncodedFunctionPrefix + "2"))
			return false;

		if(refPrefixedRelationName.endsWith("less") ||
				refPrefixedRelationName.endsWith("leq") ||
				refPrefixedRelationName.endsWith("neq") )
			return true;
		else
			return false;
				
	}
	
	static public Vector<String> getREFRelationNames(SQLUnionSet inputSQLUnionSet) {
		Vector<String> refRelations = new Vector<String>();
		if(inputSQLUnionSet!=null)
		for(int i=0; i<inputSQLUnionSet.sqlUnions.size(); i++) {
			SQLUnion sqlUnion = inputSQLUnionSet.sqlUnions.get(i);
			for(int j=0; j<sqlUnion.sqls.size(); j++) {
				SQL sql = sqlUnion.sqls.get(j);
				// each relation declaration
				for(int k=0; k<sql.from.relationDeclarationsSize(); k++) {
					String relationName = sql.from.relationDeclarations.get(k).relation;
					if(REFConstant.hasREFPrefix(relationName) &&
							//!REFConstant.isInequalityREF(relationName) &&
							!refRelations.contains(relationName))
						refRelations.add(relationName);
				}
			}
		}
		return refRelations;
	}
	
}
