package edu.ucla.cs.db.prism.backend.xquery;
import java.io.Serializable;

import edu.ucla.cs.db.prism.backend.temporalOptimize.RV;

public class VarInfoUnit implements Serializable {
	// This is a class used for XQuery semantic analysis
	static final public long serialVersionUID = 0;
	
	public String var;
	public String nodeName; // e.g. emp_acct, ts, te, ...
	public int level; // document: 0, next: 1, next: 2, ...
	public String parVar; // our input paths all in the form of "$x/node", so it has a parent
	public boolean isAttr;
	public RV rsv = null; // required source version - we use it for row level (level 3) only
	
	public VarInfoUnit(String var, String nodeName, int level, String parVar) {
		this.var = var;
		this.nodeName = nodeName;
		this.level = level;
		this.parVar = parVar;
		this.isAttr = false; // element
	}

	public VarInfoUnit(String var, String nodeName, int level, String parVar, boolean isAttr) {
		this.var = var;
		this.nodeName = nodeName;
		this.level = level;
		this.parVar = parVar;
		this.isAttr = isAttr;
	}

	public VarInfoUnit(VarInfoUnit src) {
		this.var = src.var;
		this.nodeName = src.nodeName;
		this.level = src.level;
		this.parVar = src.parVar;
		this.isAttr = src.isAttr;
		this.rsv = new RV(src.rsv);
	}
	
	@Override
	public VarInfoUnit clone() {
		return new VarInfoUnit(this);
	}
	
	@Override
	public String toString() {
		String str = "";
		str += var;
		str += "(" + nodeName + ", ";
		str += "L" + level + ", ";
		if(isAttr)
			str += "attr, ";
		else
			str += "elem, ";
		str += parVar + ")";

		return str;
	}
}
