package edu.ucla.cs.db.prism.backend.xquery;

import java.io.ByteArrayInputStream;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.parser.parser.PrimaRParser;
import edu.ucla.cs.db.prism.backend.restructure.XQueryRestructure;
import edu.ucla.cs.db.prism.backend.parser.syntaxtree.xquery;
import edu.ucla.cs.db.prism.backend.SQL.SQLUnion;
import edu.ucla.cs.db.prism.backend.SQL.SQLUnionSet;
import edu.ucla.cs.db.prism.backend.smo.util.Disjunctive;
import edu.ucla.cs.db.prism.backend.smo.util.Predicate;
import edu.ucla.cs.db.prism.backend.temporalOptimize.RVSet;

public class XQuery {
	public String name; // name of XQuery - valid for the outermost xquery
	public int type;
	   /**
	    * f0 -> type 0:   path()
	    *       type 1: | <LESS> <ID> <GREAT> <LBRACE> xquery() ( <COMMA> xquery() )* <RBRACE> <LESS> <DIV> <ID> <GREAT>
	    *       type 2: | one_for_clause() ( <COMMA> one_for_clause() )* ( <WHERE> expression() )? <RETURN> xquery()
	    */
		// type 3: xquery() ( <UNION> xquery() )*
	
	public String path = null; // used for type 0

	public String containingElementName = null; // used for type 1
	public Vector<XQuery> containedXQueries = null; // used for type 2

	// used for type 2
	public Vector<ForClause> forClauses = null;
	public Disjunctive whereClause = null;
	// let clause might be needed
	public XQuery returnClause = null;
	public SQLUnion xbind = null; // this is computed from forclause and whereclause
	//public Vector<String> subqueryRelations = null; // these relations are extracted and redefined by subqueries 
	public Vector<XQuery> relationReplacingSubqueries = null;
	// newly introduced in v0.6
	public SQLUnionSet relationReplacingXBindQueries = null;
	
	// used for type 3
	public Vector<XQuery> unionedXQueries;

	
	// semantic
	public VarInfo varInfo; // stores each variale's information, obtained from analysis
							// it also has varInfo of the containing XQuery
	public RVSet rsvSet; // stores each variable's required source versions
	
	
	public XQuery() {
	}

	public XQuery(String xqueryText) {
		// parse xqueryText
		PrimaRParser parser = new PrimaRParser( new ByteArrayInputStream(xqueryText.getBytes()) );
		xquery newxq = null;
		try {
			newxq = parser.xquery();
		}
		catch(Exception e) {
			e.printStackTrace();
		}
		XQuery parsedXQuery = XQueryRestructure.restructure(newxq);
		
		// set this element
		this.type = parsedXQuery.type;
		if(type == 0) {
			this.path = parsedXQuery.path;
		}
		else if(type == 1) {
			this.containingElementName = parsedXQuery.containingElementName;
			this.containedXQueries = new Vector<XQuery>();
			for(int i=0; i<parsedXQuery.containedXQueries.size(); i++) {
				this.containedXQueries.add(parsedXQuery.containedXQueries.get(i).clone());
			}
		}
		else if(type == 2) {
			this.forClauses = new Vector<ForClause>();
			for(int i=0; i<parsedXQuery.forClauses.size(); i++) {
				this.forClauses.add(parsedXQuery.forClauses.get(i).clone());
			}
			if(parsedXQuery.whereClause != null)
				this.whereClause = parsedXQuery.whereClause.clone();
			if(parsedXQuery.xbind != null)
				this.xbind = parsedXQuery.xbind.clone();
			this.returnClause = parsedXQuery.returnClause.clone();
		}
		else if(type == 3) {
			this.unionedXQueries = new Vector<XQuery>();
			for(int i=0; i<parsedXQuery.unionedXQueries.size(); i++) {
				this.unionedXQueries.add(parsedXQuery.unionedXQueries.get(i).clone());
			}
		}
	}
	
	public XQuery(int type) {
		this.type = type;
	}

	@Override
	public XQuery clone() {
		XQuery newXQuery = new XQuery();
		newXQuery.type = this.type;
		if(type == 0) {
			newXQuery.path = this.path;
		}
		else if(type == 1) {
			newXQuery.containingElementName = this.containingElementName;
			newXQuery.containedXQueries = new Vector<XQuery>();
			for(int i=0; i<containedXQueries.size(); i++) {
				newXQuery.containedXQueries.add(this.containedXQueries.get(i).clone());
			}
		}
		else if(type == 2) {
			newXQuery.forClauses = new Vector<ForClause>();
			for(int i=0; i<forClauses.size(); i++) {
				newXQuery.forClauses.add(this.forClauses.get(i).clone());
			}
			newXQuery.whereClause = this.whereClause.clone();
			newXQuery.xbind = this.xbind.clone();
			newXQuery.returnClause = this.returnClause.clone();
		}
		else if(type == 3) {
			newXQuery.unionedXQueries = new Vector<XQuery>();
			for(int i=0; i<unionedXQueries.size(); i++) {
				newXQuery.unionedXQueries.add(this.unionedXQueries.get(i).clone());
			}
		}
		return newXQuery;
	}
	
	@Override
	public String toString() {
		return toString(0, false);
	}
	
	public String toString(int indentLevel) {
		return toString(indentLevel, false);
	}
	
	public String toString(int indentLevel, boolean debugging) {
		String str = "";
		
		// semantic info output
		if(varInfo != null)
			str += varInfo.toString();
		if(rsvSet != null)
			str += rsvSet.toString();
		
		
		if(type == 0) {
			//str = "{" + path + "}";
			//str = indent(indentLevel) + path;
			str = path;
		}
		else if(type == 1) {
			str += indent(indentLevel) + "element " + containingElementName + " {";
			// case 1: simple printing for simple path
			if(containedXQueries.size() == 1 &&
					containedXQueries.get(0).type == 0) {
				str += containedXQueries.get(0).toString();
				str += "}";
			}
			// case 2: general
			else {
				str += "\n";
				for(int i=0; i<containedXQueries.size(); i++) {
					if(containedXQueries.get(i).type == 0)
						str += indent(indentLevel+1) + containedXQueries.get(i).toString();
					else
						str += containedXQueries.get(i).toString(indentLevel+1);
					if(i<containedXQueries.size()-1)
						str += ",";
					str += "\n";
				}
				str += indent(indentLevel) + "}";
			}
		}
		else if(type == 2){
			if(forClauses.size() > 0)
				//str += "\n" + indent(indentLevel) + "for  ";
				str += indent(indentLevel) + "for  ";
			for(int i=0; i<forClauses.size(); i++) {
				
				if(i > 0)
					str += "     " + indent(indentLevel);
				str += forClauses.get(i).toString(indentLevel+1);
				if(i<forClauses.size()-1)
					str += ",";
				str += "\n";
			}
			if(debugging) {
				str += "XBINDQUERY BEGIN\n";
				if(relationReplacingXBindQueries != null)
					str += relationReplacingXBindQueries.toString();
				str += "XBINDQUERY END\n";
				
				str += "SUBQUERY BEGIN\n";
				if(relationReplacingSubqueries != null)
					for(int i=0; i<relationReplacingSubqueries.size(); i++) {
						str += relationReplacingSubqueries.get(i).toString(indentLevel+1);
					}
				str += "SUBQUERY END\n";
			}
			if(whereClause != null && whereClause.conjunctives != null &&
					whereClause.conjunctives.size() > 0 &&
					whereClause.conjunctives.get(0).size() > 0)
				str += indent(indentLevel) + "where  " + whereClause.toString() + "\n";
			//if(xbind != null) str += xbind.toStringDebugging() + "\n"; // for debugging, comment out this line later
			if(returnClause != null)
				str += indent(indentLevel) + "return\n" + returnClause.toString(indentLevel+1, debugging);// + "\n";
		}
		else if(type == 3){
			for(int i=0; i<unionedXQueries.size(); i++) {
				str += "(" + unionedXQueries.get(i).toString(indentLevel+1) + ")";
				if(i<unionedXQueries.size()-1)
					str += "\n" + indent(indentLevel) + "union\n";
			}
		}
		
		return str;
	}
	
	public String toStringDebugging() {
		return toString(0, true);
	}
	
	public static String indent(int n) {
		String str = "";
		for(int i=0; i<n; i++)
			str += "  ";
		return str;
	}
	
	@Override
	public boolean equals(Object o) {
		XQuery xq2 = (XQuery)o;
		
		if(this.type != xq2.type)
			return false;
		else if(this.type == 0) {
			return this.path.equals(xq2.path);
		}
		else if(this.type == 1) {
			return this.containingElementName.equals(xq2.containingElementName) &&
					this.containedXQueries.equals(xq2.containedXQueries);
		}
		else if(this.type == 2) {
			if(!this.forClauses.equals(xq2.forClauses))
				return false;
			else if(this.whereClause == null && xq2.whereClause != null)
				return false;
			else if(!this.whereClause.equals(xq2.whereClause)) // TODO: this is not well-defined
				return false;
			else if(!this.returnClause.returnClause.equals(xq2.returnClause))
				return false;
			else
				return true;
		}
		else if(this.type == 3) {
			return this.unionedXQueries.equals(xq2.unionedXQueries);
		}
		else
			return false; // shouldn't reach here
	}
	
	
	public boolean readsVariable(String var) {
		// test if xquery reads var
		if(type == 0) {
			return testIfVariableUsedInXPath(path, var);
		}
		else if(type == 1) {
			for(int i=0; i<containedXQueries.size(); i++) {
				if(containedXQueries.get(i).readsVariable(var))
					return true;
			}
			return false;
		}
		else if(type == 2) {
			// case 1: check for-clause
			for(int i=0; i<forClauses.size(); i++) {
				ForClause fc = forClauses.get(i);
				if(fc.valueSeq.type == 0 && fc.valueSeq.path.startsWith(var))
					return true;
			}
			
			// case 2: check where-clause
			for(int i=0; i<whereClause.conjunctives.get(0).predicates.size(); i++) {
				Predicate p = whereClause.conjunctives.get(0).predicates.get(i);
				if(testIfVariableUsedInXPath(p.term1.constant, var))
					return true;
				else if(testIfVariableUsedInXPath(p.term2.constant, var))
					return true;
			}
			
			// case 3: return clause
			return returnClause.readsVariable(var);
		}
		else if(type == 3) {
			for(int i=0; i<unionedXQueries.size(); i++) {
				if(unionedXQueries.get(i).readsVariable(var))
					return true;
			}
			return false;
		}
		else
			return false;

		
	}
	
	private boolean testIfVariableUsedInXPath(String xpath, String var) {
		int index = -1;
		while(true) {
			index = xpath.indexOf(var, index+1);
			if(index<0)
				break; // var not found
			else {
				// see if this is really using $col2, (as in $col2/@ts), not like $col23 
				int varLen = var.length();
				if(index+varLen == xpath.length()) // var found and no more following chars
					return true;
				char firstCharFollowingVar = xpath.charAt(index + varLen);
				if(!Character.isDigit(firstCharFollowingVar)) // var found and digit doesn't follow immediately
					return true;
			}
		}
		
		return false;
	}
}
