package edu.ucla.cs.db.prism.backend.xqueryxbind;

import java.util.Vector;

import edu.ucla.cs.db.prism.backend.SQL.RelationDeclaration;
import edu.ucla.cs.db.prism.backend.SQL.SQL;
import edu.ucla.cs.db.prism.backend.SQL.SQLUnion;
import edu.ucla.cs.db.prism.backend.SQL.SelectAttribute;
import edu.ucla.cs.db.prism.backend.smo.util.Conjunctive;
import edu.ucla.cs.db.prism.backend.smo.util.Predicate;
import edu.ucla.cs.db.prism.backend.smo.util.Term;
import edu.ucla.cs.db.prism.backend.util.Counter;
import edu.ucla.cs.db.prism.backend.xquery.ForClause;
import edu.ucla.cs.db.prism.backend.xquery.XPath;
import edu.ucla.cs.db.prism.backend.xquery.XQuery;

public class XQueryToXBind {
	static final String relationAliasPrefix = "R";

	
	public static void xqueryToXBind(XQuery xquery) {
		xqueryToXBind(xquery, null, new Counter(0));
	}
	
	// for FLWR type of XQuery, turn FLW (actually FW) into an XBind
	// and attach it to xquery's xbind field
	// recursive
	private static void xqueryToXBind(XQuery xquery, SQLUnion xbindInOuterXQuery, Counter xbindIndexCounter) {
		// path()
		if(xquery.type == 0) 
			return;
		// <LESS> <ID> <GREAT> <LBRACE> xquery() ( <COMMA> xquery() )* <RBRACE> <LESS> <DIV> <ID> <GREAT>
		else if(xquery.type == 1) {
			for(int i=0; i<xquery.containedXQueries.size(); i++) {
				xqueryToXBind(xquery.containedXQueries.get(i), xbindInOuterXQuery, xbindIndexCounter);
			}
		}
		// type 2: | one_for_clause() ( <COMMA> one_for_clause() )* ( <WHERE> expression() )? <RETURN> xquery()
		else if(xquery.type == 2) {
			xquery.xbind = xqueryNavigationPartToXBind(xquery, xbindInOuterXQuery, xbindIndexCounter);
			xqueryToXBind(xquery.returnClause, xquery.xbind, xbindIndexCounter);
		}
		// type 3: xquery() ( <UNION> xquery() )*
		else if(xquery.type == 3) {
			for(int i=0; i<xquery.unionedXQueries.size(); i++) {
				xqueryToXBind(xquery.unionedXQueries.get(i), xbindInOuterXQuery, xbindIndexCounter);
			}
		}
		else // no case falls here now
			return;
	}
	
	
	
	// turns navigational part (see MARS paper), or FLW among FLWR, into XBind
	private static SQLUnion xqueryNavigationPartToXBind(XQuery xquery, 
			SQLUnion xbindInOuterXQuery, Counter xbindIndexCounter) {

		SQL xbind = new SQL();
		xbind.where.add(new Conjunctive()); // default conjunctive
		Counter relationAliasIndexCounter = new Counter(0);
		
		// step 1: outer xquery's xbind heads are put into this xbind's body
		if(xbindInOuterXQuery != null) {
			// step 1.1: add to from-clause
			String relationAlias = relationAliasPrefix + relationAliasIndexCounter.getValue();
			//RelationDeclaration rd = new RelationDeclaration(
			//		xbindInOuterXQuery.name, relationAlias);
			RelationDeclaration rd = new RelationDeclaration(
					"xb" + xbindInOuterXQuery.sqls.get(0).from.relationDeclarationsSize(), relationAlias);
			xbind.from.relationDeclarations.add(rd);

			// step 1.2: add to select-clause
			Vector<SelectAttribute> sas = getSelectAttributeFromOuterXBind(
					xbindInOuterXQuery, relationAlias);
			xbind.select.selectAttributes.addAll(sas);
		}

		
		// step 2: newly defined variables by for-clauses
		for(int i=0; i<xquery.forClauses.size(); i++) {
			ForClause forClause = xquery.forClauses.get(i);
			if(forClause.valueSeq.type != 0) // not path
				return null;
			String path = forClause.valueSeq.path;
			
			String[] parentAliasAttribute = null;
			
			// temporary fix: document("test.xml")/db -> temporarily ignore and use it as /db
			if(path.startsWith("document(")) {
				String[] firstAndRest = XPath.extractFirstSimplePath(path);
				path = "/" + firstAndRest[1];
			}
			
			
			// case 1: handle absolute path first here
			if(XPath.isAboslutePath(path)) {
				path = path.substring(1); // we don't allow descendant axis (i.e. //element)
				// root - remember this variable to use it as a parent below
				parentAliasAttribute = addRootAccess(xbind, relationAliasIndexCounter);
			}
			
			// case 2: handle variable access in var/element
			else if(XPath.startsAtVariable(path)) {
				String[] firstAndRest = XPath.extractFirstSimplePath(path);
				String var = firstAndRest[0];
				path = firstAndRest[1];
				parentAliasAttribute = getParentAliasAttributeByVar(xbind, var.substring(1)); // eliminate prefix "$"
				if(parentAliasAttribute == null) {
					System.err.println("Trying to use undefined XQuery variable");
					System.exit(-1);
				}
			}
			
			
			while(path != null && path.length() > 0) {
				//System.out.println(path);
				String[] firstAndRest = XPath.extractFirstSimplePath(path);
				String first = firstAndRest[0];
				// level 2 or below - use a parent variable and set it equal to self
				
				if(XPath.isAttributeNode(first))
					parentAliasAttribute = 
						addAttributeAccess(xbind, relationAliasIndexCounter,
								parentAliasAttribute[0], parentAliasAttribute[1],
								first.substring(1));
				else if(XPath.isTextNode(first))
					parentAliasAttribute =
						addTextAccess(xbind, relationAliasIndexCounter,
								parentAliasAttribute[0], parentAliasAttribute[1]);
				else // element node
					parentAliasAttribute =
						addElementAccess(xbind, relationAliasIndexCounter,
								parentAliasAttribute[0], parentAliasAttribute[1],
								first);
				
				path = firstAndRest[1];
			}
			
			// add select for each for-clause
			addSelectAttribute(xbind, forClause.getVarWithoutPrefix(), 
					parentAliasAttribute[0], parentAliasAttribute[1]); // using $ now
		}
		
		// step 3: where clause - assume a single conjunctive in XQuery's where
		if(xquery.whereClause != null) {
			Conjunctive xqueryWhereClauseCloned = xquery.whereClause.conjunctives.get(0).clone();
			replaceVarAccess(xqueryWhereClauseCloned, xbind);
			xbind.where.conjunctives.get(0).addAll(xqueryWhereClauseCloned);
		}
		
		// step 4: give a name and encapsulate into SQLUnion
		SQLUnion xbindUnion = new SQLUnion();
		xbindUnion.name = "xb" + xbindIndexCounter.getValue();
		xbindUnion.sqls.add(xbind);
		return xbindUnion;
	}
	

	// used in xqueryNavigationPartToXBind()
	private static Vector<SelectAttribute> getSelectAttributeFromOuterXBind(SQLUnion xbindUnion, String relationAlias) {
		Vector<SelectAttribute> sas = new Vector<SelectAttribute>();
		//SQLUnion is assumed to contain a single SQL here for xbind
		SQL xbind = xbindUnion.sqls.get(0);
		
		for(int i=0; i<xbind.select.selectAttributesSize(); i++) {
			SelectAttribute sa = new SelectAttribute(
					xbind.select.selectAttributes.get(i).outputAttributeName,
					relationAlias, "a" + i);
			sas.add(sa);
		}
		return sas;
	}
	
	
	private static String[] getParentAliasAttributeByVar(SQL xbind, String var) {
		// search select clause
		for(int i=0; i<xbind.select.selectAttributes.size(); i++) {
			if(xbind.select.selectAttributes.get(i).outputAttributeName.equals(var)) {
				String returnValues[] = {xbind.select.selectAttributes.get(i).relationAlias,
						xbind.select.selectAttributes.get(i).attribute};
				return returnValues;
			}
		}
		return null;
	}
	
	private static void replaceVarAccess(Conjunctive xqueryConjunctive, SQL xbind) {
		// if xqueryConjunctive uses xquery variable (e.g. $t), replace it using xbind's info
		for(int i=0; i<xqueryConjunctive.size(); i++) {
			replaceVarAccess(xqueryConjunctive.get(i).term1, xbind);
			replaceVarAccess(xqueryConjunctive.get(i).term2, xbind);
		}
	}

	private static void replaceVarAccess(Term xqueryTerm, SQL xbind) {
		// if xqueryConjunctive uses xquery variable (e.g. $t), replace it using xbind's info
		if(xqueryTerm.constant.startsWith("$")) { // xquery variable is in constant field now
			String[] parentAliasAttribute = getParentAliasAttributeByVar(xbind, xqueryTerm.constant);
			if(parentAliasAttribute == null) {
				System.err.println("Trying to use undefined XQuery variable");
				System.exit(-1);
			}
			xqueryTerm.constant = null;
			xqueryTerm.tableName = parentAliasAttribute[0];
			xqueryTerm.columnName = parentAliasAttribute[1];
		}
	
	}

	
	private static String[] addRootAccess(SQL xbind, Counter xbindRelationIndexCounter) {
		String relationAlias = 
			relationAliasPrefix + xbindRelationIndexCounter.getValue();
		RelationDeclaration rd = new RelationDeclaration("root", relationAlias);
		xbind.from.relationDeclarations.add(rd);
		String[] returnValue = {relationAlias, "a0"};
		return returnValue; // returns parentAlias and attribute for the next level (i.e. return_value, a0)
	}
	private static String[] addElementAccess(SQL xbind, Counter xbindRelationIndexCounter, 
			String parentAlias, String parentAttribute, String elementName) {
		String childRelationAlias = 
			relationAliasPrefix + xbindRelationIndexCounter.getValue();
		String tagRelationAlias = 
			relationAliasPrefix + xbindRelationIndexCounter.getValue();
		RelationDeclaration childRD = new RelationDeclaration("child", childRelationAlias);
		RelationDeclaration tagRD = new RelationDeclaration("tag", tagRelationAlias);
		xbind.from.relationDeclarations.add(childRD);
		xbind.from.relationDeclarations.add(tagRD);
		xbind.where.conjunctives.get(0).add(new Predicate(parentAlias, parentAttribute, "=",
				childRelationAlias, "a0"));
		xbind.where.conjunctives.get(0).add(new Predicate(childRelationAlias, "a1", "=",
				tagRelationAlias, "a0"));		
		xbind.where.conjunctives.get(0).add(new Predicate(tagRelationAlias, "a1", "=",
				"\"" + elementName + "\""));
		String[] returnValue = {childRelationAlias, "a1"};
		return returnValue; // returns parentAlias to the next level (i.e. return_value.a1)
	}
	private static String[] addAttributeAccess(SQL xbind, Counter xbindRelationIndexCounter,
			String parentAlias, String parentAttribute, String attributeName) {
		String relationAlias = 
			relationAliasPrefix + xbindRelationIndexCounter.getValue();
		RelationDeclaration rd = new RelationDeclaration("attr", relationAlias);
		xbind.from.relationDeclarations.add(rd);
		xbind.where.conjunctives.get(0).add(new Predicate(parentAlias, parentAttribute, "=",
				relationAlias, "a0"));
		xbind.where.conjunctives.get(0).add(new Predicate(relationAlias, "a1", "=",
				"\"" + attributeName + "\""));
		String[] returnValue = {relationAlias, "a2"};
		return returnValue; // returns attribute relation for accessing the value (i.e. return_value.a2)
	}
	private static String[] addTextAccess(SQL xbind, Counter xbindRelationIndexCounter,
			String parentAlias, String parentAttribute) {
		String relationAlias = 
			relationAliasPrefix + xbindRelationIndexCounter.getValue();
		RelationDeclaration rd = new RelationDeclaration("text", relationAlias);
		xbind.from.relationDeclarations.add(rd);
		xbind.where.conjunctives.get(0).add(new Predicate(parentAlias, parentAttribute, "=",
				relationAlias, "a0"));
		String[] returnValue = {relationAlias, "a1"};
		return returnValue; // returns text relation for accessing the value (i.e. return_value.a1) 
	}
	private static void addSelectAttribute(SQL xbind, String outputAttributeName,
			String relationAlias, String attribute) {
		xbind.select.selectAttributes.add(new SelectAttribute(outputAttributeName,
				relationAlias, attribute));
	}
}
