/*
 * Copyright 2007 Google Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package edu.ucla.cs.db.prism.gui.client;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Cookies;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.ServiceDefTarget;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;
import com.google.gwt.user.client.ui.HorizontalPanel;

/**
 * Demonstrates {@link com.google.gwt.user.client.ui.PopupPanel} and
 * {@link com.google.gwt.user.client.ui.DialogBox}.
 */
public class Deployment extends Sink implements ClickListener {

	
  public static SinkInfo init(ClientState cs) {
    return new SinkInfo(
        "Deployment",
        "<h2>Deployment</h2>"
            + "<p>In this phase the system will generate for you all the <tt>SQL</tt> scritps" +
            		" needed for a full deployment of the evolution solution." +
            		" Optimized views and migration scripts will be generate out of your specification</p>",cs) {

      @Override
	public Sink createInstance(ClientState cs) {
        return new Deployment(cs);
      }

      @Override
	public String getColor() {
        return "#bf2a2a";
      }
    };
  }

  private Button mysqlButton = new Button("MySQL Submit", this);
  private Button db2Button = new Button("DB2 Submit", this);
  private ClientState cs;
  private VerticalPanel panel = new VerticalPanel();
  private ScrollPanel viewScroller = null;
  private ScrollPanel dmScroller = null;
  //Sink.Images images;
  //private ListBox combo = new ListBox();
  //private ListBox list = new ListBox();

  public Deployment(ClientState cs) {
	this.cs = cs;

	
	VerticalPanel viewtitle = new VerticalPanel();
	viewtitle.add(new HTML("Views"));
	viewtitle.setStyleName("viewtitle");
    panel.add(viewtitle);
    
    viewScroller = new ScrollPanel();
    dmScroller = new ScrollPanel();

    viewScroller.setStyleName("dep_view");
    dmScroller.setStyleName("dep_dm"); 
    
    panel.add(viewScroller);
    
	VerticalPanel dmtitle = new VerticalPanel();
	dmtitle.add(new HTML("Data Migration SQL"));
	dmtitle.setStyleName("dmtitle");
    panel.add(dmtitle);
    panel.add(dmScroller);
   
    initWidget(panel);
  }

  public void onClick(Widget sender) {

  }
  
  /*
  public void onChange(Widget sender) {
		if (sender == combo) {
			fillList(combo.getSelectedIndex());
		} else if (sender == list) {
	    } 
  }
  */
  
  @Override
public void onShow() {
		String cookie = Cookies.getCookie("prismstate");
		if (cookie != null) {
			getViews(cookie, viewScroller);
			getDM(cookie, dmScroller);
			//getDMDB2(cookie,querysuccessrate);
		}
  }
  
  

  
  /**
  * Invoke the server to obtain the list of Views 
  * @param cookiekey
  * @param widget
  */
  
 
  protected void getViews(String cookiekey, final ScrollPanel widget) {
	
	  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT
				.create(PrimBackendService.class);
		ServiceDefTarget endpoint = (ServiceDefTarget) service;
		String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
		endpoint.setServiceEntryPoint(moduleRelativeURL);

		
		  HorizontalPanel h = new HorizontalPanel();
		  h.add(new Image("spin.gif"));
		  h.addStyleName("spinningbal");
		  widget.setWidget(h);
		
		AsyncCallback callback = new AsyncCallback() {

			public void onSuccess(Object outcome) {
				if (outcome == null)
					widget.setWidget(new HTML(
							"<b> No View in the set. </b> <br /> <br />"));
				else {
					
 					Widget temp = generateTextList((String[]) outcome);
 					//temp.setStyleName("dep_text");
					widget.setWidget(temp);
					
				}
			}

			public void onFailure(Throwable caught) {
				System.out.println(caught.toString());
				System.out.println("Deployement.getViews.onFailure:  ERROR");
			}
		};

		String tempc = Cookies.getCookie("prismstate");
		if (tempc != null)
			service.getViews(cookiekey, callback);
		else
			System.err
					.println("Deployement.getViews: NO COOKIE STATE ERROR");

	}
  /**
   * Invoke the server to obtain the list of data migration sqls 
   * @param cookiekey
   * @param widget
   */
   
  
   protected void getDM(String cookiekey, final ScrollPanel widget) {
 	
 	  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT
 				.create(PrimBackendService.class);
 		ServiceDefTarget endpoint = (ServiceDefTarget) service;
 		String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
 		endpoint.setServiceEntryPoint(moduleRelativeURL);

 	  HorizontalPanel h = new HorizontalPanel();
	  h.add(new Image("spin.gif"));
	  h.addStyleName("spinningbal");
	  widget.setWidget(h);
 		
 		AsyncCallback callback = new AsyncCallback() {

 			public void onSuccess(Object outcome) {
 				if (outcome == null)
 					widget.setWidget(new HTML(
 							"<b> No DM in the set. </b> <br /> <br />"));
 				else {
 					Widget temp = generateTextList((String[]) outcome);
 					//temp.setStyleName("dep_text");
 					widget.setWidget(temp);
 					
 				}
 			}

 			public void onFailure(Throwable caught) {
 				System.out.println(caught.toString());
 				System.out.println("Deployement.getDM.onFailure:  ERROR");
 			}
 		};

 		String tempc = Cookies.getCookie("prismstate");
 		if (tempc != null)
 			service.getDM(cookiekey, callback);
 		else
 			System.err
 					.println("Deployement.getDM: NO COOKIE STATE ERROR");

 	}

     /**
	 * Generates a TextArea representation of an array of strings
	 */

  public HTML generateTextList(String[] stringData) {
	  
		HTML ta = new HTML();
		//ta.setVisibleLines(12);
		//ta.isReadOnly();
		String temp="";
		int i=0;
		//System.out.println(stringData.length);
		while(stringData.length>i && stringData[i]!=null) {			
			//System.out.println(stringData[i]);
			temp = temp + "\n" + stringData[i];
			i++;
			
		}
		ta.setHTML(temp.replaceAll("\n", "<br>"));
		return ta;
	}


}
