/*
 * Copyright 2007 Google Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package edu.ucla.cs.db.prism.gui.client;

import java.util.Vector;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Cookies;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.ServiceDefTarget;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.ChangeListener;
import com.google.gwt.user.client.ui.CheckBox;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.DialogBox;
import com.google.gwt.user.client.ui.DockPanel;
import com.google.gwt.user.client.ui.FileUpload;
import com.google.gwt.user.client.ui.FormHandler;
import com.google.gwt.user.client.ui.FormPanel;
import com.google.gwt.user.client.ui.FormSubmitCompleteEvent;
import com.google.gwt.user.client.ui.FormSubmitEvent;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.KeyboardListener;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.MultiWordSuggestOracle;
import com.google.gwt.user.client.ui.PopupPanel;
import com.google.gwt.user.client.ui.PushButton;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.SuggestBox;
import com.google.gwt.user.client.ui.ToggleButton;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;


/**
 * The tab corresponding to the phase of designing SMOs. Assist the DBA in creating new SMOS, by signaling
 * error in syntax, mismatch with the current schema, information preservation, redundancy. It also shows the resulting schema.
 */
public class EvolutionDesign extends Sink implements ClickListener{

  public ClientState cs = null;
  public ClientSchema clientSchema = null;
  public ScrollPanel scroller = null;
  public HTML syntaxhtml = new HTML("check: syntax");
  public HorizontalPanel syntaxhp,infpreshp,redundhp,targetschemamatchhp  = null;
  public HTML infpreshtml = new HTML("check information <br> preservation");
  public HTML redudnacycheckhtml = new HTML("check redundancy");
  public HTML targetschemamatchhtml = new HTML("check match with target schema");
  
  public ListBox targetschemaversions = new ListBox();

  
  
  public VerticalPanel titledSMOpanel = new VerticalPanel();
  public static NewInverses invTab = null;
  public HTML currentSMOspanel = new HTML("No SMOs defined yet.");
  public ScrollPanel status = new ScrollPanel();
  HTML statushtml = new HTML();
 
  public static SinkInfo init(final Sink.Images images,ClientState cs) {
    return new SinkInfo( "Evolution-Design",
            "<h2>Schema Evolution Design</h2>" +
            "<p> You will be assist in the " +
            " process of defining the series of operations required to evolve your current DB " +
            "schema to the desired target one." +
            "The system will provide continuous feedback on right panels.</p>",cs) {

      public Sink createInstance(ClientState cs) {
        return new EvolutionDesign(images,cs);
      }
      

      public String getColor() {
        return "#bf2a2a";
      }
    };
  }
  
  

	
	
	public static SinkInfo init(Images images2, ClientState cs2, NewInverses i) {
		invTab = i;
		return init(images2,cs2);
	}
  

/*  private Button disabledButton = new Button("Disabled Button");
  private CheckBox disabledCheck = new CheckBox("Disabled Check");
  private Button normalButton = new Button("Normal Button");
  private CheckBox normalCheck = new CheckBox("Normal Check");
  
  private RadioButton radio0 = new RadioButton("group0", "Choice 0");
  private RadioButton radio1 = new RadioButton("group0", "Choice 1");
  private RadioButton radio2 = new RadioButton("group0", "Choice 2 (Disabled)");
  private RadioButton radio3 = new RadioButton("group0", "Choice 3");
  private PushButton pushButton;
  private ToggleButton toggleButton;
  */
  
  private VerticalPanel panel = new VerticalPanel();
  
  public static MultiWordSuggestOracle oracle = new MultiWordSuggestOracle();
  private static SuggestBox suggestBox = new SuggestBox(oracle);
  private static SuggestBox suggestBoxPopup = new SuggestBox(oracle);

  private Button showSchemaButton = new Button("Show Resulting Schema", this);
  private ToggleButton typeCheck = new ToggleButton(" Show Types");
  private HTML targetSchema = new HTML("wait...");
  private Button addsmoButton = new Button("Add SMO", this);
  private Sink.Images images= null;
  protected ClientOperatorSequence fwdSMOList = new ClientOperatorSequence();
  //int indexSmo = 0;
  private int popupedit = -1;
  private Image syntaximage = new Image();
  private Image infpresimage = new Image();
  private Image redundimage = new Image();
  private Image targetimage = new Image();
  private VerticalPanel checkers;
  public PopupPanel popupexportsmo = new PopupPanel(true);
  public HorizontalPanel targhp;
  public HorizontalPanel targhp2;
  
  public CheckBox targetchecktype = new CheckBox();
  
  public EvolutionDesign(final Sink.Images images, final ClientState cs) {
//    pushButton = new PushButton(images.gwtLogo().createImage());
//    toggleButton = new ToggleButton(images.gwtLogo().createImage());
	this.images = images; 

	
	fwdSMOList.smos = new Vector<ClientOperator>(); // initialize with an empty vector
	this.cs = cs;
    HorizontalPanel hp;
    VerticalPanel leftsideanel;
    
    panel.add(hp = new HorizontalPanel());
    panel.setHorizontalAlignment(VerticalPanel.ALIGN_LEFT);
    hp.setStyleName("overallsmopanel");

    VerticalPanel rightsidepanel;
    hp.add(leftsideanel = new VerticalPanel());
    hp.add(rightsidepanel = new VerticalPanel());
     
    rightsidepanel.setStyleName("rightsidepangel_evodesign");
    
    VerticalPanel tt = new VerticalPanel();
    tt.add(new HTML("Evolution Feedback"));
    tt.setStyleName("evotitle");
    rightsidepanel.add(tt);
    
    //----------------- SCHEMA REPRESENTATION
    
    HorizontalPanel schhp = new HorizontalPanel();
    leftsideanel.add(schhp);
    
    ScrollPanel schvp = new ScrollPanel();
    schvp.add(targetSchema);
    schvp.setStyleName("schemasmodesign");
    leftsideanel.add(schvp);
    
 //   schhp.add(showSchemaButton);
    schhp.setStyleName("smotitle");
    schhp.add(new HTML("Predicted Schema")); 
    schhp.add(typeCheck);
    
    //typeCheck.setStyleName("typebutton");
    
	typeCheck.addClickListener(new ClickListener(){
		
		public void onClick(Widget sender) {
		if(typeCheck.isDown())
			targetSchema.setHTML(clientSchema.toPopupString());
    	else
    		targetSchema.setHTML(clientSchema.toPopupStringNoTypes());
		}
	});
	
	showSchemaButton.addStyleName("showschemabutton");   
	
	
    
    //-------------- SMO INPUT AND HISTORY
    
	
	HorizontalPanel smoin = new HorizontalPanel();
    leftsideanel.add(smoin);
    
    smoin.setStyleName("smoinbox");
    smoin.add(new HTML("<b>Operator:</b>"));
    smoin.add(suggestBox);
    suggestBox.setStyleName("smoinput_green");
    suggestBoxPopup.setStyleName("smoinput_green");
    smoin.add(addsmoButton);
    addsmoButton.setStyleName("addsmobutton");

    HorizontalPanel titlesmopanel = new HorizontalPanel();
    HTML tit = new HTML("Working Operator set:");
    tit.setStyleName("workingsettitle");
    titlesmopanel.add(tit);
    
    titlesmopanel.addStyleName("smopaneltitle");
    leftsideanel.add(titlesmopanel);
    
    // ---------------------- lateral helpers 
    
    rightsidepanel.add(checkers = new VerticalPanel());

    
    //CARL:  STATUS is now substituted by upload file
    //VerticalPanel statusbox = new VerticalPanel();
    //VerticalPanel statustitle = new VerticalPanel();
    //statusbox.add(statustitle);
    //statusbox.add(status);
    
    //status.setStyleName("status");
    //statustitle.setStyleName("statustitle");
    
    //statustitle.add(new HTML("Connection Status"));
    
    //leftpanel.add(statusbox);
    
    
    
    //--------------- FILE UPLOAD ------------------
    
    
    // Create a FormPanel and point it at a service.
    final FormPanel uploadForm = new FormPanel();
    uploadForm.setAction(GWT.getModuleBaseURL() + "UploadFileServlet");

    
    final FormPanel exportForm = new FormPanel();
    exportForm.setAction(GWT.getModuleBaseURL() + "ExportFileServlet");

    
    // Because we're going to add a FileUpload widget, we'll need toset the
    // form to use the POST method, and multipart MIME encoding.
    uploadForm.setEncoding(FormPanel.ENCODING_MULTIPART);
    uploadForm.setMethod(FormPanel.METHOD_POST);

    // Create a panel to hold all of the form widgets.
    VerticalPanel uploadvpanel = new VerticalPanel();
    uploadForm.setWidget(uploadvpanel);

    // Create a FileUpload widget.
    FileUpload upload = new FileUpload();
    upload.setName("uploadFormElement");
    uploadvpanel.add(upload);

    // Add a 'submit' button.
    Button uploadSubmitButton = new Button("Submit");
    uploadSubmitButton.setStyleName("submitbutton");
    uploadvpanel.add(uploadSubmitButton);
    uploadvpanel.setCellHorizontalAlignment(uploadSubmitButton, HasHorizontalAlignment.ALIGN_RIGHT);

    uploadSubmitButton.addClickListener(new ClickListener() {
      public void onClick(Widget sender) {
        uploadForm.submit();
      }
    });

    
 // Add a 'export' button.
    Button exportButton = new Button("Show MO export");
    exportButton.setStyleName("smallsubmitbutton");
    titlesmopanel.add(exportButton);
    
    exportButton.addClickListener(new ClickListener() {
      public void onClick(Widget sender) {
    	 popupexportsmo.setStyleName("popupsmoexport");
    	 
    	 if(fwdSMOList!=null)
    		 popupexportsmo.setWidget(new HTML(fwdSMOList.toString()));
   	  	 else
   	  		 popupexportsmo.setWidget(new HTML("No Operator defined."));
   		 popupexportsmo.center();
   		 popupexportsmo.show();
    	  
      }
    });

    
    
    
    
    uploadForm.addFormHandler(new FormHandler() {
      public void onSubmit(FormSubmitEvent event) {
      }

      // the  file has been loaded and the content returned into event
      public void onSubmitComplete(FormSubmitCompleteEvent event) {
    	//  System.out.println("HTML:\n " + event.getResults());
    	 String filecontent = new HTML(event.getResults()).getText();
    	 String smotext[] = filecontent.split("\n");
		 addMultiSMO(smotext,scroller);
		 

     }

    }); 

    
    VerticalPanel uploadbox = new VerticalPanel();
    VerticalPanel uploadtitle = new VerticalPanel();
    uploadbox.add(uploadtitle);
    uploadbox.add(uploadForm);
    
    uploadForm.setStyleName("uploadform");
    uploadtitle.setStyleName("uploadtitle");
    
    uploadtitle.add(new HTML("Upload SMO File"));
    rightsidepanel.add(uploadbox);
    
     
     
     //---------- end of file upload mod -----------
     
    checkers.setStyleName("checkersbox");
    
    syntaxhp = new HorizontalPanel();
    images.oksign().applyTo(syntaximage);
    syntaxhp.add(syntaximage);
    syntaxhp.add(syntaxhtml);
    
    checkers.add(syntaxhp);
    syntaxhtml.setStyleName("smosyntaxbox_green");
    

    infpreshp  = new HorizontalPanel();
    images.oksign().applyTo(infpresimage);
    infpreshp.add(infpresimage);
    infpreshp.add(infpreshtml);
    checkers.add(infpreshp);
    infpreshtml.setStyleName("infpreshtml_green");  
    
    
    redundhp = new HorizontalPanel();
    images.oksign().applyTo(redundimage);
    redundhp.add(redundimage);
    redundhp.add(redudnacycheckhtml);
    checkers.add(redundhp);
    redudnacycheckhtml.setStyleName("redudnacycheckhtml_green");  
   	
    
    HTML targhtml = new HTML("Version to check:");
    targhtml.setStyleName("versioncheck");
    
    targhp = new HorizontalPanel();
    targhp.add(targhtml);
     
    
    targetschemaversions.addChangeListener(new ChangeListener() {
	     
		public void onChange(Widget sender) {
			  String tempc = Cookies.getCookie("prismstate");
			  compareCurrentSchemaWithReferenceTarget(tempc);
		}
	});
    
    
    targhp.add(targetschemaversions);
    
    targhp2 = new HorizontalPanel();
    
    HTML targhtml2 = new HTML("check types:");
    targhtml2.setStyleName("versioncheck");
    
    targhp2.add(targhtml2);
    targhp2.add(targetchecktype);
    
    
    checkers.add(targhp);
    checkers.add(targhp2);
    
    
    // creating the feedback box for target schema match
    targetschemamatchhp = new HorizontalPanel();
    targetchecktype.setEnabled(true);
   
    images.oksign().applyTo(targetimage);
    targetschemamatchhp.add(targetimage);
    targetschemamatchhp.add(targetschemamatchhtml);
    checkers.add(targetschemamatchhp);
    targetschemamatchhtml.setStyleName("targethtml_green");  
   
    
    /* SUGGEST BOX LISTENER */
    suggestBox.addKeyboardListener(new KeyboardListener() {
     
		public void onKeyDown(Widget sender, char keyCode, int modifiers) {
			
		}

		public void onKeyPress(Widget sender, char keyCode, int modifiers) {
			if(keyCode=='\r'){
				smoSyntaxCheck(suggestBox.getText(), syntaxhtml, suggestBox);
	    		addSMO(suggestBox.getText(),scroller);
	    		suggestBox.setStyleName("smosyntaxbox_green");
			}
		}

		public void onKeyUp(Widget sender, char keyCode, int modifiers) {
		
			if(keyCode =='\r')
				return;
			if(!suggestBox.getText().equals(""))
				smoSyntaxCheck(suggestBox.getText(), syntaxhtml,suggestBox);
			else{
				suggestBox.setStyleName("smoinput_green");
				syntaxhtml.setStyleName("smosyntaxbox_green");
				images.oksign().applyTo(syntaximage);
			}
				
		}
      });
       
    
    //SYNTAX HELPERS FOR THE POPUP VERSION OF THE EDIT BOX
    suggestBoxPopup.addKeyboardListener(new KeyboardListener() {
        
		public void onKeyDown(Widget sender, char keyCode, int modifiers) {
			
		}

		public void onKeyPress(Widget sender, char keyCode, int modifiers) {
			if(keyCode=='\r'){
				smoSyntaxCheck(suggestBoxPopup.getText(), syntaxhtml,suggestBoxPopup);
			}
		}

		public void onKeyUp(Widget sender, char keyCode, int modifiers) {
			smoSyntaxCheck(suggestBoxPopup.getText(), syntaxhtml,suggestBoxPopup);
		}
      });
    
    oracle.add("ADD COLUMN <col> <type> AS <const/funct> INTO <tab>");
    oracle.add("DROP COLUMN <col> FROM <tab>");
    oracle.add("CREATE TABLE <tab> (<col1> <type1>,<col2> <type2>)");
    oracle.add("DROP TABLE <tab>");
    oracle.add("RENAME TABLE <tab1> INTO <tab2>");
    oracle.add("RENAME COLUMN <col1> IN <tab1> TO <col2>");
    oracle.add("COPY TABLE <tab1> INTO <tab2>");
    oracle.add("JOIN TABLE <tab1>, <ta2> INTO <tab3> WHERE <cond>");
    oracle.add("DECOMPOSE TABLE <tab1> INTO <tab2(col1,col2)>, <tab3(col1,col3)>");
    oracle.add("PARTITION TABLE <tab1> INTO <tab2> WITH <cond1>, <tab3> WITH <cond2>");
    oracle.add("MERGE TABLE <tab1>, <tab2> INTO <tab3>");
    oracle.add("NOP");
    oracle.add("ALTER TABLE <tab> ADD PRIMARY KEY <id> (col1,col2) CHECK");
    oracle.add("ALTER TABLE <tab> ADD PRIMARY KEY <id> (col1,col2) ENFORCE");
    oracle.add("ALTER TABLE <tab> ADD PRIMARY KEY <id> (col1,col2) DECLARE");
    oracle.add("ALTER TABLE <tab> ADD FOREGN KEY <id> (col1,col2) REFERENCES <tab2>(col3,col4) CHECK");
    oracle.add("ALTER TABLE <tab> ADD FOREGN KEY <id> (col1,col2) REFERENCES <tab2>(col3,col4) ENFORCE");
    oracle.add("ALTER TABLE <tab> ADD FOREGN KEY <id> (col1,col2) REFERENCES <tab2>(col3,col4) DECLARE");
    oracle.add("ALTER TABLE <tab> ADD VALUE CONSTRAINT <id> <cond> CHECK");
    oracle.add("ALTER TABLE <tab> ADD VALUE CONSTRAINT <id> <cond> ENFORCE");
    oracle.add("ALTER TABLE <tab> ADD VALUE CONSTRAINT <id> <cond> DECLARE");
    oracle.add("ALTER TABLE <tab> DROP PRIMARY KEY <id>");
    oracle.add("ALTER TABLE <tab> DROP FOREIGN KEY <id>");
    oracle.add("ALTER TABLE <tab> DROP VALUE CONSTRAINT <id>");
    
    
    panel.add(hp = new HorizontalPanel());
    
    scroller = new ScrollPanel();
   	scroller.setStyleName("smopanelscroller"); // show existing SMOs
   	leftsideanel.add(scroller);
        
    // based on the current configuration retrieve SMO from the server
	//getSMO(temphtml);
   
    //use the scroller to store current SMOs
   	
	scroller.setWidget(currentSMOspanel);
	
    panel.add(hp = new HorizontalPanel());
    
    initWidget(panel);
    
    
  }

  /**
   * Ask the Server to retrieve the existing set of SMOs from the DB
   * @param dburltb
   * @param usertb
   * @param passwordtb
   * @param metaSchemaName
   * @param schemaName
   * @param temphtml
   */
  /*private void getHistoricalSMO(final HTML temphtml) {
		  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
		  ServiceDefTarget endpoint = (ServiceDefTarget) service;
		  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
		  endpoint.setServiceEntryPoint(moduleRelativeURL);
		  AsyncCallback callback = new AsyncCallback() {
			  
			public void onSuccess(Object css) {
				cs.smosequence = (ClientOperatorSequence) css;
			   temphtml.setHTML(((ClientOperatorSequence) css).toMetadataString());
		    }

		    public void onFailure(Throwable caught) {
		    	temphtml.setHTML("<h2> Error communicating with the server </h2>");
		    }
		  };

		  
		  String tempc = Cookies.getCookie("prismstate");
		  if(tempc != null)
			  service.getHistoricalSMO(tempc, callback);
		  else
			  System.err.println("SMODesign.getHistoricalSMO: NO COOKIE STATE ERROR");
	  
  }*/
/*
public MenuBar createMenu() {
    MenuBar menu = new MenuBar();
    menu.setAutoOpen(true);

    MenuBar subMenu = new MenuBar(true);
    subMenu.addItem("<code>Code</code>", true, this);
    subMenu.addItem("<strike>Strikethrough</strike>", true, this);
    subMenu.addItem("<u>Underlined</u>", true, this);

    MenuBar menu0 = new MenuBar(true);
    menu0.addItem("<b>Bold</b>", true, this);
    menu0.addItem("<i>Italicized</i>", true, this);
    menu0.addItem("More &#187;", true, subMenu);
    MenuBar menu1 = new MenuBar(true);
    menu1.addItem("<font color='#FF0000'><b>Apple</b></font>", true, this);
    menu1.addItem("<font color='#FFFF00'><b>Banana</b></font>", true, this);
    menu1.addItem("<font color='#FFFFFF'><b>Coconut</b></font>", true, this);
    menu1.addItem("<font color='#8B4513'><b>Donut</b></font>", true, this);
    MenuBar menu2 = new MenuBar(true);
    menu2.addItem("Bling", this);
    menu2.addItem("Ginormous", this);
    menu2.addItem("<code>w00t!</code>", true, this);

    menu.addItem(new MenuItem("Style", menu0));
    menu.addItem(new MenuItem("Fruit", menu1));
    menu.addItem(new MenuItem("Term", menu2));

    menu.setWidth("100%");

    return menu;
  }

  public void execute() {
    Window.alert("Thank you for selecting a menu item.");
  }*/

  public void onShow() {
	  //	Cookies.setCookie("prismstate",cs.toCookie());
		
	  	getState(status);
	  	getCurrentSMOs(Cookies.getCookie("prismstate"),scroller);
	  	
	  	if(cs.loadcompletehistory){
	  		//used to fill up the listbox for schema match
			targetschemamatchhp.setVisible(true);
	  		targhp.setVisible(true);
	  		targhp2.setVisible(true);
	  		getExistingSchemaHistory(Cookies.getCookie("prismstate"));
	  	}else{
	  		targetschemamatchhp.setVisible(false);
	  		targhp.setVisible(false);
	  		targhp2.setVisible(false);
	  	}
		scroller.setWidget(currentSMOspanel);
		
		if(fwdSMOList!=null)
			getSchema(fwdSMOList.smos.size());
		else
			getSchema(0);
		
  }
  
  private void getExistingSchemaHistory(String cookie){
	  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
	  ServiceDefTarget endpoint = (ServiceDefTarget) service;
	  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
	  endpoint.setServiceEntryPoint(moduleRelativeURL);
	
	  // set to wait the schema
	  //targetSchema.setHTML("computing...");
	  
	  AsyncCallback callback = new AsyncCallback() {
		  
		public void onSuccess(Object outcome) {
			if(outcome != null){
				targetschemaversions.clear();
				// use the clientSchema vector to extract valid versions.
				for(ClientSchema c:(Vector<ClientSchema>)outcome){
					targetschemaversions.addItem(Integer.toString(c.majorVersion),Integer.toString(c.majorVersion));
				}
				targetschemaversions.setItemSelected(targetschemaversions.getItemCount()-1, true);
				
			}else{
				System.err.println("FAILURE, outcome is null!");
			}	
		}


		public void onFailure(Throwable caught) {

			System.err.println("FAILURE: " + caught);

			// do some fail
		}
	  };

	  String tempc = Cookies.getCookie("prismstate");
	  if(tempc != null)
		  service.getExistingSchemaHistory(tempc, callback);
	  else
		  System.err.println(" NO COOKIE STATE ERROR");
	  
}

  
  private void compareCurrentSchemaWithReferenceTarget(String cookie){
	  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
	  ServiceDefTarget endpoint = (ServiceDefTarget) service;
	  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
	  endpoint.setServiceEntryPoint(moduleRelativeURL);
	  
		targetschemamatchhtml.setHTML("checking...");	
	  
	  // set to wait the schema
	  //targetSchema.setHTML("computing...");
	  
	  AsyncCallback callback = new AsyncCallback() {
		  
		public void onSuccess(Object outcome) {
			if(outcome != null){
				if(((String)outcome).startsWith("schema match")){
					targetschemamatchhtml.setStyleName("targethtml_green");  
					targetschemamatchhtml.setHTML((String) outcome);
				}else{
					targetschemamatchhtml.setStyleName("targethtml_red");  
					targetschemamatchhtml.setHTML((String) outcome);	
				}
			}	
		}


		public void onFailure(Throwable caught) {

			targetschemamatchhtml.setHTML("impossible to check with reference schema");	

		}
	  };

	  String tempc = Cookies.getCookie("prismstate");
	  if(tempc != null)
		  service.compareExistingSchema(tempc, Integer.parseInt(targetschemaversions.getValue(targetschemaversions.getSelectedIndex())), targetchecktype.isChecked(), callback);
	  else
		  System.err.println("NO COOKIE STATE ERROR");
	  
	  
	  
}

  




/**
   * Apply the current SMO to the most recent available schema. Update as a result  targetSchema popup.
   * @param smotext
   */
  private void applySMO(String smotext) {
	  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
	  ServiceDefTarget endpoint = (ServiceDefTarget) service;
	  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
	  endpoint.setServiceEntryPoint(moduleRelativeURL);
	  
	  // set to wait the schema
	  //targetSchema.setHTML("computing...");
	  
	  AsyncCallback callback = new AsyncCallback() {
		  
		public void onSuccess(Object outcome) {
			if(outcome == null){
				/* this should be the case in which the SMO has been added and we can't re-add it
				 * therefore we just retrieve the existing one
				 */
				if(fwdSMOList!=null)
					getSchema(fwdSMOList.smos.size());
				else
					getSchema(0);
			}
			else{
				clientSchema = (ClientSchema) outcome;
				if(typeCheck.isDown())
					targetSchema.setHTML(clientSchema.toPopupString());
				else
					targetSchema.setHTML(clientSchema.toPopupStringNoTypes());
				
			}	
			
			if(cs.loadcompletehistory){
			// refresh the comparison with following schemas
			  String tempc = Cookies.getCookie("prismstate");
			  compareCurrentSchemaWithReferenceTarget(tempc);
			}
		}


		public void onFailure(Throwable caught) {
			targetSchema.setHTML("connection error!");
	    }
	  };

	  String tempc = Cookies.getCookie("prismstate");
	  if(tempc != null)
		  service.applySMO(tempc, smotext, callback);
	  else
		  System.err.println("SMODesign.applySMO: NO COOKIE STATE ERROR");
	}
  
  
  /**
   * Apply the current SMO to the most recent available schema. Update as a result  targetSchema popup.
   * @param smotext
   */
  private void getSchema(int index) {
	  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
	  ServiceDefTarget endpoint = (ServiceDefTarget) service;
	  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
	  endpoint.setServiceEntryPoint(moduleRelativeURL);
	  
	  // set to wait the schema
	  //targetSchema.setHTML("retrieving current schema...");
	  
	  AsyncCallback callback = new AsyncCallback() {
		  
		public void onSuccess(Object outcome) {
			if(outcome == null){
				targetSchema.setHTML("No schema loaded, check your configuration in the previous tab.");
			}
			else{
				clientSchema = (ClientSchema) outcome;
				if(typeCheck.isDown())
					targetSchema.setHTML(clientSchema.toPopupString());
				else
					targetSchema.setHTML(clientSchema.toPopupStringNoTypes());
		
			}	
			
	
		
		}


		public void onFailure(Throwable caught) {
			targetSchema.setHTML("connection error!");
	    }
	  };

	  String tempc = Cookies.getCookie("prismstate");
	  if(tempc != null)
		  service.getCurrentSchema(tempc, index, callback);
	  else
		  System.err.println("SMODesign.applySMO: NO COOKIE STATE ERROR");
	}
  
  
  
  /**
   * Invoke the server to verify information preservation of the current SMO.
   * @param smotext
   */
  private void informationPreservationCheck(String smotext) {
	  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
	  ServiceDefTarget endpoint = (ServiceDefTarget) service;
	  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
	  endpoint.setServiceEntryPoint(moduleRelativeURL);
	  
	  System.out.println("checking inf pres.");
	  // set to wait the schema
	  infpreshtml.setHTML("checking...");
	  
	  AsyncCallback callback = new AsyncCallback() {
		  
		public void onSuccess(Object outcome) {
			if(outcome == null){
				infpreshtml.setHTML("ERROR: check your SMO syntax!");
				infpreshtml.setStyleName("infpreshtml_red");
	
			}
			else{
				
				String temp = outcome.toString();
				infpreshtml.setHTML(temp);
			
				if(temp.startsWith("ATTENTION:")){
					infpreshtml.setStyleName("infpreshtml_red");
					images.dangersign().applyTo(infpresimage);
				}	
				else{
					infpreshtml.setStyleName("infpreshtml_green");
					images.oksign().applyTo(infpresimage);	
				}
			}
			
			
		}


		public void onFailure(Throwable caught) {
			infpreshtml.setHTML("connection error!");
			images.stopsign().applyTo(infpresimage);
	    }
	  };

	  if(fwdSMOList!= null && fwdSMOList.smos != null)
	  service.informationPreservation(Cookies.getCookie("prismstate"),smotext,fwdSMOList.smos.size(), callback);
	
	}
  
  
  /**
   * Invoke the server to verify information preservation of the current SMO.
   * @param smotext
   */
  private void redundancyCheck(String smotext) {
	  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
	  ServiceDefTarget endpoint = (ServiceDefTarget) service;
	  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
	  endpoint.setServiceEntryPoint(moduleRelativeURL);
	  
	  System.out.println("checking inf pres.");
	  // set to wait the schema
	  redudnacycheckhtml.setHTML("checking...");
	  
	  AsyncCallback callback = new AsyncCallback() {
		  
		public void onSuccess(Object outcome) {
			if(outcome == null){
				redudnacycheckhtml.setHTML("ERROR: check your SMO syntax!");
				redudnacycheckhtml.setStyleName("redudnacycheckhtml_red");
			}
			else{
				
				String temp = outcome.toString();
				redudnacycheckhtml.setHTML(temp);
			
				if(temp.startsWith("ATTENTION:")){
					redudnacycheckhtml.setStyleName("redudnacycheckhtml_red");
					images.dangersign().applyTo(redundimage);
					
				}	
				else{					
					redudnacycheckhtml.setStyleName("redudnacycheckhtml_green");
					images.oksign().applyTo(redundimage);
				}
			}	
		}


		public void onFailure(Throwable caught) {
			redudnacycheckhtml.setHTML("connection error!");
			images.dangersign().applyTo(redundimage);
	    }
	  };

	  
	  service.checkRedundancy(Cookies.getCookie("prismstate"),smotext, callback);
	
	}

  /**
   * Check syntax of the current SMO and update visualization of Syntax errors... invoke also targetSchema updater applySMO if syntax is ok.
   * @param smotext
   * @param syntaxcheck
   */
	private void smoSyntaxCheck(String smotext, final HTML syntaxcheck, final SuggestBox sb) {
		  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
		  ServiceDefTarget endpoint = (ServiceDefTarget) service;
		  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
		  endpoint.setServiceEntryPoint(moduleRelativeURL);
		  
		  
		  // set syntax box text during computation
		  syntaxcheck.setHTML("veryfying syntax...");
		  
		  AsyncCallback callback = new AsyncCallback() {
			  
			public void onSuccess(Object outcome) {

		    	//System.out.println(outcome.toString());
				syntaxcheck.setHTML((String) outcome);
				if(((String)outcome).startsWith("syntax ok")){
					
					sb.setStyleName("smoinput_green");
					syntaxcheck.setStyleName("smosyntaxbox_green");
					images.oksign().applyTo(syntaximage);
					
			    	// change targetSchema in the as result of applying the smo
					applySMO(sb.getText()); 
					
					informationPreservationCheck(sb.getText());
					redundancyCheck(sb.getText());
					if(cs.loadcompletehistory){
						// refresh the comparison with following schemas
						String tempc = Cookies.getCookie("prismstate");
						compareCurrentSchemaWithReferenceTarget(tempc);
					}  
				}	
				else{	
					sb.setStyleName("smoinput_red");
					syntaxcheck.setStyleName("smosyntaxbox_red");
			    	sb.setStyleName("smoinput_red");
					images.dangersign().applyTo(syntaximage);
					
					infpreshtml.setStyleName("infpreshtml_green");
					infpreshtml.setHTML("waiting for valid smo...");
					images.oksign().applyTo(infpresimage);	

					redudnacycheckhtml.setStyleName("redudnacycheckhtml_green");
					redudnacycheckhtml.setHTML("waiting for valid smo...");
					images.oksign().applyTo(redundimage);	
					
					// CARLO UPDATE current schema 
					// show what would be the schema
					if(fwdSMOList!=null)
						getSchema(fwdSMOList.smos.size());
					else
						getSchema(0);
					//------
				}
			}


			public void onFailure(Throwable caught) {
				
				if(caught.toString().startsWith("com.google.gwt.user.client.rpc.StatusCodeException")){
			    	System.out.println("ERROR: Connection or Server side error");
			    	syntaxcheck.setHTML("The server is not reachable or there has been a server-side error.");		    	
				}	
		    	
		    	sb.setStyleName("smoinput_red");
				syntaxcheck.setStyleName("smosyntaxbox_red");
				images.stopsign().applyTo(syntaximage);

		    	
		    }
		  };
		  
		  String tempc = Cookies.getCookie("prismstate");
		  int ind = 0;
		  if(fwdSMOList != null)
			  ind = fwdSMOList.smos.size();
		  if(popupedit>=0)
			  ind = popupedit;
		  
		  if(tempc != null)			  
			  service.smoSyntaxCheck(tempc,smotext, ind,false, callback);
		  else
			  System.err.println("SMODesign.smoSyntaxCheck: NO COOKIE STATE ERROR");
		
	}
	
	
	private void getState(final ScrollPanel status) {
		  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
		  ServiceDefTarget endpoint = (ServiceDefTarget) service;
		  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
		  endpoint.setServiceEntryPoint(moduleRelativeURL);
		  
		  // set syntax box text during computation
		  status.setWidget(new HTML("Configure connection <br> in Configuration Tab."));
		  
		  AsyncCallback callback = new AsyncCallback() {
			  
			public void onSuccess(Object outcome) {
				if(outcome != null){
					cs = (ClientState) outcome;
					status.setWidget(((ClientState) cs).toStatusHTML());
					//status.setWidget(new HTML("status loaded"));
				}
				else{
					status.setWidget(new HTML("Error fetching status from server"));
				}
			}


			public void onFailure(Throwable caught) {

				status.setWidget(new HTML("Error fetching status from server"));

		    }
		  };
		  
		  String tempc = Cookies.getCookie("prismstate");
		  if(tempc != null)			  
			  service.getState(tempc, callback);
		  else
			  System.err.println("SMODesign.getState: NO COOKIE STATE ERROR");
		
	}
	
	  /**
	   * A very simple popup that closes automatically when you click off of it.
	   */
	  public static class MyPopup extends PopupPanel {
	    public MyPopup(Widget targetSchema) {
	      super(true);

	      setWidget(targetSchema);
	      setStyleName("showschemapopup");
	    }
	  }

	/**
	 * Method to deal with clicks.
	 */  
	public void onClick(Widget sender) {
	    if (sender == showSchemaButton) {
	        MyPopup p = new MyPopup(targetSchema);
	        int left = sender.getAbsoluteLeft() + 10;
	        int top = sender.getAbsoluteTop() + 10;
	        p.setPopupPosition(left, top);
	        p.show();
	      } 
	    else if(sender == addsmoButton){
			//smoSyntaxCheck(suggestBox.getText(), syntaxhtml,suggestBox);
	    	addSMO(suggestBox.getText(),scroller);
	    	syntaxhtml.setHTML("<b>Syntax Check:</b> waiting for new smo...");
	    	syntaxhtml.setStyleName("smosyntaxbox_green");
	    }
	    
	}
	
	
	private void addMultiSMO(String[] smotext, final ScrollPanel widget) {
		  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
		  ServiceDefTarget endpoint = (ServiceDefTarget) service;
		  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
		  endpoint.setServiceEntryPoint(moduleRelativeURL);
		  
		  AsyncCallback callback = new AsyncCallback() {
			  
			public void onSuccess(Object outcome) {	
				fwdSMOList = (ClientOperatorSequence) outcome;
				widget.setWidget(generateSMOList((ClientOperatorSequence)outcome));
				getSchema(fwdSMOList.smos.size());
				
				
				if(cs.loadcompletehistory){
				// refresh the comparison with following schemas
				  String tempc = Cookies.getCookie("prismstate");
				  compareCurrentSchemaWithReferenceTarget(tempc);
				}  

				//precompute rewriting in the inverse tab
				//prepareInverseTab();
			}

			public void onFailure(Throwable caught) {
		    	System.out.println("SMODesgin.addSMO:  ERROR");
		    }
		  };
		  
		  String tempc = Cookies.getCookie("prismstate");
		  if(tempc != null)			  
			  service.addMultipleSMO(tempc,smotext, callback);
		  else
			  System.err.println("SMODesign.addSMO: NO COOKIE STATE ERROR");
	}
	
	
	
	
	  /**
	   * Check syntax of the current SMO and update visualization of Syntax errors... invoke also targetSchema updater applySMO if syntax is ok.
	   * @param smotext
	   * @param syntaxcheck
	   */
		private void addSMO(String smotext, final ScrollPanel widget) {
			  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
			  ServiceDefTarget endpoint = (ServiceDefTarget) service;
			  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
			  endpoint.setServiceEntryPoint(moduleRelativeURL);
			  
			  AsyncCallback callback = new AsyncCallback() {
				  
				public void onSuccess(Object outcome) {	
					
					if(fwdSMOList == (ClientOperatorSequence) outcome)
						System.out.println("Operator LIST IS NOT CHANGED");
					
					fwdSMOList = (ClientOperatorSequence) outcome;
					widget.setWidget(generateSMOList((ClientOperatorSequence)outcome));
					
		    		suggestBox.setText("");	
		    		
		    		// update the schema visualization
					if(fwdSMOList != null)	
						getSchema(fwdSMOList.smos.size());
					else
						getSchema(0);
		    		// prepare next tab
		    		//prepareInverseTab();
		    		
				}

				public void onFailure(Throwable caught) {
			    	System.out.println("EvolutionDesign.addSMO:  ERROR");
			    }
			  };
			  
			  String tempc = Cookies.getCookie("prismstate");
			  if(tempc != null)			  
				  service.addSMO(tempc,smotext, callback);
			  else
				  System.err.println("EvolutionDesign.addSMO: NO COOKIE STATE ERROR");
			
		}
		
		
		/**
		 * preloads smos, inverses and more important triggers query computation.
		 */
		private void prepareInverseTab() {
			invTab.onShow();
			
		}
		
		/**
		 * Invoke the server to obtain the list of current SMOs.
		 * @param cookiekey
		 * @param widget
		 */
			protected void getCurrentSMOs(String cookiekey, final ScrollPanel widget) {
				  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
				  ServiceDefTarget endpoint = (ServiceDefTarget) service;
				  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
				  endpoint.setServiceEntryPoint(moduleRelativeURL);
				  
				  AsyncCallback callback = new AsyncCallback() {
					  
					public void onSuccess(Object outcome) {	
						if(outcome == null)
							widget.setWidget(new HTML("No SMO defined"));
						else{
							fwdSMOList = (ClientOperatorSequence) outcome;
							widget.setWidget(generateSMOList((ClientOperatorSequence)outcome));	
						}
					}


					public void onFailure(Throwable caught) {
				    	System.out.println("SMODesgin.getCurrentSMO:  ERROR");
				    }
				  };
				  
				  String tempc = Cookies.getCookie("prismstate");
				  if(tempc != null)			  
					  service.getCurrentSMOs(cookiekey, callback);
				  else
					  System.err.println("SMODesign.getCurrentSMOs: NO COOKIE STATE ERROR");
				
			}
			
		
				private void removeSMO(int smoindex) {
					  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
					  ServiceDefTarget endpoint = (ServiceDefTarget) service;
					  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
					  endpoint.setServiceEntryPoint(moduleRelativeURL);
					  
					  AsyncCallback callback = new AsyncCallback() {
						  
						public void onSuccess(Object outcome) {	
							
							// show the new SMO set in the scroller
							if(outcome == null){
								scroller.setWidget(new HTML("<b> Current SMO set: </b> <br /> <br />"));
								//Added by Swami for Error: unexisting schema issue
								//added so that the smo list returned from server when null makes the fwdsmolist corres
								//null
								fwdSMOList = (ClientOperatorSequence) outcome;
							}
							else{
								fwdSMOList = (ClientOperatorSequence) outcome;
								scroller.setWidget(generateSMOList((ClientOperatorSequence)outcome));	
							}
							// update the schema visualization
							if(fwdSMOList != null)	
								getSchema(fwdSMOList.smos.size());
							else
								getSchema(0);
							
						
							if(cs.loadcompletehistory){
							// refresh the comparison with following schemas
							  String tempc = Cookies.getCookie("prismstate");
							  compareCurrentSchemaWithReferenceTarget(tempc);
							}  
							//prepareInverseTab();
						}


						public void onFailure(Throwable caught) {
							
					    	System.out.println("SMODesgin.removeSMO:  ERROR");
					    }
					  };
					  
					  String tempc = Cookies.getCookie("prismstate");
					  if(tempc != null)			  
						  service.removeSMO(tempc, smoindex, callback);
					  else
						  System.err.println("SMODesign.removeSMO: NO COOKIE STATE ERROR");
					
				}
	
	
		
		public Widget generateSMOList(ClientOperatorSequence smos) {
			
			VerticalPanel vp = new VerticalPanel();
			vp.setStyleName("smopanel");
			if(smos != null && smos.smos != null)
				for(int i = 0 ; i < smos.smos.size(); i++){
					HorizontalPanel hp = new HorizontalPanel();
					vp.add(hp);
					hp.setStyleName("onesmo");
					PushButton deletebutton = new PushButton(images.deletesign().createImage());
					deletebutton.setTitle("delete smo " + i);
					deletebutton.addClickListener(new ClickListener() {
					     
						public void onClick(Widget sender) {
							//extract from the button title the smo to remove
							int smoindex = Integer.parseInt(sender.getTitle().substring(11));
							//call the remove
							removeSMO(smoindex);
							System.out.println(sender.getTitle());
						}
					});
					
					PushButton editbutton = new PushButton(images.editsign().createImage());
					editbutton.setTitle("edit smo " + i);
					editbutton.addClickListener(new ClickListener() {
					     
						public void onClick(Widget sender) {
							openSMOPopupEditor(Integer.parseInt(sender.getTitle().substring(9)));
							System.out.println(sender.getTitle());
						}
					});
					
					
					hp.add(deletebutton);
					hp.add(editbutton);
					hp.setSpacing(4);
					hp.add(new HTML(((ClientOperator)smos.smos.get(i)).toMetadataString()));
				}
				
			return vp;
		}

		/**
		 * open a dialog editor for the given smo
		 * @param index
		 */
		protected void openSMOPopupEditor(int index) {
		     ClientOperator c = (ClientOperator) fwdSMOList.smos.get(index);
		     popupedit = index;
		     DialogBox dlg = new SmoEditPopUp(index,c,this);
		     dlg.center();
			
			
		}

		
		 /**
		   * A simple dialog box that displays the SMO editor 
		   */
		  private static class SmoEditPopUp extends DialogBox implements ClickListener {
			  
		      Button cancelButton = new Button("Cancel", this);
		      Button okButton = new Button("OK", this);
		      int index = 0;
		      EvolutionDesign parent = null;
		    public SmoEditPopUp(int index, ClientOperator smo, EvolutionDesign p) {
		      this.index = index;
		      this.setStyleName("smodialogedit");
		      parent = p;
		      setText("SMO Editor");

		      HTML msg = new HTML(
		          "<center>You are editing Operator:" + 
		          		smo.majorSourceVersion + "." + smo.minorSourceVersion +"-"+
		          		smo.majorTargetVersion +"." + smo.minorTargetVersion + ".</center>",
		          true);

		      DockPanel dock = new DockPanel();
		      dock.setSpacing(4);

		      dock.add(okButton, DockPanel.SOUTH);
		      dock.add(cancelButton, DockPanel.SOUTH);
		      
		      dock.add(msg, DockPanel.NORTH);
		      suggestBoxPopup.setText(smo.text);
		      dock.add(suggestBoxPopup, DockPanel.CENTER); 
		      dock.setCellHorizontalAlignment(cancelButton, DockPanel.ALIGN_RIGHT);
		      dock.setCellHorizontalAlignment(okButton, DockPanel.ALIGN_RIGHT);
		      setWidget(dock);
		    }

		    
		    
		    public void onClick(Widget sender) {
		    	if(sender == okButton){
		    		parent.editSMO(index,suggestBoxPopup.getText());
		    	}
		    	parent.popupedit = -1;
		    	this.removeFromParent();
		    }
		  }
		
		
		/**
		 * invoke the server to edit an SMO in the list
		 * @param index
		 * @param smoText
		 */
		protected void editSMO(int index, String smoText) {
			 
			  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
			  ServiceDefTarget endpoint = (ServiceDefTarget) service;
			  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
			  endpoint.setServiceEntryPoint(moduleRelativeURL);
			  
			  AsyncCallback callback = new AsyncCallback() {
				  
				public void onSuccess(Object outcome) {	
					
					// show the new SMO set in the scroller
					if(outcome == null)
						scroller.setWidget(new HTML("No SMO defined"));
					else
						scroller.setWidget(generateSMOList((ClientOperatorSequence)outcome));	
					// update the schema visualization
					getSchema(fwdSMOList.smos.size());
				
					if(cs.loadcompletehistory){
					// refresh the comparison with following schemas
					  String tempc = Cookies.getCookie("prismstate");
					  compareCurrentSchemaWithReferenceTarget(tempc);
					}  
				}


				public void onFailure(Throwable caught) {
			    	System.out.println("SMODesgin.removeSMO:  ERROR");
			    }
			  };
			  
			  String tempc = Cookies.getCookie("prismstate");
			  if(tempc != null)			  
				  service.editSMO(tempc, index, smoText, callback);
			  else
				  System.err.println("SMODesign.editSMO: NO COOKIE STATE ERROR");
			
		}

		
		
	
}
