/*
 * Copyright 2007 Google Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package edu.ucla.cs.db.prism.gui.client;

import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.VerticalPanel;

/**
 * Introduction page.
 */
public class Info extends Sink {

  private VerticalPanel panel = new VerticalPanel();
  public static SinkInfo init(final Sink.Images images, ClientState cs) {
    return new SinkInfo("Intro", "<h2>Welcome to <i>PRISM</i></h2>" +
            "<p>This is <i>Prism</i> a tool built to assist you in the process of DB Schema evolution."
        + " <i>Prism</i> will support you in defining the schema changes you desire.</p>",cs) {

      @Override
	public Sink createInstance(ClientState cs) {
        return new Info(images,cs);
      }
    };
  }
  
  

	  
  
  

  public Info(final Sink.Images images,ClientState cs) {
    
	String introtext = "<br>This is Prism, a tool supporting graceful schema evolution. The system provide a simple" +
			"interface to support the definition of a set of schema changes, expressed in terms of  (Schema and Integrity Constraint) Modification Operators, " +
			"(see the project Homepage for more details: <a href=\"http://schemaevolution.org\">http://schemaevolution.org</a>). " +
			"<h3>1) Configuration</h3> In the first phase in figure the system will ask configuration parameters to connect to the MySQL database, " +
			"(leave default unchanged to test on a Demo database based on a version of the MediaWiki DB schema history)." +
			"<h3>2) Evolution-Design</h3> the second phase in figure, you can specify a series of atomic changes by issuing (Schema and Integrity Constraint) Modification Operators . The system will verify syntax for you and " +
			"give you feedback about information preservation and redundancy. The \"Show Resulting Schema\" button will provide at any moment the snapshot of the resulting schema," +
			"highlighting the changes." +
			"<h3>3) Inverse-Design</h3> In this phase the system tries to generate automatically an inverse sequence of changes, this is needed to allow query rewriting, and thus legacy application support." +
			"The user is allowed to override the default system choices at any time. The user intervention is needed to disambiguate cases in which the inverse is not unique. " +
			"The system tries to rewrite for a set of queries that have been logged during the system usage, and provides feedback on which percentage of queries is supported automatically, " +
			"based on the chosen inverses. " +
			"<h3>4) Validation</h3> In this phase the system allows to test more queries and to inspect rewritten queries and the logical mappings Disjunctive Embedded Dependencies (DEDs) derived from the schema evolution designed. " +
			"This is an assessment phase, the user should verify to be satisfied with the results of this evolution. " +
			"<h3>5) Deployment</h3> In this phase the system generates an optimized Data Migration to perform DDL and DML actions on the actual database. Furthermore the system generates a set of views that will support " +
			"old queries on top of the new schema. The user can verify this SQL and execute it on the actual data, or modify it manually." +
			"<p>For more information feel free to contact Carlo A. Curino at: <b>carlo@curino.us</b></p>";  
	
	VerticalPanel infotitle = new VerticalPanel();
	infotitle.setStyleName("introprismtitle");
	infotitle.add(new HTML("Introduction"));
	
	HTML html = new HTML(introtext);
	VerticalPanel intropanel = new VerticalPanel();
	intropanel.add(images.intro().createImage());
	intropanel.add(html);
	intropanel.setStyleName("introprism");
	panel.add(infotitle);
	panel.add(intropanel);
    initWidget(panel);
  }

  @Override
public void onShow() {
  }
}
