/*
 * Copyright 2007 Google Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package edu.ucla.cs.db.prism.gui.client;

import java.util.Iterator;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Cookies;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.ServiceDefTarget;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.ChangeListener;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.DialogBox;
import com.google.gwt.user.client.ui.DockPanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.KeyboardListener;
import com.google.gwt.user.client.ui.PopupPanel;
import com.google.gwt.user.client.ui.PushButton;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.SuggestBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

import edu.ucla.cs.db.prism.gui.client.EvolutionDesign.MyPopup;

/**
 * Represent interface to manage inverses. 
 */
public class Inverses extends Sink implements ChangeListener {

	public ClientState cs = null;
	ClientQueryRewriteResult[] queryRes = null;
	static Validation validationTab = null;
	static Deployment deploymentTab = null;

	public static SinkInfo init(final Sink.Images images, ClientState cs) {
		return new SinkInfo(
				"Inverse-Design",
				"<h2>Inverse Definition</h2>"
						+ "<p>In this step the system will assist you to make the evolution step defined in the "
						+ "previous step invertible. The system will compute most of the inverses and require "
						+ "your attention on the ambiguous one.</p>", cs) {

			public Sink createInstance(ClientState cs) {
				return new Inverses(images, cs);
			}
		};
	}
	
	public static SinkInfo init(Images images2, ClientState cs2, Validation i, Deployment d) {
		validationTab = i;	
		deploymentTab = d;
		return init(images2,cs2);
	}

	private ScrollPanel fwdSMOScroller = null;
	private ScrollPanel inverseSMOScroller = null;
	Sink.Images images;
	private ScrollPanel queryhorz;
	private ScrollPanel querysuccessrate;
	public HTML schemaInPoPup = new HTML("loading..");
	public ClientOperatorSequence inverseSMOList = null;
	public ClientOperatorSequence fwdSMOList = null;
	public static SuggestBox suggestBoxPopup = new SuggestBox(EvolutionDesign.oracle);
	public	PopupPanel pexpfwd = new PopupPanel(true);
	public	PopupPanel pexpbwd = new PopupPanel(true);

	
	public Inverses(Sink.Images images, ClientState cs) {
		this.cs = cs;
		this.images = images;
		HorizontalPanel horz;
		VerticalPanel panel = new VerticalPanel();
		panel.setHorizontalAlignment(VerticalPanel.ALIGN_LEFT);
		panel.add(horz = new HorizontalPanel());
		panel.add(horz);
		panel.add(queryhorz = new ScrollPanel());

		queryhorz.setStyleName("inverse_queryfailure_panel_green");
		initWidget(panel);
		
		
		VerticalPanel fwdsmopanel = new VerticalPanel();
		VerticalPanel fwdsmotitle = new VerticalPanel();
		fwdsmotitle.add(new HTML("Forward SMOs"));
		fwdsmotitle.setStyleName("fwdsmotitle");
		
		
		
		
	    Button pexpfwdbutton = new Button("Show/Hide Inverse MO export");
	    pexpfwdbutton.setStyleName("submitbutton");
		fwdsmotitle.add(pexpfwdbutton);

		pexpfwdbutton.addClickListener(new ClickListener() {
	      public void onClick(Widget sender) {
	    	  pexpfwd.setStyleName("popupsmoexport");
	    	  pexpfwd.setWidget(new HTML(fwdSMOList.toString()));
	    	  pexpfwd.center();
	    	  pexpfwd.show();
	    	  
	      }
	    });
		

		
		
		VerticalPanel leftpanel = new VerticalPanel();
		leftpanel.add(fwdsmotitle);
		leftpanel.add(fwdsmopanel);

		VerticalPanel rightpanel = new VerticalPanel();
		VerticalPanel inversesmotitle = new VerticalPanel();
		
		inversesmotitle.add(new HTML("Inverse SMOs"));
		inversesmotitle.setStyleName("inversesmotitle");

		
	    Button pexpbwdbutton = new Button("Show/Hide MO export");
	    pexpbwdbutton.setStyleName("submitbutton");
	    inversesmotitle.add(pexpbwdbutton);

		pexpbwdbutton.addClickListener(new ClickListener() {
	      public void onClick(Widget sender) {
	    	  pexpbwd.setStyleName("popupsmoexport");
	    	  pexpbwd.setWidget(new HTML(inverseSMOList.toString()));
	    	  pexpbwd.center();
	    	  pexpbwd.show();
	    	  
	      }
	    });
		
		
		
		
		
		
		
		VerticalPanel right = new VerticalPanel();
		
		rightpanel.add(inversesmotitle);
		rightpanel.add(right);
		
		VerticalPanel querypanel = new VerticalPanel();
		VerticalPanel querytitle = new VerticalPanel();
		querytitle.setStyleName("querytitle");
		
		querytitle.add(new HTML("Query Rewriting Success Rate"));
		leftpanel.add(querytitle);
		leftpanel.add(querypanel);
		
		querysuccessrate = new ScrollPanel();
		querypanel.setStyleName("querysuccessrate_panel");
		querypanel.add(querysuccessrate);

		fwdSMOScroller = new ScrollPanel();
		fwdSMOScroller.setWidget(new HTML("No SMOs defined"));
		fwdsmopanel.setStyleName("inverse_fwd_smo_scroller");
		fwdsmopanel.add(fwdSMOScroller);
		inverseSMOScroller = new ScrollPanel();
		inverseSMOScroller.setWidget(new HTML("No SMOs defined"));
		right.setStyleName("inverse_inverse_smo_scroller");
		right.add(inverseSMOScroller);
		
		horz.add(leftpanel);
		horz.add(rightpanel);

	}

	public void onChange(Widget sender) {

	}

	public void onShow() {
		String cookie = Cookies.getCookie("prismstate");
		if (cookie != null) {
			getCurrentSMOs(cookie, fwdSMOScroller);
			getInverseSMOs(cookie, inverseSMOScroller);
	
		}
	}

	/**
	 * Invoke the server to obtain the list of current SMOs.
	 * 
	 * @param cookiekey
	 * @param widget
	 */
	protected void getCurrentSMOs(String cookiekey, final ScrollPanel widget) {
		PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT
				.create(PrimBackendService.class);
		ServiceDefTarget endpoint = (ServiceDefTarget) service;
		String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
		endpoint.setServiceEntryPoint(moduleRelativeURL);

		AsyncCallback callback = new AsyncCallback() {

			public void onSuccess(Object outcome) {
				if (outcome == null)
					widget.setWidget(new HTML(
							"<b> No SMO in the set. </b> <br /> <br />"));
				else{
					fwdSMOList = (ClientOperatorSequence) outcome;
					widget.setWidget(generateSMOList((ClientOperatorSequence) outcome,false));
				}
			}

			public void onFailure(Throwable caught) {
				System.out.println("SMODesgin.getCurrentSMOs:  ERROR");
			}
		};

		String tempc = Cookies.getCookie("prismstate");
		if (tempc != null)
			service.getCurrentSMOs(cookiekey, callback);
		else
			System.err
					.println("SMODesign.getCurrentSMOs: NO COOKIE STATE ERROR");

	}

	/**
	 * Invoke the server to obtain the list of current SMOs.
	 * 
	 * @param cookiekey
	 * @param widget
	 */
	protected void getInverseSMOs(String cookiekey, final ScrollPanel widget) {
		PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT
				.create(PrimBackendService.class);
		ServiceDefTarget endpoint = (ServiceDefTarget) service;
		String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
		endpoint.setServiceEntryPoint(moduleRelativeURL);

		AsyncCallback callback = new AsyncCallback() {

			public void onSuccess(Object outcome) {
				if (outcome == null)
					widget.setWidget(new HTML("No SMOs in the set."));
				else{
					inverseSMOList = (ClientOperatorSequence) outcome;
					widget.setWidget(generateSMOList((ClientOperatorSequence) outcome,true));
				}
				
				// once the Inverses are available check queries
				String cookie = Cookies.getCookie("prismstate");
				getQueryResult(cookie, querysuccessrate);
			}

			public void onFailure(Throwable caught) {
				System.out.println("SMODesgin.addSMO:  ERROR");
			}
		};

		String tempc = Cookies.getCookie("prismstate");
		if (tempc != null)
			service.getInverseSMOs(cookiekey, callback);
		else
			System.err
					.println("Inverse.getInverseSMOs: NO COOKIE STATE ERROR");

	}

	/**
	 * Invoke the server to obtain the list of queries not supported by the
	 * current inverses.
	 * 
	 * @param cookiekey
	 * @param querybox
	 */
	protected void getQueryResult(String cookiekey,
			final ScrollPanel querysuccessrate) {
		PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT
				.create(PrimBackendService.class);
		ServiceDefTarget endpoint = (ServiceDefTarget) service;
		String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
		endpoint.setServiceEntryPoint(moduleRelativeURL);
		
		
		if(inverseAreIncomplete()){
			queryhorz.setWidget(new HTML("Waiting for user input..."));
			querysuccessrate.setWidget(new HTML("Inverse SMOs need editing before computing queries"));	
			return;
		}
		
		queryhorz.setWidget(new HTML("verifying query support..."));
		querysuccessrate.setWidget(new HTML("computing..."));
		AsyncCallback callback = new AsyncCallback() {

			public void onSuccess(Object outcome) {
				if (outcome == null) {
					queryhorz
							.setWidget(new HTML(
									"<b> ERROR translating queries </b> <br /> <br />"));
					queryhorz.setStyleName("inverse_queryfailure_panel_red");
				} else {
					queryRes = (ClientQueryRewriteResult[]) outcome;
					queryhorz.setStyleName("inverse_queryfailure_panel_red");
					queryhorz.setWidget(new HTML(
									"Click on each query step to see which queries can't be rewritten."));
					querysuccessrate
							.setWidget(generateQuerySuccessRate(queryRes));
					
					// precompute next tab cont 
					//validationTab.onShow();
					//deploymentTab.onShow();
				}
			}

			public void onFailure(Throwable caught) {
				querysuccessrate.setWidget(new HTML("An error occured while computing queries: " + caught.getMessage()));
				System.out.println("Inverses.getQueryResult:  ERROR\n" + caught.getMessage());
			}
		};

		String tempc = Cookies.getCookie("prismstate");
		if (tempc != null)
			service.getRewriteResult(cookiekey, callback);
		else
			System.err
					.println("SMODesign.getQueryResult: NO COOKIE STATE ERROR");

	}

	/**
	 * Check that every SMO in the inverse sequence is complete. E.g., PARTITION TABLE might lack the conditions to partition
	 * @return
	 */
	private boolean inverseAreIncomplete() {

		if(inverseSMOList == null)
			return false;
		Iterator<ClientOperator> it = inverseSMOList.smos.iterator();
		while(it.hasNext()){
			ClientOperator temp = it.next();
			if(temp.isComplete==0) // if there exists at least an incomplete SMO return true!
				return true;
		}
		return false;
	}

	/**
	 * produces the Widget of success rate out of a resultset of all the queries 
	 * @param outcome
	 * @return
	 */
	protected Widget generateQuerySuccessRate(
			final ClientQueryRewriteResult[] outcome) {

		VerticalPanel vp = new VerticalPanel();
		vp.setStyleName("querylistpanel");

		for (int i = 0; i < outcome.length; i++) {
			HorizontalPanel hp = new HorizontalPanel();
			hp.setSpacing(4);
			vp.add(hp);
			hp.setStyleName("onequeryresult");

			PushButton showschema = new PushButton(images.schemasign()
					.createImage());
			// PAY ATTENTION WE RELY ON THIS STRING TO INVOKE THE BUTTON
			// FUNCTION
			showschema.setTitle("show failing queries at step " + i);
			showschema.addClickListener(new ClickListener() {

				public void onClick(Widget sender) {

					queryhorz.setWidget(generateQueryList(outcome, Integer
							.parseInt(sender.getTitle().substring(29))));
				}
			});

			hp.add(showschema);
			hp.setSpacing(4);
			hp.add(generatePercentageDiv((int) outcome[i]
					.percentageSuccessRate()));
		}

		return vp;
	}

	/**
	 * generate the visualization bar for the query success rate
	 * 
	 * @param x
	 * @return
	 */
	protected HTML generatePercentageDiv(int x) {
		String html = "<div style=\"background-color:#FF3333; height:22px; width:200px; font-size:11pt;\"> ";
		html += "<div style=\"background-color:#33FF33; float:left; height:22px; font-size:11pt; width:"
				+ (2 * x) + "px;\" ><b>" + x + "%</b></div></div>";

		return new HTML(html + "<br>");
	}

	/**
	 * Returned the list of queries failing for the first time at this step.
	 * 
	 * @param c
	 * @param step
	 * @return
	 */
	protected Widget generateQueryList(ClientQueryRewriteResult[] c, int step) {
		boolean onefail = false;
		String html = "";
		for (int i = 0; i < c[step].rewrittenqueries.length; i++) {
			if (c[step].results[i] == 0) {
				if (step > 0) {
					if (c[step - 1].results[i] == 1) {
						onefail = true;
						html += c[step].inputqueries[i] + "</tt><br>";
					}
				} else {
					onefail = true;
					html += c[step].inputqueries[i] + "</tt><br>";
				}
			}
		}

		if (onefail) {
			queryhorz.setStyleName("inverse_queryfailure_panel_red");
			return new HTML("<b>The following queries are invalidated at step "
					+ step + " and cannot be rewritten :</b><br>" + html);
		}
		queryhorz.setStyleName("inverse_queryfailure_panel_green");
		return new HTML(
				"<b>All the query surviving so far are rewritten through step: "
						+ step + "</b>");
	}

	/**
	 * Generates the smo list for the Inverse TAB (different from the one of
	 * SMO-Design)
	 * 
	 * @param smos
	 * @return
	 */
	public Widget generateSMOList(ClientOperatorSequence smos, boolean isInverse) {

		VerticalPanel vp = new VerticalPanel();
		vp.setStyleName("smopanel");
		int i = 0;
		for (i = 0; i < smos.smos.size(); i++) {
			// the schema buttons are added only for the inverse SMOs panel
			if(isInverse){
				PushButton showschemaButton = new PushButton("show supported schema at: " + ((ClientOperator) smos.smos.get(i)).majorTargetVersion + ":"+((ClientOperator) smos.smos.get(i)).minorTargetVersion);
				// PAY ATTENTION WE RELY ON THIS STRING TO INVOKE THE BUTTON
				// FUNCTION
				showschemaButton.setTitle("show schema of smo " + ((ClientOperator) smos.smos.get(i)).majorTargetVersion + ":"+((ClientOperator) smos.smos.get(i)).minorTargetVersion );
				showschemaButton.addClickListener(new ClickListener() {
	
					public void onClick(Widget sender) {
							MyPopup p = new MyPopup(schemaInPoPup);
							
							String[] version = sender.getTitle().substring(19).split(":");
							
							getInverseSchemaByVersion(Integer.parseInt(version[0]),Integer.parseInt(version[1]));
							int left = sender.getAbsoluteLeft() - 200;
					        int top = sender.getAbsoluteTop() + 10;
					        p.setPopupPosition(left, top);
					        p.show();
					}
				});
	
				vp.add(showschemaButton);
			}
			
			vp.setSpacing(4);
			if(isInverse){
				ClientOperator tmpSmo = ((ClientOperator) smos.smos.get(i));
				HorizontalPanel hp = new HorizontalPanel();
				PushButton p = new PushButton(images.editsign().createImage());
				p.setTitle("edit " + i);
				p.addClickListener(new ClickListener() {
	
					public void onClick(Widget sender) {
							openSMOPopupEditor(Integer.parseInt(sender.getTitle().substring(5)));
					}
				});
				
				if(tmpSmo.isComplete==0){
					hp.addStyleName("neededitinverse");
					hp.setTitle("This SMO needs editing!");
				}
				
				hp.add(p);
				hp.add(new HTML(tmpSmo.toMetadataString()));
				vp.add(hp);
	
			}else{
				HTML temp = new HTML(((ClientOperator) smos.smos.get(i)).toMetadataString());
				temp.setStyleName("onesmo");
				vp.add(temp);	
			}
		}
		
		// ADD THE LAST SCHEMA
		if(isInverse){
			// ADD LAST SCHEMA BUTTON 	
			PushButton showschemaButton = new PushButton("show final resulting schema");
			showschemaButton.setTitle("show final resulting schema");
			showschemaButton.addClickListener(new ClickListener() {
	
				public void onClick(Widget sender) {
						MyPopup p = new MyPopup(schemaInPoPup);
						getCurrentSchema();
						int left = sender.getAbsoluteLeft() - 200;
				        int top = sender.getAbsoluteTop() + 10;
				        p.setPopupPosition(left, top);
				        p.show();
				}
			});
	
			vp.add(showschemaButton);
		}
		
		
		
		return vp;
	}

	/**
	 * open a dialog editor for the given smo
	 * @param index
	 */
	protected void openSMOPopupEditor(int index) {
	     ClientOperator c = (ClientOperator) inverseSMOList.smos.get(index);
	     DialogBox dlg = new InverseSMOPopUpEdit(index,c,this);
	     dlg.center();
	}
	
	
	 /**
	   * A simple dialog box that displays the SMO editor 
	   */
	  private static class InverseSMOPopUpEdit extends DialogBox implements ClickListener {
		  
	      Button cancelButton = new Button("Cancel", this);
	      Button okButton = new Button("OK", this);
	      int index = -1;
	      Inverses parent = null;
	      HTML syntaxhtml = new HTML("syntax ok");
	      ClientOperator smo  = null;
	      
	    public InverseSMOPopUpEdit(final int index, final ClientOperator smo, final Inverses p) {
	      setText("SMO Editor");
	      parent = p;
	      this.smo = smo;
	      HTML msg = new HTML(
	          "<center>You are editing SMO:" + 
	          		smo.majorSourceVersion + "." + smo.minorSourceVersion +"-"+
	          		smo.majorTargetVersion +"." + smo.minorTargetVersion + ".</center>",
	          true);

	      DockPanel dock = new DockPanel();
	      dock.setSpacing(4);
	      p.suggestBoxPopup.addKeyboardListener(new KeyboardListener() {
	          
	  		public void onKeyDown(Widget sender, char keyCode, int modifiers) {
	  			
	  		}

	  		public void onKeyPress(Widget sender, char keyCode, int modifiers) {
	  			if(keyCode=='\r'){
	  				smoSyntaxCheck(suggestBoxPopup.getText(), syntaxhtml,suggestBoxPopup, smo.majorSourceVersion, smo.minorSourceVersion );
	  			}
	  		}

	  		public void onKeyUp(Widget sender, char keyCode, int modifiers) {
	  			smoSyntaxCheck(suggestBoxPopup.getText(), syntaxhtml,suggestBoxPopup, smo.majorSourceVersion, smo.minorSourceVersion );
	  		}
	        });
	      
	      p.suggestBoxPopup.setHeight("2em");
	      
	      HorizontalPanel hp = new HorizontalPanel();
	      hp.add(syntaxhtml);
	      syntaxhtml.setStyleName("inv_syntax_green");
	      hp.add(cancelButton);
	      hp.add(okButton);
	      hp.setSpacing(4);
	      
	      suggestBoxPopup.setStyleName("inverseedit_green");
	      dock.add(msg, DockPanel.NORTH);
	      suggestBoxPopup.setText(smo.text);
	      dock.add(suggestBoxPopup, DockPanel.CENTER);

	      dock.add(hp,DockPanel.SOUTH);
	      dock.setWidth("100%");
	      setWidget(dock);
	    }

	    
	    
	    public void onClick(Widget sender) {
	    	if(sender == okButton)
	    		parent.editInverseSMO(smo.majorSourceVersion,smo.minorSourceVersion,suggestBoxPopup.getText());
	    	this.removeFromParent();
	    }
	  }
	
	 private void getInverseSchema(int index) {
		  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
		  ServiceDefTarget endpoint = (ServiceDefTarget) service;
		  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
		  endpoint.setServiceEntryPoint(moduleRelativeURL);
		  
		  // set to wait the schema
		  schemaInPoPup.setHTML("retrieving current schema...");
		  
		  AsyncCallback callback = new AsyncCallback() {
			  
			public void onSuccess(Object outcome) {
				if(outcome == null){
					schemaInPoPup.setHTML("no valid schema, check your SMO syntax");
				}
				else{
					ClientSchema temp = (ClientSchema) outcome;
					schemaInPoPup.setHTML(temp.toPopupStringNoTypes());
				}	
			}


			public void onFailure(Throwable caught) {
				schemaInPoPup.setHTML("connection error!");
		    }
		  };

		  String tempc = Cookies.getCookie("prismstate");
		  if(tempc != null)
			  service.getInverseSchema(tempc, index, callback);
		  else
			  System.err.println("SMODesign.applySMO: NO COOKIE STATE ERROR");
		}
	 
	  public void editInverseSMO(int major, int minor, String text) {

		System.out.println("EDIT OF SMO " + major + ":" + minor + " " + text);
	  
		 PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
		  ServiceDefTarget endpoint = (ServiceDefTarget) service;
		  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
		  endpoint.setServiceEntryPoint(moduleRelativeURL);
		  
		  // set syntax box text during computation
		  inverseSMOScroller.setWidget(new HTML("veryfying syntax..."));
		  
		  AsyncCallback callback = new AsyncCallback() {
			  
			public void onSuccess(Object outcome) {
				//System.out.println("EDIT INVERSE SMO:" + outcome.toString());
				//REINVOKE THE DRAWING OF THE COMPONENT, AS A CONSEQUENCE SMOS AND SCHEMAS ARE RELOADED FROM THE SERVER...
				onShow();
			}


			public void onFailure(Throwable caught) {
		    	System.out.println("SYNTAX ERROR");
		    	inverseSMOScroller.setWidget( new HTML(caught.toString()));
		    }
		  };
		  
		  String tempc = Cookies.getCookie("prismstate");
		  if(tempc != null)			  
			  service.editInverseSMO(tempc,text, major, minor, callback);
		  else
			  System.err.println("SMODesign.editInverseSMO: NO COOKIE STATE ERROR");
		
	  }

	/**
	   * Check syntax of the current SMO and update visualization of Syntax errors... invoke also targetSchema updater applySMO if syntax is ok.
	   * @param smotext
	   * @param syntaxcheck
	   */
		private static void smoSyntaxCheck(String smotext, final HTML syntaxcheck, final SuggestBox sb, int major, int minor) {
			  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
			  ServiceDefTarget endpoint = (ServiceDefTarget) service;
			  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
			  endpoint.setServiceEntryPoint(moduleRelativeURL);
			  
			  // set syntax box text during computation
			  syntaxcheck.setHTML("veryfying syntax...");
			  
			  AsyncCallback callback = new AsyncCallback() {
				  
				public void onSuccess(Object outcome) {

			    	System.out.println(outcome.toString());
					syntaxcheck.setHTML((String) outcome);
					if(((String)outcome).startsWith("syntax ok")){
						sb.setStyleName("inverseedit_green");
						syntaxcheck.setStyleName("inv_syntax_green");
					}	
					else{	
						sb.setStyleName("inverseedit_red");
						syntaxcheck.setStyleName("inv_syntax_red");
					}
				}


				public void onFailure(Throwable caught) {
			    	System.out.println("SYNTAX ERROR");
			    	syntaxcheck.setHTML(caught.toString());
					syntaxcheck.setStyleName("inv_syntax_red");
			    	sb.setStyleName("smoinput_red");

			    }
			  };
			  
			  String tempc = Cookies.getCookie("prismstate");
			  if(tempc != null)			  
				  service.smoSyntaxCheckByVersion(tempc,smotext, major, minor, true, callback);
			  else
				  System.err.println("SMODesign.smoSyntaxCheck: NO COOKIE STATE ERROR");
			
		}
	 
	 private void getInverseSchemaByVersion(int major,int minor) {
		  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
		  ServiceDefTarget endpoint = (ServiceDefTarget) service;
		  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
		  endpoint.setServiceEntryPoint(moduleRelativeURL);
		  
		  // set to wait the schema
		  schemaInPoPup.setHTML("retrieving current schema...");
		  
		  AsyncCallback callback = new AsyncCallback() {
			  
			public void onSuccess(Object outcome) {
				if(outcome == null){
					schemaInPoPup.setHTML("no valid schema, check your SMO syntax");
				}
				else{
					ClientSchema temp = (ClientSchema) outcome;
					schemaInPoPup.setHTML(temp.toPopupStringNoTypes());
				}	
			}


			public void onFailure(Throwable caught) {
				schemaInPoPup.setHTML("connection error!");
		    }
		  };

		  String tempc = Cookies.getCookie("prismstate");
		  if(tempc != null)
			  service.getInverseSchemaByVersion(tempc,major,minor, callback);
		  else
			  System.err.println("SMODesign.applySMO: NO COOKIE STATE ERROR");
		}
	 
	 

	 private void getCurrentSchema() {
		  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
		  ServiceDefTarget endpoint = (ServiceDefTarget) service;
		  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
		  endpoint.setServiceEntryPoint(moduleRelativeURL);
		  
		  // set to wait the schema
		  schemaInPoPup.setHTML("retrieving current schema...");
		  
		  AsyncCallback callback = new AsyncCallback() {
			  
			public void onSuccess(Object outcome) {
				if(outcome == null){
					schemaInPoPup.setHTML("no valid schema, check your SMO syntax");
				}
				else{
					ClientSchema temp = (ClientSchema) outcome;
					schemaInPoPup.setHTML(temp.toPopupString());
				}	
			}


			public void onFailure(Throwable caught) {
				schemaInPoPup.setHTML("connection error!");
		    }
		  };

		  String tempc = Cookies.getCookie("prismstate");
		  if(tempc != null)
			  service.getCurrentSchema(tempc,fwdSMOList.smos.size(), callback);
		  else
			  System.err.println("SMODesign.applySMO: NO COOKIE STATE ERROR");
		}
	  
	
}
