/*
 * Copyright 2007 Google Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package edu.ucla.cs.db.prism.gui.client;

import java.util.Iterator;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Cookies;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.ServiceDefTarget;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.ChangeListener;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.DialogBox;
import com.google.gwt.user.client.ui.DockPanel;
import com.google.gwt.user.client.ui.Grid;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.KeyboardListener;
import com.google.gwt.user.client.ui.PopupPanel;
import com.google.gwt.user.client.ui.PushButton;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.SuggestBox;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

import edu.ucla.cs.db.prism.gui.client.EvolutionDesign.MyPopup;

/**
 * Represent interface to manage inverses. 
 */
public class NewInverses extends Sink implements ChangeListener {

	public ClientState cs = null;
	ClientQueryRewriteResult[] queryRes = null;
	static Validation validationTab = null;
	static Deployment deploymentTab = null;

	public static SinkInfo init(final Sink.Images images, ClientState cs) {
		return new SinkInfo(
				"Inverse-Design",
				"<h2>Inverse Definition</h2>"
						+ "<p>In this step the system will assist you to make the evolution step defined in the "
						+ "previous step invertible. The system will compute most of the inverses and require "
						+ "your attention on the ambiguous one.</p>", cs) {

			public Sink createInstance(ClientState cs) {
				return new NewInverses(images, cs);
			}
		};
	}
	
	public static SinkInfo init(Images images2, ClientState cs2, Validation i, Deployment d) {
		validationTab = i;	
		deploymentTab = d;
		return init(images2,cs2);
	}


	Sink.Images images;
	private VerticalPanel queryresbox = new VerticalPanel();
	private VerticalPanel queryresboxtitle = new VerticalPanel();
	private ScrollPanel queryresboxcontent = new ScrollPanel();
	public HTML schemaInPoPup = new HTML("loading..");
	public ClientOperatorSequence inverseSMOList = null;
	public ClientOperatorSequence fwdSMOList = null;
	public static SuggestBox suggestBoxPopup = new SuggestBox(EvolutionDesign.oracle);
	public Grid resulttable = null;
	public	PopupPanel pexpfwd = new PopupPanel(true);
	public	PopupPanel pexpbwd = new PopupPanel(true);

	
	public NewInverses(Sink.Images images, ClientState cs) {
		this.cs = cs;
		this.images = images;
			
		VerticalPanel mainpanel = new VerticalPanel();
		mainpanel.setStyleName("newinverse_mainpanel");
		
		HorizontalPanel titles = new HorizontalPanel();
	
		titles.setStyleName("titlesbar");
		HorizontalPanel fwdsmotitle = new HorizontalPanel();
		fwdsmotitle.add(new HTML("Forward SMOs"));
		fwdsmotitle.setStyleName("newinverse_fwdsmotitle");
		titles.add(fwdsmotitle);

	
		
	    Button pexpfwdbutton = new Button("Show MO Export");
	    pexpfwdbutton.setStyleName("smallsubmitbutton");
		fwdsmotitle.add(pexpfwdbutton);

		pexpfwdbutton.addClickListener(new ClickListener() {
	      public void onClick(Widget sender) {
	    	  pexpfwd.setStyleName("popupsmoexport");
	    	  if(fwdSMOList!=null)
	    		  pexpfwd.setWidget(new HTML(fwdSMOList.toString()));
	    	  else
	    		  pexpfwd.setWidget(new HTML("No Operator defined."));
	    	  pexpfwd.center();
	    	  pexpfwd.show();
	    	  
	      }
	    });
		
		
		
		HorizontalPanel inversesmotitle = new HorizontalPanel();
		inversesmotitle.add(new HTML("Inverse SMOs"));
		inversesmotitle.setStyleName("newinverse_inversesmotitle");
		titles.add(inversesmotitle);

		
	    Button pexpbwdbutton = new Button("Show Inverse MO export");
	    pexpbwdbutton.setStyleName("smallsubmitbutton");
	    inversesmotitle.add(pexpbwdbutton);

		pexpbwdbutton.addClickListener(new ClickListener() {
	      public void onClick(Widget sender) {
	    	  pexpbwd.setStyleName("popupsmoexport");
	    	  if(inverseSMOList!=null)
	    		  pexpbwd.setWidget(new HTML(inverseSMOList.toString()));
	    	  else
	    		  pexpfwd.setWidget(new HTML("No Operator defined."));
	    	  pexpbwd.center();
	    	  pexpbwd.show();
	    	  
	      }
	    });
		
		
		
		
		VerticalPanel querytitle = new VerticalPanel();
		querytitle.add(new HTML("Query Success"));
		querytitle.setStyleName("newinverse_querytitle");
		titles.add(querytitle);
		mainpanel.add(titles);
		
		VerticalPanel resultscontaininer = new VerticalPanel();
		resultscontaininer.setStyleName("newinverse_resultscontaininer");
		mainpanel.add(resultscontaininer);
		
		
		
		resulttable = new Grid(1,3);
		resulttable.resizeColumns(3);
		resulttable.getColumnFormatter().addStyleName(0, "fwdsmocolumn");
		resulttable.getColumnFormatter().addStyleName(1, "bkwdsmocolumn");
		resulttable.getColumnFormatter().addStyleName(2, "querycolumn");
		
		
	
		resulttable.setStyleName("newinverse_resulttable");
		mainpanel.add(resulttable);
		
		
		
		
		
		queryresbox.setStyleName("newinverse_qrbox");
		
		queryresboxtitle = new VerticalPanel();
		queryresboxtitle.add(new HTML("Query Rewriting Results "));
		queryresboxtitle.setStyleName("newinverse_qrbox_title");
		queryresbox.add(queryresboxtitle);
		queryresbox.add(queryresboxcontent);
		HTML tempht = new HTML("Click on the query success to show the results of the deatils of the rewriting at each step");
		tempht.setStyleName("queryboxmessage");
		queryresboxcontent.setWidget(tempht);
	
		mainpanel.add(queryresbox);
		
		initWidget(mainpanel);
		
	
	}

	public void onChange(Widget sender) {

	}

	public void onShow() {
		String cookie = Cookies.getCookie("prismstate");
		if (cookie != null) {
			getCurrentSMOs(cookie, resulttable);
			getInverseSMOs(cookie, resulttable);
	
		}
	}

	/**
	 * Invoke the server to obtain the list of current SMOs.
	 * 
	 * @param cookiekey
	 * @param widget
	 */
	protected void getCurrentSMOs(String cookiekey, final Grid widget) {
		PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT
				.create(PrimBackendService.class);
		ServiceDefTarget endpoint = (ServiceDefTarget) service;
		String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
		endpoint.setServiceEntryPoint(moduleRelativeURL);

		AsyncCallback callback = new AsyncCallback() {

			public void onSuccess(Object outcome) {
					fwdSMOList = (ClientOperatorSequence) outcome;
					if(fwdSMOList != null){
					
						int size = fwdSMOList.smos.size();
						resulttable.resizeRows(size);	
						rowformatting(resulttable);
						generateSMOList((ClientOperatorSequence) outcome,false, widget);
					}
			}

	

			public void onFailure(Throwable caught) {
				System.out.println("SMODesgin.getCurrentSMOs:  ERROR");
			}
		};

		String tempc = Cookies.getCookie("prismstate");
		if (tempc != null)
			service.getCurrentSMOs(cookiekey, callback);
		else
			System.err
					.println("SMODesign.getCurrentSMOs: NO COOKIE STATE ERROR");

	}
	
	private void rowformatting(Grid g) {
	    for (int row = 0; row < g.getRowCount(); row++) {
	        if (row == 0 || row % 2 == 0) {
	        	g.getRowFormatter().setStyleName(row, "lightbluerow");
	        } else {
	        	g.getRowFormatter().setStyleName(row, "whiterow");
	        }
	     } 
		
	}

	/**
	 * Invoke the server to obtain the list of current SMOs.
	 * 
	 * @param cookiekey
	 * @param widget
	 */
	protected void getInverseSMOs(String cookiekey, final Grid widget) {
		PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT
				.create(PrimBackendService.class);
		ServiceDefTarget endpoint = (ServiceDefTarget) service;
		String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
		endpoint.setServiceEntryPoint(moduleRelativeURL);

		AsyncCallback callback = new AsyncCallback() {

			public void onSuccess(Object outcome) {
	
				inverseSMOList = (ClientOperatorSequence) outcome;
				if(inverseSMOList!=null){
					resulttable.resizeRows(inverseSMOList.smos.size());	
					rowformatting(resulttable);
					generateSMOList((ClientOperatorSequence) outcome,true,resulttable);
				
					// once the Inverses are available check queries
					String cookie = Cookies.getCookie("prismstate");
					getQueryResult(cookie, resulttable);
				}	
			}

			public void onFailure(Throwable caught) {
				System.out.println("SMODesgin.addSMO:  ERROR");
			}
		};

		String tempc = Cookies.getCookie("prismstate");
		if (tempc != null)
			service.getInverseSMOs(cookiekey, callback);
		else
			System.err
					.println("Inverse.getInverseSMOs: NO COOKIE STATE ERROR");

	}

	/**
	 * Invoke the server to obtain the list of queries not supported by the
	 * current inverses.
	 * 
	 * @param cookiekey
	 * @param querybox
	 */
	protected void getQueryResult(String cookiekey,
			final Grid rtab) {
		PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT
				.create(PrimBackendService.class);
		ServiceDefTarget endpoint = (ServiceDefTarget) service;
		String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
		endpoint.setServiceEntryPoint(moduleRelativeURL);
		
		
		if(inverseAreIncomplete()){
			queryresboxcontent.setWidget(new HTML("Waiting for user input..."));
			rtab.setWidget(0,2,new HTML("Inverse SMOs need editing before computing queries"));	 
			return;
		}
		
		
		for(int i = 0; i< rtab.getRowCount(); i++){
			rtab.setWidget(i,2,new Image("spin.gif"));
		}
		
		AsyncCallback callback = new AsyncCallback() {

			public void onSuccess(Object outcome) {
				if (outcome == null) {
					queryresboxcontent
							.setWidget(new HTML(
									"<b> ERROR translating queries </b> <br /> <br />"));
					queryresboxcontent.setStyleName("inverse_queryfailure_panel_red");
				} else {
					queryRes = (ClientQueryRewriteResult[]) outcome;
					generateQuerySuccessRate(queryRes,rtab);
					
					// precompute next tab cont 
					//validationTab.onShow();
					//deploymentTab.onShow();
				}
			}

			public void onFailure(Throwable caught) {
				rtab.setWidget(0,2,new HTML("An error occured while computing queries: " + caught.getMessage()));
				System.out.println("Inverses.getQueryResult:  ERROR\n" + caught.getMessage());
			}
		};

		String tempc = Cookies.getCookie("prismstate");
		if (tempc != null)
			service.getRewriteResult(cookiekey, callback);
		else
			System.err
					.println("SMODesign.getQueryResult: NO COOKIE STATE ERROR");

	}

	/**
	 * Check that every SMO in the inverse sequence is complete. E.g., PARTITION TABLE might lack the conditions to partition
	 * @return
	 */
	private boolean inverseAreIncomplete() {

		if(inverseSMOList == null)
			return false;
		Iterator<ClientOperator> it = inverseSMOList.smos.iterator();
		while(it.hasNext()){
			ClientOperator temp = it.next();
			if(temp.isComplete==0) // if there exists at least an incomplete SMO return true!
				return true;
		}
		return false;
	}

	/**
	 * produces the Widget of success rate out of a resultset of all the queries 
	 * @param outcome
	 * @return
	 */
	protected Widget generateQuerySuccessRate(
			final ClientQueryRewriteResult[] outcome,Grid rtab) {

		//VerticalPanel vp = new VerticalPanel();
		//vp.setStyleName("querylistpanel");

		for (int i = 0; i < outcome.length; i++) {
			HorizontalPanel hp = new HorizontalPanel();
			rtab.setWidget(i, 2, hp);
			
			hp.setStyleName("onequeryresult");
			Image img = images.querymarksign().createImage();
			img.setHeight("15px");
			img.setWidth("15px");
			PushButton showschema = new PushButton(img);
			// PAY ATTENTION WE RELY ON THIS STRING TO INVOKE THE BUTTON
			// FUNCTION
			showschema.setTitle("show failing queries at step " + i);
			showschema.addClickListener(new ClickListener() {

				public void onClick(Widget sender) {

					queryresboxcontent.setWidget(generateQueryList(outcome, Integer
							.parseInt(sender.getTitle().substring(29))));
				}
			});

			hp.add(showschema);
			hp.add(generatePercentageDiv((int) outcome[i]
					.percentageSuccessRate()));
		}

		return rtab;
	}

	/**
	 * generate the visualization bar for the query success rate
	 * 
	 * @param x
	 * @return
	 */
	protected HTML generatePercentageDiv(int x) {
		String html = "<div style=\"background-color:#FF3333; height:22px; width:100px; font-size:11pt;\"> ";
		html += "<div style=\"background-color:#33FF33; float:left; height:22px; font-size:11pt; width:"
				+ (x) + "px;\" ><b>" + x + "%</b></div></div>";

		return new HTML(html + "<br>");
	}

	/**
	 * Returned the list of queries failing for the first time at this step.
	 * 
	 * @param c
	 * @param step
	 * @return
	 */
	protected Widget generateQueryList(ClientQueryRewriteResult[] c, int step) {
		boolean onefail = false;
			
		
		Grid resultset = new Grid(c[step].rewrittenqueries.length,2);
		rowformatting(resultset);
		for (int i = 0; i < c[step].rewrittenqueries.length; i++) {
			
			HTML inq = new HTML(c[step].inputqueries[i]);
			inq.setStyleName("queryres");
			resultset.setWidget(i, 0,inq);
			
			if(c[step].results[i]==1){
				HorizontalPanel ok = new HorizontalPanel();
				ok.add(images.oksign().createImage());
				ok.add(new HTML(c[step].rewrittenqueries[i]));
				ok.setStyleName("queryres");
				resultset.setWidget(i, 1,ok);		
			}		
			else{
				HorizontalPanel dan = new HorizontalPanel();
				dan.add(images.stopsign().createImage());
				dan.add(new HTML("<b>No Rewriting</b>"));
				dan.setStyleName("queryres");
				resultset.setWidget(i, 1,dan);
				onefail=true;
			}
		}
		
		queryresboxtitle.remove(0);
		queryresboxtitle.add(new HTML("Query Rewriting at step: " + (step+1)));

		return resultset; 
	}

	/**
	 * Generates the smo list for the Inverse TAB (different from the one of
	 * SMO-Design)
	 * 
	 * @param smos
	 * @return
	 */
	public Widget generateSMOList(ClientOperatorSequence smos, boolean isInverse, Grid rtab) {

		if(smos == null){
			//FIXME do something better than just nothing
			return rtab;
		}
			
		for (int i = 0; i < smos.smos.size(); i++) {
			HorizontalPanel hh = new HorizontalPanel();
			// the schema buttons are added only for the inverse SMOs panel
			if(isInverse){
				PushButton showschemaButton = new PushButton(images.schemasign().createImage());
				// PAY ATTENTION WE RELY ON THIS STRING TO INVOKE THE BUTTON
				// FUNCTION (horrible I know!)
				showschemaButton.setTitle("show schema of smo " + ((ClientOperator) smos.smos.get(i)).majorTargetVersion + ":"+((ClientOperator) smos.smos.get(i)).minorTargetVersion );
				showschemaButton.addClickListener(new ClickListener() {
	
					public void onClick(Widget sender) {
							MyPopup p = new MyPopup(schemaInPoPup);
							
							String[] version = sender.getTitle().substring(19).split(":");
							
							getInverseSchemaByVersion(Integer.parseInt(version[0]),Integer.parseInt(version[1]));
							int left = sender.getAbsoluteLeft() - 200;
					        int top = sender.getAbsoluteTop() + 10;
					        p.setPopupPosition(left, top);
					        p.show();
					}
				});
				hh.add(showschemaButton);
					
			}
			
			if(isInverse){
				ClientOperator tmpSmo = ((ClientOperator) smos.smos.get(i));
				HorizontalPanel hp = new HorizontalPanel();
				PushButton p = new PushButton(images.editsign().createImage());
				p.setTitle("edit " + i);
				p.addClickListener(new ClickListener() {
	
					public void onClick(Widget sender) {
							openSMOPopupEditor(Integer.parseInt(sender.getTitle().substring(5)));
					}
				});
				
				if(tmpSmo.isComplete==0){
					hp.addStyleName("neededitinverse");
					hp.setTitle("This SMO needs editing!");
				}
				
				hp.add(p);
				HTML temp = new HTML(tmpSmo.toString());
				temp.setStyleName("newinverse_bwdonesmo");
				hp.add(temp);

				hh.add(hp);
	
			}else{
				HTML temp = new HTML(((ClientOperator) smos.smos.get(i)).toMetadataString());
				temp.setStyleName("newinverse_fwdonesmo");
				hh.add(temp);	
			}
			if(isInverse)
				rtab.setWidget(i, 1, hh);
			else
				rtab.setWidget(i, 0, hh);
				
		}
		
		/*
		// ADD THE LAST SCHEMA
		if(isInverse){
			// ADD LAST SCHEMA BUTTON 	
			PushButton showschemaButton = new PushButton("show final resulting schema");
			showschemaButton.setTitle("show final resulting schema");
			showschemaButton.addClickListener(new ClickListener() {
	
				public void onClick(Widget sender) {
						MyPopup p = new MyPopup(schemaInPoPup);
						getCurrentSchema();
						int left = sender.getAbsoluteLeft() - 200;
				        int top = sender.getAbsoluteTop() + 10;
				        p.setPopupPosition(left, top);
				        p.show();
				}
			});
	
			vp.add(showschemaButton);
		}
		
		*/
		
		return rtab;
	}

	/**
	 * open a dialog editor for the given smo
	 * @param index
	 */
	protected void openSMOPopupEditor(int index) {
	     ClientOperator c = (ClientOperator) inverseSMOList.smos.get(index);
	     DialogBox dlg = new InverseSMOPopUpEdit(index,c,this);
	     dlg.center();
	}
	
	
	 /**
	   * A simple dialog box that displays the SMO editor 
	   */
	  private static class InverseSMOPopUpEdit extends DialogBox implements ClickListener {
		  
	      Button cancelButton = new Button("Cancel", this);
	      Button okButton = new Button("OK", this);
	      int index = -1;
	      NewInverses parent = null;
	      HTML syntaxhtml = new HTML("syntax ok");
	      ClientOperator smo  = null;
	      
	    public InverseSMOPopUpEdit(final int index, final ClientOperator smo, final NewInverses newInverses) {
	      setText("SMO Editor");
	      parent = newInverses;
	      this.smo = smo;
	      HTML msg = new HTML(
	          "<center>You are editing SMO:" + 
	          		smo.majorSourceVersion + "." + smo.minorSourceVersion +"-"+
	          		smo.majorTargetVersion +"." + smo.minorTargetVersion + ".</center>",
	          true);

	      DockPanel dock = new DockPanel();
	      dock.setSpacing(4);
	      newInverses.suggestBoxPopup.addKeyboardListener(new KeyboardListener() {
	          
	  		public void onKeyDown(Widget sender, char keyCode, int modifiers) {
	  			
	  		}

	  		public void onKeyPress(Widget sender, char keyCode, int modifiers) {
	  			if(keyCode=='\r'){
	  				smoSyntaxCheck(suggestBoxPopup.getText(), syntaxhtml,suggestBoxPopup, smo.majorSourceVersion, smo.minorSourceVersion );
	  			}
	  		}

	  		public void onKeyUp(Widget sender, char keyCode, int modifiers) {
	  			smoSyntaxCheck(suggestBoxPopup.getText(), syntaxhtml,suggestBoxPopup, smo.majorSourceVersion, smo.minorSourceVersion );
	  		}
	        });
	      
	      newInverses.suggestBoxPopup.setHeight("2em");
	      
	      HorizontalPanel hp = new HorizontalPanel();
	      hp.add(syntaxhtml);
	      syntaxhtml.setStyleName("inv_syntax_green");
	      hp.add(cancelButton);
	      hp.add(okButton);
	      hp.setSpacing(4);
	      
	      suggestBoxPopup.setStyleName("inverseedit_green");
	      dock.add(msg, DockPanel.NORTH);
	      suggestBoxPopup.setText(smo.text);
	      dock.add(suggestBoxPopup, DockPanel.CENTER);

	      dock.add(hp,DockPanel.SOUTH);
	      dock.setWidth("100%");
	      setWidget(dock);
	    }

	    
	    
	    public void onClick(Widget sender) {
	    	if(sender == okButton)
	    		parent.editInverseSMO(smo.majorSourceVersion,smo.minorSourceVersion,suggestBoxPopup.getText());
	    	this.removeFromParent();
	    }
	  }
	
	 private void getInverseSchema(int index) {
		  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
		  ServiceDefTarget endpoint = (ServiceDefTarget) service;
		  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
		  endpoint.setServiceEntryPoint(moduleRelativeURL);
		  
		  // set to wait the schema
		  schemaInPoPup.setHTML("retrieving current schema...");
		  
		  AsyncCallback callback = new AsyncCallback() {
			  
			public void onSuccess(Object outcome) {
				if(outcome == null){
					schemaInPoPup.setHTML("no valid schema, check your SMO syntax");
				}
				else{
					ClientSchema temp = (ClientSchema) outcome;
					schemaInPoPup.setHTML(temp.toPopupStringNoTypes());
				}	
			}


			public void onFailure(Throwable caught) {
				schemaInPoPup.setHTML("connection error!");
		    }
		  };

		  String tempc = Cookies.getCookie("prismstate");
		  if(tempc != null)
			  service.getInverseSchema(tempc, index, callback);
		  else
			  System.err.println("SMODesign.applySMO: NO COOKIE STATE ERROR");
		}
	 
	  public void editInverseSMO(int major, int minor, String text) {

		System.out.println("EDIT OF SMO " + major + ":" + minor + " " + text);
	  
		 PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
		  ServiceDefTarget endpoint = (ServiceDefTarget) service;
		  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
		  endpoint.setServiceEntryPoint(moduleRelativeURL);
		   
		  AsyncCallback callback = new AsyncCallback() {
			  
			public void onSuccess(Object outcome) {
				//System.out.println("EDIT INVERSE SMO:" + outcome.toString());
				//REINVOKE THE DRAWING OF THE COMPONENT, AS A CONSEQUENCE SMOS AND SCHEMAS ARE RELOADED FROM THE SERVER...
				onShow();
			}


			public void onFailure(Throwable caught) {
		    	System.out.println("SYNTAX ERROR");
		    	resulttable.setWidget(0,1,new HTML(caught.toString()));
		    }
		  };
		  
		  String tempc = Cookies.getCookie("prismstate");
		  if(tempc != null)			  
			  service.editInverseSMO(tempc,text, major, minor, callback);
		  else
			  System.err.println("SMODesign.editInverseSMO: NO COOKIE STATE ERROR");
		
	  }

	/**
	   * Check syntax of the current SMO and update visualization of Syntax errors... invoke also targetSchema updater applySMO if syntax is ok.
	   * @param smotext
	   * @param syntaxcheck
	   */
		private static void smoSyntaxCheck(String smotext, final HTML syntaxcheck, final SuggestBox sb, int major, int minor) {
			  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
			  ServiceDefTarget endpoint = (ServiceDefTarget) service;
			  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
			  endpoint.setServiceEntryPoint(moduleRelativeURL);
			  
			  // set syntax box text during computation
			  syntaxcheck.setHTML("veryfying syntax...");
			  
			  AsyncCallback callback = new AsyncCallback() {
				  
				public void onSuccess(Object outcome) {

			    	System.out.println(outcome.toString());
					syntaxcheck.setHTML((String) outcome);
					if(((String)outcome).startsWith("syntax ok")){
						sb.setStyleName("inverseedit_green");
						syntaxcheck.setStyleName("inv_syntax_green");
					}	
					else{	
						sb.setStyleName("inverseedit_red");
						syntaxcheck.setStyleName("inv_syntax_red");
					}
				}


				public void onFailure(Throwable caught) {
			    	System.out.println("SYNTAX ERROR");
			    	syntaxcheck.setHTML(caught.toString());
					syntaxcheck.setStyleName("inv_syntax_red");
			    	sb.setStyleName("smoinput_red");

			    }
			  };
			  
			  String tempc = Cookies.getCookie("prismstate");
			  if(tempc != null)			  
				  service.smoSyntaxCheckByVersion(tempc,smotext, major, minor, true, callback);
			  else
				  System.err.println("SMODesign.smoSyntaxCheck: NO COOKIE STATE ERROR");
			
		}
	 
	 private void getInverseSchemaByVersion(int major,int minor) {
		  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
		  ServiceDefTarget endpoint = (ServiceDefTarget) service;
		  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
		  endpoint.setServiceEntryPoint(moduleRelativeURL);
		  
		  // set to wait the schema
		  schemaInPoPup.setHTML("retrieving current schema...");
		  
		  AsyncCallback callback = new AsyncCallback() {
			  
			public void onSuccess(Object outcome) {
				if(outcome == null){
					schemaInPoPup.setHTML("no valid schema, check your SMO syntax");
				}
				else{
					ClientSchema temp = (ClientSchema) outcome;
					schemaInPoPup.setHTML(temp.toPopupStringNoTypes());
				}	
			}


			public void onFailure(Throwable caught) {
				schemaInPoPup.setHTML("connection error!");
		    }
		  };

		  String tempc = Cookies.getCookie("prismstate");
		  if(tempc != null)
			  service.getInverseSchemaByVersion(tempc,major,minor, callback);
		  else
			  System.err.println("SMODesign.applySMO: NO COOKIE STATE ERROR");
		}
	 
	 

	 private void getCurrentSchema() {
		  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
		  ServiceDefTarget endpoint = (ServiceDefTarget) service;
		  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
		  endpoint.setServiceEntryPoint(moduleRelativeURL);
		  
		  // set to wait the schema
		  schemaInPoPup.setHTML("retrieving current schema...");
		  
		  AsyncCallback callback = new AsyncCallback() {
			  
			public void onSuccess(Object outcome) {
				if(outcome == null){
					schemaInPoPup.setHTML("no valid schema, check your SMO syntax");
				}
				else{
					ClientSchema temp = (ClientSchema) outcome;
					schemaInPoPup.setHTML(temp.toPopupString());
				}	
			}


			public void onFailure(Throwable caught) {
				schemaInPoPup.setHTML("connection error!");
		    }
		  };

		  String tempc = Cookies.getCookie("prismstate");
		  if(tempc != null)
			  service.getCurrentSchema(tempc,fwdSMOList.smos.size(), callback);
		  else
			  System.err.println("SMODesign.applySMO: NO COOKIE STATE ERROR");
		}
	  
	
}
