/*
 * Copyright 2007 Google Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package edu.ucla.cs.db.prism.gui.client;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Cookies;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.ServiceDefTarget;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.ChangeListener;
import com.google.gwt.user.client.ui.CheckBox;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.FlowPanel;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.Image;
import com.google.gwt.user.client.ui.Label;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.PasswordTextBox;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.TabPanel;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.TextBoxBase;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

import edu.ucla.cs.db.prism.gui.client.ClientSchema;
import edu.ucla.cs.db.prism.gui.client.PrimBackendService;
import edu.ucla.cs.db.prism.gui.client.PrimBackendServiceAsync;

/**
 * Demonstrates various panels and the way they lay widgets out.
 */
public class PrismConfiguration extends Sink {

 public ClientState cs = new ClientState();	
 public String cookie = null;
 //final HorizontalSplitPanel hSplit = new HorizontalSplitPanel();
 final ScrollPanel inputschema = new ScrollPanel();
 boolean connected = false;
 public ListBox demomode = new ListBox();
 public Sink.Images globalimages = null;
 
  public static SinkInfo init(final Sink.Images images,ClientState cs) {
    return new SinkInfo(
       "Configuration", "<h2>Starting Configuration</h2>" +
        "<p>Please provide credentials to access your database, and configure system behavior in the Advanced Configuration pane. Leave default for demo mode.</p>",cs) {

      public Sink createInstance(ClientState cs) {
        return new PrismConfiguration(images,cs);
      }

      public String getColor() {
        return "#fe9915";
      }
    };
  }

 
  
  public PrismConfiguration(Sink.Images images,final ClientState cs) {
    HTML contents = new HTML("This is a <code>ScrollPanel</code> contained at "
        + "the center of a <code>DockPanel</code>.  "
        + "By putting some fairly large contents "
        + "in the middle and setting its size explicitly, it becomes a "
        + "scrollable area within the page, but without requiring the use of "
        + "an IFRAME."
        + "Here's quite a bit more meaningless text that will serve primarily "
        + "to make this thing scroll off the bottom of its visible area.  "
        + "Otherwise, you might have to make it really, really small in order "
        + "to see the nifty scroll bars!");
    this.cs = cs;
    ScrollPanel scroller = new ScrollPanel(contents);
    scroller.setStyleName("ks-layouts-Scroller");
    globalimages = images;

 /*   DisclosurePanel disc = new DisclosurePanel("Click to disclose something:");
    disc.setContent(new HTML("This widget is is shown and hidden<br>by the "
        + "disclosure panel that wraps it."));
*/
    FlowPanel flow = new FlowPanel();
    for (int i = 0; i < 8; ++i) {
      flow.add(new CheckBox("Flow " + i));
    }

    final TextBox dburltb = new TextBox();
    final TextBox usertb = new TextBox();
    final PasswordTextBox passwordtb = new PasswordTextBox();
    final TextBox metaSchemaName = new TextBox();
    final TextBox schemaName = new TextBox();
    final TextBox schemaVersion = new TextBox();
    final CheckBox icswitch = new CheckBox();
    icswitch.setChecked(false);
    final CheckBox bcswitch = new CheckBox();
    bcswitch.setChecked(false);   
    final CheckBox loadhistoryswitch = new CheckBox();
    loadhistoryswitch.setChecked(false);   
    
    
    // MAIN split
    HorizontalPanel main = new HorizontalPanel();
    
    
    //left panel
    VerticalPanel configuration = new VerticalPanel();
    main.add(configuration);
    
/*     Panel collapsiblePanel = new Panel();  
             collapsiblePanel.setTitle("Collapsible Panel");  
             collapsiblePanel.setWidth(200);  
             collapsiblePanel.setCollapsible(true);  
    main.add(collapsiblePanel);
  */  
    
    
    // right panel
    VerticalPanel showschema = new VerticalPanel();
    main.add(showschema);
    
    // title of the left panel
    HorizontalPanel configurationtitle = new HorizontalPanel();
    configurationtitle.setStyleName("configurationtitle");
    configurationtitle.add(images.infosign().createImage());
    configurationtitle.add(new HTML("Configuration"));
    configuration.add(configurationtitle);
    
    //title of the right panel
    VerticalPanel showschematitle = new VerticalPanel();
    showschematitle.setStyleName("showschematitle");
    showschematitle.add(new HTML("Starting Schema"));
    showschema.add(showschematitle);
    
    inputschema.setStyleName("inputschema");
    showschema.add(inputschema);
    
    VerticalPanel vp = new VerticalPanel();
    VerticalPanel advancedvp = new VerticalPanel();
    advancedvp.add(makeLabel("Integrity Constraints"));
    
    HorizontalPanel horz2 = new HorizontalPanel();
    horz2.addStyleName("padded");
    horz2.add(new Label("Turn on IC support:"));
    horz2.add(icswitch);
    advancedvp.add(horz2);
    
    
    advancedvp.add(makeLabel("Rewriting Optimization"));
    horz2 = new HorizontalPanel();
    horz2.add(new Label("Turn on backchase:"));
    horz2.addStyleName("padded");
    horz2.add(bcswitch);
    advancedvp.add(horz2);
   
    
    advancedvp.add(makeLabel("Evolution Mimicking"));
    horz2 = new HorizontalPanel();
    horz2.add(new Label("Turn on match-next-schema feedback:"));
    horz2.addStyleName("padded");
    
    horz2.add(loadhistoryswitch);
    advancedvp.add(horz2);
    
    
    showschema.add(vp);
    
    vp.add(makeLabel("DBMS Credentials"));
    
    horz2 = new HorizontalPanel();
    horz2.add(new Label("DB url:"));
    horz2.add(dburltb);
    horz2.setCellHorizontalAlignment(dburltb, HasHorizontalAlignment.ALIGN_RIGHT);
    dburltb.setTextAlignment(TextBoxBase.ALIGN_LEFT);
    horz2.setStyleName("smallinput");
    dburltb.setText("jdbc:mysql://yellowstone.cs.ucla.edu/");
    dburltb.addStyleName("smallinputbox");
    vp.add(horz2);
    horz2.setWidth("100%");
    
    horz2 = new HorizontalPanel();
    horz2.setStyleName("smallinput");
    horz2.add(new Label("user:"));
    usertb.setText("prismuser");
    
    usertb.addStyleName("smallinputbox");
    horz2.add(usertb);
    horz2.setCellHorizontalAlignment(usertb, HasHorizontalAlignment.ALIGN_RIGHT);
    vp.add(horz2);
    horz2.setWidth("100%");
    
    
    horz2 = new HorizontalPanel();
    horz2.setStyleName("smallinput");
    horz2.add(new Label("password:"));
    passwordtb.setText("p1r0s5m700435");
    passwordtb.addStyleName("smallinputbox");
    
    horz2.add(passwordtb);
    horz2.setCellHorizontalAlignment(passwordtb, HasHorizontalAlignment.ALIGN_RIGHT);
    vp.add(horz2);
    horz2.setWidth("100%");

    vp.add(makeLabel("MetaSchema selection"));
    horz2 = new HorizontalPanel();
    horz2.setStyleName("smallinput");
    horz2.add(new Label("Meta Schema:"));
    metaSchemaName.setText("evolution_db");
    
    metaSchemaName.addStyleName("smallinputbox");
    
    horz2.add(metaSchemaName);
    horz2.setCellHorizontalAlignment(metaSchemaName, HasHorizontalAlignment.ALIGN_RIGHT);
    vp.add(horz2);
    horz2.setWidth("100%");
    
    vp.add(makeLabel("Schema selection"));
    horz2 = new HorizontalPanel();
    horz2.setStyleName("smallinput");
    horz2.add(new Label("schema:"));
    schemaName.setText("wikipedia");
    schemaName.addStyleName("smallinputbox");
    horz2.add(schemaName);
    horz2.setCellHorizontalAlignment(schemaName, HasHorizontalAlignment.ALIGN_RIGHT);
    vp.add(horz2);
    horz2.setWidth("100%");
    
    vp.add(makeLabel("Version selection"));
    horz2 = new HorizontalPanel();
    horz2.setStyleName("smallinput");
    horz2.add(new Label("version:"));
    schemaVersion.setText("48");
    schemaVersion.setTitle("Insert the version you want to use. You can specify 'first', 'last' or a valid numeric value.");
    schemaVersion.addStyleName("smallinputbox");
    horz2.add(schemaVersion);
    horz2.setCellHorizontalAlignment(schemaVersion, HasHorizontalAlignment.ALIGN_RIGHT);
    vp.add(horz2);
    horz2.setWidth("100%");
    
    
    vp.add(makeLabel("Demo Mode"));
    horz2 = new HorizontalPanel();
    horz2.setStyleName("smallinput");
    horz2.add(new Label("select a demo use case:"));
    
    
    demomode.addItem("");
    
    // prepare a set of demo scenarios
    demomode.addItem("ensembl NO IC");
    demomode.addItem("ensembl IC");
    demomode.addItem("ensembl IC + HISTORY");
    demomode.addItem("wikipedia NO IC");
    demomode.setSelectedIndex(0);
    
    demomode.addChangeListener(new ChangeListener() {
	     
		public void onChange(Widget sender) {
			
			String selection = ((ListBox)sender).getItemText(((ListBox)sender).getSelectedIndex());

			if(selection.equals("ensembl NO IC")){

				// fill basic configuration
			    dburltb.setText("jdbc:mysql://yellowstone.cs.ucla.edu/");
			    usertb.setText("prismuser");
			    passwordtb.setText("p1r0s5m700435");
			    metaSchemaName.setText("evolution_db");
				schemaName.setText("ensembl");
			    schemaVersion.setText("53");

			    // fill advanced configuration
			    icswitch.setChecked(false);
			    bcswitch.setChecked(false);
			    loadhistoryswitch.setChecked(false);
			}
			
			if(selection.equals("ensembl IC")){

				// fill basic configuration
			    dburltb.setText("jdbc:mysql://yellowstone.cs.ucla.edu/");
			    usertb.setText("prismuser");
			    passwordtb.setText("p1r0s5m700435");
			    metaSchemaName.setText("evolution_db");
				schemaName.setText("ensembl");
			    schemaVersion.setText("53");

			    // fill advanced configuration
			    icswitch.setChecked(true);
			    bcswitch.setChecked(true);
			    loadhistoryswitch.setChecked(false);
			}
			if(selection.equals("ensembl IC + HISTORY")){

				inputschema.setWidget(new HTML("The loading of the full ensembl history might take a long time."));
				// fill basic configuration
			    dburltb.setText("jdbc:mysql://yellowstone.cs.ucla.edu/");
			    usertb.setText("prismuser");
			    passwordtb.setText("p1r0s5m700435");
			    metaSchemaName.setText("evolution_db");
				schemaName.setText("ensembl");
			    schemaVersion.setText("25");
			    
			    // fill advanced configuration
			    icswitch.setChecked(true);
			    bcswitch.setChecked(true);
			    loadhistoryswitch.setChecked(true);
			}
		
			if(selection.equals("wikipedia NO IC")){

				// fill basic configuration
			    dburltb.setText("jdbc:mysql://yellowstone.cs.ucla.edu/");
			    usertb.setText("prismuser");
			    passwordtb.setText("p1r0s5m700435");
			    metaSchemaName.setText("evolution_db");
				schemaName.setText("wikipedia");
			    schemaVersion.setText("48");

			    // fill advanced configuration
			    icswitch.setChecked(false);
			    bcswitch.setChecked(false);
			    loadhistoryswitch.setChecked(false);
			}
		
			
			
			
		}
	});
    
    horz2.add(demomode);
    horz2.setCellHorizontalAlignment(schemaVersion, HasHorizontalAlignment.ALIGN_RIGHT);
    vp.add(horz2);
    horz2.setWidth("100%");
    
    
    Button loadSchema=new Button("Load Schema");
    loadSchema.setStyleName("loadschemabutton");
    vp.add(loadSchema);
    
    
    loadSchema.addClickListener(new ClickListener() {
        public void onClick(Widget sender) {
        	
        	if(dburltb.getText().equalsIgnoreCase(""))
        		inputschema.setWidget(new HTML("<h3> dburltb.getText() You must set the DB url! </h3>"));
        	else
        	if(usertb.getText().equalsIgnoreCase(""))
        		inputschema.setWidget(new HTML("<h3> You must set the user! </h3>"));
        	else
        	if(passwordtb.getText().equalsIgnoreCase(""))
        		inputschema.setWidget(new HTML("<h3> You must set the password! </h3>"));
        	else
        	if(schemaName.getText().equalsIgnoreCase(""))
        		inputschema.setWidget(new HTML("<h3> You must set the schema Name! </h3>"));
        	else
            if(metaSchemaName.getText().equalsIgnoreCase(""))
            	inputschema.setWidget(new HTML("<h3> You must set the meta schema Name! </h3>"));
            else{        		
        		
        
    			cs.setDburltb(dburltb.getText());
    			cs.setPasswordtb(passwordtb.getText());
    			cs.setSchemaName(schemaName.getText());
    			cs.setMetaSchemaName(metaSchemaName.getText());
    			
    			cs.setUsertb(usertb.getText());
    			cs.setICSupport(icswitch.isChecked());	
    			cs.setBackchase(bcswitch.isChecked());
    			cs.setLoadcompletehistory(loadhistoryswitch.isChecked());
    			cs.setVersion(schemaVersion.getText());
    			//FIXME this is the IC/non-IC switch
    			setStateAndLoadSchema(cs,inputschema);
    			//getBaseSchemaByName(dburltb.getText(),usertb.getText(),passwordtb.getText(),schemaName.getText(),temphtml); 
    			

    			
            }
            
        }
      });
  /*  
    vp.add(flow);
    vp.add(makeLabel("Horizontal Panel"));
    vp.add(horz);
    vp.add(makeLabel("Vertical Panel"));
    vp.add(vert);

    Grid grid = new Grid(4, 4);
    for (int r = 0; r < 4; ++r) {
      for (int c = 0; c < 4; ++c) {
        grid.setWidget(r, c, images.gwtLogo().createImage());
      }
    }
*/
    TabPanel tabs = new TabPanel();
    tabs.setStyleName("configurationpanel");
    tabs.add(vp, "Basic Configuration");
    tabs.add(advancedvp, "Advanced Configuration");
    tabs.setWidth("100%");
    tabs.selectTab(0);

    configuration.add(tabs);
    initWidget(main); 
  }

  public void onShow() {
	  HTML temphtml = new HTML();
	  
	  if(connected)
		  getCurrentSchema(temphtml);
	  else{
		  temphtml.setHTML("Input your custom configuration parameters or load a preset Demo case.");
		  temphtml.setStyleName("openingtext");
	  }
	  inputschema.setWidget(temphtml);
  }

  private HTML makeLabel(String caption) {
    HTML html = new HTML(caption);
    html.setStyleName("ks-layouts-Label");
    html.setWidth("100%");
    return html;
  }
  
  /*public void getBaseSchemaByName(String dburl, String user, String password,  String schemaName, final HTML html){
	  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
	  ServiceDefTarget endpoint = (ServiceDefTarget) service;
	  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
	  endpoint.setServiceEntryPoint(moduleRelativeURL);
	  AsyncCallback callback = new AsyncCallback() {
		  
		public void onSuccess(Object schema) {
			connected = true;
	       html.setHTML(((ClientSchema) schema).toString());
	    }

	    public void onFailure(Throwable caught) {
	       html.setHTML("<h2> Error communicating with the server </h2>");
	    }
	  };

	  service.getSchemaByName(dburl, user, password, schemaName, callback);
  }*/

  public void setStateAndLoadSchema(ClientState cs,final ScrollPanel inputschema){
	  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
	  ServiceDefTarget endpoint = (ServiceDefTarget) service;
	  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
	  endpoint.setServiceEntryPoint(moduleRelativeURL);
	  
	  HorizontalPanel h = new HorizontalPanel();
	  h.add(new Image("spin.gif"));
	  h.addStyleName("spinningbal");
	  
	 inputschema.setWidget(h);
	  
	  
	  AsyncCallback callback = new AsyncCallback() {
		  
		public void onSuccess(Object outcome) {
			HTML temphtml = new HTML();
			getCurrentSchema(temphtml);
			inputschema.setWidget(temphtml);
	    }

	    public void onFailure(Throwable caught) {
	    	System.err.println("ERROR SAVING STATE ON THE SERVER");
	    }
	  };
	  
	  //
	  service.setState(Cookies.getCookie("prismstate"),cs, callback);
  }


  public void getCurrentSchema(final HTML html){
	  PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT.create(PrimBackendService.class);
	  ServiceDefTarget endpoint = (ServiceDefTarget) service;
	  String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
	  endpoint.setServiceEntryPoint(moduleRelativeURL);
	  AsyncCallback callback = new AsyncCallback() {
		  
			public void onSuccess(Object schema) {
				if(schema == null)
					html.setHTML("No schema loaded");
				else{
					
					html.setHTML(((ClientSchema) schema).toString());
				}
			}

		    public void onFailure(Throwable caught) {
		       html.setHTML("<h2> Error communicating with the server </h2>");
		    }
	 };
	  
	  service.getCurrentSchema(Cookies.getCookie("prismstate"), 0, callback);
  }

}  
  
  
  
