/*
 * Copyright 2007 Google Inc.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package edu.ucla.cs.db.prism.gui.client;

import com.google.gwt.core.client.GWT;
import com.google.gwt.user.client.Cookies;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.rpc.ServiceDefTarget;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.ClickListener;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HasHorizontalAlignment;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.KeyboardListenerAdapter;
import com.google.gwt.user.client.ui.RichTextArea;
import com.google.gwt.user.client.ui.ScrollPanel;
import com.google.gwt.user.client.ui.TextArea;
import com.google.gwt.user.client.ui.TextBox;
import com.google.gwt.user.client.ui.TextBoxBase;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

/**
 * Demonstrates the various text widgets.
 */
public class Validation extends Sink implements ClickListener{

	public ClientState cs = null;
	private Button rewriteQueryButton = null;
  public static SinkInfo init(ClientState cs) {
    return new SinkInfo(
        "Validation",
        "<h2>Schema Evolution Validation</h2>"
            + "<p>In this phase the system allows you to test some query rewriting and " +
              "shows the forward and backward logical mapping, as DEDs, and schema definitions " +
              "(in the default <a href=\"http://rocinante.ucsd.edu:8080/mars/tutorial.html\">MARS</a> format)." +
              "</p>",cs) {

      @Override
	public Sink createInstance(ClientState cs) {
        return new Validation(cs);
      }

      @Override
	public String getColor() {
        return "#2fba10";
      }
    };
  }

  private TextArea textArea = new TextArea();
  private ScrollPanel outputBox = new ScrollPanel();
  private ScrollPanel rewtimeBox = new ScrollPanel();
  private ScrollPanel fwddeds = new ScrollPanel();
  private ScrollPanel invdeds = new ScrollPanel();
  
 
  public Validation(ClientState cs ) {
	  this.cs = cs;
    TextBox readOnlyTextBox = new TextBox();
    readOnlyTextBox.setReadOnly(true);
    readOnlyTextBox.setText("read only");

    VerticalPanel vp = new VerticalPanel();
    
    VerticalPanel rewritequerytitle = new VerticalPanel();
    
    rewritequerytitle.setStyleName("rewritequerytitle");
    rewritequerytitle.add(new HTML("Rewrite a Query posed against the old schema version:"));
    vp.add(rewritequerytitle);
    
    HorizontalPanel qrpanel = new HorizontalPanel();
    qrpanel.setStyleName("qrpanel");
    VerticalPanel inputquery = new VerticalPanel();
    inputquery.add(createTextThing(textArea, false));
    qrpanel.add(inputquery);
    
    rewriteQueryButton = new Button("Rewrite Query", this);
    rewriteQueryButton.addStyleName("rewritebutton");
    outputBox = new ScrollPanel();
    outputBox.setStyleName("validation_queries_green");
    rewtimeBox.setStyleName("rewtimeBox");
    fwddeds = new ScrollPanel();
    fwddeds.setStyleName("validation_deds");
    invdeds = new ScrollPanel();
    invdeds.setStyleName("validation_deds");
    inputquery.add(rewriteQueryButton);
    inputquery.add(rewtimeBox);
    qrpanel.add(outputBox);
   
    vp.add(qrpanel);
    
    VerticalPanel fwddedtitle = new VerticalPanel();
    
    fwddedtitle.setStyleName("fwddedtitle");
    fwddedtitle.add(new HTML("Forward DEDs"));
    
    vp.add(fwddedtitle);
    vp.add(fwddeds);
    
    
    VerticalPanel bwddedtitle = new VerticalPanel();
    
    bwddedtitle.setStyleName("bwddedtitle");
    bwddedtitle.add(new HTML("Backward DEDs"));
    vp.add(bwddedtitle);
    vp.add(invdeds);
    
   
    textArea.setVisibleLines(5);

    //Widget richText = createRichText();
    //richText.setWidth("32em");

    HorizontalPanel hp = new HorizontalPanel();
    hp.add(vp);
    //hp.add(richText);
    hp.setCellHorizontalAlignment(vp, HasHorizontalAlignment.ALIGN_LEFT);
    //hp.setCellHorizontalAlignment(richText, HorizontalPanel.ALIGN_RIGHT);

    initWidget(hp);
    hp.setWidth("100%");
  }

  
  
  public static HTML makeLabel(String caption) {
	    HTML html = new HTML(caption);
	    html.setStyleName("ks-layouts-Label");
	    return html;
  }
  
  @Override
public void onShow() {
	  String tempc = Cookies.getCookie("prismstate");
	  getDEDs(tempc,false,fwddeds);
	  getDEDs(tempc,true,invdeds);
	  
  }

  private Widget createRichText() {
    RichTextArea area = new RichTextArea();
    RichTextToolbar tb = new RichTextToolbar(area);

    VerticalPanel p = new VerticalPanel();
    p.add(tb);
    p.add(area);

    area.setHeight("14em");
    area.setWidth("100%");
    tb.setWidth("100%");
    p.setWidth("100%");
    DOM.setStyleAttribute(p.getElement(), "margin-right", "4px");
    return p;
  }

  private Widget createTextThing(final TextBoxBase textBox,
      boolean addSelection) {
    HorizontalPanel p = new HorizontalPanel();

//    textBox.setWidth("300px");
    textBox.setStyleName("val_inputquerybox");
    
    p.add(textBox);

    if (addSelection) {
      final HTML echo = new HTML();
  
      p.add(echo);
      textBox.addKeyboardListener(new KeyboardListenerAdapter() {
        @Override
		public void onKeyUp(Widget sender, char keyCode, int modifiers) {
          updateText(textBox, echo);
        }
      });
  
      textBox.addClickListener(new ClickListener() {
        public void onClick(Widget sender) {
          updateText(textBox, echo);
        }
      });
  
      updateText(textBox, echo);
    }
    return p;
  }

  private void updateText(TextBoxBase text, HTML echo) {
    echo.setHTML("Selection: " + text.getCursorPos() + ", "
        + text.getSelectionLength());
  }

public void onClick(Widget sender) {
	if(sender == rewriteQueryButton){
		
		rewriteQuery(Cookies.getCookie("prismstate"),textArea.getText(),outputBox);
		
	}	

}

/**
 * Invoke the server to obtain the list of queries not supported by the current inverses.
 * @param cookiekey
 * @param querybox
 */
protected void rewriteQuery(String cookiekey, String query, final ScrollPanel querysuccessrate) {
	PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT
			.create(PrimBackendService.class);
	ServiceDefTarget endpoint = (ServiceDefTarget) service;
	String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
	endpoint.setServiceEntryPoint(moduleRelativeURL);
	outputBox.setWidget(new HTML("rewriting query..."));
	AsyncCallback callback = new AsyncCallback() {

		public void onSuccess(Object outcome) {
			if (outcome == null){
				outputBox.setWidget(new HTML("<b> ERROR translating queries </b> <br/> <br />"));
				outputBox.setStyleName("validation_queries_red");
			}
			else{
				outputBox.setStyleName("validation_queries_green");
				if(((SingleQueryRewriteResult)outcome).isSucceeded()){
					outputBox.setWidget(new HTML(((SingleQueryRewriteResult)outcome).getQueryResult()));
					rewtimeBox.setWidget(new HTML("Rewriting Time:" + ((SingleQueryRewriteResult)outcome).getRewritingtime()));
				}else{
					outputBox.setStyleName("validation_queries_red");
					outputBox.setWidget(new HTML(((SingleQueryRewriteResult)outcome).getError()));
						
				}	
			}
				
		}

		public void onFailure(Throwable caught) {
			System.out.println("Validation.rewriteQuery: ERROR");
		}
	};

	String tempc = Cookies.getCookie("prismstate");
	if (tempc != null)
		service.rewriteQuery(cookiekey,query, callback);
	else
		System.err.println("Validation.rewriteQuery: NO COOKIE STATE ERROR");

}


/**
 * Invoke the server to obtain the list of queries not supported by the current inverses.
 * @param cookiekey
 * @param querybox
 */
protected void getDEDs(String cookiekey, boolean inverse, final ScrollPanel deds) {
	PrimBackendServiceAsync service = (PrimBackendServiceAsync) GWT
			.create(PrimBackendService.class);
	ServiceDefTarget endpoint = (ServiceDefTarget) service;
	String moduleRelativeURL = GWT.getModuleBaseURL() + "service";
	endpoint.setServiceEntryPoint(moduleRelativeURL);
	deds.setWidget(new HTML("fetching DEDs..."));
	AsyncCallback callback = new AsyncCallback() {

		public void onSuccess(Object outcome) {
			if (outcome == null){
				deds.setWidget(new HTML("<b> ERROR fetching DEDs </b> <br /> <br />"));
			}
			else{
				deds.setWidget(new HTML((String) outcome));
			}	
		}

		public void onFailure(Throwable caught) {
			System.out.println("Validation.rewriteQuery: ERROR");
		}
	};

	String tempc = Cookies.getCookie("prismstate");
	if (tempc != null)
		service.getDEDs(cookiekey,inverse, callback);
	else
		System.err.println("Validation.rewriteQuery: NO COOKIE STATE ERROR");

}


}
