package edu.ucla.cs.db.prism.gui.server;

import it.polimi.elet.cautils.configurator.CACmdLineParser;
import it.polimi.elet.cautils.configurator.CAConfiguration;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Timestamp;

import org.apache.log4j.Logger;

import edu.ucla.cs.db.prism.backend.schema.Schema;
import edu.ucla.cs.db.prism.backend.schema.SchemaHistory;
import edu.ucla.cs.db.prism.backend.schema.SchemaHistoryLoader;
import edu.ucla.cs.db.prism.backend.schema.Table;
import edu.ucla.cs.db.prism.backend.smo.SMOImpl;
import edu.ucla.cs.db.prism.backend.smo.exception.SMOParsingErrorException;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequenceHistory;
import edu.ucla.cs.db.prism.backend.smo.util.SMOSequenceHistoryLoader;
import edu.ucla.cs.db.prism.backend.util.Log4JHelper;
import edu.ucla.cs.db.prism.backend.util.db.DBUtil;

public class DBLoader {
	
	//STATIC LOGGER
	static Logger logger = Logger.getLogger(DBLoader.class);
	
	public static void main(String[] args) {
		 
         //LOG4J SET UP
         Log4JHelper.log4JSetup(logger);
        
        
         System.out.println( getSchema(args));
		 //SchemaHistoryLoader shl2 = new SchemaHistoryLoader(conn2,dbname);
		// shl2.storeSchemaHistoryToDB(sh);
	
	
	}


	public static SchemaHistory getSchema(String[] args) {
		//Use CAConfigurator to load configuration from shell or xml-configuration file (-c inputfile)
		 CACmdLineParser par = null;
	 	 try {
			par = CACmdLineParser.getInstance();
		 } catch (InstantiationException e) {
			e.printStackTrace();
		 }
		//List here all the interesting parameters
		 par.addStringParameter('h',"host");	
		 par.addStringParameter('u',"dburl");
		 par.addStringParameter('m',"metadbname");	
		 par.addStringParameter('u',"username");
		 par.addStringParameter('p',"password");
		 par.addStringParameter('l',"logfile");
		 
		//parse them all 
		 CAConfiguration conf = par.parse(args);
		 
		//read the parameter you are interested in 
		 String host = (String) conf.getParameterValue("host");
		 String dburl = (String) conf.getParameterValue("dburl");
		 String metadbname = (String) conf.getParameterValue("metadbname");
		 String dbname = (String) conf.getParameterValue("dbname");
		 String username = (String) conf.getParameterValue("username");
		 String password = (String) conf.getParameterValue("password");
		 
		//use them
		 System.out.println("PRIM is about to be executed with the follwing parameters:");
		 System.out.println("  - host: " + host);		
		 System.out.println("  - user: " + username);
		 System.out.println("  - metadb: " + metadbname);
		 
		// MySQL DRIVER LOADING 
        try {
            Class.forName("com.mysql.jdbc.Driver").newInstance();
        } catch (Exception ex) { ex.printStackTrace();}
	 

         Connection conn = DBUtil.connect(dburl,metadbname,username,password);
		
    //     Connection conn2 = DBUtil.connect(dburl,"prim3",username,password);
         
         // example of usage of the logger
       //  logger.info("Connected to the DB:" + dburl +" " +metadbname+" " +username);
		 
		 SchemaHistoryLoader shl = new SchemaHistoryLoader(conn,dbname);
		 SchemaHistory sh = shl.loadSchemaHistoryFromDB();
	
		 return sh;

	}
	
	
	public static String getFirstSchemaString(){
		
		String[] s = {"-c","config/rewriting.xml"};
		
		SchemaHistory sh = getSchema(s);
		
		return sh.schemas.firstElement().toString();
	}

	
	public static Schema getSchemaByVersion(int major,int minor){
		
		String[] s = {"-c","config/rewriting.xml"};
		
		SchemaHistory sh = getSchema(s);
		
		return sh.getSchemaByVersion(major, minor);
	}
	
	
	public static Schema getSchemaByVersion(String dburl, String username,String password, String metadbname, String schemaName,int major,int minor){
		System.err.println(major + ":" +minor			);
		// MySQL DRIVER LOADING 
        try {
            Class.forName("com.mysql.jdbc.Driver").newInstance();
        } catch (Exception ex) { ex.printStackTrace();}
	 

        Connection conn = DBUtil.connect(dburl,metadbname,username,password);
		SchemaHistoryLoader shl = new SchemaHistoryLoader(conn,schemaName);
		SchemaHistory sh = shl.loadSchemaHistoryFromDB(false);
		
		return sh.getSchemaByVersion(major,  minor);
	}


	public static Schema getSchemaByName(String dburl, String user,String password, String schemaName) {
		// MySQL DRIVER LOADING 
        try {
            Class.forName("com.mysql.jdbc.Driver").newInstance();
        } catch (Exception ex) { ex.printStackTrace();}
	 

			System.out.println(dburl + " " + user);
		  	Connection conn = DBUtil.connect(dburl,"information_schema",user,password);
		  	
			String schemaListSQL ="SELECT '1' AS schema_version," +
						 "'0' AS schema_version_minor, " +
						 "sc.DEFAULT_CHARACTER_SET_NAME, " +
						 "sc.DEFAULT_COLLATION_NAME," +
						 "sc.SQL_PATH " +
				  "FROM information_schema.SCHEMATA sc " +
				  "WHERE sc.schema_name = '" + schemaName +"';";
			
			ResultSet schemaList = DBUtil.executeSQLQuery(schemaListSQL,conn);
			
			String schemaVersion = null;
			String schemaVersionMinor = null;
			Schema schema = null;
			
			try {
			while(schemaList.next()){
				schemaVersion = schemaList.getString("schema_version");
				schemaVersionMinor = schemaList.getString("schema_version_minor");
				
				schema = new Schema(schemaList.getInt("schema_version"),
							schemaList.getInt("schema_version_minor"),
							schemaName,
							schemaList.getString("DEFAULT_CHARACTER_SET_NAME"),
							schemaList.getString("DEFAULT_COLLATION_NAME"),
							schemaList.getString("SQL_PATH"),
							"",
							"",									
							new Timestamp(1970, 01, 01, 0, 0, 0, 0), null
							);
				schema.setSchemaName(schemaName);
				String tableListSQL = "SELECT table_name  " +
				   		  "FROM information_schema.TABLES t " +
				   		  "WHERE t.table_schema = '" + schemaName + "';";
				
				ResultSet tableList = DBUtil.executeSQLQuery(tableListSQL,conn);
				
				String tableName = null;
				String columnQuery = null;
				
				while(tableList.next()){
				
				tableName = tableList.getString("table_name");
				columnQuery = "SELECT  c.column_name, c.COLUMN_TYPE " +
							  "FROM information_schema.COLUMNS c " +
							  "WHERE c.table_schema = '" + schemaName + "' AND "+
							  		"c.table_name = '" + tableName + "';"; 
				
				ResultSet columnList = DBUtil.executeSQLQuery(columnQuery,conn);			  		
				Table tt = new Table(tableName);
				String columnName = null;
				String columnType = null;
				while(columnList.next()){
					columnName = columnList.getString("column_name");
					columnType = columnList.getString("COLUMN_TYPE");		
					tt.addColumn(columnName,columnType);		
				}
				schema.addTable(tt);
				}
				
				
				}
				
			} catch (SQLException e) {
				// TODO Auto-generated catch block
			e.printStackTrace();
		}	
			
			return schema;	
	}


	public static SMOSequenceHistory getSMO(String dburl, String user,
			String password, String metaSchemaName, String schemaName) {
	
		// MySQL DRIVER LOADING 
        try {
            Class.forName("com.mysql.jdbc.Driver").newInstance();
        } catch (Exception ex) { ex.printStackTrace();}
	 
        
			System.out.println(dburl + " " + user);
		  	Connection conn = DBUtil.connect(dburl,metaSchemaName, user,password);
		  	SMOSequenceHistoryLoader sshl = new SMOSequenceHistoryLoader(conn,schemaName);
		  	return sshl.loadSMOSequenceHistoryFromDB(false);
	}


	public static String testSMOSyntax(String dburltb, String usertb,
			String passwordtb, String metaSchemaName, String schemaName,
			String smotext, int majorTargetVersion, int minorTargetVersion) {
		boolean applied = true;
		Schema s = getSchemaByVersion(dburltb,usertb,passwordtb,metaSchemaName, schemaName, majorTargetVersion, minorTargetVersion);
		if(s == null)
			System.err.println("SCHEMA IS NULL");
		try{
			applied=s.applySMO(SMOSequenceHistoryLoader.parseSMOText(smotext));
		}catch(Exception i){
			i.printStackTrace();
			return "<b>syntax error:</b> " +smotext + " is not a valid SMO.";
		}
		if(applied)
			return "syntax ok";
		else
			return "<b>syntax error:</b> schema doesn't match!";
	}


	public static Schema applySMO(String dburltb, String usertb,
			String passwordtb, String metaSchemaName, String schemaName,
			String smotext, int majorTargetVersion, int minorTargetVersion) {
			boolean applied = true;
	Schema s = getSchemaByVersion(dburltb,usertb,passwordtb,metaSchemaName, schemaName, majorTargetVersion, minorTargetVersion);
	if(s == null)
		System.err.println("SCHEMA IS NULL");
	try{
		applied=s.applySMO(SMOSequenceHistoryLoader.parseSMOText(smotext));
	}catch(Exception i){
		i.printStackTrace();
		return null;
	}
	if(applied)
		return s;
	
	return null;
	}


	public static String informationPreservation(String smotext) {
		SMOImpl smo = null;
		try {
			smo = SMOSequenceHistoryLoader.parseSMOText(smotext);
		} catch (SMOParsingErrorException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		if(smo.hasPerfectInverse())
			return "information preservation check: OK";
		else
			return "ATTENTION: <b>" + smo.getOperatorType() + "</b> is not information preserving.<br> See <a href=\"" + smo.getOperatorType() + ".html\">details</a> .";
			
	}	
	
}
