/**
 * 
 */
package edu.ucla.cs.db.prism.tests;

import static org.junit.Assert.*;

import java.io.BufferedWriter;
import java.io.OutputStreamWriter;
import java.util.Vector;

import org.junit.Test;

import edu.ucla.cs.db.prism.backend.SQL.SQL;
import edu.ucla.cs.db.prism.backend.SQL.SQLUnion;
import edu.ucla.cs.db.prism.backend.SQL.SQLUnionSet;
import edu.ucla.cs.db.prism.backend.ded.DEDSet;
import edu.ucla.cs.db.prism.backend.ded.DEDSetHistory;
import edu.ucla.cs.db.prism.backend.marsManage.ICCodiGen;
import edu.ucla.cs.db.prism.backend.schema.exception.IllegalSMOException;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ForeignKey;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchemaHistory;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICTable;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.PrimaryKey;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ValueConstraint;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.exceptions.IntegrityConstraintsExistsException;
import edu.ucla.cs.db.prism.backend.smo.SMOImpl;
import edu.ucla.cs.db.prism.backend.smo.exception.SMOParsingErrorException;
import edu.ucla.cs.db.prism.backend.smo.util.SMOFactory;

/**
 * This class tests the ICCodiGen class responsible to generate the codi script to run MARS when IC are in place.
 * @author Carlo A. Curino (carlo@curino.us)
 *
 */
public class ICCodiGenTest {

	ICSchema testschema = null;
	boolean verbose = false;
	
	public void init(){
		
	    testschema = new ICSchema(1, 0, "testschema","", "","","", "0",null,null);
		
		ICTable test1 = new ICTable("test1");
		test1.addColumn("k1");
		test1.addColumn("k2");
		test1.addColumn("a");
		test1.addColumn("b");
		test1.addColumn("c");
		Vector<String> keys1 = new Vector<String>();
		keys1.add("k1");
		keys1.add("k2");
		try {
			test1.addConstraint(new PrimaryKey("pk1",keys1));
		} catch (IntegrityConstraintsExistsException e2) {
			e2.printStackTrace();
		}
		
		ICTable test2 = new ICTable("test2");
		test2.addColumn("k1");
		test2.addColumn("a");
		test2.addColumn("b");
		test2.addColumn("c");
		test2.addColumn("fk1");
		test2.addColumn("fk2");
		
		// add a primary key
		Vector<String> keys2 = new Vector<String>();
		keys2.add("k1");
		try {
			test2.addConstraint(new PrimaryKey("pk2",keys2));
		} catch (IntegrityConstraintsExistsException e2) {
		
			e2.printStackTrace();
		}
		
		// add a foreign key to table test2
		Vector<String> foreignkey2 = new Vector<String>();
		foreignkey2.add("fk1");
		foreignkey2.add("fk2");
		Vector<String> ref2 = new Vector<String>();
		ref2.add("k1");
		ref2.add("k2");
		try {
			test2.addConstraint(new ForeignKey("test2fk1", foreignkey2, "test1", ref2));
		} catch (IntegrityConstraintsExistsException e1) {
			e1.printStackTrace();
		}
		
		ICTable test3 = new ICTable("test3");
		test3.addColumn("a");
		test3.addColumn("b");
		test3.addColumn("c");
		
		Vector<String> foreignkey3 = new Vector<String>();
		foreignkey3.add("c");
		Vector<String> ref3 = new Vector<String>();
		ref3.add("c");
		try {
			test3.addConstraint(new ForeignKey("test3fk1", foreignkey3, "test1", ref3));
		} catch (IntegrityConstraintsExistsException e) {
			e.printStackTrace();
		}

		testschema.addTable(test1);
		testschema.addTable(test2);
		testschema.addTable(test3);
	}
	
	
	
	
	
	/**
	 * Test method for {@link edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICTable#getConstraintsAsDEDs(java.lang.String)}.
	 */
	@Test
	public final void testGetConstraintsAsDEDs() {
		
		init();
		
		try {
			testschema.getTable("test1").addConstraint(new ValueConstraint("vc1","test1","a",">","0"));
		} catch (IntegrityConstraintsExistsException e1) {
			e1.printStackTrace();
			fail("Wrong value constraint");
		}
		ICSchemaHistory sh = new ICSchemaHistory();
		sh.addSchema(testschema);
		
		try {
			SMOImpl smo = SMOFactory.createSMOFromString("RENAME TABLE test1 INTO test7");
			ICSchema ne= testschema.clone();
			ne.applySMO(smo);
			sh.addSchema(ne);
			DEDSet ds = smo.getAllDEDs(testschema);
			DEDSetHistory dh = new DEDSetHistory();
			dh.add(ds);
			BufferedWriter bf = new BufferedWriter(new OutputStreamWriter(System.out));
			
			SQLUnionSet su = new SQLUnionSet();
			SQLUnion sq = new SQLUnion();
			sq.sqls.add(new SQL("SELECT t.a FROM test1 t;"));
			
			su.sqlUnions.add(sq);
			
			// not much of a test so far... complete the input and debug
			ICCodiGen.writeCodiPrelude(sh, dh, su, bf);
			
			
			
		} catch (SMOParsingErrorException e) {
			e.printStackTrace();
		} catch (IllegalSMOException e) {
			e.printStackTrace();
		}
		
		fail("manual check needed here, read the output...  sorry about it");
	}

}
