/**
 * 
 */
package edu.ucla.cs.db.prism.tests;

import static org.junit.Assert.*;

import java.util.Vector;

import org.junit.Test;
import java.util.Arrays;

import edu.ucla.cs.db.prism.backend.rewrite.ICPrimRewriteEngine;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchema;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchemaHistory;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ValueConstraint;
import edu.ucla.cs.db.prism.backend.smo.RenameTableSMO;
import edu.ucla.cs.db.prism.backend.smo.exception.SMOParsingErrorException;
import edu.ucla.cs.db.prism.backend.smo.util.SMOFactory;
import edu.ucla.cs.db.prism.backend.SQL.SQL;
import edu.ucla.cs.db.prism.backend.SQL.SQLUnion;
import edu.ucla.cs.db.prism.backend.ded.DEDSetHistory;
import edu.ucla.cs.db.prism.backend.exception.ErrorRewritingException;
import edu.ucla.cs.db.prism.backend.icmo.AddForeignKey;
import edu.ucla.cs.db.prism.backend.icmo.AddPrimaryKey;
import edu.ucla.cs.db.prism.backend.icmo.AddValueConstraint;
import edu.ucla.cs.db.prism.backend.icmo.DropForeignKey;
import edu.ucla.cs.db.prism.backend.icmo.DropPrimaryKey;
import edu.ucla.cs.db.prism.backend.icmo.DropValueConstraint;
import edu.ucla.cs.db.prism.backend.icmo.ICMOSMOSequence;
import edu.ucla.cs.db.prism.backend.icmo.exceptions.AddForeignKeyException;
import edu.ucla.cs.db.prism.backend.icmo.exceptions.AddPrimaryKeyException;
import edu.ucla.cs.db.prism.backend.icmo.exceptions.AddValueConstraintException;

/**
 * @author Carlo A. Curino (carlo@curino.us)
 *
 */
public class ICMOSMOSequenceTest {

	ICSchema testschema = null;
	
	/**
	 * Test method for {@link edu.ucla.cs.db.prism.backend.icmo.ICMOSMOSequence#ICMOSMOSequence()}.
	 */
	@Test
	public final void testDEDGenerationAndRewrite() throws SMOParsingErrorException {

		
		 // THIS CASE IS A CYCLIC FK that drives MARS crazy
		 //icmosmosequence.addMos(new AddForeignKey("fk2",0,0,1,0,"test1", "test3", new Vector<String>(Arrays.asList("a","b")), new Vector<String>(Arrays.asList("a","b")), false, true, false));	 

		try {
			
			//----------------------- FIRST TEST ------------------
			// 					    ADD VC
			
			 //initialize operator sequence and schema
			 ICMOSMOSequence icmosmosequence = new ICMOSMOSequence();
			 ICSchema testschema = ICSchemaTest.init();

			 //create operator sequence
			 icmosmosequence.addMos(new AddValueConstraint("vc2",0,0,1,0,"test1", "c = \"test\"", false, true, false));			 

			 //define query and challenge and oracle
			 SQL inputSQL = new SQL("SELECT t.k1,t.a,t.b,t.c FROM test1 t;");
			 String refRewriting = "SELECT x0.k1, x0.a, x0.b, \"test\"\nFROM testschema.test1 x0\nWHERE \"test\" = x0.c";
			
			 //run the rewriting
			 runRewriting(icmosmosequence,testschema,inputSQL,refRewriting);
			
			
			//----------------------- FIRST TEST ------------------
			// 					    ADD VC
			
			 //initialize operator sequence and schema
			  icmosmosequence = new ICMOSMOSequence();
			  testschema = ICSchemaTest.init();
			  testschema.getTable("test1").addConstraint(new ValueConstraint("vc1","c = \"test\""));
			 //create operator sequence
			 icmosmosequence.addMos(new AddValueConstraint("vc2",0,0,1,0,"test1", "c = \"test\"", true, false, false));			 

			 //define query and challenge and oracle
			  inputSQL = new SQL("SELECT t.k1,t.a,t.b,t.c FROM test1 t;");
			  refRewriting = "SELECT x0.k1, x0.a, x0.b, \"test\"\nFROM testschema.test1 x0\nWHERE \"test\" = x0.c";
			
			 //run the rewriting
			 runRewriting(icmosmosequence,testschema,inputSQL,refRewriting);
			 
			//----------------------- SECOND TEST ------------------
			//                     DROP VC 
			 
			 //initialize operator sequence and schema
			 icmosmosequence = new ICMOSMOSequence();
			 testschema = ICSchemaTest.init();
			 //add a valueconstraints that we will remove via ICMO
			 
			 testschema.getTable("test1").addConstraint(new ValueConstraint("vc1","test1.k1 = \"1\""));
			 //managmenet of inequalities for VC is still troubled
			 //testschema.getTable("test1").addConstraint(new ValueConstraint("vc1","test1.k1 > \"1\""));

			 //create operator sequence
			 icmosmosequence.addMos(new DropValueConstraint("vc1",0,0,1,0,"test1"));			 
			 
			 //define query and challenge and oracle
			 inputSQL = new SQL("SELECT t.k1,t.a,t.b,t.c FROM test1 t;");
			 refRewriting = "SELECT \"1\", x0.a, x0.b, x0.c\nFROM testschema.test1 x0\nWHERE \"1\" = x0.k1";
			
			 //run the rewriting
			 runRewriting(icmosmosequence,testschema,inputSQL,refRewriting);
				
			 
			//----------------------- THIRD TEST ------------------
			 // 		REMOVE AND  ADD POTENTIALLY CICLYC FKs
			
			 //initialize operator sequence and schema
			 icmosmosequence = new ICMOSMOSequence();
			 testschema = ICSchemaTest.init();

			 //create operator sequence
			 icmosmosequence.addMos(new DropForeignKey("test3fk1",0,0,0,1,"test3"));			 
			 icmosmosequence.addMos(new AddForeignKey("fk2",0,1,1,0,"test1", "test3", new Vector<String>(Arrays.asList("a","b")), new Vector<String>(Arrays.asList("a","b")), false, true, false));
			 
			 //define query and challenge and oracle
			 inputSQL = new SQL("SELECT t.k1,t.a,t.b,t.c FROM test1 t;");
			 refRewriting = "SELECT x1.k1, x0.a, x0.b, x0.c\nFROM testschema.test3 x0, testschema.test1 x1\nWHERE x0.c = x1.c and x0.b = x1.b and x0.a = x1.a";
			
			 //run the rewriting
			 runRewriting(icmosmosequence,testschema,inputSQL,refRewriting);
				
			 
			//----------------------- FOURTH TEST ------------------
			 // 		REMOVE AND different type of ADD POTENTIALLY CICLYC FKs
			
			 //initialize operator sequence and schema
			 icmosmosequence = new ICMOSMOSequence();
			 testschema = ICSchemaTest.init();

			 //create operator sequence
			 icmosmosequence.addMos(new DropForeignKey("test3fk1",0,0,0,1,"test3"));			 
			 icmosmosequence.addMos(new AddForeignKey("fk2",0,1,1,0,"test1", "test3", new Vector<String>(Arrays.asList("a","b")), new Vector<String>(Arrays.asList("a","b")), true, false, false));
			 
			 //define query and challenge and oracle
			 inputSQL = new SQL("SELECT t.k1,t.a,t.b,t.c FROM test1 t;");
			 // under CHECK semantics the rewriting is an identity!
			 refRewriting = "SELECT x0.k1, x0.a, x0.b, x0.c\nFROM testschema.test1 x0";
			 //run the rewriting
			 runRewriting(icmosmosequence,testschema,inputSQL,refRewriting);
				


			//----------------------- FIFTH TEST ------------------
			//						DROP PK
			 
			 //initialize operator sequence and schema
			 icmosmosequence = new ICMOSMOSequence();
			 testschema = ICSchemaTest.init();

			 //create operator sequence
			 icmosmosequence.addMos(new DropPrimaryKey("pk1",0,0,1,0,"test1"));			 
			 
			 //define query and challenge and oracle
			 inputSQL = new SQL("SELECT t.k1,t.a,t.b,t.c FROM test1 t;");
			 // the rewritten query is identical to the original one, as expected
			 refRewriting = "SELECT x0.k1, x0.a, x0.b, x0.c\nFROM testschema.test1 x0";
			
			 //run the rewriting
			 runRewriting(icmosmosequence,testschema,inputSQL,refRewriting);

		
		 } catch (Exception e1) {	 
			 	fail("An unexpected exception occured");
				e1.printStackTrace();
		 }

	}

	/**
	 * 
	 */
	private void runRewriting(ICMOSMOSequence icmosmosequence,ICSchema testschema, SQL inputSQL, String refRewriting) {
		ICSchemaHistory icsh;
		DEDSetHistory dsh;
		ICPrimRewriteEngine pre = null;
		SQLUnion sqlUnion;
		SQL res;
		 
		 icsh = icmosmosequence.generateICSchemaHistory(testschema, false);
		 dsh = icmosmosequence.getAllDED(testschema, false);
		 try {
			pre = new ICPrimRewriteEngine(icsh,dsh);
		} catch (ErrorRewritingException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		
		//System.out.println("ICSchemaHistory:" + icsh.toStringWithMetadata());
		//System.out.println(icmosmosequence.getAllDED(testschema, false).toStringNoMetadata());
		
		
		// target and source are switched w.r.t. the rewrite parameter naming... is think is ok!
		sqlUnion=null;
		try {
			
			sqlUnion = pre.rewrite(inputSQL, 0, 0, 1, 0);
			
		} catch (ErrorRewritingException e) {
			e.printStackTrace();
		}
		 res = sqlUnion.sqls.firstElement();
		//VERY WEAK FORM OF CHECK, it might break due to syntactic sugar
		 if(!res.toViewString("testschema").equals(refRewriting)){
			 System.err.println("WRONG REWRITTEN QUERY:\n" + sqlUnion.toViewString("testschema"));
			fail("Some of the tables we were expecting in the written query are missing");
		}
		 else{
			 System.out.println("Test Ok: query rewritten correctly");
			 System.out.println("REWRITTEN QUERY:\n" + sqlUnion.toViewString("testschema"));
		 }
	}

	/**
	 * 
	 */
	private void addRemoveFK() {
		testschema = ICSchemaTest.init();
		 ICMOSMOSequence icmosmosequence = new ICMOSMOSequence();
		 
		 try {

			 // THIS CASE IS A CYCLIC FK that drives MARS crazy
			 //icmosmosequence.addMos(new AddForeignKey("fk2",0,0,1,0,"test1", "test3", new Vector<String>(Arrays.asList("a","b")), new Vector<String>(Arrays.asList("a","b")), false, true, false));
			
			 icmosmosequence.addMos(new DropForeignKey("test3fk1",0,0,0,1,"test3"));			 
			 icmosmosequence.addMos(new AddForeignKey("fk2",0,1,1,0,"test1", "test3", new Vector<String>(Arrays.asList("a","b")), new Vector<String>(Arrays.asList("a","b")), false, true, false));
			  
		 } catch (Exception e1) {
			e1.printStackTrace();
		}
		 
		ICSchemaHistory icsh = icmosmosequence.generateICSchemaHistory(testschema, false);
		DEDSetHistory dsh = icmosmosequence.getAllDED(testschema, false);
		ICPrimRewriteEngine pre = null;
		try {
			pre = new ICPrimRewriteEngine(icsh,dsh);
		} catch (ErrorRewritingException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		
		//System.out.println("ICSchemaHistory:" + icsh.toStringWithMetadata());
		//System.out.println(icmosmosequence.getAllDED(testschema, false).toStringNoMetadata());
		
		
		SQL inputSQL = new SQL("SELECT t.k1,t.a,t.b,t.c FROM test1 t;");
		// target and source are switched w.r.t. the rewrite parameter naming... is think is ok!
		SQLUnion sqlUnion=null;
		try {
			sqlUnion = pre.rewrite(inputSQL, 0, 0, 1, 0);			
		} catch (ErrorRewritingException e) {
			e.printStackTrace();
		}
		SQL res = sqlUnion.sqls.firstElement();
		//VERY WEAK FORM OF CHECK, it might break due to syntactic sugar
		if(!res.toViewString("testschema").equals("SELECT x1.k1, x0.a, x0.b, x0.c\nFROM testschema.test3 x0, testschema.test1 x1\nWHERE x0.c = x1.c and x0.b = x1.b and x0.a = x1.a")){
			System.out.println("REWRITTEN QUERY:\n" + sqlUnion.toViewString("testschema"));
			fail("Some of the tables we were expecting in the written query are missing");
		}
		
		System.out.println("Test Ok: query rewritten correctly");
	}

	/**
	 * Test method for {@link edu.ucla.cs.db.prism.backend.icmo.ICMOSMOSequence#ICMOSMOSequence()}.
	 */
	@Test
	public final void testICMOSMOSequence() {
	
		 testschema = ICSchemaTest.init();
		 ICMOSMOSequence icmosmosequence = new ICMOSMOSequence();
		 
		 try {
			icmosmosequence.addMos(SMOFactory.createSMOFromString("RENAME TABLE test1 INTO renamed;"));
			icmosmosequence.addMos(new DropPrimaryKey("pk1",0,1,0,2,"renamed"));			
			icmosmosequence.addMos(new AddPrimaryKey("pk2",0,2,0,3,"renamed",new Vector<String>(Arrays.asList("c")),true,false,false));			
			icmosmosequence.addMos(new AddForeignKey("fk2",0,3,0,4,"renamed", "test3", new Vector<String>(Arrays.asList("a","b")), new Vector<String>(Arrays.asList("a","b")), false, true, false));
			icmosmosequence.addMos(new AddValueConstraint("vc3",0,4,0,5,"renamed","a > 0",false,false,true));
			icmosmosequence.addMos(new DropValueConstraint("vc3",0,5,0,6,"renamed"));
			icmosmosequence.addMos(new DropForeignKey("fk2",0,6,1,0,"renamed"));
		} catch (SMOParsingErrorException e) {
			fail("The operation generated an exception, which were not supposed to be generated"); 
			e.printStackTrace();
		} catch (AddForeignKeyException e) {
			fail("The operation generated an exception, which were not supposed to be generated"); 
			e.printStackTrace();
		} catch (AddValueConstraintException e) {
			fail("The operation generated an exception, which were not supposed to be generated"); 
			e.printStackTrace();
		} catch (AddPrimaryKeyException e) {
			fail("The operation generated an exception, which were not supposed to be generated"); 
			e.printStackTrace();
		} 
		 
		//System.out.println(icmosmosequence.toString());
		//System.out.println("Inverting the ICMOSMOSequence as:\n ");
		
		ICMOSMOSequence inv = icmosmosequence.getICCandidateInverseSequence(testschema.clone());
		//System.out.println(inv.toString());

		if(inv.getMos().size() != icmosmosequence.getMos().size())
			fail("Something went wrong, the inversion succeeded but with unexpected outcome");
			
		if(!(inv.getMos().firstElement() instanceof RenameTableSMO))
			fail("Something went wrong, the inversion succeeded but with unexpected outcome");
			
			
		if(!(inv.getMos().elementAt(1) instanceof AddPrimaryKey))
			fail("Something went wrong, the inversion succeeded but with unexpected outcome");
			
		if(!((AddPrimaryKey)inv.getMos().elementAt(1)).id.equals("pk1"))
			fail("Something went wrong, the inversion succeeded but with unexpected outcome");
			
		if(!((AddPrimaryKey)inv.getMos().elementAt(1)).getTableName().equals("renamed"))
			fail("Something went wrong, the inversion succeeded but with unexpected outcome");
			
		if(!((AddPrimaryKey)inv.getMos().elementAt(1)).getColumns().firstElement().equals("k1"))
			fail("Something went wrong, the inversion succeeded but with unexpected outcome");
			
		if(!((AddPrimaryKey)inv.getMos().elementAt(1)).getColumns().elementAt(1).equals("k2"))
			fail("Something went wrong, the inversion succeeded but with unexpected outcome");
			
		if(!(inv.getMos().elementAt(2) instanceof DropPrimaryKey) )
			fail("Something went wrong, the inversion succeeded but with unexpected outcome");
			
		if(!((DropPrimaryKey)inv.getMos().elementAt(2)).id.equals("pk2"))
			fail("Something went wrong, the inversion succeeded but with unexpected outcome");
			
		if(!((DropPrimaryKey)inv.getMos().elementAt(2)).getTableName().equals("renamed"))
			fail("Something went wrong, the inversion succeeded but with unexpected outcome");
			
		if(!(inv.getMos().elementAt(3) instanceof DropForeignKey))
			fail("Something went wrong, the inversion succeeded but with unexpected outcome");
			
		if(!((DropForeignKey)inv.getMos().elementAt(3)).id.equals("fk2"))
			fail("Something went wrong, the inversion succeeded but with unexpected outcome");
			
		if(!((DropForeignKey)inv.getMos().elementAt(3)).getTableName().equals("renamed"))
			fail("Something went wrong, the inversion succeeded but with unexpected outcome");
			
		if(!(inv.getMos().elementAt(4) instanceof DropValueConstraint))
			fail("Something went wrong, the inversion succeeded but with unexpected outcome");
			
		if(!((DropValueConstraint)inv.getMos().elementAt(4)).id.equals("vc3"))
			fail("Something went wrong, the inversion succeeded but with unexpected outcome");
			
		if(!((DropValueConstraint)inv.getMos().elementAt(4)).getTableName().equals("renamed"))
			fail("Something went wrong, the inversion succeeded but with unexpected outcome");
			
		if(!(inv.getMos().elementAt(5) instanceof AddValueConstraint))
			fail("Something went wrong, the inversion succeeded but with unexpected outcome");
			
		if(!(inv.getMos().elementAt(6) instanceof AddForeignKey))
			fail("Something went wrong, the inversion succeeded but with unexpected outcome");
		//FIXME add a couple of tests under here	
			
			//System.out.println("Test ok: ICMOSMOSequence inverted correctly");
			
			
	}

	

}
