/**
 * 
 */
package edu.ucla.cs.db.prism.tests;

import static org.junit.Assert.*;

import org.junit.Test;

import java.sql.Connection;
import java.util.Vector;

import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ForeignKey;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ForeignKeyCreationException;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ForeignKeyLoopException;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchemaHistory;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchemaHistoryLoader;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.PrimaryKey;
import edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ValueConstraint;
import edu.ucla.cs.db.prism.backend.util.db.DBUtil;

/**
 * @author Carlo A. Curino (carlo@curino.us)
 *
 */
public class ICSchemaHistoryLoaderTest {

	/**
	 * Test method for {@link edu.ucla.cs.db.prism.backend.schema.integrityconstraints.ICSchemaHistoryLoader#loadSchemaHistoryFromDB(boolean)}.
	 */
	@Test
	public final void testLoadSchemaHistoryFromDBBoolean() {
		
		// MySQL DRIVER LOADING
		try {
			Class.forName("com.mysql.jdbc.Driver").newInstance();
		} catch (Exception ex) {
			ex.printStackTrace();
		}

		String dburltb = "jdbc:mysql://127.0.0.1/";
		String usertb = "prism";
		String passwordtb = "prism1057";
		String schemaName = "wikipedia";
		String metaSchemaName = "prismdemo2";
		
		
		Connection conn = DBUtil.connect(dburltb, metaSchemaName, usertb, passwordtb);
		
		
		ICSchemaHistory sh = null;
		try {
			//load only one schema for the test
			sh = (new ICSchemaHistoryLoader(conn, schemaName)).loadSchemaHistoryFromDB(true,false,"48");
		} catch (ForeignKeyCreationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ForeignKeyLoopException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		if(sh.schemas.size()<1)
			fail("Error loading schemas");
		
		Vector<ForeignKey> fkvec = sh.getMostRecent().getTable("revision").getForeignKeys();
		if(fkvec == null | fkvec.size() != 1 || !fkvec.get(0).isSourceColumn("rev_page") || !fkvec.get(0).isTargetColumn("page_id"))
			fail("Foreign key for revision table not loaded");
		Vector<ValueConstraint> vcvec = sh.getMostRecent().getTable("page").getValueConstraint();
		if(vcvec == null || vcvec.size() != 1 || !vcvec.get(0).getOperator().equals(">") || !vcvec.get(0).isConstrained("page_id"))
			fail("Value Constraint for page table not loaded");
		PrimaryKey pkvec = sh.getMostRecent().getTable("page").getPrimaryKey();
		if(pkvec== null || !pkvec.isKey("page_id"))
			fail("PrimaryKey for page table not loaded");
		
		//System.out.println("SCHEMA:" + sh.getMostRecent().toStringWithMetadata());
		System.out.println("The integrity Constraints loading from DB seems all good!");
	}

}
