/**
 * Copyright (C) 2009 
 * Developer: Fabrizio Moroni fabrizio.moroni@yahoo.it
 * Contributor / Contact Person: Carlo Curino	carlo@curino.us
 * 
 *  * This file is part of SchemEvolutionToolsuite
 * 
 * SchemEvolutionToolsuite is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SchemaEvolutionToolsuite is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with SchemEvolutionToolsuite.  If not, see <http://www.gnu.org/licenses/>.
 */
package edu.ucla.cs.dbevolution;

import it.polimi.elet.cautils.configurator.CACmdLineParser;
import it.polimi.elet.cautils.configurator.CAConfigFileParser;
import it.polimi.elet.cautils.configurator.CAConfiguration;
import it.polimi.elet.cautils.configurator.CAConfigurator;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.Iterator;

import org.apache.log4j.Logger;

import edu.ucla.cs.Exceptions.UndefinedApplicationParameterException;
import edu.ucla.cs.statistics.StatisticsParameters;


/**
 * Parameters for all the functionality of the application
 * Avoid every function to read the configuration file
 * @author fangar
 *
 */
public class ApplicationParameters {

	private static HashMap<String,Boolean> functions = new HashMap<String, Boolean>();
	private static HashMap<String,String> collectionTables = new HashMap<String, String>();
	private static Hashtable<String, Object> parameters = new Hashtable<String, Object>();

	private static Logger logger = Logger.getLogger(ApplicationParameters.class);


	public static String getDbbaseuri() {
		return (String)parameters.get("dburi");
	}

	public static String getMysqlhost() {
		return (String)parameters.get("mysqlhost");
	}

	public static String getMysqlport() {
		return (String)parameters.get("mysqlport");
	}

	public static String getDbbasename() {
		return (String)parameters.get("dbbasename");
	}

	public static String getDbevolution() {
		return (String)parameters.get("dbevolution");
	}

	public static String getUser() {
		return (String)parameters.get("user");
	}

	public static String getPass() {
		return (String)parameters.get("pass");
	}

	public static String getRepoType() {
		return (String)parameters.get("repoType");
	}

	public static String getSvnpwd() {
		return (String)parameters.get("svnpwd");
	}

	public static String getSvnuser() {
		return (String)parameters.get("svnuser");
	}

	public static String getSvnurl() {
		return (String)parameters.get("svnurl");
	}

	public static String getSvnschemaurl() {
		return (String)parameters.get("svnschemaurl");
	}

	public static void addParameter (String pName, Boolean pValue){
		functions.put(pName, pValue);
	}

	public static Boolean getParameterValue (String pName){
		return(functions.get(pName));
	}

	public static String getPathtoschema() {
		return (String)parameters.get("pathtoschema");
	}

	public static String getCvsLocalPath() {
		return (String)parameters.get("cvsLocalPath");
	}

	public static String getCvsmodule() {
		return (String)parameters.get("cvsmodule");
	}

	public static String getCvsRoot() {
		return (String)parameters.get("cvsRoot");
	}

	public static String getCvsSaveSchemaPath() {
		return (String)parameters.get("cvsSaveSchemaPath");
	}

	public static HashMap<String, Boolean> getFunctions() {
		return functions;
	}


	public static void setFunctions(HashMap<String, Boolean> functions) {
		ApplicationParameters.functions = functions;
	}

	public static String getHeaderEvolutionTables() {
		return (String)parameters.get("headerEvolutionTables");
	}

	public static String getCleaningType() {
		return (String)parameters.get("cleaningType");
	}

	public static String getStatisticsFile() {
		return (String)parameters.get("statisticsFile");
	}

	public static String getCollectionQuery() {
		return (String)parameters.get("queryfile");
	}

	public static void addCollectionTable (String pName, String pValue){
		collectionTables.put(pName, pValue);
	}

	public static String getCollectionColumn (String pName){
		return(collectionTables.get(pName));
	}

	public static HashMap<String, String> getCollectionTables() {
		return collectionTables;
	}

	public static int getRevisionNumber(){
		return ((Integer)parameters.get("versionNumber")).intValue();
	}

	/**
	 * Load the application parameters from the configuration
	 * @param conf the application configuration
	 */
	public static void loadConfiguration(CAConfiguration conf){


		parameters.put("dburi",((String) conf.getParameterValue("dburi")));	
		parameters.put("dbbasename",((String) conf.getParameterValue("dbbasename")));
		parameters.put("user",((String) conf.getParameterValue("user")));
		parameters.put("pass",((String) conf.getParameterValue("pass")));
		parameters.put("dbevolution",((String) conf.getParameterValue("dbevolution")));
		parameters.put("pathtoschema",((String) conf.getParameterValue("pathtoschema")));
		addParameter("drop", Boolean.parseBoolean((String)conf.getParameterValue("drop")));
		addParameter("download", Boolean.parseBoolean((String)conf.getParameterValue("download")));
		addParameter("install", Boolean.parseBoolean((String)conf.getParameterValue("install")));
		addParameter("collect", Boolean.parseBoolean((String)conf.getParameterValue("collect")));
		addParameter("global", Boolean.parseBoolean((String)conf.getParameterValue("global")));
		addParameter("statistics", Boolean.parseBoolean((String)conf.getParameterValue("statistics")));
		parameters.put("repositoryType",((String) conf.getParameterValue("repositoryType")));
		parameters.put("headerEvolutionTables",((String) conf.getParameterValue("headerEvolutionTables")));
		parameters.put("evolutionTable",((String) conf.getParameterValue("evolutionTable")));

		parameters.put("versionNumber",Integer.parseInt((String) conf.getParameterValue("versionNumber")));
		parameters.put("baseRevision",((String) conf.getParameterValue("baseRevision")));
		parameters.put("cleaningSchema",((String) conf.getParameterValue("cleaningSchema")));
		parameters.put("statisticsFile",((String)conf.getParameterValue("statisticsFile")));
		parameters.put("queryfile",((String)conf.getParameterValue("queryfile")));
		parameters.put("svnurl",((String) conf.getParameterValue("svnurl")));
		parameters.put("svnuser",((String) conf.getParameterValue("svnuser")));
		parameters.put("svnpwd",((String) conf.getParameterValue("svnpwd")));  
		parameters.put("cvsRoot",((String) conf.getParameterValue("cvsRoot")));
		parameters.put("cvsLocalPath",((String) conf.getParameterValue("cvsLocalPath")));
		parameters.put("cvsModule",((String) conf.getParameterValue("cvsModule")));
		//parameters.put("cvsSaveSchemaPath",((String) conf.getParameterValue("cvsSaveSchemaPath")));

		//SMO translator parameters
		parameters.put("evolutionFile",((String) conf.getParameterValue("evolutionFile")));
	}

	/**
	 * Load the application parameters from the command line and xml configuration files
	 * @param args
	 */
	public static void loadApplicationParameters(String[] args) throws UndefinedApplicationParameterException{

		CAConfiguration conf = null;
		CAConfiguration confStatistics = null;
		CAConfigFileParser filepar = null;

		//parse the command line argument and set the application parameter
		//USE CAConfigurator to load configuration from inputline
		CACmdLineParser cmdParser = null;
		try {
			cmdParser = CACmdLineParser.getInstance();
		} 
		catch (InstantiationException e) {
			logger.error("Unable to parse the command line arguments",e);
			System.exit(1);
		}

		//set the configuration parameter that can be read from the command line
		cmdParser.addStringParameter("repositoryType");
		cmdParser.addStringParameter("pathtoschema");
		cmdParser.addStringParameter("dbbasename");
		//SVN
		cmdParser.addStringParameter("svnurl");
		cmdParser.addStringParameter("svnuser");
		cmdParser.addStringParameter("svnpass");
		//CVS
		cmdParser.addStringParameter("cvsRoot");
		cmdParser.addStringParameter("cvsModule");
		cmdParser.addStringParameter("cvsSaveSchemaPath");
		cmdParser.addStringParameter("cvsLocalPath");
		//OPERATIONS
		cmdParser.addStringParameter("collect");
		cmdParser.addStringParameter("download");
		cmdParser.addStringParameter("drop");
		cmdParser.addStringParameter("global");
		cmdParser.addStringParameter("install");


		if(args.length<1){
			logger.error("No command line option set, the configuration file must be present\n");
			System.exit(1);
		}
		try{ 
			conf = cmdParser.parse(args);
			String configpath = (String) conf.getParameterValue("config");

			//if the config path is set and equals to a configuration file, read and parse the configuration file
			if(configpath != null){
				filepar = CAConfigurator.getConfigFileParserInstance();
				conf = filepar.parse(configpath);
			}

			//parse again the configuration to override the parameters from the command line
			conf = cmdParser.parse(args);

			//override the parameters from the command line
			if(((String)conf.getParameterValue("repositoryType")).equals("svn")){
				if((String) conf.getParameterValue("svnurl")!=null)
					parameters.put("svnurl",((String) conf.getParameterValue("svnurl")));
				if((String) conf.getParameterValue("svnuser")!=null)
					parameters.put("svnuser",((String) conf.getParameterValue("svnuser")));
				if((String) conf.getParameterValue("svnpwd")!=null)
					parameters.put("svnpwd",((String) conf.getParameterValue("svnpwd")));  
			}
			else if (((String)conf.getParameterValue("repositoryType")).equals("cvs")){
				if((String) conf.getParameterValue("cvsRoot")!=null)
					parameters.put("cvsRoot",((String) conf.getParameterValue("cvsRoot")));
				if((String) conf.getParameterValue("cvsLocalPath")!=null)
					parameters.put("cvsLocalPath",((String) conf.getParameterValue("cvsLocalPath")));
				if((String) conf.getParameterValue("cvsModule")!=null)
					parameters.put("cvsModule",((String) conf.getParameterValue("cvsModule")));
				if((String) conf.getParameterValue("cvsSaveSchemaPath")!=null)
					parameters.put("cvsSaveSchemaPath",((String) conf.getParameterValue("cvsSaveSchemaPath")));
			}
			else{
				//Check which operation is required
				if (	((String) conf.getParameterValue("download")==null) || 
						((String) conf.getParameterValue("install")==null)
				)
				{
					logger.error("At least a repository Type need to be set in the configuration file for the required operation");
					System.exit(-1);
				}
			}
			if((String) conf.getParameterValue("pathtoschema")!=null)
				parameters.put("pathtoschema",((String) conf.getParameterValue("pathtoschema")));
			if((String) conf.getParameterValue("drop")!=null)
				addParameter("drop", Boolean.parseBoolean((String)conf.getParameterValue("drop")));
			if((String) conf.getParameterValue("download")!=null)
				addParameter("download", Boolean.parseBoolean((String)conf.getParameterValue("download")));
			if((String) conf.getParameterValue("install")!=null)
				addParameter("install", Boolean.parseBoolean((String)conf.getParameterValue("install")));
			if((String) conf.getParameterValue("collect")!=null)
				addParameter("collect", Boolean.parseBoolean((String)conf.getParameterValue("collect")));
			if((String) conf.getParameterValue("global")!=null)
				addParameter("global", Boolean.parseBoolean((String)conf.getParameterValue("global")));
			if((String) conf.getParameterValue("dbbasename")!=null)
				parameters.put("dbbasename",((String) conf.getParameterValue("dbbasename")));		


			//reads the application parameter
			loadConfiguration(conf);

			//load the COLLECTION TABLES info
			filepar = CAConfigurator.getConfigFileParserInstance();
			conf = filepar.parse(getCollectionQuery());

			//iterator over the keyset, all the keys of the hashmap 
			Iterator it = conf.keys();
			while(it.hasNext()){
				//print all the keys of the statistics file
				String table = it.next().toString();
				addCollectionTable(table, (String)conf.getParameterValue(table));
			}

			//STATISTICS parameters
			if(getStatisticsFile() != null){
				filepar = CAConfigurator.getConfigFileParserInstance();
				confStatistics = filepar.parse(getStatisticsFile());

				//set the application parameter
				StatisticsParameters.loadConfiguration(confStatistics);
			}
			else{
				//if the statistics operation is required and no parameter is set
				if(getParameterValue("statistics"))
					throw new UndefinedApplicationParameterException("Statistics configuration file missing");
			}
		}
		catch(NullPointerException nE){
			System.out.println("All the parameters in the configuration file must have a value\n");
			System.exit(1);
		}
	}

	public static String getEvolutionTable() {
		return (String)parameters.get("headerEvolutionTables") + (String)parameters.get("evolutionTable");
	}

	public static String getEvolutionFile() {
		return (String)parameters.get("evolutionFile");
	}

	public static String getBaseRevision() {
		return (String)parameters.get("baseRevision");
	}

	public static String getEvolutionRevision() {
		return (String)parameters.get("evolutionRevision");
	}

	public static String getDatabaseQueryLikeName(){
		return (String)parameters.get("dbbasename")+"%";
	}

}
