package edu.ucla.cs.dbevolution.DbBackend;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import net.sf.jsqlparser.schema.Table;
import edu.ucla.cs.dbevolution.ApplicationParameters;

/**
 * Map the information_schema tables in the evolution database
 * @author fangar
 *
 */
public class DbColumn {


	private String TABLE_CATALOG;
	private String TABLE_SCHEMA;
	private String TABLE_NAME;
	private String COLUMN_NAME;
	private int ORDINAL_POSITION;
	private String COLUMN_DEFAULT;
	private String IS_NULLABLE;
	private String DATA_TYPE;
	private Integer CHARACTER_MAXIMUM_LENGTH;
	private Integer CHARACTER_OCTET_LENGTH;
	private Integer NUMERIC_PRECISION;
	private Integer NUMERIC_SCALE;
	private String CHARACTER_SET_NAME;
	private String COLLATION_NAME;
	private String COLUMN_TYPE;
	private String COLUMN_KEY;
	private String EXTRA;
	private String PRIVILEGES;
	private String COLUMN_COMMENT;



	public static int compareColumnsName(ArrayList column1, ArrayList column2){

		int change = 0;
		boolean exist = false;
		String columnName = null;
		String column2Name = null;


		for(int j=0;j<column1.size();j++){
			if(column1.get(j) instanceof DbColumn){
				columnName = ((DbColumn)column1.get(j)).getCOLUMN_NAME(); 
			}
			else if(column1.get(j) instanceof DbKeyColumnUsage){
				columnName = ((DbKeyColumnUsage)column1.get(j)).getCOLUMN_NAME();
			}
			for(int i=0; i<column2.size();i++){
				if(column2.get(i) instanceof DbColumn){
					column2Name = ((DbColumn)column2.get(i)).getCOLUMN_NAME(); 
				}
				else if(column2.get(i) instanceof DbKeyColumnUsage){
					column2Name = ((DbKeyColumnUsage)column2.get(i)).getCOLUMN_NAME();
				}
				if(columnName.equals(column2Name)){
					exist = true;
					break;
				}
			}
			if(!exist){
				System.out.println("POSSIBLE NAME CHANGE");
				change++;
			}
		}
		return change;
	}


	public static int compareColumnsSize(ArrayList column1, ArrayList column2){
		if(column1.size() != column2.size()){
			System.out.println("Size Change");
			return 1;
		}
		return 0;
	}


	/**
	 * Return the columns with a possible name clash
	 * @return 
	 * @throws SQLException 
	 */
	public static HashMap<String,ArrayList<String>> getColumnNameClash(ArrayList<Table> tables) throws SQLException{

		Connection conn = null;
		PreparedStatement pStmt = null;
		ResultSet rs = null;

		String currentTable = null;
		String currentColumn = null;
		String oldTable = null;
		String oldColumn = null;
		ArrayList<String> tablesPerColumn = null;
		//ArrayList<ColumnNameClash> clashes = null;

		HashMap<String, ArrayList<String>> clashes = null;

		String tablesList = "(";

		int countTables = 0;

		if(tables.size()>1){

			tablesList += "\"" + tables.get(countTables).getName() + "\"";
			for(int count = countTables +1; count < tables.size();  count++){
				tablesList += ",";
				tablesList += "\"" + tables.get(count).getName() + "\"";
			}
		}
		else{
			tablesList += tables.get(countTables).getName();
		}
		tablesList += ")";


		String sql = "	SELECT table_name, column_name" +
		"				FROM PRISM_COLUMNS p" +
		"				WHERE	p.table_schema = ?" +
		"				AND p.table_name	IN " + tablesList +
		"				GROUP BY p.column_name " +
		"				HAVING count( column_name ) >1";

		try{
			conn = DbUtil.connect(ApplicationParameters.getDbevolution());
			pStmt = conn.prepareStatement(sql);
			pStmt.setString(1,ApplicationParameters.getBaseRevision());
			rs = pStmt.executeQuery();

			if(rs != null){

				tablesPerColumn = new ArrayList<String>();
				clashes = new HashMap<String, ArrayList<String>>();

				if(rs.next()){
					oldColumn = rs.getString(1);
					oldTable = rs.getString(2);
					tablesPerColumn.add(oldTable);
				}
				while(rs.next()){

					currentColumn = rs.getString(1);
					currentTable = rs.getString(2);

					if(currentColumn.equals(oldColumn)){
						tablesPerColumn.add(currentTable);
					}
					else{
						clashes.put(oldColumn,tablesPerColumn);

						tablesPerColumn = new ArrayList<String>();
					}
				}
				clashes.put(currentColumn, tablesPerColumn);
			}
			conn.close();
		}
		catch(SQLException sqlEx){
			sqlEx.printStackTrace();
		}
		return clashes;
	}



	public static int getMaxColumnLifetime() throws SQLException{
		
		PreparedStatement ps = null;
		ResultSet rs = null;
		Connection conn = null;
		
		HashMap<String, Object> params = new HashMap<String,Object>();
		params.put("tableSchema", ApplicationParameters.getDatabaseQueryLikeName());

		String sql = "	SELECT max(count)" +
		"		FROM (" +
		"		SELECT COLUMN_NAME, COUNT(*) AS count" +
		"		FROM PRISM_COLUMNS" +
		"		WHERE TABLE_SCHEMA LIKE ?" +
		"		GROUP BY TABLE_NAME,COLUMN_NAME" +
		"		) AS x";

		int lifetime = 0;
		
		try{
			conn = DbUtil.connect(ApplicationParameters.getDbevolution());
			ps = conn.prepareStatement(sql);
			ps.setString(1, (String)params.get("tableSchema"));
			rs = ps.executeQuery();

			while(rs.next()){
				lifetime = rs.getInt(1);
			}
		}
		catch(SQLException sEx){
			throw sEx;
		}
		finally{
			conn.close();
			rs.close();
			ps.close();
			rs = null;
			ps = null;
		}
		return lifetime;
	}


	/**
	 * Return a {@link DbColumn} object with the given column data type
	 * @param schema
	 * @param column
	 * @param table
	 * @return
	 * @throws SQLException 
	 */
	public static DbColumn getColumnDataType(String schema, String column, String table) throws SQLException{

    	String sql = "	SELECT	data_type AS DATA_TYPE," +
    			"				character_maximum_length AS CHARACTER_MAXIMUM_LENGTH," +
    			"				numeric_precision as NUMERIC_PRECISION," +
    			"				numeric_scale as NUMERIC_SCALE," +
    			"				column_type as COLUMN_TYPE" +
    			"		FROM 	PRISM_COLUMNS" +
    			"		WHERE 	table_schema = ?" +
    			"		AND		table_name = ?" +
    			"		AND		column_name = ?";
		
    	
    	PreparedStatement ps = null;
		ResultSet rs = null;
		Connection conn = null;
		
		DbColumn col = null;
		
		HashMap<String, Object> params = new HashMap<String,Object>();

		params.put("tableSchema", schema );
		params.put("tableName", table );
		params.put("columnName", column);
		
		try{
			conn = DbUtil.connect(ApplicationParameters.getDbevolution());
			ps = conn.prepareStatement(sql);
			ps.setString(1,(String)params.get("tableSchema"));
			ps.setString(2,(String)params.get("tableName"));
			ps.setString(3,(String)params.get("columnName"));
			rs = ps.executeQuery();
			
			while(rs.next()){
				col = new DbColumn();
				col.setDATA_TYPE(rs.getString(1));
				col.setCHARACTER_MAXIMUM_LENGTH(rs.getInt(2));
				col.setNUMERIC_PRECISION(rs.getInt(3));
				col.setNUMERIC_SCALE(rs.getInt(4));
				col.setCOLUMN_TYPE(rs.getString(5));
			}
		}
		catch(SQLException sEx){
			throw sEx;
		}
		finally{
			conn.close();
			rs.close();
			ps.close();
			rs = null;
			ps = null;
		}
		return col;
	}



	//TODO to be replaced by the IBATIS version
	/**
	 * Return the names of the columns given the input tables
	 * @param tablesFromInsert
	 */
	public static ArrayList<DbColumn> getColumnsFromTables(ArrayList<Table> tablesFromInsert) {

		Connection conn = null;
		PreparedStatement pStmt = null;
		ResultSet rs = null;

		String tablesList = "(";

		String sql = "	SELECT	TABLE_NAME,COLUMN_NAME" +
		"				FROM	PRISM_COLUMNS" +
		"				WHERE	TABLE_SCHEMA like ? AND table_name in ";

		String sqlOrderBy = " ORDER BY TABLE_NAME, COLUMN_NAME";

		int countTables = 0;
		ArrayList<DbColumn> columnNames = null;
		DbColumn column = null;

		if(tablesFromInsert.size()>1){

			tablesList += "\"" + tablesFromInsert.get(countTables).getName() + "\"";
			for(int count = countTables +1; count < tablesFromInsert.size();  count++){
				tablesList += ",";
				tablesList += "\"" + tablesFromInsert.get(count).getName() + "\"";
			}
		}
		else{
			tablesList += "\"" + tablesFromInsert.get(countTables).getName() + "\"";
		}
		tablesList += ")";

		sql += tablesList;
		sql += sqlOrderBy;

		try{
			conn = DbUtil.connect(ApplicationParameters.getDbevolution());
			pStmt = conn.prepareStatement(sql);
			pStmt.setString(1,ApplicationParameters.getBaseRevision());
			rs = pStmt.executeQuery();

			if(rs != null){
				columnNames = new ArrayList<DbColumn>();
				while(rs.next()){
					column = new DbColumn();
					column.setTABLE_NAME(rs.getString(1));
					column.setCOLUMN_NAME(rs.getString(2));
					columnNames.add(column);
				}
			}
			conn.close();
		}
		catch(SQLException sqlEx){
			sqlEx.printStackTrace();
		}

		return columnNames;

	}


	public static List<DbColumn> getColumnByTableAndVersionNumber(int version, int versionMinor, String schemaName, String tableName) throws SQLException{

		PreparedStatement ps = null;
		ResultSet rs = null;
		Connection conn = null;
		
		HashMap<String, Object> params = new HashMap<String,Object>();

		params.put("version", version );
		params.put("versionMinor", versionMinor );
		params.put("schemaName", schemaName);
		params.put("tableName", tableName);
		
		String sql = "	SELECT  c.column_name, c.COLUMN_TYPE" +
				"		FROM COLUMNS c" +
				"		WHERE	c.schema_version = ?" +
				"		AND		c.schema_version_minor = ?" +
				"		AND		c.table_schema = ? " +
				"		AND		c.table_name = ?";
		
		ArrayList<DbColumn> columns = null; 
		
		try{
			conn = DbUtil.connect(ApplicationParameters.getDbevolution());
			ps = conn.prepareStatement(sql);
			ps.setInt(1, ((Integer)params.get("version")).intValue());
			ps.setInt(2,((Integer)params.get("versionMinor")).intValue());
			ps.setString(3,(String)params.get("schemaName"));
			ps.setString(4,(String)params.get("tableName"));
			rs = ps.executeQuery();
			
			columns = new ArrayList<DbColumn>();
			
			while(rs.next()){
				DbColumn column = new DbColumn();
				column.setCOLUMN_NAME(rs.getString(1));
				column.setCOLUMN_TYPE(rs.getString(2));
				columns.add(column);
			}
		}
		catch(SQLException sEx){
			throw sEx;
		}
		finally{
			conn.close();
			rs.close();
			ps.close();
			rs = null;
			ps = null;
		}
		return columns;
	}

	public String getTABLE_CATALOG() {
		return TABLE_CATALOG;
	}




	public void setTABLE_CATALOG(String table_catalog) {
		TABLE_CATALOG = table_catalog;
	}




	public String getTABLE_SCHEMA() {
		return TABLE_SCHEMA;
	}




	public void setTABLE_SCHEMA(String table_schema) {
		TABLE_SCHEMA = table_schema;
	}




	public String getTABLE_NAME() {
		return TABLE_NAME;
	}




	public void setTABLE_NAME(String table_name) {
		TABLE_NAME = table_name;
	}




	public String getCOLUMN_NAME() {
		return COLUMN_NAME;
	}




	public void setCOLUMN_NAME(String column_name) {
		COLUMN_NAME = column_name;
	}






	public String getCOLUMN_DEFAULT() {
		return COLUMN_DEFAULT;
	}




	public void setCOLUMN_DEFAULT(String column_default) {
		COLUMN_DEFAULT = column_default;
	}




	public String getIS_NULLABLE() {
		return IS_NULLABLE;
	}




	public void setIS_NULLABLE(String is_nullable) {
		IS_NULLABLE = is_nullable;
	}




	public String getDATA_TYPE() {
		return DATA_TYPE;
	}




	public void setDATA_TYPE(String data_type) {
		DATA_TYPE = data_type;
	}



	public int getCHARACTER_OCTET_LENGTH() {
		return CHARACTER_OCTET_LENGTH;
	}




	public void setCHARACTER_OCTET_LENGTH(int character_octet_length) {
		CHARACTER_OCTET_LENGTH = character_octet_length;
	}


	public String getCHARACTER_SET_NAME() {
		return CHARACTER_SET_NAME;
	}




	public void setCHARACTER_SET_NAME(String character_set_name) {
		CHARACTER_SET_NAME = character_set_name;
	}




	public String getCOLLATION_NAME() {
		return COLLATION_NAME;
	}




	public void setCOLLATION_NAME(String collation_name) {
		COLLATION_NAME = collation_name;
	}




	public String getCOLUMN_TYPE() {
		return COLUMN_TYPE;
	}




	public void setCOLUMN_TYPE(String column_type) {
		COLUMN_TYPE = column_type;
	}




	public String getCOLUMN_KEY() {
		return COLUMN_KEY;
	}




	public void setCOLUMN_KEY(String column_key) {
		COLUMN_KEY = column_key;
	}




	public String getEXTRA() {
		return EXTRA;
	}




	public void setEXTRA(String extra) {
		EXTRA = extra;
	}




	public String getPRIVILEGES() {
		return PRIVILEGES;
	}




	public void setPRIVILEGES(String privileges) {
		PRIVILEGES = privileges;
	}




	public String getCOLUMN_COMMENT() {
		return COLUMN_COMMENT;
	}




	public void setCOLUMN_COMMENT(String column_comment) {
		COLUMN_COMMENT = column_comment;
	}


	public int getORDINAL_POSITION() {
		return ORDINAL_POSITION;
	}


	public void setORDINAL_POSITION(int ordinal_position) {
		ORDINAL_POSITION = ordinal_position;
	}


	public Integer getCHARACTER_MAXIMUM_LENGTH() {
		return CHARACTER_MAXIMUM_LENGTH;
	}


	public void setCHARACTER_MAXIMUM_LENGTH(Integer character_maximum_length) {
		CHARACTER_MAXIMUM_LENGTH = character_maximum_length;
	}


	public Integer getNUMERIC_PRECISION() {
		return NUMERIC_PRECISION;
	}


	public void setNUMERIC_PRECISION(Integer numeric_precision) {
		NUMERIC_PRECISION = numeric_precision;
	}


	public Integer getNUMERIC_SCALE() {
		return NUMERIC_SCALE;
	}


	public void setNUMERIC_SCALE(Integer numeric_scale) {
		NUMERIC_SCALE = numeric_scale;
	}


	public void setCHARACTER_OCTET_LENGTH(Integer character_octet_length) {
		CHARACTER_OCTET_LENGTH = character_octet_length;
	}


}
