/**
 * Copyright (C) 2009 
 * Developer: Fabrizio Moroni fabrizio.moroni@yahoo.it
 * Contributor: Carlo Curino	carlo@curino.us
 * 
 *  * This file is part of SchemEvolutionToolsuite
 * 
 * SchemEvolutionToolsuite is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SchemaEvolutionToolsuite is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with SchemEvolutionToolsuite.  If not, see <http://www.gnu.org/licenses/>.
 */

package edu.ucla.cs.dbevolution.DbBackend;

import java.io.File;
import java.io.IOException;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import edu.ucla.cs.Exceptions.RepositoryTypeUnknownException;
import edu.ucla.cs.dbevolution.ApplicationParameters;
import edu.ucla.cs.dbevolution.Query;
import edu.ucla.cs.dbevolution.RepositoryFactory;
import edu.ucla.cs.dbevolution.SqlScript;
import edu.ucla.cs.dbevolution.Utils;

public class DbOperations {

	/**
	 * This DropDbs method drops all database which are made by InstallDbs
	 * method
	 * 
	 * @throws SQLException
	 */
	public void DropDbs() throws SQLException {
		String sqlDropDbs = "DROP DATABASE IF EXISTS ";

		Connection conn = null;
		Statement stmt = null;

		try {
			conn = DbUtil.connect(ApplicationParameters.getDbevolution());
			conn.setAutoCommit(false);
			// drop each database in the evolution db dbs tables from the dbms
			for (DbVersion dbv : DbVersion.getDbInstalled()) {
				String db = dbv.getSchemaUniqueName();
				stmt = conn.createStatement();
				stmt.execute(sqlDropDbs + db);
			}
			conn.commit();
			System.out.println("All database droppedb");
		} catch (SQLException s) {
			try {
				conn.rollback();
				System.out
				.println("Unable to drop database: " + s.getMessage());
				throw s;
			} catch (SQLException se) {
			}
		} finally {
			// check if there are still some resources left open to mysql and
			// close everything
			closeStatement(stmt);
			closeConnection(conn);
		}
	}

	/**
	 * Collect the data from the databases installed and fill the evolution
	 * database Check if the table has been already filled before filling it
	 * 
	 * @throws SQLException
	 */
	public void collectEvolutionData() throws SQLException {

		String sqlInsertInto = "INSERT INTO ";
		String sqlSelect = "SELECT * FROM information_schema.";
		String sql = null;
		PreparedStatement pStmt = null;
		Connection conn = null;
		try {
			System.out.println("Starting Data collection.....");
			conn = DbUtil.connect(ApplicationParameters.getDbevolution());
			conn.setAutoCommit(false);
			/* foreach db installed and foreach table of the information schema,
			 * check if the corresponding evolution table has been filled, and
			 * if not fill it
			 */
			for (DbVersion dbv : DbVersion.getDbInstalled()) {

				String db = dbv.getSchemaUniqueName();

				for (String table : getEvolutionTables()) {
					int filled = checkDataCollected(db, table, conn);
					if (filled == 0) {

						System.out.println("TABLE " + table + " DB: " + db + "  NOT FILLED");

						// create the statement with the insert query
						sql = sqlInsertInto + ApplicationParameters.getHeaderEvolutionTables() + table;
						sql += "(";
						sql += sqlSelect;
						sql += table;
						sql += " WHERE ";
						sql += ApplicationParameters.getCollectionColumn(table);
						sql += " like ?)";

						System.out.println("Generated INSERT INTO statement: " + sql);

						pStmt = conn.prepareStatement(sql);
						pStmt.setString(1, db);
						pStmt.execute();
						conn.commit();

						pStmt.close();
						pStmt = null;
					} else {
						System.out.println("TABLE " + table + " DB: " + db + "  ALREADY FILLED");
					}
				}
			}
			conn.close();
			conn = null;
		} catch (SQLException sEx) {
			try {
				conn.rollback();
				System.out.println("An error occured during filling of the evolution database: "+ sEx.getMessage());
				throw sEx;
			} catch (SQLException e) {
			}
			sEx.printStackTrace();
		} finally {
			closePreparedStmt(pStmt);
			closeConnection(conn);
		}
	}



	/**
	 * Check if the data for the given db have already been collected
	 * 
	 * @param db the name of the database to check for
	 * @param table the name of the table to check for
	 * @return the number of rows of the given table
	 * @throws SQLException
	 */
	private int checkDataCollected(String db, String table,Connection conn) throws SQLException {

		/* SQL to check if the corresponding table of the evolution database has
		 *  been filled or not for the selected database
		 */
		String sqlCheckFilled = "SELECT COUNT(*) FROM ";
		PreparedStatement pStmt = null;
		ResultSet rs = null;

		int filled = 0;

		try {

			sqlCheckFilled += ApplicationParameters.getHeaderEvolutionTables()
			+ table + " WHERE "
			+ ApplicationParameters.getCollectionColumn(table) + "= ?";
			System.out.println("SQL CHECK FILLED statement: " + sqlCheckFilled);

			// connect to the evolution database
			pStmt = conn.prepareStatement(sqlCheckFilled);

			pStmt.setString(1, db);
			rs = pStmt.executeQuery();

			while (rs.next()) {
				filled = rs.getInt(1);
			}
		} catch (SQLException sEx) {
			System.out.println("An error occured during checking if the table into db evolution is already filled: "
					+ sEx.getMessage());
			throw sEx;
		} finally {
			closePreparedStmt(pStmt);
		}
		return filled;
	}




	/**
	 * Get the table of the information schema to collect the data from
	 * @return vector with the list of the tables of the information_schema
	 */
	private Vector<String> getEvolutionTables() {

		Vector<String> tables = new Vector<String>();

		Iterator it = ApplicationParameters.getCollectionTables().keySet().iterator();
		while (it.hasNext()) {
			// print all the keys of the statistics file
			String table = it.next().toString();
			tables.add(table);
		}
		return tables;
	}


	/**
	 * Install into the evolution DB all the revisions of the db analyzed
	 * 
	 * @throws SQLException
	 */
	public void DownloadRevisions() throws SQLException {

		String sql = "INSERT INTO " + ApplicationParameters.getEvolutionTable() + " (schema_name,svn_version,schema_unique_name,major_ordinal_version, minor_ordinal_version,tstart,tend) VALUES(?,?,?,?,?,?,?)";
		String sqlCheck = "SELECT * FROM " + ApplicationParameters.getEvolutionTable() + " WHERE schema_name = ? AND svn_version= ?";
		String sqlMajorVersion = "SELECT MAX(major_ordinal_version) FROM " + ApplicationParameters.getEvolutionTable() + " WHERE schema_name like ? ";

		Vector<DbVersion> revisions = null;

		Connection conn = null;
		PreparedStatement pStmt = null;
		PreparedStatement pStmtCheck = null;
		PreparedStatement pStmtMajor = null;
		ResultSet rs = null;
		ResultSet rsMajor = null;
		int maxMajorVersion = 0;
		try{
			revisions = RepositoryFactory.createRepository().GetRevisions();
		}
		catch(RepositoryTypeUnknownException rEx){
			System.out.println("Unable to create the repository:");
			System.out.println(rEx.getMessage());
			System.exit(-1);
		}

		try {

			//createDriver();
			conn = DbUtil.connect(ApplicationParameters.getDbevolution());

			pStmt = conn.prepareStatement(sql);
			pStmtCheck = conn.prepareStatement(sqlCheck);
			pStmtMajor = conn.prepareStatement(sqlMajorVersion);

			pStmtMajor.setString(1, ApplicationParameters.getDbbasename()+"%");
			rsMajor = pStmtMajor.executeQuery();

			while (rsMajor.next()) {
				maxMajorVersion = rsMajor.getInt(1);
			}

			if(ApplicationParameters.getRepoType().equals("cvs")){
				maxMajorVersion +=  revisions.size();
			}

			// disabel the auto-commit property of the connection, I'll call the
			// commit method after all the operations have been executed on the DB
			conn.setAutoCommit(false);

			// execute all the insert query;
			// if one fails the transaction is rolled back
			for (int i = 0; i < revisions.size(); i++) {

				// check if there is already a tuple in the VERSIONS table with the same name values
				pStmtCheck.setString(1, revisions.get(i).getSchemaName());
				pStmtCheck.setString(2, revisions.get(i).getSvnVersion());
				rs = pStmtCheck.executeQuery();

				// the result set is empty
				// add the new revision to the databases table
				if (!rs.first()) {
					pStmt.setString(1, revisions.get(i).getSchemaName());
					pStmt.setString(2, revisions.get(i).getSvnVersion());
					pStmt.setString(3, revisions.get(i).getSchemaName()+ Utils.cleanRevisionNumber(revisions.get(i).getSvnVersion()));
					if(ApplicationParameters.getRepoType().equals("cvs")){
						pStmt.setInt(4, maxMajorVersion--);
					}
					else{
						pStmt.setInt(4, ++maxMajorVersion);
					}
					pStmt.setInt(5, 0);
					pStmt.setTimestamp(6, new java.sql.Timestamp(revisions.get(i).getTStart().getTime()));
					pStmt.setTimestamp(7, new java.sql.Timestamp(revisions.get(i).getTEnd().getTime()));
					pStmt.execute();
					System.out.println("DB:" + revisions.get(i).getSchemaName()+ revisions.get(i).getSvnVersion() + "INSERTED");
				}
			}
			conn.commit();
		} catch (SQLException e) {
			try {
				conn.rollback();
			} catch (SQLException error) {
			}
			throw e;
		} finally {
			closePreparedStmt(pStmt);
			closePreparedStmt(pStmtCheck);
			closeResultSet(rs);
			closeResultSet(rsMajor);
			closeConnection(conn);
		}

	}

	/**
	 * Install all the revision of the schema
	 */
	public void InstallDbs() throws SQLException {

		String pathToSchema = ApplicationParameters.getPathtoschema();
		String sqlCreateDb = "CREATE DATABASE IF NOT EXISTS ";
		String sqlCheckIS = "SELECT COUNT(*) FROM SCHEMATA WHERE SCHEMA_NAME = ?";
		String sqlStateUpd = "UPDATE " + ApplicationParameters.getEvolutionTable() + " SET state = 1 WHERE schema_unique_name =?";
		String sqlMysqlErr = "UPDATE " + ApplicationParameters.getEvolutionTable() + " SET error_code = ?, error_msg = ? WHERE schema_unique_name=?";
		String sqlExt = ".sql";

		PreparedStatement pStmt = null;
		Statement stmt = null;
		Connection conn = null;
		Connection connS = null;

		int rowCount = 0;

		//GET ALL THE DBS TO INSTALL FROM THE EVOLUTION DB
		List<DbVersion> dbs = DbVersion.getDownloadedDatabases();
		for(DbVersion db: dbs){
			String version = db.getSvnVersion();
			String name = db.getSchemaUniqueName();
			int state = db.getState();

			/* 
			 * FIRST CHECK
			 * the state's flag is equal to 0, the database is not installed
			 */
			if (state == 0) {
				/* 
				 * SECOND CHECK
				 * check the database name against the information schema
				 */
				connS = DbUtil.connect("information_schema");
				pStmt = connS.prepareStatement(sqlCheckIS);
				pStmt.setString(1, db.getSchemaUniqueName());
				ResultSet rsCount = pStmt.executeQuery();
				while (rsCount.next()) {
					rowCount = rsCount.getInt(1);
				}
				rsCount.close();
				connS.close();

				if (rowCount == 0) {
					/* THE DB MUST BE INSTALLED
					 * CHECK IF THE SCHEMA IS AVAILABLE IF NOT AVAILABLE, DOWNLOAD IT
					 */
					File f = new File(pathToSchema + version + sqlExt);
					String fdir = ApplicationParameters.getPathtoschema();
					Utils.checkCreateDir(fdir);
					System.out.println("Checking the schema: "+ f.getAbsolutePath() + f.getName());
					if (!f.exists()) {
						try{
							RepositoryFactory.createRepository().DownloadRevision(version,f);
						}
						catch(RepositoryTypeUnknownException rEx){
							System.out.println("Unable to create the repository:");
							System.out.println(rEx.getMessage());
							System.exit(-1);
						}
						System.out.println("Schema: " + version + " downloaded");
					}
					Vector<Query> queries = new Vector<Query>();
					SqlScript script = new SqlScript((pathToSchema
							+ version + sqlExt));
					try {
						queries = script.loadScript();
					} catch (IOException e) {
						System.out
						.println("Error whil loading the schema: "
								+ e.getMessage());
						continue;
					}

					// INSTALL THE DB
					try {
						connS = DbUtil.connect("information_schema");
						stmt = connS.createStatement();
						stmt.executeUpdate(sqlCreateDb + name);
						connS.close();
						System.out.println("THE DB " + name
								+ " WAS SUCCESSFULLY CREATED");
						//the database has been successfully created, install the tables from the script file
						try {
							connS = DbUtil.connect(name);

							stmt = connS.createStatement();
							stmt.addBatch("SET FOREIGN_KEY_CHECKS=0;");
							for (int i = 0; i < queries.size(); i++) {
								// System.out.println(queries.get(i).getQuerytext());
								stmt.addBatch(queries.get(i).getQuerytext());
							}
							stmt.addBatch("SET FOREIGN_KEY_CHECKS=1;");
							stmt.executeBatch();
							connS.close();
							try {
								//IF no errors occurred, update the state's flag into the VERSIONS table
								connS = DbUtil.connect(ApplicationParameters
										.getDbevolution());
								connS.setAutoCommit(false);
								pStmt = connS.prepareStatement(sqlStateUpd);
								pStmt.setString(1, name);
								pStmt.executeUpdate();
								connS.commit();
							} catch (SQLException e) {
								connS.rollback();
								System.out
								.println("AN ERROR OCCUPRED DURING THE UPDATE OF THE STATUS FLAG: "
										+ name
										+ " MESSAGE: "
										+ e.getMessage());
							}
						} catch (SQLException se) {
							System.out
							.println("AN ERROR IS OCCURED DURING THE INSTALLATION OF THE SCHEMA: "
									+ name
									+ " MESSAGE: "
									+ se.getMessage());
							try {
								//UPDATE THE ERROR_CODE AND ERROR_MSG USING THE SERVER STATUS'S INFORMATIONS
								connS = DbUtil.connect(ApplicationParameters
										.getDbevolution());
								connS.setAutoCommit(false);
								pStmt = connS.prepareStatement(sqlMysqlErr);
								pStmt.setString(1, Integer.toString(se
										.getErrorCode()));
								// the field is unsed at the moment due to
								// space problem
								pStmt.setString(2, "");
								pStmt.setString(3, name);
								pStmt.executeUpdate();
								connS.commit();
								connS.close();
							} catch (SQLException se2) {
								throw se2;
							}
						}

					} catch (SQLException e) {
						System.out
						.println("ERROR DURING THE CREATION OF THE DB: "
								+ name
								+ " MESSAGE: "
								+ e.getMessage());
					}
					finally{
						closeStatement(stmt);
						closePreparedStmt(pStmt);
						closeConnection(connS);
						closeConnection(conn);
					}
				} else {
					System.out.println("DB: " + name + " IS ALREADY INSTALLED");
				}
			} else {
				System.out.println("DB: " + name + " IS ALREADY INSTALLED");
			}
		}
	}




	/**
	 * Close a ResulSet object
	 * @param rs
	 */
	private void closeResultSet(ResultSet rs){
		if (rs != null) {
			try {
				rs.close();
			} catch (SQLException e) {
			}
		}
	}

	/**
	 * Close a Connection object
	 * @param conn
	 */
	private void closeConnection(Connection conn){
		if (conn != null) {
			try {
				conn.close();
			}
			catch (SQLException e) {
			}
		}
	}

	/**
	 * Close a PreparedStatement object
	 * @param pStmt
	 */
	private void closePreparedStmt(PreparedStatement pStmt){
		if (pStmt != null) {
			try {
				pStmt.close();
			} catch (SQLException sqlex) {
			}
			pStmt = null;
		}
	}

	/**
	 * Close a Statement object
	 * @param stmt
	 */
	private void closeStatement(Statement stmt){
		if (stmt != null) {
			try {
				stmt.close();
			} catch (SQLException sqlex) {
			}
			stmt = null;
		}
	}


	/**
	 * Generic method to execute SQL statements
	 * @param sql
	 * @param params
	 * @return
	 * @throws SQLException
	 */
	public Vector<String[]> executeStatement(String sql, String[] params) throws SQLException{

		Connection conn = null;
		PreparedStatement pStmt = null;
		ResultSet rs = null;
		int colonne = 0;
		String [] record;

		Vector<String[]> result;

		try {
			conn = DbUtil.connect(ApplicationParameters.getDbevolution());
			pStmt = conn.prepareStatement(sql);

			for(int i=1;i<=params.length; i++){
				pStmt.setString(i, params[i-1]);
			}

			rs = pStmt.executeQuery();
			result = new Vector<String[]>();
			ResultSetMetaData rsmd = rs.getMetaData();
			colonne = rsmd.getColumnCount();

			while(rs.next()) {   // Creo il vettore risultato scorrendo tutto il ResultSet
				record = new String[colonne];
				for (int i=0; i<colonne; i++){ 
					record[i] = rs.getString(i+1);
				}
				result.add( (String[]) record.clone() );
			}
			closeConnection(conn);
			closePreparedStmt(pStmt);
		} 
		catch (SQLException sEx) {
			System.out.println("An error is occured during executing statistic: "+ sEx.getMessage());
			throw sEx;
		} finally {
			closeResultSet(rs);
			closeConnection(conn);
			closePreparedStmt(pStmt);
		}
		return result;
	}
}
