package edu.ucla.cs.dbevolution.DbBackend;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;

import edu.ucla.cs.dbevolution.ApplicationParameters;

public class DbStatistics {


	private String TABLE_CATALOG;
	private String TABLE_SCHEMA;
	private String TABLE_NAME;
	private int NON_UNIQUE;
	private String INDEX_SCHEMA;
	private String INDEX_NAME;
	private int SEQ_IN_INDEX;
	private String COLUMN_NAME;
	private String COLLATION;
	private int CARDINALITY;
	private int SUB_PART;
	private String PACKED;
	private String NULLABLE;
	private String INDEX_TYPE;
	private String COMMENT;


	/**
	 * Get the name of the columns composing the index for all the revision of a database
	 * @param dbName
	 * @param tableName
	 * @param indexName
	 * @return
	 * @throws SQLException
	 */
	public static LinkedHashMap<DbVersion, List<DbColumn>> getColumnsInIndex(String dbName,String tableName,String indexName) throws SQLException {


		String sql = "	SELECT table_schema, column_name " +
		"		FROM " + ApplicationParameters.getHeaderEvolutionTables() + "STATISTICS AS s " +
		"		WHERE 	s.table_schema LIKE ? AND " +
		"				s.table_name like ? AND " +
		"				s.index_name like ?";


		LinkedHashMap<DbVersion, List<DbColumn>> columnsInIndex = new LinkedHashMap<DbVersion, List<DbColumn>>();
		ArrayList<DbColumn> dbColumns = null;
		DbColumn dbColumn = null;
		DbVersion db = null;

		PreparedStatement pStmt = null;
		ResultSet rs = null;
		Connection conn = null;

		String tempDbVersion;
		String tempColumnName;
		String currentColumnName = null;
		String currentDbVersion = null;

		try {
			conn = DbUtil.connect(ApplicationParameters.getDbevolution());
			pStmt = conn.prepareStatement(sql);
			pStmt.setString(1,dbName);
			pStmt.setString(2,tableName);
			pStmt.setString(3,indexName);
			rs = pStmt.executeQuery();

			if(rs!= null) {
				if(rs.next()){
					tempDbVersion = rs.getString(1);
					tempColumnName = rs.getString(2);

					dbColumn = new DbColumn();
					dbColumn.setCOLUMN_NAME(tempColumnName);
					dbColumns = new ArrayList<DbColumn>();
					dbColumns.add(dbColumn);

					while(rs.next()){
						currentDbVersion = rs.getString(1);
						currentColumnName = rs.getString(2);
						if(currentDbVersion.equals(tempDbVersion)){
							dbColumn = new DbColumn();
							dbColumn.setCOLUMN_NAME(currentColumnName);
							dbColumns.add(dbColumn);
						}
						else{
							db = new DbVersion();
							db.setSchemaName(tempDbVersion);
							columnsInIndex.put(db, dbColumns);

							//update the temporary schema name
							tempDbVersion = currentDbVersion;

							dbColumn = new DbColumn();
							dbColumn.setCOLUMN_NAME(currentColumnName);
							dbColumns = new ArrayList<DbColumn>();
							dbColumns.add(dbColumn);
						}
					}
					//add the last entry
					db = new DbVersion();
					db.setSchemaName(currentDbVersion);
					columnsInIndex.put(db, dbColumns);
				}
			}
			conn.close();
		}
		catch(SQLException sEx){
			sEx.printStackTrace();
			throw sEx;
		}
		return columnsInIndex;
	}


	public String getTABLE_CATALOG() {
		return TABLE_CATALOG;
	}
	public void setTABLE_CATALOG(String table_catalog) {
		TABLE_CATALOG = table_catalog;
	}
	public String getTABLE_SCHEMA() {
		return TABLE_SCHEMA;
	}
	public void setTABLE_SCHEMA(String table_schema) {
		TABLE_SCHEMA = table_schema;
	}
	public String getTABLE_NAME() {
		return TABLE_NAME;
	}
	public void setTABLE_NAME(String table_name) {
		TABLE_NAME = table_name;
	}
	public int getNON_UNIQUE() {
		return NON_UNIQUE;
	}
	public void setNON_UNIQUE(int non_unique) {
		NON_UNIQUE = non_unique;
	}
	public String getINDEX_SCHEMA() {
		return INDEX_SCHEMA;
	}
	public void setINDEX_SCHEMA(String index_schema) {
		INDEX_SCHEMA = index_schema;
	}
	public String getINDEX_NAME() {
		return INDEX_NAME;
	}
	public void setINDEX_NAME(String index_name) {
		INDEX_NAME = index_name;
	}
	public int getSEQ_IN_INDEX() {
		return SEQ_IN_INDEX;
	}
	public void setSEQ_IN_INDEX(int seq_in_index) {
		SEQ_IN_INDEX = seq_in_index;
	}
	public String getCOLUMN_NAME() {
		return COLUMN_NAME;
	}
	public void setCOLUMN_NAME(String column_name) {
		COLUMN_NAME = column_name;
	}
	public String getCOLLATION() {
		return COLLATION;
	}
	public void setCOLLATION(String collation) {
		COLLATION = collation;
	}
	public int getCARDINALITY() {
		return CARDINALITY;
	}
	public void setCARDINALITY(int cardinality) {
		CARDINALITY = cardinality;
	}
	public int getSUB_PART() {
		return SUB_PART;
	}
	public void setSUB_PART(int sub_part) {
		SUB_PART = sub_part;
	}
	public String getPACKED() {
		return PACKED;
	}
	public void setPACKED(String packed) {
		PACKED = packed;
	}
	public String getNULLABLE() {
		return NULLABLE;
	}
	public void setNULLABLE(String nullable) {
		NULLABLE = nullable;
	}
	public String getINDEX_TYPE() {
		return INDEX_TYPE;
	}
	public void setINDEX_TYPE(String index_type) {
		INDEX_TYPE = index_type;
	}
	public String getCOMMENT() {
		return COMMENT;
	}
	public void setCOMMENT(String comment) {
		COMMENT = comment;
	}



}
