package edu.ucla.cs.dbevolution.DbBackend;

import java.sql.Connection;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;

import net.sf.jsqlparser.schema.Column;
import net.sf.jsqlparser.schema.Table;
import edu.ucla.cs.dbevolution.ApplicationParameters;

public class DbTable {


	private String TABLE_CATALOG;
	private String TABLE_SCHEMA;
	private String TABLE_NAME;
	private String TABLE_TYPE;
	private String ENGINE;
	private int VERSION;
	private String ROW_FORMAT;
	private int TABLE_ROWS;
	private int AVG_ROW_LENGTH;
	private int DATA_LENGTH;
	private int MAX_DATA_LENGTH;
	private int INDEX_LENGTH;
	private int DATA_FREE;
	private int AUTO_INCREMENT;
	private Date CREATE_TIME;
	private Date UPDATE_TIME;
	private Date CHECK_TIME;
	private String TABLE_COLLATION;
	private int CHECKSUM;
	private String CREATE_OPTIONS;
	private String TABLE_COMMENT;



	/**
	 * Get the difference between the tables of two different revisions
	 * @param first the first revision to compare
	 * @param second the second revision to compare
	 * @return the number of tables different between the two revisions
	 */
	public static int getDifference(ArrayList<DbTable> first, ArrayList<DbTable> second){
		
		int change=0;
		boolean exist = false;
		
		for(int j=0;j<first.size();j++){
			String tableName = ((DbTable)first.get(j)).getTABLE_NAME(); 
			for(int i=0; i<second.size();i++){
				String table2Name = ((DbTable)second.get(i)).getTABLE_NAME();
				if(tableName.equals(table2Name)){
					exist = true;
				}
			}
			if(!exist){
				change++;
			}
		}
		return change;
	}
	

	/**
	 * TO BE REPLACED BY THE IBATIS VERSION
	 * Get the name of the columns
	 * @param dbName the database name
	 * @param tableName the table name
	 * @return column's name for each revision of the table
	 * @throws SQLException
	 */
	public static LinkedHashMap<DbVersion, List<DbColumn>>  getColumnsInTable(String dbName,String tableName) throws SQLException {

		String sql = "	SELECT table_schema, column_name " +
		"		FROM " + ApplicationParameters.getHeaderEvolutionTables() + "COLUMNS c, " + ApplicationParameters.getEvolutionTable() + " d " +
		"		WHERE 	d.schema_unique_name = c.table_schema AND " +
		"				table_schema like ? AND table_name like ? " +
		"		ORDER BY d.tstart";

		LinkedHashMap<DbVersion, List<DbColumn>> columnsInTable = new LinkedHashMap<DbVersion, List<DbColumn>>();
		ArrayList<DbColumn> dbColumns = null;
		DbColumn dbColumn = null;
		DbVersion db = null;

		PreparedStatement pStmt = null;
		ResultSet rs = null;
		Connection conn = null;

		String tempDbVersion;
		String tempColumnName;
		String currentColumnName = null;
		String currentDbVersion = null;

		try {
			conn = DbUtil.connect(ApplicationParameters.getDbevolution());
			pStmt = conn.prepareStatement(sql);
			pStmt.setString(1,dbName);
			pStmt.setString(2,tableName);
			rs = pStmt.executeQuery();

			if(rs!= null){
				rs.next();
				tempDbVersion = rs.getString(1);
				tempColumnName = rs.getString(2);

				dbColumn = new DbColumn();
				dbColumn.setCOLUMN_NAME(tempColumnName);
				dbColumns = new ArrayList<DbColumn>();
				dbColumns.add(dbColumn);

				while(rs.next()){
					currentDbVersion = rs.getString(1);
					currentColumnName = rs.getString(2);
					if(currentDbVersion.equals(tempDbVersion)){
						dbColumn = new DbColumn();
						dbColumn.setCOLUMN_NAME(currentColumnName);
						dbColumns.add(dbColumn);
					}
					else{
						db = new DbVersion();
						db.setSchemaName(tempDbVersion);
						columnsInTable.put(db, dbColumns);

						//update the temporary schema name
						tempDbVersion = currentDbVersion;

						dbColumn = new DbColumn();
						dbColumn.setCOLUMN_NAME(currentColumnName);
						dbColumns = new ArrayList<DbColumn>();
						dbColumns.add(dbColumn);
					}
				}
				//add the last entry
				db = new DbVersion();
				db.setSchemaName(currentDbVersion);
				columnsInTable.put(db, dbColumns);
			}
			conn.close();

		}
		catch(SQLException sEx){
			sEx.printStackTrace();
			throw sEx;
		}
		return columnsInTable;
	}


	/**
	 * Get the table the given column belongs to
	 * The search is performed over a set of columns. It is assumed the column belongs only to one table, no name clash
	 * @param tables
	 * @param columnPartitioned
	 * @return the name of the table
	 * @throws SQLException 
	 */
	public static String getTableNameFromColumnAndRevision(ArrayList<Table> tables, Column columnPartitioned) throws SQLException {
		
		Connection conn = null;
		PreparedStatement pStmt = null;
		ResultSet rs = null;

		String tableName = null;
		
		String tablesList = "(";
		
		int countTables = 0;
		
		if(tables.size()>1){
			
			tablesList += "\"" + tables.get(countTables).getName() + "\"";
			for(int count = countTables +1; count < tables.size();  count++){
				tablesList += ",";
				tablesList += "\"" + tables.get(count).getName() + "\"";
			}
		}
		else{
			tablesList += tables.get(countTables).getName();
		}
		tablesList += ")";

		
		String sql = "	SELECT	table_name" +
				"		FROM	PRISM_COLUMNS" +
				"		WHERE	table_schema = ? AND column_name = ? AND" +
				"				table_name IN " + tablesList;
				
		try{
			conn = DbUtil.connect(ApplicationParameters.getDbevolution());
			pStmt = conn.prepareStatement(sql);
			pStmt.setString(1,ApplicationParameters.getBaseRevision());
			pStmt.setString(2,columnPartitioned.getColumnName());
			rs = pStmt.executeQuery();

			if(rs!= null){
				//only one row for this query
				rs.first();
				tableName = rs.getString(1);
			}
			conn.close();
		}
		catch(SQLException sEx){
			sEx.printStackTrace();
			throw sEx;
		}
		return tableName;
	}
	
	public boolean hasSameSignature(Table table2){
		ArrayList<Table> tables1 = new ArrayList<Table>();
		tables1.add(new Table(null, this.getTABLE_NAME()));
		ArrayList<Table> tables2 = new ArrayList<Table>();
		tables2.add(table2);
		
		ArrayList<DbColumn> columnsTable1 = DbColumn.getColumnsFromTables(tables1);
		ArrayList<DbColumn> columnsTable2 = DbColumn.getColumnsFromTables(tables2);
		
		if (columnsTable1.size() != columnsTable2.size()){
			return false;
		}
		for(int i=0; i<columnsTable1.size(); i++){
			if((!columnsTable1.get(i).getCOLUMN_NAME().equalsIgnoreCase(columnsTable2.get(i).getCOLUMN_NAME())) || !columnsTable1.get(i).getCOLUMN_TYPE().equalsIgnoreCase(columnsTable2.get(i).getCOLUMN_TYPE()))
				return false;
		}
		return true;
	}
	
	
	public boolean hasSameSignature(DbTable table2){
		Table table = new Table();
		table.setName(table2.getTABLE_NAME());
		return hasSameSignature(table);
	}
	
	
	/**
	 * @param c1
	 * @param c2
	 * @return
	 */
	public static boolean compareTableStructure(ArrayList<DbColumn> c1, ArrayList<DbColumn> c2){
		
		if(compareColumnsCount(c1, c2) && compareColumnsName(c1, c2)){
			return true;
		}
		return  false;
		
	}
	
	/**
	 * @param c1
	 * @param c2
	 * @return
	 */
	private static boolean compareColumnsCount(ArrayList<DbColumn> c1, ArrayList<DbColumn> c2){
		if(c1.size() == c2.size()){
			return true;
		}
		return  false;
		
	}
	
	
	/**
	 * @param c1
	 * @param c2
	 * @return
	 */
	private static boolean compareColumnsName(ArrayList<DbColumn> c1, ArrayList<DbColumn> c2){
		
		if(DbColumn.compareColumnsName(c1, c2) ==0 && DbColumn.compareColumnsName(c2, c1)==0){
			return false;
		}
		return  true;
	}

	/**
	 * Get the list of tables in the given revision  
	 * @param revision
	 * @param revisionMinor
	 * @param schemaName
	 * @return
	 * @throws SQLException
	 */
	public static List<DbTable> getTableByVersionNumber(int revision, int revisionMinor, String schemaName) throws SQLException{
		
		PreparedStatement ps = null;
		ResultSet rs = null;
		Connection conn = null;
		
		HashMap<String, Object> params = new HashMap<String,Object>();
		
		params.put("version", revision );
		params.put("versionMinor", revisionMinor );
		params.put("schemaName", schemaName);

		String sql = "	SELECT	table_name" +
				"		FROM 	`TABLES` t" +
				"		WHERE  	t.schema_version = ?" +
				"		AND		t.schema_version_minor = ?" +
				"		AND		t.table_schema = ?";
		
		ArrayList<DbTable> tables = null; 
		
		try{
			conn = DbUtil.connect(ApplicationParameters.getDbevolution());
			ps = conn.prepareStatement(sql);
			ps.setInt(1, ((Integer)params.get("version")).intValue());
			ps.setInt(2,((Integer)params.get("versionMinor")).intValue());
			ps.setString(3,(String)params.get("schemaName"));
			rs = ps.executeQuery();
			
			tables = new ArrayList<DbTable>();
			
			while(rs.next()){
				DbTable table = new DbTable();
				table.setTABLE_NAME(rs.getString(1));
				tables.add(table);
			}
		}
		catch(SQLException sEx){
			throw sEx;
		}
		finally{
			conn.close();
			rs.close();
			ps.close();
			rs = null;
			ps = null;
		}
		return tables;
	}
	

	public String getTABLE_CATALOG() {
		return TABLE_CATALOG;
	}



	public void setTABLE_CATALOG(String table_catalog) {
		TABLE_CATALOG = table_catalog;
	}



	public String getTABLE_SCHEMA() {
		return TABLE_SCHEMA;
	}



	public void setTABLE_SCHEMA(String table_schema) {
		TABLE_SCHEMA = table_schema;
	}



	public String getTABLE_NAME() {
		return TABLE_NAME;
	}



	public void setTABLE_NAME(String table_name) {
		TABLE_NAME = table_name;
	}



	public String getTABLE_TYPE() {
		return TABLE_TYPE;
	}



	public void setTABLE_TYPE(String table_type) {
		TABLE_TYPE = table_type;
	}



	public String getENGINE() {
		return ENGINE;
	}



	public void setENGINE(String engine) {
		ENGINE = engine;
	}



	public int getVERSION() {
		return VERSION;
	}



	public void setVERSION(int version) {
		VERSION = version;
	}



	public String getROW_FORMAT() {
		return ROW_FORMAT;
	}



	public void setROW_FORMAT(String row_format) {
		ROW_FORMAT = row_format;
	}



	public int getTABLE_ROWS() {
		return TABLE_ROWS;
	}



	public void setTABLE_ROWS(int table_rows) {
		TABLE_ROWS = table_rows;
	}



	public int getAVG_ROW_LENGTH() {
		return AVG_ROW_LENGTH;
	}



	public void setAVG_ROW_LENGTH(int avg_row_length) {
		AVG_ROW_LENGTH = avg_row_length;
	}



	public int getDATA_LENGTH() {
		return DATA_LENGTH;
	}



	public void setDATA_LENGTH(int data_length) {
		DATA_LENGTH = data_length;
	}



	public int getMAX_DATA_LENGTH() {
		return MAX_DATA_LENGTH;
	}



	public void setMAX_DATA_LENGTH(int max_data_length) {
		MAX_DATA_LENGTH = max_data_length;
	}



	public int getINDEX_LENGTH() {
		return INDEX_LENGTH;
	}



	public void setINDEX_LENGTH(int index_length) {
		INDEX_LENGTH = index_length;
	}



	public int getDATA_FREE() {
		return DATA_FREE;
	}



	public void setDATA_FREE(int data_free) {
		DATA_FREE = data_free;
	}



	public int getAUTO_INCREMENT() {
		return AUTO_INCREMENT;
	}



	public void setAUTO_INCREMENT(int auto_increment) {
		AUTO_INCREMENT = auto_increment;
	}



	public Date getCREATE_TIME() {
		return CREATE_TIME;
	}



	public void setCREATE_TIME(Date create_time) {
		CREATE_TIME = create_time;
	}



	public Date getUPDATE_TIME() {
		return UPDATE_TIME;
	}



	public void setUPDATE_TIME(Date update_time) {
		UPDATE_TIME = update_time;
	}



	public Date getCHECK_TIME() {
		return CHECK_TIME;
	}



	public void setCHECK_TIME(Date check_time) {
		CHECK_TIME = check_time;
	}



	public String getTABLE_COLLATION() {
		return TABLE_COLLATION;
	}



	public void setTABLE_COLLATION(String table_collation) {
		TABLE_COLLATION = table_collation;
	}



	public int getCHECKSUM() {
		return CHECKSUM;
	}



	public void setCHECKSUM(int checksum) {
		CHECKSUM = checksum;
	}



	public String getCREATE_OPTIONS() {
		return CREATE_OPTIONS;
	}



	public void setCREATE_OPTIONS(String create_options) {
		CREATE_OPTIONS = create_options;
	}



	public String getTABLE_COMMENT() {
		return TABLE_COMMENT;
	}



	public void setTABLE_COMMENT(String table_comment) {
		TABLE_COMMENT = table_comment;
	}
}
