package edu.ucla.cs.dbevolution.DbBackend;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import edu.ucla.cs.dbevolution.ApplicationParameters;

public class DbTableConstraints {


	private String CONSTRAINT_CATALOG;
	private String CONSTRAINT_SCHEMA;
	private String CONSTRAINT_NAME;
	private String TABLE_SCHEMA;
	private String TABLE_NAME;
	private String CONSTRAINT_TYPE;


	/**
	 * Returns the tables with a constraint
	 */  
	public static ArrayList<DbTableConstraints> getTablesWithKeys(String db) throws SQLException {

		DbTableConstraints table = null;
		ArrayList<DbTableConstraints> tables = new ArrayList<DbTableConstraints>();
		String sql = "	SELECT DISTINCT table_name " +
		"		FROM " + ApplicationParameters.getHeaderEvolutionTables() + "TABLE_CONSTRAINTS " +
		"		WHERE 	table_schema like ? AND " +
		"				constraint_name like 'PRIMARY'";

		PreparedStatement pStmt = null;
		ResultSet rs = null;
		Connection conn = null;

		try {
			conn = DbUtil.connect(ApplicationParameters.getDbevolution());
			pStmt = conn.prepareStatement(sql);
			pStmt.setString(1,db);
			rs = pStmt.executeQuery();

			if(rs!= null){
				while(rs.next()){
					table = new DbTableConstraints();
					table.setTABLE_NAME(rs.getString(1));
					tables.add(table);
				}
			}
			conn.close();
		} catch (SQLException e) {
			e.printStackTrace();
		}

		return tables;
	}

	/**
	 * Return the tables with a foreign key
	 * @param db
	 * @return
	 * @throws SQLException
	 */
	public static HashMap<DbTable, List<DbTableConstraints>> getTablesWithForeignKeys(String db) throws SQLException {

		String sql = "	SELECT table_name, constraint_name " +
		"		FROM " + ApplicationParameters.getHeaderEvolutionTables() + "TABLE_CONSTRAINTS " +
		"		WHERE 	table_schema like ? AND " +
		"				constraint_type like 'FOREIGN%' " +
		"		ORDER BY constraint_schema";

		HashMap<DbTable, List<DbTableConstraints>> indexInTable = new HashMap<DbTable, List<DbTableConstraints>>();
		ArrayList<DbTableConstraints> dbStats = null;
		DbTable dbTable = null;
		DbTableConstraints dbStat = null;

		PreparedStatement pStmt = null;
		ResultSet rs = null;
		Connection conn = null;

		String tempDbTableName;
		String tempIndexName;
		String currentDbTableName = null;
		String currentIndexName = null;

		try {
			conn = DbUtil.connect(ApplicationParameters.getDbevolution());
			pStmt = conn.prepareStatement(sql);
			pStmt.setString(1,ApplicationParameters.getDbbasename() + "%");
			rs = pStmt.executeQuery();

			if(rs!= null){
				rs.next();
				tempDbTableName = rs.getString(1);
				tempIndexName = rs.getString(2);

				dbStat = new DbTableConstraints();
				dbStat.setCONSTRAINT_NAME(tempIndexName);
				dbStats = new ArrayList<DbTableConstraints>();
				dbStats.add(dbStat);

				while(rs.next()){
					currentDbTableName = rs.getString(1);
					currentIndexName = rs.getString(2);
					if(currentDbTableName.equals(tempDbTableName)){
						dbStat = new DbTableConstraints();
						dbStat.setCONSTRAINT_NAME(currentIndexName);
						dbStats.add(dbStat);
					}
					else{
						dbTable = new DbTable();
						dbTable.setTABLE_NAME(tempDbTableName);
						indexInTable.put(dbTable, dbStats);

						//update the temporary schema name
						tempDbTableName = currentDbTableName;

						dbStat = new DbTableConstraints();
						dbStat.setCONSTRAINT_NAME(currentIndexName);
						dbStats = new ArrayList<DbTableConstraints>();
						dbStats.add(dbStat);
					}
				}
				//add the last entry
				dbTable = new DbTable();
				dbTable.setTABLE_NAME(currentDbTableName);
				indexInTable.put(dbTable, dbStats);
			}
			conn.close();
		}
		catch(SQLException sEx){
			sEx.printStackTrace();
			throw sEx;
		}
		return indexInTable;

	}


	public String getCONSTRAINT_CATALOG() {
		return CONSTRAINT_CATALOG;
	}
	public void setCONSTRAINT_CATALOG(String constraint_catalog) {
		CONSTRAINT_CATALOG = constraint_catalog;
	}
	public String getCONSTRAINT_SCHEMA() {
		return CONSTRAINT_SCHEMA;
	}
	public void setCONSTRAINT_SCHEMA(String constraint_schema) {
		CONSTRAINT_SCHEMA = constraint_schema;
	}
	public String getCONSTRAINT_NAME() {
		return CONSTRAINT_NAME;
	}
	public void setCONSTRAINT_NAME(String constraint_name) {
		CONSTRAINT_NAME = constraint_name;
	}
	public String getTABLE_SCHEMA() {
		return TABLE_SCHEMA;
	}
	public void setTABLE_SCHEMA(String table_schema) {
		TABLE_SCHEMA = table_schema;
	}
	public String getTABLE_NAME() {
		return TABLE_NAME;
	}
	public void setTABLE_NAME(String table_name) {
		TABLE_NAME = table_name;
	}
	public String getCONSTRAINT_TYPE() {
		return CONSTRAINT_TYPE;
	}
	public void setCONSTRAINT_TYPE(String constraint_type) {
		CONSTRAINT_TYPE = constraint_type;
	} 	

}
