/**
 * Copyright (C) 2009 
 * Developer: Fabrizio Moroni fabrizio.moroni@yahoo.it
 * Contributor: Carlo Curino	carlo@curino.us
 * 
 *  * This file is part of SchemEvolutionToolsuite
 * 
 * SchemEvolutionToolsuite is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SchemaEvolutionToolsuite is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with SchemEvolutionToolsuite.  If not, see <http://www.gnu.org/licenses/>.
 */

package edu.ucla.cs.dbevolution;

import org.apache.log4j.Logger;

import edu.ucla.cs.Exceptions.UndefinedApplicationParameterException;
import edu.ucla.cs.dbevolution.DbBackend.DbOperations;
import edu.ucla.cs.statistics.StatisticsGenerator;

/**
 * This is the menu of the application you can use to control all the functionality provided
 * Read the command line parameters and set the application parameter
 * @author fangar
 */
public class SchemaEvolutionToolsuite {

	private static Logger logger = Logger.getLogger(SchemaEvolutionToolsuite.class);
	
	public static void main(String[] args){
		
		try {
			ApplicationParameters.loadApplicationParameters(args);
		}
		catch (UndefinedApplicationParameterException e) {
			logger.error(e);
			System.exit(-1);
		}
		SchemaEvolutionToolsuite toolsuite = new SchemaEvolutionToolsuite();
		toolsuite.executeToolsuiteFunction();
	}
	


	/**
	 * Execute the operation required by the user
	 */
	public void executeToolsuiteFunction(){
		DbOperations dbOp = new DbOperations();
		
		try{
			if(ApplicationParameters.getParameterValue("global")){ 
				logger.info("ALL THE OPERATION WILL BE PERFORMED");
				logger.info("Start dropping the databases");
				dbOp.DropDbs();
				logger.info("Start downloading revisions");
				dbOp.DownloadRevisions();
				logger.info("Start installing the schemas");
				dbOp.InstallDbs();
				logger.info("Start collecting data");
				dbOp.collectEvolutionData();
				logger.info("Start dropping databases");
				dbOp.DropDbs();
			}
			else{
				if(ApplicationParameters.getParameterValue("drop")){
					logger.info("DROP");
					dbOp.DropDbs();
				}
				if(ApplicationParameters.getParameterValue("download")){
					//check all the necessary parameter are set
					logger.info("Start downloading the schema");
					dbOp.DownloadRevisions();
				}
				if(ApplicationParameters.getParameterValue("install")){
					//check all the necessary parameter are set			
					logger.info("Start installing the schema");
					dbOp.InstallDbs();
				}
				if(ApplicationParameters.getParameterValue("collect")){
					logger.info("Start collecting data");
					dbOp.collectEvolutionData();
				}
				if(ApplicationParameters.getParameterValue("statistics")){
					logger.info("Start executing the required statistics");
					StatisticsGenerator stat = new StatisticsGenerator();
					//execute all the statistics set in the configuration file
					stat.executeStatistics();
				}
			}
		}
		catch(Exception e){
			logger.error(e);
			System.exit(1);
		}
	}
	
}
