/**
 * Copyright (C) 2009 
 * Developer: Fabrizio Moroni fabrizio.moroni@yahoo.it
 * Contributor: Carlo Curino	carlo@curino.us
 * 
 *  * This file is part of SchemEvolutionToolsuite
 * 
 * SchemEvolutionToolsuite is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SchemaEvolutionToolsuite is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with SchemEvolutionToolsuite.  If not, see <http://www.gnu.org/licenses/>.
 */

package edu.ucla.cs.dbevolution;

import java.io.File;

import edu.ucla.cs.statistics.StatisticsParameters;


/**
 * Class of utility available to all the program's class
 * @author fangar
 *
 */
public class Utils {

	/**
	 * Clean the revision number from special characters, to avoid error or conflict during the installation of the db
	 * For example if the revision number contains a period, MySQL fails the installation or creation of the db
	 * Up to now the only character that created this sort of problem was the period
	 * @param revision
	 * @return
	 */
	public static String cleanRevisionNumber(String revision){
		String newRev = revision.replace(".", "_"); 
		return newRev;
	}
	
	
	
	/**
	 * Check if the directory exist, if not create it
	 * <br>Create the directory and any others non existing parent directory
	 * @param dir path to the filesystem directory
	 * @return true if the directory has been created
	 */
	public static boolean checkCreateDir(String dir){
	
		File fdir = new File(dir);
		if (!fdir.exists()) {
			try {
				System.out.println("Creating directory: " + dir);
				boolean dirCreated = fdir.mkdirs();
				if (dirCreated) {
					System.out.println("Directory schema path created");
					return true;
				}
			} catch (SecurityException sEx) {
				System.out.println("The user doesn't have the privileges to create the directory schema");
				System.exit(1);
			}
		}
		return true;
	}
	
	
	/**
	 * TODO
	 * @return
	 */
	public static String getIntroPage(){
		return  StatisticsParameters.getStatPagePath() + ApplicationParameters.getDbbasename() + StatisticsParameters.getIntroPage();
	}
	
}
