/**
 * Copyright (C) 2009 
 * Developer: Fabrizio Moroni fabrizio.moroni@yahoo.it
 * Contributor: Carlo Curino	carlo@curino.us
 * 
 *  * This file is part of SchemEvolutionToolsuite
 * 
 * SchemEvolutionToolsuite is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SchemaEvolutionToolsuite is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with SchemEvolutionToolsuite.  If not, see <http://www.gnu.org/licenses/>.
 */

package edu.ucla.cs.statistics;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Set;

import edu.ucla.cs.dbevolution.ApplicationParameters;
import edu.ucla.cs.dbevolution.DbBackend.DbColumn;
import edu.ucla.cs.dbevolution.DbBackend.DbTable;
import edu.ucla.cs.dbevolution.DbBackend.DbVersion;
import edu.ucla.cs.dbevolution.persistence.sqlDAO.DbTableDAO;

public class ColumnsAddedDeleted extends Statistic implements StatisticRisk{


	private HashMap<String, Integer> columnsEvolutionChanges;


	public ColumnsAddedDeleted() {

		//create the statistic
		//set the statistic parameters
		setTitle("Table Added and Deleted");
		setStatDescription("The statistic show the number of columns added and deleted per table every change of revision ;\n The statistic doesn't show " +
				"the detail of the transition between one revision and the other, while in this way it is possibile to better understand the change between " +
		"the revisions looking into the modifications occured to the database schema");
		setSqlStmt("The sql compare the name of the tables of every revision with the table name of the previuos and following revision");
		setRange("Added/Deleted");
		setDomain("Revisions");
		setStatName("statTablesAddedDeleted");
		setChartOrientation("v");
		setChartType("bar");
		setTitlePage("Statistic: Tables Added/Deleted");
		setStatPage(ApplicationParameters.getDbbasename() + getStatName() + ".html");
		setStatGroup("tables");
	}



	/**
	 * Create the dataset for the statistic
	 * @return the dataset
	 * @throws SQLException
	 */
	//	public void execute() throws SQLException{
	//		
	//		int tableChanges = 0; 
	//		
	//		//array to store the column's name
	//		Vector<String> columnsBase = new Vector<String>();
	//		Vector<String> columnsAc = new Vector<String>();
	//		
	//		HashMap<String, Integer> columnsEvolutionChanges = new HashMap<String, Integer>();
	//		
	//		String dbName = ApplicationParameters.getDbbasename()+"%";
	//		
	//		Vector<DbTable> tables = DbTable.getTables(dbName);
	//    	
	//		for(DbTable table: tables){
	//			
	//			System.out.println("COLUMN EVOLUTION - PERFORMING TABLE: "+ table.getTABLE_NAME());
	//			
	//			//retrieve the name of the tables for the revision and revision +1
	//			Vector<String[]> columnsInTable = getDb().getColumnsInTable(dbName, table);
	//			
	//			String revisionBase =  ((String[]) columnsInTable.get(0))[0];
	//			String columnNameBase =  ((String[]) columnsInTable.get(0))[1];
	//			
	//			//add the first column's name to the array
	//			//use the actual column's array
	//			columnsAc.add(columnNameBase);
	//			
	//			for(int count=1;count<columnsInTable.size();count++){
	//				
	//				String revisionAc = ((String[]) columnsInTable.get(count))[0];
	//				String columnNameAc = ((String[]) columnsInTable.get(count))[1];
	//				
	//				if(revisionAc.equals(revisionBase)){
	//					//same revision, add the column's name to the array
	//					columnsAc.add(columnNameAc);
	//				}
	//				else{
	//					//different column's name,
	//					if(columnsBase.size()!=0){
	//						//it's not the first comparison, the base array has already been initialized
	//						//compare the two list of columns
	//						int deleted = Utils.getDifference(columnsBase, columnsAc);
	//						int added = Utils.getDifference(columnsAc, columnsBase);
	//						
	//						tableChanges += deleted;
	//						tableChanges += added;
	//					}
	//					columnsBase = (Vector<String>)columnsAc.clone();		
	//					columnsAc.clear();
	//					columnsAc.add(columnNameAc);
	//					
	//					revisionBase = revisionAc;
	//				}
	//			}
	//			
	//			//add the number of changes to the riskmapping table
	//			columnsEvolutionChanges.put(table.getTABLE_NAME(), tableChanges);
	//			tableChanges = 0;
	//		}
	//		setColumnsEvolutionChanges(columnsEvolutionChanges);
	//
	//	}
	public void execute() throws SQLException{

		int tableChanges = 0; 

		HashMap<String, Integer> columnsEvolutionChanges = new HashMap<String, Integer>();

		String dbName = ApplicationParameters.getDbbasename()+"%";

		ArrayList<DbTable> tables = DbTableDAO.getTables(dbName);

		for(DbTable table: tables){

			System.out.println("COLUMN EVOLUTION - PERFORMING TABLE: "+ table.getTABLE_NAME());

			//retrieve the columns for the current table in evry revision of the system's history
			//HashMap<DbVersion, List<DbColumn>> columnsInTable = DbTable.getColumnsInTable(dbName, table.getTABLE_NAME());
			LinkedHashMap<DbVersion, List<DbColumn>> columnsInTable = DbTable.getColumnsInTable(dbName, table.getTABLE_NAME());

			//Iterator i = columnsInTable.entrySet().iterator();
			Set<DbVersion> set = columnsInTable.keySet();
			
			Object[] dbs = set.toArray();
			
			for(int j=0;j<columnsInTable.size()-1;j++){
				
				ArrayList<DbColumn> columns = (ArrayList<DbColumn>)columnsInTable.get(dbs[j]);
				ArrayList<DbColumn> columnsNext = (ArrayList<DbColumn>)columnsInTable.get(dbs[j+1]);
				tableChanges += countDifferentColumns(columns, columnsNext);
				tableChanges += countDifferentColumns(columnsNext,columns);
			}
			System.out.println(tableChanges);
			columnsEvolutionChanges.put(table.getTABLE_NAME(), tableChanges);
			tableChanges = 0;
		}

		//add the number of changes to the riskmapping table
		setColumnsEvolutionChanges(columnsEvolutionChanges);

	}


	/**
	 * Get the difference between the tables of two different revisions
	 * @param first the first revision to compare
	 * @param second the second revision to compare
	 * @return the number of tables different between the two revisions
	 */
	public static int countDifferentColumns(ArrayList<DbColumn> first, ArrayList<DbColumn> second){

		boolean existInSecond = false;
		int countDiff = 0;
		for(DbColumn column: first){
			for(DbColumn columnSecond: second){
				if(column.getCOLUMN_NAME().equals(columnSecond.getCOLUMN_NAME())){
					existInSecond = true;
				}
			}
			if(!existInSecond){
				countDiff++;
			}
			existInSecond = false;
		}
		return countDiff;
	}


	/**
	 * Return the risk factor associated to the statistic, corresponding to tableName
	 * @param tableName
	 * @return
	 */
	public int getRiskMapping(String tableName) {
		try{
			return columnsEvolutionChanges.get(tableName);
		}
		catch(NullPointerException nEx){
			return 0;
		}
	}



	public void setColumnsEvolutionChanges(
			HashMap<String, Integer> columnsEvolutionChanges) {
		this.columnsEvolutionChanges = columnsEvolutionChanges;
	}


}
