/**
 * Copyright (C) 2009 
 * Developer: Fabrizio Moroni fabrizio.moroni@yahoo.it
 * Contributor: Carlo Curino	carlo@curino.us
 * 
 *  * This file is part of SchemEvolutionToolsuite
 * 
 * SchemEvolutionToolsuite is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SchemaEvolutionToolsuite is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with SchemEvolutionToolsuite.  If not, see <http://www.gnu.org/licenses/>.
 */

package edu.ucla.cs.statistics;

import java.sql.SQLException;
import java.util.Vector;

import edu.ucla.cs.dbevolution.DbBackend.DbColumn;
import edu.ucla.cs.dbevolution.DbBackend.StatisticDbOperation;
import edu.ucla.cs.dbevolution.ApplicationParameters;

public class ColumnsLifetime extends StatisticLifetime {

	public ColumnsLifetime() {
	
		super();
		setStatName("statColumnsLifetime");
		setStatDescription("The statistics show the lifetimes of each column: the life of the columns is the number of revision where it is present; " +
				"it doesn't necessary mean the column was live for consecutive revisions, this is just a qualitative chart to represent the number of revision where column was present");
		setSqlStmt("SELECT COLUMN_NAME, COUNT(*) FROM " + ApplicationParameters.getHeaderEvolutionTables() + "COLUMNS WHERE TABLE_SCHEMA LIKE ? GROUP BY TABLE_NAME,COLUMN_NAME ORDER BY count(*) DESC");
		setTitlePage("Statistic: Columns Lifetime");
		setRange("Lifetime");
		setDomain("Columns");
		setChartType("bar");
		setTitle("Columns Lifetime");
		setImageAlt("columns lifetime");
		setChartOrientation("h");
		setStatGroup("columns");
		
	}
	
	/**
	 * Call the executeLifetime method of the superclass giving in input the sql statement to retrieve the lifetime of the tables
	 * and the list of parameters
	 * @throws SQLException 
	 */
	public void executeColumnsLifetime() throws SQLException{
		
		//retrieve the dataset values
		Vector<StatisticValue> values = StatisticDbOperation.GetStatCount(ApplicationParameters.getDbbasename(), this.getSqlStmt());
		int lifetime = DbColumn.getMaxColumnLifetime();
		
		//call the super method
		executeLifetimeStatistic(values, lifetime);
	}
	
	
}
