/**
 * Copyright (C) 2009 
 * Developer: Fabrizio Moroni fabrizio.moroni@yahoo.it
 * Contributor: Carlo Curino	carlo@curino.us
 * 
 *  * This file is part of SchemEvolutionToolsuite
 * 
 * SchemEvolutionToolsuite is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SchemaEvolutionToolsuite is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with SchemEvolutionToolsuite.  If not, see <http://www.gnu.org/licenses/>.
 */

package edu.ucla.cs.statistics;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.Properties;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;

import edu.ucla.cs.dbevolution.ApplicationParameters;
import edu.ucla.cs.dbevolution.Utils;


/**
 * Create and manage the html page based on the Velocity engine
 * @author fangar
 *
 */
public class HtmlStatisticPage {

	
	private ArrayList<StatisticGroup> groups;
	
	/**
	 * 
	 * @param groups
	 */
	public HtmlStatisticPage(ArrayList<StatisticGroup> groups){
		this.groups = groups;
	}
	
	
	/**
	 * Create the Introductory Html page 
	 * @param results 
	 * @param databaseRisk 
	 */
	public void createStatIntroPage(/*ArrayList<RiskResult> results, double databaseRisk*/RiskAnalysis riskAnalysis){
		
		VelocityEngine ve=null;
		Template t=null;
		
		try{
			//get and initialize an engine  
	        ve = getVelocityEngine();
	        
	        try{
		        //get the Template
		        t = ve.getTemplate(StatisticsParameters.getTemplateStatHome());
	        }
	        catch(ResourceNotFoundException rEx){
	        	System.out.println("Unable to find the page template.\nCheck if you have the template page located in the path configured " +
	        			"in the statistics.xml file and repeat the execution");
	        	System.exit(1);
	        }
	        catch(ParseErrorException pEx){
	        	System.out.println("An error occurred during the parsing of the template file: " + StatisticsParameters.getTemplateStatHome());
	        	System.exit(1);
	        }
	        //create a context and add data
	        VelocityContext context = new VelocityContext();
	        context.put("title","Welcome to Schema Evolution Toolsuite");
	        context.put("titlePage","Statistics Generator");
	        context.put("dbname",ApplicationParameters.getDbbasename());
	        context.put("specs",riskAnalysis.getDatabaseSpecs());
	        context.put("globalBg",riskAnalysis.getDatabaseSpecs().getBgPosition());
	        context.put("results",riskAnalysis.getResults());
	        //context.put("menu","templateMenu.vm");
	        
	        createMenuLink(context,"summary");
	       	        
	        //render the template into a StringWriter
	        StringWriter writer = new StringWriter();
	        t.merge( context, writer );
	        
	        File f = new File(Utils.getIntroPage());
			FileWriter fr = new FileWriter(f);
			//write the html page to the file...
			BufferedWriter bwr = new BufferedWriter(fr);
			bwr.write(writer.toString());
			
			bwr.close();
			fr.close();
	        
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
	
	
	
	
	
	/**
	 * Render the parametric menu and create the link to the statistics pages 
	 * @param context the velocity context to customize; insert the parameter into the velocity template
	 * @param current the current page 
	 */
	private void createMenuLink(VelocityContext context,String current/*, ArrayList<StatisticGroup> groups*/){
		
		context.put("href_summary",ApplicationParameters.getDbbasename() + StatisticsParameters.getIntroPage());
		for(StatisticGroup group: groups){
	        context.put("href_" + group.getGroupName(),ApplicationParameters.getDbbasename() + group.getGroupPage());
		}
		
        //set the css class for the currnt page
        if(current.equals("summary")){
        	context.put("current_summary","currentpage");
        	context.put("current_tables","");
        	context.put("current_columns","");
        	context.put("current_constraints","");
        	context.put("current_indexes","");
        	context.put("current_revisions","");
        }
        else if (current.equals("tables")){
        	context.put("current_summary","");
        	context.put("current_tables","currentpage");
        	context.put("current_columns","");
        	context.put("current_constraints","");
        	context.put("current_indexes","");
        	context.put("current_revisions","");
        }
        else if(current.equals("columns")){
        	context.put("current_summary","");
        	context.put("current_tables","");
        	context.put("current_columns","currentpage");
        	context.put("current_constraints","");
        	context.put("current_indexes","");
        	context.put("current_revisions","");
        }
        else if (current.equals("indexes")){
        	context.put("current_summary","");
        	context.put("current_tables","");
        	context.put("current_columns","");
        	context.put("current_constraints","");
        	context.put("current_indexes","currentpage");
        	context.put("current_revisions","");
        }
        else if (current.equals("constraints")){
        	context.put("current_summary","");
        	context.put("current_tables","");
        	context.put("current_columns","");
        	context.put("current_constraints","currentpage");
        	context.put("current_indexes","");
        	context.put("current_revisions","");
        }
        else if (current.equals("revisions")){
        	context.put("current_summary","");
        	context.put("current_tables","");
        	context.put("current_columns","");
        	context.put("current_constraints","");
        	context.put("current_indexes","");
        	context.put("current_revisions","currentpage");
        }
    	//draw the color of the title bar
    	context.put("title_group",current);
        
	}
	
	
	/**
	 * Create the html page for every statistic group
	 * @param group the group for which create the html page
	 */
	public void createStatHtmlPage(StatisticGroup group){
		
		VelocityEngine ve=null;
		Template t=null;
		VelocityContext context=null;
		
		try{
			//get and initialize an engine  
	        ve  = getVelocityEngine();
	        
	        try{
		        //get the Template
		        t = ve.getTemplate(StatisticsParameters.getTemplateStatPage());
	        }
	        catch(ResourceNotFoundException rEx){
	        	System.out.println("Unable to find the page template.\nCheck if you have the template page located in the path configured " +
	        			"in the statistics.xml file and repeat the execution");
	        	System.exit(1);
	        }
	        catch(ParseErrorException pEx){
	        	System.out.println("An error occurred during the parsing of the template file: " + StatisticsParameters.getTemplateStatHome());
	        	System.exit(1);
	        }
	        //create a context and add data
	        context = new VelocityContext();
	        
	        context.put("title",group.getGroupTitle());
	        context.put("subtitle",group.getGroupSubTitle());
	        context.put("titlePage", "Statistics: " + group.getGroupName());
	        context.put("statistics", group.getStats());
	        //context.put("menu","templateMenu.vm");
	        
	        //create the menu
	        //createMenuLink(context,group.getGroupName());
	        createMenuLink(context,group.getGroupName());
	        
	        //render the template into a StringWriter
	        StringWriter writer = new StringWriter();
	        t.merge( context, writer );
	        
	        File f = new File( StatisticsParameters.getStatPagePath() + ApplicationParameters.getDbbasename() +  group.getGroupPage());
			FileWriter fr = new FileWriter(f);

			//write the html page to the file...
			BufferedWriter bwr = new BufferedWriter(fr);
			bwr.write(writer.toString());
			
			bwr.close();
			fr.close();
		}
		catch(Exception e){
			e.printStackTrace();
		}
	}
	
	/**
	 * Create a VelocityEngine object
	 * @return
	 */
	private VelocityEngine getVelocityEngine(){
		VelocityEngine ve = null;
		try{
			ve = new VelocityEngine();
			Properties properties = new Properties();
			properties.setProperty("file.resource.loader.path",StatisticsParameters.getTemplateDir());
			ve.init(properties);
		}
		catch(ResourceNotFoundException rEx){
			System.out.println("Unable to find the template in the directory selected");
			System.exit(1);
		}
		catch(Exception e){
			e.printStackTrace();
		}
        return ve;
	}
	




}
