/**
 * Copyright (C) 2009 
 * Developer: Fabrizio Moroni fabrizio.moroni@yahoo.it
 * Contributor: Carlo Curino	carlo@curino.us
 * 
 *  * This file is part of SchemEvolutionToolsuite
 * 
 * SchemEvolutionToolsuite is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SchemaEvolutionToolsuite is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with SchemEvolutionToolsuite.  If not, see <http://www.gnu.org/licenses/>.
 */

package edu.ucla.cs.statistics;

import java.sql.SQLException;
import java.util.Vector;

import org.jfree.data.time.Month;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.data.xy.IntervalXYDataset;

import edu.ucla.cs.dbevolution.ApplicationParameters;
import edu.ucla.cs.dbevolution.DbBackend.DbVersion;

public class MonthlyRevisions extends Statistic{

	
	public MonthlyRevisions(){
		setTitle("Revision released per month");
		setStatDescription("The statistics show the number of revisions commited every month");
		setSqlStmt("SELECT YEAR(tstart), MONTH(tstart), COUNT(*) FROM " + ApplicationParameters.getEvolutionTable() + " WHERE schema_name LIKE ? GROUP BY YEAR(tstart), MONTH(tstart)");
		setRange("Revisions");
		setDomain("Month");
		setStatName("statRevisionsPerMonth");
		setChartOrientation("v");
		setChartType("barxy");
		setTitlePage("Statistic: Monthly Revisions");
		setStatGroup("revisions");
	}
	
	
	public void execute() throws SQLException{
		setIntervalDataset(monthRevisionsDataset(this));
		executeStatistic(this);
	}
	
	/**
	 * Create a dataset of type IntervalXyDataset and the csv file related to the dataset
	 * @param stat the statistic to create the dataset for
	 * @return the created dataset
	 * @throws SQLException
	 */
	private IntervalXYDataset monthRevisionsDataset(Statistic stat) throws SQLException {
		
		Vector<MonthRevision> revMonth = DbVersion.getMonthRevisions(ApplicationParameters.getDbbasename(),stat.getSqlStmt());
        TimeSeries t1 = new TimeSeries("Executions", "Year", "Count", Month.class);

        try {
			//create the csv file and write the headers
        	createCSVFile(stat);
			addValueToCsv(stat, new String[]{"DomainValueYear","DomainValueMonth","RangeValue"});
			
        	for (MonthRevision month : revMonth) {
        		t1.add(new Month(Integer.parseInt(month.getMonth()),Integer.parseInt(month.getYear())),month.getRevisions());
        		
        		//add value to csv
        		addValueToCsv(stat, new String[]{month.getYear(),month.getMonth(),String.valueOf(month.getRevisions())});	
        	}
        }
        catch (NumberFormatException nEx) {
            System.err.println("Number format ex:" + nEx.getMessage());
        }
        TimeSeriesCollection tsc = new TimeSeriesCollection(t1);
        
        return tsc;
	}
	
	
}
