/**
 * Copyright (C) 2009 
 * Developer: Fabrizio Moroni fabrizio.moroni@yahoo.it
 * Contributor: Carlo Curino	carlo@curino.us
 * 
 *  * This file is part of SchemEvolutionToolsuite
 * 
 * SchemEvolutionToolsuite is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SchemaEvolutionToolsuite is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with SchemEvolutionToolsuite.  If not, see <http://www.gnu.org/licenses/>.
 */

package edu.ucla.cs.statistics;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Set;

import org.jfree.data.category.DefaultCategoryDataset;

import edu.ucla.cs.dbevolution.ApplicationParameters;
import edu.ucla.cs.dbevolution.DbBackend.ColumnsInPrimaryBySchemaVersion;
import edu.ucla.cs.dbevolution.DbBackend.DbColumn;
import edu.ucla.cs.dbevolution.DbBackend.DbKeyColumnUsage;
import edu.ucla.cs.dbevolution.DbBackend.DbTableConstraints;
import edu.ucla.cs.dbevolution.DbBackend.DbVersion;


/**
 * Create the statistic that inspect every single table, looking for a change in the key
 * The statistic analyze a table, checking if the keys are changed during the different revisions
 * @throws SQLException
 */
public class PrimaryKeyChanges extends Statistic implements StatisticRisk{


	/**
	 * TODO
	 * Set the default filed for the statistic
	 */
	public PrimaryKeyChanges(){

		setTitle("Primary Keys Changes");
		setStatDescription("The statistic show the number of primary keys modified during the lifetime of a table\n The aggregation level is the table name. " +
		"This can be used as an index of database stability");
		setSqlStmt("The sql analyze the number of columns belonging to the key in each revision of the databas during the table lifetime");
		setRange("Primary key Changes");
		setDomain("Tables");
		setStatName("statPrimaryKeysChanges");
		setChartOrientation("h");
		setChartType("bar");
		setTitlePage("Statistic: Primary Keys Changes");
		setStatPage(ApplicationParameters.getDbbasename() + getStatName() + ".html");
		setStatGroup("constraints");
	}


	private HashMap<String, Integer> tablesPrimaryChanges;


	/**
	 * TODO
	 * @throws SQLException 
	 * 
	 */
	public void execute() throws SQLException{
		setDs(primaryKeyChangesDataset());
		executeStatistic(this);
	}


	/**
	 * Set the dataset of the statistic
	 * @param ds the DefaultCategoryDataset
	 */
	public void setDataset(DefaultCategoryDataset ds){
		setDs(ds);
	}


	/**
	 * TODO
	 * @return
	 * @throws SQLException
	 */
	public DefaultCategoryDataset primaryKeyChangesDataset() throws SQLException{

		int columnNameChanges = 0;
		int columnSizeChanges = 0;

		int tableNameChanges = 0;
		int tableSizeChanges = 0;

		String dbName = ApplicationParameters.getDbbasename() + "%";

		//risk mapping
		HashMap<String,Integer> tablesPrimaryChanges = new HashMap<String, Integer>();

		//create the dataset
		DefaultCategoryDataset ds = new DefaultCategoryDataset();
		ArrayList<DbTableConstraints> tables = DbTableConstraints.getTablesWithKeys(dbName);


		//create the CSV file and add the header to the file
		createCSVFile(this);
		addValueToCsv(this, new String[]{"TableName","Changes Columns", "Changes Col Name"});

		for(int countTbl = 0;countTbl<tables.size()-1;countTbl++){

			String tableName = ((DbTableConstraints)tables.get(countTbl)).getTABLE_NAME();
			System.out.println("PERFORMING TABLE: " + tableName);


			/* BEFORE IBATIS	*/

//			LinkedHashMap<DbVersion, List<DbKeyColumnUsage>> result = DbKeyColumnUsage.getColumnsInPrimary(tableName, dbName);
//
//			Set<DbVersion> set = result.keySet();
//
//			Object[] dbs = set.toArray();
//
//			for(int j=0;j<result.size()-1;j++){
//
//				ArrayList<DbKeyColumnUsage> column1 = (ArrayList<DbKeyColumnUsage>)result.get(dbs[j]);
//				ArrayList<DbKeyColumnUsage> column2 = (ArrayList<DbKeyColumnUsage>)result.get(dbs[j+1]);
//				columnNameChanges += DbColumn.compareColumnsName(column1, column2);
//				columnSizeChanges += DbColumn.compareColumnsSize(column1, column2);
//			}


			List<ColumnsInPrimaryBySchemaVersion> result = DbKeyColumnUsage.getColumnsInPrimary(tableName, dbName);
			
			for(int j=0;j<result.size()-1;j++){

				ArrayList<DbKeyColumnUsage> column1 = (ArrayList<DbKeyColumnUsage>)result.get(j).getColumns();
				ArrayList<DbKeyColumnUsage> column2 = (ArrayList<DbKeyColumnUsage>)result.get(j+1).getColumns();
				columnNameChanges += DbColumn.compareColumnsName(column1, column2);
				columnSizeChanges += DbColumn.compareColumnsSize(column1, column2);
			}
			
			
			tableNameChanges += columnNameChanges;
			tableSizeChanges += columnSizeChanges;


			//add values to the risk mapping
			//add the value to the mapping only if there's a column added in the primary key
			tablesPrimaryChanges.put(tableName, tableNameChanges);


			//add the value to the csv
			addValueToCsv(this, new String[]{tableName,String.valueOf(tableNameChanges), String.valueOf(tableSizeChanges)});

			//add values to the dataset
			addValueToDataset(ds, tableNameChanges, "colname", tableName);
			addValueToDataset(ds, tableSizeChanges, "colSize", tableName);

			tableNameChanges = 0;
			tableSizeChanges = 0;


			columnNameChanges = 0;
			columnSizeChanges = 0;

		}
		setTablesPrimaryChanges(tablesPrimaryChanges);

		return ds;
	}


	/**
	 * TODO
	 * @param tableName
	 * @return
	 */
	public int getRiskMapping(String tableName) {
		try{
			return tablesPrimaryChanges.get(tableName);
		}
		catch(NullPointerException nEx){
			return 0;
		}
	}


	public void setTablesPrimaryChanges(
			HashMap<String, Integer> tablesPrimaryChanges) {
		this.tablesPrimaryChanges = tablesPrimaryChanges;
	}


}
