/**
 * Copyright (C) 2009 
 * Developer: Fabrizio Moroni fabrizio.moroni@yahoo.it
 * Contributor: Carlo Curino	carlo@curino.us
 * 
 *  * This file is part of SchemEvolutionToolsuite
 * 
 * SchemEvolutionToolsuite is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SchemaEvolutionToolsuite is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with SchemEvolutionToolsuite.  If not, see <http://www.gnu.org/licenses/>.
 */

package edu.ucla.cs.statistics;

import java.sql.SQLException;
import java.util.Vector;

import edu.ucla.cs.dbevolution.ApplicationParameters;
import edu.ucla.cs.dbevolution.DbBackend.DbVersion;

/**
 * Create the statistic "Revisions Lifetime" highlighting the lifetime of each single revision
 * @author fangar
 */
public class RevisionsLifetime extends StatisticLifetime{

	/**
	 * Set the default filed for the statistic
	 */
	public RevisionsLifetime(){

		setTitle("Revision Lifetime");
		setStatDescription("The statistics show the lifetime, in days, of each revision");
		setSqlStmt("SELECT schema_unique_name, DATEDIFF(tend,tstart) FROM " + ApplicationParameters.getEvolutionTable() + " WHERE schema_unique_name like ? AND tend < (SELECT MAX(tend) FROM " + ApplicationParameters.getEvolutionTable() + " WHERE schema_unique_name like ?) ORDER BY tstart");
		setRange("Days");
		setDomain("Revisions");
		setStatName("statRevisionsLifetime");
		setChartOrientation("h");
		
		setChartType("bar");
		setTitlePage("Statistic: Revisions Lifetime");
		setStatGroup("revisions");
	}
	


	/**
	 * Execute the statistic
	 * @throws SQLException
	 */
	public void executeRevisionLifetime() throws SQLException{
		
		//retrieve the dataset values
		Vector<StatisticValue> values = DbVersion.getRevisionLifetime();
		int lifetime = DbVersion.getMaxRevisionLifetime();
		//call the super method
		executeLifetimeStatistic(values, lifetime);
	}

	
	
}
