/**
 * Copyright (C) 2009 
 * Developer: Fabrizio Moroni fabrizio.moroni@yahoo.it
 * Contributor: Carlo Curino	carlo@curino.us
 * 
 *  * This file is part of SchemEvolutionToolsuite
 * 
 * SchemEvolutionToolsuite is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SchemaEvolutionToolsuite is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with SchemEvolutionToolsuite.  If not, see <http://www.gnu.org/licenses/>.
 */

package edu.ucla.cs.statistics;

import java.math.BigDecimal;
import java.sql.SQLException;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Hashtable;
import java.util.Vector;

import edu.ucla.cs.dbevolution.ApplicationParameters;
import edu.ucla.cs.dbevolution.DbBackend.DbTable;
import edu.ucla.cs.dbevolution.DbBackend.DbVersion;
import edu.ucla.cs.dbevolution.persistence.sqlDAO.DbTableDAO;

public class RiskAnalysis extends Statistic{

	
	
	//list of statistics containing the risk mapping
	private ArrayList<Statistic> statsRisk;
	private ArrayList<RiskResult> results;
	
	private double databaseRisk;
	
	private DatabaseSpecs databaseSpecs;
	
	/**
	 * @throws SQLException
	 */
	public /*ArrayList<RiskResult>*/void executeRiskAnalysis() throws SQLException{
		
		double globalRiskFactor=0.0;
		databaseSpecs = new DatabaseSpecs();
		
		//retrieve the name of all the tables present in the database
		ArrayList<DbTable> tables = DbTableDAO.getTables(ApplicationParameters.getDbbasename()+"%");
		
		//list of risk result computed during the analysis
		ArrayList<RiskResult> results = null;
		
		
		if(tables!=null){
						
			results = new ArrayList<RiskResult>();
			
			for(DbTable table:tables){
				
				RiskResult result = new RiskResult(table.getTABLE_NAME());
				
				Hashtable<String, Hashtable<String,String>> changes = getTableColumnsEvolutionDataType(table.getTABLE_NAME());
				result.setTypeChange(changes);
				
				//fill the remaining risk factor previously computed
				fillRiskFactors(result);
				result.computeRiskFactor();
				
				//add the table's risk factor to the global risk factor
				globalRiskFactor+=result.getTableRisk();
				results.add(result);
			}
			globalRiskFactor = globalRiskFactor/results.size();
			//this.databaseRisk = globalRiskFactor;
			
			//compute all the data related to the database history
			this.databaseSpecs.setDatabaseEvolutionRate(globalRiskFactor);
			this.databaseSpecs.setFirstRelease(DbVersion.getDatabaseFirstRelease().getTStart().toString());
			this.databaseSpecs.setLastRelease(DbVersion.getDatabaseLastRelease().getTEnd().toString());
			this.databaseSpecs.setInitialSize(DbTableDAO.getDatabaseInitialSize());
			this.databaseSpecs.setEndSize(DbTableDAO.getDatabaseEndSize());
		}
		this.setResults(results);
	}
	
	
	public String getDatabaseRiskFactor() {
		
		//this.computeRiskFactor();
		//DecimalFormat format = new DecimalFormat("##.##");
		DecimalFormat format = new DecimalFormat("%");
		format.setPositivePrefix("");
		format.setPositiveSuffix("%");
		return format.format(databaseRisk);
	}
	
	
	/**
	 * Fill the risk result object with the risk factors computed during the previous statistics generation
	 * @param result
	 */
	private void fillRiskFactors(RiskResult result) {
		
		for(Statistic stat : statsRisk){
			if(stat instanceof PrimaryKeyChanges){
				result.setPrimaryKeyChanges(((PrimaryKeyChanges) stat).getRiskMapping(result.getTable()));
			}
			else if(stat instanceof ForeignKeyChanges){
				result.setForeignKeyChanges(((ForeignKeyChanges)stat).getRiskMapping(result.getTable()));
			}
			else if(stat instanceof RolledBackTables){
				result.setRollbackCount(((RolledBackTables)stat).getRiskMapping(result.getTable()));
			}
			else if(stat instanceof IndexChanges){
				result.setIndexChanges(((IndexChanges)stat).getRiskMapping(result.getTable()));
			}
			else if(stat instanceof ColumnsAddedDeleted){
				result.setColumnsEvolution(( (ColumnsAddedDeleted) stat).getRiskMapping(result.getTable()));
			}
		}
		
	}


	
	/**
	 * Compute the data type evolution for each table
	 * @param table
	 * @return
	 * @throws SQLException
	 */
	//public Hashtable<String, String> getTableColumnsEvolutionDataType(String table) throws SQLException{
	public Hashtable<String, Hashtable<String, String>> getTableColumnsEvolutionDataType(String table) throws SQLException{

		String sql = "SELECT table_schema,column_name, data_type FROM " + ApplicationParameters.getHeaderEvolutionTables() + 
		"COLUMNS c, " + ApplicationParameters.getEvolutionTable() + " d WHERE c.table_schema = d.schema_unique_name AND table_schema like ? AND table_name like ? GROUP BY column_name, table_schema" +
				" ORDER BY c.column_name ,d.tstart";
		String[] params = new String[]{ApplicationParameters.getDbbasename() + "%", table};

		boolean dataTypeChanges = false;
		//to store all the tables with a column that changed data type
		Hashtable<String, String> typeChanges = new Hashtable<String, String>();
		Hashtable<String, Hashtable<String, String>> typeChanges2 = new Hashtable<String, Hashtable<String,String>>();

		Vector<String[]> result = getDb().executeStatement(sql, params);

		if(result!=null){

			//DEBUG	String db = ( (String[]) result.get(0) )[0];	//only to compare in the debug
			String col = ( (String[]) result.get(0) )[1];
			String type = ( (String[]) result.get(0) )[2];

			for(int i=1;i<result.size(); i++){

				//DEBUG	String dbAc = ( (String[]) result.get(i) )[0]; 
				String colAc = ( (String[]) result.get(i) )[1];
				String typeAc = ( (String[]) result.get(i) )[2];

				/*if(colAc.equals(col) && typeAc.equals(type)){
				 * 
				 * 	not interested if the column name and table name are the same....
					//both the column name and the column datatype are equals, we're referring to the same column
					//of the same table, with the same datatyep
				}*/
				/*else */if(colAc.equals(col) && !typeAc.equals(type)){

					//the same column but with a different datatype
					dataTypeChanges = true;
					
					Hashtable<String, String> check = typeChanges2.get(col);
					if(check!=null){
						//a mapping already exist, check if the columns data type are the same
						String value = check.get(type);
						if(!value.equals(typeAc)){
							//the two data type aren't the same, i can add another column data type change
							Hashtable<String, String> toAdd = new Hashtable<String, String>();
							toAdd.put(type, typeAc);
							typeChanges2.put(col, toAdd);
						}
					}
					else{
						//no mapping exist, put a new mapping inside
						Hashtable<String, String> toAdd = new Hashtable<String, String>();
						toAdd.put(type, typeAc);
						typeChanges2.put(col, toAdd);
					}
				
					
				}
				else if(!colAc.equals(col)){
					//both the column and the datatype are different, we're changed the column.
					
					//before changing the column, check if a data type change occured, and save the column
					//in the hash table
					if(dataTypeChanges){
						typeChanges.put(col, type);
						dataTypeChanges = false;
					}
					
					//change the column
					col = colAc;
					type = typeAc;
				}
			}
		}
		//return typeChanges;
		return typeChanges2;
	}

	
	public ArrayList<Statistic> getStatsRisk() {
		return statsRisk;
	}


	public void setStatsRisk(ArrayList<Statistic> statsRisk) {
		this.statsRisk = statsRisk;
	}


	public void setDatabaseRisk(double databaseRisk) {
		this.databaseRisk = databaseRisk;
	}


	public double getDatabaseRisk() {
		return databaseRisk;
	}


	public void setResults(ArrayList<RiskResult> results) {
		this.results = results;
	}


	public ArrayList<RiskResult> getResults() {
		return results;
	}


	public void setDatabaseSpecs(DatabaseSpecs databaseSpecs) {
		this.databaseSpecs = databaseSpecs;
	}


	public DatabaseSpecs getDatabaseSpecs() {
		return databaseSpecs;
	}



	
	public class DatabaseSpecs {
		
		
		private String firstRelease;
		private String lastRelease;
		private int initialSize;
		private int endSize;
		private double databaseEvolutionRate;
		
		
		public String getFirstRelease() {
			return firstRelease;
		}
		public void setFirstRelease(String firstRelease) {
			this.firstRelease = firstRelease;
		}
		public String getLastRelease() {
			return lastRelease;
		}
		public void setLastRelease(String lastRelease) {
			this.lastRelease = lastRelease;
		}
		public int getInitialSize() {
			return initialSize;
		}
		public void setInitialSize(int initialSize) {
			this.initialSize = initialSize;
		}
		public int getEndSize() {
			return endSize;
		}
		public void setEndSize(int endSize) {
			this.endSize = endSize;
		}
		public double getDatabaseEvolutionRate() {
			return databaseEvolutionRate;
		}
		public void setDatabaseEvolutionRate(double databaseEvolutionRate) {
			this.databaseEvolutionRate = databaseEvolutionRate;
		}
		
		
		/**
		 * TODO
		 * @return
		 */
		public String getDatabaseEvolutionFactor() {
			
			//this.computeRiskFactor();
			//DecimalFormat format = new DecimalFormat("##.##");
			DecimalFormat format = new DecimalFormat("%");
			format.setPositivePrefix("");
			format.setPositiveSuffix("%");
			return format.format(getDatabaseEvolutionRate());
		}
		
		
		/**
		 * TODO..
		 * @return
		 */
		public String getBgPosition(){

			//round the decimal value with two decimal digit
			BigDecimal d = new BigDecimal(getDatabaseEvolutionRate());
			d.setScale(2,BigDecimal.ROUND_HALF_UP);
			double risk = d.doubleValue();
			
			//image width = 500;
			double shift = (1 - risk) * 100;
			
			return "" + String.valueOf(shift) + "%;";
		}
		
		

	}
	
	
	
	
	
	
	
	
	
}
