/**
 * Copyright (C) 2009 
 * Developer: Fabrizio Moroni fabrizio.moroni@yahoo.it
 * Contributor: Carlo Curino	carlo@curino.us
 * 
 *  * This file is part of SchemEvolutionToolsuite
 * 
 * SchemEvolutionToolsuite is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SchemaEvolutionToolsuite is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with SchemEvolutionToolsuite.  If not, see <http://www.gnu.org/licenses/>.
 */

package edu.ucla.cs.statistics;

import java.math.BigDecimal;
import java.text.DecimalFormat;
import java.util.Hashtable;
import java.util.Iterator;

public class RiskResult {

	
	private String table;
	//private Hashtable<String, String> typeChanges;
	private int rollbackCount;
	private int columnsEvolution;
	private int primaryKeyChanges;
	private int foreignKeyChanges;
	private int indexChanges;
	private double tableRiskFactor;
	
	//hashtable for type changes
	//contains a entry per column, the first fild is a string representing the name of the column, the second field contains the type changes
	//HashTable<String, String> contains the source and destination type
	private Hashtable<String, Hashtable<String, String>> typeChange;
	
	
	/**
	 * Compute the number of data type changes in the table
	 * @return
	 */
	public int getDataTypeChanges(){
		int changes = 0;
		
		Iterator it = this.typeChange.keySet().iterator();
		if(it!=null){
			while(it.hasNext()){
				Hashtable<String, String> t = typeChange.get(it.next());
				changes += t.size();
			}
		}
		return 0;
	}
	
	
	public RiskResult(String table){
		this.table = table;
	}
	
	public String getTable() {
		return table;
	}
	public void setTable(String table) {
		this.table = table;
	}
	/*public Hashtable<String, String> getTypeChanges() {
		return typeChanges;
	}
	public void setTypeChanges(Hashtable<String, String> typeChanges) {
		this.typeChanges = typeChanges;
	}*/
	public int getRollbackCount() {
		return rollbackCount;
	}
	public void setRollbackCount(int rollbackCount) {
		this.rollbackCount = rollbackCount;
	}

	public int getColumnsEvolution() {
		return columnsEvolution;
	}

	public void setColumnsEvolution(int columnsDiffCount) {
		this.columnsEvolution = columnsDiffCount;
	}
	
	/*public Iterator getIteratorTypes (){
		return typeChanges.keySet().iterator();
	}*/

	public Hashtable<String, Hashtable<String, String>> getTypeChange() {
		return typeChange;
	}

	public void setTypeChange(
			Hashtable<String, Hashtable<String, String>> typeChange) {
		this.typeChange = typeChange;
	}

	public int getPrimaryKeyChanges() {
		return primaryKeyChanges;
	}

	public void setPrimaryKeyChanges(int primaryKeyChanges) {
		this.primaryKeyChanges = primaryKeyChanges;
	}

	public String getTableRiskFactor() {
		
		//this.computeRiskFactor();
		//DecimalFormat format = new DecimalFormat("##.##");
		DecimalFormat format = new DecimalFormat("%");
		format.setPositivePrefix("");
		format.setPositiveSuffix("%");
		return format.format(tableRiskFactor);
	}
	
	public double getTableRisk(){
		return this.tableRiskFactor;
	}

	/**
	 * Computes the risk factor corresponding to this table
	 * taking into account the risk coefficients defined by the user in the configuration file 
	 * the risk factor is computed as a weighted sum of the different risk factors
	 */
	public void computeRiskFactor() {
		this.tableRiskFactor += StatisticsParameters.getRiskColumnsCount()* this.columnsEvolution;
		this.tableRiskFactor += StatisticsParameters.getRiskColumnsTypeChange()* this.typeChange.size();
		this.tableRiskFactor += StatisticsParameters.getRiskPrimaryKeyChange()* this.primaryKeyChanges;
		this.tableRiskFactor += StatisticsParameters.getRiskForeignKeyChange()* this.foreignKeyChanges;
		this.tableRiskFactor += StatisticsParameters.getRiskIndexChange()* this.indexChanges;
		this.tableRiskFactor += StatisticsParameters.getRiskRollbackCount()* this.rollbackCount;
	}


	/*public void setTableRiskFactor(double tableRiskFactor) {
		this.tableRiskFactor = tableRiskFactor;
	}*/
	
	/*public String getBgPosition(){

		//round the decimal value with two decimal digit
		BigDecimal d = new BigDecimal(this.tableRiskFactor);
		d.setScale(2,BigDecimal.ROUND_HALF_UP);
		double risk = d.doubleValue();
		
		//image width = 500;
		double shift = risk * 500;
		
		return "-" + String.valueOf(shift) + "px";
	}*/
	
	
	public String getBgPosition(){

		//round the decimal value with two decimal digit
		BigDecimal d = new BigDecimal(this.tableRiskFactor);
		d.setScale(2,BigDecimal.ROUND_HALF_UP);
		double risk = d.doubleValue();
		
		//image width = 500;
		double shift = (1 - risk) * 100;
		
		return "" + String.valueOf(shift) + "%;";
	}


	public int getForeignKeyChanges() {
		return foreignKeyChanges;
	}


	public void setForeignKeyChanges(int foreignKeyChanges) {
		this.foreignKeyChanges = foreignKeyChanges;
	}


	public int getIndexChanges() {
		return indexChanges;
	}


	public void setIndexChanges(int indexKeyChanges) {
		this.indexChanges = indexKeyChanges;
	}
	
}
