/**
 * Copyright (C) 2009 
 * Developer: Fabrizio Moroni fabrizio.moroni@yahoo.it
 * Contributor: Carlo Curino	carlo@curino.us
 * 
 *  * This file is part of SchemEvolutionToolsuite
 * 
 * SchemEvolutionToolsuite is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SchemaEvolutionToolsuite is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with SchemEvolutionToolsuite.  If not, see <http://www.gnu.org/licenses/>.
 */

package edu.ucla.cs.statistics;

import java.sql.SQLException;
import java.util.Hashtable;
import java.util.Vector;

import org.jfree.data.category.DefaultCategoryDataset;

public class StatisticLifetime extends Statistic {

	/*
	 * the groups are computed based on the following bounds
	 * from 0 to 25% of the lifetime
	 * from 25 to 50% of the lifetime
	 * from 50 to 75% of the lifetime
	 * from 75 to 100% of the lifetime
	 */
	private final double[] THRESHOLDS = {0.25, 0.5, 0.75, 1};
	
	//the interval value corresponding to the groups
	private int[] groupsInterval;
	
	private Hashtable<Integer, Vector<StatisticValue>> groupValues = new Hashtable<Integer, Vector<StatisticValue>>(THRESHOLDS.length);
	private Vector<LifetimeGroup> groups;
	
	private String imgReducedPath;
	private String csvReducedPath;
	
//	/**
//	 * Return the max table's lifetime
//	 * @return
//	 * @throws SQLException
//	 */
//	private int getMaxLifetime(String sql, String[] params) throws SQLException{
//		
//		Vector<String[]> result = getDb().executeStatement(sql, params);
//		String lifetime = ( (String[]) result.get(0))[0];
//		
//		return Integer.valueOf(lifetime);
//	}
	
	
	/**
	 * return the bounds of the groups based on the max lifetime
	 * @param lifetime
	 * @return
	 */
	private int[] computeGroupsInterval(int lifetime){
	
		int[] groupsInterval = new int[THRESHOLDS.length];
		
		for(int count=0;count<THRESHOLDS.length;count++){
			groupsInterval[count] = (int) (lifetime*THRESHOLDS[count]);
			
			//create the hashtable's element corresponding to the groups just defined
			groupValues.put(Integer.valueOf(count), new Vector<StatisticValue>());
		}
		return groupsInterval;
	}
	
	
	/**
	 * Get the interval where the value belongs to
	 * @param value
	 * @param groupsInterval
	 * @return
	 */
	private int getGroupFromValue(int value, int[] groupsInterval){
		int count=0;
		
		for(;count<groupsInterval.length-1;count++){
			if(( value>= groupsInterval[count] ) && (value <= groupsInterval[count+1]) ){
				return count+1;
			}
		}
		//if( (value > 0) && (value < groupsInterval[count]) ){
		return 0;
	}
	
	
	/**
	 * Query the database and split the result into the groups computed 
	 * @param values 
	 * @throws SQLException
	 */
	public void executeLifetimeStatistic(Vector<StatisticValue> values, int lifetime ) throws SQLException{
		
		//Vector<StatisticValue> values = getDb().GetStatCount(ApplicationParameters.getDbbasename(), this.getSqlStmt());
		
		//compute the group's intervals
		this.groupsInterval = computeGroupsInterval(lifetime);
		
		//create the csv for the entire result set 
		//the csv with the all the values, now splitted in groups
		createCSVFile(this);
		addValueToCsv(this, new String[]{"X-Value","Y-Value"});
		String csvFullPath = getCsvData();
		
		for(StatisticValue value: values){
			int group = getGroupFromValue(value.getY(), this.groupsInterval);
			
			//retrieve the value's Vector corresponding to the group
			//and add the input value to it
			//the vector has already been defined, no need to check for null values or initialize it
			Vector<StatisticValue> vectorValues = groupValues.get(Integer.valueOf(group));
			vectorValues.add(value);
			
			//add the value to the full csv 
			addValueToCsv(this, new String[]{value.getX(),String.valueOf(value.getY())});
		}
		
		String statName = getStatName();
		groups = new Vector<LifetimeGroup>();
		
		for(int count = 0; count<groupValues.size(); count++){
			
			//define the group's name
			String groupName = createGroupName(count);
			//retrieve the value belonging to the group
			Vector<StatisticValue> val = groupValues.get(count);
			
			//create a new lifetime group
			LifetimeGroup lifeGroup = new LifetimeGroup();
			lifeGroup.setGroupName(groupName);
			lifeGroup.setGroupHeader(groupName);
			//set the number of entry belonging to the group into the lifetime group
			//this entry will be used to create the global chart grouping all the groups
			lifeGroup.setValue(val.size());
			//set the new statistic name
			//the name will be used in the createDatasetMethod to fill the csv file
			this.setStatName(statName+count+"expanded");

			//create the dataset
			//set the dataset to the group
			DefaultCategoryDataset dsExpanded = createExpandedDataset(count, val);
			setDs(dsExpanded);

			//execute the static (use the method of the superclass)
			executeStatistic(this);
			
			//set the expanded parameter to the lifetimeGroup
			lifeGroup.setGroupImage(getImagePath());
			lifeGroup.setGroupCsvData(getCsvData());
			
			
			//create the reduced dataset
			//the dataset contains only the number of table fitting in the group interval
			//one series -> empty row key
			//one category -> the value of the interval ( the name of the group ) 
			
			//MODIFIED ON 20/01
			//CREATE A SINGLE CATEGORY CHART FOR THE REDUCED DATA DEVLITING THE SINGLE REDUCED CHART
			/*DefaultCategoryDataset dsReduced = new DefaultCategoryDataset();
			addValueToDataset(dsReduced, groupValues.get(count).size(), "", groupName);
			//set the group reduced dataset
			setDs(dsReduced);
			this.setStatName(statName+count+"reduced");
			executeStatistic(this);
			
			//set the reduced parameter to the lifetimeGroup
			lifeGroup.setImgReduced(getImagePath());
			END OF THE 20/01 CHANGE */
			
			
			//add the group to the list of lifetime group
			groups.add(lifeGroup);
		}
		
		//NEW SINGLE CHART, MULTIPLE CATEGORY ON 20/01
		DefaultCategoryDataset dsReduced = new DefaultCategoryDataset();
		for(LifetimeGroup group: groups){
			addValueToDataset(dsReduced, group.getValue(), "", group.getGroupName());
		}
		//set the group reduced dataset
		setDs(dsReduced);
		this.setChartOrientation("v");
		this.setStatName(statName+"reduced");
		executeStatistic(this);
		
		setImgReducedPath(getImagePath());
		//setCsvData(getCsvData());
		setCsvData(csvFullPath);
	}
	
	
	/**
	 * Create the dataset linked to the expanded value
	 * the reduced dataset is simply computed adding the count of the values to the dataset 
	 * @param group
	 * @param groupValues
	 * @return
	 */
	private DefaultCategoryDataset createExpandedDataset(int group, Vector<StatisticValue> groupValues){
		
		//create csv file for the expanded statistic
		//create the file index
		createCSVFile(this);
		addValueToCsv(this, new String[]{"X-Value","Y-Value"});
		
		
		DefaultCategoryDataset ds = new DefaultCategoryDataset();
		for(StatisticValue value: groupValues){
			//one serie -> empty row key
			//multiple category -> table name
			addValueToDataset(ds, value.getY(), "", value.getX());
			
			//add value to the csv file
			addValueToCsv(this, new String[]{value.getX(),String.valueOf(value.getY())});
		}
		return ds;
		
	}
	

	/**
	 * Create the group name to assign to the reduced graph
	 * @param group
	 * @return
	 */
	private String createGroupName(int group){
		String groupName = "";
		if(group==0){
			groupName = "From 0 to " + groupsInterval[group];
		}
		else{
			groupName = "From " + groupsInterval[group-1] + " to " + groupsInterval[group];
		}
		return groupName;
	}

	
	public Hashtable<Integer, Vector<StatisticValue>> getGroupValues() {
		return groupValues;
	}


	public void setGroupValues(
			Hashtable<Integer, Vector<StatisticValue>> groupValues) {
		this.groupValues = groupValues;
	}
	
	
	/* (non-Javadoc)
	 * @see edu.ucla.cs.statistics.Statistic#isLifetimeCategory()
	 */
	public boolean isLifetimeCategory(){
		return true;
	}


	public Vector<LifetimeGroup> getGroups() {
		return groups;
	}


	public void setGroups(Vector<LifetimeGroup> groups) {
		this.groups = groups;
	}
	
	
	
	public void setImgReducedPath(String imgReducedPath) {
		this.imgReducedPath = imgReducedPath;
	}


	public String getImgReducedPath() {
		return imgReducedPath;
	}



	public void setCsvReducedPath(String csvReducedPath) {
		this.csvReducedPath = csvReducedPath;
	}


	public String getCsvReducedPath() {
		return csvReducedPath;
	}



	/**
	 * private class LifetimeGroup
	 * used for the lifetime type statistics to encapsulate information about the groups of charts
	 * @author fangar
	 *
	 */
	public class LifetimeGroup {

		
		private String groupImage;
		private String groupName;
		private String groupHeader;
		private String groupImageAlt;
		private String groupCsvData;
		
		private int value;
		

		public String getGroupName() {
			return groupName;
		}
		public void setGroupName(String groupName) {
			this.groupName = groupName;
		}
		public String getGroupHeader() {
			return groupHeader;
		}
		public void setGroupHeader(String groupHeader) {
			this.groupHeader = groupHeader;
		}
		public void setValue(int value) {
			this.value = value;
		}
		public int getValue() {
			return value;
		}
		public String getGroupImage() {
			return groupImage;
		}
		public void setGroupImage(String groupImage) {
			this.groupImage = groupImage;
		}
		public void setGroupCsvData(String groupCsvData) {
			this.groupCsvData = groupCsvData;
		}
		public String getGroupCsvData() {
			return groupCsvData;
		}
		public String getGroupImageAlt() {
			return groupImageAlt;
		}
		public void setGroupImageAlt(String groupImageAlt) {
			this.groupImageAlt = groupImageAlt;
		}
		
	}	
	
	
	
}
