/**
 * Copyright (C) 2009 
 * Developer: Fabrizio Moroni fabrizio.moroni@yahoo.it
 * Contributor: Carlo Curino	carlo@curino.us
 * 
 *  * This file is part of SchemEvolutionToolsuite
 * 
 * SchemEvolutionToolsuite is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SchemaEvolutionToolsuite is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with SchemEvolutionToolsuite.  If not, see <http://www.gnu.org/licenses/>.
 */

package edu.ucla.cs.statistics;

import it.polimi.elet.cautils.configurator.CAConfiguration;

import java.util.Hashtable;

import org.apache.log4j.Logger;

import edu.ucla.cs.Exceptions.UndefinedStatisticParameterException;
import edu.ucla.cs.dbevolution.ApplicationParameters;


public class StatisticsParameters {

	private static Hashtable<String, Object> statParams = new Hashtable<String, Object>();
	private static String pageExt = ".html";
	private static Logger log = Logger.getLogger(StatisticsParameters.class);

	//private static Vector<String> stats;	
	
	public static String getStatPagePath() {
		return (String)statParams.get("statPage") + ApplicationParameters.getDbbasename() + "Stat/";
	}

	public static String getStatImagePath() {
		return StatisticsParameters.getStatPagePath() + "images/";
	}

	public static String getStatCsvPath() {
		return StatisticsParameters.getStatPagePath() + "csv/";
	}

//	public static Vector<String> getStats(){
//		return stats;
//	}
//
//	public static void addStat(String stat){
//		stats.add(stat);
//	}

	public static String getTemplateStatPage() {
		return (String)statParams.get("templateStatPage");
	}

	public static String getTemplateStatHome() {
		return (String)statParams.get("templateStatHome");
	}

	public static String getStatList() {
		return (String)statParams.get("statisticsList");
	}

	public static String getIntroPage() {
		return (String)statParams.get("introPage");
	}

	public static String getGroups() {
		return (String)statParams.get("groups");
	}

	public static String getTemplateDir() {
		return (String)statParams.get("templateDir");
	}

	public static String getPageExt() {
		return pageExt;
	}




	/**
	 * Load the statistics parameters from the configuration
	 * @param conf the statistics configuration
	 */
	public static void loadConfiguration(CAConfiguration confStatistics){

		//parse the statistics configuration file
		if(ApplicationParameters.getParameterValue("statistics")){
			try{
				if((String)confStatistics.getParameterValue("pathPage") == null ){
					throw new UndefinedStatisticParameterException("pathPage undefined value");
				}
				else{
					//statPagePath = (String)confStatistics.getParameterValue("pathPage");				
					statParams.put("pathPage",(String)confStatistics.getParameterValue("pathPage"));
				}
				if((String)confStatistics.getParameterValue("templateDir") == null ){
					throw new UndefinedStatisticParameterException("templateDir undefined value");
				}
				else{
					statParams.put("templateDir",(String)confStatistics.getParameterValue("templateDir"));			
				}
				if((String)confStatistics.getParameterValue("templateStatPage") == null ){
					throw new UndefinedStatisticParameterException("templateStatPage undefined value");
				}
				else{
					statParams.put("templateStatPage",(String)confStatistics.getParameterValue("templateStatPage"));	
				}
				if((String)confStatistics.getParameterValue("templateStatHome") == null ){
					throw new UndefinedStatisticParameterException("templateStatHome undefined value");
				}
				else{
					statParams.put("templateStatHome",(String)confStatistics.getParameterValue("templateStatHome"));
				}
				if((String)confStatistics.getParameterValue("statisticsList") == null ){
					throw new UndefinedStatisticParameterException("statisticsList undefined value");
				}
				else{
					statParams.put("statisticsList",(String)confStatistics.getParameterValue("statisticsList"));
				}
				if((String)confStatistics.getParameterValue("introPage") == null ){
					throw new UndefinedStatisticParameterException("introPage undefined value");
				}
				else{
					statParams.put("introPage",(String)confStatistics.getParameterValue("introPage"));
				}
				if((String)confStatistics.getParameterValue("groups") == null ){
					throw new UndefinedStatisticParameterException("groups undefined value");
				}
				else{
					statParams.put("groups",(String)confStatistics.getParameterValue("groups"));
				}

				try{
					if( 	((String)confStatistics.getParameterValue("columnsCount")) == null || 
							((String)confStatistics.getParameterValue("columnsTypeChange")) == null ||
							((String)confStatistics.getParameterValue("rollbackCount")) == null ||
							((String)confStatistics.getParameterValue("primaryKeyChanges")) == null ||
							((String)confStatistics.getParameterValue("indexChanges")) == null || 
							((String)confStatistics.getParameterValue("foreignKeyChanges")) == null
					)
						throw new UndefinedStatisticParameterException("Check the risk value in the statistics configuration file ");
					else{
						statParams.put("columnsCount",Double.parseDouble( ( String )confStatistics.getParameterValue("columnsCount")));
						statParams.put("columnsTypeChange",Double.parseDouble( ( String )confStatistics.getParameterValue("columnsTypeChange")));
						statParams.put("rollbackCount",Double.parseDouble( ( String )confStatistics.getParameterValue("rollbackCount")));
						statParams.put("primaryKeyChanges",Double.parseDouble( ( String )confStatistics.getParameterValue("primaryKeyChanges")));
						statParams.put("indexChanges",Double.parseDouble( ( String )confStatistics.getParameterValue("indexChanges")));
						statParams.put("foreignKeyChanges",Double.parseDouble( ( String )confStatistics.getParameterValue("foreignKeyChanges")));
					}
				}
				catch(NumberFormatException fEx){
					log.error("The risk values must be in double format", fEx);
					System.exit(-1);
				}
			}
			catch(UndefinedStatisticParameterException uEx){
				log.error("All the statistics parameters must be set, review the statistics configuration file", uEx);
				System.exit(-1);
			}
		}

	}



	public static double getRiskColumnsCount() {
		return (Double)statParams.get("riskColumnsCount");
	}

	public static double getRiskColumnsTypeChange() {
		return (Double)statParams.get("riskColumnsTypeChange");
	}

	public static double getRiskRollbackCount() {
		return (Double)statParams.get("riskRollbackCount");
	}

	public static double getRiskPrimaryKeyChange() {
		return (Double)statParams.get("riskPrimaryKeyChange");
	}

	public static double getRiskForeignKeyChange() {
		return (Double)statParams.get("riskForeignKeyChange");
	}

	public static double getRiskIndexChange() {
		return (Double)statParams.get("riskIndexChange");
	}


}
