/**
 * Copyright (C) 2009 
 * Developer: Fabrizio Moroni fabrizio.moroni@yahoo.it
 * Contributor: Carlo Curino	carlo@curino.us
 * 
 *  * This file is part of SchemEvolutionToolsuite
 * 
 * SchemEvolutionToolsuite is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SchemaEvolutionToolsuite is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with SchemEvolutionToolsuite.  If not, see <http://www.gnu.org/licenses/>.
 */

package edu.ucla.cs.statistics;

import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Vector;

import org.jfree.data.category.DefaultCategoryDataset;

import edu.ucla.cs.dbevolution.ApplicationParameters;
import edu.ucla.cs.dbevolution.DbBackend.DbTable;
import edu.ucla.cs.dbevolution.DbBackend.DbVersion;
import edu.ucla.cs.dbevolution.persistence.sqlDAO.DbTableDAO;


/**
 * Create and execute the statistic that performs an analysis over the database, retrieving how the databse is evolved between two 
 * following revision.
 * Retrieve the number of tables that have been added and deleted between a revision transition, in order to better understand the
 * databse evolutions
 * @author fangar
 *
 */
public class TablesAddedDeleted extends Statistic{

	/**
	 * Set the default filed for the statistic
	 */
	public TablesAddedDeleted() throws SQLException{
		
		//create the statistic
		//set the statistic parameters
		setTitle("Table Added and Deleted");
		setStatDescription("The statistic show the number of tables added and deleted every change of revision;\n The tables count statistic doesn't show " +
				"the detail of the transition between one revision and the other, while in this way it is possibile to better understand the change between " +
				"the revisions looking into the modifications occured to the database schema");
		setSqlStmt("The sql compare the name of the tables of every revision with the table name of the previuos and following revision");
		setRange("Added/Deleted");
		setDomain("Revisions");
		setStatName("statTablesAddedDeleted");
		setChartOrientation("v");
		setChartType("bar");
		setTitlePage("Statistic: Tables Added/Deleted");
		setStatPage(ApplicationParameters.getDbbasename() + getStatName() + ".html");
		setStatGroup("tables");
		
		
	}
	
	
	/**
	 * Execute the statistic
	 * @throws SQLException
	 */
	public void execute() throws SQLException{
		setDs(addedDeletedDataset());
		executeStatistic(this);
	}
	
	
	/**
	 * Create the dataset for the statistic
	 * @return the dataset
	 * @throws SQLException
	 */
	private DefaultCategoryDataset addedDeletedDataset() throws SQLException{
		boolean toAdd;
		int addedDeleted = 0; 
		DefaultCategoryDataset ds = new DefaultCategoryDataset();
		//create the dataset
		Vector<DbVersion> dbs = DbVersion.getDbInstalled();
    	
		//create the CSV file and add the header to the file
		createCSVFile(this);
		addValueToCsv(this, new String[]{"DomainValue","DomainValueRevisions","DomainValueAdded/Deleted","RangeValue"});
		
		
		for(int count = 0;count<dbs.size()-1;count++){
			toAdd  = false;
			
			String dbPrev = dbs.get(count).getSchemaUniqueName();
			String dbNext = dbs.get(count+1).getSchemaUniqueName();

			//retrieve the name of the tables for the revision and revision +1
			ArrayList<DbTable> tablesPrev = DbTableDAO.getTables(dbPrev);
			ArrayList<DbTable> tablesNext = DbTableDAO.getTables(dbNext);
			
			//calculate the number of operations over the tables
			int tablesAdded = DbTable.getDifference(tablesNext,tablesPrev);
			int tablesDeleted = DbTable.getDifference(tablesPrev,tablesNext);
			
			//add value to cvs /tables added and deleted) form each revision
    		addValueToCsv(this, new String[]{String.valueOf(addedDeleted),dbPrev,"added",String.valueOf(tablesAdded)});
    		addValueToCsv(this, new String[]{String.valueOf(addedDeleted),dbPrev,"deleted",String.valueOf(tablesDeleted)});			

    		//add the value to the chart (dataset) only if there have been operations (added or deleted different from zero)
    		if((tablesAdded!=0) ||(tablesDeleted!=0)) {
				addValueToDataset(ds, tablesAdded, "added", String.valueOf(addedDeleted));
				addValueToDataset(ds, tablesDeleted*-1, "deleted", String.valueOf(addedDeleted));
				toAdd = true;
			}
			if(toAdd){
				addedDeleted++;
			}
		}
		return ds;
	}
	
}
