/**
 * Copyright (C) 2009 
 * Developer: Fabrizio Moroni fabrizio.moroni@yahoo.it
 * Contributor: Carlo Curino	carlo@curino.us
 * 
 *  * This file is part of SchemEvolutionToolsuite
 * 
 * SchemEvolutionToolsuite is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * SchemaEvolutionToolsuite is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with SchemEvolutionToolsuite.  If not, see <http://www.gnu.org/licenses/>.
 */

package edu.ucla.cs.statistics;

import java.sql.SQLException;
import java.util.Vector;

import edu.ucla.cs.dbevolution.ApplicationParameters;
import edu.ucla.cs.dbevolution.DbBackend.StatisticDbOperation;
import edu.ucla.cs.dbevolution.persistence.sqlDAO.DbTableDAO;

public class TablesLifetime extends StatisticLifetime{

	public TablesLifetime() {
	
		super();
		setStatName("statTablesLifetime");
		setStatDescription("The statistics show the lifetimes of each table: the life of a table is the number of revision where the table is present"+
			"it doesn't necessary mean the table was live for consecutive revisions, " +
			"this is just a qualitative chart to represent the number of revision where table was present");
		setSqlStmt("SELECT TABLE_NAME, COUNT(*) FROM " + ApplicationParameters.getHeaderEvolutionTables() + "TABLES WHERE TABLE_SCHEMA LIKE ? GROUP BY TABLE_NAME ORDER BY count(*) DESC");
		setTitlePage("Statistic: Tables Lifetime");
		setRange("Lifetime");
		setDomain("Tables");
		setChartType("bar");
		setTitle("Tables Lifetime");
		setImageAlt("tables lifetime");
		setChartOrientation("h");
		setStatGroup("tables");
		
	}
	
	/**
	 * Call the executeLifetime method of the superclass giving in input the sql statement to retrieve the lifetime of the tables
	 * and the list of parameters
	 * @throws SQLException 
	 */
	public void executeTablesLifetime() throws SQLException{
		
		//retrieve the dataset values
		Vector<StatisticValue> values = StatisticDbOperation.GetStatCount(ApplicationParameters.getDbbasename(), this.getSqlStmt());
		int lifetime = DbTableDAO.getMaxTablesLifetime();
		
		//call the super method
		executeLifetimeStatistic(values, lifetime);
	}
	
	
	
}
