package edu.ucla.cs.velocity.template;

import java.io.IOException;
import java.io.StringWriter;
import java.util.Properties;

import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.VelocityEngine;
import org.apache.velocity.exception.MethodInvocationException;
import org.apache.velocity.exception.ParseErrorException;
import org.apache.velocity.exception.ResourceNotFoundException;

import edu.ucla.cs.statistics.StatisticsParameters;

public class VelocityTemplateDispatcher {


	/**
	 * Create a VelocityEngine object
	 * @return
	 */
	private static VelocityEngine getVelocityEngine(){
		VelocityEngine ve = null;
		try{
			ve = new VelocityEngine();
			Properties properties = new Properties();
			properties.setProperty("file.resource.loader.path","./input/smoSyntax/");
			ve.init(properties);
		}
		catch(ResourceNotFoundException rEx){
			System.out.println("Unable to find the template in the directory selected");
			System.exit(1);
		}
		catch(Exception e){
			e.printStackTrace();
		}
		return ve;
	}

	
	/**
	 * Get the given context and template, create a {@link String} with merged output result 
	 * @param context
	 * @param t
	 * @return String representation of the context
	 */
	public static String mergeContextIntoString(VelocityContext context,Template t){
		
		//render the template into a StringWriter
        StringWriter writer = new StringWriter();
        try {
			t.merge( context, writer );
		} catch (ResourceNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ParseErrorException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (MethodInvocationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		return writer.toString();
	}
	
	

	protected static Template getVelocityTemplate(String template){
		VelocityEngine ve=null;
		Template t=null;

		try{
			//get and initialize an engine  
			ve = getVelocityEngine();

			try{
				//get the template
				t = ve.getTemplate(template);
			}
			catch(ResourceNotFoundException rEx){
				System.out.println("Unable to find the page template.\nCheck if you have the template page located in the path configured " +
				"in the statistics.xml file and repeat the execution");
				System.exit(1);
			}
			catch(ParseErrorException pEx){
				System.out.println("An error occurred during the parsing of the template file: verify che template syntax");
				System.exit(1);
			}
		}
		catch(Exception e){
			e.printStackTrace();
		}
		return t; 
	}
}
