/**
 * Copyright (C) 2007 Giorgio Orsi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */
package it.polimi.elet.cautils.configurator;

import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Iterator;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Attr;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

/**
 * @author Giorgio Orsi (orsi@elet.polimi.it)
 * @version 0.1
 */
public class CAConfigFileParser {

    private static boolean existing;
    
    /**
     * Sets the existing flag to true. 
     */
    private CAConfigFileParser() {
	existing = true;
    }
    
    /**
     * @return A new instance of the configuration file parser.
     * @throws InstantiationException If an instance of the parser already exists.
     */
    public static CAConfigFileParser getInstance() throws InstantiationException{
	if (!existing) {
	    return (new CAConfigFileParser());
	}
	throw new InstantiationException("Only one parser at time is allowed");
    }
    
    /**
     * Parses an xml file and loads the configuration's parameters.
     * @param fileName The path to the configuration file.
     * @return A configuration.
     */
    public CAConfiguration parse(String fileName) {
	CAConfiguration conf = new CAConfiguration();
	
	DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        String parameterName;
        Object parameterValue;
        
        try {
            DocumentBuilder builder = factory.newDocumentBuilder();
            Document doc = builder.parse(fileName);
            Element root = doc.getDocumentElement();
            NodeList rootNodeList = root.getChildNodes();
            for (int i=0; i<rootNodeList.getLength();i++) {
                if (rootNodeList.item(i).getNodeName().compareTo("param")==0){
                    NamedNodeMap attributesMap = rootNodeList.item(i).getAttributes();
                    if (attributesMap.item(0).getNodeName().compareTo("name")==0){
                            parameterName = attributesMap.item(0).getNodeValue();
                            parameterValue = rootNodeList.item(i).getFirstChild().getNodeValue();
                            conf.addParameter(parameterName, parameterValue);
                    }  
                    else {
                        throw new SAXException("Unknown Attribute");
                    }
                }
            }
        }
        catch (ParserConfigurationException pce) {
            pce.printStackTrace(System.err);
        }
        catch (IOException ioe) {
            ioe.printStackTrace(System.err);
        }
        catch (SAXException saxe) {
            saxe.printStackTrace(System.err);
        }
	return (conf);
    }
    
    /**
     * Writes a configuration to an xml file.
     * @param conf The configuration to write out.
     * @param filePath the path to the xml configuration file.
     */
    public void writeConfiguration (CAConfiguration conf, String filePath) {
	
	DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        
        try {
            DocumentBuilder builder = factory.newDocumentBuilder();
            Document doc = builder.newDocument();
            Element rootElement = doc.createElement("parameters");
            
            Iterator it = conf.keys();
            while (it.hasNext()) {
        	String currentKey = (String)it.next();
        	Attr curName = doc.createAttribute("name");
        	curName.setNodeValue(currentKey);
        	Element curElement = doc.createElement("param"); 
        	curElement.setTextContent(conf.getParameterValue(currentKey).toString());
        	curElement.setAttributeNode(curName);
        	rootElement.appendChild(curElement);
            }
            doc.appendChild(rootElement);
            
            // Use a Transformer for output
            TransformerFactory tFactory = TransformerFactory.newInstance();
            Transformer transformer = tFactory.newTransformer();

            DOMSource source = new DOMSource(doc);
            StreamResult result = new StreamResult(new FileOutputStream(filePath));
            transformer.transform(source, result);
            
        }
        catch (IOException ioe) { ioe.printStackTrace(System.err); }
        catch (TransformerConfigurationException tce) { tce.printStackTrace(System.err); }
        catch (ParserConfigurationException pce) { pce.printStackTrace(System.err); }
        catch (TransformerException te) { te.printStackTrace(System.err); }    
    }
}
