/**
 * Copyright (C) 2007 Giorgio Orsi
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */
package it.polimi.elet.cautils.configurator;

import java.util.HashMap;
import java.util.Iterator;
import java.util.NoSuchElementException;

/**
 * @author Giorgio Orsi (orsi@elet.polimi.it)
 * @version 0.1
 */
public class CAConfiguration{
    
    HashMap<String,Object> config;
    
    /**
     * Generates a new empty Context-ADDICT configuration. 
     */
    public CAConfiguration () {
	config = new HashMap<String,Object>();
    }
    
    /**
     * Adds a new parameter to the configuration
     * @param pName The parameter's name
     * @param pValue The parameter's value
     * @return The previous value associated with specified pName, or null
     *         if there was no mapping for key. A null return can also 
     *         indicate that the configuration previously associated null 
     *         with the specified key.
     */
    public Object addParameter (String pName, Object pValue){
	return (this.config.put(pName, pValue));
    }
    
    /**
     * @param pName The parameter's name
     * @return The value associated with the given name 
     */
    public Object getParameterValue (String pName){
	return (this.config.get(pName));
    }
    
    public Iterator keys () {
	return (this.config.keySet().iterator());
    }
    
    /**
     * Remove the parameter corresponding to the given name
     * @param pName The parameter's name
     * @return The previous value associated with specified key, or null if 
     *         there was no mapping for key. A null return can also indicate 
     *         that the map previously associated null  with the specified key.
     */
    public Object removeParameter (String pName){
	return (this.config.remove(pName));
    }
    
    /**
     * @return True if the configuration is empty.
     */
    public boolean isEmpty (){
	return (this.config.isEmpty());
    }
    
    public boolean contains (String pName){
	return (this.config.containsKey(pName));
    }
    
    /**
     * Clears the current configuration.
     */
    public void clearConfiguration (){
	this.config.clear();
    }
     
    /**
     * @return The entries in the order they appear into the configuration.
     */
    public Iterator entries (){
	return (new ParametersGen(this));
    }
    
    //Internal unordered iterator class
    private static class ParametersGen implements Iterator{
    
	//The configuration being iterated
        private CAConfiguration config;
        //The key set iterator for this configuration 
        private Iterator keySetIt;
        
        /**
         * @param config
         */
        public ParametersGen(CAConfiguration config){
            this.config = config;
            this.keySetIt = config.config.keySet().iterator();
        }
        
        /* (non-Javadoc)
         * @see java.util.Iterator#hasNext()
         */
        public boolean hasNext () {
            return (this.keySetIt.hasNext());
        }
        
        /* (non-Javadoc)
         * @see java.util.Iterator#next()
         */
        public Object next () throws NoSuchElementException{
            return (this.config.getParameterValue((String)keySetIt.next()));
        }
        
        /* (non-Javadoc)
         * @see java.util.Iterator#remove()
         */
        public void remove () throws UnsupportedOperationException{
            throw new UnsupportedOperationException();
        }
    }
    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {
	String output="";
	
	if (this.isEmpty()) {
	    output += "<Empty Configuration>";
	} else {
	    Iterator it = this.config.keySet().iterator();
	    while (it.hasNext()) {
	        String curKey = (String)it.next();
	        output += "<"+curKey+" , "+config.get(curKey)+">\n";
	    }
        }
	return (output);
    }
}
